// $Id: BenefitAmount.h 1.19 2006/11/08 10:57:57EST 044579 Development  $
//
// Declarations needed for Social Security benefit amount.

#pragma once

#include <cmath>  // for floor and fmod
#include <utility>  // for rel_ops
#include "qtryear.h"
#include "piaparms.h"
#include "Floor.h"

#if !defined(__SGI_STL_INTERNAL_RELOPS)
using namespace std::rel_ops;
#endif

// {group:Calculation Classes}
// Description: Represents a benefit amount for one year.
//
// Remarks: There are instances of this class in PiaData and the various
// PiaMethod classes.
class BenefitAmount
{
protected:
   // Description: Year 1973, change in rounding rules.
   //
   // Remarks: The margin for dime rounding was changed from 0.005 to
   // 0.0001.
   static const int amend73year = 1973;
   // Description: Year 1982, change in rounding rules.
   //
   // Remarks: Dime rounding was changed from rounding up to rounding down,
   // when applied to benefit amounts.
   static const int amend82year = 1982;
   // Description: Year 2000, change in rounding rules for Social Security
   // Statement.
   //
   // Remarks: Amounts for Statements were rounded down to $5 before 2000, but
   // to $1 in 2000 and later.
   static const int amend2000year = 2000;
   // Description: Benefit amount.
   double theAmt;
public:
   // Description: Initializes amount.
   //
   // Remarks: If argument is omitted, amount is set to 0.0.
   //
   // Arguments:
   //   newAmount: New amount to use.
   BenefitAmount( double newAmount = 0.0 ) : theAmt(newAmount) { }
   // Description: Applies cola percent.
   //
   // Remarks: The version with 2 arguments uses the
   //   stored amount to call the static version with 3 arguments.
   //
   // Returns: The increased amount (in the static version).
   //
   // Arguments:
   //   percent: Percentage increase.
   //   year: Year of increase.
   void applyCola( double percent, int year )
      { theAmt = applyCola(theAmt, percent, year); }
   // Description: Returns the benefit amount.
   //
   // Returns: The benefit amount.
   double get() const { return theAmt; }
   // Description: Compares benefit with benefit from another instance.
   //
   // Returns: True if this benefit is less than argument.
   //
   // Arguments:
   //   bentemp: Benefit to compare to.
   bool operator<( const BenefitAmount& bentemp ) const
      { return(theAmt < bentemp.theAmt); }
   // Description: Rounds a PIA or MFB to the appropriate multiple of $.10.
   //
   // Remarks: The version with 1 argument uses the stored amount to call the
   // static version with 2 arguments.  Round a cola calculation up for June
   // 1981 and earlier colas, down for June 1982 and later colas.  Round an
   // aime pia calculation up for 1982 and earlier eligibility, down for 1983
   // and later eligibility.
   //
   // Returns: The rounded amount (in the static version).
   //
   // Throws: PiaException of type PIA_IDS_ROUND if year is less than 1937
   // (only in debug mode).
   //
   // Arguments:
   //   year: Year of benefit increase, or year prior to year of wage-indexed
   //     formula.
   void round( int year ) { theAmt = round(theAmt, year); }
   // Description: Rounds a PIA or MFB down to a whole dollar for June 1982 and
   //   later.
   //
   // Remarks: The version with 1 argument uses the stored amount to call the
   //   static version with 2 arguments.
   //
   // Returns: The rounded amount (in the static version).
   //
   // Arguments:
   //   benDate: The date of benefit.
   void round1( const DateMoyr& benDate )
      { if (benDate >= PiaParams::amend82) { theAmt = FLOOR(theAmt); } }
   // Description: Rounds benefit down to $5 multiple.
   //
   // Returns: The rounded benefit.
   //
   // Remarks: The version with 1 argument uses the stored amount to call the
   //   static version with 2 arguments.Use this function for Social Security
   //   Statements before 2000. For years 2000 and later, round down to $1
   //   multiple.
   //
   // Arguments:
   //   year: Year of benefit.
   int round5( int year ) const { return round5(theAmt, year); }
   // Description: Sets the benefit amount.
   //
   // Arguments:
   //   newAmount: New benefit amount
   void set( double newAmount ) { theAmt = newAmount; }
   std::string toString() const;
   std::string toString( const std::string& format ) const;
   // Description: Removes cola percent from quantity.
   //
   // Remarks:  The version with 2 arguments uses the stored amount
   //   to call the static version with 3 arguments.
   //
   // Returns: The decreased amount (in the static version).
   //
   // Arguments:
   //   percent: Percentage decrease.
   //   year: Year of decrease.
   void unApplyCola( double percent, int year )
      { theAmt = unApplyCola(theAmt, percent, year); }
   // Description: Unrounds a PIA or MFB to the appropriate multiple of $.10.
   //
   // Remarks: The version with 1 argument uses the stored amount to call the
   //   static version with 2 arguments.
   //
   // Returns: The unrounded amount (in the static version).
   //
   // Throws: PiaException of type PIA_IDS_ROUND if year is less than 1937
   // (only in debug mode).
   //
   // Arguments:
   //   year: Year of benefit increase, or year prior to year of wage-indexed
   //    formula.
   void unround( int year ) { theAmt = unround(theAmt, year); }
   // Arguments:
   //   theAmount: Amount to increase.
   static double applyCola( double theAmount, double percent, int year )
      { theAmount *= (1. + percent / 100.); return round(theAmount, year); }
   static double round( double amount, int year );
   // Arguments:
   //   theAmount: Amount to be rounded.
   static double round1( double theAmount, const DateMoyr& benDate )
      { if (benDate >= PiaParams::amend82) { return FLOOR(theAmount); }
        else { return theAmount; } }
   // Arguments:
   //   theAmount: Amount to be rounded.
   static int round5( double theAmount, int year )
      { if (year < amend2000year) {
           return 5 * static_cast<int>((theAmount + .01) / 5.); }
        else { return static_cast<int>(theAmount + .01); } }
   // Arguments:
   //   theAmount: Amount to decrease.
   static double unApplyCola( double theAmount, double percent, int year)
      { theAmount *= ( 100. / (100. + percent) );
        return unround(theAmount, year); }
   static double unround( double theAmount, int year );
};
