// $Id: DibGuar.h 1.2 2006/11/08 10:58:37EST 044579 Development  $
//
// Declarations needed for class to manage disability guarantee pia calculation
// functions.

#pragma once

#include "PiaMethod.h"

// {group:Calculation Classes}
// Description: Manages the functions required for calculation of
//   a disability guarantee Social Security benefit.
//
// Remarks: This method is used when a person has recovered from one period of
// disability and is claiming a subsequent benefit (old-age, disability, or
// death).
class DibGuar : public PiaMethod
{
public:
   // Description: The type of mfb at cessation, after conversion.
   //
   // Remarks: The type of mfb first depends on the date of entitlement to the
   // subsequent benefit. If it is January 1996 or later, then the type is
   // POST1995_NOCHANGE if the new benefit is a disability benefit, or if the
   // Disability Maximum does not apply. The type is POST1995_DECONVERTED if it
   // is an old-age or survivor benefit and the Disability Maximum applies.
   //
   // If the date of entitlement to the subsequent benefit is before January
   // 1996, then the type depends on if the prior DIB entitlement continued
   // in the 12 months preceding the new entitlement. If so, the type is
   // PRE1996_PRE1979_LAST12 if eligibility to the prior DIB benefit was
   // before 1979, or PRE1996_POST1978_LAST12 if eligibility to the prior
   // DIB benefit was after 1978. Otherwise, the type is PRE1996_NOTLAST12.
   enum ConvertedMfbType {
      // Subsequent entitlement in January 1996 or later, new benefit is DI,
      // or the Disability Maximum does not apply (no conversion required).
      POST1995_NOCHANGE = 0,
      // Subsequent entitlement in January 1996 or later, new benefit is
      // OASI, and Disability Maximum applies (requires deconversion to
      // prior eligibility year).
      POST1995_DECONVERTED,
      // Subsequent entitlement before January 1996, prior DIB continued in
      // the 12 months preceding the new entitlement, prior DIB eligibility
      // was before 1979.
      PRE1996_PRE1979_LAST12,
      // Subsequent entitlement before January 1996, prior DIB continued in
      // the 12 months preceding the new entitlement, prior DIB eligibility
      // was after 1978.
      PRE1996_POST1978_LAST12,
      // Subsequent entitlement before January 1996, prior DIB did not
      // continue in the 12 months preceding the new entitlement.
      PRE1996_NOTLAST12
   };
protected:
   // Description: True if colas apply from cessation date of prior
   // disability to new entitlement date.
   bool colasApply;
   // Description: True if mfb at cessation date must be refigured by
   // deconverting the pia at cessation date to onset date, calculating
   // raw mfb, and converting forward to cessation date.
   ConvertedMfbType convertedMfbType;
   // Description: True if the mfb from the prior disability was determined
   // using the Disability Maximum method.
   bool diMaxApplies;
   // Description: Which disability period to use (0 for most recent, 1 for
   // next most recent).
   int whichPeriod;
   // Description: The pia deconverted from cessation date to eligibility date.
   double rawPia;
   // Description: The mfb based on the deconverted pia at eligibility date.
   double rawMfb;
   // Description: The original mfb at the cessation date.
   //
   // Remarks: This could be based on the Disability Maximum, and so may
   // need to be converted (see cessationMfbConv).
   double cessationMfbOrig;
   // Description: The mfb based on the deconverted pia brought up to the
   // cessation date.
   double cessationMfbConv;
   // Description: The date of entitlement in a life case, or date of death
   // in a survivor case.
   DateMoyr entDeathDate;
   // Description: Portion of AIME PIA in each interval of MFB formula.
   double portionPiaElig[4];
   // Description: Bend points for MFB formula.
   BendMfb bendMfb;
   // Description: Wage-indexed MFB formula percentages.
   PercMfb percMfb;
   // Description: Year of eligibility for prior DIB benefit.
   int eligYear;
protected:
   double cessationMfbCal( const DisabPeriod& disabPeriod );
   ConvertedMfbType convertedMfbTypeCal() const;
   DateMoyr dateCpiCal() const;
   DateMoyr entDeathDateCal() const;
   int yearCpi1Cal() const;
public:
   DibGuar( const WorkerDataGeneral& newWorkerData,
      const PiaData& newPiaData, const PiaParams& newPiaParams,
      int newMaxyear );
   virtual ~DibGuar();
   void calculate();
   // Description: Returns the mfb based on the deconverted pia brought up to
   // the cessation date.
   //
   // Returns: The mfb based on the deconverted pia brought up to the
   // cessation date.
   double getCessationMfbConv() const { return cessationMfbConv; }
   // Description: Returns the original mfb at the cessation date.
   //
   // Returns: The original mfb at the cessation date.
   //
   // Remarks: This could be based on the Disability Maximum, and so may
   // need to be converted (see getCessationMfbConv).
   double getCessationMfbOrig() const { return cessationMfbOrig; }
   // Description: Returns true if colas apply from cessation date of prior
   // disability to new entitlement date.
   //
   // Returns: True if colas apply from cessation date of prior
   // disability to new entitlement date.
   bool getColasApply() const { return colasApply; }
   // Description: Returns the type of converted mfb.
   //
   // Returns: The type of converted mfb.
   ConvertedMfbType getConvertedMfbType() const { return convertedMfbType; }
   // Description: Returns true if the mfb from the prior disability was
   // determined using the Disability Maximum method.
   //
   // Returns: True if the mfb from the prior disability was
   // determined using the Disability Maximum method.
   bool getDiMaxApplies() const { return diMaxApplies; }
   // Description: Returns the mfb based on the deconverted pia at eligibility
   // date.
   //
   // Returns: The mfb based on the deconverted pia at eligibility date.
   double getRawMfb() const { return rawMfb; }
   // Description: Returns the pia deconverted from cessation date to
   // eligibility date.
   //
   // Returns: The pia deconverted from cessation date to eligibility date.
   double getRawPia() const { return rawPia; }
   bool isApplicable();
   static bool isApplicable( const WorkerDataGeneral& workerData,
      const PiaData& piaData, WorkerDataGeneral::ben_type ioasdi );
private:
   DibGuar operator=( DibGuar newDibGuar );
};
