// $Id: DisabPeriod.h 1.7 2007/02/28 15:03:46EST 044579 Development  $
//
// Declarations needed for class to manage a period of disability.

#pragma once

#include <string>
#include "datemoyr.h"
#include "boost/serialization/access.hpp"
#include "boost/date_time/gregorian/greg_date.hpp"

// {group:Calculation Classes}
// Description: Manages a period of disability.
//
// Remarks: It has slots for the date of onset, first month of the waiting
// period, date of entitlement, date of cessation, and PIA and MFB as of the
// month before cessation. (The month of cessation is the month after the last
// month for which benefits were received.) If the disability is still open, as
// in a disability case, the date of cessation, and PIA and MFB just before
// cessation, are zero.
//
// An array of two periods of disability are created in WorkerDataGeneral. The
// one at index 0 is the most recent, while the one at index 1 is the next most
// recent. The values in those structures are ignored if the indicator for the
// number of periods of disability is zero. In a disability case, the most
// recent period of disability is the current one.
class DisabPeriod
{
   friend class boost::serialization::access;
   // Description: Archives the data.
   //
   // Remarks: When the class Archive corresponds to an output archive, the
   // & operator is defined similar to <<. Likewise, when the class Archive is
   // a type of input archive, the & operator is defined similar to >>.
   //
   // Arguments:
   //   ar: The archive to use.
   //   version: The version of the archive to use.
   template< class Archive >
   void serialize( Archive& ar, const unsigned int version )
      { ar & onsetDate & cessationDate & entDate & waitperDate;
        ar & cessationPia & cessationMfb; }
protected:
   // Description: Month, day, and year of disability onset.
   boost::gregorian::date onsetDate;
   // Description: Month and year of disability cessation.
   //
   // Remarks: This will be 0 for a current period of disability.
   DateMoyr cessationDate;
   // Description: Month and year of DIB entitlement.
   DateMoyr entDate;
   // Description: Month and year of beginning of waiting period.
   DateMoyr waitperDate;
   // Description: The PIA just before cessation.
   //
   // Remarks: This will be 0 for a current period of disability.
   float cessationPia;
   // Description: The MFB just before cessation.
   //
   // Remarks: This will be 0 for a current period of disability.
   float cessationMfb;
public:
   DisabPeriod();
   // Description: Deletes the cessation date.
   //
   // Remarks: Also deletes the pia and mfb at cessation.
   void deleteCessationDate()
      { cessationDate.deleteContents(); cessationPia = cessationMfb = 0.0f; }
   void deleteContents();
   // Description: Deletes the entitlement date.
   void deleteEntDate() { entDate.deleteContents(); }
   // Description: Deletes the onset date.
   void deleteOnsetDate()
   { onsetDate = boost::gregorian::date(boost::date_time::not_a_date_time); }
   // Description: Deletes the first month of the waiting period.
   void deleteWaitperDate() { waitperDate.deleteContents(); }
   // Description: Returns date of disability cessation.
   //
   // Returns: The date of disability cessation.
   const DateMoyr& getCessationDate() const { return(cessationDate); }
   // Description: Returns the mfb in the month before cessation.
   //
   // Returns: The mfb at cessation.
   float getCessationMfb() const { return(cessationMfb); }
   // Description: Returns the pia in the month before cessation.
   //
   // Returns: The pia at cessation.
   float getCessationPia() const { return(cessationPia); }
   // Description: Returns date of entitlement to disability.
   //
   // Returns: The date of entitlement to disability.
   const DateMoyr& getEntDate() const { return(entDate); }
  // Description: Returns date of disability onset.
  //
  // Returns: The date of disability onset.
   const boost::gregorian::date& getOnsetDate() const { return(onsetDate); }
   // Description: Returns first month of waiting period.
   //
   // Returns: First month of waiting period.
   const DateMoyr& getWaitperDate() const { return(waitperDate); }
   void parseString( const std::string& line );
  // Description: Sets the cessation date.
   //
   // Arguments:
   //   dateMoyr: The new cessation date.
   void setCessationDate( const DateMoyr& dateMoyr )
      { cessationDate = dateMoyr; }
   // Description: Sets the mfb at cessation.
   //
   // Arguments:
   //   newMfb: The new mfb at cessation.
   void setCessationMfb( float newMfb ) { cessationMfb = newMfb; }
   // Description: Sets the pia at cessation.
   //
   // Arguments:
   //   newPia: The new pia at cessation.
   void setCessationPia( float newPia ) { cessationPia = newPia; }
   // Description: Sets the entitlement date.
   //
   // Arguments:
   //   dateMoyr: The new entitlement date.
   void setEntDate( const DateMoyr& dateMoyr ) { entDate = dateMoyr; }
   // Description: Sets the onset date.
   //
   // Arguments:
   //   dateMoyr: The new onset date.
   void setOnsetDate( const boost::gregorian::date& dateModyyr )
      { onsetDate = dateModyyr; }
   // Description: Sets the first month of the waiting period.
   //
   // Arguments:
   //   dateMoyr: The new first month of the waiting period.
   void setWaitperDate( const DateMoyr& dateMoyr )
      { waitperDate = dateMoyr; }
   std::string toString() const;
   std::string toString( const std::string& format ) const;
   DateMoyr waitperDateCal() const;
   static void cessationDateCheck( const DateMoyr& dateMoyr );
   static void entDateCheck( const DateMoyr& dateMoyr );
   static void onsetDateCheck( const boost::gregorian::date& dateModyyr );
   static void waitperDateCheck( const DateMoyr& dateMoyr );
};
