// $Id: EarnProject.h 1.14 2006/11/08 10:58:37EST 044579 Development  $
//
// Declarations needed for class to manage projected earnings.

#pragma once

#include "avgwg.h"
#include "intann.h"
#include "oactcnst.h"
class WorkerDataGeneral;

// {group:Calculation Classes}
// Description: Manages an array of earnings, with forward
//   and backward projections and specified levels of earnings.
//
// Remarks: An instance of this class is used in the Anypia program.
class EarnProject
{
public:
   // Description: Type of earnings.
   enum earn_type {
      ENTERED,         // Entered earnings.
      MAXIMUM,         // Maximum earnings.
      HIGH,            // High earnings.
      AVERAGE,         // Average earnings.
      LOW,             // Low earnings.
      OLDLAW_MAXIMUM,  // Old-law maximum.
      CHILDCARE_YEAR,  // Childcare year.
      NUM_TYPES        // Number of earnings types.
   };
   // Description: Type of earnings projection.
   //
   // Remarks: There are two instances of this enum in this class, projback
   // for a backward projection and projfwrd for a forward projection.
   enum earn_proj_type {
      NO_PROJ,       // No projection.
      AVGWAGE_PROJ,  // Projection related to average wage increase.
      CONSTANT_PROJ  // Projection by a constant percentage.
   };
   // Description: Worker earnings, 1937 to maxyear, as read in.
   AverageWage earnpebs;
   // Description: Lowest acceptable projection percentage.
   static const float PERC_LOW;
   // Description: Highest acceptable projection percentage.
   static const float PERC_HIGH;
protected:
   // Description: Type of earnings (see earn_type).
   IntAnnual earntype;
   // Description: First year of earnings before projection.
   int firstYear;
   // Description: Last year of earnings before projection.
   int lastYear;
   // Description: Indicator for backward projection.
   earn_proj_type projback;
   // Description: Percentage used in backward projection.
   //
   // Remarks: If projback is AVGWAGE_PROJ, it is the increment to average
   // wage increase. If projback is CONSTANT_PROJ, it is the constant
   // percentage increase.
   float percback;
   // Description: Indicator for forward projection.
   earn_proj_type projfwrd;
   // Description: Percentage used in forward projection.
   //
   // Remarks: If projfwrd is AVGWAGE_PROJ, it is the increment to average
   // wage increase. If projfwrd is CONSTANT_PROJ, it is the constant
   // percentage increase.
   float percfwrd;
   // Description: Maximum year of projection.
   static int maxyear;
   // Description: Earnings selection titles.
   static const char *earnmal[NUM_TYPES];
public:
   explicit EarnProject( int newMaxYear );
   virtual ~EarnProject();
   // Description: Returns true if any earnings are entered.
   //
   // Returns: True if any earnings are entered.
   bool anyEntered() const
   { for (int i1 = firstYear; i1 <= lastYear; i1++) {
        if (earntype[i1] == ENTERED) return true; }
     return false; }
   virtual void deleteContents();
   // Description: Returns array of indicators for type of earnings.
   //
   // Returns: Array of indicators for type of earnings (see earntype).
   const IntAnnual& getEarntype() const { return(earntype); }
   // Description: Returns first year of earnings.
   //
   // Returns: First year of earnings.
   int getFirstYear() const { return firstYear; }
   // Description: Returns last year of earnings.
   //
   // Returns: Last year of earnings.
   int getLastYear() const { return lastYear; }
   // Description: Returns percentage used in backwards projection.
   //
   // Returns: Percentage used in backwards projection (see percback).
   float getPercback() const { return(percback); }
   // Description: Returns percentage used in forwards projection.
   //
   // Returns: Percentage used in forwards projection (see percfwrd).
   float getPercfwrd() const { return(percfwrd); }
   // Description: Returns backward projection indicator.
   //
   // Returns: Backward projection indicator.
   earn_proj_type getProjback() const { return(projback); }
   // Description: Returns forward projection indicator.
   //
   // Returns: Forward projection indicator.
   earn_proj_type getProjfwrd() const { return(projfwrd); }
   // Description: Checks for last year of earnings within bounds.
   //
   // Remarks: The version with 2 arguments calls the static version with 4
   // arguments, adding firstYear and lastYear as arguments.
   //
   // Throws: PiaException of type PIA_IDS_END1 if first year of earnings in
   // forward projection period is before last year of earnings in backward
   // projection period. PiaException of type PIA_IDS_END3 if first year of
   // earnings in forward projection period is after last year of earnings in
   // forward projection period. PiaException of type PIA_IDS_END4 if the total
   // number of years of earnings is greater than the maximum allowed (MAXYRS).
   // PiaException of type PIA_IDS_END1 if last year of earnings is
   // before 1937. PiaException of type PIA_IDS_END2 if last year of earnings
   // is after maximum year allowed.
   //
   // Arguments:
   //   ibegint: First year of earnings in backward projection period.
   //   iendt: Last year of earnings in forward projection period.
   void iendCheck( int ibegint, int iendt ) const
   { iendCheck(ibegint, firstYear, iendt, lastYear); }
   bool need_aw( int year, int iendt ) const;
   bool project( WorkerDataGeneral& workerData, const Awinc& fqinc ) const;
   // Description: Sets earnings for specified year.
   //
   // Arguments:
   //   year: Year of earnings.
   //   newEarnpebs: Amount of earnings.
   void setEarnpebs( int year, double newEarnpebs )
   {
   #ifndef NDEBUG
      earnpebsCheck(newEarnpebs);
   #endif
      earnpebs[year] = newEarnpebs;
   }
   // Description: Sets type of earnings for specified year.
   //
   // Throws: PiaException of type PIA_IDS_EARNTYPEYR if year is out of range
   //   (only in debug mode). PiaException of type PIA_IDS_EARNTYPE if type of
   //   earnings is out of range (only in debug mode).
   //
   // Arguments:
   //   year: Year of earnings.
   //   newEarntype: Type of earnings.
   void setEarntype( int year, int newEarntype )
   {
   #ifndef NDEBUG
      if (year < YEAR37)
         throw PiaException(PIA_IDS_EARNTYPEYR);
      earntypeCheck(newEarntype);
   #endif
      earntype[year] = newEarntype;
   }
   // Description: Sets first year of specified earnings (earnings could
   //   be projected backward before this year).
   //
   // Arguments:
   //   newFirstYear: New first year of earnings before projection.
   void setFirstYear( int newFirstYear )
   {
   #ifndef NDEBUG
      ibegin2Check(newFirstYear);
   #endif
      firstYear = newFirstYear;
   }
   // Description: Sets last year of specified earnings (earnings could
   //   be projected forward after this year).
   //
   // Arguments:
   //   newLastYear: New last year of earnings before projection.
   void setLastYear( int newLastYear )
   {
   #ifndef NDEBUG
      iendCheck(newLastYear);
   #endif
      lastYear = newLastYear;
   }
   void setPebsData( int year, int ibegint );
   void setPebsEarn( int year );
   // Description: Sets backward projection percentage.
   //
   // Arguments:
   //   newPercback: New backward projection percentage (see percback).
   void setPercback( float newPercback )
   {
   #ifndef NDEBUG
      percbackCheck(newPercback);
   #endif
      percback = newPercback;
   }
   // Description: Sets percentage in forward projection.
   //
   // Arguments:
   //   newPercfwrd: New forward projection percentage (see percfwrd).
   void setPercfwrd( float newPercfwrd )
   {
   #ifndef NDEBUG
      percfwrdCheck(newPercfwrd);
   #endif
      percfwrd = newPercfwrd;
   }
   // Description: Sets backward projection indicator.
   //
   // Arguments:
   //   newProjback: New backward projection indicator (see projback).
   void setProjback( int newProjback )
   {
   #ifndef NDEBUG
      projbackCheck1(newProjback);
   #endif
      projback = (earn_proj_type)newProjback;
   }
   // Description: Sets forward projection indicator.
   //
   // Arguments:
   //   newProjfwrd: New forward projection indicator (see projfwrd).
   void setProjfwrd( int newProjfwrd )
   {
   #ifndef NDEBUG
      projfwrdCheck1(newProjfwrd);
   #endif
      projfwrd = (earn_proj_type)newProjfwrd;
   }
   std::string toString( int year ) const;
   // Description: Zeroes out earnings before firstYear and after lastYear.
   void zeroEarn()
      { earnpebs.assign(0.0, YEAR37, firstYear - 1);
        earnpebs.assign(0.0, lastYear + 1, maxyear); }
   static void earnpebsCheck( double earnpebst );
   // Description: Checks type of earnings.
   //
   // Throws: PiaException of type PIA_IDS_EARNTYPE if type of earnings is out
   // of range.
   //
   // Arguments:
   //   earntypet: Type of earnings to check.
   static void earntypeCheck( int earntypet )
   { if (earntypet < static_cast<int>(ENTERED) ||
       earntypet > static_cast<int>(LOW))
        throw PiaException(PIA_IDS_EARNTYPE); }
   // Description: Checks for first year of earnings within bounds, using
   //   passed values, with a backward projection.
   //
   // Throws: PiaException of type PIA_IDS_BEGIN2 if last year of earnings is
   //   before first year of earnings in backward projection period.
   //
   // Arguments:
   //   ibegint: First year of earnings in backward projection period.
   //   ibegin1t: Last year of earnings in backward projection period.
   static void ibeginCheck( int ibegint, int ibegin1t )
   { if (ibegin1t < ibegint) throw PiaException(PIA_IDS_BEGIN2); }
   // Description: Checks for first year of earnings in bounds.
   //
   // Throws: PiaException of type PIA_IDS_BEGIN3 if first year of earnings is
   // before 1937. PiaException of type PIA_IDS_BEGIN5 if first year of
   // earnings is after maximum year allowed.
   //
   // Arguments:
   //   ibegint: Year to check.
   static void ibegin2Check( int ibegint )
   { if (ibegint < YEAR37) throw PiaException(PIA_IDS_BEGIN3);
     if (ibegint > maxyear) throw PiaException(PIA_IDS_BEGIN5); }
   // Arguments:
   //   ibegint: First year of earnings in backward projection period.
   //   ibegin1t: Last year of earnings in backward projection period.
   //   iendt: Last year of earnings in forward projection period.
   //   iend1t: First year of earnings in forward projection period.
   static void iendCheck( int ibegint, int ibegin1t, int iendt, int iend1t )
   { if (iend1t < ibegin1t) throw PiaException(PIA_IDS_END1);
     if (iend1t > iendt) throw PiaException(PIA_IDS_END3);
     if (iendt > ibegint + MAXYRS - 1) throw PiaException(PIA_IDS_END4); }
   // Arguments:
   //   iendt: Year to check.
   static void iendCheck( int iendt )
   { if (iendt < YEAR37) throw PiaException(PIA_IDS_END1);
     if (iendt > maxyear) throw PiaException(PIA_IDS_END2); }
   // Description: Checks backward projection percentage.
   //
   // Throws: PiaException of type PIA_IDS_PERCBACK if backward projection
   //   percentage is out of range.
   //
   // Arguments:
   //   percbackt: Percentage to check.
   static void percbackCheck( float percbackt )
   { if (percbackt < PERC_LOW || percbackt > PERC_HIGH)
        throw PiaException(PIA_IDS_PERCBACK); }
   // Description: Checks forward projection percentage.
   //
   // Throws: PiaException of type PIA_IDS_PERCFWRD if backward projection
   //   percentage is out of range.
   //
   // Arguments:
   //   percfwrdt: Percentage to check.
   static void percfwrdCheck( float percfwrdt )
   { if (percfwrdt < PERC_LOW || percfwrdt > PERC_HIGH)
        throw PiaException(PIA_IDS_PERCFWRD); }
   // Description: Checks backward projection indicator in general case.
   //
   // Throws: PiaException of type PIA_IDS_PROJBACK1 if backward projection
   //   indicator is not in the earn_proj_type range.
   //
   // Arguments:
   //   projbackt: Projection indicator to check.
   static void projbackCheck1( int projbackt )
   { if (projbackt != (int)NO_PROJ && projbackt != (int)AVGWAGE_PROJ &&
       projbackt != (int)CONSTANT_PROJ)
        throw PiaException(PIA_IDS_PROJBACK1); }
   // Description: Checks backward projection indicator in backward projection
   //   case.
   //
   // Throws: PiaException of type PIA_IDS_PROJBACK2 if backward projection
   //   indicator is not an earn_proj_type other than NO_PROJ.
   //
   // Arguments:
   //   projbackt: Projection indicator to check.
   static void projbackCheck2( int projbackt )
   { if (projbackt != (int)NO_PROJ && projbackt != (int)AVGWAGE_PROJ)
        throw PiaException(PIA_IDS_PROJBACK2); }
   // Description: Checks forward projection indicator in general case.
   //
   // Throws: PiaException of type PIA_IDS_PROJFWRD1 if forward projection
   //   indicator is not in the earn_proj_type range.
   //
   // Arguments:
   //   projfwrdt: Projection indicator to check.
   static void projfwrdCheck1( int projfwrdt )
   { if (projfwrdt != (int)NO_PROJ && projfwrdt != (int)AVGWAGE_PROJ &&
       projfwrdt != (int)CONSTANT_PROJ)
        throw PiaException(PIA_IDS_PROJFWRD1); }
   // Description: Checks forward projection indicator in forward projection
   // case.
   //
   // Throws: PiaException of type PIA_IDS_PROJFWRD2 if forward projection
   //   indicator is not an earn_proj_type other than NO_PROJ.
   //
   // Arguments:
   //   projfwrdt: Projection indicator to check.
   static void projfwrdCheck2( int projfwrdt )
   { if (projfwrdt != (int)NO_PROJ && projfwrdt != (int)AVGWAGE_PROJ)
        throw PiaException(PIA_IDS_PROJFWRD1); }
};
