// $Id: LawChange.h 1.7 2006/01/05 09:38:28EST 044579 Exp  $
//
// Declarations needed for class to manage LawChange parameters.

#pragma once

#include <vector>
#include <string>
#include <iosfwd>

// {group:Calculation Classes}
// Description: The basic class that manages all of the extra data and
//   functions required for one change in law concerning benefit calculations.
//
// Remarks: This class has enough data for simple law-changes. If a particular
//   law-change requires additional data, it should subclass from this class
//   and add the additional data.
class LawChange
{
   public:
      // Short names of changes in law.
      enum lawChangeType {
         PRE1977LAW,  // Pre-1977 law benefits.
         BPFRACWAGE,  // Bend points increasing at fraction of wage rate.
         BPCPI,  // Bend points increasing at CPI rate.
         EARNINDCPI,  // Earnings indexed to CPI rate.
         NEWFORMULA,  // New benefit formula.
         BPSPECRATE,  // Bend points increasing at specified rates.
         BPMINCONST,  // Bend points increasing at wage rate minus a constant.
         AGE65COMP,  // Change to age-65 computation point.
         DECLINEPERC,  // Declining benefit formula percents.
         NOCPIELIG,  // No benefit increase in year of eligibility.
         NOREINDWID,  // No re-indexed widow guarantee.
         WINDFALL,  // No windfall elimination provision.
         NEWSPECMIN,  // New special minimum (years and/or amount).
         NOOLDSTART,  // No old-start method.
         DIDROP5,  // 5-year DI dropout.
         TRANSGUAR1,  // Use earnings after age 62 for transitional guarantee.
         MARRLENGTH,  // Number of years married for divorced spouse benefit.
         TRANSGUAR3,  // Use average of type-16 and aime for transitional guar.
         ALLEARN,  // Use all earnings in AIME computation.
         DROPOUTCHG,  // Reduce number of dropout years.
         RETTESTCHG,  // Retirement test ad hoc change.
         SANFORD1,  // Original Sanford proposal (3% to retirement).
         NOPIATABLE,  // No PIA table method.
         NOTRANSGUAR,  // No transitional guarantee method.
         TAXRATECHG,  // Tax rate change.
         SANFORD2,  // Second Sanford notch prop (3.25% to age 62; red yrs).
         NRACHANGE,  // Change full retirement age.
         CHILDCARECREDIT,  // Child care credit years.
         RETROWAGEIND,  // Retroactive wage indexing.
         TRANSGUAR4,  // Corman-Myers COLA limitation (notch proposal).
         WIFEFACTOR,  // Reduce aged wife factor to .33.
         WIDFACTOR,  // Modify the aged widow(er) benefit calculation.
         TAXBENCHG,  // Change in tax on benefits provision.
         PSAACCT,  // Add PSA accounts.
         WAGEBASECHG,  // Ad hoc wage base change.
         STATELOCAL,  // Cover State and local employees.
         FEDERAL,  // Cover Federal employees.
         CHILDCAREDROPOUT,  // Child care dropout years.
         COLACHANGE,  // Change in the COLA increase.
         NODIBGUAR, // No DIB guarantee.
         MAXLCH  // Number of different law changes.
      };
      // Earliest starting year for changes.
      static int startYearLC;
      // Latest year for changes.
      static int endYearLC;
   protected:
      // Last year of proposal to change law.
      int endYear;
      // Indicators for change from present law.
      int ind;
      // Description: Type of phase-in for this law-change.
      //
      // Remarks:
      //   0 if phased in by year of eligiblity,
      //   1 if effective immediately.
      int phaseType;
      // First year of proposal to change law.
      int startYear;
      // Title of this law-change.
      std::string titleLC;
public:
   LawChange();
   LawChange( const std::string& newTitle );
   virtual ~LawChange();
   // Description: Returns ending year for this law-change.
   //
   // Returns: Ending year for this law-change.
   int getEndYear() const { return endYear; }
   std::vector< std::string > getFirstLine( std::istream& infile );
   // Description: Returns indicator for this law-change.
   //
   // Returns: Indicator for this law-change.
   int getInd() const { return ind; }
   // Description: Returns starting year for this law-change.
   //
   // Returns: Starting year for this law-change.
   int getStartYear() const { return startYear; }
   // Description: Returns indicator for type of phase-in.
   //
   // Returns: Indicator for type of phase-in.
   int getPhaseType() const { return phaseType; }
   // Description: Returns title of this law change.
   //
   // Returns: Title of this law change.
   std::string getTitle() const { return titleLC; }
   bool isEffective( int eligYear, int benYear ) const;
   virtual void prepareStrings( std::vector<std::string> &outputString );
   virtual void read( std::istream& infile );
   // Description: Sets ending year for this law-change.
   //
   // Arguments:
   //   newEndYear:  New ending year for this law-change.
   void setEndYear( int newEndYear ) { endYear = newEndYear; }
   // Description: Sets change indicator for this law-change.
   //
   // Arguments:
   //   newInd:  New change indicator for this law-change.
   void setInd( int newInd ) { ind = newInd; }
   // Description: Sets starting year for this law-change.
   //
   // Arguments:
   //   newStartYear:  New starting year for this law-change.
   void setStartYear( int newStartYear) { startYear = newStartYear; }
   // Description: Sets indicator for type of phase-in.
   //
   // Arguments:
   //   newPhaseType:  New indicator for type of phase-in.
   void setPhaseType( int newPhaseType ) { phaseType = newPhaseType; }
   virtual void write( std::ostream& out ) const;
   void writeFirstLine( std::ostream& out ) const;
   static void bendPointCheck( double bendPoint );
   static void bpincCheck( double bpinc );
   static void constantPercCheck( double constantPerc );
   static void startCheck( int start );
   static void percCheck ( double perc );
   static void proportionBendPointsCheck  ( double proportionBendPoints );
   // Description: Sets earliest possible starting year for a law-change.
   //
   // Arguments:
   //   newStartYearLC: Earliest possible starting year for a law-change.
   static void setStartYearLC( int newStartYearLC )
      { startYearLC = newStartYearLC; }
   // Description: Sets latest possible ending year for a law-change.
   //
   // Arguments:
   //   newEndYearLC: Latest possible starting year for a law-change.
   static void setEndYearLC( int newEndYearLC ) { endYearLC = newEndYearLC; }
};
