// $Id: LawChangeCHILDCARECREDIT.h 1.7 2006/11/08 10:58:59EST 044579 Development  $
//
// Declarations needed for class to manage parameters for a proposal to
// provide child care credits.

#pragma once

#include "LawChange.h"

// {group:Calculation Classes}
// Description: Parameters for a proposal to provide child care credits.
//
// Remarks: There are 2 types of child care credit proposals handled by this
// class. The first type provides an earnings credit for anybody that has an
// eligible child in care. To be eligible, the child must be under a specified
// age (maxAge). The amount of the credit is equal to a specified percentage
// (fqRatio) of the wage base in the year of the credit. The number of years
// that the person is eligible for the credit is specifed (maxYears), and the
// creditable years are determined such that the person's PIA will be
// maximized (i.e., not just taking the first years in which the person is
// elgible for the credit.) Although the proposal starts in a specified year,
// the historical earnings (before that start year) used for the person's
// PIA calculation may or may nott have credits applied. This is specified
// by childCareYearsBeforeEffYear.
//
// The second type of child care credit proposal handled by this class also
// provides earnings credits to those who have an eligible child in care.
// But these credits are based on the person earnings history up to the point
// of the potential credit. The person uses the his/her top years of earnings.
// The number of these years is specified (numTopEarnYears). After finding
// this group of top years of earnings, some extreme years (the years with the
// highest and lowest of earnings) are dropped out of this group. The number
// of extreme years is specified (numDropExtremeEarnYears). For example,
// dropping 2 extreme years means to drop the 2 highest and 2 lowest years of
// earnings. The remaining years of earnings in the selected group are
// averaged together and a specified percentage of that average (avgEarnPct)
// is used as the credit amount. The other parameters are the same as the
// first version of the child care credit proposal, except that fqRatio is
// not used.
//
// One additional note is that for both versions of the proposal, the credit
// given is not added on to any earnings that the person already has in that
// year. It is the total amount of earnings the person is credited in that
// year. For example, if a person has $10,000 of earnings in the year, and the
// credit amount is calculated to be $15,000, then the new amount of earnings
// for that person in that year is $15,000 (i.e., $5,000 on top of his/her
// regular earnings for that year).
class LawChangeCHILDCARECREDIT : public LawChange
{
protected:
   // Description: Ratio of average earnings credited for a child care year.
   double fqRatio;
   // Description: Maximum age of child to get credit for a child care year.
   int maxAge;
   // Description: Maximum number of child care years.
   int maxYears;
   // Description: The percentage to be multiplied by the person's average
   // earnings to calculate the credit amount.
   double avgEarnPct;
   // Description: Number of highest earnings years to look at for the
   // calculation of the credit amount.
   int numTopEarnYears;
   // Description: Number of years of highest earnings to drop out of the
   // calculation of the credit amount.
   //
   // Remarks: Drop the "extreme" years. So if numTopEarnYears is 5, and
   // numDropExtremeEarnYears is 1, then we'll look at the highest 5 years of
   // earnings, and drop 1 extreme year at each end (i.e. the highest and
   // lowest earnings amounts) leaving the 3 middle years.
   int numDropExtremeEarnYears;
   // Description: Whether or not the person can get credited for years of
   // child-care before the proposal went into effect.
   bool childCareYearsBeforeEffYear;
public:
   LawChangeCHILDCARECREDIT();
   // Description: Returns the percentage to be multiplied by the person's
   // average earnings to calculate the credit amount.
   //
   // Returns: The percentage to be multiplied by the person's average
   //   earnings to calculate the credit amount.
   double getAvgEarnPct() const { return avgEarnPct; }
   // Description: Returns true if the person can be credited for child care
   // years before the first effective year of the proposal, false otherwise.
   //
   // Returns: True if the person can be credited for child care years before
   // the first effective year of the proposal, false otherwise.
   bool getChildCareYearsBeforeEffYear() const
      { return childCareYearsBeforeEffYear; }
   // Description: Returns ratio of average earnings credited for a child care
   //   year.
   //
   // Returns: Ratio of average earnings credited for a child care year.
   double getFqRatio() const { return fqRatio; }
   // Description: Returns maximum age of child to get credited for a
   //   child care year.
   //
   // Returns: Maximum age of child to get credited for a child care year.
   int getMaxAge() const { return maxAge; }
   // Description: Returns maximum number of child care years.
   //
   // Returns: Maximum number of child care years.
   int getMaxYears() const { return maxYears; }
   // Description: Returns the number of years of highest earnings to drop out
   // of the calculation of the credit amount.
   //
   // Returns: The number of years of highest earnings to drop out of the
   //   calculation of the credit amount.
   int getNumDropExtremeEarnYears() const { return numDropExtremeEarnYears; }
   // Description: Returns the number of highest earnings years to look at for
   // the calculation of the credit amount.
   //
   // Returns: The number of highest earnings years to look at for the
   // calculation of the credit amount.
   int getNumTopEarnYears() const { return numTopEarnYears; }
   void prepareStrings( std::vector< std::string > &outputString );
   void read( std::istream& infile );
   // Description: Sets the percentage to be multiplied by the person's average
   //   earnings to calculate the credit amount.
   //
   // Arguments:
   //   newAvgEarnPct: The percentage to be used.
   void setAvgEarnPct( double newAvgEarnPct ) { avgEarnPct = newAvgEarnPct; }
   // Description: Sets childCareYearsBeforeEffYear based on the argument.
   //
   // Remarks: Sets childCareYearsBeforeEffYear to true if the argument is 1,
   // false otherwise.
   //
   // Arguments:
   //   newChildCareYearsBeforeEffYear: 1 if childCareYearsBeforeEffYear should
   //     be true.
   void setChildCareYearsBeforeEffYear( int newChildCareYearsBeforeEffYear )
      { childCareYearsBeforeEffYear = (newChildCareYearsBeforeEffYear == 1); }
   // Description: Sets ratio of average earnings credited for a child care
   // year.
   //
   // Arguments:
   //   newFqRatio: New ratio of average earnings credited for a child care
   //     year.
   void setFqRatio( double newFqRatio ) { fqRatio = newFqRatio; }
   // Description: Sets maximum age of child to get credit for a child care
   // year.
   //
   // Arguments:
   //   newMaxAge: New maximum age of child.
   void setMaxAge( int newMaxAge ) { maxAge = newMaxAge; }
   // Description: Sets maximum number of child care years.
   //
   // Arguments:
   //   newMaxYears: New maximum number of child care years.
   void setMaxYears( int newMaxYears ) { maxYears = newMaxYears; }
   // Description: Sets the number of years of highest earnings to drop out of
   // the calculation of the credit amount.
   //
   // Arguments:
   //   newNumDropExtremeEarnYears: Number of years.
   void setNumDropExtremeEarnYears( int newNumDropExtremeEarnYears )
      { numDropExtremeEarnYears = newNumDropExtremeEarnYears; }
   // Description: Sets the number of highest earnings years to look at for the
   //   calculation of the credit amount.
   //
   // Arguments:
   //   newNumTopEarnYears: Number of years.
   void setNumTopEarnYears( int newNumTopEarnYears )
      { numTopEarnYears = newNumTopEarnYears; }
   void write( std::ostream& out ) const;
};
