// $Id: LawChangeDECLINEPERC.h 1.5 2006/11/08 10:58:59EST 044579 Development  $
//
// Declarations needed for class to manage parameters for a proposal to
// decrease the benefit formula percentages by a constant rate.

#pragma once

#include "LawChange.h"
#include "dbleann.h"

// {group:Calculation Classes}
// Description: Parameters for a proposal to decrease the benefit formula
//   percentages by a constant rate.
//
// Remarks: There are three additional data fields compared to the base class.
//
// If the indicator is greater than zero, it indicates the number of intervals
// of declining benefit formula percents. The rates of decline can vary for
// each benefit formula percentage within each interval, and between intervals.
class LawChangeDECLINEPERC : public LawChange
{
public:
   // Description: Array sizes.
   enum DimLimits {
      // Maximum number of declining formula percentages.
      MAXDECLINEPERC = 5
   };
protected:
   // Description: Percentage decline in benefit formula percents (3 percentages
   //   for up to MAXDECLINEPERC intervals).
   double factorDeclinePerc[3][MAXDECLINEPERC];
   // Description: Benefit formula percentages for PIA formula.
   DoubleAnnual* percpa[4];
   // Description: Years benefit formula percentage declines are effective.
   int yearDeclinePerc[MAXDECLINEPERC];
public:
   LawChangeDECLINEPERC();
   ~LawChangeDECLINEPERC();
   // Description: Returns benefit formula percentage for specified year.
   //
   // Returns: Benefit formula percentage for specified year.
   //
   // Arguments:
   //   year: Year of desired benefit formula percentage.
   //   percNum: Percentage number (0, 1, or 2 for 3-part formula; 0, 1, 2, or
   //     3 for 4-part formula).
   double getAltPercPia( int year, int percNum ) const
   { if (percNum >= 0 && percNum < 4) return((*percpa[percNum])[year]);
     else return(0.0); }
   // Description: Returns percentage decline in benefit formula percents.
   //
   // Returns: Percentage decline in benefit formula percents.
   //
   // Arguments:
   //   yearNum: Number of declining percentage interval (0 to
   //     jind(DECLINEPERC) - 1).
   //   percNum: Percentage number (0, 1, or 2 for 3-part formula).
   double getFactor( int yearNum, int percNum ) const
      { return factorDeclinePerc[percNum][yearNum]; }
   // Description: Returns year that declining benefit formula percentage is
   //   effective.
   //
   // Returns: Year that declining benefit formula percentage is effective.
   //
   // Arguments:
   //   yearNum: Number of declining benefit formula percentage interval (0 to
   //     ind - 1).
   int getYear( int yearNum ) const { return yearDeclinePerc[yearNum]; }
   void percPiaCal();
   void prepareStrings( std::vector<std::string> &outputString );
   void read( std::istream& infile );
   // Description: Sets percentage decline in benefit formula percents.
   //
   // Arguments:
   //   yearNum: Number of declining percentage interval (0 to ind - 1).
   //   percNum: Percentage number (0, 1, or 2 for 3-part formula).
   //   newFactor: New percentage decline.
   void setFactor( int yearNum, int percNum, double newFactor )
      { factorDeclinePerc[percNum][yearNum] = newFactor; }
   // Description: Sets year that declining benefit formula percentage is
   //   effective.
   //
   // Arguments:
   //   yearNum: Number of declining benefit formula percentage interval (0 to
   //     ind - 1).
   //   year: New year of declining benefit formula percentage.
   void setYear( int yearNum, int year ) { yearDeclinePerc[yearNum] = year; }
   void write( std::ostream& out ) const;
};
