// $Id: LawChangePSAACCT.h 1.5 2005/10/06 11:56:15EDT 044579 Exp  $
//
// Declarations needed for class to manage parameters for a proposal to
// introduce individual accounts.

#ifndef __LAWCHANGEPSAACCT_H
#define __LAWCHANGEPSAACCT_H

#include "LawChange.h"
#include "Psa.h"

// {group:Calculation Classes}
// Description: Parameters for a proposal to introduce individual accounts.
//
// Remarks: There are many additional data fields compared to the base class.
// If the indicator is set (greater than 0), then there are individual accounts
// with contributions of a certain percent of covered wages, where the
// indicator equals the number of different contribution rates over time.
class LawChangePSAACCT : public LawChange
{
   public:
      // Array sizes.
      enum DimLimits {
         MAXCONTRIBRATEYEARS = 100 // Maximum number of PSA contribution rates.
      };
   protected:
      // Indicator for whether or not a person starts getting an annuity
      //   from his PSA upon beginning DI benefits.
      bool   annuityToDib;
      // Percentage of contribution going to spouse.
      double percToSpouse;
      // Annual PSA administrative fee, stored in dollars.
      double adminFee[Psa::NUM_INVESTMENTS];
      // The interest rate used for the PSA annuities.
      double annIntRate;
      // The indicator on whether or not the PSA annuity will offset the
      //   regular OASDI benefits.
      bool annuityOffset;
      // Type of annuities bought from PSA (0 if always straight life, 1 if
      //   using joint and 2/3 survivor for married people).
      Psa::AnnuityType annuityType;
      // Description: Indicator for whether PSA buys an annuity.
      //
      // Remarks: True indicates that the PSA will be used to buy an annuity,
      //   thus the amount in the PSA is zeroed out after the purchase. False
      //   indicates that the an annuity is paid from the PSA, so the PSA is
      //   diminished each year by the amount of monthly annuity payments
      //   over the course of the year.
      bool buysAnnuity;
      // Bend points for the PSA contribution rate, as a percent of the
      //   wage base.
      double contribBps[MAXCONTRIBRATEYEARS][Psa::MAXCONTRIBRATES];
      // Description: Indicator defining how the PSA contributions should be
      //   distributed.
      //
      // Remarks:
      //
      //   0 - Use built-in split that varies by age.
      //   1 - Constant (user-specified) split, stored in psaInvestPerc.
      int contribDist;
      // PSA contribution rates (used if PSAACCT is set).
      double contribRates[MAXCONTRIBRATEYEARS][Psa::MAXCONTRIBRATES];
      // Starting age for PSA contributions.
      int contribStartAge;
      // Starting years correspodning to the PSA contribution rates.
      int contribYears[MAXCONTRIBRATEYEARS];
      // Mean to be used for the return rate on each investment.
      double investMean[Psa::NUM_INVESTMENTS];
      // Proportion of PSA contributions going to each investment.
      double investPerc[Psa::NUM_INVESTMENTS];
      // Standard deviation to be used for the return rate on each
      //   investment.
      double investStdDev[Psa::NUM_INVESTMENTS];
      // True if person gets entire PSA as a lump sum payment if attains
      //   FRA prior to becoming eligible for an annuity from the PSA,
      //   false otherwise.
      bool lumpSumAtFra;
      // Percent of PSA paid as a lump sum payment upon conversion to annuity.
      double lumpSumPerc;
      // Annual PSA management fee, stored as a multiplier (i.e. .01 means
      //   that 1% of the fund goes to fees).
      double mngmtFee[Psa::NUM_INVESTMENTS];
      // Number of bend points in the contribution formula for PSA's.
      //   Currently, can be 0 or 1.
      int numContribBps;
      // Description: Indicator for method by which PSA annuities get taxed.
      //
      // Remarks:
      //
      //   0 - Not taxed at all.
      //   1 - Taxed like Social Security benefits.
      //   2 - Taxed like regular income.
      int taxAnnuityInd;
      // Description: Indicator regarding the transfer of the PSA to a spouse
      //   upon the death of the person.
      //
      // Remarks:
      //
      //   0 - No transfer.
      //   1 - Transfer to current (at time of death) spouse, if any.
      //   2 - Transfer to most recent spouse (current or divorced).
      int toSpouseOnDeathInd;
      // True if the PSA investment accounts get re-balanced each year so as
      //   to preserve the same percentage split used for the contributions,
      //   false otherwise.
      bool rebalanceAccounts;
   // Description: Indicator regarding the use of unisex annuity factors.
   Psa::UnisexType unisexInd;
public:
   LawChangePSAACCT();
   // Description: Returns the administrative fee for the specified investment.
   //
   // Returns: The administrative fee for the specified investment.
   //
   // Arguments:
   //   type: Type of investment.
   double getAdminFee( Psa::InvestmentType type ) const
      { return adminFee[type]; }
   // Description: Returns the interst rate for the PSA annuities.
   //
   // Returns: The interest rate for the PSA annuities.
   double getAnnIntRate() const { return annIntRate; }
   // Description: Returns true if the PSA annuity offsets the regular
   //   OASDI benefits, false otherwise.
   //
   // Returns: True if the PSA annuity offsets the regular OASDI benefits,
   //  false otherwise.
   bool getAnnuityOffset() const { return annuityOffset; }
   // Description: Returns true if a person converts his PSA into an
   //   annuity upon beginning DI benefits.
   //
   // Returns: True if a person converts his PSA into an annuity upon
   //   beginning DI benefits.
   bool getAnnuityToDib() const { return annuityToDib; }
   // Description: Returns the type of annuity (0 for always straight life, 1
   // for joint and 2/3 survivor if married).
   //
   // Returns: The annuity type.
   Psa::AnnuityType getAnnuityType() const { return annuityType; }
   // Description: Returns true if the PSA is used to buy an annuity, false
   //   if it is just diminished by the amount of the annuity payments as
   //   the payments are made.
   //
   // Returns: True if the PSA is used to buy an annuity, false if it
   //   is just diminished by the amount of the annuity payments as
   //   the payments are made.
   bool getBuysAnnuity() const { return buysAnnuity; }
   // Description: Returns the percent of the wage base to be used as a
   //   bend point for the contribution rate to the PSA.
   //
   // Returns: the percent of the wage base to be used as a bend point
   //   for the contribution rate to the PSA.
   //
   // Arguments:
   //   yearNum: Year number.
   //   bpNum: Bend point number.
   double getContribBps( int yearNum, int bpNum ) const
      { return contribBps[yearNum][bpNum]; }
   // Description: Returns the type of distribution to use for PSA
   //  contributions.
   //
   // Returns: The type of distribution to use for PSA contributions.
   int getContribDist() const { return contribDist; }
   // Description: Returns PSA contribution rate.
   //
   // Returns: PSA contribution rate.
   //
   // Arguments:
   //   yearNum: Year number.
   //   rateNum: Rate number.
   double getContribRates( int yearNum, int rateNum ) const
      { return contribRates[yearNum][rateNum]; }
   // Description: Returns the starting age for PSA contributions.
   //
   // Returns: The starting age for PSA contributions.
   int getContribStartAge() const { return contribStartAge; }
   // Description: Returns PSA contribution year index.
   //
   // Returns: PSA contribution year index.
   //
   // Arguments:
   //   yearNum: Year number.
   int getContribYears( int yearNum ) const { return contribYears[yearNum]; }
   // Description: Returns mean to be used for the return rate on stocks.
   //
   // Returns: Mean to be used for the return rate on stocks.
   //
   // Arguments:
   //   type: Type of investment.
   double getInvestMean( Psa::InvestmentType type ) const
      { return investMean[type]; }
   // Description: Returns proportion of PSA contributions going to specified
   //   investment.
   //
   // Returns: Proportion of PSA contributions going to specified investment.
   //
   // Arguments:
   //   type: Type of investment.
   double getInvestPerc( Psa::InvestmentType type ) const
      { return investPerc[type]; }
   // Description: Returns standard deviation to be used for the return rate on
   //   specified investment.
   //
   // Returns: Standard deviation to be used for the return rate on
   //   specified investment.
   //
   // Arguments:
   //   type: Type of investment.
   double getInvestStdDev( Psa::InvestmentType type ) const
      { return investStdDev[type]; }
   // Description: Returns true if the entire PSA is paid as a lump sum to a
   // person who attains FRA and is not eligible for an annuity, false
   // otherwise.
   //
   // Returns: True if the entire PSA is paid as a lump sum to a person
   // who attains FRA and is not eligible for an annuity, false otherwise.
   bool getLumpSumAtFra() const { return lumpSumAtFra; }
   // Description: Returns the percent of the PSA that is paid out as a lump
   //   sum upon conversion to annuity.
   //
   // Returns: The percent of the PSA that is paid out as a lump sum upon
   //   conversion to annuity.
   double getLumpSumPerc() const { return lumpSumPerc; }
   // Description: Returns the management fee for the specified investment.
   //
   // Returns: The management fee for the specified investment.
   //
   // Arguments:
   //   type: Type of investment.
   double getMngmtFee( Psa::InvestmentType type ) const
      { return mngmtFee[type]; }
   // Description: Returns the number of bend point in the contribution
   //   rate for the PSA's.
   //
   // Returns: The number of bend point in the contribution rate for
   //   the PSA's.
   int getNumContribBps() const { return numContribBps; }
   // Description: Returns percentage of contribution going to spouse, if any.
   //
   // Returns: Percentage of contribution going to spouse, if any.
   double getPercToSpouse() const { return percToSpouse; }
   // Description: Returns true if the PSA investment accounts need to be
   //   re-balanced each year, false otherwise.
   //
   // Returns: True if the PSA investment accounts need to be re-balanced
   //   each year, false otherwise.
   bool getRebalanceAccounts() const { return rebalanceAccounts; }
   // Description: Returns the indicator on how PSA annuities are taxed.
   //
   // Returns: The indicator on how PSA annuities are taxed.
   int getTaxAnnuityInd() const { return taxAnnuityInd; }
   // Description: Returns the indicator regarding the transfer of the PSA to a
   //   spouse upon death of the person.
   //
   // Returns: The indicator regarding the transfer of the PSA to a spouse upon
   //   death of the person.
   int getToSpouseOnDeathInd() const { return toSpouseOnDeathInd; }
   // Description: Returns the unisex indicator.
   //
   // Returns: The unisex indicaor.
   Psa::UnisexType getUnisexInd() const { return unisexInd; }
   void prepareStrings( std::vector<std::string> &outputString );
   void read( std::istream& infile );
   // Description: Sets the administrative fee for the specified investment.
   //
   // Arguments:
   //   type: Type of investment.
   //   newFee: New administrative fee.
   void setAdminFee( Psa::InvestmentType type, double newFee )
      { adminFee[type] = newFee; }
   // Description: Sets the interest rate for the PSA annuities.
   //
   // Arguments:
   //   newIntRate: The new interest rate to use for the PSA annuities.
   void setAnnIntRate( double newIntRate ) { annIntRate = newIntRate; }
   // Description: Sets the indicator for offsetting OASDI benefits with
   //   the PSA annuity.
   //
   // Arguments:
   //   newOffset: The new indicator.
   void setAnnuityOffset( int newOffset )
      { annuityOffset = (newOffset > 0) ? true : false; }
   // Description: Sets the indicator for whether or not a person starts
   //   getting an annuity from his PSA upon beginning DI benefits.
   //
   // Arguments:
   //   newAnnuityToDib: New indicator as to whether or not a person converts
   //     his PSA into an annuity upon getting DI benefits.
   void setAnnuityToDib( int newAnnuityToDib )
      { annuityToDib = (newAnnuityToDib > 0) ? true : false; }
   // Description: Sets the type of annuity (0 for always straight life, 1 for
   //   joint and 2/3 survivor if married).
   //
   // Arguments:
   //   newAnnuityType: New type of annuity.
   void setAnnuityType( Psa::AnnuityType newAnnuityType )
      { annuityType = newAnnuityType; }
   // Description: Sets the indicator for how the PSA is treated when the
   //   person starts getting an annuity payment.
   //
   // Arguments:
   //   newBuysAnnuity: Indicator how to treat the PSA once the annuity starts.
   void setBuysAnnuity( int newBuysAnnuity )
      { buysAnnuity = (newBuysAnnuity > 0) ? true : false; }
   // Description: Sets the percent of the wage base used a s a bend point
   //   for the contribution rate into the PSA's.
   //
   // Arguments:
   //   yearNum: Year number.
   //   bpNum: Bend point number.
   //   newBp: New bend point.
   void setContribBps( int yearNum, int bpNum, double newBp )
      { contribBps[yearNum][bpNum] = newBp; }
   // Description: Sets the type of distribution to use for PSA contributions.
   //
   // Arguments:
   //   newContribDist: The new type of distribution to use.
   void setContribDist( int newContribDist )
      { contribDist = newContribDist; }
   // Description: Sets psa contribution rate.
   //
   // Arguments:
   //   yearNum: Year number.
   //   rateNum: Rate number.
   //   newContribRate: New PSA contribution rate.
   void setContribRates( int yearNum, int rateNum, double newContribRate )
      { contribRates[yearNum][rateNum] = newContribRate; }
   // Description: Sets the starting age for PSA contributions.
   //
   // Arguments:
   //   newStartAge: New starting age for PSA contributions.
   void setContribStartAge( int newStartAge )
      { contribStartAge = newStartAge; }
   // Description: Sets PSA contribution year index.
   //
   // Arguments:
   //   yearNum: Year number.
   //   newYear: New PSA contribution year index.
   void setContribYears( int yearNum, int newYear )
      { contribYears[yearNum] = newYear; }
   // Description: Sets mean to be used for the return of specified investment.
   //
   // Arguments:
   //   theType: Type of investment.
   //   newMean: New mean.
   void setInvestMean( Psa::InvestmentType theType, double newMean )
      { investMean[theType] = newMean; }
   // Description: Sets proportion of psa contributions going to specified
   //   investment.
   //
   // Arguments:
   //   theType: Type of investment.
   //   newInvestPerc: New proportion of PSA contributions going to specified
   //     investment.
   void setInvestPerc( Psa::InvestmentType theType, double newInvestPerc )
      { investPerc[theType] = newInvestPerc; }
   // Description: Sets standard deviation to be used for the return rate on
   //   specified investment.
   //
   // Arguments:
   //   theType: Type of investment.
   //   newStdDev: New standard deviation.
   void setInvestStdDev( Psa::InvestmentType theType, double newStdDev )
      { investStdDev[theType] = newStdDev; }
   // Description: Sets the indicator for whether or not a person receives
   //   the entire PSA as a lump sum if they attain FRA and are not
   //   eligible for an annuity.
   //
   // Arguments:
   //   newLumpSumAtFra: New indicator for receiving the entire PSA as a lump
   //     sum.
   void setLumpSumAtFra( int newLumpSumAtFra )
      { lumpSumAtFra = (newLumpSumAtFra > 0) ? true : false; }
   // Description: Sets the new percent of the PSA that is paid out as a lump
   //   sum upon conversion to annuity.
   //
   // Arguments:
   //   newLumpSumPerc: New percent of the PSA that is paid out as a lump sum.
   void setLumpSumPerc( double newLumpSumPerc )
      { lumpSumPerc = newLumpSumPerc; }
   // Description: Sets the management fee for the specified investment.
   //
   // Arguments:
   //   type: Type of investment.
   //   newFee: New management fee.
   void setMngmtFee( Psa::InvestmentType type, double newFee )
      { mngmtFee[type] = newFee; }
   // Description: Sets the number of bend point in the contribution rate
   //   into the PSA's.
   //
   // Arguments:
   //   newNumBps: new number of bend points.
   void setNumContribBps( int newNumBps ) { numContribBps = newNumBps; }
   // Description: Sets percentage of contribution going to spouse, if any.
   //
   // Arguments:
   //   newPercToSpouse: New percentage going to spouse.
   void setPercToSpouse( double newPercToSpouse )
      { percToSpouse = newPercToSpouse; }
   // Description: Sets the rebalanceAccounts indicator.
   //
   // Arguments:
   //   newRebalanceAccounts: New rebalanceAccounts.
   void setRebalanceAccounts( int newRebalanceAccounts )
      { rebalanceAccounts = (newRebalanceAccounts > 0) ? true : false; }
   // Description: Sets the indicator on how PSA annuities are taxed.
   //
   // Arguments:
   //   newInd: New indicator on how PSA annuities are taxed.
   void setTaxAnnuityInd( int newInd ) { taxAnnuityInd = newInd; }
   // Description: Sets the indicator regarding the transfer of the PSA to a
   //   spouse upon death of the person.
   //
   // Arguments:
   //   newToSpouseOnDeathInd: New indicator regarding the transfer of the PSA
   //     to a spouse upon the person's death.
   //
   //       0 - No transfer.
   //       1 - Transfer to current (at time of death) spouse, if any.
   //       2 - Transfer to most recent spouse (current or divorced).
   void setToSpouseOnDeathInd( int newToSpouseOnDeathInd )
      { toSpouseOnDeathInd = newToSpouseOnDeathInd; }
   // Description: Sets unisex indicator.
   //
   // Arguments:
   //   newUnisexInd: New indicator for unisex annuity factors.
   void setUnisexInd( Psa::UnisexType newUnisexInd )
      { unisexInd = newUnisexInd; }
   void write( std::ostream& out ) const;
};

#endif  // __LAWCHANGEPSAACCT_H
