// $Id: OldStart.h 1.18 2005/10/06 11:56:16EDT 044579 Exp  $
//
// Declarations needed for class to manage old-start pia calculation functions.

#pragma once
#ifndef __OLDSTART_H
#define __OLDSTART_H

#include "OldPia.h"

// {group:Calculation Classes}
// Description: Manages the functions required for calculation of
//   an old start Social Security benefit.
//
// Remarks: Manages the functions required for calculation of
//   an old start Social Security benefit.
class OldStart : public OldPia
{
   public:
      // Description: Old-start PIA calculation methods.
      enum OldStartType {
         OS1939 = 0,  // 1939 Amendments.
         OS1950,      // 1950 old-start.
         OS1958,      // 1950 old-start with 1958 conversion table.
         OS1965,      // 1965 old-start.
         OS1967,      // 1967 old-start.
         OS1977_78,   // 1977 old-start with Jan 1978 old-law PIA table.
         OS1977_79,   // 1977 old-start with Dec 1978 frozen PIA table.
         OS1990,      // 1990 old-start.
         NUM_OLD_START_TYPES  // Number of old-start PIA methods (8).
      };
      // Description: Factor for amount of increment per year (0.01).
      static const double INC_PER_YEAR;
   public:
      // Bend points for MFB formula.
      BendMfb bendMfb;
      // Wage-indexed MFB formula percentages.
      PercMfb percMfb;
      // Portion of AME in each interval of PIB formula.
      double portionAme[2];
      // Portion of AIME PIA in each interval of MFB formula.
      double portionPiaElig[4];
   protected:
      // Old-start AME.
      int    ameOs;
      // Divisor for old-start pre-1951 earnings.
      int    divisorOs;
      // The date of entitlement.
      const DateMoyr entDate;
      // Number of old-start increment years.
      int    incyrs;
      // Description: Old-start PIA calculation method.
      OldStartType methodOs;
      // Old-start PIB before incrementing.
      double pib;
      // Old-start PIB after incrementing.
      double pibInc;
public:
   OldStart( const WorkerDataGeneral& newWorkerData, const PiaData& newPiaData,
      const PiaParams& newPiaParams, int newMaxyear,
      const DateMoyr& newEntDate );
   virtual ~OldStart();
   void calculate();
   // Description: Returns old-start AME.
   //
   // Returns: Old-start AME.
   int getAmeOs() const { return(ameOs); }
   // Description: Returns divisor for old-start pre-1951 earnings.
   //
   // Returns: Divisor for old-start pre-1951 earnings.
   int getDivisorOs() const { return(divisorOs); }
   // Description: Returns number of old-start increment years.
   //
   // Returns: Number of old-start increment years.
   int getIncyrs() const { return(incyrs); }
   // Description: Returns old-start calculation method number.
   //
   // Returns: Old-start calculation method number.
   OldStartType getMethodOs() const { return(methodOs); }
   // Description: Returns old-start PIB before incrementing.
   //
   // Returns: Old-start PIB before incrementing.
   double getPib() const { return(pib); }
   // Description: Returns old-start PIB after incrementing.
   //
   // Returns: Old-start PIB after incrementing.
   double getPibInc() const { return(pibInc); }
   void imputeEarnings();
   int incrementCal() const;
   int incrementOldCal() const;
   void initialize();
   bool isApplicable();
   // Description: Returns true if the old-start method is 1977 or 1990.
   //
   // Returns: True if the old-start method is 1977 or 1990.
   bool isOs1977_90() const
      { return (methodOs == OS1977_78 || methodOs == OS1977_79 ||
        methodOs == OS1990); }
      OldStartType methodOsCal();
   void oldStart39Cal();
   void oldStart50Cal();
   // Description: Sets old-start AME.
   //
   // Arguments:
   //   newAmeOs: New old-start AME.
   void setAmeOs ( int newAmeOs ) { ameOs = newAmeOs; }
   // Description: Sets divisor for old-start pre-1951 earnings.
   //
   // Arguments:
   //   newDivisorOs: New divisor for old-start pre-1951 earnings.
   void setDivisorOs( int newDivisorOs ) { divisorOs = newDivisorOs; }
   // Description: Sets number of old-start increment years.
   //
   // Arguments:
   //   newIncyrs: New number of old-start increment years.
   void setIncyrs( int newIncyrs ) { incyrs = newIncyrs; }
   // Description: Sets old-start calculation method number.
   //
   // Arguments:
   //   newMethodOs: New old-start calculation method number.
   void setMethodOs( OldStartType newMethodOs ) { methodOs = newMethodOs; }
   // Description: Sets old-start PIB before incrementing.
   //
   // Arguments:
   //   newPib: New PIB before incrementing.
   void setPib( double newPib ) { pib = newPib; }
   // Description: Sets old-start PIB with increment years.
   void setPibInc()
      { pibInc = pib * (1.0 + INC_PER_YEAR * static_cast<double>(incyrs)); }
   static bool isApplicable( const PiaData& piaData );
private:
   OldStart operator=( OldStart newOldStart );
};

#endif  // __OLDSTART_H
