// $Id: PiaMethod.h 1.33 2006/01/05 09:38:28EST 044579 Exp  $
//
// Declarations needed for class to manage generic pia calculation functions.

#pragma once

#include "piadata.h"
#include "bendpia.h"
#include "bendmfb.h"
#include "percpia.h"
#include "percmfb.h"
#include "BenefitAmount.h"
class PiaParams;

// {group:Calculation Classes}
// Description: Parent of the various methods required for
//   calculation of a Social Security benefit.
//
// Remarks: Parent of the various methods required for
//   calculation of a Social Security benefit.
class PiaMethod
{
public:
   // Description: Type of windfall elimination (WEP).
   enum WindfallElimType {
      // WEP not applicable (no noncovered pension, or eligible before 1986).
      NOWINDFALLELIM,
      // WEP applicable, but have 30 years of coverage.
      HAS30YEARS,
      // PIA reduced by one-half of noncovered pension.
      ONEHALFPENSION,
      // First benefit formula factor reduced.
      REDUCEDPERC
   };
   // Description: Years of benefit increases applied.
   enum cpi_year_type {
      // First year.
      FIRST_YEAR,
      // First year starting with year of eligibility.
      YEAR_ELIG,
      // Last year ending with year of entitlement.
      YEAR_ENT,
      // Last year ending with year of benefit.
      YEAR_BEN,
      // Last year for table changes.
      YEAR_TABLE,
      // Number of start and end years of CPI increases.
      MAXIYCPI
   };
   // Description: Type of calculation method.
   enum pia_type {
      OLD_START,  // Old-start method.
      PIA_TABLE,  // Pia table method.
      WAGE_IND,  // Wage-indexed method.
      TRANS_GUAR,  // Transitional guarantee method.
      SPEC_MIN,  // Special minimum method.
      REIND_WID,  // Re-indexed widow(er) method.
      FROZ_MIN,  // Frozen minimum method.
      CHILD_CARE,  // Child care dropout years method.
      DIB_GUAR,  // Disability pia guarantee method.
      NO_PIA_TYPE  // No method.
   };
   // Description: Type of method applicability.
   enum app_type {
      // Method not applicable.
      NOT_APPLICABLE,
      // Method is applicable, but is not the highest or the support PIA.
      APPLICABLE,
      // Method has highest PIA.
      HIGH_PIA,
      // Method has support PIA.
      SUPPORT_PIA
   };
   // Description: Disability MFB cap indicator.
   enum dis_mfb_cap_type {
      NO_CAP,   // No disability MFB cap applies.
      PIA_1_5,  // 1.5 * PIA.
      AIME_85,  // 0.85 * AIME.
      PIA_1_0   // 1.0 * PIA.
   };
   // Description: Old-law PIA table indicator.
   enum table_type {
      NO_TABLE = -1,  // PIA table does not apply.
      PL_1952 = 0,    // 1952 Act.
      PL_1954,        // 1954 Act.
      PL_1958,        // 1958 Act.
      PL_1965,        // 1965 Act.
      PL_1967,        // 1967 Act.
      PL_1969,        // 1969 Act.
      PL_1971,        // 1971 Act.
      PL_1972,        // 1972 Act.
      PL_1973,        // 1973 Act.
      PL_1977,        // 1977 Act.
      NUM_TABLES      // Number of PIA tables.
   };
protected:
      // Average (indexed) monthly earnings.
      double ame;
      // Average (indexed) monthly earnings before application of 
      // totalization provision, if applicable.
      double ameTotal;
      // Percentage cap for MFB (for DI cases).
      double cap;
      // Sum of high n years of earnings.
      double ftearn;
      // Description: PIA method applicability.
      app_type iapp;
      // Description: Disability MFB cap indicator.
      dis_mfb_cap_type indCap;
      // Description: Type of benefit, set even if Statement case.
      WorkerDataGeneral::ben_type ioasdi;
      // Description: Number of applicable PIA table for each method.
      table_type tableNum;
      // Temporary pia.
      double piasub;
      // Calculation method.
      enum pia_type methodType;
      // Temporary mfb.
      double mfbsub;
      // True if old Pebs assumptions (with 1-percent real wage gain).
      bool pebsAssumptions;
      // PIA at eligibility before application of totalization
      // provision, if applicable (0 otherwise).
      BenefitAmount piaEligTotal;
      // PIA before application of totalization provision, if
      // applicable (0 otherwise).
      BenefitAmount piaTotal;
      // PIA at eligibility before application of windfall provision,
      // if applicable (0 otherwise).
      BenefitAmount piaWindfall;
      // Title of this method.
      const std::string methodTitle;
      // Pointer to worker data.
      const WorkerDataGeneral& workerData;
      // Pointer to pia calculation data.
      const PiaData& piaData;
      // Pointer to pia calculation parameters.
      const PiaParams& piaParams;
      // Indicator for values having been set (via setAt).
      bool dirty;
      // Description: Years of benefit increases applied.
      int yearCpi[MAXIYCPI];
   public:
      // Description: Indicators for use of earnings.
      //
      // Remarks:
      //
      // 1 - earnings are among highest n.
      // 0 - otherwise
      IntAnnual iorder;
      // MFB at eligibility and succeeding MFB's.
      DoubleAnnual mfbElig;
      // MFB at entitlement.
      BenefitAmount mfbEnt;
      // PIA at eligibility and succeeding PIA's.
      DoubleAnnual piaElig;
      // PIA at entitlement.
      BenefitAmount piaEnt;
      // Earnings multiplied by fq of base year.
      DoubleAnnual earnMultiplied;
      // Unordered earnings as in wearn.
      DoubleAnnual earnIndexed;
public:
   PiaMethod( const WorkerDataGeneral& newWorkerData,
      const PiaData& newPiaData, const PiaParams& newPiaParams,
      int newMaxyear, const std::string& newTitle, pia_type newMethod );
   virtual ~PiaMethod();
   // Description: Applies benefit increases to 1977 Amendments PIA or MFB.
   //
   // Returns: PIA or MFB increased by benefit increases.
   //
   // Remarks: Calls applyColas, with a fourth argument equal to the second
   // argument.
   //
   // Arguments:
   //   pia77: PIA or MFB to be increased.
   //   year1: First year of benefit increases.
   //   dateMoyr2: Date up to which to apply benefit increases.
   double applyColas( DoubleAnnual& pia77, int year1,
      const DateMoyr& dateMoyr2 ) const
      { return applyColasElig(pia77, year1, dateMoyr2, year1); }
   double applyColasElig( DoubleAnnual& pia77, int year1,
      const DateMoyr& dateMoyr2, int catchupYear ) const;
   // Description: Calculates pia and mfb for this method.
   virtual void calculate() = 0;
   double diMax( double amedimax, double piamax );
   // Description: Returns ame or aime.
   //
   // Returns: Ame or aime.
   double getAme() const { return(ame); }
   // Description: Returns average (indexed) monthly earnings before
   // application of totalization provision, if applicable.
   //
   // Returns: Average (indexed) monthly earnings before application
   // of totalization provision, if applicable.
   double getAmeTotal() const { return(ameTotal); }
   // Description: Returns indicator for use of pia calculation method.
   //
   // Returns: Indicator for use of pia calculation method.
   app_type getApplicable() const { return(iapp); }
   // Description: Returns percentage cap for MFB (for DI cases).
   //
   // Returns: Percentage cap for MFB (for DI cases).
   double getCap() const { return(cap); }
   // Description: Returns year that benefit increases were applied.
   //
   // Returns: Year that benefit increases were applied.
   //
   // Arguments:
   //   index: The type of year that benefit increases were applied for.
   int getCpiYear( cpi_year_type index ) const
   { return yearCpi[index]; }
   // Description: Returns sum of high n years of earnings.
   //
   // Returns: Sum of high n years of earnings.
   double getftearn() const { return(ftearn); }
   // Description: Returns disability MFB cap indicator.
   //
   // Returns: Disability MFB cap indicator.
   dis_mfb_cap_type getIndCap() const { return(indCap); }
   // Description: Returns type of benefit.
   //
   // Returns: Type of benefit.
   WorkerDataGeneral::ben_type getIoasdi() const { return(ioasdi); }
   // Description: Returns the calculation method.
   //
   // Returns: The calculation method.
   pia_type getMethod() const { return(methodType); }
   // Description: Returns PIA at eligibility before application of
   // totalization provision, if applicable (0 otherwise).
   //
   // Returns: PIA at eligibility before application of totalization
   // provision, if applicable (0 otherwise).
   BenefitAmount getPiaEligTotal() const { return(piaEligTotal); }
   // Description: Returns PIA before application of totalization provision,
   //   if applicable (0 otherwise).
   //
   // Returns: PIA before application of totalization provision,
   //   if applicable (0 otherwise).
   BenefitAmount getPiaTotal() const { return(piaTotal); }
   // Description: Returns PIA at eligibility before application of windfall
   //   provision, if applicable (0 otherwise).
   //
   // Returns: PIA at eligibility before application of windfall
   //   provision, if applicable (0 otherwise).
   BenefitAmount getPiaWindfall() const { return(piaWindfall); }
   // Description: Returns number of applicable PIA table.
   //
   // Returns: Number of applicable PIA table.
   table_type getTableNum() const { return(tableNum); }
   // Description: Returns title of this method.
   //
   // Returns: Title of this method.
   const std::string& getTitle() const { return(methodTitle); }
   virtual PiaMethod::WindfallElimType getWindfall() const;
   virtual int getYearsTotal() const;
   virtual void initialize();
   // Description: Returns true if this method is applicable.
   //
   // Returns: True if this method is applicable.
   virtual bool isApplicable() = 0;
   // Description: Returns old Pebs assumptions indicator.
   //
   // Returns: True if old Pebs assumptions (with 1-percent real wage gain).
   bool isPebsAssumptions() const { return pebsAssumptions; }
   double mfbCal( const double portionPiaElig[],
      const PercMfb& percMfbTemp, int year );
   void orderEarnings( int first, int last, int number );
   void prorate();
   // Description: Sets ame or aime.
   //
   // Arguments:
   //   newAme: New ame.
   void setAme( double newAme ) { ame = newAme; }
   void setAmeTotal();
   // Description: Sets indicator for use of pia calculation method.
   //
   // Arguments:
   //   newIapp: Indicator for use of method.
   void setApplicable( app_type newIapp ) { iapp = newIapp; }
   // Description: Sets percentage cap for MFB (for DI cases).
   //
   // Arguments:
   //   newCap: New cap.
   void setCap( double newCap ) { cap = newCap; }
   // Description: Sets indicator for non-zero values.
   void setDirty() { dirty = true; }
   // Description: Sets sum of high n years of earnings,
   //
   // Arguments:
   //   newftearn: New sum of high n years of earnings.
   void setftearn( double newftearn ) { ftearn = newftearn; }
   // Description: Sets disability MFB cap indicator.
   //
   // Arguments:
   //   newIndCap: New disability MFB cap indicator.
   void setIndCap( dis_mfb_cap_type newIndCap ) { indCap = newIndCap; }
   // Description: Sets the type of benefit.
   //
   // Arguments:
   //   ioasdit: New type of benefit.
   void setIoasdi( WorkerDataGeneral::ben_type ioasdit ) { ioasdi = ioasdit; }
   // Description: Sets Pebs assumptions indicator.
   //
   // Arguments:
   //   newPebsAssumptions: True if old Pebs assumptions (with 1-percent real
   //     wage gain).
   void setPebsAssumptions( bool newPebsAssumptions )
      { pebsAssumptions = newPebsAssumptions; }
   void setPiaEligTotal();
   void setPiaTotal();
   void setPiaWindfall();
   // Description: Sets number of applicable PIA table.
   //
   // Arguments:
   //   newTableNum: Number of applicable PIA table.
   void setTableNum( table_type newTableNum ) { tableNum = newTableNum; }
   void setYearCpi();
   int specMinYearsTotalCal();
   void totalEarnCal( int first, int last, int number );
   bool wep_app() const;
   static double dimaxElig( double amedimax, double piaelig,
      int year, double& capt, dis_mfb_cap_type& icap );
   static void setPortionPiaElig( double piasub, double piapart[],
      const BendMfb& bendMfbTemp );
   static int specMinYearsCal( double pre1951, const DoubleAnnual& earnings,
      int year, const DoubleAnnual& yocAmount );
   static int specMinYearsCal( double pre1951, const DoubleAnnual& earnings,
      int year, const DoubleAnnual& yocAmount, IntAnnual& iordert );
private:
   PiaMethod& operator=( PiaMethod& newPiaMethod );
};
