// $Id: Psa.h 1.30 2006/07/17 14:26:59EDT 044579 Development  $
//
// Declarations for class to manage PSA accounts.

#pragma once

#include <string>
#include <algorithm>
#include "dbleann.h"
#include "oactcnst.h"
#include "Sex.h"
class LawChangeArray;
class PiaParams;

// {group:Calculation Classes}
// Description: Manages the (theoretical) PSA accounts.
//
// Remarks: Manages the (theoretical) PSA accounts.
class Psa
{
public:
   // Description: Types of investments.
   //
   // Remarks: If any more investment types are added, the initialization
   // of contribPerc in Psa.cpp must be updated!
   enum InvestmentType {
      BONDS,  // Bonds.
      STOCKS,  // Stocks.
      NUM_INVESTMENTS  // Number of types of investments.
   };
   // Description: Constants for dimensioning.
   enum DimLimits {
      // The maximum number of bend points allowed in the contribution rate.
      MAXCONTRIBRATES = 3,
      // The maximum number of bend points allowed in the contribution rate,
      // when they vary by age.
      MAXCONTRIBAGES = 3,
      // The lowest age considered for annuities.
      FIRST_AGE = 16,
      // The number of ages for which annuities can start (16 to 75).
      NUM_ANNUITY_AGES = 60
   };
   // Description: Sex types for PSA annuities.
   enum PsaSexType {
      PSA_MALE,    // Male.
      PSA_FEMALE,  // Female.
      PSA_UNISEX,  // Unisex.
      PSA_NUM_SEX  // Number of sex types considered.
   };
   // Description: Codes for using unisex annuity factors.
   enum UnisexType {
      // Unisex for SL and JS annuities.
      UNISEX_SL_AND_JS,
      // Unisex for SL annuitites, sex-specific for JS annuities.
      UNISEX_SL_NOT_JS,
      // Sex-specific for SL annuitites, unisex for JS annuities.
      UNISEX_JS_NOT_SL,
      // Sex-specific for SL and JS annuitites.
      NO_UNISEX
   };
   // Description: Type of annuity.
   enum AnnuityType {
      ANNUITY_SL,  // Always straight life.
      ANNUITY_JS   // Using joint and 2/3 survivor for married people.
   };
   // Description: The age splits for contribution rates, when using rates
   // based on age.
   static const int contribAge[MAXCONTRIBAGES];
   // Description: Contribution rates when using rates based on age, as
   // opposed to using the same set rates for everybody.
   static const double contribPerc[NUM_INVESTMENTS][MAXCONTRIBAGES + 1];
protected:
   // Description: File information.
   std::string pathname;
   // Description: Annual administrative fee, stored in dollars.
   double adminFee[NUM_INVESTMENTS];
   // Description: Joint and 2/3 survivor annuity factors.
   //
   // Remarks: Factors are for 3 sexes (male, female, and unisex), for the
   //   years TRyear through 2100, and 60 worker ages (16-75), and 60
   //   spouse ages (16-75). Using these factors gives a monthly annuity.
   DoubleAnnual* annFacJS[PSA_NUM_SEX][NUM_ANNUITY_AGES][NUM_ANNUITY_AGES];
   // Description: Straight life annuity factors.
   //
   // Remarks: Factors are for 3 sexes (male, female, and unisex), for the
   //   years TRyear through 2100, and 60 ages (16-75). Using these factors
   //   gives a monthly annuity.
   DoubleAnnual* annFacSL[PSA_NUM_SEX][NUM_ANNUITY_AGES];
   // Description: Type of annuity.
   AnnuityType annuityType;
   // Description: Interest rate for calculating annuities.
   double intRate;
   // Description: Percentage of contribution going to investments (used
   //   if LawChange::PSAACCT is 1).
   double investPerc[NUM_INVESTMENTS];
   // Description: Mean of the normal distribution for investments.
   double mean[NUM_INVESTMENTS];
   // Description: Annual management fee, stored as a multiplier (i.e. 0.01
   // means that 1 percent of the fund is paid as a management fee).
   double mngmtFee[NUM_INVESTMENTS];
   // Description: Percentage of contribution going to spouse, if any.
   double percToSpouse;
   // Description: Standard deviation of the normal distribution for
   // investments.
   double stdDev[NUM_INVESTMENTS];
   // Description: Current Trustees Report year.
   int TRyear;
   // Description: Indicator for using unisex annuity factors.
   UnisexType unisexInd;
protected:
   // Description: Contribution rates by year.
   DoubleAnnual* contribRates[MAXCONTRIBRATES];
   // Description: Contribution rates bend point amounts (in dollars).
   DoubleAnnual* contribRatesBpAmounts[MAXCONTRIBRATES];
   // Description: Law change data.
   const LawChangeArray& lawChange;
   // Description: Benefit calculation parameters.
   const PiaParams& piaParams;
protected:
   PsaSexType getSexType( Sex::sex_type sex, bool isSLorUnmarried ) const;
   int getYearIndex( int year ) const;
   void interpolateFactors( int year1, int year2, PsaSexType sex );
   static int getAgeIndex( int theAge );
public:
   Psa( const LawChangeArray& newLawChange, const PiaParams& piaParams,
      int newAlt );
   ~Psa();
   double annuityCal( double amount, int year, Sex::sex_type sex,
      int workerAge, int spouseAge ) const;
   double annuityRevCal( double annuity, int year, Sex::sex_type sex,
      int WorkerAge, int spouseAge ) const;
   double contribCal( int eligYear, int benYear, double earnings ) const;
   // Description: Returns the annual administrative fee.
   //
   // Returns: The annual administrative fee.
   //
   // Arguments:
   //   type: Type of investment.
   double getAdminFee( InvestmentType type ) const { return adminFee[type]; }
   // Description: Returns psa contribution rate for a specified year.
   //
   // Returns: Psa contribution rate for a specified year.
   //
   // Arguments:
   //   benYear:  Year number.
   //   bpNum:  Bend point number.
   double getContribRate( int benYear, int bpNum ) const
      { return (*contribRates[bpNum])[benYear]; }
   double getInvestPerc( InvestmentType type, int age, int eligYear,
      int benYear ) const;
   // Description: Returns the mean of the normal distribution for bonds.
   //
   // Returns: The mean of the normal distribution for bonds.
   //
   // Arguments:
   //   type: Type of investment.
   double getMean( InvestmentType type ) const { return mean[type]; }
   // Description: Returns the annual management fee.
   //
   // Returns: The annual management fee.
   //
   // Arguments:
   //   type: Type of investment.
   double getMngmtFee( InvestmentType type ) const { return mngmtFee[type]; }
   // Description: Returns percentage of contribution going to spouse, if any.
   //
   // Returns: Percentage of contribution going to spouse, if any.
   double getPercToSpouse() const { return percToSpouse; }
   // Description: Returns standard deviation of the normal distribution for
   //   specified type.
   //
   // Returns: Standard deviation of the normal distribution for specified
   //   type.
   //
   // Arguments:
   //   type: Type of investment.
   double getStdDev( InvestmentType type ) const { return stdDev[type]; }
   void setParameters( int newAlt );
private:
   Psa operator=( Psa newPsa );
};
