// $Id: Secondary.h 1.14 2006/11/08 10:59:52EST 044579 Development  $
//
// Declarations needed for secondary benefit.

#pragma once

#include <vector>
#include "bic.h"
#include "datemoyr.h"
#include "Pifc.h"

// {group:Calculation Classes}
// Description: Contains the information about a secondary benefit.
//
// Remarks: Contains the information about a secondary benefit.
class Secondary
{
public:
   // Description: Bic code.
   Bic bic;
   // Description: Type of pia calculation method.
   Pifc pifc;
   // Description: The date of entitlement.
   DateMoyr entDate;
protected:
   // Description: Full-rate benefit (before actuarial reduction or reduction
   //   for family maximum).
   double fullBenefit;
   // Description: Benefit after reduction for family maximum but before
   //   actuarial reduction.
   double benefit;
   // Description: Benefit after actuarial reduction and reduction for family
   // maximum.
   double reducedBenefit;
   // Description: Benefit after all reductions and rounding.
   double roundedBenefit;
   // Description: Benefit factor.
   double benefitFactor;
   // Description: Months of actuarial reduction or delayed retirement credit.
   int monthsArdri;
   // Description: Reduction or increment factor.
   double arf;
public:
   Secondary();
   void check();
   void deleteContents();
   // Description: Returns true if this benefit is affected by family maximum.
   //
   // Returns: True if this benefit is affected by family maximum.
   bool eligibleForMax() const { return bic.eligibleForMax(); }
   // Description: Returns reduction or increment factor.
   //
   // Returns: Reduction or increment factor.
   double getArf() const { return arf; }
   // Description: Returns benefit after reduction for family maximum but
   //   before actuarial reduction.
   //
   // Returns: Benefit after reduction for family maximum but
   //   before actuarial reduction.
   double getBenefit() const { return benefit; }
   // Description: Returns benefit factor.
   //
   // Returns: Benefit factor.
   double getBenefitFactor() const { return benefitFactor; }
   // Description: Returns full-rate benefit (before actuarial reduction or
   //   reduction for family maximum).
   //
   // Returns: Full-rate benefit (before actuarial reduction or
   //   reduction for family maximum).
   double getFullBenefit() const { return fullBenefit; }
   // Description: Returns months of actuarial reduction or increment.
   //
   // Returns: Months of actuarial reduction or increment.
   int getMonthsArdri() const { return monthsArdri; }
   // Description: Returns benefit after actuarial reduction and reduction
   //   for family maximum.
   //
   // Returns: Benefit after actuarial reduction and reduction
   //   for family maximum.
   double getReducedBenefit() const { return reducedBenefit; }
   // Description: Returns benefit after all reductions and rounding.
   //
   // Returns: Benefit after all reductions and rounding.
   double getRoundedBenefit() const { return roundedBenefit; }
   // Description: Returns true if widow (aged or disabled) false otherwise.
   //
   // Returns: True if widow (aged or disabled), false otherwise.
   bool isWidow() const { return bic.isWidow(); }
   // Description: Sets reduction or increment factor.
   //
   // Arguments:
   //   newArf: New reduction or increment factor.
   void setArf( double newArf ) { arf = newArf; }
   // Description: Sets benefit after reduction for family maximum but
   //   before actuarial reduction.
   //
   // Arguments:
   //   newBenefit: New benefit.
   void setBenefit( double newBenefit ) { benefit = newBenefit; }
   // Description: Sets benefit factor.
   //
   // Arguments:
   //   newBenefitFactor: New benefit factor.
   void setBenefitFactor( double newBenefitFactor )
      { benefitFactor = newBenefitFactor; }
   // Description: Sets full-rate benefit (before actuarial reduction or
   //   reduction for family maximum).
   //
   // Arguments:
   //   newFullBenefit: New full benefit.
   void setFullBenefit( double newFullBenefit )
      { fullBenefit = newFullBenefit; }
   // Description: Sets months of actuarial reduction or increment.
   //
   // Arguments:
   //   newMonthsArdri: New months of actuarial reduction or increment.
   void setMonthsArdri( int newMonthsArdri )
      { monthsArdri = newMonthsArdri; }
   // Description: Sets benefit after actuarial reduction and reduction
   //   for family maximum.
   //
   // Arguments:
   //   newReducedBenefit: New reduced benefit.
   void setReducedBenefit( double newReducedBenefit )
      { reducedBenefit = newReducedBenefit; }
   // Description: Sets benefit after all reductions and rounding.
   //
   // Arguments:
   //   newRoundedBenefit: New rounded benefit.
   void setRoundedBenefit( double newRoundedBenefit )
      { roundedBenefit = newRoundedBenefit; }
};
