// $Id: TaxRates.h 1.9 2006/11/08 11:00:08EST 044579 Development  $
//
// Declarations for class to manage tax rates.

#pragma once

#include "BitAnnual.h"
#include "dbleann.h"

// {group:Calculation Classes}
// Description: Abstract parent of classes that manage the Social Security
//   payroll and self-employed tax rates.
//
// Remarks: Abstract parent of classes that manage the Social Security payroll
//   and self-employed tax rates. The child classes are TaxRatesLC and
//   TaxRatesPL.
class TaxRates
{
public:
   // Description: Type of tax rate (employee or self-employed).
   enum taxrateType {
      EMPLOYEERATE,    // Employee rate.
      SELFEMPRATE,     // Self-employed rate.
      NUMTAXRATETYPES  // Number of tax rate types.
   };
   // Description: Trust fund indicator.
   enum trustfundType {
      OASITF,  // Oasi trust fund.
      DITF,  // Di trust fund.
      OASDITF,  // Oasdi trust fund.
      HITF,  // Hi trust fund.
      OASDHITF,  // Oasdhi trust fund.
      NUMTRUSTFUNDTYPES  // Number of trust fund types.
   };
protected:
   // Description: Historical OASI employee/employer rates, 1937-2000.
   static const float oasieeerrate[];
   // Description: Historical DI employee/employer rates, 1937-2000.
   static const float dieeerrate[];
   // Description: Historical HI employee/employer rates, 1937-2000.
   static const float hieeerrate[];
   // Description: Historical OASI self-employed rates, 1937-2000.
   static const float oasiserate[];
   // Description: Historical DI self-employed rates, 1937-2000.
   static const float diserate[];
   // Description: Historical HI self-employed rates, 1937-2000.
   static const float hiserate[];
   // Description: Tax rates.
   //
   // Remarks: Tax rates are by tax rate type (see taxrateType) and trust
   //   fund (see trustfundType).
   DoubleAnnual* taxrate[2][5];
   // Description: Maximum year allowed.
   int maxyear;
public:
   TaxRates( int lastYear );
   virtual ~TaxRates();
   // Description: Returns an array of tax rates.
   //
   // Returns: An array of tax rates.
   //
   // Arguments:
   //   empeeEmper: FICA or SECA rate.
   //   trustfund: Type of taxes.
   const DoubleAnnual& getTaxRates( taxrateType empeeEmper,
      trustfundType trustfund ) const
      { return(*taxrate[empeeEmper][trustfund]); }
   // Description: Initializes all tax rates.
   virtual void initialize() = 0;
   void setHistRates( int lastYear );
   void setRates( int year, taxrateType taxType, double oasiRate,
      double diRate, double hiRate );
   void taxCal( const DoubleAnnual& earnOasdi, const BitAnnual& taxType,
      DoubleAnnual& taxes, trustfundType fundType ) const;
   double taxCal( int year, double earnings, taxrateType taxType,
      trustfundType fundType ) const;
};
