// $Id: UserAssumptions.h 1.16 2006/11/08 11:00:08EST 044579 Development  $
//
// Declarations needed for class to manage user-specified assumptions.

#pragma once

#include "cachup.h"
#include "biproj.h"
#include "dbleann.h"
#include "intann.h"
#include "AssumptionType.h"
#include "BaseChangeType.h"

// {group:Calculation Classes}
// Description: Manages all the assumptions about future average wage
//   increases, wage bases, benefit increases, and catch-up benefit increases
//   that a user might specify.
//
// Remarks: An instance of this class is used in the Anypia program.
class UserAssumptions
{
public:
   // Description: Projected increases in annual average earnings.
   DoubleAnnual awincproj;
   // Description: Wage base under present law, 1937 to maxyear, as read in.
   DoubleAnnual baseOasdi;
   // Description: Old-law wage bases (ignoring 1979-81 ad hoc increases), 1937
   //   to maxyear, as read in.
   DoubleAnnual base77;
   // Description: Projected benefit increases.
   DoubleAnnual biproj;
   // Description: Catch-up benefit increases.
   Catchup catchup;
   // Description: Title of user's average wages.
   std::string titleAw;
   // Description: Title of user's benefit increases
   std::string titleBi;
protected:
   // Description: Benefit increase assumption.
   AssumptionType::assum_type ialtbi;
   // Description: Answer to catch-up benefit increase prompt.
   char anscch;
   // Description: Average wage increase assumption.
   AssumptionType::assum_type ialtaw;
   // Description: Ad hoc base change (1=none, 2=change).
   BaseChangeType::change_type ibasch;
   // Description: Last benefit increase assumption read in.
   AssumptionType::assum_type jaltbi;
   // Description: Last average wage increase assumption read in.
   AssumptionType::assum_type jaltaw;
   // Description: Last ad hoc base change read in.
   BaseChangeType::change_type jbasch;
   // Description: Last year of projection.
   int lastYear;
   // Description: Current year (year after last known benefit increase).
   static int istart;
   // Description: Maximum year allowed.
   static int maxyear;
public:
   UserAssumptions( int newLastYear );
   void deleteContents();
   // Description: Returns catch-up benefit increase indicator.
   //
   // Returns: Catch-up benefit increase indicator.
   char getAnscch() const { return(anscch); }
   // Description: Returns average wage increase assumption.
   //
   // Returns: Average wage increase assumption.
   AssumptionType::assum_type getIaltaw() const { return(ialtaw); }
   // Description: Returns benefit increase assumption.
   //
   // Returns: Benefit increase assumption.
   AssumptionType::assum_type getIaltbi() const { return(ialtbi); }
   // Description: Returns wage base projection indicator.
   //
   // Returns: Wage base projection indicator.
   BaseChangeType::change_type getIbasch() const { return(ibasch); }
   // Description: Returns last average wage increase assumption read in.
   //
   // Returns: Last average wage increase assumption read in.
   AssumptionType::assum_type getJaltaw() const { return(jaltaw); }
   // Description: Returns last benefit increase assumption read in.
   //
   // Returns: Last benefit increase assumption read in.
   AssumptionType::assum_type getJaltbi() const { return(jaltbi); }
   // Description: Returns last wage base projection indicator read in.
   //
   // Returns: Last wage base projection indicator read in.
   BaseChangeType::change_type getJbasch() const { return(jbasch); }
   void pebsasmCheck() const;
   void resetTriggers();
   void setAnscch( char ansccht );
   void setIaltaw( int ialtawt );
   void setIaltbi( int ialtbit );
   void setIbasch( int ibascht );
   void setIstart( int istartt );
   // Description: Sets the previous average wage increase assumption.
   //
   // Arguments:
   //   jaltawt: New value of previous average wage increase assumption.
   void setJaltaw( AssumptionType::assum_type jaltawt ) { jaltaw = jaltawt; }
   // Description: Sets the previous benefit increase assumption.
   //
   // Arguments:
   //   jaltbit: New value of previous benefit increase assumption.
   void setJaltbi( AssumptionType::assum_type jaltbit ) { jaltbi = jaltbit; }
   // Description: Sets the previous wage base assumption.
   //
   // Arguments:
   //   jbascht: New value of previous wage base assumption.
   void setJbasch( BaseChangeType::change_type jbascht ) { jbasch = jbascht; }
   void setLastYear( int year );
   // Description: Sets average wage change assumption title.
   //
   // Arguments:
   //   newTitleAw: New title to use.
   void setTitleAw( const std::string& newTitleAw ) { titleAw = newTitleAw; }
   // Description: Sets benefit increase assumption title.
   //
   // Arguments:
   //   newTitleBi: New title to use.
   void setTitleBi( const std::string& newTitleBi ) { titleBi = newTitleBi; }
   // Description: Initializes previous assumptions indicators.
   void zeroJalt()
   {  jaltbi = jaltaw = AssumptionType::NONE;
      jbasch = BaseChangeType::NONE; }
   static void anscchCheck( char ansccht );
   // Description: Returns current year (year after last known benefit
   // increase).
   //
   // Returns: Current year (year after last known benefit increase).
   static int getIstart() { return(istart); }
   static void istartCheck( int istartt );
};
