// $Id: WorkerDataGeneral.h 1.46 2007/03/21 10:41:06EDT 044579 Development  $
//
// Declarations needed for parent class to manage worker data.

#pragma once

#include <bitset>
#include "boost/serialization/access.hpp"
#include "boost/date_time/gregorian/greg_date.hpp"
#include "datemoyr.h"
#include "cachup.h"
#include "biproj.h"
#include "qc.h"
#include "BitAnnual.h"
#include "QcArray.h"
#include "Sex.h"
#include "DisabPeriod.h"

// {group:Calculation Classes}
// Description: The abstract parent of all classes that manage the basic data
//   required to calculate a Social Security benefit.
//
// Remarks: The abstract parent of all classes that manage the basic data
//   required to calculate a Social Security benefit.
class WorkerDataGeneral
{
   friend class boost::serialization::access;
   // Description: Archives the data.
   //
   // Remarks: When the class Archive corresponds to an output archive, the
   // & operator is defined similar to <<. Likewise, when the class Archive is
   // a type of input archive, the & operator is defined similar to >>.
   //
   // Arguments:
   //   ar: The archive to use.
   //   version: The version of the archive to use.
   template< class Archive >
   void serialize( Archive& ar, const unsigned int version )
      { ar & childCareYears & qc & disabPeriod & qctot51td;
        ar & qctottd & joasdi & valdi & ibegin & iend & theBits;
        ar & birthDate & benefitDate & entDate & deathDate; }
public:
   // Description: Type of benefit calculation.
   enum ben_type {
      NO_BEN,  // No benefit calculation.
      OLD_AGE,  // Old-age benefit calculation.
      SURVIVOR,  // Survivor benefit calculation.
      DISABILITY,  // Disability benefit calculation.
      PEBS_CALC  // Statement benefit calculation.
   };
   // Description: Dimension limits.
   enum DimLimits {
      // Number of periods of disability.
      NUM_DISAB_PERIODS = 2
   };
   // Description: Members of the bitset.
   enum BitsetMembers {
      SEX_BIT = 0,    // The sex of worker (see Sex::sex_type).
      RECALC_BIT,     // Recalculation code.
      TOTALIZE_BIT,   // Totalization code.
      DEEMED_BIT,     // Indicator for deemed insured status possible.
      BLIND_BIT,      // Indicator for disability insured status allowed based
                      // on fully insured status only because of blindness.
      MQGE_BIT,       // Medicare earnings code.
      RESERVIST_BIT,  // Indicator for military reservist pension.
      INDEARN_BIT,    // Indicator for regular earnings available.
      INDRR_BIT,      // Indicator for railroad qcs and earnings available.
      QCSBYYEAR_BIT,  // Indicator for QCs by single year prior to 1978.
      DEADOAB_BIT     // Indicator for deceased OAB in survivor case.
   };
   // Description: Child care years, 1937 to 2100.
   BitAnnual childCareYears;
   // Description: Quarters of coverage, 1937 to maxyear.
   QcArray qc;
   // Description: Two periods of disability.
   DisabPeriod disabPeriod[NUM_DISAB_PERIODS];
protected:
   // Description: Number of quarters of coverage earned from 1951 through
   // 1977.
   unsigned char qctot51td;
   // Description: Number of quarters of coverage earned from 1937 through
   // 1977.
   unsigned char qctottd;
   // Description: Type of benefit calculation (see ben_type).
   unsigned char joasdi;
   // Description: Number of periods of disability (0-2).
   unsigned char valdi;
   // Description: First year of earnings after backward projection.
   unsigned short ibegin;
   // Description: Last year of earnings after forward projection.
   unsigned short iend;
   // Description: A set of bools, used for various purposes.
   //
   // Remarks: The fields have the following meanings:
   //
   //  0 - The sex of worker (see Sex::sex_type).
   //  1 - Recalculation code (false for initial benefit, true for
   //      recalculation).
   //  2 - Totalization code (false for no totalization, true if totalization
   //      applies).
   //  3 - Indicator for deemed insured status possible (false for no deemed
   //      insured status possible, true if deemed insured status possible).
   //  4 - Indicator for disability insured status allowed based on fully
   //      insured status only because of blindness (false for no blindness,
   //      true if blindness).
   //  5 - Medicare earnings code (false for no Medicare earnings, true if
   //      Medicare earnings).
   //  6 - Indicator for military reservist pension (false if not a military
   //      reservist pension, true if a military reservist pension).
   //  7 - Indicator for regular earnings available.
   //  8 - Indicator for railroad qcs and earnings available.
   //  9 - Indicator for QCs by single year prior to 1978.
   // 10 - Indicator for deceased OAB in survivor case. (True if deceased
   //      worker was an OAB before death. This is used in application of the
   //      RIB-LIM.)
   std::bitset<32> theBits;
   // Description: Month, day, and year of birth.
   boost::gregorian::date birthDate;
   // Description: Month and year of benefit date.
   DateMoyr benefitDate;
   // Description: Month and year of entitlement.
   DateMoyr entDate;
   // Description: Month, day, and year of death.
   boost::gregorian::date deathDate;
   // Description: Maximum year allowed.
   static int maxyear;
   // Description: Year through which quarters of coverage are specified as
   // a lump sum.
   static int qclumpyear;
public:
   WorkerDataGeneral();
   WorkerDataGeneral( const WorkerDataGeneral& workerData );
   virtual ~WorkerDataGeneral();
   // Description: Returns true if any self-employed taxes.
   //
   // Remarks: Needs to be implemented by a child class.
   //
   // Returns: True if any self-employed taxes.
   virtual bool anySelfEmp() const = 0;
   void benefitDate1Check() const;
   // Description: Checks that date of benefit is at or after entitlement.
   //
   // Remarks: The version with no arguments uses stored values, passing them
   // to the static version with 2 arguments.
   //
   // Throws: PiaException of type PIA_IDS_BENDATE1 if benefit date is before
   //   entitlement date.
   void benefitDate2Check() const { benefitDate2Check(entDate, benefitDate); }
   // Description: Checks date of birth in a Statement case.
   //
   // Remarks: The version with 1 argument uses stored values, passing them to
   // the static version with 2 arguments.
   //
   // Throws: PiaException of type PIA_IDS_BIRTH3 if month of birth is out of
   // range. PiaException of type PIA_IDS_BIRTH4 if day of birth is out of
   // range. PiaException of type PIA_IDS_BIRTH2 if year of birth is before
   // 1800. PiaException of type PIA_IDS_BIRTH1 if year of birth is in current
   // year or later.
   //
   // Arguments:
   //   istartt: Starting year of projections.
   void birth1Check( int istartt ) const { birth1Check(birthDate, istartt); }
   // Description: Checks date of birth in a non-Statement case.
   //
   // Remarks: The version with no arguments uses stored values, passing them
   // to the static version with 1 argument.
   //
   // Throws: PiaException of type PIA_IDS_BIRTH3 if month of birth is out of
   // range. PiaException of type PIA_IDS_BIRTH4 if day of birth is out of
   // range. PiaException of type PIA_IDS_BIRTH2 if year of birth is before
   // 1800. PiaException of type PIA_IDS_BIRTH5 if year of birth is after
   // maximum allowed.
   void birth2Check() const { birth2Check(birthDate); }
   // Description: Checks date of disability cessation, using stored values.
   //
   // Throws: PiaException of type PIA_IDS_DICESS4 if cesation is before onset.
   // PiaException of type PIA_IDS_DICESS6 if entitlement is before cessation.
   // PiaException of type PIA_IDS_DICESS7 if cessation is after death in
   // a survivor case.
   void cessationCheck() const
   { cessationCheck2(disabPeriod[0].getCessationDate(),
     disabPeriod[0].getOnsetDate(), deathDate, entDate,
     static_cast<ben_type>(joasdi)); }
   void cessation1Check() const;
   // Description: Checks that date of death is after birth.
   //
   // Remarks: The version with no arguments uses stored values, passing them
   // to the static version with 2 arguments.
   //
   // Throws: PiaException of type PIA_IDS_DEATH5 if death of worker is before
   // birth.
   void deathCheck() const { deathCheck(deathDate, birthDate); }
   virtual void deleteContents();
   // Description: Deletes date of death.
   void deleteDeathDate()
   { deathDate = boost::gregorian::date(boost::date_time::not_a_date_time); }
   // Description: Checks date of disability onset.
   //
   // Remarks:  The version with no arguments uses stored values, passing them
   // to the static version with 7 arguments.
   //
   // Throws: PiaException of type PIA_IDS_ONSET5 if disability is before
   // birth. PiaException of type PIA_IDS_ONSET6 if disability is after
   // entitlement in a life case or after benefit date in a survivor case.
   // PiaException of type PIA_IDS_ONSET7 if death of worker is before onset
   // in a survivor case.
   void disCheck() const
   { disCheck(disabPeriod[0].getOnsetDate(), deathDate, valdi, birthDate,
     static_cast<ben_type>(joasdi), entDate, benefitDate); }
   // Description: Checks date of prior disability onset.
   //
   // Remarks:  The version with no arguments uses stored values, passing them
   // to the static version with 4 arguments.
   //
   // Throws: PiaException of type PIA_IDS_ONSET5 if disability is before
   // birth. PiaException of type PIA_IDS_ONSET8 if prior disability is after
   // current disability.
   void dis1Check() const
   { dis1Check(disabPeriod[1].getOnsetDate(), disabPeriod[0].getOnsetDate(),
     valdi, birthDate); }
   // Description: Checks for no earnings of any type.
   //
   // Remarks:  The version with no arguments uses stored values, passing them
   // to the static version with 3 arguments.
   //
   // Throws: PiaException of type PIA_IDS_BEGIN6 if there are no regular,
   //   railroad, or military service earnings.
   void earningsCheck() const
   { earningsCheck(getIndearn(), getIndrr(), getIndms()); }
   // Description: Checks that current date of entitlement is within bounds.
   //
   // Remarks:  The version with no arguments uses stored values, passing them
   // to the static version with 1 argument.
   //
   // Throws: PiaException of type PIA_IDS_ENT1 if month of entitlement is out
   // of range. PiaException of type PIA_IDS_ENT2 if year of entitlement is
   // before 1940. PiaException of type PIA_IDS_ENT3 if year of entitlement is
   // after maximum allowed.
   void entCheck() const { entCheck(entDate); }
   // Description: Returns first year of Medicare earnings, if any.
   //
   // Returns: First year of Medicare earnings, if any.
   int firstEarnHiYear() const { return (1983 < ibegin) ? ibegin : 1983; }
   // Description: Returns date of benefit.
   //
   // Returns: The date of benefit.
   const DateMoyr& getBenefitDate() const { return(benefitDate); }
   // Description: Returns date of birth of worker.
   //
   // Returns: The date of birth of worker.
   const boost::gregorian::date& getBirthDate() const { return(birthDate); }
   // Description: Returns the blind indicator.
   //
   // Returns: True if blind, false otherwise.
   bool getBlindind() const { return(theBits[BLIND_BIT]); }
   // Description: Returns date of death.
   //
   // Returns: The date of death.
   const boost::gregorian::date& getDeathDate() const { return(deathDate); }
   // Description: Returns the deceased OAB indicator.
   //
   // Returns: True if the deceased worker was an OAB before death.
   bool getDeceasedOAB() const { return(theBits[DEADOAB_BIT]); }
   // Description: Returns the deemed insured indicator.
   //
   // Returns: True if the deemed insured status provision applies, false
   //   otherwise.
   bool getDeemedind() const { return(theBits[DEEMED_BIT]); }
   const DateMoyr& getEarliestEntDate() const;
   // Description: Returns Medicare earnings for specified year.
   //
   // Remarks: Needs to be implemented by a child class.
   //
   // Returns: Medicare earnings for specified year.
   virtual double getEarnHi( int ) const = 0;
   // Description: Returns Medicare earnings for specified year if
   // there are no OASDI earnings; otherwise return zero.
   //
   // Remarks: Needs to be implemented by a child class.
   //
   // Returns: Medicare earnings for specified year if 
   // there are no OASDI earnings; otherwise return zero.
   virtual double getEarnHiOnly( int ) const = 0;
   // Description: Returns military service earnings for specified year.
   //
   // Remarks: Needs to be implemented by a child class.
   //
   // Returns: Military service earnings for specified year.
   virtual double getEarnMS( int ) const = 0;
   // Description: Returns total military service earnings, 1937-50.
   //
   // Remarks: Needs to be implemented by a child class.
   //
   // Returns: Total military service earnings, 1937-50.
   virtual double getEarnMS3750() const = 0;
   // Description: Returns OASDI plus Medicare earnings for specified year.
   //
   // Remarks: Needs to be implemented by a child class.
   //
   // Returns: OASDI plus Medicare earnings for specified year.
   virtual double getEarnOasdhi( int ) const = 0;
   // Description: Returns OASDI-covered earnings for the specified year.
   //
   // Remarks: Needs to be implemented by a child class.
   //
   // Returns: OASDI-covered earnings for the specified year.
   virtual double getEarnOasdi( int ) const = 0;
   // Description: Returns railroad earnings for specified year.
   //
   // Remarks: Needs to be implemented by a child class.
   //
   // Returns: Railroad earnings for specified year.
   virtual double getEarnRR( int ) const = 0;
   // Description: Returns total railroad earnings, 1937-50.
   //
   // Remarks: Needs to be implemented by a child class.
   //
   // Returns: Total railroad earnings, 1937-50.
   virtual double getEarnRR3750() const = 0;
   bool getEarnEnable() const;
   // Description: Returns date of entitlement.
   //
   // Returns: The date of entitlement.
   const DateMoyr& getEntDate() const { return(entDate); }
   // Description: Returns first year of military service (0 if none).
   //
   // Remarks: Needs to be implemented by a child class.
   //
   // Returns: First year of military service (0 if none).
   virtual int getFirstYearMS() const = 0;
   // Description: Returns first year of railroad (0 if none).
   //
   // Remarks: Needs to be implemented by a child class.
   //
   // Returns: First year of railroad (0 if none).
   virtual int getFirstYearRR() const = 0;
   // Description: Returns first year of earnings after any projection.
   //
   // Returns: First year of earnings after any projection.
   int getIbegin() const { return(ibegin); }
   // Description: Returns Social Security number as a number.
   //
   // Remarks: Needs to be implemented by a child class.
   //
   // Returns: Social Security number as a number.
   virtual unsigned long getIdNumber() const = 0;
   // Description: Returns Social Security number as a string.
   //
   // Remarks: Needs to be implemented by a child class.
   //
   // Returns: Social Security number as a string.
   virtual std::string getIdString() const = 0;
   // Description: Returns last year of earnings after any projection.
   //
   // Returns: Last year of earnings after any projection.
   int getIend() const { return(iend); }
   // Description: Returns earnings indicator.
   //
   // Returns: False if no regular earnings, true if some regular earnings.
   bool getIndearn() const { return(theBits[INDEARN_BIT]); }
   // Description: Returns military service indicator.
   //
   // Remarks: Needs to be implemented by a child class.
   //
   // Returns: True if any military service, false otherwise.
   virtual bool getIndms() const = 0;
   // Description: Returns railroad indicator.
   //
   // Returns: False if no railroad earnings, true if some railroad earnings.
   bool getIndrr() const { return(theBits[INDRR_BIT]); }
   // Description: Returns type of benefit calculation, including possibly
   //   Statement.
   //
   // Returns: Type of benefit calculation.
   //
   //   NO_BEN - No benefit calculation.
   //   OLD_AGE - Old-age benefit calculation.
   //   SURVIVOR - Survivor benefit calculation.
   //   DISABILITY - Disability benefit calculation.
   //   PEBS_CALC - Statement benefit calculation.
   WorkerDataGeneral::ben_type getJoasdi() const
      { return(static_cast<ben_type>(joasdi)); }
   // Description: Returns last year of military service (0 if none).
   //
   // Remarks: Needs to be implemented by a child class.
   //
   // Returns: Last year of military service (0 if none).
   virtual int getLastYearMS() const = 0;
   // Description: Returns last year of military service (0 if none).
   //
   // Remarks: Needs to be implemented by a child class.
   //
   // Returns: Last year of military service (0 if none).
   virtual int getLastYearRR() const = 0;
   // Description: Returns Medicare-qualified government employment indicator.
   //
   // Returns: True if any Medicare-qualified government employment.
   bool getMqge() const { return(theBits[MQGE_BIT]); }
   // Description: Returns one line of number holder's address.
   //
   // Remarks: Needs to be implemented by a child class.
   //
   // Returns: One line of number holder's address.
   virtual std::string getNhaddr( int ) const = 0;
   // Description: Returns number holder's name.
   //
   // Remarks: Needs to be implemented by a child class.
   //
   // Returns: Number holder's name.
   virtual std::string getNhname() const = 0;
   // Description: Returns date of cessation of oab entitlement prior to
   // most recent DIB entitlement.
   //
   // Remarks: Needs to be implemented by a child class.
   //
   // Returns: The date of cessation of oab entitlement prior to
   //   most recent DIB entitlement.
   virtual DateMoyr getOabCessDate() const = 0;
   // Description: Returns date of entitlement to oab prior to most
   // recent DIB entitlement.
   //
   // Remarks: Needs to be implemented by a child class.
   //
   // Returns: The date of entitlement to oab prior to most recent DIB
   // entitlement.
   virtual DateMoyr getOabEntDate() const = 0;
   // Description: Returns monthly noncovered pension.
   //
   // Remarks: Needs to be implemented by a child class.
   //
   // Returns: Monthly noncovered pension.
   virtual float getPubpen() const = 0;
   // Description: Returns date of entitlement for noncovered pension.
   //
   // Remarks: Needs to be implemented by a child class.
   //
   // Returns: Date of noncovered pension entitlement.
   virtual DateMoyr getPubpenDate() const = 0;
   // Description: Returns monthly noncovered pension after December 1994,
   // after removing reservist pension.
   //
   // Remarks: Needs to be implemented by a child class.
   //
   // Returns: Monthly noncovered pension after December 1994,
   // after removing reservist pension.
    virtual float getPubpenReservist() const = 0;
   // Description: Returns military service quarters of coverage for
   // specified year.
   //
   // Remarks: Needs to be implemented by a child class.
   //
   // Returns: Military service quarters of coverage for specified year.
   virtual unsigned getQcovMS( int ) const = 0;
   // Description: Returns total military service quarters of coverage,
   //  1937-50.
   //
   // Remarks: Needs to be implemented by a child class.
   //
   // Returns: Total military service quarters of coverage, 1937-50.
   virtual unsigned getQcovMS3750() const = 0;
   // Description: Returns railroad quarters of coverage for specified year.
   //
   // Remarks: Needs to be implemented by a child class.
   //
   // Returns: Railroad quarters of coverage for specified year.
   virtual unsigned getQcovRR( int ) const = 0;
   // Description: Returns total railroad quarters of coverage, 1937-50.
   //
   // Remarks: Needs to be implemented by a child class.
   //
   // Returns: Total railroad quarters of coverage, 1937-50.
   virtual unsigned getQcovRR3750() const = 0;
   // Description: Returns total railroad quarters of coverage, 1951-77.
   //
   // Remarks: Needs to be implemented by a child class.
   //
   // Returns: Total railroad quarters of coverage, 1951-77.
   virtual unsigned getQcovRR5177() const = 0;
   // Description: Returns quarters of coverage from 1951 to 1977.
   //
   // Returns: Quarters of coverage from 1951 to 1977.
   int getQctot51td() const { return(qctot51td); }
   // Description: Returns quarters of coverage from 1937 to 1977.
   //
   // Returns: Quarters of coverage from 1937 to 1977.
   int getQctottd() const { return(qctottd); }
   // Description: Returns recalculation code.
   //
   // Returns: True if benefit is being recalculated after initial entitlement,
   //   false otherwise.
   bool getRecalc() const { return(theBits[RECALC_BIT]); }
   // Description: Returns reservist indicator.
   //
   // Returns: False if no reservist, true if pubpen is based at least
   // partially on reservist pension.
   bool getReservist() const { return(theBits[RESERVIST_BIT]); }
   // Description: Returns the sex of the worker.
   //
   // Returns: MALE (0) if male, FEMALE (1) if female.
   Sex::sex_type getSex() const
      { return static_cast<Sex::sex_type>(theBits[SEX_BIT]); }
   // Description: Returns type of taxes for specified year.
   //
   // Remarks: Needs to be implemented by a child class.
   //
   // Returns: Type of taxes for specified year.
   virtual bool getTaxType( int ) const = 0;
   // Description: Returns totalization indicator.
   //
   // Returns: False if no totalization, true if totalization applies.
   bool getTotalize() const { return(theBits[TOTALIZE_BIT]); }
   // Description: Returns number of periods of disability.
   //
   // Returns: 0, 1, or 2.
   int getValdi() const { return(valdi); }
   // Description: Returns indicator for QCs prior to 1978 by single year.
   //
   // Returns: True if has QCs prior to 1978 by single year, false otherwise.
   bool hasQcsByYear() const { return theBits[QCSBYYEAR_BIT]; }
   // Description: Checks for first year of earnings within bounds.
   //
   // Remarks:  The version with no arguments uses stored values, passing them
   // to the static version with 2 arguments.
   //
   // Throws: PiaException of type PIA_IDS_BEGIN1 if first year of earnings is
   // out of range.
   void ibeginCheck() const
      { ibeginCheck(ibegin, static_cast<int>(birthDate.year())); }
   // Description: Returns true if alive at end of year, false otherwise.
   //
   // Returns: True if alive at end of year, false otherwise.
   //
   // Arguments:
   //   year:  Year being checked.
   bool isalive( int year ) const
   { if (deathDate.is_not_a_date())
        return (static_cast<int>(birthDate.year()) <= year);
     else return((static_cast<int>(birthDate.year()) <= year) &&
        (year < deathDate.year())); }
   // Description: Returns true if dead at end of year or end of month, false
   //   otherwise.
   //
   // Returns: True if dead at end of year or month, false otherwise.
   //
   // Arguments:
   //   year:  Year being checked.
   bool isdead( int year ) const
      { if (deathDate.is_not_a_date()) return false;
        else return(year >= deathDate.year()); }
   // Arguments:
   //   date:  Month and year being checked.
   bool isdead( const DateMoyr& date ) const
      { return(!deathDate.is_not_a_date() && !(date < DateMoyr(deathDate))); }
   // Description: Return true if person is Sex::FEMALE, false otherwise.
   //
   // Returns: True if person is Sex::FEMALE, false otherwise.
   bool isFemale() const { return theBits[SEX_BIT] == Sex::FEMALE; }
   // Description: Return true if person is Sex::MALE, false otherwise.
   //
   // Returns: True if person is Sex::MALE, false otherwise.
   bool isMale() const { return theBits[SEX_BIT] == Sex::MALE; }
   int isPrimary( ben_type ioasdi ) const;
   // Description: Checks type of benefit.
   //
   // Remarks:  The version with no arguments uses
   //   stored values, passing them to the static version with 1 argument.
   //
   // Throws: PiaException of type PIA_IDS_JOASDI if type of benefit is out of
   //   range.
   void joasdiCheck() const { joasdiCheck(joasdi); }
   // Description: Returns last year of annual QCs.
   //
   // Returns: Last year of annual QCs (lesser of last year of earnings and
   // 1977).
   int lastQcyr() const { return(iend < 1977 ? iend : 1977); }
   bool need_aw( int year, bool lastTwoYears ) const;
   bool need_bases( int year ) const;
   bool need_bi( int year ) const;
   // Description: Returns true if date of cessation to disability is needed.
   //
   // Remarks: The version with no arguments uses stored values, passing them
   // to the static version with 2 arguments.
   //
   // Returns: True if date of cessation to disability is needed.
   bool needDiCess() const
      { return(needDiCess(valdi, static_cast<ben_type>(joasdi))); }
   // Description: Checks date of prior entitlement to disability.
   //
   // Remarks: The version with no arguments uses stored values, passing them
   // to the static version with 4 arguments.
   //
   // Throws: PiaException of type PIA_IDS_PRRENT1 if month of prior
   // entitlement is out of range. PiaException of type PIA_IDS_PRRENT2 if year
   // of prior entitlement is before 1940. PiaException of type PIA_IDS_PRRENT3
   // if year of prior entitlement is after maximum allowed. PiaException of
   // type PIA_IDS_PRRENT4 if prior entitlement is not before current
   // entitlement in an old-age case. (Prior entitlement should equal current
   // entitlement in a disability case, and there is no date of entitlement for
   // the worker in a survivor case.) PiaException of type PIA_IDS_PRRENT5 if
   // prior entitlement to disability is before disability onset.
   void priorentCheck() const
   { priorentCheck(disabPeriod[0].getEntDate(), disabPeriod[0].getOnsetDate(),
       entDate, static_cast< ben_type >(joasdi)); }
   // Description: Checks date of second prior entitlement to disability.
   //
   // Remarks: The version with no arguments uses stored values, passing them
   // to the static version with 5 arguments.
   //
   // Throws: PiaException of type PIA_IDS_PRRENT1 if month of prior
   // entitlement is out of range. PiaException of type PIA_IDS_PRRENT2 if year
   // of prior entitlement is before 1940. PiaException of type PIA_IDS_PRRENT3
   // if year of prior entitlement is after maximum allowed. PiaException of
   // type PIA_IDS_PRRENT4 if prior entitlement is not before current
   // entitlement. PiaException of type PIA_IDS_PRRENT5 if prior entitlement to
   // disability is before disability onset.
   void priorent1Check() const
   { priorent1Check(disabPeriod[1].getEntDate(), disabPeriod[1].getOnsetDate(),
       disabPeriod[0].getEntDate()); }
   bool needQcsByYear() const;
   // Description: Checks for correct numbers of QC's.
   //
   // Remarks: The version with no arguments uses stored values, passing them
   // to the static version with 2 arguments.
   //
   // Throws: PiaException of type PIA_IDS_QC if number of quarters of
   // coverage, 1951-77, is greater than number of quarters of coverage,
   // 1937-77.
   void qcCheck() const { qcCheck(qctottd, qctot51td); }
   void qctdCheck2();
   void setBenefitDate( const DateMoyr& datetemp );
   void setBenefitDate();
   void setBirthDate( const boost::gregorian::date& datetemp );
   // Description: Sets blind indicator.
   //
   // Arguments:
   //   newBlindind: New blind indicator.
   void setBlindind( bool newBlindind ) { theBits[BLIND_BIT] = newBlindind; }
   void setCessationDate( unsigned index, const DateMoyr& dateMoyr );
   void setCessationMfb( unsigned index, float newMfb );
   void setCessationPia( unsigned index, float newPia );
   void setDeathDate( const boost::gregorian::date& datetemp );
   // Description: Sets deceased OAB indicator.
   //
   // Arguments:
   //   newDeceasedOAB: New deceased OAB indicator.
   void setDeceasedOAB( bool newDeceasedOAB )
      { theBits[DEADOAB_BIT] = newDeceasedOAB; }
   // Description: Sets deemed insured indicator.
   //
   // Arguments:
   //   newDeemedind: New deemed insured indicator.
   void setDeemedind( bool newDeemedind )
      { theBits[DEEMED_BIT] = newDeemedind; }
   // Description: Sets Medicare earnings for the specified year.
   //
   // Remarks: Needs to be implemented by a child class.
   virtual void setEarnHi( int, double ) = 0;
   // Description: Sets OASDI-covered earnings.
   //
   // Remarks: Needs to be implemented by a child class.
   virtual void setEarnOasdi( int, double ) = 0;
   void setEntDate( const DateMoyr& datetemp );
   void setIbegin( int newIbegin );
   void setIend( int newIend );
   void setIndearn( bool newIndearn );
   // Description: Sets railroad indicator.
   //
   // Arguments:
   //   newIndrr: New railroad indicator.
   void setIndrr( bool newIndrr ) { theBits[INDRR_BIT] = newIndrr; }
   void setJoasdi( int newJoasdi );
   // Description: Sets mqge indicator.
   //
   // Arguments:
   //   newMqge: New mqge indicator.
   void setMqge( bool newMqge ) { theBits[MQGE_BIT] = newMqge; }
   // Description: Sets one line of address.
   //
   // Remarks: Needs to be implemented by a child class.
   virtual void setNhaddr ( int, const std::string& ) = 0;
   // Description: Sets number holder's name.
   //
   // Remarks: Needs to be implemented by a child class.
   virtual void setNhname ( const std::string& ) = 0;
   // Description: Sets date of cessation of oab prior to most recent dib.
   //
   // Remarks: Needs to be implemented by a child class.
   virtual void setOabCessDate( const DateMoyr& ) = 0;
   // Description: Sets date of oab entitlement prior to most recent dib.
   //
   // Remarks: Needs to be implemented by a child class.
   virtual void setOabEntDate( const DateMoyr& ) = 0;
   void setOnsetDate( unsigned index, const boost::gregorian::date& dateModyyr );
   void setPebsData();
   // Description: Sets last year of Statement earnings when computation year
   //   changes, or after a Statement calculation has been done.
   //
   // Arguments:
   //   year: First year of projection period.
   void setPebsEarn( int year ) { iend = (unsigned short)(1 + year); }
   void setPriorentDate( unsigned index, const DateMoyr& dateMoyr );
   void setPriorentDateCheck( unsigned index, const DateMoyr& dateMoyr );
   // Description: Sets amount of monthly noncovered pension.
   //
   // Remarks: Needs to be implemented by a child class.
   virtual void setPubpen( float ) = 0;
   // Description: Sets date of monthly noncovered pension entitlement.
   //
   // Remarks: Needs to be implemented by a child class.
   virtual void setPubpenDate( const DateMoyr& ) = 0;
   // Description: Sets amount of monthly noncovered pension after December
   // 1994, after removing military reservist pension.
   //
   // Remarks: Needs to be implemented by a child class.
   virtual void setPubpenReservist( float ) = 0;
   void setQc51td( int qct );
   // Description: Sets indicator for QCs by year prior to 1978.
   //
   // Arguments:
   //   newQcsByYear: Indicator for QCs by year prior to 1978.
   void setQcsByYear( bool newQcsByYear )
      { theBits[QCSBYYEAR_BIT] = newQcsByYear; }
   void setQctd( int qct );
   // Description: Sets recalculation indicator.
   //
   // Arguments:
   //   newRecalc: New recalculation indicator.
   void setRecalc( bool newRecalc ) { theBits[RECALC_BIT] = newRecalc; }
   // Description: Sets reservist indicator.
   //
   // Arguments:
   //   newReservist: New reservist indicator.
   void setReservist( bool newReservist )
      { theBits[RESERVIST_BIT] = newReservist; }
   void setSex( Sex::sex_type newSex );
   // Description: Sets type of taxes for the specified year.
   //
   // Remarks: Needs to be implemented by a child class.
   virtual void setTaxType( int, int ) = 0;
   // Description: Sets totalization indicator.
   //
   // Arguments:
   //   newTotalize: New totalization indicator.
   void setTotalize( bool newTotalize )
      { theBits[TOTALIZE_BIT] = newTotalize; }
   void setValdi( int newValdi );
   void setWaitperDate( unsigned index );
   void setWaitperDate( unsigned index, const DateMoyr& dateMoyr );
   // Description: Checks number of periods of disability, using stored value.
   //
   // Throws: PiaException of type PIA_IDS_VALDI1 if number of periods of
   //   disability is not 0, 1, or 2 in a nondisability case. PiaException of
   //   type PIA_IDS_VALDI2 if number of periods of disability is not 1 or 2 in
   //   a disability case.
   void valdiCheck() const
      { if (joasdi == DISABILITY) valdi2Check(valdi);
        else valdi1Check(valdi); }
   // Description: Checks first month of waiting period.
   //
   // Remarks: The version with no arguments uses stored values, passing them
   // to the static version with 3 arguments.
   //
   // Throws: PiaException of type PIA_IDS_WAITPD1 if first month of waiting
   // period is out of range. PiaException of type PIA_IDS_WAITPD2 if year of
   // first month of waiting period is before 1940. PiaException of type
   // PIA_IDS_WAITPD3 if year of first month of waiting period is after maximum
   // allowed. PiaException of type PIA_IDS_WAITPD4 if waiting period starts
   // before onset. PiaException of type PIA_IDS_WAITPD5 if entitlement is
   // before waiting period.
   void waitpdCheck() const
   { waitpdCheck(disabPeriod[0].getWaitperDate(),
        disabPeriod[0].getOnsetDate(), entDate); }
   // Description: Checks first month of prior waiting period.
   //
   // Remarks: The version with no arguments uses stored values, passing them
   // to the static version with 3 arguments.
   //
   // Throws: PiaException of type PIA_IDS_WAITPD4 if waiting period starts
   // before onset. PiaException of type PIA_IDS_WAITPD6 if cessation is before
   // waiting period.
   void waitpd1Check() const
   { waitpd1Check(disabPeriod[1].getWaitperDate(),
       disabPeriod[1].getOnsetDate(), disabPeriod[1].getCessationDate()); }
   virtual void zeroEarn( int ibegint, int iendt );
   // Description: Zeroes out HI earnings before first year
   //   and after last year of specified earnings.
   virtual void zeroEarnHi( int, int ) { };
   // Description: Zeroes out OASDI earnings before first year
   //   and after last year of specified earnings.
   virtual void zeroEarnOasdi( int, int ) { };
   // Description: Zeroes out the type of benefit.
   void zeroOasdi() { joasdi = NO_BEN; }
   static void benefitDate1Check( const DateMoyr& datetemp );
   // Arguments:
   //   entDatet: The date of entitlement.
   //   benefitDatet: The date of benefit.
   static void benefitDate2Check( const DateMoyr& entDatet,
      const DateMoyr& benefitDatet )
   { if (benefitDatet < entDatet) throw PiaException(PIA_IDS_BENDATE1); }
   static void birth1Check( const boost::gregorian::date& datetemp, int istartt );
   static void birth2Check( const boost::gregorian::date& datetemp );
   static void cessationCheck2( const DateMoyr& dicesst,
      const boost::gregorian::date& onsetDatet,
      const boost::gregorian::date& deathDatet,
      const DateMoyr& entDatet, ben_type ioasdit );
   // Description: Checks date of prior disability cessation against date of
   // prior onset.
   //
   // Remarks: The version with no arguments uses stored values, passing them
   // to the static versions with 2 or 3 arguments.
   //
   // Throws: PiaException of type PIA_IDS_DICESS4 if disability cessation
   // before onset. PiaException of type PIA_IDS_DICESS5 if prior disability
   // cessation after second entitlement.
   //
   // Arguments:
   //   cessation1Datet: The date of prior disability cessation.
   //   onset1Datet: The date of prior disability onset.
   static void cessation1Check( const DateMoyr& cessation1Datet,
      const boost::gregorian::date& onset1Datet )
   { if (cessation1Datet < onset1Datet) throw PiaException(PIA_IDS_DICESS4); }
   static void cessation1Check( const DateMoyr& cessation1Datet,
      const boost::gregorian::date& onset1Datet, const DateMoyr& entDatet );
   // Arguments:
   //   deathDatet: The date of death.
   //   birthDatet: The date of birth.
   static void deathCheck( const boost::gregorian::date& deathDatet,
      const boost::gregorian::date& birthDatet )
   { if (deathDatet < birthDatet) throw PiaException(PIA_IDS_DEATH5); }
   static void deathCheck1( const boost::gregorian::date& datetemp,
      ben_type ioasdit );
   static void deathCheck2( const boost::gregorian::date& datetemp );
   // Description: Checks that date of death is before entitlement, using
   // passed values.
   //
   // Remarks:  Assumes that this is a survivor case.  Should be called for
   // each survivor's date of entitlement.
   //
   // Throws: PiaException of type PIA_IDS_DEATH4 if entitlement of survivor is
   // before death of worker.
   //
   // Arguments:
   //   deathDatet: The date of death.
   //   entDatet: The date of survivor's entitlement.
   static void deathCheck3( const boost::gregorian::date& deathDatet,
      const DateMoyr& entDatet )
   { if (entDatet < DateMoyr(deathDatet)) throw PiaException(PIA_IDS_DEATH4); }
   static void disCheck( const boost::gregorian::date& onsetDatet,
      const boost::gregorian::date& deathDatet, int valdit,
      const boost::gregorian::date& birthDatet,
      ben_type ioasdit, const DateMoyr& entDatet, const DateMoyr& benDatet );
   static void disCheck( const boost::gregorian::date& onsetDatet,
      const boost::gregorian::date& birthDatet, ben_type ioasdit,
      const DateMoyr& entDatet, const DateMoyr& benDatet );
   static void disCheck( const boost::gregorian::date& onsetDatet,
      const boost::gregorian::date& deathDatet, ben_type ioasdi )
   { if (ioasdi == SURVIVOR && deathDatet < onsetDatet)
        throw PiaException(PIA_IDS_ONSET7); }
   static void dis1Check( const boost::gregorian::date& onset1Datet,
      const boost::gregorian::date& onsetDatet, int valdit,
      const boost::gregorian::date& birthDatet );
   // Arguments:
   //   indEarn: True if there are any regular OASDI earnings.
   //   indRailroad: True if there are any railroad earnings.
   //   indMS: True if there are any military service wage credits.
   static void earningsCheck( bool indEarn, bool indRailroad, bool indMS )
   { if (!indEarn && !indRailroad && !indMS)
        throw PiaException(PIA_IDS_BEGIN6); }
   static void entCheck( const DateMoyr& datetemp );
   // Description: Returns maximum year allowed.  Note that this is a static
   //   function.
   //
   // Returns: Maximum year allowed (default 2100).
   static int getMaxyear() { return(maxyear); }
   // Description: Returns last year of period for which quarters of
   //   coverage are entered as a lump sum.
   //
   // Returns: Last year of period for which quarters of coverage are entered
   // as a lump sum (default 1977).
   static int getQcLumpYear() { return(qclumpyear); }
   static void ibeginCheck( int ibegint, int ibirtht );
   static void ibegin1Check( int ibegint, int istartt );
   static void ibegin2Check( int ibegint );
   static void iendCheck( int iendt );
   // Arguments:
   //   joasdit: Type of benefit to check.
   static void joasdiCheck( int joasdit )
   { if (joasdit < static_cast<int>(NO_BEN) ||
        joasdit > static_cast<int>(PEBS_CALC))
        throw PiaException(PIA_IDS_JOASDI); }
   // Arguments:
   //   valdit: Number of periods of disability.
   //   ioasdi: Type of benefit.
   static bool needDiCess( int valdit, ben_type ioasdi )
   { return(valdit > 0 && (ioasdi == OLD_AGE || ioasdi == SURVIVOR
        || ioasdi == NO_BEN)); }
   static void priorentCheck ( const DateMoyr& priorentt,
      const boost::gregorian::date& ionsett, const DateMoyr& ientt,
      ben_type ioasdit );
   static void priorent1Check( const DateMoyr& priorent1t,
      const boost::gregorian::date& ionsett, const DateMoyr& priorentt );
   static void pubpenCheck( float pubpent );
   static void pubpenDateCheck( const DateMoyr& pubpenDatet,
      const boost::gregorian::date& birthDatet,
      const boost::gregorian::date& deathDatet );
   // Arguments:
   //   qctottdt: QCs from 1937 to 1977.
   //   qctot51tdt: QCs from 1951 to 1977.
   static void qcCheck( int qctottdt, int qctot51tdt )
      { if (qctot51tdt > qctottdt) throw PiaException(PIA_IDS_QC); }
   // Description: Checks summary quarters of coverage, 1951 to 1977.
   //
   // Throws: PiaException of type PIA_IDS_QC51TD if number of quarters of
   //   coverage, 1951-77, is out of range.
   //
   // Arguments:
   //   qct: Quarters of coverage to check.
   static void qc51tdCheck( int qct )
      { if ( qct < 0 || qct > 108) throw PiaException(PIA_IDS_QC51TD); }
   // Description: Checks summary quarters of coverage, 1937 to 1977.
   //
   // Throws: PiaException of type PIA_IDS_QCTD if number of quarters of
   //   coverage, 1937-77, is out of range.
   //
   // Arguments:
   //   qct: Quarters of coverage to check.
   static void qctdCheck( int qct )
      { if ( qct < 0 || qct > 164) throw PiaException(PIA_IDS_QCTD); }
   // Description: Sets maximum year allowed.  Note that this is a static
   //   function.
   //
   // Arguments:
   //   newMaxyear: New maximum year allowed.
   static void setMaxyear( int newMaxyear ) { maxyear = newMaxyear; }
   // Description: Sets last year of period for which quarters of
   //   coverage are entered as a lump sum.
   //
   // Arguments:
   //   newQcLumpYear: New last year of period for which quarters of
   //     coverage are entered as a lump sum.
   static void setQcLumpYear( int newQcLumpYear )
      { qclumpyear = newQcLumpYear; }
   // Description: Checks type of taxes.
   //
   // Throws: PiaException of type PIA_IDS_TAXTYPE if type of taxes is not 0 or
   // 1.
   //
   // Arguments:
   //   taxTypet: Type of taxes to check.
   static void taxTypeCheck( int taxTypet )
      { if (taxTypet > 1 || taxTypet < 0)
           throw PiaException(PIA_IDS_TAXTYPE); }
   // Description: Checks number of periods of disability in nondisability
   // case.
   //
   // Throws: PiaException of type PIA_IDS_VALDI1 if number of periods of
   //    disability is not 0, 1, or 2.
   //
   // Arguments:
   //   valdit: Number of periods to check.
   static void valdi1Check( int valdit )
   { if (valdit < 0 || valdit > NUM_DISAB_PERIODS)
        throw PiaException(PIA_IDS_VALDI1); }
   // Description: Checks number of periods of disability in disability case.
   //
   // Throws: PiaException of type PIA_IDS_VALDI2 if number of periods of
   //    disability is not 1 or 2.
   //
   // Arguments:
   //   valdit: Number of periods of disability to check.
   static void valdi2Check( int valdit )
   { if (valdit < 1 || valdit > NUM_DISAB_PERIODS)
        throw PiaException(PIA_IDS_VALDI2); }
   // Arguments:
   //   waitperDatet: First month of waiting period.
   //   onsetDatet: The date of onset.
   //   entDatet: The date of entitlement.
   static void waitpdCheck( const DateMoyr& waitperDatet,
      const boost::gregorian::date& onsetDatet, const DateMoyr& entDatet )
   { if (waitperDatet < onsetDatet) throw PiaException(PIA_IDS_WAITPD4);
     if (entDatet < waitperDatet) throw PiaException(PIA_IDS_WAITPD5); }
   // Arguments:
   //   waitper1Datet: First month of prior waiting period.
   //   onset1Datet: The date of prior onset.
   //   cessation1Datet: The date of prior cessation.
   static void waitpd1Check( const DateMoyr& waitper1Datet,
      const boost::gregorian::date& onset1Datet, const DateMoyr& cessation1Datet )
   { if (waitper1Datet < onset1Datet) throw PiaException(PIA_IDS_WAITPD4);
     if (cessation1Datet < waitper1Datet) throw PiaException(PIA_IDS_WAITPD6); }
};
