// $Id: age.h 1.18 2006/11/08 10:57:56EST 044579 Development  $
//
// Declarations for class to manage age in years and months.

#pragma once

#include <string>
#include "boost/serialization/access.hpp"

// {group:Miscellaneous Classes}
// Description: Manages a year and month age stored as integers.
//
// Remarks: The year and month are stored as 16-bit shorts to save space.
class Age
{
   friend class boost::serialization::access;
   // Description: Archives the data.
   //
   // Remarks: When the class Archive corresponds to an output archive, the
   // & operator is defined similar to <<. Likewise, when the class Archive is
   // a type of input archive, the & operator is defined similar to >>.
   //
   // Arguments:
   //   ar: The archive to use.
   //   version: The version of the archive to use.
   template< class Archive >
   void serialize( Archive& ar, const unsigned int version )
      { ar & yearsOfAge & monthsOfAge; }
protected:
   // Description: Years of age.
   unsigned short yearsOfAge;
   // Description: Months of age (0-11).
   unsigned short monthsOfAge;
public:
   // Description: Initializes age.
   //
   // Remarks: Default constructor initializes years and months to zero.
   Age() : yearsOfAge(0u), monthsOfAge(0u) { }
   // Arguments:
   //   newAge: The age to copy from.
   Age( const Age& newAge ) :
      yearsOfAge(newAge.yearsOfAge), monthsOfAge(newAge.monthsOfAge) { }
   // Arguments:
   //   newYrMo: The years and months in an array.
   Age( unsigned newYrMo[2] ) :
      yearsOfAge((unsigned short)newYrMo[0]),
      monthsOfAge((unsigned short)newYrMo[1]) { }
   // Arguments:
   //   newYears: The years of age.
   //   newMonths: The months of age.
   Age( unsigned newYears, unsigned newMonths ) :
      yearsOfAge((unsigned short)newYears),
      monthsOfAge((unsigned short)newMonths) { }
   // Description: Destructor.
   //
   // Remarks: Does nothing.
   ~Age() { }
   // Description: Copies from another Age.
   //
   // Arguments:
   //   newAge: The Age to copy from.
   Age& operator=( const Age& newAge ) { yearsOfAge = newAge.yearsOfAge;
      monthsOfAge = newAge.monthsOfAge; return *this; }
   bool operator<( const Age& newAge ) const;
   // Description: Compares (years,months) ages for equality.
   //
   // Returns: True if years and months agree, false otherwise.
   //
   // Arguments:
   //   newAge: Age to compare this one to.
   bool operator==( const Age& newAge ) const
      { return (yearsOfAge == newAge.yearsOfAge &&
        monthsOfAge == newAge.monthsOfAge); }
   // Description: Zeroes years and months.
   void deleteContents() { yearsOfAge = monthsOfAge = 0u; }
   // Description: Returns months of age.
   //
   // Returns: Number of months of age (0-11).
   unsigned getMonths() const { return(monthsOfAge); }
   // Description: Returns years of age.
   //
   // Returns: Number of years of age.
   unsigned getYears() const { return(yearsOfAge); }
   void monthsCheck() const;
   std::string toString() const;
};
