// $Id: bic.h 1.15 2006/11/08 10:58:17EST 044579 Development  $
//
// Declarations needed for class to manage bic code.

#pragma once

#include <string>

// {group:Calculation Classes}
// Description: Represents one 2-character bic (Beneficiary Identification
//   code) code.
//
// Remarks: An instance of this class is used in Secondary.
class Bic
{
protected:
   // Description: Major bic code.
   //
   // Remarks:
   //
   // A - primary (old-age or disability).
   // B - wife or husband (young or aged).
   // C - child (minor, student, or disabled).
   // D - aged widow(er).
   // E - mother or father.
   // F - parent.
   // T - transitionally insured.
   // U - transitionally uninsured.
   // W - disabled widow(er).
   char majorBic;
   // Description: Minor bic code.  We use a subset of the actual codes.
   //
   // Remarks:
   //
   // If majorBic is B, then
   //
   //   blank - aged wife or husband.
   //   2 - young wife or husband.
   //   6 - divorced aged wife or husband.
   //
   // If majorBic is D, then
   //
   //   blank - aged widow(er).
   //   6 - surviving divorced wife or husband.
   //
   // If majorBic is W, then
   //
   //   blank - disabled widow(er).
   //   6 - surviving disabled divorced wife or husband.
   char minorBic;
   // Description: Type of survivor titles.
   static const std::string survivorType[6];
   // Description: Type of life auxiliary titles.
   static const std::string lifeAuxType[4];
public:
   Bic();
   Bic( const std::string& newCode );
   Bic( char newMajorBic, char newMinorBic );
   // Description: Compares to another Bic.
   //
   // Returns: True if major and minor codes agree, false otherwise.
   //
   // Arguments:
   //   bictemp: Bic to compare to.
   bool operator==( const Bic& bictemp ) const
      { return majorBic == bictemp.majorBic && minorBic == bictemp.minorBic; }
   // Description: Returns true if this benefit is affected by family maximum.
   //
   // Remarks: Divorced spouses are not affected by family maximum.
   //
   // Returns: True if this benefit is affected by family maximum.
   bool eligibleForMax() const { return minorBic != '6'; }
   std::string getLifeAuxType() const;
   // Description: Returns the major bic code.
   //
   // Returns: The major bic code.
   char getMajorBic() const { return((majorBic == 'T') ? 'U' : majorBic); }
   // Description: Returns the minor bic code.
   //
   // Returns: The minor bic code.
   char getMinorBic() const { return(minorBic); }
   std::string getSurvivorType() const;
   // Description: Returns true if auxiliary beneficiary (B, C, D, E, F, or W).
   //
   // Returns: True if auxiliary beneficiary (B, C, D, E, F, or W).
   bool isAuxiliary() const
   { return(getMajorBic() == 'B' || getMajorBic() == 'C' ||
       getMajorBic() == 'D' || getMajorBic() == 'E' ||
       getMajorBic() == 'F' || getMajorBic() == 'W'); }
   // Description: Returns true if disabled widow(er), false otherwise.
   //
   // Returns: True if disabled widow(er), false otherwise.
   bool isDisabledWidow() const { return(getMajorBic() == 'W'); }
   // Description: Returns true if benefit is reducible for age.
   //
   // Remarks: Children's, young spouse's, and parent's benefits are not
   //   reducible for age.
   //
   // Returns: True if benefit is reducible for age.
   bool isReducible() const
   { return(majorBic == 'D' || majorBic == 'W' ||
       (majorBic == 'B' && minorBic != '2') || majorBic == 'A'); }
   // Description: Returns true if widow(er) (aged or disabled), false
   //   otherwise.
   //
   // Returns: True if widow(er) (aged or disabled), false otherwise.
   bool isWidow() const
      { return(getMajorBic() == 'D' || getMajorBic() == 'W'); }
   // Description: Returns true if young spouse.
   //
   // Returns: True if young spouse.
   bool isYoungSpouse() const { return majorBic == 'B' && minorBic == '2'; }
   // Description: Returns true if young survivor (child or mother/father),
   //   false otherwise.
   //
   // Returns: True if young survivor (child or mother/father), false otherwise.
   bool isYoungSurvivor() const
      { return(getMajorBic() == 'C' || getMajorBic() == 'E'); }
   void set( const std::string& newCode );
   // Description: Sets the major bic code.
   //
   // Arguments:
   //   newMajorBic: New major bic code.
   void setMajorBic( char newMajorBic ) { majorBic = newMajorBic; }
   // Description: Sets the minor bic code.
   //
   // Arguments:
   //   newMinorBic: New minor bic code.
   void setMinorBic( char newMinorBic ) { minorBic = newMinorBic; }
   static void survivorCheck( char newMajorBic );
   std::string toString() const;
};
