// $Id: cachup.h 1.15 2006/11/08 10:58:17EST 044579 Development  $
//
// Declarations needed for class to manage catch-up benefit increases.

#pragma once

#include <string>

// {group:Calculation Classes}
// Description: Manages the catch-up benefit increases for one year of
//   eligibility.
//
// Remarks: There have not been any catch-up benefit increases to date; any
//   catch-up increases used would be projected ones.
class CatchupIncreases
{
public:
   // Description: The number of catch-up increases for one year of
   // eligibility (8).
   static const int NUM_CATCHUP_INCREASES = 8;
protected:
   // Catch-up benefit increases for 8 calendar years.
   double theData[NUM_CATCHUP_INCREASES];
public:
   CatchupIncreases();
   ~CatchupIncreases();
   // Description: Returns one catch-up increase.
   //
   // Returns: One catch-up increase.
   //
   // Arguments:
   //   yearIndex: Index of calendary year of increase
   double get( int yearIndex ) const { return(theData[yearIndex]); }
   // Description: Sets one catch-up increase.
   //
   // Arguments:
   //   yearIndex: Index of calendar year of increase.
   //   newCatchup: New value of catch-up increase.
   void set( int yearIndex, double newCatchup )
      { theData[yearIndex] = newCatchup; }
   void parseString( std::string str );
   std::string toString() const;
};

// {group:Calculation Classes}
// Description: Manages the catch-up benefit increases required to
//   calculate a Social Security benefit.
//
// Remarks: There have not been any catch-up benefit increases to date; any
//   catch-up increases used would be projected ones.
class Catchup
{
public:
   // Description: The number of years of eligibility considered (10).
   static const int NUM_CATCHUP_ELIG_YEARS = 10;
protected:
   // Description: Starting year of catch-up benefit increases.
   int cstart;
   // Description: Catch-up benefit increases for 10 years of eligibility.
   //
   // Remarks: Array index is year of eligibility (0 for cstart, 1 for
   //   cstart+1, ..., 9 for cstart+9). For a given array index, calendar
   //   year of increase in catchup goes from 0 for cstart+2, 1 for
   //   cstart+3, ..., to 7 for cstart+9.
   CatchupIncreases catchup[NUM_CATCHUP_ELIG_YEARS];
public:
   Catchup();
   explicit Catchup( int newCstart );
   ~Catchup();
   void deleteContents();
   bool exist( int eligYear, int calYear ) const;
   // Description: Returns one catch-up increase.
   //
   // Returns: One catch-up increase.
   //
   // Arguments:
   //   eligYear: Year of eligibility.
   //   calYear: Calendary year of increase.
   double get( int eligYear, int calYear ) const
   { return((eligYear >= cstart && eligYear < cstart + NUM_CATCHUP_ELIG_YEARS
      && calYear >= cstart + 2 &&
      calYear < cstart + 2 + CatchupIncreases::NUM_CATCHUP_INCREASES) ?
      catchup[eligYear-cstart].get(calYear-cstart-2) : 0.0); }
   // Description: Returns a CatchupIncreases array.
   //
   // Returns: A CatchupIncreases array.
   //
   // Arguments:
   //   index: Index of array to get (0-9).
   const CatchupIncreases& getCatchupIncreases( int index ) const
      { return(catchup[index]); }
   // Description: Gets first year of catch-up increases.
   //
   // Returns: First year of catch-up increases.
   int getCstart() const { return(cstart); }
   void parseString( int index, std::string inputLine );
   void set( int eligYear, int calYear, double newCatchup );
   // Description: Sets first year of catch-up increases.
   //
   // Arguments:
   //   newCstart: New first year of catch-up increases.
   void setCstart( int newCstart ) { cstart = newCstart; }
   static void check( double newCatchup );
};
