// $Id: datemoyr.h 1.28 2008/04/25 13:03:53EDT 277133 Development  $
//
// Declarations for class to manage date as month and year.

#pragma once

#include "date.h"
#include "boost/serialization/access.hpp"
#include "boost/date_time/gregorian/greg_date.hpp"
class Age;

// {group:Miscellaneous Classes}
// Description: Manages a month and year stored as unsigned integers.
//
// Remarks: The two data fields are stored as unsigned shorts to save space.
class DateMoyr
{
   friend class boost::serialization::access;
   // Description: Archives the data.
   //
   // Remarks: When the class Archive corresponds to an output archive, the
   // & operator is defined similar to <<. Likewise, when the class Archive is
   // a type of input archive, the & operator is defined similar to >>.
   //
   // Arguments:
   //   ar: The archive to use.
   //   version: The version of the archive to use.
   template< class Archive >
   void serialize( Archive& ar, const unsigned int version )
      { ar & monthOfYear & yearPart; }
protected:
   // Description: Month of year (1-12).
   unsigned short monthOfYear;
   // Description: The year (1-65000).
   unsigned short yearPart;
public:
   // Description: Constructor.
   //
   // Remarks: The default constructor initializes the month and year to zero.
   // The version with a string argument expects a string with date as mo/year.
   //
   // Throws: PiaException of type PIA_IDS_DATEREAD if unable to set month and
   //   year, or if they have invalid values.
   DateMoyr() : monthOfYear(0), yearPart(0) { }
   // Arguments:
   //   dateMoyr: The date to copy from (month and year).
   DateMoyr( const DateMoyr& dateMoyr ) :
      monthOfYear(dateMoyr.monthOfYear), yearPart(dateMoyr.yearPart) { }
   DateMoyr( const boost::gregorian::date& dateModyyr );
   // Arguments:
   //   newMonth: New month of year (1-12).
   //   newYear: New year.
   DateMoyr( unsigned newMonth, unsigned newYear ) :
      monthOfYear((unsigned short)newMonth),
      yearPart((unsigned short)newYear) { }
   DateMoyr( const std::string& dateTemp );
   // Description: Destroys date.
   ~DateMoyr() { }
   bool operator<( const DateMoyr& dateMoyr ) const;
   // Description: Compares this date to another one.
   //
   // Returns: True if this date equals argument date.
   //
   // Arguments:
   //   dateMoyr: The date to compare to.
   bool operator==( const DateMoyr& dateMoyr ) const
      { return (yearPart == dateMoyr.getYear() &&
        monthOfYear == dateMoyr.getMonth()); }
   // Description: Sets month and year.
   //
   // Remarks: The version that copies from a boost::gregorian::date ignores
   // the day.
   //
   // Arguments:
   //   dateMoyr: The date to copy from (month and year).
   DateMoyr& operator=( const DateMoyr& dateMoyr )
      { monthOfYear = dateMoyr.monthOfYear; yearPart = dateMoyr.yearPart;
        return *this; }
   DateMoyr& operator=( const boost::gregorian::date& dateModyyr );
   DateMoyr operator+( const Age& age ) const;
   Age operator-( const DateMoyr& dateMoyr ) const;
   void check() const;
   // Description: Zeroes month and year.
   void deleteContents() { monthOfYear = yearPart = 0; }
   static Age diff( const DateMoyr& date1, const DateMoyr& date2 );
   int getMonths( const DateMoyr& dateMoyr ) const;
   // Description: Returns month of year.
   //
   // Returns: Month of year (1-12).
   unsigned short getMonth() const { return monthOfYear; }
   DateMoyr getMonthBefore() const;
   // Description: Returns the year.
   //
   // Returns: The year.
   unsigned getYear() const { return yearPart; }
   // Description: Returns index of the month and year, based on January 1971
   //   being 0.
   //
   // Returns: Index of the month and year, based on January 1971 being 0.
   int index() const
      { return(12 * (yearPart - 1971) + monthOfYear - boost::date_time::Jan); }
   // Description: Returns true if the default values of 0 for the year and 
   //   month are present, which shows that it has never been set.
   //
   // Returns: A bool that says whether the default (invalid) values are present.
   bool isNotADate() const { return (monthOfYear == 0 && yearPart == 0); }
   // Description: Checks for month in bounds.
   //
   // Throws: PiaException of type PIA_IDS_DATEMONTH if month is out of range.
   void monthCheck() const { Date::monthCheck(monthOfYear); }
};
