// $Id: dbleann.h 1.29 2006/11/24 08:52:40EST 044579 Development  $
//
// Declarations needed for class to manage an array of annual doubles

#pragma once

#include <numeric>
#include <vector>
#include "boost/serialization/access.hpp"
#include "Resource.h"
#include "PiaException.h"
class IntAnnual;

// {group:Miscellaneous Classes}
// Description: Manages an array of annual doubles.
//
// Remarks: See FloatAnnual and IntAnnual
//   for similar classes that manage arrays of annual floats and integers.
class DoubleAnnual
{
   friend class boost::serialization::access;
   // Description: Archives the data.
   //
   // Remarks: When the class Archive corresponds to an output archive, the
   // & operator is defined similar to <<. Likewise, when the class Archive is
   // a type of input archive, the & operator is defined similar to >>.
   //
   // Arguments:
   //   ar: The archive to use.
   //   version: The version of the archive to use.
   template< class Archive >
   void serialize( Archive& ar, const unsigned int version )
      { ar & theData & width & baseYear & precision & lineWidth; }
protected:
   // Description: Vector of doubles.
   std::vector< double > theData;
   // Description: Width of field when writing to an output file.
   int width;
   // Description: Base year of data.
   int baseYear;
   // Description: Number of decimal places when writing to an output file.
   int precision;
   // Description: Width of output line when writing to an output file.
   int lineWidth;
public:
   DoubleAnnual();
   DoubleAnnual( int newBaseYear, int newLastYear );
   DoubleAnnual( const DoubleAnnual& newData );
   virtual ~DoubleAnnual();
   DoubleAnnual& operator=( const DoubleAnnual& newData );
   // Description: Returns one value of data.
   //
   // Returns: One value of data for the specified year.
   //
   // Throws: PiaException of type PIA_IDS_DBLEANN1 if year is out of range
   //   (only in debug mode).
   //
   // Arguments:
   //   year: Year for which data is desired.
   double& operator[]( int year )
   {
#if !defined(NDEBUG)
      if (year < baseYear || year > getLastYear()) {
         throw PiaException(PIA_IDS_DBLEANN1);
      }
#endif
      return theData[year - baseYear];
   }
   const double operator[]( int year ) const
   {
#if !defined(NDEBUG)
      if (year < baseYear || year > getLastYear()) {
         throw PiaException(PIA_IDS_DBLEANN1);
      }
#endif
      return theData[year - baseYear];
   }
   // Description: Accumulates data over a range of years.
   //
   // Remarks: Accumulates data over a range of years.  Does not check to be
   // sure range of years is within allowable range, unless in debug mode.
   //
   // Throws: PiaException of type PIA_IDS_DBLEANN1 if years are out of range,
   // if in debug mode.
   //
   // Returns: Total of data in the vector over the specified range.
   //
   // Arguments:
   //   year1: Year with which to start accumulating (inclusive).
   //   year2: Year with which to end accumulating (inclusive).
   //   startValue: Starting value.
   double accumulate( int year1, int year2, double startValue ) const
   {
#if !defined(NDEBUG)
      if (year1 < baseYear || year1 > year2 || year2 > getLastYear()) {
         throw PiaException(PIA_IDS_DBLEANN1);
      }
#endif
      return std::accumulate(theData.begin() + year1 - baseYear,
         theData.begin() + year2 + 1 - baseYear, startValue);
   }
   void add( const DoubleAnnual& newData, int firstYeart, int lastYeart );
   void assign( const DoubleAnnual& newData, int firstYeart, int lastYeart );
   void assign( const IntAnnual& newData, int firstYeart, int lastYeart );
   void assign( double newData, int firstYeart, int lastYeart );
   void deleteContents();
   void divide( const DoubleAnnual& newData, int firstYeart, int lastYeart );
   void divide( const IntAnnual& newData, int firstYeart, int lastYeart );
   // Description: Returns base year of data.
   //
   // Returns: Base year of data.
   int getBaseYear() const { return(baseYear); }
   // Description: Returns last year of data.
   //
   // Remarks: This is the baseYear plus the size of theData, minus 1. Note
   // that this is less than the baseYear if the size is zero.
   //
   // Returns: Last year of data.
   int getLastYear() const
      { return(baseYear + static_cast< int >(theData.size()) - 1); }
   // Description: Returns output line width.
   //
   // Returns: Output line width.
   int getLineWidth() const { return lineWidth; }
   // Description: Returns precision of output field.
   //
   // Returns: Precision of output field.
   int getPrecision() const { return precision; }
   // Description: Returns width of output field.
   //
   // Returns: Width of output field.
   int getWidth() const { return width; }
   // Description: Increments one value of data by a given amount.
   //
   // Throws: PiaException of type PIA_IDS_DBLEANN1 if year is out of range, if
   //   in debug mode.
   //
   // Arguments:
   //   year: Year for which data is incremented.
   //   amount: Amount by which data is incremented.
   void increment( int year, double amount )
   {
#ifndef NDEBUG
      if (year < baseYear || year > getLastYear())
         throw PiaException(PIA_IDS_DBLEANN1);
#endif
      theData[year - baseYear] += amount;
   }
   void multiply( const DoubleAnnual& series, int firstYeart, int lastYeart );
   void multiply( double factor, int firstYeart, int lastYeart );
   void read( std::istream& in, int firstYeart, int lastYeart );
   void resetYears( int newBaseYear, int newLastYear );
   // Description: Initializes line width.
   //
   // Arguments:
   //   newLineWidth: New line width.
   void setLineWidth( int newLineWidth ) { lineWidth = newLineWidth; }
   // Description: Initializes precision of output field.
   //
   // Arguments:
   //   newPrecision: New value of precision.
   void setPrecision( int newPrecision ) { precision = newPrecision; }
   // Description: Initializes width of output field.
   //
   // Arguments:
   //   newWidth: New width.
   void setWidth( int newWidth ) { width = newWidth; }
   void subtract( const DoubleAnnual& newData, int firstYeart, int lastYeart );
   void write( std::ostream& out, int firstYeart, int lastYeart );
};
