// $Id: inscode.h 1.15 2006/11/08 10:58:58EST 044579 Development  $
//
// Declarations needed for class to manage insured status code.

#pragma once

// {group:Calculation Classes}
// Description: Represents one insured status code.
//
// Remarks: An instance of this class is used in PiaData. An array of instances
// is used in InsCodeArray.
class InsCode
{
public:
   // Description: Insured status code types.
   //
   // Remarks: Most of these codes come from the CWHS.  The last two are added
   // for use in OACT programs.
   enum InsCodeType {
      NOQCS,            // 0 - active (no QC's to date).
      FULLANDCURRENT,   // 1 - fully and currently insured.
      CURRENTNOTFULL,   // 2 - currently but not fully insured.
      FULLNOTCURRENT,   // 3 - fully but not currently insured.
      UNINSURED,        // 4 - uninsured (but some QC's to date).
      PERMANDCURRENT,   // 5 - permanently and currently insured.
      PERMNOTCURRENT,   // 6 - permanently but not currently insured.
      PRIMARYDEATH,     // 7 - primary-entitled death.
      TRANSITIONAL,     // 8 - transitionally insured.
      NONPRIMARYDEATH,  // 9 - non-primary-entitled death.
      DEEMED,           // D - deemed fully insured (1983 amendments).
      TOTALIZED         // T - insured for totalization benefits.
   };
protected:
   // Description: Insured status code.
   InsCodeType theCode;
public:
   InsCode();
   InsCode( InsCodeType newCode );
   // Description: Returns true if the InsCodeType's are equal.
   //
   // Returns: True if the InsCodeType's are equal.
   //
   // Arguments:
   //   newCode: The insured status code to compare against.
   bool operator==( const InsCode& newCode ) const
      { return (theCode == newCode.theCode); }
   // Description: Returns the insured status code.
   //
   // Returns: The insured status code.
   InsCodeType get() const { return theCode; }
   // Description: Returns the currently insured status.
   //
   // Returns: True if currently insured, false otherwise.
   bool isCurrentlyInsured() const
   { return(theCode == FULLANDCURRENT || theCode == CURRENTNOTFULL ||
     theCode == PERMANDCURRENT); }
   // Description: Returns the fully insured status.
   //
   // Returns: True if fully insured, false otherwise.
   bool isFullyInsured() const
   { return(theCode == FULLANDCURRENT || theCode == FULLNOTCURRENT ||
     theCode == PERMANDCURRENT || theCode == PERMNOTCURRENT ||
     theCode == DEEMED); }
   // Description: Returns the fully or totalization insured status.
   //
   // Returns: True if fully insured or insured for totalization benefits,
   //   false otherwise.
   bool isFullyOrTotalInsured() const
      { return(isFullyInsured() || theCode == TOTALIZED); }
   // Description: Returns the fully or currently insured status.
   //
   // Returns: True if currently insured or fully insured, false otherwise.
   bool isFullyOrCurrentlyInsured() const
      { return(isFullyInsured() || isCurrentlyInsured()); }
   // Description: Sets the insured status code.
   //
   // Arguments:
   //   newCode: New insured status code.
   void set( InsCodeType newCode ) { theCode = newCode; }
   // Description: Returns the insured status code as a character.
   //
   // Remarks: Calls the static version of this function using the stored code.
   //
   // Returns: The insured status code as a character.
   char toChar() const { return(toChar(theCode)); }
   static char toChar( InsCodeType newCode );
   static InsCodeType ChartoCode( char inscodechar );
};
