// $Id: intann.h 1.21 2007/01/09 14:36:11EST 044579 Development  $
//
// Declarations needed for class to manage an array of annual integers.

#pragma once

#include <vector>
#include "PiaException.h"
#include "Resource.h"

// {group:Miscellaneous Classes}
// Description: Manages an array of annual integers.
//
// Remarks: See DoubleAnnual and FloatAnnual for similar classes that manage
//   arrays of annual doubles and floats.
class IntAnnual
{
protected:
   // Description: Integer array.
   std::vector< int > theData;
   // Description: Width of output field.
   int width;
   // Description: Base year of data.
   int baseYear;
   // Description: Width of output line.
   int lineWidth;
public:
   IntAnnual();
   IntAnnual( int newBaseYear, int newLastYear );
   IntAnnual( const IntAnnual& intAnnual );
   // Description: Destructor.
   //
   // Remarks: Does nothing.
   ~IntAnnual() { }
   IntAnnual& operator=( const IntAnnual& intAnnual );
   // Description: Returns one value of data.
   //
   // Throws: PiaException of type PIA_IDS_INTANN1 if year is out of range
   //   (only in debug mode).
   //
   // Returns: One value of data for specified year.
   //
   // Arguments:
   //   year: Year for which data is desired.
   int& operator[]( int year )
   {
   #if !defined(NDEBUG)
      if (year < baseYear || year > getLastYear()) {
         throw PiaException(PIA_IDS_INTANN1);
      }
   #endif
      return theData[year - baseYear];
   }
   const int operator[]( int year ) const
   {
   #if !defined(NDEBUG)
      if (year < baseYear || year > getLastYear()) {
         throw PiaException(PIA_IDS_INTANN1);
      }
   #endif
      return theData[year - baseYear];
   }
   void add( const IntAnnual& vector1, int firstYeart, int lastYeart );
   void assign( const IntAnnual& intAnnual, int firstYeart, int lastYeart );
   void assign( int newData );
   void assign( int newData, int firstYeart, int lastYeart );
   void deleteContents();
   void divide( const IntAnnual& vector1, int firstYeart, int lastYeart );
   // Description: Returns base year of data.
   //
   // Returns: Base year of data.
   int getBaseYear() const { return(baseYear); }
   // Description: Returns last year of data.
   //
   // Remarks: This is the baseYear plus the size of theData, minus 1. Note
   // that this is less than the baseYear if the size is zero.
   //
   // Returns: Last year of data.
   int getLastYear() const
      { return(baseYear + static_cast< int >(theData.size()) - 1); }
   // Description: Returns output line width.
   //
   // Returns: Output line width.
   int getLineWidth() const { return lineWidth; }
   // Description: Returns field width.
   //
   // Returns: Field width.
   int getWidth() const{ return width; }
   // Description: Increments one value of data. When there is no amount, the
   //   increment is 1.
   //
   // Throws: PiaException of type PIA_IDS_INTANN1 if year is out of range
   //   (only in debug mode).
   //
   // Arguments:
   //   year: Year for which data is incremented.
   void increment( int year )
   {
   #ifndef NDEBUG
      if (year < baseYear || year > getLastYear())
         throw PiaException(PIA_IDS_INTANN1);
   #endif
      theData[year - baseYear]++;
   }
   // Arguments:
   //   year: Year for which data is incremented.
   //   amount: Amount by which data is incremented.
   void increment( int year, int amount )
   {
   #ifndef NDEBUG
      if (year < baseYear || year > getLastYear())
         throw PiaException(PIA_IDS_INTANN1);
   #endif
      theData[year - baseYear] += amount;
   }
   void multiply( const IntAnnual& vector1, int firstYeart, int lastYeart );
   void read( std::istream& in, int firstYeart, int lastYeart );
   void resetYears( int newBaseYear, int newLastYear );
   // Description: Initializes output line width.
   //
   // Arguments:
   //   newLineWidth: New line width.
   void setLineWidth( int newLineWidth ) { lineWidth = newLineWidth; }
   // Description: Initializes field width.
   //
   // Arguments:
   //   newWidth: New value of field width.
   void setWidth( int newWidth ) { width = newWidth; }
   void subtract( const IntAnnual& vector1, int firstYeart, int lastYeart );
   void write( std::ostream& out, int firstYeart, int lastYeart );
};
