// $Id: outfile.h 1.23 2007/03/08 14:10:44EST 044579 Development  $
//
// Declarations for class to manage output file with page counters.

#pragma once

#include "document.h"
#include "boost/date_time/gregorian/greg_date.hpp"

// {group:Miscellaneous Classes}
// Description: Manages an output file in runstreams with multiple programs.
//
// Remarks: Manages an output file in
//   runstreams with multiple programs, such as ssgiat, ssgbp, and
//   jrunfunds.  It also manages a 2-line title and a date and time string
//   for output at the top of each page, along with the page number.  It
//   is an abstract class; one of its subclasses (OutFile80, OutDC80,
//   OutFile132, or OutDC132) should be instantiated.
class Outfile : public Document
{
public:
   // Description: Page number.
   int pageNum;
   // Description: The current date as month, day, and year.
   boost::gregorian::date dateModyyr;
   // Description: Number of title lines.
   static const int PAGE_TITLE_LINES = 2;
protected:
   // Description: Two-line title at the top of each output page.
   std::string pageTitle[PAGE_TITLE_LINES];
   // Description: First part of command to send output to printer.
   std::string command1;
   // Description: Second part of command to send output to printer.
   std::string command2;
public:
   Outfile( const std::string& newCorename, const std::string& newExtension );
   virtual ~Outfile();
   // Description: Move to next line.
   virtual void advanceLine() = 0;
   void footer();
   // Description: Move to next page.
   virtual void formfeed() = 0;
   // Description: Returns page number.
   //
   // Returns: Page number.
   int getPageNum() const { return pageNum; }
   // Description: Returns one page title line.
   //
   // Returns: Specified page title line.
   //
   // Arguments:
   //   number: Page title line number (0 or 1).
   const std::string& gettitle ( int number ) const
      { return pageTitle[number]; }
   void header();
   // Description: Writes date, time, title, and page to print file.
   virtual void pgtitle() = 0;
   void pgtitlehtml();
   // Description: Print a line of output with a carriage return.
   virtual void printLine( const std::string& ) = 0;
   virtual void printLine( const char* ) = 0;
   // Description: Print a line of output with no carriage return.
   virtual void printLinena( const std::string& ) = 0;
   virtual void printLinena( const char* ) = 0;
   int printout();
   // Description: Sets page number.
   //
   // Arguments:
   //   newPageNum: New page number.
   void setPageNum( int newPageNum ) { pageNum = newPageNum; }
   void setprninfo( const std::string& newCommand1,
      const std::string& newCommand2 );
   // Description: Sets one page title line.
   //
   // Arguments:
   //   number: Page title line number (0 or 1).
   //   newPageTitle: New page title line.
   void settitle( int number, const std::string& newPageTitle )
      { pageTitle[number].assign(newPageTitle); }
};

// {group:Miscellaneous Classes}
// Description: An Outfile tailored for 132-column output.
//
// Remarks: This is an abstract class; one of its subclasses (OutFile132 or
//   OutDC132) should be instantiated.
class Out132 : public Outfile
{
public:
   Out132( const std::string& newCorename, const std::string& newExtension );
   void pgtitle();
   static void prepareStrings( std::string* stringTable, int* count,
      int* pageNum, const std::string* newPageTitle );
};

// {group:Miscellaneous Classes}
// Description: An Outfile tailored for 80-column output.
//
// Remarks: This is an abstract class; one of its subclasses (OutFile80 or
//   OutDC80) should be instantiated.
class Out80 : public Outfile
{
public:
   Out80( const std::string& newCorename, const std::string& newExtension );
   void pgtitle();
   static void prepareStrings( std::string* stringTable, int* count,
      int* pageNum, const std::string* newPageTitle );
};
