// $Id: pathname.h 1.10 2002/06/07 12:22:27EDT sfmckay Exp  $
//
// Declarations for class to manage pathname of a file.

#ifndef __PATHNAME_H
#define __PATHNAME_H    1    // Prevent multiple #includes

#include <string>

// {group:Miscellaneous Classes}
// Description: Manages the pathname of a file.
//
// Remarks: The pathname is stored in three parts, as well as a full pathname.
class Pathname
{
   public:
      // Description: Separator between directory names and file name.
      //
      // Remarks: This is determined at compile time to be either a backward
      //   slash (for Windows) or a forward slash (for anything else).
      static const std::string separator;
      // Description: Separator between directory names and file name.
      //
      // Remarks: This is determined at compile time to be either a backward
      //   slash (for Windows) or a forward slash (for anything else).
      static const char separatorChar;
   protected:
      // Path to file.
      std::string pathname;
      // Core file name.
      std::string corename;
      // Filename extension.
      std::string extension;
      // Full pathname.
      std::string fullname;
   public:
      Pathname();
      explicit Pathname( const std::string& newDirectory );
      std::string getCorename() const;
      std::string getExtension() const;
      std::string getFullname() const;
      std::string getPathname() const;
      void  setCorename( const std::string& newCorename );
      void  setCorename( int start, const std::string& newCorename );
      void  setExtension( const std::string& newExtension );
      void  setFullname( const std::string& newFullname );
      void  setPathname( const std::string& newPathname );
   protected:
      void  pastename();
};

// Description: Returns core filename.
//
// Returns: Core filename.
inline std::string Pathname::getCorename() const
   { return(corename); }

// Description: Returns extension of filename.
//
// Returns: Extension of filename.
inline std::string Pathname::getExtension() const
   { return(extension); }

// Description: Returns full filename.
//
// Returns: Full filename.
inline std::string Pathname::getFullname() const
   { return(fullname); }

// Description: Returns path part of filename.
//
// Returns: Path part of filename.
inline std::string Pathname::getPathname() const
   { return(pathname); }

// Description: Sets path leading to filename.
//
// Arguments:
//   newPathname: New pathname.
inline void Pathname::setPathname( const std::string& newPathname )
   { pathname = newPathname; pastename(); }

// Description: Sets core filename.
//
// Remarks: The whole core filename is replaced unless a starting position is
//   specified.
//
// Arguments:
//   newCorename: New core filename.
inline void Pathname::setCorename( const std::string& newCorename )
   { corename = newCorename; pastename(); }

// Arguments:
//   start: Starting point of new core filename. New core filename starts
//     at position 0 if not specified.
//   newCorename: New core filename.
inline void Pathname::setCorename( int start, const std::string& newCorename )
   { corename = corename.substr(0, start);
     corename += newCorename; pastename(); }

// Description: Sets extension of filename.
//
// Arguments:
//   newExtension: New extension of filename.
inline void Pathname::setExtension( const std::string& newExtension )
   { extension = newExtension; pastename(); }

// Description: Sets full filename from a string.
//
// Arguments:
//   newFullname: New full filename.
inline void Pathname::setFullname( const std::string& newFullname )
   { fullname = newFullname; }

#endif     // __PATHNAME_H
