// $Id: pebs.h 1.11 2006/11/08 10:59:16EST 044579 Development  $
//
// Declarations needed for class to manage PEBS information for benefit
// calculation.

#pragma once

#include <vector>
#include "age.h"

// {group:Calculation Classes}
// Description: Manages most of the data required to prepare and
//   print out the results of a Statement calculation.
//
// Remarks: Manages most of the data required to prepare and
//   print out the results of a Statement calculation.
class Pebs
{
public:
   // Description: Type of Statement calculation.
   enum pebes_type {
      PEBS_OAB_DELAYED,  // Delayed retirement.
      PEBS_OAB_FULL,  // Full retirement age.
      PEBS_OAB_EARLY,  // Early retirement.
      PEBS_SURV,  // Survivor benefit.
      PEBS_DISAB,  // Disability benefit.
      MAXPEBS  // Number of Statement calculations.
   };
   // Description: Current age.
   Age ageNow;
   // Description: Age 70 and 0 months.
   static const Age age70;
   // Description: Planned early retirement age, after adjusting for current
   // age and earliest possible age (62).
   Age agePlan1;
   // Description: Worker's full retirement age.
   Age fullRetAge;
   // Description: Age for full retirement calculation (could be between full
   // retirement and 70).
   Age oab1;
   // Description: Age for late retirement calculation (could be 70 or later).
   Age oab2;
protected:
   // Description: Benefit actually payable in Statement calculation.
   std::vector< int > benefitPebs;
   // Description: Planned retirement age as entered.
   int agePlan;
   // Description: Planned retirement age (adjusted to 62 if none
   // entered), or current age, if greater.
   int agePlan2;
   // Description: MFB for Statement calculation.
   std::vector< int > mfbPebs;
   // Description: Month of processing (1-12).
   int monthnow;
   // Description: PIA for PEBS calculation.
   std::vector< int > piaPebs;
   // Description: Quarters of coverage earned through current year.
   int qcTotal;
   // Description: Quarters of coverage required for disability
   // insured status for Statement calculation.
   int qcDisReq;
   // Description: Quarters of coverage earned for disability
   // insured status for Statement calculation.
   int qcDisTotal;
   // Description: Number of years over which quarters of coverage must be
   // earned for disability insured status for Statement calculation.
   int qcDisYears;
   // Description: Quarters of coverage required for fully
   // insured status for Statement calculation.
   std::vector< int > qcReq;
   // Description: Number of oab calculations to do.
   //
   // Remarks:
   //
   // 1 - age 70 or later only.
   // 2 - add full retirement age or later.
   // 3 - add early retirement.
   int pebsOab;
   // Description: Indicator for dib calculation.
   //
   // Remarks:
   //
   // 1 - do dib calculation.
   // 0 - do not do it.
   int pebsDib;
public:
   Pebs();
   void deleteContents();
   // Description: Returns planned age of retirement.
   //
   // Returns: Planned age of retirement.
   int getAgePlan() const { return(agePlan); }
   // Description: Returns planned retirement age.
   //
   // Returns: Planned retirement age.
   int getAgePlan2() const { return(agePlan2); }
   // Description: Returns benefit actually payable in Statement calculation.
   //
   // Returns: Benefit actually payable in Statement calculation.
   //
   // Arguments:
   //   type: Type of calculation (0-4).
   int getBenefitPebs( pebes_type type ) const { return(benefitPebs[type]); }
   // Description: Returns maximum family benefit in Statement calculation.
   //
   // Returns: Maximum family benefit in Statement calculation.
   //
   // Arguments:
   //   type: Type of calculation (0-4).
   int getMfbPebs( pebes_type type ) const { return(mfbPebs[type]); }
   // Description: Returns month of processing.
   //
   // Returns: Month of processing.
   int getMonthnow() const { return(monthnow); }
   // Description: Returns number of dib computations to do.
   //
   // Returns: Number of dib computations to do.
   int getPebsDib() const { return(pebsDib); }
   // Description: Returns number of oab computations to do.
   //
   // Returns: Number of oab computations to do.
   int getPebsOab() const { return(pebsOab); }
   // Description: Returns PIA in Statement calculation.
   //
   // Returns: PIA in Statement calculation.
   //
   // Arguments:
   //   type: Type of calculation (0-4).
   int getPiaPebs( pebes_type type ) const { return(piaPebs[type]); }
   // Description: Returns quarters of coverage required for disability insured
   //   status for Statement calculation.
   //
   // Returns: Quarters of coverage required for disability insured
   //   status for Statement calculation.
   int getQcDisReq() const { return(qcDisReq); }
   // Description: Returns quarters of coverage earned for disability insured
   //   status for Statement calculation.
   //
   // Returns: Quarters of coverage earned for disability insured
   //   status for Statement calculation.
   int getQcDisTotal() const { return(qcDisTotal); }
   // Description: Returns number of years over which quarters of coverage must
   //   be earned for disability insured status for Statement calculation.
   //
   // Returns: Number of years over which quarters of coverage must
   //   be earned for disability insured status for Statement calculation.
   int getQcDisYears() const { return(qcDisYears); }
   // Description: Returns required quarters of coverage in Statement
   // calculation.
   //
   // Returns: Required quarters of coverage in Statement calculation.
   //
   // Arguments:
   //   type: Type of calculation (0-4).
   int getQcReq( pebes_type type ) const { return(qcReq[type]); }
   // Description: Returns quarters of coverage earned through current year.
   //
   // Returns: Quarters of coverage earned through current year.
   int getQcTotal() const { return(qcTotal); }
   void monthnowCheck() const;
   void setAgePlan( int newAgePlan );
   // Description: Sets planned retirement age.
   //
   // Arguments:
   //   newAgePlan2: New planned retirement age.
   void setAgePlan2( int newAgePlan2 ) { agePlan2 = newAgePlan2; }
   // Description: Sets benefit actually payable in Statement calculation.
   //
   // Arguments:
   //   type: Type of calculation (0-4).
   //   newBenefit: New benefit.
   void setBenefitPebs( pebes_type type, int newBenefit )
      { benefitPebs[type] = newBenefit; }
   // Description: Sets maximum family benefit in Statement calculation.
   //
   // Arguments:
   //   type: Type of calculation (0-4).
   //   newMfb: New maximum family benefit.
   void setMfbPebs( pebes_type type, int newMfb ) { mfbPebs[type] = newMfb; }
   void setMonthnow( int newMonthnow );
   void setMonthnow();
   // Description: Sets number of dib computations to do.
   //
   // Arguments:
   //   newPebsDib: New number of dib computations.
   void setPebsDib( int newPebsDib ) { pebsDib = newPebsDib; }
   // Description: Sets number of oab computations to do.
   //
   // Arguments:
   //   newPebsOab: New number of oab computations.
   void setPebsOab( int newPebsOab ) { pebsOab = newPebsOab; }
   // Description: Sets PIA in Statement calculation.
   //
   // Arguments:
   //   type: Type of calculation (0-4).
   //   newPia: New PIA.
   void setPiaPebs( pebes_type type, int newPia ) { piaPebs[type] = newPia; }
   // Description: Sets number of DI quarters of coverage required.
   //
   // Arguments:
   //   newQcDisReq: New value for number of DI quarters of coverage required.
   void setQcDisReq( int newQcDisReq ) { qcDisReq = newQcDisReq; }
   // Description: Sets number of DI quarters of coverage earned.
   //
   // Arguments:
   //   newQcDisTotal: New value for number of DI quarters of coverage earned.
   void setQcDisTotal( int newQcDisTotal ) { qcDisTotal = newQcDisTotal; }
   // Description: Sets number of years allowed to earn DI quarters of
   // coverage.
   //
   // Arguments:
   //   newQcDisYears: New value for years allowed to earn DI quarters of
   //     coverage.
   void setQcDisYears( int newQcDisYears ) { qcDisYears = newQcDisYears; }
   // Description: Sets required quarters of coverage in Statement calculation.
   //
   // Arguments:
   //   type: Type of calculation (0-4).
   //   newQcReq: New required quarters of coverage.
   void setQcReq( pebes_type type, int newQcReq ) { qcReq[type] = newQcReq; }
   // Description: Sets number of quarters of coverage earned through current
   //   year.
   //
   // Arguments:
   //   newQcTotal: New number of quarters of coverage.
   void setQcTotal( int newQcTotal ) { qcTotal = newQcTotal; }
   static void agePlanCheck( int agePlant );
   static void monthnowCheck( int monthnowt );
};
