// $Id: piacal.h 1.69 2007/03/08 14:10:55EST 044579 Development  $
//
// Declarations needed for class to manage pia calculation functions.

#pragma once

#include <vector>
#include "PiaMethod.h"
#include "SecondaryArray.h"
#include "inscode.h"
class Secondary;
class PiaDataArray;
class WorkerDataArray;
class PiaParams;
class DateMoyr;
class WageInd;
class ReindWid;
class OldStart;
class SpecMin;
class PiaTable;
class FrozMin;
class ChildCareCalc;
class TransGuar;
class DibGuar;

// {group:Calculation Classes}
// Description: Parent of all classes that manage the functions required for
//   calculation of a Social Security benefit.
//
// Remarks: This class is abstract; child classes are PiaCalAny,
//   PiaCalLC, and PiaCalPL.
class PiaCal
{
   public:
      // Worker's personal data.
      WorkerDataGeneral& workerData;
      // Worker's pia calculation data.
      PiaData& piaData;
      // Pia calculation parameters.
      const PiaParams& piaParams;
      // Pia calculation methods that apply to all benies in the family.
      std::vector<PiaMethod *> piaMethod;
      // Pia calculation methods that apply only to widow(er)s.
      std::vector<ReindWid *> widowMethod;
      // Pointer to wage-indexed method.
      WageInd* wageInd;
      // Pointer to old-start method.
      OldStart* oldStart;
      // Pointer to special minimum method.
      SpecMin* specMin;
      // Pointer to pia table method.
      PiaTable* piaTable;
      // Pointer to frozen minimum method.
      FrozMin* frozMin;
      // Pointer to child care calculation.
      ChildCareCalc* childCareCalc;
      // Pointer to transitional guarantee.
      TransGuar* transGuar;
      // Pointer to disability guarantee.
      DibGuar* dibGuar;
      // Pointer to method with high pia.
      PiaMethod* highPiaMethod;
   protected:
      // Description: Type of benefit, set even if Statement case.
      //
      // Remarks:
      //
      //   1 - old-age.
      //   2 - survivor.
      //   3 - disability.
      WorkerDataGeneral::ben_type ioasdi;
      // True if old Pebs assumptions (with 1-percent real wage gain).
      bool pebsAssumptions;
public:
   PiaCal( WorkerDataGeneral& newWorkerData, PiaData& newPiaData,
      const PiaParams& newPiaParams );
   virtual ~PiaCal();
   void ageEntCheck() const;
   // Description: Applies maximum family benefit (already calculated) to
   //   family's benefits in a life case.
   //
   // Remarks: The version with 3 arguments calls the static version with 6
   //   arguments, passing saved values for the other arguments. The worker's
   //   pia is subtracted from the mfb before applying the mfb to the
   //   auxiliaries' total benefits.
   //
   // Arguments:
   //   secondaryArray: Benefits for all family members.
   //   famSize: Number of family members.
   //   year: Year of rounding.
   void applyLifeMfb( SecondaryArray& secondaryArray, int famSize,
      int year ) const
      { applyLifeMfb(secondaryArray, famSize, year, piaData.highMfb.get(),
          piaData.highPia.get(), secondaryArray.getTotalFullBenefitMax()); }
   // Description: Applies maximum family benefit (already calculated) to
   //   family's benefits in a survivor case.
   //
   // Remarks: The version with 3 arguments calls the static version with 5
   //   arguments, passing saved values for the other arguments.
   //
   // Arguments:
   //   secondaryArray: Benefits for all family members.
   //   famSize: Number of family members.
   //   year: Year of rounding.
   void applySurvivorMfb( SecondaryArray& secondaryArray, int famSize,
       int year ) const
      { applySurvivorMfb(secondaryArray, famSize, year, piaData.highMfb.get(),
          secondaryArray.getTotalFullBenefitMax()); }
   void ardriAuxCal( WorkerDataGeneral *widowData, PiaData *widowPiaData,
      Secondary *secondaryData, const DateMoyr& entDate ) const;
   void ardriCal() const;
   virtual void calculate2( const DateMoyr& entDate );
   virtual void dataCheck( const DateMoyr& entDate );
   void dataCheckAux( WorkerDataArray& widowDataArray,
      PiaDataArray& widowArray, SecondaryArray& secondaryArray );
   virtual void earnProjection() const;
   DateMoyr fullInsDateCal() const;
   // Description: Returns type of benefit.
   //
   // Returns: Type of benefit.
   WorkerDataGeneral::ben_type getIoasdi() const { return(ioasdi); }
   InsCode::InsCodeType insCal( const boost::gregorian::date& dateModyyr,
      int isWasPrimary );
   // Description: Returns Statement assumptions indicator.
   //
   // Returns: True if old Pebs assumptions (with 1-percent real wage gain).
   bool isPebsAssumptions() const { return pebsAssumptions; }
      void   monthsArAgedSpouseCal( PiaData *widowData,
         Secondary *secondaryData ) const;
      void   monthsArCal() const;
      void   monthsArDICal() const;
      void   monthsArDisWidCal( PiaData *widowData,
         Secondary *secondaryData, const DateMoyr& entDate ) const;
      void   monthsArWidCal( PiaData *widowData,
         Secondary *secondaryData ) const;
      void   monthsDriCal() const;
      void   monthsYngLifeCal( Secondary *secondaryData ) const;
      void   monthsYngSurvCal( Secondary *secondaryData ) const;
      virtual void nCal( CompPeriod& compPeriod, const DateMoyr& entDate );
      void   nCal1    ( CompPeriod& compPeriod, const DateMoyr& entDate );
      virtual int nelapsed2Cal( const DateMoyr& entDate );
      void   nelapsedCal( CompPeriod& compPeriod, const DateMoyr& entDate );
      void   nelapsedCal( CompPeriod& compPeriod, int elapsed2 );
      // Calculates PIA and MFB, not including re-indexed widow(er).
      virtual void piaCal( const DateMoyr& ) = 0;
      void   piaCal1();
      void   piaCal2();
      void   piaCal3( PiaDataArray& widowArray,
         SecondaryArray& secondaryArray );
      void   qcCal();
      void   reindWidCalAll( const WorkerDataArray& widowDataArray,
         PiaDataArray& widowArray, SecondaryArray& secondaryArray );
      // Calculates PIA for one re-indexed widow(er).
      virtual void reindWidCal( const WorkerDataGeneral&, const PiaData&,
         Secondary& ) = 0;
      float  relEarnPositionCal();
      void   releaseMemory();
      void   resetAmend90();
      void   setAmend90( const DateMoyr& entDate );
      void   setArfApp();
      void   setEligYearWidow( WorkerDataArray& widowDataArray,
         PiaDataArray& widowArray, SecondaryArray& secondaryArray ) const;
      void   setHighMfb();
      void   setHighPia();
   // Description: Sets the type of benefit.
   //
   // Arguments:
   //   ioasdit: New type of benefit.
   void setIoasdi( WorkerDataGeneral::ben_type ioasdit ) { ioasdi = ioasdit; }
   // Description: Sets Statement assumptions indicator.
   //
   // Arguments:
   //   newPebsAssumptions: True if old Pebs assumptions (with 1-percent real
   //     wage gain).
   void setPebsAssumptions( bool newPebsAssumptions )
      { pebsAssumptions = newPebsAssumptions; }
   void setPifc();
   void setSupportPia();
   // Arguments:
   //   secondaryArray: Benefits for all family members.
   //   famSize: Number of family members.
   //   year: Year of rounding.
   //   mfb: Maximum family benefit.
   //   pia: Primary insurance amount.
   //   totalBenefit: Total family benefit before applying maximum.
   static void applyLifeMfb( SecondaryArray& secondaryArray, int famSize,
      int year, double mfb, double pia, double totalBenefit )
      { applyMfb(secondaryArray, famSize, (mfb - pia) / totalBenefit, year); }
   static void applyMfb( SecondaryArray& secondaryArray, int famSize,
      double ratio, int year );
   // Arguments:
   //   secondaryArray: Benefits for all family members.
   //   famSize: Number of family members.
   //   year: Year of rounding.
   //   mfb: Maximum family benefit.
   //   totalBenefit: Total family benefit before applying maximum.
   static void applySurvivorMfb( SecondaryArray& secondaryArray,
      int famSize, int year, double mfb, double totalBenefit )
      { applyMfb(secondaryArray, famSize, mfb / totalBenefit, year); }
   static InsCode::InsCodeType deadIns( int iswasPrimary );
      static bool deemedInsCal( PiaData& piaData, const QtrYear& qtrYear );
      static int didropCal( const WorkerDataGeneral& workerData, int elapsed1,
         int elapsed2, FreezeYears& frzyrst );
      static DisInsCode::DisInsCodeType disInsCal(
         const WorkerDataGeneral& workerData, PiaData& piaData,
         const DateMoyr& dateMoyr, int iswasPrimary );
      static void eligYearCal( const WorkerDataGeneral& workerData,
         PiaData& piaData, WorkerDataGeneral::ben_type ioasdi );
      static DateMoyr eligYearCal1( const WorkerDataGeneral& workerData,
         PiaData& piaData, WorkerDataGeneral::ben_type ioasdi, int jind7 );
      static int eligYearCal2 ( const WorkerDataGeneral& workerData,
         PiaData& piaData, WorkerDataGeneral::ben_type ioasdi, int eligYear );
      static int eligYearWidowCal( char majorBic, int byear,
         boost::gregorian::date onsetDate );
      static InsCode::InsCodeType fins1Cal ( const WorkerDataGeneral& workerData,
         PiaData& piaData, const QtrYear& qtrYear, int iswasPrimary );
      static char fins2Cal ( const WorkerDataGeneral& workerData,
         const PiaData& piaData, WorkerDataGeneral::ben_type ioasdi );
      static bool getPubpenEnable( const WorkerDataGeneral& workerData );
      static InsCode::InsCodeType insCal( const WorkerDataGeneral& workerData,
         PiaData& piaData, WorkerDataGeneral::ben_type ioasdi,
         const QtrYear& qtrYear, int isWasPrimary );
      static int qcCurrentCal( const PiaData& piaData,
         const QtrYear& qtrYear );
      static int qcReqCal ( const WorkerDataGeneral& workerData,
         PiaData& piaData, WorkerDataGeneral::ben_type ioasdi, int year );
      static int qcReqPermCal( const WorkerDataGeneral& workerData,
         PiaData& piaData, WorkerDataGeneral::ben_type ioasdi );
   // Description: Checks disabled widow's date of disability.
   //
   // Remarks: The static version with 2 arguments calls the static version
   // with 3 arguments.
   //
   // Throws: PiaException of type PIA_IDS_JONSET5 if disability is before
   // birth. PiaException of type PIA_IDS_JONSET6 if disability is after
   // entitlement.
   //
   // Arguments:
   //   widowData: Family member's information.
   //   secondaryData: Family member's benefit.
   static void widowCheck( WorkerDataGeneral& widowData,
      Secondary& secondaryData )
      { widowCheck(widowData.disabPeriod[0].getOnsetDate(),
          widowData.getBirthDate(), secondaryData.entDate); }
   // Arguments:
   //   jonsett: The date of disability onset.
   //   jbirtht: The date of birth.
   //   ientt: The date of entitlement.
   static void widowCheck( const boost::gregorian::date& jonsett,
      const boost::gregorian::date& jbirtht, const DateMoyr& ientt )
      { if (jonsett < jbirtht) throw PiaException(PIA_IDS_JONSET5);
        if (ientt < jonsett) throw PiaException(PIA_IDS_JONSET6); }
private:
   PiaCal& operator=( PiaCal& newPiaCal );
};
