// $Id: piadata.h 1.64 2007/03/08 14:10:55EST 044579 Development  $
//
// Declarations needed for class to manage pia calculation data.

#pragma once

#include "WorkerDataGeneral.h"
#include "age.h"
#include "qtryear.h"
#include "comppd.h"
#include "dinscode.h"
#include "inscode.h"
#include "frzyrs.h"
#include "datemoyr.h"
#include "qcamt.h"
#include "awinc.h"
#include "avgwg.h"
#include "BenefitAmount.h"
#include "BitAnnual.h"
#include "boost/date_time/gregorian/greg_date.hpp"

// {group:Calculation Classes}
// Description: Manages all of the worker-specific amounts calculated in
// preparation for and during the calculation of a Social Security benefit.
//
// Remarks: Manages all of the worker-specific amounts calculated in
// preparation for and during the calculation of a Social Security benefit.
class PiaData
{
   public:
      // Description: Special minimum-drc indicator.
      enum arf_app_type {
         // No special minimum, or no delayed retirement credits.
         NO_SPEC_MIN_DRC,
         // Highest PIA is special minimum, delayed retirement credits are
         // involved, and benefit equals special minimum PIA.
         SPEC_MIN_BEN,
         // Highest PIA is special minimum, delayed retirement credits are
         // involved, and benefit equals support PIA increased by increment
         // factor.
         SUPPORT_BEN
      };
      // Description: Types of earnings (without or with totalization).
      enum earn_total_type {
         // Without considering totalization.
         EARN_NO_TOTALIZATION,
         // After considering attributed earnings for totalization.
         EARN_WITH_TOTALIZATION,
         // Number of types of earnings (without or with totalization).
         NUM_EARN_TOTALIZATION
      };
      // Age at benefit date of worker.
      Age    ageBen;
      // Age at entitlement of worker.
      Age    ageEnt;
      // Description: Benefit actually payable, before dollar rounding.
      //
      // Remarks: See roundedBenefit for the benefit after dollar rounding.
      BenefitAmount benefit;
      // Description: Indicators for use of child care credits.
      //
      // Remarks: True (1) if using child care credit that year, false (0)
      //   otherwise.
      BitAnnual childCareOrder;
      // Description: The maximum amount of credit for each child care year,
      // and the year in which it should be credited.
      std::vector< std::pair<double, int> > childCareMaxCredit;
      // Computation period for new-start calculations.
      CompPeriod compPeriodNew;
      // Computation period for old-start calculation.
      CompPeriod compPeriodOld;
      // Disability insured status code.
      DisInsCode disInsCode;
      // Earliest possible retirement age.
      Age    earlyRetAge;
      // Military service wage credits, 1951 to date.
      DoubleAnnual earnMilServ;
      // Description: Worker HI earnings, 1937 to maxyear.
      AverageWage earnHi;
      // Description: Worker HI earnings limited to wage base, 1937 to maxyear.
      AverageWage earnHiLimited;
      // Description: Worker OASDI earnings, 1937 to maxyear, including
      //   railroad and military service.
      AverageWage earnOasdi;
      // Description: Worker OASDI earnings limited to wage base, 1937 to
      //   maxyear, including railroad and military service.
      AverageWage earnOasdiLimited;
      // Description: Worker OASDI earnings attributed for totalization, 1937
      //   to maxyear.
      AverageWage earnTotalized;
      // Description: Worker OASDI earnings attributed for totalization,
      //   limited to wage base, 1937 to maxyear.
      AverageWage earnTotalizedLimited;
      // Fully insured status code.
      InsCode finsCode;
      // Full retirement age.
      Age    fullRetAge;
      // Years wholly within a freeze period.
      FreezeYears  freezeYears;
      // Highest applicable MFB.
      BenefitAmount highMfb;
      // Highest applicable PIA.
      BenefitAmount highPia;
      // Years partially within a freeze period.
      FreezeYears  partialFreezeYears;
      // Quarters of coverage, including railroad and military service.
      QcArray qcov;
      // Military service quarters of coverage, 1937 to 1956.
      QcArray qcovMilServ;
      // Annual relative earnings position, 1937 to maxyear.
      DoubleAnnual relEarnPosition;
      // Description: Benefit actually payable, after dollar rounding.
      //
      // Remarks: See benefit for the benefit before dollar rounding.
      BenefitAmount roundedBenefit;
      // Support PIA when highest PIA is special minimum and
      //   delayed retirement credits are involved.
      BenefitAmount supportPia;
protected:
   // Description: True if entitlement after 1990 amendments.
   bool   amend90;
   // Description: Reduction or increment factor.
   double arf;
   // Description: Special minimum-drc indicator.
   arf_app_type arfApp;
   // Description: Number of child care years used in PIA computation.
   int childCareYearsTotal;
   // Description: Number of quarters of coverage required for deemed fully
   // insured status.
   int deemedQcReq;
   // Description: Total number of quarters of coverage earned for deemed fully
   // insured status.
   int deemedQctot;
   // Description: Indicator for calculation of quarters of coverage for
   //   early years.
   //
   // Remarks:
   //
   // true  - Calculate total by adding earnings qcs, railroad qcs, and
   //         military service qcs (as in Anypia).
   // false - Use annual totals as read in (as for microsimulation data).
   bool doEarlyQcs;
   // Military service wage credits, 1937-50.
   double earn3750ms;
   // Description: First year of earnings after 1950, calculated 2 different
   //   ways.
   //
   // Remarks: Index 0 is without considering totalization. Index 1 is
   //   after considering attributed earnings for totalization.
   int earn50[NUM_EARN_TOTALIZATION];
   // Description: Total earnings prior to 1951, calculated 2 different ways.
   //
   // Remarks: Index 0 is actual. Index 1 is attributed for totalization.
   double earnTotal50[NUM_EARN_TOTALIZATION];
      // Year prior to year of retirement or disability,
      // or exact year of death (last year of earnings considered).
      int    earnYear;
      // Worker's month and year of eligibility, before considering prior
      // disability (date of worker's attainment of age 62, for old-age and
      // disability, or date of worker's death, if earlier, for survivor).
      DateMoyr eligDate;
      // Fully insured status code, as used in OACT printouts.
      char   finsCode2;
      // The date of attainment of fully insured status.
      DateMoyr fullInsDate;
      // The date of attainment of full retirement age.
      DateMoyr fullRetDate;
      // Description: Applicable PIA method number (see PiaMethod::pia_type).
      int    iappn;
      // Support PIA method number (see PiaMethod::pia_type).
      int    iapps;
      // First year of earnings after considering railroad and
      //   military service.
      int    ibeginAll;
      // First year of attributed earnings in totalization case.
      int    ibeginTotal;
      // Year of eligibility after considering prior disability onset.
      int    ielgyr;
      // Last year of earnings after considering railroad and military service.
      int    iendAll;
      // Last year of attributed earnings in totalization case.
      int    iendTotal;
      // Day before birth of worker.
      boost::gregorian::date kbirth;
      // Mumber of months of early or delayed retirement.
      int    monthsArdri;
      // Description: Indicator for earnings over maximum.
      //
      // Remarks:
      //
      // false - no earnings over MAXEARN.
      // true - some earnings over MAXEARN.
      bool   overMax;
      // Code for type of pia (see Pifc::code).
      char   pifc;
      // Military service quarters of coverage, 1937-50.
      int    qc3750ms;
      // Simplified quarters of coverage, 1937-50.
      int    qc3750simp;
      // Quarters of coverage in prior 3 years.
      int    qcCurrent;
      // Beginning quarter and year of period of required
      // quarters of coverage for disability insured status.
      QtrYear qcDisDate1;
      // Ending quarter and year of period of required
      // quarters of coverage for disability insured status.
      QtrYear qcDisDate2;
      // Beginning quarter and year of period of
      // required quarters of coverage for disability insured
      // status, for period after disability cessation.
      QtrYear qcDisDate3;
      // Ending quarter and year of period of
      // required quarters of coverage for disability insured
      // status, for period after disability cessation.
      QtrYear qcDisDate4;
      // Beginning quarter and year of period of
      // required quarters of coverage for disability insured
      // status, for period before prior disability.
      QtrYear qcDisDate5;
      // Ending quarter and year of period of
      // required quarters of coverage for disability insured
      // status, for period before prior disability.
      QtrYear qcDisDate6;
      // Number of quarters over which quarters of
      // coverage must be earned for disability insured status.
      int    qcDisQtr;
      // Number of quarters over which quarters of coverage
      // must be earned for disability insured status, based on elapsed
      // quarters after disability cessation (included in qcDisQtr).
      int    qcDisQtr2;
      // Number of quarters of coverage required for
      // disability insured status.
      int    qcDisReq;
      // Number of years over which quarters of coverage
      // must be earned for disability insured status.
      int    qcDisYears;
      // Number of quarters of coverage required for fully insured status.
      int    qcReq;
      // Number of quarters of coverage required for permanently insured
      // status.
      int    qcReqPerm;
      // Total number of quarters of coverage earned.
      int    qcTotal;
      // Number of quarters of coverage earned from 1937 to 1950.
      int    qcTotal50;
      // Number of quarters of coverage earned after 1950.
      int    qcTotal51;
      // Number of quarters of coverage earned for
      // disability insured status.
      int    qcTotalDis;
      // Average relative earnings position.
      double relEarnPositionAverage;
      // Description: Starting point of annual qc's, if using sum of qc's
      // through 1950.
      static const QtrYear qtr151;
      // Description: Starting point of annual qc's, if using sum of qc's
      // through 1977.
      static const QtrYear qtr178;
      // Description: The earliest possible retirement date.
      static const DateMoyr jan1937;
public:
   PiaData();
   ~PiaData();
   void deleteContents();
   void earn50Cal( const WorkerDataGeneral& workerData );
   void earnHiCal( const WorkerDataGeneral& workerData );
   void earnProjection( const WorkerDataGeneral& workerData );
   void earnTotal50Cal0();
   void earnTotal50Cal1();
   void earnYearCal( const WorkerDataGeneral& workerData,
      WorkerDataGeneral::ben_type ioasdi);
   void freezeYearsCal( const WorkerDataGeneral& workerData,
      WorkerDataGeneral::ben_type ioasdi, const Age& fullRetAgeDI );
   // Description: Returns indicator for entitlement after 1990 amendments.
   //
   // Returns: Indicator for entitlement after 1990 amendments.
   bool getAmend90() const { return(amend90); }
   // Description: Returns actuarial reduction factor or delayed retirement
   //   credit.
   //
   // Returns: Actuarial reduction factor or delayed retirement credit.
   double getArf() const { return(arf); }
   // Description: Returns special minimum-delayed retirement credit indicator.
   //
   // Returns: Special minimum-delayed retirement credit indicator.
   arf_app_type getArfApp() const { return(arfApp); }
   // Description: Returns day before worker's birth.
   //
   // Returns: Day before worker's birth.
   const boost::gregorian::date& getBirthDateMinus1() const { return(kbirth); }
   double getChildCareMaxCredit( int yearOfCredit ) const;
   // Description: Returns total number of child care years.
   //
   // Returns: Total number of child care years.
   int getChildCareYearsTotal() const { return childCareYearsTotal; }
   // Description: Returns number of quarters required for deemed fully insured
   //   status.
   //
   // Returns: Number of quarters required for deemed fully insured status.
   int getDeemedQcReq() const { return(deemedQcReq); }
   // Description: Returns military service earnings, 1937-50.
   //
   // Returns: Military service earnings, 1937-50.
   double getEarn3750ms() const { return(earn3750ms); }
   // Description: Returns first year of earnings after 1950.
   //
   // Returns: First year of earnings after 1950.
   //
   // Arguments:
   //   type: Type of earnings.
   int getEarn50( earn_total_type type ) const { return(earn50[(int)type]); }
   // Description: Returns total earnings prior to 1951.
   //
   // Returns: Total earnings prior to 1951.
   //
   // Arguments:
   //   type: Type of earnings.
   double getEarnTotal50( earn_total_type type ) const
      { return(earnTotal50[(int)type]); }
   // Description: Returns last year of earnings considered.
   //
   // Returns: Last year of earnings considered.
   int getEarnYear() const { return(earnYear); }
   // Description: Returns worker's month and year of eligibility before
   //   considering prior disability onset.
   //
   // Returns: Worker's month and year of eligibility before
   //   considering prior disability onset.
   const DateMoyr& getEligDate() const { return(eligDate); }
   // Description: Returns worker's year of eligibility after considering
   //   prior disability onset.
   //
   // Returns: Worker's year of eligibility after considering
   //   prior disability onset.
   int getEligYear() const { return(ielgyr); }
   // Description: Returns fully insured code as used in oact printouts.
   //
   // Returns: Fully insured code as used in oact printouts.
   char getFinsCode2() const { return(finsCode2); }
   int getFirstQcYear( const WorkerDataGeneral& workerData ) const;
   // Description: Returns date fully insured status is attained.
   //
   // Returns: The date fully insured status is attained.
   const DateMoyr& getFullInsDate() const { return(fullInsDate); }
   // Description: Returns date full retirement age is attained.
   //
   // Returns: The date full retirement age is attained.
   const DateMoyr& getFullRetDate() const { return(fullRetDate); }
   // Description: Returns number of applicable pia calculation method.
   //
   // Returns: Number of applicable pia calculation method.
   int getIappn () const { return(iappn); }
   // Description: Returns support PIA method number.
   //
   // Returns: Support PIA method number.
   int getIapps () const { return(iapps); }
   // Description: Returns first year of earnings after considering railroad
   // and military service.
   //
   // Returns: First year of earnings after considering railroad and
   // military service.
   int getIbeginAll() const { return ibeginAll; }
   // Description: Returns first year of attributed earnings in totalization
   // case.
   //
   // Returns: First year of attributed earnings in totalization case.
   int getIbeginTotal() const { return(ibeginTotal); }
   // Description: Returns last year of earnings after considering railroad and
   //   military service.
   //
   // Returns: Last year of earnings after considering railroad and
   //   military service.
   int getIendAll() const { return iendAll; }
   // Description: Returns last year of attributed earnings in totalization
   // case.
   //
   // Returns: Last year of attributed earnings in totalization case.
   int getIendTotal() const { return(iendTotal); }
   int getLastQcYear( const WorkerDataGeneral& workerData ) const;
   // Description: Returns number of months of increment or reduction.
   //
   // Returns: Number of months of increment or reduction.
   int getMonthsArdri() const { return(monthsArdri); }
   // Description: Returns indicator for earnings over maximum.
   //
   // Returns: True if any earnings over maximum, false otherwise.
   bool getOverMax() const { return(overMax); }
   // Description: Returns code for type of pia.
   //
   // Returns: Code for type of pia.
   char getPifc() const { return(pifc); }
   float getPubpenApp( const WorkerDataGeneral& workerData ) const;
   // Description: Returns military service quarters of coverage, 1937-50.
   //
   // Returns: Military service quarters of coverage, 1937-50.
   int getQc3750ms() const { return(qc3750ms); }
   // Description: Returns simplified quarters of coverage, 1937-50.
   //
   // Returns: Simplified quarters of coverage, 1937-50.
   int getQc3750simp() const { return(qc3750simp); }
   // Description: Returns number of quarters earned in prior 3 years.
   //
   // Returns: Number of quarters earned in prior 3 years.
   int getQcCurrent() const { return(qcCurrent); }
   // Description: Returns number of quarters over which quarters of coverage
   // must be earned for disability insured status.
   //
   // Returns: Number of quarters over which quarters of coverage must
   // be earned for disability insured status.
   int getQcDisQtr() const { return(qcDisQtr); }
   // Description: Returns number of quarters over which quarters of coverage
   // must be earned for disability insured status, based on elapsed quarters
   // after disability cessation.
   //
   // Returns: Number of quarters over which quarters of coverage must
   // be earned for disability insured status, based on elapsed quarters
   // after disability cessation.
   int getQcDisQtr2() const { return(qcDisQtr2); }
   // Description: Returns number of quarters required for disability insured
   //   status.
   //
   // Returns: Number of quarters required for disability insured status.
   int getQcDisReq() const { return(qcDisReq); }
   // Description: Returns beginning quarter and year of period of required
   //   quarters of coverage for disability insured status.
   //
   // Returns: Beginning quarter and year of period of required
   //   quarters of coverage for disability insured status.
   const QtrYear& getQcDisDate1() const { return qcDisDate1; }
   // Description: Returns ending quarter and year of period of required
   // quarters of coverage for disability insured status.
   //
   // Returns: Ending quarter and year of period of required quarters
   //   of coverage for disability insured status.
   const QtrYear& getQcDisDate2() const { return qcDisDate2; }
   // Description: Returns beginning quarter and year of period of required
   //   quarters of coverage for disability insured status, for period after
   //   disability cessation.
   //
   // Returns: Beginning quarter and year of period of required quarters of
   //   coverage for disability insured status, for period after disability
   //   cessation.
   const QtrYear& getQcDisDate3() const { return qcDisDate3; }
   // Description: Returns ending quarter and year of period of required
   // quarters of coverage for disability insured status, for period after
   // disability cessation.
   //
   // Returns: Ending quarter and year of period of required quarters of
   // coverage for disability insured status, for period after disability
   // cessation.
   const QtrYear& getQcDisDate4() const { return qcDisDate4; }
   // Description: Returns beginning quarter and year of period of required
   // quarters of coverage for disability insured status, for period before
   // prior disability.
   //
   // Returns: Beginning quarter and year of period of required quarters of
   // coverage for disability insured status, for period before prior
   // disability.
   const QtrYear& getQcDisDate5() const { return qcDisDate5; }
   // Description: Returns ending quarter and year of period of required
   // quarters of coverage for disability insured status, for period before
   // prior disability.
   //
   // Returns: Ending quarter and year of period of required quarters of
   // coverage for disability insured status, for period before prior
   // disability.
   const QtrYear& getQcDisDate6() const { return qcDisDate6; }
   // Description: Returns number of years over which quarters of coverage must
   // be earned for disability insured status.
   //
   // Returns: Number of years over which quarters of coverage must be
   //   earned for disability insured status.
   int getQcDisYears() const { return(qcDisYears); }
   // Description: Returns number of quarters required for fully insured
   // status.
   //
   // Returns: Number of quarters required for fully insured status.
   int getQcReq() const { return(qcReq); }
   // Description: Returns number of quarters required for permanently insured
   //   status.
   //
   // Returns: Number of quarters required for permanently insured status.
   int getQcReqPerm() const { return(qcReqPerm); }
   // Description: Returns total number of quarters earned for fully insured
   //   status.
   //
   // Returns: Total number of quarters earned for fully insured status.
   int getQcTotal() const { return(qcTotal); }
   // Description: Returns total number of quarters earned from 1937 to 1950.
   //
   // Returns: Total number of quarters earned from 1937 to 1950.
   int getQcTotal50() const { return(qcTotal50); }
   // Description: Returns total number of quarters earned after 1950.
   //
   // Returns: Total number of quarters earned after 1950.
   int getQcTotal51() const { return(qcTotal51); }
   // Description: Returns total number of quarters earned for disability
   // insured status.
   //
   // Returns: Total number of quarters earned for disability insured status.
   int getQcTotalDis() const { return(qcTotalDis); }
   // Description: Returns average relative earnings position.
   //
   // Returns: Average relative earnings position.
   double getRelEarnPositionAverage() const { return(relEarnPositionAverage); }
   void initialize();
   void qcCal( WorkerDataGeneral& workerData, const Qcamt& qcamt );
   void qcDisCal( const WorkerDataGeneral& workerData );
   void qcDisReqCal( const WorkerDataGeneral& workerData,
      const DateMoyr& date, int trial, bool dospecial, bool dofreeze );
   void qcDisReq2Cal( const WorkerDataGeneral& workerData,
      const DateMoyr& date );
   void qcDiSpec( int qcdiqtrt, int period );
   void qcTotalCal( const WorkerDataGeneral& workerData,
      const QtrYear& qtrYear );
   // Description: Sets indicator for entitlement after 1990 amendments.
   //
   // Arguments:
   //   newAmend90: New indicator.
   void setAmend90( bool newAmend90 ) { amend90 = newAmend90; }
  // Description: Sets actuarial reduction factor or delayed retirement credit.
   //
   // Arguments:
   //   newArf: New actuarial reduction factor or delayed retirement credit.
   void setArf( double newArf ) { arf = newArf; }
   // Description: Sets special minimum-delayed retirement credit indicator.
   //
   // Arguments:
   //   newArfApp: New indicator.
   void setArfApp( arf_app_type newArfApp ) { arfApp = newArfApp; }
   void setBirthDateMinus1( const boost::gregorian::date& birthDate );
   // Description: Sets total number of child care years.
   //
   // Arguments:
   //   totalYears: New total number of child care years.
   void setChildCareYearsTotal( int totalYears )
      { childCareYearsTotal = totalYears; }
   // Description: Sets number of quarters of coverage required for deemed
   // insured status.
   //
   // Arguments:
   //   newDeemedQcReq: New required quarters for deemed insured status.
   void setDeemedQcReq( int newDeemedQcReq ) { deemedQcReq = newDeemedQcReq; }
   // Description: Sets indicator for calculation of quarters of coverage for
   //   early years.
   //
   // Arguments:
   //   newDoEarlyQcs: New indicator.
   //
   //     true  - Calculate total by adding earnings qcs, railroad qcs, and
   //             military service qcs (as in Anypia).
   //     false - Use annual totals as read in (as for microsimulation data).
   void setDoEarlyQcs( bool newDoEarlyQcs ) { doEarlyQcs = newDoEarlyQcs; }
   // Description: Sets military service earnings, 1937-50.
   //
   // Arguments:
   //   newEarn3750ms: New value of military service earnings, 1937-50.
   void setEarn3750ms( double newEarn3750ms ) { earn3750ms = newEarn3750ms; }
   // Description: Sets month and year of eligibility before considering
   //   disability.
   //
   // Arguments:
   //   newEligDate: New date of eligibility.
   void setEligDate( const DateMoyr& newEligDate ) { eligDate = newEligDate; }
   // Description: Sets eligibility year after considering prior disability
   // onset.
   //
   // Arguments:
   //   newEligYear: New eligibility year.
   void setEligYear( int newEligYear ) { ielgyr = newEligYear; }
   // Description: Sets fully insured status code as used in oact printouts.
   //
   // Arguments:
   //   newFinsCode2: New insured status code.
   void setFinsCode2( char newFinsCode2 ) { finsCode2 = newFinsCode2; }
   // Description: Sets date fully insured status is attained.
   //
   // Arguments:
   //   newFullInsDate: New date fully insured status is attained.
   void setFullInsDate( const DateMoyr& newFullInsDate )
      { fullInsDate = newFullInsDate; }
   void setFullRetDate();
   // Description: Sets number of applicable pia calculation method.
   //
   // Arguments:
   //   newIappn: New number of applicable pia method.
   void setIappn( int newIappn ) { iappn = newIappn; }
   // Description: Sets support pia method number.
   //
   // Arguments:
   //   newIapps: New support pia method number.
   void setIapps( int newIapps ) { iapps = newIapps; }
   void setIbeginTotal( const WorkerDataGeneral& workerData );
   void setIendTotal( const WorkerDataGeneral& workerData );
   // Description: Sets number of months of increment or reduction.
   //
   // Arguments:
   //   newMonthsArdri: New number of months.
   void setMonthsArdri( int newMonthsArdri ) { monthsArdri = newMonthsArdri; }
   // Description: Sets indicator for earnings over maximum.
   //
   // Arguments:
   //   newOverMax: New indicator.
   void setOverMax( bool newOverMax ) { overMax = newOverMax; }
   // Description: Sets code for type of pia.
   //
   // Arguments:
   //   newPifc: New pia factor code.
   void setPifc( char newPifc ) { pifc = newPifc; }
   void setQc3( WorkerDataGeneral& workerData );
   // Description: Sets military service quarters of coverage, 1937-50.
   //
   // Arguments:
   //   newQc3750ms: New value of military service qcs, 1937-50.
   void setQc3750ms( int newQc3750ms ) { qc3750ms = newQc3750ms; }
   // Description: Sets simplified quarters of coverage, 1937-50.
   //
   // Remarks: The version with an argument uses that as the number of qcs.
   //   The version with no argument sets the number of qcs by using total
   //   earnings in the period 1937-50.
   //
   // Arguments:
   //   newQc3750simp: New value of simplified qcs, 1937-50.
   void setQc3750simp( int newQc3750simp ) { qc3750simp = newQc3750simp; }
   void setQc3750simp();
   // Description: Sets quarters of coverage earned in prior 3 years.
   //
   // Arguments:
   //   newQcCurrent: New quarters of coverage earned in prior 3 years.
   void setQcCurrent( int newQcCurrent ) { qcCurrent = newQcCurrent; }
   // Description: Sets number of quarters required for fully insured status.
   //
   // Arguments:
   //   newQcReq: New required number of quarters.
   void setQcReq( int newQcReq ) { qcReq = newQcReq; }
   // Description: Sets number of quarters required for permanently insured
   //   status.
   //
   // Arguments:
   //   newQcReqPerm: New required number of quarters.
   void setQcReqPerm( int newQcReqPerm ) { qcReqPerm = newQcReqPerm; }
   // Description: Sets total quarters of coverage.
   //
   // Arguments:
   //   newQcTotal: New total quarters of coverage.
   void setQcTotal( int newQcTotal ) { qcTotal = newQcTotal; }
   // Description: Sets quarters of coverage prior to 1951.
   //
   // Arguments:
   //   newQcTotal50: New quarters of coverage prior to 1951.
   void setQcTotal50( int newQcTotal50 ) { qcTotal50 = newQcTotal50; }
   // Description: Sets average relative earnings position.
   //
   // Arguments:
   //   newRelEarnPosition: New average relative earnings position.
   void setRelEarnPositionAverage( double newRelEarnPosition )
      { relEarnPositionAverage = newRelEarnPosition; }
   void updateMilServ( const WorkerDataGeneral& workerData );
};
