// $Id: piaparms.h 1.85 2007/02/28 15:03:46EST 044579 Development  $
//
// Declarations needed for pia calculation parameters class.

#pragma once

#include <vector>
#include "datemoyr.h"
#include "dbleann.h"
#include "intann.h"
#include "age.h"
#include "qcamt.h"
#include "avgwg.h"
#include "bendmfb.h"
#include "bpmfbout.h"
#include "bppiaout.h"
#include "percpia.h"
#include "PercPiaOut.h"
#include "BitAnnual.h"
#include "qc.h"
#include "WageBase.h"
#include "Sex.h"
#include "cachup.h"
#include "awinc.h"
#include "qtryear.h"
#include "boost/date_time/gregorian/greg_date.hpp"
class AwbiData;

// {group:Calculation Classes}
// Description: Abstract parent of classes to manage the Social Security
//   parameters required to calculate a Social Security benefit.
//
// Remarks: You should instantiate a sub-class, or a sub-sub-class. PiaParamsLC
//   is used in the microsimulation work; PiaParamsAny is used in the Anypia
//   program.
class PiaParams
{
public:
   // Description: The year range for the special minimum percentage.
   enum SPECMINPERCTYPE {
      PRE1991 = 0,         // Special minimum before 1991.
      POST1990,            // Special minimum after 1990.
      NUMSPECMINPERCTYPES  // Number of year ranges for the spec min percentage.
   };
   // Description: The date of 1950 amendments.
   static const DateMoyr amend50;
   // Description: The date of 1952 amendments.
   static const DateMoyr amend52;
   // Description: The date of 1954 amendments.
   static const DateMoyr amend54;
   // Description: The date of 1956 amendments.
   static const DateMoyr amend56;
   // Description: First birth date for which age 62 is effective for females.
   static const boost::gregorian::date amend561;
   // Description: Second birth date for which age 62 is effective for females.
   static const boost::gregorian::date amend562;
   // Description: The date of 1958 amendments.
   static const DateMoyr amend58;
   // Description: The date of 1961 amendments.
   static const DateMoyr amend61;
   // Description: First birth date for which age 62 is effective for males.
   static const boost::gregorian::date amend611;
   // Description: Second birth date for which age 62 is effective for males.
   static const boost::gregorian::date amend612;
   // Description: The date of 1965 amendments for benefit increase.
   static const DateMoyr amend651;
   // Description: The date of 1965 amendments for change to age 60.
   static const DateMoyr amend652;
   // Description: The date of 1965 amendments for old start.
   static const DateMoyr amend653;
   // Description: The date of 1967 amendments for old start.
   static const DateMoyr amend671;
   // Description: The date of 1967 amendments for benefit increase.
   static const DateMoyr amend672;
   // Description: The date of 1969 amendments.
   static const DateMoyr amend69;
      // Description: The date of 1970 amendments.
      static const DateMoyr amend70;
      // Description: The date of 1972 amendments for benefit increase.
      static const DateMoyr amend721;
      // Description: The date of 1972 amendments for special minimum.
      static const DateMoyr amend722;
      // Description: The date of 1974 amendments for partial increase.
      static const DateMoyr amend741;
      // Description: The date of 1974 amendments for full increase.
      static const DateMoyr amend742;
      // Description: The date of 1977 amendments for frozen pia table.
      static const DateMoyr amend771;
      // Description: The date of 1977 amendments for start of wage indexing.
      static const DateMoyr amend772;
      // Description: The date of 1980 amendments.
      static const DateMoyr amend80;
   // Description: The date of birth for age 62 and 1 month in 1981 amendments.
   static const boost::gregorian::date amend811;
   // Description: The date of 1981 amendments (month, day, and year).
   static const boost::gregorian::date amend812;
   // Description: The date of 1981 amendments (month and year).
   static const DateMoyr amend813;
      // Description: The date of 1982 amendments.
      static const DateMoyr amend82;
      // Description: The date of 1983 amendments.
      static const DateMoyr amend83;
      // Description: The date of 1988 amendments.
      static const DateMoyr amend88;
      // Description: The date of 1990 amendments.
      static const DateMoyr amend90;
      // Description: The date of special 0.1 percent benefit increase.
      static const DateMoyr amend01;
      // Description: Amount per old start increment year.
      static const double amtperincyr;
      // Description: Age 16.
      static const Age age16;
      // Description: Age 18.
      static const Age age18;
      // Description: Age 19.
      static const Age age19;
      // Description: Age 22.
      static const Age age22;
      // Description: Age 50.
      static const Age age50;
      // Description: Age 60.
      static const Age age60;
      // Description: Age 62.
      static const Age age62;
      // Description: Age 62 and 1 month.
      static const Age age621;
      // Description: Age 65.
      static const Age age65;
      // Description: Age 65 and 2 months.
      static const Age age65_2;
      // Description: Age 65 and 4 months.
      static const Age age65_4;
      // Description: Age 65 and 6 months.
      static const Age age65_6;
      // Description: Age 65 and 8 months.
      static const Age age65_8;
      // Description: Age 65 and 10 months.
      static const Age age65_10;
      // Description: Age 66.
      static const Age age66;
      // Description: Age 66 and 2 months.
      static const Age age66_2;
      // Description: Age 66 and 4 months.
      static const Age age66_4;
      // Description: Age 66 and 6 months.
      static const Age age66_6;
      // Description: Age 66 and 8 months.
      static const Age age66_8;
      // Description: Age 66 and 10 months.
      static const Age age66_10;
      // Description: Age 67.
      static const Age age67;
      // Description: Age 70.
      static const Age age70;
      // Description: Age 72.
      static const Age age72;
      // Description: Age 75.
      static const Age age75;
      // Description: Age 199 (impossible age).
      static const Age age199;
      // Description: Temporary benefit increase, March-May 1974.
      static const double beninc74;
      // Description: Factor for calculating widow(er) benefits.
      static const double factor825;
      // Description: Factor for calculating mfbs.
      static const double factor150;
      // Description: Factor for calculating mfbs.
      static const double factor175;
      // Description: First quarter for which a quarter of coverage could be
      // earned.
      static const QtrYear year37qtr1;
      // Description: Starting point of deemed insured.
      static const QtrYear qtr184;
      // Description: Amount required for quarter of coverage, 1937 to maxyear.
      Qcamt   qcamt;
      // Description: Table of mfb formula bend points.
      BpMfbOut bpMfbOut;
      // Description: Table of pia formula bend points.
      BpPiaOut bpPiaOut;
      // Description: Table of pia formula percentages.
      PercPiaOut percPiaOut;
   protected:
      // HI wage bases.
      WageBaseHI baseHi;
      // Month of benefit increase, 1937 to maxyear.
      IntAnnual monthBeninc;
      // Indicator for no change allowed, 1937 to maxyear.
      IntAnnual noChange;
      // Percentage of old-law wage base required for year of coverage,
      // year 1937 to maxyear.
      DoubleAnnual percSpecMin;
      // Bend points for old-start PIB formula.
      static const int bendOS[3];
      // Old-start PIB formula percentages.
      static const double percOS[2];
      // Description: Percentage of old-law wage base required for year of
      //   coverage for special minimum.
      //
      // Remarks:
      //
      // i=0 - before 1991.
      // i=1 - after 1990.
      static const double specMinPerc[NUMSPECMINPERCTYPES];
      // Amount required for year of coverage for special minimum.
      DoubleAnnual yocAmountSpecMin;
      // Amount required for year of coverage for windfall elimination
      //   provision.
      DoubleAnnual yocAmountWindfall;
      // Benefit increases, 1937 to maxyear.
      DoubleAnnual cpiinc;
      // Catch-up benefit increases.
      Catchup catchup;
      // Annual average earnings, 1937 to maxyear.
      AverageWage fq;
      // Increase in annual average earnings, 1937 to maxyear.
      Awinc fqinc;
      // Current year (year after last known benefit increase).
      int     istart;
      // Maximum year allowed.
      int     maxyear;
      // Special minimum pias.
      std::vector< DoubleAnnual > specMinPia;
      // Special minimum pias for August-November 2001.
      std::vector< double > specMinPia2001;
      // Special minimum mfbs.
      std::vector< DoubleAnnual > specMinMfb;
      // Special minimum mfbs for August-November 2001.
      std::vector< double > specMinMfb2001;
      // Indicator for which year(s) the PIA should be recalculated
      //   due to a newly effective law change.
      BitAnnual recalcInd;
      // Title of average wage assumptions.
      std::string titleAw;
      // Title of benefit increase assumptions.
      std::string titleBi;
public:
   PiaParams( int newIstart, int newMaxyear );
   explicit PiaParams( int newMaxyear );
   virtual ~PiaParams();
   virtual double applyCola( double pia, int year ) const;
   virtual double applyCola( double pia, int year, int eligYear ) const;
   double applyCola99( double pia ) const;
   double unApplyCola( double pia, int year ) const;
   double unApplyCola( double pia, int year, int eligyear ) const;
   double unApplyCola99( double pia ) const;
   virtual double applyColaMfb( double mfb, int year, double pia ) const;
   virtual double applyColaMfb( double mfb, int year, int eligYear,
      double pia ) const;
   double applyColaMfb99( double mfb, double pia ) const;
   double benincCatchUp( double bcatch, int year, int eligYear ) const;
   // Description: Returns true if should recalculate PIAs this year
   //   because of a newly effective law change, else false.
   //
   // Returns: True if should recalculate PIAs this year
   //   because of a newly effective law change, else false.
   //
   // Arguments:
   //   year: Year to check if a recalc is needed.
   bool doRecalc( int year ) const { return recalcInd.getBit(year); }
   double deconvertPia( int eligYear, int number, double pia77,
      const DateMoyr& benDate) const;
   // Returns earliest possible retirement age for oab or aged spouse.
   //
   // Returns: Earliest possible retirement age for oab or aged spouse.
   virtual Age earlyAgeOabCal( Sex::sex_type, const boost::gregorian::date& ) const = 0;
   // Description: Returns factor for widow(er) benefit before age reduction.
   //
   // Returns: Factor for widow(er) benefit before age reduction.
   virtual double factorAgedWidCal( int, const Age&, int,
      const DateMoyr& ) const = 0;
   // Returns factor for wife/husband benefit before age reduction.
   //
   // Returns: Factor for wife/husband benefit before age reduction.
   virtual double factorAgedSpouseCal( int, int ) const = 0;
   // Calculates reduction factor for wife/husband.
   //
   // Returns: Reduction factor for wife/husband.
   virtual double factorArAgedSpouseCal( int ) const = 0;
   // Calculates reduction factor for OAB.
   //
   // Returns: Reduction factor for OAB.
   virtual double factorArCal( int ) const = 0;
   virtual double factorArWidCal( int monthsArdri, const Age& age,
      const DateMoyr& benefitDate, const Age& fullRetAge ) const;
   // Description: Returns factor for disabled widow(er) benefit before age
   // reduction.
   //
   // Returns: Factor for disabled widow(er) benefit before age reduction.
   virtual double factorDisWidCal( int, const DateMoyr& ) const = 0;
   virtual double factorYngLifeCal() const;
   virtual double factorYngSurvCal() const;
   // Returns full retirement age.
   //
   // Returns: Full retirement age.
   virtual Age fullRetAgeCal( int ) const = 0;
   // Returns full retirement age for benefit calculations
   //   involving disability freeze.
   //
   // Returns: Full retirement age for benefit calculations
   //   involving disability freeze.
   virtual Age fullRetAgeCalDI( int, int ) const = 0;
   // Description: Returns title of average wage assumptions.
   //
   // Returns: Title of average wage assumptions.
   const std::string& getAverageWage() const { return titleAw; }
   // Description: Returns the array of OASDI wage bases.
   //
   // Returns: The array of OASDI wage bases.
   virtual const WageBase& getBaseOasdiArray() const = 0;
   // Description: Returns one OASDI wage base.
   //
   // Returns: One OASDI wage base.
   virtual double getBaseOasdi( int ) const = 0;
   // Description: Returns the array of old-law wage bases.
   //
   // Returns: The array of old-law wage bases.
   virtual const WageBaseOldLaw& getBase77Array() const = 0;
   // Description: Returns one old-law wage base.
   //
   // Returns: One old-law wage base.
   virtual double getBase77( int ) const = 0;
   // Description: Returns the array of HI wage bases.
   //
   // Returns: The array of HI wage bases.
   const WageBaseHI& getBaseHiArray() const { return baseHi; }
   // Description: Returns one HI wage base.
   //
   // Returns: One HI wage base.
   //
   // Arguments:
   //   year: Year for which base is desired.
   double getBaseHi( int year ) const { return baseHi[year]; }
   // Description: Returns title of benefit increase assumptions.
   //
   // Returns: Title of benefit increase assumptions.
   const std::string& getBenefitInc() const { return titleBi; }
   // Description: Returns catch-up benefit increases.
   //
   // Returns: Catch-up benefit increases.
   const Catchup& getCatchup() const { return catchup; }
   // Returns maximum amount of earnings in a childcare dropout year.
   //
   // Returns: Maximum amount of earnings in a childcare dropout year.
   virtual double getChildcareDropoutAmount( int, int ) const = 0;
   // Returns the amount of the cap on the COLA for the given year.
   //
   // Returns: The amount of the cap on the COLA for the given year.
   virtual double getColaCap( int ) const = 0;
   // Description: Returns one benefit increase.
   //
   // Returns: One benefit increase.
   //
   // Arguments:
   //   year: Year for which benefit increase is desired.
   virtual double getCpiinc( int year ) const { return(cpiinc[year]); }
   // Description: Returns the array of benefit increases.
   //
   // Returns: The array of benefit increases.
   virtual const DoubleAnnual& getCpiincArray() const { return(cpiinc); }
   // Description: Returns array of average wages for indexing.
   //
   // Returns: Array of average wages for indexing.
   const AverageWage& getFqArray() const { return(fq); }
   // Description: Returns one average wage for indexing.
   //
   // Returns: One average wage for indexing.
   //
   // Arguments:
   //   year: Year for which base is desired.
   double getFq( int year ) const { return(fq[year]); }
   // Description: Returns series used for indexing pia formula bend points.
   //
   // Returns: Series used for indexing pia formula bend points.
   virtual const AverageWage& getFqBppia() const = 0;
   // Description: Returns one increase in average wage for indexing.
   //
   // Returns: Increase in average wage for indexing.
   //
   // Arguments:
   //   year: Year for which average wage increase is desired.
   double getFqinc( int year ) const { return(fqinc[year]); }
   // Description: Returns array of increases in average wage for indexing.
   //
   // Returns: Array of increases in average wage for indexing.
   const Awinc& getFqincArray() const { return(fqinc); }
   // Description: Returns series used for indexing an earnings record.
   //
   // Returns: Series used for indexing an earnings record.
   virtual const AverageWage& getFqIndex() const = 0;
   // Description: Returns current year.
   //
   // Returns: Current year.
   int getIstart() const { return(istart); }
   // Description: Returns the number of years of marraige needed to be
   //   eligible for a divroced spouse benefit.
   //
   // Returns: Number of years of marriage needed to be eligible
   //   for a divorced spouse benefit.
   virtual int getMarrLengthForDivBen( const DateMoyr& ) const = 0;
   // Description: Returns maximum age of child for childcare dropout year.
   //
   // Returns: Maximum age of child for childcare dropout year.
   virtual int getMaxChildcareAge() const = 0;
   // Description: Returns maximum number of childcare dropout years.
   //
   // Returns: Maximum number of childcare dropout years.
   virtual int getMaxChildcareDropoutYears( int, int ) const = 0;
   // Description: Returns maximum possible year.
   //
   // Returns: Maximum possible year.
   int getMaxyear() const { return(maxyear); }
   // Returns the amount of the cap on the COLA for the MFB
   // for the given year.
   //
   // Returns: The amount of the cap on the COLA for the MFB.
   virtual double getMfbColaCap( int ) const = 0;
   // Description: Returns array of months of benefit increase.
   //
   // Returns: Array of months of benefit increase.
   const IntAnnual& getMonthBenincArray() const { return(monthBeninc); }
   // Description: Returns one month of benefit increase.
   //
   // Returns: Month of benefit increase for specified year.
   //
   // Arguments:
   //   year:  Year of desired month of benefit increase.
   int getMonthBeninc( int year ) const { return(monthBeninc[year]); }
   // Description: Returns array of percents of old-law base required for year
   // of coverage.
   //
   // Returns: Array of percents of old-law base required for year of coverage.
   const DoubleAnnual& getPercSpecMinArray() const { return(percSpecMin); }
   // Description: Returns percent of old-law base required for year of
   // coverage.
   //
   // Returns: Percent of old-law base required for year of coverage.
   //
   // Arguments:
   //   year: The year for which the percent is required.
   double getPercSpecMin( int year ) const { return(percSpecMin[year]); }
   virtual double getSpecMinPia( const DateMoyr& dateMoyr, int yoc ) const;
   virtual double getSpecMinMfb( const DateMoyr& dateMoyr, int yoc ) const;
   // Description: Returns array of amounts required for year of coverage for
   // special minimum.
   //
   // Returns: Array of amounts required for year of coverage for special
   // minimum.
   const DoubleAnnual& getYocAmountSpecMinArray() const
      { return(yocAmountSpecMin); }
   // Description: Returns amount required for year of coverage for
   // special minimum.
   //
   // Returns: Amount required for year of coverage for special minimum.
   //
   // Arguments:
   //   year: Year for which amount is required.
   double getYocAmountSpecMin( int year ) const
      { return(yocAmountSpecMin[year]); }
   // Description: Returns array of amounts required for year of coverage for
   // windfall elimination.
   //
   // Returns: Array of amounts required for year of coverage for windfall
   // elimination.
   const DoubleAnnual& getYocAmountWindfallArray() const
      { return(yocAmountWindfall); }
   virtual void initdata();
   long lastAmw( int year ) const;
   // Description: Returns maximum DI beneficiary age at end of year.
   //
   // Returns: Maximum DI beneficiary age at end of year.
   virtual Age maxDibAge( int ) const = 0;
   // Description: Resets benefit formula percents.
   //
   // Arguments:
   //   eligYear: Year of eligibility.
   //   percPia: Benefit formula percents.
   void percPiaCal( int eligYear, PercPia& percPia ) const
   { percPia[0] = percPiaOut.getPercPia1(eligYear);
     percPia[1] = percPiaOut.getPercPia2(eligYear);
     percPia[2] = percPiaOut.getPercPia3(eligYear); }
   // Description: Projects amounts dependent on benefit increases.
   virtual void projectCpiinc() = 0;
   virtual void projectFq();
   virtual void projectPerc();
   virtual void projectSpecMin( int baseyear, int lastyear );
   // Description: Returns number of quarters of coverage for one year,
   //   applying annual test.
   //
   // Returns: Number of quarters of coverage for one year.
   //
   // Arguments:
   //   earnings: Annual earnings.
   //   year: Year of earnings.
   int qcCal( double earnings, int year ) const
      { return Qc::qcCal(earnings, qcamt[year]); }
   virtual void resizeSpecMin();
   // Description: Returns widow(er) benefit reduced for RIB-LIM.
   //
   // Returns: Reduced widow(er) benefit.
   virtual double ribLimCal( double, double, double,
         const DateMoyr& ) const = 0;
   // Sets the amount of the cap on the COLA for the given year.
   virtual void setColaCap( double, int ) = 0;
   virtual void setData( AwbiData& awbiData );
   void setHistFqinc();
   void setIstart( int newIstart );
   // Description: Sets the amount of the cap on the COLA for the MFB
   // for the given year.
   virtual void setMfbColaCap( double, int ) = 0;
   void setNoChange( int firstYear );
   // Description: Returns maximum number of usable years for special minimum.
   //
   // Returns: Maximum number of usable years for special minimum.
   virtual int specMinMaxYears() const = 0;
   virtual int specMinMaxYears( int, int ) const = 0;
   // Description: Sets amount per year of coverage in special minimum.
   //
   // Returns: Amount per year of coverage in special minimum.
   virtual double specMinAmountCal( const DateMoyr& ) const = 0;
   double unbiCatchUp( int year, double bcatch, int eligYear ) const;
   // Description: Updates wage bases.
   //
   // Remarks: The version with no arguments updates bases for average
   //   wage changes. The version with arguments updates wage bases for
   //   user entries and/or for average wage changes.
   virtual void updateBases() = 0;
   virtual void updateBases( const DoubleAnnual&, const DoubleAnnual&,
      int, int, int ) = 0;
   void updateCpiinc( const DoubleAnnual& userBiproj,
      const std::string& userTitle, int firstYeart, int lastYeart );
   void updateCpiinc( const DoubleAnnual& userBiproj,
      const Catchup& userCatchup, const std::string& userTitle,
      int firstYeart, int lastYeart );
   void updateFq( const DoubleAnnual& userfq, int firstYeart,
      int lastYeart );
   void updateFqinc( const DoubleAnnual& userfqinc,
      const std::string& userTitle, int firstYeart, int lastYeart );
   void updateYocAmountSpecMin();
   static Age childAgeForMFCal( const DateMoyr& benefitDate,
      bool isDisabled );
   static int deemedQcReqCal( int year );
   static Age earlyAgeDisWidCal( const DateMoyr& benefitDate );
   static Age earlyAgeOabCalPL( Sex::sex_type sex,
      const boost::gregorian::date& kbirth );
   static Age earlyAgeWidCal( const DateMoyr& benefitDate );
   // Description: Returns factor for wife/husband benefit before age
   // reduction.
   //
   // Returns: 0.5 for present law.
   static double factorAgedSpouseCalPL() { return(0.5); }
   static double factorAgedWidCalPL( int monthsArdri, const Age& age,
      const DateMoyr& benefitDate );
   static double factorArAgedSpouseCalPL( int monthsArdri );
   static double factorArCalPL( int monthsArdri );
   static double factorArDisWidCal( int monthsArdri,
      const DateMoyr& entDate );
   static double factorDisWidCalPL( const DateMoyr& benefitDate );
   static double factorDriCal( int monthsArdri, int eligYear );
   static Age fullRetAgeCalDIPL( int eligYear, int currentYear );
   static Age fullRetAgeCalPL( int eligYear );
   // Description: Returns a bend point for old-start PIB formula.
   //
   // Returns: A bend point for old-start PIB formula.
   //
   // Arguments:
   //   number: Number of bend point (0-2).
   static int getBendOS( int number ) { return(bendOS[number]); }
   // Description: Returns maximum amount of earnings in a childcare dropout
   // year.
   //
   // Returns: 0 for present law.
   static double getChildcareDropoutAmountPL() { return 0.0; }
   // Description: Returns the number of years of marraige needed to be
   //   eligible for a divorced spouse benefit.
   //
   // Arguments:
   //   dateMoyr: The date (year and month) when checking if eligible for
   //     divorced spouse benefit.
   //
   // Returns: Number of years of marriage needed to be eligible
   //   for a divorced spouse benefit.
   static int getMarrLengthForDivBenPL( const DateMoyr& dateMoyr )
      { return (dateMoyr < amend772) ? 20 : 10; }
   // Description: Returns maximum age of child for childcare dropout year.
   //
   // Returns: 3 for present law.
   static int getMaxChildcareAgePL() { return 3; }
   // Description: Returns maximum number of childcare dropout years.
   //
   // Returns: 3 for present law.
   static int getMaxChildcareDropoutYearsPL() { return 3; }
   // Description: Returns old-start PIB formula percentage.
   //
   // Returns: Old-start PIB formula percentage.
   //
   // Arguments:
   //   number: Number of percentage (0-1).
   static double getPercOS( int number ) { return(percOS[number]); }
   // Description: Returns percentage of old-law base required for year of
   //   coverage.
   //
   // Returns: Percentage of old-law base required for year of coverage.
   //
   // Arguments:
   //   number: Type of year of coverage.
   static double getSpecMinPerc( SPECMINPERCTYPE number )
      { return(specMinPerc[number]); }
   static double lumpSum();
   static Age maxChildAge( const DateMoyr& benefitDate, bool isDisabled,
      bool isStudent );
   static Age maxDibAgePL( int year );
   static int monthsArCal( const Age& age, const Age& fullRetAge );
   static int monthsArDICal( const DateMoyr& oabEntDate,
      const DateMoyr& oabCessDate );
   static int monthsArDisWidCal( const Age& age,
      const DateMoyr& benefitDate, const Age& fullRetAge );
   static int monthsArWidCal( const Age& age, const DateMoyr& benefitDate,
      const Age& fullRetAge );
   static int monthsArAgedSpouseCal( const Age& age,
      const DateMoyr& benefitDate, const Age& fullRetAge );
   static int monthsDriCal( const DateMoyr& fullRetDate, int eligYear,
      const boost::gregorian::date& dobadj, const DateMoyr& entDate,
      const DateMoyr& benefitDate, const DateMoyr& fullInsDate );
   static double retCredit( int eligYear );
   static double ribLimCalPL( double widowBen, double oabPia, double oabBen,
      const DateMoyr& benefitDate );
   static double specMinAmountCalPL( const DateMoyr& date );
   // Description: Returns maximum number of usable years for special minimum.
   //
   // Returns: 20 for present law.
   static int specMinMaxYearsPL() { return(20); }
};
