// $Id: qtryear.h 1.20 2007/02/28 15:03:45EST 044579 Development  $
//
// Declarations for class to manage year and quarter.

#pragma once

#include <string>
#include "boost/serialization/access.hpp"
#include "boost/date_time/gregorian/greg_date.hpp"
class DateMoyr;

// {group:Miscellaneous Classes}
// Description: Manages a year and quarter stored as unsigned integers.
//
// Remarks: The quarter and year are stored as bitfields to save space.
class QtrYear
{
   friend class boost::serialization::access;
   // Description: Archives the data.
   //
   // Remarks: When the class Archive corresponds to an output archive, the
   // & operator is defined similar to <<. Likewise, when the class Archive is
   // a type of input archive, the & operator is defined similar to >>.
   //
   // Arguments:
   //   ar: The archive to use.
   //   version: The version of the archive to use.
   template< class Archive >
   void serialize( Archive& ar, const unsigned int version )
      { ar & quarterOfYear & yearPart; }
protected:
   // Description: Quarter of year (0-3).
   unsigned short quarterOfYear;
   // Description: The year (4 digits).
   unsigned short yearPart;
public:
   // Description: Creates quarter and year.
   //
   // Remarks: The default constructor initializes the quarter and year to 0.
   QtrYear() : quarterOfYear(0u), yearPart(0u) { }
   // Arguments:
   //   newQtrYear: Quarter and year in an array.
   QtrYear( unsigned newQtrYear[2] ) :
      quarterOfYear(static_cast<unsigned short>(newQtrYear[0])),
      yearPart(static_cast<unsigned short>(newQtrYear[1])) { }
   // Arguments:
   //   newQuarter: Quarter of year (0-3) to use.
   //   newYear: Year to use.
   QtrYear( unsigned newQuarter, unsigned newYear ) :
      quarterOfYear(static_cast<unsigned short>(newQuarter)),
      yearPart(static_cast<unsigned short>(newYear)) { }
   QtrYear( const DateMoyr& dateMoyr );
   QtrYear( const boost::gregorian::date& dateModyyr );
   // Description: Destructor.
   //
   // Remarks: Does nothing.
   ~QtrYear() { }
   QtrYear& operator=( const QtrYear& qtrYear );
   QtrYear& operator=( const DateMoyr& date );
   QtrYear& operator=( const boost::gregorian::date& date );
   void add( unsigned addQuarter );
   void add( const QtrYear& addQtrYear );
   bool operator<( const QtrYear& qtrYear ) const;
   // Description: Compares this date to another one for equality.
   //
   // Returns: True if this date is equal to argument, false otherwise.
   //
   // Arguments:
   //   qtrYear: Date to compare to.
   bool operator==( const QtrYear& qtrYear ) const
      { return(yearPart == qtrYear.yearPart &&
        quarterOfYear == qtrYear.quarterOfYear); }
   // Description: Checks for valid quarter and year.
   //
   // Throws: PiaException if date is not ok.
   void check() const { qtrCheck(); yearCheck(); }
   // Description: Zeroes quarter and year.
   void deleteContents() { quarterOfYear = yearPart = 0u; }
   static int diff( const QtrYear& qtrYear1, const QtrYear& qtrYear2 );
   // Description: Returns quarter (0-3).
   //
   // Returns: Quarter of year (0-3).
   unsigned getQuarter() const { return(quarterOfYear); }
   // Description: Returns year.
   //
   // Returns: The year.
   unsigned getYear() const { return(yearPart); }
   void qtrCheck () const;
   // Description: Sets quarter.
   //
   // Arguments:
   //   newQuarter: New quarter of year (0-3).
   void setQuarter( unsigned newQuarter )
      { quarterOfYear = static_cast<unsigned short>(newQuarter); }
   // Description: Sets year.
   //
   // Arguments:
   //   newYear: New year.
   void setYear( unsigned newYear )
      { yearPart = static_cast<unsigned short>(newYear); }
   void subtract( unsigned subQuarter );
   void subtract( const QtrYear& subQtrYear );
   DateMoyr toDateMoyr() const;
   std::string toString() const;
   void yearCheck() const;
};
