// $Id: rtgenrl.h 1.16 2006/11/08 10:59:51EST 044579 Development  $
//
// Declarations needed for class to manage retirement test projections.

#pragma once

#include "dbleann.h"
#include "age.h"
#include "oactcnst.h"

// {group:Calculation Classes}
// Description: Parent of classes to manage the
//   retirement test exempt amount projections.
//
// Remarks: Parent of classes to manage the
//   retirement test exempt amount projections.
class RtGeneral : public DoubleAnnual
{
   public:
      RtGeneral( int newLastYear );
      RtGeneral( int newBaseYear, int newLastYear );
      virtual ~RtGeneral();
      void convertMonthToAnnual( const RtGeneral& rteam );
      static Age getMaxAgePL( int year );
      static double project1( double baseAmount, double numerator,
         double denominator, double previous );
};

// {group:Calculation Classes}
// Description: Manages the annual retirement test exempt amount
//   projections under full retirement age.
//
// Remarks: Manages the annual retirement test exempt amount
//   projections under full retirement age.
class Rtea62 : public RtGeneral
{
   public:
      Rtea62( int newLastYear );
      Rtea62( int newBaseYear, int newLastYear );
};

// {group:Calculation Classes}
// Description: Manages the monthly retirement test exempt amount
//   projections under full retirement age.
//
// Remarks: Manages the monthly retirement test exempt amount
//   projections under full retirement age.
class Rtea62m : public RtGeneral
{
protected:
   // Description: Historical retirement test exempt amounts.
   static const double theAmts[];
public:
   Rtea62m( int newLastYear );
   Rtea62m( int newBaseYear, int newLastYear );
   // Description: Returns historical monthly exempt amount for specified year.
   //
   // Returns: Exempt amount for specified year.
   //
   // Arguments:
   //   year:  Year for which exempt amount is desired.
   static double getHistAmount( int year ) { return theAmts[year - YEAR51]; }
   static int getFraction( int year );
   void initialize();
   void project( int firstYear, const DoubleAnnual& averageWages,
      const DoubleAnnual& cpi );
};

// {group:Calculation Classes}
// Description: Manages the annual retirement test exempt amount
//   projections over full retirement age.
//
// Remarks: Manages the annual retirement test exempt amount
//   projections over full retirement age.
class Rtea65 : public RtGeneral
{
   public:
      Rtea65( int newLastYear );
      Rtea65( int newBaseYear, int newLastYear );
};

// {group:Calculation Classes}
// Description: Manages the monthly retirement test exempt amount
//   projections over full retirement age.
//
// Remarks: Manages the monthly retirement test exempt amount
//   projections over full retirement age.
class Rtea65m : public RtGeneral
{
protected:
   // Description: Historical retirement test exempt amounts.
   static const double theAmts[];
public:
   Rtea65m( int newLastYear );
   Rtea65m( int newBaseYear, int newLastYear );
   // Description: Returns historical monthly exempt amount for specified year.
   //
   // Returns: Exempt amount for specified year.
   //
   // Arguments:
   //   year:  Year for which exempt amount is desired.
   static double getHistAmount( int year ) { return theAmts[year - YEAR51]; }
   static int getFraction( int year );
   void initialize();
   void project( int firstYear, const DoubleAnnual& averageWages,
      const DoubleAnnual& cpi );
};

// {group:Calculation Classes}
// Description: Manages the upper annual retirement test exempt
//   amounts when there were two tiers.
//
// Remarks: There were two tiers from 1961 to 1972.
class RteaHigh : public RtGeneral
{
protected:
   // Description: First year of two-tier retirement test.
   static const int YEAR61 = 1961;
   // Description: Last year of two-tier retirement test.
   static const int YEAR72 = 1972;
   // Description: Exempt amounts, 1961-1972.
   static const double rteaHigh[YEAR72 - YEAR61 + 1];
public:
   RteaHigh();
   static int getFraction();
   void initialize();
};
