// $Id: wrkrdata.h 1.60 2007/03/01 15:36:20EST 044579 Development  $
//
// Declarations needed for class to manage worker data.

#pragma once

#include "WorkerDataGeneral.h"
#include "ssn.h"
#include "MilServDatesVec.h"
#include "railroad.h"
#include "wbgenrl.h"
#include "avgwg.h"

// {group:Calculation Classes}
// Description: Manages the basic data required to calculate
//   a Social Security benefit.
//
// Remarks: It includes extra fields for more detailed calculations and
//   fuller output, compared to WorkerDataGeneral.  In particular, it allows
//   for periods of military service (which could generate wage credits),
//   railroad earnings, and a noncovered pension (which could trigger the
//   Windfall Elimination Provision).
class WorkerData : public WorkerDataGeneral
{
   friend class boost::serialization::access;
   // Description: Archives the data.
   //
   // Remarks: When the class Archive corresponds to an output archive, the
   // & operator is defined similar to <<. Likewise, when the class Archive is
   // a type of input archive, the & operator is defined similar to >>.
   //
   // Arguments:
   //   ar: The archive to use.
   //   version: The version of the archive to use.
   template< class Archive >
   void serialize( Archive& ar, const unsigned int version )
      { ar & boost::serialization::base_object<WorkerDataGeneral>(*this);
        ar & ssn & milServDatesVec & railRoadData & earnOasdi;
        ar & earnHi & taxType & nhname & nhaddr & pubpen;
        ar & pubpenDate & pubpenReservist & oabEntDate & oabCessDate; }
public:
   // Description: Dimension limits.
   enum DimLimits {
      // Number of address lines.
      NUM_ADDRESS_LINES = 3
   };
   // Description: Social security number of wage earner.
   Ssn ssn;
   // Description: Military service dates.
   MilServDatesVec milServDatesVec;
   // Description: Railroad quarters of coverage and earnings.
   RailRoadData railRoadData;
protected:
   // Description: Worker earnings, 1937 to maxyear, after any projection.
   AverageWage earnOasdi;
   // Description: Worker additional Medicare earnings, 1937 to maxyear.
   AverageWage earnHi;
   // Description: Type of taxes paid (see TaxRates::taxrateType).
   BitAnnual taxType;
   // Description: Name of wage earner.
   std::string nhname;
   // Description: Address of wage earner.
   std::string nhaddr[NUM_ADDRESS_LINES];
   // Description: Monthly noncovered pension.
   float pubpen;
   // Description: Month and year of noncovered pension entitlement.
   DateMoyr pubpenDate;
   // Description: Monthly noncovered pension, excluding military
   //   reservist pension, for benefits after December 1994.
   //
   // Remarks: Used only if pubpen is positive and getReservist is true.
   float pubpenReservist;
   // Description: Month and year of OAB entitlement prior to most recent
   //   DIB entitlement.
   DateMoyr oabEntDate;
   // Description: Month and year of OAB cessation prior to most recent DIB.
   DateMoyr oabCessDate;
public:
   WorkerData();
   ~WorkerData();
   // Description: Returns true if any self-employed taxes.
   //
   // Returns: True if any self-employed taxes.
   bool anySelfEmp() const { return taxType.any(); }
   void deleteContents();
   void clearPubpenDate();
   // Description: Returns Medicare earnings for specified year.
   //
   // Returns: Medicare earnings for specified year.
   //
   // Arguments:
   //   year: Year of earnings.
   double getEarnHi( int year ) const { return earnHi[year]; }
   // Description: Returns Medicare earnings for specified year if
   // there are no OASDI earnings; otherwise return zero.
   //
   // Returns: Medicare earnings for specified year if 
   // there are no OASDI earnings; otherwise return zero.
   //
   // Arguments:
   //   year: Year of earnings.
   double getEarnHiOnly( int year ) const
      { return (earnOasdi[year] > 4.5) ? 0.0 : earnHi[year]; }
   // Description: Returns military service earnings for specified year.
   //
   // Returns: Military service earnings for specified year.
   //
   // Arguments:
   //   year: Year for which earnings are desired.
   double getEarnMS( int year ) const { return milServDatesVec.getEarn(year); }
   // Description: Returns total military service earnings, 1937-50.
   //
   // Returns: Total military service earnings, 1937-50.
   double getEarnMS3750() const { return milServDatesVec.getEarn3750(); }
   // Description: Returns OASDI plus Medicare earnings for specified year.
   //
   // Returns: OASDI plus Medicare earnings for specified year.
   //
   // Arguments:
   //   year: Year of earnings.
   double getEarnOasdhi( int year ) const
      { return (earnOasdi[year] + earnHi[year]); }
   // Description: Returns OASDI-covered earnings for the specified year.
   //
   // Returns: OASDI-covered earnings for the specified year.
   //
   // Arguments:
   //   year: Year for which earnings are desired.
   double getEarnOasdi( int year ) const { return earnOasdi[year]; }
   // Description: Returns railroad earnings for specified year.
   //
   // Returns: Railroad earnings for specified year.
   //
   // Arguments:
   //   year: Year for which earnings are desired.
   double getEarnRR( int year ) const { return railRoadData.earnings[year]; }
   // Description: Returns total railroad earnings, 1937-50.
   //
   // Returns: Total railroad earnings, 1937-50.
   double getEarnRR3750() const { return railRoadData.getEarn3750(); }
   // Description: Returns first year of OASDI earnings (0 if none).
   //
   // Returns: First year of OASDI earnings (0 if none).
   int getFirstYearEarn() const { return earnOasdi.startYear(); }
   // Description: Returns first year of HI earnings (0 if none).
   //
   // Returns: First year of HI earnings (0 if none).
   int getFirstYearEarnHi() const { return earnHi.startYear(); }
   // Description: Returns first year of military service (0 if none).
   //
   // Returns: First year of military service (0 if none).
   int getFirstYearMS() const { return milServDatesVec.getFirstYear(); }
   // Description: Returns first year of railroad (0 if none).
   //
   // Returns: First year of railroad (0 if none).
   int getFirstYearRR() const { return railRoadData.getFirstYear(); }
   // Description: Returns Social Security number as a number.
   //
   // Returns: Social Security number as a number.
   unsigned long getIdNumber() const { return ssn.toNumber(); }
   // Description: Returns Social Security number as a string.
   //
   // Returns: Social Security number as a string.
   std::string getIdString() const { return ssn.toString(); }
   // Description: Returns military service indicator.
   //
   // Returns: True if any military service, false otherwise.
   bool getIndms() const { return milServDatesVec.getMSCount() > 0; }
   // Description: Returns last year of OASDI earnings (0 if none).
   //
   // Returns: Last year of OASDI earnings (0 if none).
   int getLastYearEarn() const { return earnOasdi.endYear(); }
   // Description: Returns last year of HI earnings (0 if none).
   //
   // Returns: Last year of HI earnings (0 if none).
   int getLastYearEarnHi() const { return earnHi.endYear(); }
   // Description: Returns last year of military service (0 if none).
   //
   // Returns: Last year of military service (0 if none).
   int getLastYearMS() const { return milServDatesVec.getLastYear(); }
   // Description: Returns last year of military service (0 if none).
   //
   // Returns: Last year of military service (0 if none).
   int getLastYearRR() const { return railRoadData.getLastYear(); }
   // Description: Returns one line of number holder's address.
   //
   // Returns: One line of number holder's address.
   //
   // Arguments:
   //   line: Desired line number (0-2).
   std::string getNhaddr( int line ) const { return(nhaddr[line]); }
   // Description: Returns number holder's name.
   //
   // Returns: Number holder's name.
   std::string getNhname() const { return(nhname); }
   // Description: Returns date of cessation of oab entitlement prior to
   //   most recent DIB entitlement.
   //
   // Returns: The date of cessation of oab entitlement prior to
   //   most recent DIB entitlement.
   DateMoyr getOabCessDate() const { return(oabCessDate); }
   // Description: Returns date of entitlement to oab prior to most
   //   recent DIB entitlement.
   //
   // Returns: The date of entitlement to oab prior to most recent DIB
   // entitlement.
   DateMoyr getOabEntDate() const { return(oabEntDate); }
   // Description: Returns monthly noncovered pension.
   //
   // Returns: Monthly noncovered pension.
   float getPubpen() const { return(pubpen); }
   // Description: Returns monthly noncovered pension.
   //
   // Returns: Monthly noncovered pension.
   DateMoyr getPubpenDate() const { return(pubpenDate); }
   // Description: Returns monthly noncovered pension after December 1994,
   //   after removing reservist pension.
   //
   // Returns: Monthly noncovered pension after December 1994,
   //   after removing reservist pension.
   float getPubpenReservist() const { return(pubpenReservist); }
   // Description: Returns military service quarters of coverage for specified
   //   year.
   //
   // Returns: Military service quarters of coverage for specified year.
   //
   // Arguments:
   //   year: Year for which quarters are desired.
   unsigned getQcovMS( int year ) const
      { return milServDatesVec.getQcov(year); }
   // Description: Returns total military service quarters of coverage, 1937-50.
   //
   // Returns: Total military service quarters of coverage, 1937-50.
   unsigned getQcovMS3750() const { return milServDatesVec.getQc3750(); }
   // Description: Returns railroad quarters of coverage for specified year.
   //
   // Returns: Railroad quarters of coverage for specified year.
   //
   // Arguments:
   //   year: Year for which quarters are desired.
   unsigned getQcovRR( int year ) const { return railRoadData.qc.get(year); }
   // Description: Returns total railroad quarters of coverage, 1937-50.
   //
   // Returns: Total railroad quarters of coverage, 1937-50.
   unsigned getQcovRR3750() const { return railRoadData.getQc3750(); }
   // Description: Returns total railroad quarters of coverage, 1951-77.
   //
   // Returns: Total railroad quarters of coverage, 1951-77.
   unsigned getQcovRR5177() const
      { return railRoadData.qc.accumulate(1951, 1977, 0); }
   // Description: Returns type of taxes for specified year.
   //
   // Returns: Type of taxes for specified year.
   //
   // Arguments:
   //   year: Year of taxes.
   bool getTaxType( int year ) const { return taxType.getBit(year); }
   // Description: Returns array of type of taxes.
   //
   // Returns: Array of type of taxes.
   const BitAnnual& getTaxTypeArray() const { return taxType; }
   void pubpenDateCheck2() const;
   void setEarnHi( int year, double newEarnHi );
   void setEarnOasdi( int year, double newEarn );
   // Description: Sets one line of address.
   //
   // Arguments:
   //   line: Line of address to set (0-2).
   //   newNhaddr: New address to use.
   void setNhaddr( int line, const std::string& newNhaddr )
      { nhaddr[line] = newNhaddr; }
   // Description: Sets number holder's name.
   //
   // Arguments:
   //   newNhname: New name to use.
   void setNhname( const std::string& newNhname ) { nhname = newNhname; }
   void setOabCessDate( const DateMoyr& datetemp );
   void setOabEntDate( const DateMoyr& datetemp );
   void setPubpen( float newPubpen );
   void setPubpenDate( const DateMoyr& newPubpenDate );
   void setPubpenReservist( float newPubpen );
   void setTaxType( int year, int newTaxType );
   void zeroEarn( int ibegint, int iendt );
   void zeroEarnHi( int ibegint, int iendt );
   void zeroEarnOasdi( int ibegint, int iendt );
};
