// $Id: DateFormatter.cpp 1.6 2008/04/25 12:58:23EDT 277133 Development  $
//
// Functions to format dates.

#include "DateFormatter.h"
#include "Resource.h"
#include "datemoyr.h"
#include "PiaException.h"

using namespace std;
#include "boost/date_time/gregorian/gregorian.hpp"
#include "boost/date_time/gregorian/gregorian_io.hpp"

// Description: Returns string with formatted date.
//
// Returns: If no format is specified, a string with date using default format.
// Otherwise, see formatModyyr or formatMoyr.
//
// Arguments:
//   dateModyyr: The month, day, and year date to format.
std::string DateFormatter::toString( const boost::gregorian::date& dateModyyr )
{
   if (dateModyyr.is_not_a_date()) {
      return string("");
   }
   try {
      boost::gregorian::date_facet* output_facet =
         new boost::gregorian::date_facet("%B %d, %Y");
      stringstream ss;
      ss.imbue(locale(locale::classic(), output_facet));
      ss << dateModyyr;
      return ss.str();
   } catch (exception&) {
      // ignore any exception (return empty string)
      return string("");
   }
}

// Arguments:
//   formatModyyr:  Format to use for month, day, and year dates.  If none
//     used, formatted as "month day, year".
//
//     @table
//     value  meaning
//     "s"    format as "mo/dy/year", with leading 0's if necessary.
//     "n"    format as "modyyear", with leading 0's if necessary.
//     other  format as "month day, year".
std::string DateFormatter::toString( const boost::gregorian::date& dateModyyr,
const std::string& formatModyyr )
{
   if (formatModyyr.length() < 1) {
      return toString(dateModyyr);
   }
   if (dateModyyr.is_not_a_date()) {
      switch (formatModyyr[0]) {
      case 's':
        return string("00/00/0000");
      default:
        return string("");
      }
   }
   try {
      boost::gregorian::date_facet* output_facet =
         new boost::gregorian::date_facet("%B %d, %Y");
      ostringstream ss;
      ss.imbue(locale(locale::classic(), output_facet));
      switch (formatModyyr[0]) {
      case 's':
         output_facet->format("%m/%d/%Y");
         ss << dateModyyr;
         return ss.str();
      case 'n':
         output_facet->format("%m%d%Y");
         ss << dateModyyr;
         return ss.str();
      default:
         return toString(dateModyyr);
      }
   } catch (exception&) {
      // ignore any exception (return empty string)
      return string("");
   }
}

// Description: Parses a date from an undelimited date string where the order
// is month (2 digits), day (2 digits), and year (4 digits).
//
// Returns: The date parsed from the date string.
//
// Arguments:
//   line: Line to parse.
boost::gregorian::date DateFormatter::date_from_undelimited_us_string(
 const std::string& line )
{
   try {
      const unsigned short month =
         static_cast<unsigned short>(atoi(line.substr(0, 2).c_str()));
      const unsigned short day =
         static_cast<unsigned short>(atoi(line.substr(2, 2).c_str()));
      const unsigned short year =
         static_cast<unsigned short>(atoi(line.substr(4, 4).c_str()));
      return boost::gregorian::date(year, month, day);
   } catch (boost::gregorian::bad_year) {
     throw PiaException(PIA_IDS_DATEYEAR);
   } catch (boost::gregorian::bad_month) {
     throw PiaException(PIA_IDS_DATEMONTH);
   } catch (boost::gregorian::bad_day_of_month) {   
     throw PiaException(PIA_IDS_DATEDAY);
   } catch (out_of_range&) {
     throw PiaException(PIA_IDS_READERR);
   }
}

// Description: Parses a month and year date from an undelimited date string
// where the order is month (2 digits) and year (4 digits).
//
// Returns: The month and year date parsed from the date string.
//
// Arguments:
//   line: Line to parse.
DateMoyr DateFormatter::month_year_from_undelimited_us_string(
 const std::string& line )
{
   try {
      const unsigned month =
         static_cast<unsigned>(atoi(line.substr(0, 2).c_str()));
      const unsigned year =
         static_cast<unsigned>(atoi(line.substr(2, 4).c_str()));
      return DateMoyr(month, year);
   } catch (boost::gregorian::bad_year) {
     throw PiaException(PIA_IDS_DATEYEAR);
   } catch (boost::gregorian::bad_month) {
     throw PiaException(PIA_IDS_DATEMONTH);
   } catch (out_of_range&) {
      throw PiaException(PIA_IDS_READERR);
   }
}

// Arguments:
//   dateMoyr: The month and year date to format.
std::string DateFormatter::toString( const DateMoyr& dateMoyr )
{
   try {
      ostringstream ss;
      ss << boost::gregorian::greg_month(dateMoyr.getMonth()).as_long_string()
         << " " << dateMoyr.getYear();
      return(ss.str());
   } catch (exception&) {
      // ignore any exception (return empty string)
      return string("");
   }
}

// Arguments:
//   formatMoyr:  Format to use for month and year dates.  If none used,
//     formatted as "month year".
//
//     @table
//     value  meaning
//     "s"    format as "mo/year", with leading 0 if necessary.
//     "n"    format as "moyear", with leading 0 if necessary.
//     other  format as "month year".
std::string DateFormatter::toString( const DateMoyr& dateMoyr,
const std::string& formatMoyr )
{
   if (formatMoyr.length() < 1)
      return toString(dateMoyr);
   try {
      ostringstream ss;
      ss.fill('0');
      switch (formatMoyr[0]) {
      case 's':
         ss << setw(2) << dateMoyr.getMonth() << "/"
            << setw(4) << dateMoyr.getYear();
         return(ss.str());
      case 'n':
         ss << setw(2) << dateMoyr.getMonth()
            << setw(4) << dateMoyr.getYear();
         return(ss.str());
      default:
         return toString(dateMoyr);
      }
   } catch (exception&) {
      // ignore any exception (return empty string)
      return string("");
   }
}
