// $Id: Path.cpp 1.2 2005/12/14 17:39:04EST 044579 Exp  $
//
// Functions for class that provides operations on string instances that
// contain file or directory path information.

#include "Path.h"

using namespace std;

// Description: A string with all the possible separator characters.
const std::string Path::allSeparators = "\\/";

// Description: Combines two path strings.
//
// Returns: A string containing the combined paths.
//
// Arguments:
//   path1: The first path.
//   path2: The second path.
std::string Path::combine( const std::string& path1, const std::string& path2 )
{
   if (path1.length() == 0) {
      return path2;
   }
   if (path2.length() == 0) {
      return path1;
   }
   const string::size_type path1Length = path1.length();
   const char path1LastChar = path1[path1Length - 1];
   if (path1LastChar == directorySeparatorChar ||
      path1LastChar == altDirectorySeparatorChar) {
      return string(path1 + path2);
   }
   else {
      return string(path1 + directorySeparatorChar + path2);
   }
}

// Description: Returns the directory information for the specified path
// string.
//
// Returns: A string containing directory information for path.
//
// Arguments:
//   path: The path of a file or directory.
std::string Path::getDirectoryName( const std::string& path )
{
   const string::size_type index = path.find_last_of(allSeparators);
   return (index == string::npos) ? path : path.substr(index);
}

// Description: Changes the extension of a path string.
//
// Returns: The string containing the modified path information.
//
// Remarks: If neither path nor extension contains a period, changeExtention
// adds the period.
//
// Arguments:
//   path: The path information to modify.
//   extension: The new extension (with a leading period).
std::string Path::changeExtension( const std::string& path,
const std::string& extension )
{
   const string::size_type indexSeparator = path.find_last_of(allSeparators);
   const string::size_type indexPeriod = path.find_last_of('.');
   if (indexPeriod == std::string::npos) {
      // there is no current extension
      string temp(path);
      if (extension[0] != '.') {
         temp += string(".");
      }
      temp += extension;
      return temp;
   }
   else if (indexSeparator == std::string::npos ||
      indexSeparator < indexPeriod) {
      // there is no separator, or the separator comes before the extension,
      // so overwrite it
      string temp(path.substr(0, indexPeriod));
      if (extension[0] != '.') {
         temp += string(".");
      }
      temp += extension;
      return temp;
   }
   else {
      // there is no extension after the separator
      string temp(path);
      if (extension[0] != '.') {
         temp += string(".");
      }
      temp += extension;
      return temp;
   }
}
