// $Id: datemoyr.cpp 1.34 2007/03/09 17:08:46EST 044579 Development 277133(2008/04/08 10:12:09EDT) $
//
// Functions to handle date as month and year.

#include <vector>
#include <string>
#include "datemoyr.h"
#include "age.h"
#include "Resource.h"
#include "PiaException.h"
#include "StringParser.h"

using namespace std;

// Arguments:
//   dateModyyr: The date to copy from (month, day, and year).
DateMoyr::DateMoyr( const boost::gregorian::date& dateModyyr ) :
monthOfYear(static_cast<unsigned short>(dateModyyr.month())),
yearPart(static_cast<unsigned short>(dateModyyr.year()))
{ }

// Arguments:
//   dateTemp: The date string to parse (as mo/year).
DateMoyr::DateMoyr( const std::string& dateTemp )
{
   vector< string > strvec = parseString(dateTemp, '/');
   if (strvec.size() < 2)
      throw PiaException(PIA_IDS_DATEREAD);
   monthOfYear = static_cast<unsigned short>(atoi(strvec[0].c_str()));
   yearPart = static_cast<unsigned short>(atoi(strvec[1].c_str()));
#ifndef NDEBUG
   check();
#endif
}

// Description: Compares this date to another one.
//
// Returns: True if this date is before argument date.
//
// Arguments:
//   dateTemp: The date to compare to.
bool DateMoyr::operator<( const DateMoyr& dateMoyr ) const
{
   // check year
   if (yearPart > dateMoyr.getYear())
      return(false);
   if (yearPart < dateMoyr.getYear())
      return(true);
   // same year, so check month
   if (monthOfYear < dateMoyr.getMonth())
      return(true);
   // same year and month is not less than specified month
   return(false);
}

// Arguments:
//   dateModyyr: The date to copy from (month, day, and year). Ignores the day.
DateMoyr& DateMoyr::operator=( const boost::gregorian::date& dateModyyr )
{
   monthOfYear = static_cast<unsigned short>(dateModyyr.month());
   yearPart = static_cast<unsigned short>(dateModyyr.year());
   return *this;
}

// Description: Checks for valid month and year.
//
// Throws: PiaException of type PIA_IDS_DATEMONTH or PIA_IDS_DATEYEAR if bad
//   month or year.
void DateMoyr::check() const
{
   Date::monthCheck(monthOfYear);
   Date::yearCheck(yearPart);
}

// Description: Calculates a date by adding an Age.
//
// Returns: The instance date plus the argument age.
//
// Arguments:
//   age: Age to add to this date.
DateMoyr DateMoyr::operator+( const Age& age ) const
{
   unsigned month = monthOfYear + age.getMonths();
   unsigned year = yearPart + age.getYears();
   if (month > 12) {
      month -= 12;
      year += 1;
   }
   return DateMoyr(month, year);
}

// Description: Calculates an age by subtracting argument date from this one.
//
// Returns: The calculated age.
//
// Throws: PiaException if argument is greater than this instance.
//
// Remarks: This would typically be used when this instance is the date
//   of entitlement to a benefit and the argument is the date of birth.
//
// Arguments:
//   dateMoyr: The date to subtract from this instance.
Age DateMoyr::operator-( const DateMoyr& dateMoyr ) const
{
   if (monthOfYear >= dateMoyr.getMonth()) {
      if (yearPart < dateMoyr.getYear()) {
         throw PiaException("Dates out of order in operator-");
      }
      return Age(yearPart - dateMoyr.getYear(),
         monthOfYear - dateMoyr.getMonth());
   }
   else {
      if (yearPart <= dateMoyr.getYear()) {
         throw PiaException("Dates out of order in operator-");
      }
      return Age(yearPart - dateMoyr.getYear() - 1,
         12u + monthOfYear - dateMoyr.getMonth());
   }
}

// Description: Returns number of months from this date to given date.
//
// Returns: Number of months from this date to given date. Result is negative
//   if argument date is before this date.
//
// Arguments:
//   dateMoyr: Given date.
int DateMoyr::getMonths( const DateMoyr& dateMoyr ) const
{
   int i1 = 12 * yearPart + monthOfYear;
   int i2 = 12 * dateMoyr.getYear() + dateMoyr.getMonth();
   return(i2 - i1);
}

// Description: Calculates difference of two dates as an age.
//
// Returns: Number of years and months as an Age.
//
// Throws: PiaException of type PIA_IDS_DATEDIFF if second date is before
//   first date.
//
// Arguments:
//   date1: First date.
//   date2: Second date (must not be before first date).
Age DateMoyr::diff( const DateMoyr& date1, const DateMoyr& date2 )
{
   // must not have second date before first date
   if (date2 < date1)
      throw PiaException(PIA_IDS_DATEDIFF);
   if (date2.getMonth() >= date1.getMonth()) {
      return Age((unsigned)(date2.getYear() - date1.getYear()),
         (unsigned)(date2.getMonth() - date1.getMonth()));
   }
   else {
      return Age((unsigned)(date2.getYear() - date1.getYear() - 1),
         (unsigned)(12 + date2.getMonth() - date1.getMonth()));
   }
}

// Description: Returns the date that is the month before this date.
//
// Returns: The month and year that is the month before this date.
DateMoyr DateMoyr::getMonthBefore() const
{
#ifndef NDEBUG
   check();
#endif
   unsigned month = getMonth() - 1;
   unsigned year = getYear();
   // take care of January dates
   if (month < boost::date_time::Jan) {
      month = boost::date_time::Dec;
      year -= 1;
   }
   return DateMoyr(month, year);
}
