// $Id: dbleann.cpp 1.23 2006/11/24 08:53:00EST 044579 Development  $
//
// Functions to handle DoubleAnnuals.

#include <fstream>
#include <algorithm>
#include <iomanip>
#if !defined(SHORTFILENAMES)
#include "dbleann.h"
#else
#include "dbleannh"
#endif
#include "intann.h"

using namespace std;

// Description: Constructor.
//
// Remarks: The version with no arguments sets the range of years to an
// impossible combination. They must be reset with the resetYears function
// before an instance created with the default constructor is usable.
//
// The version with two int arguments sets the baseYear,
// and initializes the size of the data vector to the span of years. The
// version with a DoubleAnnual argument copies all data from that argument.
DoubleAnnual::DoubleAnnual() :
theData(), width(9), baseYear(0), precision(0), lineWidth(75)
{ }

// Arguments:
//   newBaseYear: New base year.
//   newLastYear: New last year.
DoubleAnnual::DoubleAnnual( int newBaseYear, int newLastYear ) :
theData(newLastYear - newBaseYear + 1), width(9), baseYear(newBaseYear),
precision(0), lineWidth(75)
{ }

// Arguments:
//   newData: The DoubleAnnual to copy from.
DoubleAnnual::DoubleAnnual( const DoubleAnnual& newData ) :
theData(newData.theData), width(newData.width), baseYear(newData.baseYear),
precision(newData.precision), lineWidth(newData.lineWidth)
{ }

// Description: Deletes allocated memory.
DoubleAnnual::~DoubleAnnual()
{ }

// Description: Copies from another DoubleAnnual.
//
// Arguments:
//   newData: The DoubleAnnual to copy from.
DoubleAnnual& DoubleAnnual::operator=( const DoubleAnnual& newData )
{
   theData = newData.theData;
   width = newData.width;
   baseYear = newData.baseYear;
   precision = newData.precision;
   lineWidth = newData.lineWidth;
   return *this;
}

// Description: Resets the range of years.
//
// Remarks: This function needs to be called if the instance was created with
// the default constructor. If it is called again, or if the constructor with
// a range of years was used, this function clears all values in the vector.
//
// Arguments:
//   newBaseYear: New base year.
//   newLastYear: New last year.
void DoubleAnnual::resetYears( int newBaseYear, int newLastYear )
{
  theData.clear();
  theData.resize(newLastYear - newBaseYear + 1);
  baseYear = newBaseYear;
}

// Description: Zeroes out all data values.
void DoubleAnnual::deleteContents()
{
   fill(theData.begin(), theData.end(), 0.0);
}

// Description: Sets data for specified years.
//
// Arguments:
//   newData: Array of new values of data.
//   firstYeart: First year of data to set.
//   lastYeart: Last year of data to set.
void DoubleAnnual::assign( const DoubleAnnual& newData, int firstYeart,
   int lastYeart )
{
   int year1 = max(baseYear, newData.baseYear);
   year1 = max(year1, firstYeart);
   int year2 = min(getLastYear(), newData.getLastYear());
   year2 = min(year2, lastYeart);
   copy(newData.theData.begin() + year1 - newData.baseYear,
      newData.theData.begin() + year2 + 1 - newData.baseYear,
      theData.begin() + year1 - baseYear);
}

// Arguments:
//   newData: Array of new values of data.
//   firstYeart: First year of data to set.
//   lastYeart: Last year of data to set.
void DoubleAnnual::assign( const IntAnnual& newData, int firstYeart,
   int lastYeart )
{
   int year1 = max(baseYear, newData.getBaseYear());
   year1 = max(year1, firstYeart);
   int year2 = min(getLastYear(), newData.getLastYear());
   year2 = min(year2, lastYeart);
   for (int year = year1; year <= year2; year++)
      theData[year - baseYear] = static_cast<double>(newData[year]);
}

// Arguments:
//   newData: New value of data to be used for all specified years.
//   firstYeart: First year of data to set.
//   lastYeart: Last year of data to set.
void DoubleAnnual::assign( double newData, int firstYeart, int lastYeart )
{
   const int year1 = max(baseYear, firstYeart);
   const int year2 = min(getLastYear(), lastYeart);
   fill(theData.begin() + year1 - baseYear,
      theData.begin() + year2 + 1- baseYear, newData);
}

// Description: Multiplies data by either a scalar or another vector of annual
//   data.
//
// Arguments:
//   series: Array of multipliers.
//   firstYeart: First year of data to multiply.
//   lastYeart: Last year of data to multiply.
void DoubleAnnual::multiply( const DoubleAnnual& series, int firstYeart,
   int lastYeart )
{
   int year1 = max(baseYear, series.baseYear);
   year1 = max(year1, firstYeart);
   int year2 = min(getLastYear(), series.getLastYear());
   year2 = min(year2, lastYeart);
   for (int year = year1; year <= year2; year++)
      theData[year - baseYear] *= series[year];
}

// Arguments:
//   factor: The scalar multiplier. It is applied to all specified years of
//     data.
//   firstYeart: First year of data to multiply.
//   lastYeart: Last year of data to multiply.
void DoubleAnnual::multiply( double factor, int firstYeart, int lastYeart )
{
   const int year1 = max(baseYear, firstYeart);
   const int year2 = min(getLastYear(), lastYeart);
   for (int i = year1; i <= year2; i++) 
      theData[i - baseYear] *= factor;
}

// Description: Adds a DoubleAnnual to this one.
//
// Arguments:
//   newData: Array of doubles to add.
//   firstYeart: First year to add.
//   lastYeart: Last year to add.
void DoubleAnnual::add( const DoubleAnnual& newData, int firstYeart,
int lastYeart )
{
   for (int i = firstYeart; i <= lastYeart; i++)
      theData[i - baseYear] += newData[i];
}

// Description: Subtracts a DoubleAnnual from this one.
//
// Arguments:
//   newData: Array of doubles to subtract.
//   firstYeart: First year to subtract.
//   lastYeart: Last year to subtract.
void DoubleAnnual::subtract( const DoubleAnnual& newData, int firstYeart,
int lastYeart )
{
   for (int i = firstYeart; i <= lastYeart; i++)
      theData[i - baseYear] -= newData[i];
}

// Description: Divides this DoubleAnnual by another one.
//
// Arguments:
//   newData: Array of doubles to divide by.
//   firstYeart: First year to divide.
//   lastYeart: Last year to divide.
void DoubleAnnual::divide( const DoubleAnnual& newData, int firstYeart,
int lastYeart )
{
   int year1 = max(baseYear, newData.baseYear);
   year1 = max(year1, firstYeart);
   int year2 = min(getLastYear(), newData.getLastYear());
   year2 = min(year2, lastYeart);
   for (int year = year1; year <= year2; year++) {
      if (newData[year] == 0.0)
         theData[year - baseYear] = 0.0;
      else
         theData[year - baseYear] /= newData[year];
   }
}

// Arguments:
//   newData: Array of doubles to divide by.
//   firstYeart: First year to divide.
//   lastYeart: Last year to divide.
void DoubleAnnual::divide( const IntAnnual& newData, int firstYeart,
int lastYeart )
{
   int year1 = max(baseYear, newData.getBaseYear());
   year1 = max(year1, firstYeart);
   int year2 = min(getLastYear(), newData.getLastYear());
   year2 = min(year2, lastYeart);
   for (int year = year1; year <= year2; year++) {
      if (newData[year] == 0)
         theData[year - baseYear] = 0.0;
      else
         theData[year - baseYear] /= static_cast<double>(newData[year]);
   }
}

// Description: Reads array from an open stream.
//
// Throws: PiaException of type PIA_IDS_DBLEANN2 if unable to read specified
//   fields.
//
// Arguments:
//   in: Input stream.
//   firstYeart: First year to read.
//   lastYeart: Last year to read.
void DoubleAnnual::read( std::istream& in, int firstYeart, int lastYeart )
{
   double temp;  // temporary value

   for (int i = firstYeart; i <= lastYeart; i++)  {
      if (!(in >> temp))
         throw PiaException(PIA_IDS_DBLEANN2);
      theData[i - baseYear] = temp;
   }
}

// Description: Writes array to an open stream.
//
// Arguments:
//   out: Output stream.
//   firstYeart: First year to write.
//   lastYeart: Last year to write.
void DoubleAnnual::write( std::ostream& out, int firstYeart, int lastYeart )
{
   const int year1 = max(baseYear, firstYeart);
   const int year2 = min(getLastYear(), lastYeart);
   const int numperline = lineWidth / width;  // number of numbers per line
   out.precision(precision);
   for (int i = year1; i <= year2; i++) {
      out << setw(width) << theData[i - baseYear];
      if ((i - firstYeart + 1) % numperline == 0)
         out << endl;
   }
   if ((year2 - year1 + 1) % numperline != 0)
      out << endl;
}
