// $Id: dblemth.cpp 1.5 2006/11/08 11:03:41EST 044579 Development  $
//
// Functions to handle DoubleMonth arrays.

#include <fstream>
#include <iomanip>
#include "dblemth.h"
#include "Resource.h"
#include "PiaException.h"
#include "datemoyr.h"
#include "qtryear.h"

using namespace std;

// Description: Constructor.
//
// Remarks: The default constructor initializes the size of the array to 0,
//   so the initialize function must be called before the instance is used.
DoubleMonth::DoubleMonth() : theData(0), width(9), baseYear(0),
lastYear(0), precision(0), lineWidth(60)
{ }

// Arguments:
//   newBaseYear: New base year.
//   newLastYear: New last year.
DoubleMonth::DoubleMonth( int newBaseYear, int newLastYear ) :
theData((newLastYear - newBaseYear + 1) * 12), width(9), baseYear(newBaseYear),
lastYear(newLastYear), precision(0), lineWidth(60)
{ }

// Description: Initializes size, base year, and last year.
//
// Remarks: Resizes the data array to cover the specified span of years.
//
// Arguments:
//   newBaseYear: New base year.
//   newLastYear: New last year.
void DoubleMonth::initialize( int newBaseYear, int newLastYear )
{
   int newSize = (newLastYear - newBaseYear + 1) * 12;
   theData.resize(newSize);
   baseYear = newBaseYear;
   lastYear = newLastYear;
}

// Description: Returns one value of data.
//
// Throws: PiaException of type PIA_IDS_FLOATMTH1 if year is out of range
//   (only in debug mode).
//
// Returns: One value of data for the specified year and month.
//
// Arguments:
//   dateMoyr: Year and month (0-11) for which data is desired.
double DoubleMonth::getAt( const DateMoyr& dateMoyr ) const
{
   const int year = (int)dateMoyr.getYear();
#ifndef NDEBUG
   if (year < baseYear || year > lastYear)
      throw PiaException(PIA_IDS_FLOATMTH1);
#endif
   return theData[12 * (year - baseYear) + (int)dateMoyr.getMonth()];
}

// Description: Sets one value of data.
//
// Throws: PiaException of type PIA_IDS_FLOATMTH1 if year is out of range
//   (only in debug mode).
//
// Arguments:
//   dateMoyr: Year and month (0-11) for which data is to be set.
//   newData: New value of data.
void DoubleMonth::setAt( const DateMoyr& dateMoyr, double newData )
{
   const int year = (int)dateMoyr.getYear();
#ifndef NDEBUG
   if (year < baseYear || year > lastYear)
      throw PiaException(PIA_IDS_FLOATMTH1);
#endif
   theData[12 * (year - baseYear) + (int)dateMoyr.getMonth()] = newData;
}

// Description: Sets data for specified years.
//
// Arguments:
//   newData: Array of new values of data.
//   firstYeart: First year of data to set.
//   lastYeart: Last year of data to set.
void DoubleMonth::assign( const DoubleMonth& newData, int firstYeart,
int lastYeart )
{
   for (int year = firstYeart; year <= lastYeart; year++) {
      for (int month = 0; month < 12; month++) {
         theData[12 * (year - baseYear) + month] =
            newData.getAt(DateMoyr(month, year));
      }
   }
}

// Arguments:
//   newDouble: New value of data to be used for all years.
//   firstYeart: First year of data to set.
//   lastYeart: Last year of data to set.
void DoubleMonth::assign( double newDouble, int firstYeart, int lastYeart )
{
   for (int year = firstYeart; year <= lastYeart; year++)
      for (int month = 0; month < 12; month++)
         theData[12 * (year - baseYear) + month] = newDouble;
}

// Description: Calculates annual average.
//
// Returns: Annual average.
//
// Arguments:
//   year: Year for which average is desired.
double DoubleMonth::average( int year ) const
{
   double temp = 0.0;
   for (int i = 0; i < 12; i++)
      temp += getAt(DateMoyr(i, year));
   return(temp / 12.0);
}

// Description: Calculates average for one quarter.
//
// Throws: PiaException of type PIA_IDS_FLOATMTH3 if quarter is out of range
//   (only in debug mode).
//
// Returns: Average for one quarter.
//
// Arguments:
//   qtrYear: Year and quarter (0-3) for which average is desired.
double DoubleMonth::qaverage( const QtrYear& qtrYear ) const
{
#ifndef NDEBUG
   if (qtrYear.getQuarter() < 0 || qtrYear.getQuarter() > 3)
      throw PiaException(PIA_IDS_FLOATMTH3);
#endif
   double temp = 0.f;
   for (unsigned i = 3u * qtrYear.getQuarter();
   i < 3u * (qtrYear.getQuarter() + 1u); i++) {
      temp += getAt(DateMoyr(i, qtrYear.getYear()));
   }
   return(temp / 3.f);
}

// Description: Reads array from an open stream.
//
// Throws: PiaException of type PIA_IDS_FLOATMTH2 if unable to read data for
//   all specified years.
//
// Arguments:
//   in: Input stream.
//   firstYeart: First year of data to read.
//   lastYeart: Last year of data to read.
void DoubleMonth::read( std::istream& in, int firstYeart, int lastYeart )
{
   double temp;  // temporary value

   for (int i = firstYeart; i <= lastYeart; i++) {
      for (int month = 0; month < 12; month++) {
         if (!(in >> temp))
            throw PiaException(PIA_IDS_FLOATMTH2);
         setAt(DateMoyr(month, i), temp);
      }
   }
}

// Description: Writes array to an open stream.
//
// Arguments:
//   out: Output stream.
//   firstYeart: First year of data to write.
//   lastYeart: Last year of data to write.
void DoubleMonth::write( std::ostream& out, int firstYeart, int lastYeart )
{
   int numperline = lineWidth / width;  // number of numbers per line
   out.precision(precision);
   for (int i = firstYeart; i <= lastYeart; i++)  {
      for (int month = 0; month < 12; month++) {
         out << setw(width) << getAt(DateMoyr(month, i));
         if ((12 * (i - firstYeart) + month) % numperline == numperline - 1)
            out << endl;
      }
   }
   // Put an endline if an odd number of numbers
   if ((12 * (lastYeart - firstYeart + 1)) % numperline != 0)
      out << endl;
}

// Description: Finds number of months of (nonzero) data in a given year.
//
// Returns: Number of months of (nonzero) data in a given year.
//
// Arguments:
//   year: Year to test.
int DoubleMonth::nmonCal( int year ) const
{
   for (int month = 0; month < 12; month++) {
      if (getAt(DateMoyr(month, year)) < 1.)
         return(month);
   }
   return(12);
}
