// $Id: dbleqtr.cpp 1.7 2007/03/09 17:08:46EST 044579 Development  $
//
// Functions to handle DoubleQuarter arrays.

#include <fstream>
#include <iomanip>
#include <cmath>  // for pow
#include "dblemth.h"
#include "dbleqtr.h"
#include "qtryear.h"
#include "Resource.h"
#include "PiaException.h"
#ifndef NDEBUG
#include <sstream>
#include "Trace.h"
#endif

using namespace std;

// Description: Constructor.
//
// Remarks: The default constructor initializes the size of the array to 0,
//   so the initialize function must be called before the instance is used.
DoubleQuarter::DoubleQuarter() : theData(), width(9), baseYear(0),
lastYear(0), precision(0), lineWidth(75)
{ }

// Arguments:
//   newBaseYear: New base year.
//   newLastYear: New last year.
DoubleQuarter::DoubleQuarter( int newBaseYear, int newLastYear ) :
theData((newLastYear - newBaseYear + 1) * 4), width(9), baseYear(newBaseYear),
lastYear(newLastYear), precision(0), lineWidth(75)
{ }

// Description: Initializes array with specified years.
//
// Remarks: Resizes the data array to cover the specified span of years.
//
// Arguments:
//   newBaseYear: New base year.
//   newLastYear: New last year.
void DoubleQuarter::initialize( int newBaseYear, int newLastYear )
{
   int newSize = (newLastYear - newBaseYear + 1) * 4;
   theData.resize(newSize);
   baseYear = newBaseYear;
   lastYear = newLastYear;
}

// Description: Gets one value of data.
//
// Throws: PiaException of type PIA_IDS_FLOATQTR1 if year is out of range
//   (only in debug mode).
//
// Returns: One value of data for the specified year and quarter.
//
// Arguments:
//   qtrYear: Year and quarter (0-3) for which data is desired.
double DoubleQuarter::getAt( const QtrYear& qtrYear ) const
{
#ifndef NDEBUG
   if (static_cast<int>(qtrYear.getYear()) < baseYear ||
       static_cast<int>(qtrYear.getYear()) > lastYear)
      throw PiaException(PIA_IDS_FLOATQTR1);
#endif
   return theData[4 * (qtrYear.getYear() - baseYear) + qtrYear.getQuarter()];
}

// Description: Sets one value of data.
//
// Throws: PiaException of type PIA_IDS_FLOATQTR1 if year is out of range
//   (only in debug mode).
//
// Arguments:
//   qtrYear: Year and quarter (0-3) for which data is desired.
//   newData: New value of data.
void DoubleQuarter::setAt( const QtrYear& qtrYear, double newData )
{
#ifndef NDEBUG
   if (static_cast<int>(qtrYear.getYear()) < baseYear ||
       static_cast<int>(qtrYear.getYear()) > lastYear) {
      ostringstream strm;
      strm << "year = " << qtrYear.getYear() << ", baseYear = " << baseYear
         << ", lastYear = " << lastYear;
      Trace::writeLine(strm.str());
      throw PiaException(PIA_IDS_FLOATQTR1);
   }
#endif
   theData[4 * (qtrYear.getYear() - baseYear) + qtrYear.getQuarter()] = newData;
}

// Description: Gets one value of data after specified quarter.
//
// Throws: PiaException of type PIA_IDS_FLOATQTR1 if year is out of range,
//   or of type PIA_IDS_FLOATQTR3 if quarter is out of range
//   (only in debug mode).
//
// Returns: One value of data after specified quarter.
//
// Arguments:
//   qtrYear: Year and quarter (0-3) after which data is desired.
double DoubleQuarter::getAfter( const QtrYear& qtrYear ) const
{
#ifndef NDEBUG
   if (static_cast<int>(qtrYear.getYear()) < baseYear ||
       static_cast<int>(qtrYear.getYear()) > lastYear)
      throw PiaException(PIA_IDS_FLOATQTR1);
   if (qtrYear.getQuarter() < 0u || qtrYear.getQuarter() > 3u)
      throw PiaException(PIA_IDS_FLOATQTR3);
#endif
   if (static_cast<int>(qtrYear.getYear()) >= lastYear &&
      qtrYear.getQuarter() >= 3u) {
      return theData[4 * (qtrYear.getYear() - baseYear) + qtrYear.getQuarter()];
   }
   else {
      return theData[4 * (qtrYear.getYear() - baseYear) + qtrYear.getQuarter() +
         1];
   }
}

// Description: Gets one value of data before specified quarter.
//
// Throws: PiaException of type PIA_IDS_FLOATQTR1 if year is out of range,
//   or of type PIA_IDS_FLOATQTR3 if quarter is out of range
//   (only in debug mode).
//
// Returns: One value of data before specified quarter.
//
// Arguments:
//   qtrYear: Year and quarter (0-3) before which data is desired.
double DoubleQuarter::getBefore( const QtrYear& qtrYear ) const
{
#ifndef NDEBUG
   if (static_cast<int>(qtrYear.getYear()) < baseYear ||
       static_cast<int>(qtrYear.getYear()) > baseYear)
      throw PiaException(PIA_IDS_FLOATQTR1);
   if (qtrYear.getQuarter() < 0u || qtrYear.getQuarter() > 3u)
      throw PiaException(PIA_IDS_FLOATQTR3);
#endif
   if (static_cast<int>(qtrYear.getYear()) <= baseYear &&
      qtrYear.getQuarter() <= 0u) {
      return theData[4 * (qtrYear.getYear() - baseYear) + qtrYear.getQuarter()];
   }
   else {
      return theData[4 * (qtrYear.getYear() - baseYear) + qtrYear.getQuarter()
         - 1];
   }
}

// Description: Initializes data for specified years.
//
// Arguments:
//   newData: New value of data to be used for all years.
//   firstYeart: First year of data to set.
//   lastYeart: Last year of data to set.
void DoubleQuarter::assign( double newData, int firstYeart, int lastYeart )
{
   for (int year = firstYeart; year <= lastYeart; year++)
      for (int quarter = 0; quarter < 4; quarter++)
         theData[4 * (year - baseYear) + quarter] = newData;
}

// Arguments:
//   doubleQuarter: Array of new quarterly values of data.
//   firstYeart: First year of data to set.
//   lastYeart: Last year of data to set.
void DoubleQuarter::assign( const DoubleQuarter& doubleQuarter, int firstYeart,
   int lastYeart )
{
   for (int year = firstYeart; year <= lastYeart; year++)
      for (int quarter = 0; quarter < 4; quarter++)
         theData[4 * (year - baseYear) + quarter] =
            doubleQuarter.getAt(QtrYear(quarter, year));
}

// Arguments:
//   doubleMonth: Array of monthly values. The three values in each quarter are
//     averaged.
//   firstQtrYear: First year and quarter of data to set.
//   lastQtrYear: Last year and quarter of data to set.
void DoubleQuarter::assign( const DoubleMonth& doubleMonth,
const QtrYear& firstQtrYear, const QtrYear& lastQtrYear )
{
   unsigned j;  // loop counter
   unsigned i;  // loop counter

   // Handle first year
   i = firstQtrYear.getYear();
   for (j = firstQtrYear.getQuarter(); j < 4u; j++) {
      setAt(QtrYear(j, i), doubleMonth.qaverage(QtrYear(j, i)));
   }
   // Handle full years
   for (i = firstQtrYear.getYear() + 1u; i < lastQtrYear.getYear(); i++) {
      for (j = 0u; j < 4u; j++) {
         setAt(QtrYear(j, i), doubleMonth.qaverage(QtrYear(j, i)));
      }
   }
   // Handle last year
   i = lastQtrYear.getYear();
   for (j = 0u; j <= lastQtrYear.getQuarter(); j++) {
      setAt(QtrYear(j, i), doubleMonth.qaverage(QtrYear(j, i)));
   }
}

// Description: Multiplies data by another DoubleQuarter.
//
// Arguments:
//   doubleQuarter: Array of multipliers.
//   firstYeart: First year of data to multiply.
//   lastYeart: Last year of data to multiply.
void DoubleQuarter::multiply( const DoubleQuarter& doubleQuarter, int firstYeart,
   int lastYeart )
{
   for (int year = firstYeart; year <= lastYeart; year++)
      for (int quarter = 0; quarter < 4; quarter++)
         theData[4 * (year - baseYear) + quarter] *=
            doubleQuarter.getAt(QtrYear(quarter, year));
}

// Description: Calculates annual average.
//
// Throws: PiaException of type PIA_IDS_FLOATQTR1 if year is out of range
//   (only in debug mode).
//
// Returns: Annual average.
//
// Arguments:
//   year: Year for which average is desired.
double DoubleQuarter::average( int year ) const
{
#ifndef NDEBUG
   if (year < baseYear || year > lastYear)
      throw PiaException(PIA_IDS_FLOATQTR1);
#endif
   return((getAt(QtrYear(0, year)) + getAt(QtrYear(1, year)) +
      getAt(QtrYear(2, year)) + getAt(QtrYear(3, year))) / 4.f);
}

// Description: Calculates annual percentage increase.
//
// Throws: PiaException of type PIA_IDS_FLOATQTR1 if year is out of range,
//   or of type PIA_IDS_FLOATQTR3 if quarter is out of range, or of type
//   PIA_IDS_FLOATQTR4 if value is zero in prior year (only in debug mode).
//
// Returns: Annual percentage increase.
//
// Arguments:
//   qtrYear: Year and quarter (0-3) for which percentage increase is desired.
double DoubleQuarter::anninc( const QtrYear& qtrYear ) const
{
#ifndef NDEBUG
   if (static_cast<int>(qtrYear.getYear()) < baseYear ||
       static_cast<int>(qtrYear.getYear()) > lastYear)
      throw PiaException(PIA_IDS_FLOATQTR1);
   if (qtrYear.getQuarter() < 0u || qtrYear.getQuarter() > 3u)
      throw PiaException(PIA_IDS_FLOATQTR3);
   if (getAt(QtrYear(qtrYear.getQuarter(), qtrYear.getYear() - 1)) < .001)
      throw PiaException(PIA_IDS_FLOATQTR4);
#endif
   return(100.f * (getAt(qtrYear) /
      getAt(QtrYear(qtrYear.getQuarter(), qtrYear.getYear() - 1)) - 1.f));
}

// Description: Calculates 3-year percentage increase at annual rate.
//
// Throws: PiaException of type PIA_IDS_FLOATQTR1 if year is out of range,
//   or of type PIA_IDS_FLOATQTR3 if quarter is out of range, or of type
//   PIA_IDS_FLOATQTR4 if value is zero in third prior year (only in debug
//   mode).
//
// Returns: 3-year percentage increase at annual rate.
//
// Arguments:
//   qtrYear: Year and quarter (0-3) for which percentage increase is desired.
double DoubleQuarter::anninc3( const QtrYear& qtrYear ) const
{
#ifndef NDEBUG
   if (static_cast<int>(qtrYear.getYear()) < baseYear + 3 ||
       static_cast<int>(qtrYear.getYear()) > lastYear)
      throw PiaException(PIA_IDS_FLOATQTR1);
   if (qtrYear.getQuarter() < 0u || qtrYear.getQuarter() > 3u)
      throw PiaException(PIA_IDS_FLOATQTR3);
   if (getAt(QtrYear(qtrYear.getQuarter(), qtrYear.getYear() - 3)) < .001)
      throw PiaException(PIA_IDS_FLOATQTR4);
#endif
   return((double)(100. * (pow((double)getAt(qtrYear) /
      (double)getAt(QtrYear(qtrYear.getQuarter(), qtrYear.getYear() - 3)),
      0.333333) - 1.)));
}

// Description: Reads array from an open stream.
//
// Throws: PiaException of type PIA_IDS_FLOATQTR2 if unable to read data for
//   all specified years.
//
// Arguments:
//   in: Input stream.
//   firstYeart: First year of data to read.
//   lastYeart: Last year of data to read.
void DoubleQuarter::read( std::istream& in, int firstYeart, int lastYeart )
{
   double temp;  // temporary value

   for (int i = firstYeart; i <= lastYeart; i++) {
      for (int quarter = 0; quarter < 4; quarter++) {
         if (!(in >> temp))
            throw PiaException(PIA_IDS_FLOATQTR2);
         setAt(QtrYear(quarter, i), temp);
      }
   }
}

// Description: Writes array to an open stream.
//
// Arguments:
//   out: Output stream.
//   firstYeart: First year of data to write.
//   lastYeart: Last year of data to write.
void DoubleQuarter::write( std::ostream& out, int firstYeart, int lastYeart )
{
   int numperline = lineWidth / width;  // number of numbers per line
   out.precision(precision);
   for (int i = firstYeart; i <= lastYeart; i++)  {
      for (int quarter = 0; quarter < 4; quarter++) {
         out << setw(width) << getAt(QtrYear(quarter, i));
         if ((4 * (i - firstYeart) + quarter) % numperline == numperline - 1)
            out << endl;
      }
   }
   if ((4 * (lastYeart - firstYeart) + 3) % numperline != numperline - 1) {
      out << endl;
   }
}
