// $Id: intann.cpp 1.23 2007/01/09 14:36:33EST 044579 Development  $
//
// Functions to handle IntAnnual arrays.

#include <fstream>
#include <iomanip>
#include <algorithm>
#if !defined(SHORTFILENAMES)
#include "intann.h"
#else
#include "intannh"
#endif

using namespace std;

// Description: Constructor.
//
// Remarks: The version with no arguments sets the range of years to an
// impossible combination. They must be reset with the resetYears function
// before an instance created with the default constructor is usable.
//
// The version with two int arguments sets the baseYear,
// and initializes the size of the data vector to the span of years. The
// version with an IntAnnual argument copies all data from that argument.
IntAnnual::IntAnnual() :
theData(), width(9), baseYear(0), lineWidth(75)
{ }

// Arguments:
//   newBaseYear: New base year.
//   newLastYear: New last year.
IntAnnual::IntAnnual( int newBaseYear, int newLastYear ) :
theData(newLastYear - newBaseYear + 1), width(9),
baseYear(newBaseYear), lineWidth(75)
{ }

// Arguments:
//   intAnnual: The IntAnnual to copy from.
IntAnnual::IntAnnual( const IntAnnual& intAnnual ) :
theData(intAnnual.theData), width(intAnnual.width),
baseYear(intAnnual.baseYear), lineWidth(intAnnual.lineWidth)
{ }

// Description: Zeroes out all data values.
void IntAnnual::deleteContents()
{
   fill(theData.begin(), theData.end(), 0);
}

// Description: Copies from another IntAnnual.
//
// Arguments:
//   intAnnual: The IntAnnual to copy from.
IntAnnual& IntAnnual::operator=( const IntAnnual& intAnnual )
{
   theData = intAnnual.theData;
   width = intAnnual.width;
   baseYear = intAnnual.baseYear;
   lineWidth = intAnnual.lineWidth;
   return *this;
}

// Description: Resets the range of years.
//
// Remarks: This function needs to be called if the instance was created with
// the default constructor. If it is called again, or if the constructor with
// a range of years was used, this function clears all values in the vector.
//
// Arguments:
//   newBaseYear: New base year.
//   newLastYear: New last year.
void IntAnnual::resetYears( int newBaseYear, int newLastYear )
{
   theData.clear();
   theData.resize(newLastYear - newBaseYear + 1);
   baseYear = newBaseYear;
}

// Description: Sets data for specified years, or for all years if no years are
//   specified.
//
// Arguments:
//   intAnnual: Array of new values of data.
//   firstYeart: First year of data to set.
//   lastYeart: Last year of data to set.
void IntAnnual::assign( const IntAnnual& intAnnual, int firstYeart,
int lastYeart )
{
   int year1 = max(baseYear, intAnnual.baseYear);
   year1 = max(year1, firstYeart);
   int year2 = min(getLastYear(), intAnnual.getLastYear());
   year2 = min(year2, lastYeart);
   copy(intAnnual.theData.begin() + year1 - intAnnual.baseYear,
      intAnnual.theData.begin() + year2 + 1 - intAnnual.baseYear,
      theData.begin() + year1 - baseYear);
}

// Arguments:
//   newData: New value of data to be used for all years.
void IntAnnual::assign( int newData )
{
   fill(theData.begin(), theData.end(), newData);
}

// Arguments:
//   newData: New value of data to be used for all years.
//   firstYeart: First year of data to set.
//   lastYeart: Last year of data to set.
void IntAnnual::assign( int newData, int firstYeart, int lastYeart )
{
   int year1 = max(baseYear, firstYeart);
   int year2 = min(getLastYear(), lastYeart);
   fill(theData.begin() + year1 - baseYear,
      theData.begin() + year2 + 1 - baseYear, newData);
}

// Description: Reads array from an open stream.
//
// Throws: PiaException of type PIA_IDS_INTANN2 if unable to read specified
//   fields.
//
// Arguments:
//   in: Input stream.
//   firstYeart: First year of data to read.
//   lastYeart: Last year of data to read.
void IntAnnual::read( std::istream& in, int firstYeart, int lastYeart )
{
   int temp;  // temporary value

   for (int i = firstYeart; i <= lastYeart; i++)  {
      if (!(in >> temp))
         throw PiaException(PIA_IDS_INTANN2);
      theData[i - baseYear] = temp;
   }
}

// Description: Writes array to an open stream.
//
// Arguments:
//   out: Output stream.
//   firstYeart: First year of data to write.
//   lastYeart: Last year of data to write.
void IntAnnual::write( std::ostream& out, int firstYeart, int lastYeart )
{
   int numperline = lineWidth / width;  // number of numbers per line
   for (int i = firstYeart; i <= lastYeart; i++)  {
      out << setw(width) << theData[i - baseYear];
      if ((i - firstYeart) % numperline == numperline - 1)
         out << endl;
   }
   if ((firstYeart - lastYeart + 1) % numperline != 0)
      out << endl;
}

// Description: Adds an IntAnnual to this one.
//
// Arguments:
//   vector1: Vector of terms to add.
//   firstYeart: First year of data to sum.
//   lastYeart: Last year of data to sum.
void IntAnnual::add( const IntAnnual& vector1, int firstYeart,
int lastYeart )
{
   for (int i = firstYeart; i <= lastYeart; i++) {
      theData[i - baseYear] += vector1[i];
   }
}

// Description: Subtracts an IntAnnual from this one.
//
// Arguments:
//   vector1: Vector of terms to subtract.
//   firstYeart: First year of data to subtract.
//   lastYeart: Last year of data to subtract.
void IntAnnual::subtract( const IntAnnual& vector1, int firstYeart,
int lastYeart )
{
   for (int i = firstYeart; i <= lastYeart; i++) {
      theData[i - baseYear] -= vector1[i];
   }
}

// Description: Divides this IntAnnual by another one.
//
// Arguments:
//   vector1: Vector of terms to divide by.
//   firstYeart: First year of data to divide.
//   lastYeart: Last year of data to divide.
void IntAnnual::divide( const IntAnnual& vector1, int firstYeart,
int lastYeart )
{
   for (int i = firstYeart; i <= lastYeart; i++) {
      theData[i - baseYear] /= vector1[i];
   }
}

// Description: Multiplies this IntAnnual by another one.
//
// Arguments:
//   vector1: Vector of terms to multiply by.
//   firstYeart: First year of data to multiply.
//   lastYeart: Last year of data to multiply.
void IntAnnual::multiply( const IntAnnual& vector1, int firstYeart,
int lastYeart )
{
   for (int i = firstYeart; i <= lastYeart; i++) {
      theData[i - baseYear] *= vector1[i];
   }
}
