// $Id: qtryear.cpp 1.20 2007/02/28 15:05:27EST 044579 Development  $
//
// Functions to handle year and quarter.

#include <sstream>
#include "qtryear.h"
#include "datemoyr.h"
#include "Resource.h"
#include "PiaException.h"

using namespace std;

// Arguments:
//   dateMoyr: The date to use. Quarter is set to 0 for months 0-2, 1 for
//   months 3-5, 2 for months 6-8, or 3 for months 9-11.
QtrYear::QtrYear( const DateMoyr& dateMoyr ) :
quarterOfYear(static_cast<unsigned short>((dateMoyr.getMonth() -
boost::date_time::Jan) / 3)),
yearPart(static_cast<unsigned short>(dateMoyr.getYear()))
{ }

// Arguments:
//   dateModyyr: The date to use. Quarter is set to 0 for months 0-2, 1 for
//   months 3-5, 2 for months 6-8, or 3 for months 9-11.
QtrYear::QtrYear( const boost::gregorian::date& dateModyyr ) :
quarterOfYear(static_cast<unsigned short>((dateModyyr.month() -
boost::date_time::Jan) / 3)),
yearPart(static_cast<unsigned short>(dateModyyr.year()))
{ }

// Description: Sets quarter and year from another object.
//
// Remarks: The other object can be another Qtryear, or a date with months and
// years.
//
// Arguments:
//   qtrYear: The other QtrYear to use.
QtrYear& QtrYear::operator=( const QtrYear& qtrYear )
{
   quarterOfYear = qtrYear.quarterOfYear;
   yearPart = qtrYear.yearPart;
   return *this;
}

// Arguments:
//   date: The date to use.
QtrYear& QtrYear::operator=( const DateMoyr& date )
{
   quarterOfYear = static_cast<unsigned short>((date.getMonth() -
      boost::date_time::Jan) / 3);
   yearPart = static_cast<unsigned short>(date.getYear());
   return *this;
}

// Arguments:
//   date: The date to use.
QtrYear& QtrYear::operator=( const boost::gregorian::date& date )
{
   quarterOfYear = static_cast<unsigned short>((date.month() -
      boost::date_time::Jan) / 3);
   yearPart = static_cast<unsigned short>(date.year());
   return *this;
}

// Description: Adds a number of quarters or a (quarter,year) date to this one.
//
// Arguments:
//   addQuarter: Number of quarters to add.
void QtrYear::add( unsigned addQuarter )
{
   quarterOfYear = static_cast<unsigned short>(quarterOfYear + addQuarter);
   if (quarterOfYear > 3u) {
      const unsigned temp = quarterOfYear / 4u;
      quarterOfYear = static_cast<unsigned short>(quarterOfYear - 4u * temp);
      yearPart = static_cast<unsigned short>(yearPart + temp);
   }
}

// Arguments:
//   addQtrYear: (quarter,year) date to add.
void QtrYear::add( const QtrYear& addQtrYear )
{
   quarterOfYear = static_cast<unsigned short>(quarterOfYear + addQtrYear.quarterOfYear);
   if (quarterOfYear > 3u) {
      const unsigned temp = quarterOfYear / 4u;
      quarterOfYear = static_cast<unsigned short>(quarterOfYear - 4u * temp);
      yearPart = static_cast<unsigned short>(yearPart + temp);
   }
   yearPart = static_cast<unsigned short>(yearPart + addQtrYear.yearPart);
}

// Description: Subtracts a number of quarters or a (quarter,year) date from
//   this one.
//
// Arguments:
//   subQuarter: Number of quarters to subtract.
void QtrYear::subtract( unsigned subQuarter )
{
   if (quarterOfYear >= subQuarter) {
      quarterOfYear = static_cast<unsigned short>(quarterOfYear - subQuarter);
   }
   else {
      const unsigned temp = (subQuarter + 3u - quarterOfYear) / 4u;
      quarterOfYear = static_cast<unsigned short>(quarterOfYear + 4u * temp - subQuarter);
      if (yearPart >= temp) {
         yearPart = static_cast<unsigned short>(yearPart - temp);
      }
      else {
         throw PiaException(PIA_IDS_QTRYRQTR);
      }
   }
}

// Arguments:
//   subQtrYear: (quarters,year) date to subtract.
void QtrYear::subtract( const QtrYear& subQtrYear )
{
   const unsigned subQuarter = subQtrYear.quarterOfYear;
   if (quarterOfYear >= subQuarter) {
      quarterOfYear = static_cast<unsigned short>(quarterOfYear - subQuarter);
   }
   else {
      const unsigned temp = (subQuarter + 3u - quarterOfYear) / 4u;
      quarterOfYear =
         static_cast<unsigned short>(quarterOfYear + 4u * temp - subQuarter);
      if (yearPart >= temp) {
         yearPart = static_cast<unsigned short>(yearPart - temp);
      }
      else {
         throw PiaException(PIA_IDS_QTRYRQTR);
      }
   }
   if (yearPart >= subQtrYear.yearPart) {
      yearPart = static_cast<unsigned short>(yearPart - subQtrYear.yearPart);
   }
   else {
      throw PiaException(PIA_IDS_QTRYRQTR);
   }
}

// Description: Compares this date to another one.
//
// Returns: True if this date is before argument, false otherwise.
//
// Arguments:
//   qtrYear: Date to compare to.
bool QtrYear::operator<( const QtrYear& qtrYear ) const
{
   // check year
   if (yearPart != qtrYear.yearPart)
      return(yearPart < qtrYear.yearPart);
   // same year, so check quarter
   return(quarterOfYear < qtrYear.quarterOfYear);
}

// Description: Checks for quarter in bounds.
//
// Throws: PiaException of type PIA_IDS_QTRYRQTR if quarter is greater than 3.
void QtrYear::qtrCheck() const
{
   if (quarterOfYear > 3u)
      throw PiaException(PIA_IDS_QTRYRQTR);
}

// Description: Checks for year in bounds.
//
// Throws: PiaException of type PIA_IDS_QTRYRYR if year is greater than
// Date::highyear.
void QtrYear::yearCheck() const
{
   if (yearPart > Date::highyear)
      throw PiaException(PIA_IDS_QTRYRYR);
}

// Description: Returns string representation of quarter and year.
//
// Returns: String representation of quarter and year, for example
//   "qtr 3 of 1999".
std::string QtrYear::toString() const
{
   ostringstream strm;
   strm << "qtr " << quarterOfYear + 1 << " of " << yearPart;
   return strm.str();
}

// Description: Calculates difference of two quarter-year periods as an integer
//   number of quarters.
//
// Returns: Difference as an integer number of quarters. Result is negative if
//   second period is before first period.
//
// Arguments:
//   qtrYear1: First period.
//   qtrYear2: Second period.
int QtrYear::diff( const QtrYear& qtrYear1, const QtrYear& qtrYear2 )
{
   int rv = static_cast<int>(qtrYear2.getQuarter()) -
      static_cast<int>(qtrYear1.getQuarter());
   rv += 4 * (static_cast<int>(qtrYear2.getYear()) -
      static_cast<int>(qtrYear1.getYear()));
   return rv;
}

// Converts this quarter and year to a month and year.
//
// Returns: The converted month and year.
DateMoyr QtrYear::toDateMoyr() const
{
   return DateMoyr(quarterOfYear * 3 + boost::date_time::Jan, yearPart);
}
