// $Id: ssn.cpp 1.16 2006/11/08 11:03:48EST 044579 Development  $
//
// Functions to handle Social Security number.

#include <stdexcept>
#include "ssn.h"
#include "PiaException.h"
#include "Resource.h"

using namespace std;

// Description: Creates ssn.
//
// Remarks: The default constructor initializes all fields with zeroes.
Ssn::Ssn () : ssnFull("000000000"), ssnWithHyphens("000-00-0000")
{
   ssnPart[0] = "000";
   ssnPart[1] = "00";
   ssnPart[2] = "0000";
}

// Arguments:
//   newSsnPart1: First part of ssn.
//   newSsnPart2: Second part of ssn.
//   newSsnPart3: Third part of ssn.
Ssn::Ssn( const std::string& newSsnPart1, const std::string& newSsnPart2,
const std::string& newSsnPart3 )
{
   setSsnFull(newSsnPart1, newSsnPart2, newSsnPart3);
}

// Arguments:
//   newSsn: String to use for full ssn (without hyphens).
Ssn::Ssn( const std::string& newSsn )
{
   string temp;
   if (newSsn.length() < 9) {
      temp.append(9 - newSsn.length(), '0');
      temp += newSsn;
   }
   else {
      temp = newSsn;
   }
   ssnFull = temp;
   try {
      ssnPart[0] = temp.substr(0,3);
      ssnPart[1] = temp.substr(3, 2);
      ssnPart[2] = temp.substr(5, 4);
   // if substr argument is out of range, catch the out_of_range exception
   } catch (out_of_range&) {
      throw PiaException(PIA_IDS_SSN1);
   }
}

// Arguments:
//   ssn: The Ssn to copy from.
Ssn::Ssn( const Ssn& ssn ) : ssnFull(ssn.ssnFull),
ssnWithHyphens(ssn.ssnWithHyphens)
{
   for (int i = 0; i < 3; i++)
      ssnPart[i] = ssn.ssnPart[i];
}

// Description: Copies from another Ssn.
//
// Arguments:
//   ssn: The Ssn to copy from.
Ssn& Ssn::operator=( const Ssn& ssn )
{
   for (int i = 0; i < 3; i++)
      ssnPart[i] = ssn.ssnPart[i];
   ssnFull = ssn.ssnFull;
   ssnWithHyphens = ssn.ssnWithHyphens;
   return *this;
}

// Description: Sets first part of ssn.
//
// Arguments:
//   newPart1: New first part of ssn.
void Ssn::setPart1( const std::string& newPart1 )
{
   string temp;
   if (newPart1.length() < 3) {
      temp.append(3 - newPart1.length(), '0');
      temp += newPart1;
   }
   else {
      temp = newPart1;
   }
   ssnPart[0] = temp.substr(0,3);
   update();
}

// Description: Sets second part of ssn.
//
// Arguments:
//   newPart2: New second part of ssn.
void Ssn::setPart2( const std::string& newPart2 )
{
   string temp;
   if (newPart2.length() < 2) {
      temp.append(2 - newPart2.length(), '0');
      temp += newPart2;
   }
   else {
      temp = newPart2;
   }
   ssnPart[1] = temp.substr(0,2);
   update();
}

// Description: Sets third part of ssn.
//
// Arguments:
//   newPart3: New third part of ssn.
void Ssn::setPart3( const std::string& newPart3 )
{
   string temp;
   if (newPart3.length() < 4) {
      temp.append(4 - newPart3.length(), '0');
      temp += newPart3;
   }
   else {
      temp = newPart3;
   }
   ssnPart[2] = temp.substr(0,4);
   update();
}

// Description: Updates full ssn from three parts.
void Ssn::update ()
{
   ssnFull = ssnPart[0].substr(0,3);
   ssnFull += ssnPart[1].substr(0,2);
   ssnFull += ssnPart[2].substr(0,4);
   ssnWithHyphens = ssnPart[0].substr(0,3);
   ssnWithHyphens += "-";
   ssnWithHyphens += ssnPart[1].substr(0,2);
   ssnWithHyphens += "-";
   ssnWithHyphens += ssnPart[2].substr(0,4);
}

// Description: Returns formatted ssn.
//
// Returns: Formatted ssn.
std::string Ssn::toString() const
{
   return(ssnWithHyphens);
}

// Arguments:
//   format: Format to use.  If none used, formatted with hyphens separating
//     the three parts.
//
//     @table
//     value  meaning
//     "n"    do not use hyphens.
//     other  use hyphens.
std::string Ssn::toString( const std::string& format ) const
{
   if (format.length() < 1)
      return ssnWithHyphens;
   switch (format[0]) {
   case 'n':
      return ssnFull;
   default:
      return ssnWithHyphens;
   }
}

// Description: Sets the full ssn and the three parts.
//
// Remarks: If one string is used, the full ssn is set from that, then the
//   three parts are parsed.  If three strings are used, the three parts are
//   set, then the full ssn is set from the concatenated parts.
//
// Arguments:
//   newSsnPart1: First part of ssn.
//   newSsnPart2: Second part of ssn.
//   newSsnPart3: Third part of ssn.
void Ssn::setSsnFull( const std::string& newSsnPart1,
const std::string& newSsnPart2, const std::string& newSsnPart3 )
{
   string temp;
   if (newSsnPart1.length() < 3) {
      temp.append(3 - newSsnPart1.length(), '0');
      temp += newSsnPart1;
   }
   else {
      temp = newSsnPart1;
   }
   ssnPart[0] = temp.substr(0,3);
   if (newSsnPart2.length() < 2) {
      temp = "";
      temp.append(2 - newSsnPart2.length(), '0');
      temp += newSsnPart2;
   }
   else {
      temp = newSsnPart2;
   }
   ssnPart[1] = temp.substr(0,2);
   if (newSsnPart3.length() < 4) {
      temp = "";
      temp.append(4 - newSsnPart3.length(), '0');
      temp += newSsnPart3;
   }
   else {
      temp = newSsnPart3;
   }
   ssnPart[2] = temp.substr(0,4);
   update();
}

// Throws: PiaException of type PIA_IDS_SSN1 if there any string-handling
//   problems.
//
// Arguments:
//   newSsnFull: String to use for full ssn (without hyphens).
void Ssn::setSsnFull( const std::string& newSsnFull )
{
   string temp;
   if (newSsnFull.length() < 9) {
      temp.append(9 - newSsnFull.length(), '0');
      temp += newSsnFull;
   }
   else {
      temp = newSsnFull;
   }
   try {
      ssnPart[0] = temp.substr(0,3);
      ssnPart[1] = temp.substr(3, 2);
      ssnPart[2] = temp.substr(5, 4);
   // if substr argument is out of range, catch the out_of_range exception
   } catch (out_of_range&) {
      throw PiaException(PIA_IDS_SSN1);
   }
   update();
}

// Description: Compares to another ssn.
//
// Returns: True if this ssn is less than the argument ssn, false otherwise.
//
// Arguments:
//   ssntemp: Ssn to compare to.
bool Ssn::operator<( const Ssn& ssntemp ) const
{
   return (ssnFull < ssntemp.ssnFull);
}

// Description: Compares to another ssn.
//
// Returns: True if this ssn is equal to the argument ssn, false otherwise.
//
// Arguments:
//   ssntemp: Ssn to compare to.
bool Ssn::operator==( const Ssn& ssntemp ) const
{
   return (ssnFull == ssntemp.ssnFull);
}

// Description: Zeroes the three parts.
void Ssn::deleteContents()
{
   ssnPart[0] = "000";
   ssnPart[1] = "00";
   ssnPart[2] = "0000";
   update();
}

// Description: Returns ssn converted to a number.
//
// Returns: The ssn converted to a number. It could be as large as 999999999.
long Ssn::toNumber() const
{
   return atoi(ssnPart[0].c_str()) * 1000000 +
      atoi(ssnPart[1].c_str()) * 10000 + atoi(ssnPart[2].c_str());
}
