// $Id: Bic.cpp 1.13 2002/06/07 12:21:10EDT sfmckay Exp  $
//
// Functions to handle bic (Beneficiary Identification Code).

#include "bic.h"
#include "PiaException.h"
#include "Resource.h"

using namespace std;

// Type of survivor titles.
const string Bic::survivorType[6] = {
   "child survivor (minor, student, or disabled)",
   "mother/father (parent of child)",
   "disabled widow(er) (aged 50-59)",
   "divorced disabled widow(er) (aged 50-59)",
   "aged widow(er) (aged 60 or over)",
   "divorced aged widow(er) (aged 60 or over)"
};

// Type of life auxiliary titles.
const string Bic::lifeAuxType[4] = {
   "child life (minor, student, or disabled)",
   "young wife/husband (parent of child)",
   "aged wife/husband (aged 62 or over)",
   "divorced aged wife/husband (aged 62 or over)"
};

// Description: Initializes codes.
//
// Remarks: The default version initializes the codes to blanks.
Bic::Bic() : majorBic(' '), minorBic(' ')
{ }

// Arguments:
//   newCode: New bic codes as a string.
Bic::Bic( const std::string& newCode ) :
majorBic(newCode[0]), minorBic(newCode[1])
{ }

// Arguments:
//   newMajorBic: New major bic code.
//   newMinorBic: New minor bic code.
Bic::Bic( char newMajorBic, char newMinorBic ) :
majorBic(newMajorBic), minorBic(newMinorBic)
{ }

// Description: Sets the bic codes.
//
// Arguments:
//   newCode: New bic codes as a string.
void Bic::set( const std::string& newCode )
{
   majorBic = newCode[0];
   minorBic = newCode[1];
}

// Description: Returns the bic code as a string.
//
// Returns: The bic code as a string.
std::string Bic::toString() const
{
   string str;
   str += majorBic;
   str += minorBic;
   return str;
}

// Description: Checks for valid survivor bic.
//
// Throws: PiaException of type PIA_IDS_JSURV if bic is not a valid survivor
//   bic.
//
// Arguments:
//   newMajorBic: Bic to check.
void Bic::survivorCheck( char newMajorBic )
{
   if (newMajorBic != 'C' && newMajorBic != 'D' && newMajorBic != 'E' &&
      newMajorBic != 'W')
      throw PiaException(PIA_IDS_JSURV);
}

// Description: Returns description of type of survivor.
//
// Returns: Description of type of survivor (empty string if bic is not a
//   survivor).
std::string Bic::getSurvivorType() const
{
   switch (getMajorBic()) {
   case 'C':
      return survivorType[0];
   case 'D':
      if (getMinorBic() == ' ') {
         return survivorType[4];
      }
      else if (getMinorBic() == '6') {
         return survivorType[5];
      }
      else {
         return string();
      }
   case 'E':
      return survivorType[1];
   case 'W':
      if (getMinorBic() == ' ') {
         return survivorType[2];
      }
      else if (getMinorBic() == '6') {
         return survivorType[3];
      }
      else {
         return string();
      }
   default:
      return string();
   }
}

// Description: Returns description of type of life auxiliary.
//
// Returns: Description of type of life auxiliary (empty string if bic is not
//   a life auxiliary).
std::string Bic::getLifeAuxType() const
{
   switch (getMajorBic()) {
   case 'C':
      return lifeAuxType[0];
   case 'B':
      if (getMinorBic() == ' ') {
         return lifeAuxType[2];
      }
      else if (getMinorBic() == '2') {
         return lifeAuxType[1];
      }
      else if (getMinorBic() == '6') {
         return lifeAuxType[3];
      }
      else {
         return string();
      }
   default:
      return string();
   }
}
