// $Id: ChildCareCalc.cpp 1.29 2007/02/28 15:06:13EST 044579 Development  $
//
// Functions to calculate a wage-indexed PIA with childcare dropout years.

#include "ChildCareCalc.h"
#include "UserAssumptions.h"
#include "piaparms.h"
#include "DebugCase.h"
#include "oactcnst.h"
#if defined(DEBUGCASE)
#include "Trace.h"
#include "comma.h"
#endif

using namespace std;
#if !defined(__SGI_STL_INTERNAL_RELOPS)
using namespace std::rel_ops;
#endif

// Description: Initializes a ChildCareCalc instance.
//
// Arguments:
//   newWorkerData: Worker data.
//   newPiaData: Pia calculation data.
//   newPiaParams: Pia calculation parameters.
//   newMaxyear: Maximum year of projection.
ChildCareCalc::ChildCareCalc( const WorkerDataGeneral& newWorkerData,
const PiaData& newPiaData, const PiaParams& newPiaParams, int newMaxyear ) :
WageIndGeneral(newWorkerData, newPiaData, newPiaParams, newMaxyear,
"Child-Care Dropout Year Method", CHILD_CARE), adjustedN(0), 
childCareDropMax(0), childCareDrop(0)
{ }

// Description: Destructor.
ChildCareCalc::~ChildCareCalc()
{ }

// Description: Initializes computed values to zero.
void ChildCareCalc::initialize()
{
   WageIndGeneral::initialize();
   adjustedN = childCareDropMax = childCareDrop = 0;
}

// Description: Determines applicability of method.
//
// Remarks: The version with no arguments calls the static version with 3
//   arguments.
//
// This method is applicable if worker is eligible after 1978, it is not a
// totalization benefit, and it is an old-age case with prior disability
// entitlement in June 1980 or later that lasted to age 61, or a disability
// case with entitlement in June 1980 or later.
//
// Returns: True if this method is applicable, false otherwise.
bool ChildCareCalc::isApplicable()
{
   if (isApplicable(workerData, piaData, ioasdi)) {
      iapp = APPLICABLE;
      return(true);
   }
   return(false);
}

// Arguments:
//   workerData: Worker data.
//   piaData: Pia calculation data.
//   ioasdi: Type of benefit.
bool ChildCareCalc::isApplicable( const WorkerDataGeneral& workerData,
const PiaData& piaData, WorkerDataGeneral::ben_type ioasdi )
{
   const DisabPeriod& disabPeriod = workerData.disabPeriod[0];
   if (piaData.getEligYear() >= YEAR79 && workerData.getIend() > YEAR50 &&
      !workerData.getTotalize() && workerData.childCareYears.any() &&
      ((ioasdi == WorkerDataGeneral::OLD_AGE && workerData.getValdi() > 0 &&
      disabPeriod.getEntDate() >= PiaParams::amend80 &&
      disabPeriod.getCessationDate() >=
      DateMoyr(workerData.getBirthDate().month(),
      workerData.getBirthDate().year() + 61))
      || (workerData.getEntDate() >= PiaParams::amend80 &&
      ioasdi == WorkerDataGeneral::DISABILITY))) {
      // wage-indexed PIA with childcare dropout years
      return(true);
   }
   else
      return(false);
}

// Description: Computes wage indexed PIA with childcare dropout years.
void ChildCareCalc::calculate()
{
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": Starting ChildCareCalc::calculate";
      Trace::writeLine(s);
   }
#endif
   setTableNum(PL_1977);
   const int i1 = piaData.getEarn50(PiaData::EARN_WITH_TOTALIZATION);
   const int i2 = piaData.getEarnYear();
   indexEarnings(i1, piaData.getEligYear() - 2, i2, piaData.earnOasdiLimited,
      earnMultiplied, earnIndexed, piaParams.getFqIndex());
   const int i3 = piaData.compPeriodNew.getN();
   orderEarnings(i1, i2, i3);
   // find childcare dropout years
   childCareDropoutCal();
   // adjust number of years by number of childcare dropout years
   adjustedN = i3 - childCareDrop;
   totalEarnCal(i1, i2, adjustedN);
   const int i5 = piaData.getEligYear();
   // project applicable bend points from 1979 bend points
   bendPointCal(i5, bendPia);
   // find portion of AIME in each AIME interval
   setPortionAime(ame, portionAime, bendPia);
   // set pia formula percentages
   piaParams.percPiaCal(i5, percPia);
   const int i4 = piaData.getEligYear() - 1;
   yearCpi[FIRST_YEAR] = i4;
   yearCpi[YEAR_ELIG] = i5;
   piaElig[i4] = aimepiaCal(portionAime, percPia, i4);
   // apply windfall elimination provision
   if (wep_app())
      windfallCal();
   // apply benefit increases to PIA
   setYearCpi();
   piaEnt.set(applyColas(piaElig, i5, workerData.getBenefitDate()));
   bendMfb.project(i5, piaParams.getFqArray());
   // calculate the AIME MFB
   setPortionPiaElig(piaElig[i4], portionPiaElig, bendMfb);
   mfbElig[i4] = mfbCal(portionPiaElig, percMfb, i4);
   mfbEnt.set(applyColas(mfbElig, i5, workerData.getBenefitDate()));
   if (i5 > UserAssumptions::getIstart() && isPebsAssumptions()) {
      piaRealWageGain = piaEnt;
      mfbRealWageGain = mfbEnt;
      realWageGainAdj(i5);
   }
   setDirty();
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": Returning from ChildCareCalc::calculate";
      Trace::writeLine(s);
   }
#endif
}

// Description: Adjust for childcare dropout years.
void ChildCareCalc::childCareDropoutCal()
{
   childCareDrop = 0;
   childCareDropMax = childCareDropMaxCal();
   // if no allowable childcare years, return
   if (childCareDropMax <= 0)
      return;
   // find amount of earnings allowed in a childcare dropout year
   const double childcareAmount = piaParams.getChildcareDropoutAmount(
      piaData.getEligDate().getYear(), workerData.getBenefitDate().getYear())
      + 0.01;
   // Look for usable childcare years and remove them.  Must have child in
   // care and no earnings
   const int earn50 = piaData.getEarn50(PiaData::EARN_WITH_TOTALIZATION);
   for (int i1 = earn50; i1 <= piaData.getEarnYear(); ++i1) {
      if (iorder[i1] == 1 && workerData.childCareYears.getBit(i1) &&
         piaData.earnOasdiLimited[i1] <= childcareAmount) {
         iorder[i1] = -1;
         childCareDrop++;
         if (childCareDrop >= childCareDropMax) {
            break;
         }
      }
   }
   // look for more childcare years, if we have not found enough and there
   // are some zero years without a child in care in the selected years, and
   // there are some years with a child in care in the non-selected years
   if (childCareDrop < childCareDropMax) {
      // create a vector of years with zero earnings and no child in care in
      // the selected years
      vector< int > count1;
      for (int i1 = earn50; i1 <= piaData.getEarnYear(); ++i1) {
         if (iorder[i1] == 1 && !workerData.childCareYears.getBit(i1) &&
            piaData.earnOasdiLimited[i1] <= childcareAmount) {
            count1.push_back(i1);
         }
      }
      // nothing more to do if there are no years with zero earnings and no
      // child in care in the selected years
      if (count1.size() == 0)
         return;
      // create a vector of years with zero earnings and a child in care in the
      // non-selected years
      vector< int > count2;
      for (int i1 = earn50; i1 <= piaData.getEarnYear(); ++i1) {
         if (iorder[i1] == 0 && workerData.childCareYears.getBit(i1) &&
            piaData.earnOasdiLimited[i1] <= childcareAmount) {
            count2.push_back(i1);
         }
      }
      // nothing more to do if there are no years with zero earnings and a
      // child in care in the non-selected years
      if (count2.size() == 0)
         return;
      // number swappable is minimum of the two counts
      int numswap = min(count1.size(), count2.size());
      // limit to maximum number of childcare dropout years
      numswap = min(numswap, childCareDropMax - childCareDrop);
      // swap the years in and out of the selected period
      for (int i1 = 0; i1 < numswap; ++i1) {
         iorder[count1[i1]] = 0;
         iorder[count2[i1]] = -1;
         childCareDrop++;
      }
   }
}

// Description: Returns maximum number of childcare dropout years.
//
// Remarks: Assumes that there is a specified maximum for combined regular and
//   childcare dropout years, and that number of regular dropout years
//   has already been calculated.
//
// Returns: Maximum number of childcare dropout years.
int ChildCareCalc::childCareDropMaxCal() const
{
   const int i1 = piaData.compPeriodNew.getNDrop();
   // under present law, number does not vary by year
   const int i2 = PiaParams::getMaxChildcareDropoutYearsPL();
   if (i1 < i2) {
      // can have up to 3 total dropout years, but must have at least 2
      // computation years
      return min(i2 - i1, piaData.compPeriodNew.getN() - 2);
   }
   else {
      // already have maximum dropout years
      return 0;
   }
}
