// $Id: DibGuar.cpp 1.4 2007/02/28 15:06:15EST 044579 Development  $
//
// Functions to calculate a disability guarantee PIA.

#include "DibGuar.h"
#include "DebugCase.h"
#if defined(DEBUGCASE)
#include "Trace.h"
#include "comma.h"
#endif

using namespace std;

// Description: Initializes a disability guarantee pia method.
//
// Remarks: Initializes all amounts to zero, then calls initialize.
//
// Arguments:
//   newWorkerData: Worker data.
//   newPiaData: Pia calculation data.
//   newPiaParams: Pia calculation parameters.
//   newMaxyear: Maximum year of projection.
DibGuar::DibGuar( const WorkerDataGeneral& newWorkerData,
const PiaData& newPiaData, const PiaParams& newPiaParams, int newMaxyear ) :
PiaMethod(newWorkerData, newPiaData, newPiaParams, newMaxyear,
"Disability Guarantee", DIB_GUAR), colasApply(false), whichPeriod(0),
rawPia(0.0), rawMfb(0.0), cessationMfbOrig(0.0), cessationMfbConv(0.0),
entDeathDate()
{
   initialize();
}

// Description: Destructor.
DibGuar::~DibGuar()
{ }

// Description: Determines applicability of method.
//
// Remarks: Returns true if eligibility is after 1978, and this is a disability
// case with more than 1 period of disability, or if this is an old-age or
// survivor case with any period of disability.
//
// The version with no arguments calls the static version with 3 arguments.
//
// Returns: True if method is applicable, false otherwise.
bool DibGuar::isApplicable()
{
   if (isApplicable(workerData, piaData, ioasdi)) {
      iapp = APPLICABLE;
      return(true);
   }
   else
      return(false);
}

// Arguments:
//   workerData: Worker data.
//   piaData: Pia calculation data.
//   ioasdi: Type of benefit.
bool DibGuar::isApplicable( const WorkerDataGeneral& workerData,
const PiaData& piaData, WorkerDataGeneral::ben_type ioasdi )
{
   if ((piaData.getEligYear() > 1978) &&
      ((workerData.getValdi() > 1) || (workerData.getValdi() > 0 &&
      (ioasdi == WorkerDataGeneral::OLD_AGE ||
      ioasdi == WorkerDataGeneral::SURVIVOR)))) {
      return(true);
   }
   else
      return(false);
}

// Description: Computes disability guarantee PIA.
//
// Remarks: Gets the pia at cessation from the prior disability period. For
// old-age or survivor, that is the most recent period. For disability, that is
// the prior period of disability.
//
// Apply colas from the prior cessation date to the new entitlement date only
// if the cessation was within 12 months of the new entitlement.
void DibGuar::calculate()
{
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": Starting DibGuar::calculate";
      Trace::writeLine(s);
   }
#endif
   // use prior disability in disability case, most recent disability otherwise
   whichPeriod = (ioasdi == WorkerDataGeneral::DISABILITY) ? 1 : 0;
   const DisabPeriod& disabPeriod = workerData.disabPeriod[whichPeriod];
   entDeathDate = entDeathDateCal();
   // apply cpi increases beginning with year of entitlement if there was more
   // than a 12 month gap in benefits. Otherwise apply colas starting with
   // prior year of cessation.
   const int months = disabPeriod.getCessationDate().getMonths(entDeathDate);
   colasApply = (months < 13);
   yearCpi[YEAR_ELIG] = yearCpi1Cal();
   yearCpi[FIRST_YEAR] = yearCpi[YEAR_ELIG] - 1;
   piaElig[yearCpi[FIRST_YEAR]] = disabPeriod.getCessationPia();
   setYearCpi();
   piaEnt.set(applyColasElig(piaElig, yearCpi[YEAR_ELIG],
      workerData.getBenefitDate(), piaData.getEligYear()));
   diMaxApplies = (disabPeriod.getOnsetDate().year() > 1978 &&
      workerData.getEarliestEntDate() >= PiaParams::amend80);
   eligYear = disabPeriod.getOnsetDate().year();
   convertedMfbType = convertedMfbTypeCal();
   cessationMfbOrig = workerData.disabPeriod[whichPeriod].getCessationMfb();
   cessationMfbConv = cessationMfbCal(disabPeriod);
   mfbElig[yearCpi[FIRST_YEAR]] = cessationMfbConv;
   mfbEnt.set(applyColasElig(mfbElig, yearCpi[YEAR_ELIG],
      workerData.getBenefitDate(), piaData.getEligYear()));
   setDirty();
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": Returning from DibGuar::calculate";
      Trace::writeLine(s);
   }
#endif
}

// Description: Returns first year that benefit increases are applied.
//
// Returns: First year that benefit increases are applied.
//
// Remarks: This should be called after colasApply has been calculated. If
// colasApply is true, returns year of prior disability cessation. Otherwise
// returns year of entitlement in life case, year of death in survivor case.
int DibGuar::yearCpi1Cal() const
{
   const DateMoyr dateMoyr = dateCpiCal();
   const int year = dateMoyr.getYear();
   return (static_cast< int >(dateMoyr.getMonth()) <=
      piaParams.getMonthBeninc(year)) ? year : year + 1;
}

// Description: Returns the date as of which the initial pia is calculated.
//
// Returns: The date as of which the initial pia is calculated.
//
// Remarks: Cpi increases are applied after this date.
DateMoyr DibGuar::dateCpiCal() const
{
   return (colasApply) ?
      workerData.disabPeriod[whichPeriod].getCessationDate() : entDeathDate;
}

// Description: Returns date of entitlement in a life case, or date of death
// in a survivor case.
//
// Returns: Date of entitlement in a life case, or date of death in a survivor
// case.
DateMoyr DibGuar::entDeathDateCal() const
{
   // do not use date of entitlement in a survivor case
   return (ioasdi == WorkerDataGeneral::SURVIVOR) ?
      DateMoyr(workerData.getDeathDate()) : workerData.getEntDate();
}

// Description: Calculates the applicable mfb at cessation date.
//
// Returns: The applicable mfb at cessation date.
//
// Remarks: For a disability benefit, or any benefit prior to January 1996, the
// mfb is simply the one at the date of cessation of the prior disability
// benefit. For old-age or survivors in 1996 or later, if the prior mfb was
// based on the Disability Maximum, the pia at cessation must be deconverted
// back to eligibility, then that is used to compute a raw mfb, and that is
// converted forward to the cessation date.
//
// Also sets rawPia and rawMfb, if the deconversion is necessary.
//
// Arguments:
//   disabPeriod: The period of disability being considered.
double DibGuar::cessationMfbCal( const DisabPeriod& disabPeriod )
{
   switch (convertedMfbType) {
   case POST1995_NOCHANGE:
      return cessationMfbOrig;
   case POST1995_DECONVERTED:
   {
      const int year2 = disabPeriod.getCessationDate().getYear();
      const int numYears = year2 - eligYear;
      rawPia = piaParams.deconvertPia(eligYear, numYears,
         disabPeriod.getCessationPia(), disabPeriod.getCessationDate());
      bendMfb.project(eligYear, piaParams.getFqArray());
      setPortionPiaElig(rawPia, portionPiaElig, bendMfb);
      rawMfb = mfbCal(portionPiaElig, percMfb, eligYear - 1);
      mfbElig[eligYear - 1] = rawMfb;
      return applyColas(mfbElig, eligYear,
         disabPeriod.getCessationDate().getMonthBefore());
   }
   case PRE1996_PRE1979_LAST12:
   case PRE1996_POST1978_LAST12:
   case PRE1996_NOTLAST12:
      bendMfb.project(piaData.getEligYear(), piaParams.getFqArray());
      setPortionPiaElig(disabPeriod.getCessationPia(), portionPiaElig, bendMfb);
      return mfbCal(portionPiaElig, percMfb, piaData.getEligYear() - 1);
   default:
      throw PiaException("DibGuar::cessationMfbCal: Illegal convertedMfbType");
   }
}

// Description: Returns the type of mfb at cessation, after conversion.
//
// Returns: The type of mfb at cessation, after conversion.
//
// Remarks: The type of mfb first depends on the date of entitlement to
// the subsequent benefit. If it is January 1996 or later, then the type
// is POST1995_NOCHANGE if the new benefit is a disability benefit, or
// if the Disability Maximum does not apply. The type is
// POST1995_DECONVERTED if it is an old-age or survivor benefit and the
// Disability Maximum applies.
//
// If the date of entitlement to the subsequent benefit is before January
// 1996, then the type depends on if the prior DIB entitlement continued
// in the 12 months preceding the new entitlement. If so, the type is
// PRE1996_PRE1979_LAST12 if eligibility to the prior DIB benefit was
// before 1979, or PRE1996_POST1978_LAST12 if eligibility to the prior
// DIB benefit was after 1978. Otherwise, the type is PRE1996_NOTLAST12.
DibGuar::ConvertedMfbType DibGuar::convertedMfbTypeCal() const
{
   if (entDeathDate > DateMoyr(12, 1995)) {
      if (diMaxApplies && ioasdi != WorkerDataGeneral::DISABILITY) {
         // Subsequent entitlement in January 1996 or later, new benefit is
         // OASI, and Disability Maximum applies (requires deconversion to
         // prior eligibility year).
         return POST1995_DECONVERTED;
      }
      else {
         // Subsequent entitlement in January 1996 or later, new benefit is DI,
         // or the Disability Maximum does not apply (no conversion required).
         return POST1995_NOCHANGE;
      }
   }
   else {
      if (colasApply) {
         if (eligYear < 1979) {
            // Subsequent entitlement before January 1996, prior DIB continued
            // in the 12 months preceding the new entitlement, prior DIB
            // eligibility was before 1979.
            return PRE1996_PRE1979_LAST12;
         }
         else {
            // Subsequent entitlement before January 1996, prior DIB continued
            // in the 12 months preceding the new entitlement, prior DIB
            // eligibility was after 1978.
            return PRE1996_POST1978_LAST12;
         }
      }
      else {
         // Subsequent entitlement before January 1996, prior DIB did not
         // continue in the 12 months preceding the new entitlement,
         return PRE1996_NOTLAST12;
      }
   }
}
