// $Id: EarnProject.cpp 1.19 2006/07/18 10:26:09EDT 044579 Development  $
//
// Functions to initialize an array of earnings, with forward
// and backward projections and specified levels of earnings.

#include "EarnProject.h"
#include "wbgenrl.h"
#include "awinc.h"
#include "WorkerDataGeneral.h"
#include "AssumptionType.h"

using namespace std;

// Description: Maximum year of projection.
int EarnProject::maxyear;

// Description: Earnings selection titles.
const char *EarnProject::earnmal[NUM_TYPES] = {
   "entered earnings", "maximum earnings", "high earnings", "average earnings",
   "low earnings", "old-law maximum earnings", "childcare year"
};

// Description: Lowest acceptable projection percentage.
const float EarnProject::PERC_LOW = 0.0f;
// Description: Highest acceptable projection percentage.
const float EarnProject::PERC_HIGH = 9.99f;

// Description: Initializes an EarnProject structure.
//
// Arguments:
//   newMaxYear: Maximum projected year. Arrays start in 1937.
EarnProject::EarnProject( int newMaxYear ) :
earnpebs(YEAR37, newMaxYear), earntype(YEAR37, newMaxYear), firstYear(YEAR37),
lastYear(newMaxYear), projback(NO_PROJ), percback(PERC_LOW), projfwrd(NO_PROJ),
percfwrd(PERC_LOW)
{
   maxyear = newMaxYear;
}

// Description: Destroys an EarnProject structure.
EarnProject::~EarnProject( )
{ }

// Description: Initializes all variables to default values.
void EarnProject::deleteContents()
{
   percback = percfwrd = PERC_LOW;
   projback = projfwrd = NO_PROJ;
   earntype.deleteContents();
   earnpebs.deleteContents();
}

// Description: Sets some Statement data.
//
// Arguments:
//   year: First year of projection period (used to set last year of earnings).
//   ibegint: First year of earnings.
void EarnProject::setPebsData( int year, int ibegint )
{
   setPebsEarn(year);
   // only allow entered earnings historically, unless doing an expanded PEBES
   if (!AssumptionType::EXPANDED_PEBES) {
      firstYear = ibegint;
      projback = NO_PROJ;
      percback = PERC_LOW;
      projfwrd = AVGWAGE_PROJ;
      percfwrd = PERC_LOW;
      earntype.assign(ENTERED, YEAR37, lastYear);
   }
   else {
      // must have a forward projection even in expanded PEBES
      if (projfwrd == NO_PROJ)
         projfwrd = AVGWAGE_PROJ;
   }
}

// Description: Sets last year of Statement earnings when computation year
//   changes, or after a Statement calculation has been done.
//
// Arguments:
//   year: First year of projection period.
void EarnProject::setPebsEarn( int year )
{
   const int iend2 = lastYear;  // last year of earnings in file
   lastYear = 1 + year;
   const double lastEarn = earnpebs[iend2];
   if (lastYear > iend2) {
      earnpebs.assign(lastEarn, iend2 + 1, lastYear);
   }
}

// Description: Returns true if earnings are projected in last two years of
//   historical period.
//
// Returns: True if earnings are projected in last two years of
//   historical period.
//
// Arguments:
//   year: First year of projection period.
//   iendt: Last year of projected earnings.
bool EarnProject::need_aw( int year, int iendt ) const
{
   // if faulty starting date for assumptions, return false
   if (year < YEAR79)
      return(false);
   int earntype1 = earntype[year];
   int earntype2 = earntype[year - 1];
   if (earntype1 == HIGH || earntype1 == AVERAGE || earntype1 == LOW ||
      earntype2 == HIGH || earntype2 == AVERAGE || earntype2 == LOW ||
      (projfwrd == 1 && iendt > year - 2) ||
      (projback == 1 && firstYear > year - 2)) {
      return true;
   }
   else {
      return false;
   }
}

// Description: Returns description of type of earnings.
//
// Returns: Description of type of earnings.
//
// Throws: PiaException of type PIA_IDS_EARNMAL if year is before 1937 (only in
//   debug mode).
//
// Arguments:
//   year: Year for which type of earnings is desired.
std::string EarnProject::toString( int year ) const
{
#ifndef NDEBUG
   if (year < YEAR37)
      throw PiaException(PIA_IDS_EARNMAL);
#endif
   return(string(earnmal[earntype[year]]));
}

// Description: Checks earnings.
//
// Throws: PiaException of type PIA_IDS_EARNPEBS if earnings are out of range.
//
// Arguments:
//   earnpebst: Earnings to check.
void EarnProject::earnpebsCheck( double earnpebst )
{
   try {
      WageBaseGeneral::check(earnpebst);
   } catch (PiaException&) {
      throw PiaException(PIA_IDS_EARNPEBS);
   }
}

// Description: Projects OASDI-covered earnings.
//
// Returns: True if any earnings go over the maximum allowed, false otherwise.
//
// Throws: PiaException of type PIA_IDS_EARNPROJ if projection year is out of
//   range (only in debug mode).
//
// Arguments:
//   workerData: Worker basic data.
//   fqinc: Average wage increases.
bool EarnProject::project( WorkerDataGeneral& workerData,
const Awinc& fqinc ) const
{
   bool overMax = false;  // indicator for earnings over the maximum allowed

   workerData.zeroEarnOasdi(firstYear, lastYear);
   for (int i1 = firstYear; i1 <= lastYear; i1++) {
      workerData.setEarnOasdi(i1, earnpebs[i1]);
   }
   if (getProjback() != NO_PROJ) {
      for (int i1 = firstYear - 1; i1 >= workerData.getIbegin(); i1--) {
         const double basePerc = (getProjback() == CONSTANT_PROJ) ?
            0.0 : fqinc[i1 + 1];
         const double factor = 1.0 + (basePerc + getPercback()) / 100.0;
         const double amount = AverageWage::projectBack(
            workerData.getEarnOasdi(i1 + 1), factor);
         if (amount > MAXEARN) {
            workerData.setEarnOasdi(i1, MAXEARN);
            overMax = true;
         }
         else {
            workerData.setEarnOasdi(i1, amount);
         }
      }
   }
   if (getProjfwrd() != NO_PROJ) {
      for (int i1 = lastYear + 1; i1 <= workerData.getIend(); i1++) {
         const double basePerc = (getProjfwrd() == CONSTANT_PROJ) ?
            0.0 : fqinc[i1];
         const double factor = 1.0 + (basePerc + getPercfwrd()) / 100.0;
         const double amount = AverageWage::projectForward(
            workerData.getEarnOasdi(i1 - 1), factor);
         if (amount > MAXEARN) {
            workerData.setEarnOasdi(i1, MAXEARN);
#if !defined(NDEBUG)
            if (lastYear < YEAR37)
               throw PiaException(PIA_IDS_EARNPROJ);
#endif
            if (earntype[lastYear] != MAXIMUM) {
               overMax = true;
            }
         }
         else {
            workerData.setEarnOasdi(i1, amount);
         }
      }
   }
   return overMax;
}
