// $Id: FrozMin.cpp 1.28 2007/11/07 07:27:49EST 277133 Development  $
//
// Functions to calculate a frozen minimum PIA.

#include "FrozMin.h"
#include "DebugCase.h"
#if defined(DEBUGCASE)
#include "Trace.h"
#include "comma.h"
#endif

using namespace std;

// Description: Initializes a frozen minimum method.
//
// Arguments:
//   newWorkerData: Worker data.
//   newPiaData: Pia calculation data.
//   newPiaParams: Pia calculation parameters.
//   newMaxyear: Maximum year of projection.
FrozMin::FrozMin( const WorkerDataGeneral& newWorkerData,
const PiaData& newPiaData, const PiaParams& newPiaParams, int newMaxyear ) :
PiaMethod(newWorkerData, newPiaData, newPiaParams, newMaxyear,
"Frozen Minimum (1977 Act)", FROZ_MIN)
{
   initialize();
}

// Description: Destructor.
FrozMin::~FrozMin()
{ }

// Description: Determines applicability of method.
//
// Remarks: The version with no arguments calls the static version with 2
//   arguments.
//
// Returns: True if method is applicable, false otherwise.
bool FrozMin::isApplicable()
{
   if (isApplicable(workerData, piaData)) {
      iapp = APPLICABLE;
      return(true);
   }
   else
      return(false);
}

// Arguments:
//   workerData: Worker data.
//   piaData: Pia calculation data.
bool FrozMin::isApplicable( const WorkerDataGeneral& workerData,
const PiaData& piaData )
{
   if (piaData.getEligYear() > 1978 && piaData.getEligYear() < 1982 &&
      !workerData.getTotalize()) {
      return(true);
   }
   else
      return(false);
}

// Description: Computes frozen minimum PIA.
void FrozMin::calculate()
{
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": Starting FrozMin::calculate";
      Trace::writeLine(s);
   }
#endif
   setFrozMin();
   // apply cpi increases beginning with earlier of year of entitlement.
   // or year of age 65
   setYearCpi();
   piaEnt.set(applyColasElig(piaElig, yearCpi[YEAR_ELIG],
      workerData.getBenefitDate(), piaData.getEligYear()));
   mfbEnt.set(applyColasElig(mfbElig, yearCpi[YEAR_ELIG],
      workerData.getBenefitDate(), piaData.getEligYear()));
   setDirty();
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": Returning from FrozMin::calculate";
      Trace::writeLine(s);
   }
#endif
}

// Description: Sets amounts for frozen minimum before benefit increases.
void FrozMin::setFrozMin()
{
   // do not use date of entitlement in a survivor case
   int year = (ioasdi == WorkerDataGeneral::SURVIVOR) ?
      (int)workerData.getDeathDate().year() : (int)workerData.getEntDate().getYear();
   int yearCpiTemp = min(year, piaData.getBirthDateMinus1().year() + 65);
   yearCpi[YEAR_ELIG] = yearCpiTemp;
   yearCpiTemp--;
   yearCpi[FIRST_YEAR] = yearCpiTemp;
   piaElig[yearCpiTemp] = 122.0;
   mfbElig[yearCpiTemp] = 183.0;
}
