// $Id: LawChange.cpp 1.7 2006/01/05 09:38:07EST 044579 Exp  $
//
// Functions for class to manage LawChange parameters.

#include "LawChange.h"
#include "PiaException.h"
#include "Resource.h"
#include "comma.h"
#include "StringParser.h"

// Define static class members
int  LawChange::startYearLC;
int  LawChange::endYearLC;

using namespace std;

// Description: Constructor.
LawChange::LawChange() : endYear(0), ind(0), phaseType(0), startYear(0),
titleLC()
{ }

// Arguments:
//   newTitle: Title of this law-change. If omitted, an empty string is used.
LawChange::LawChange( const std::string& newTitle ) : endYear(0), ind(0),
phaseType(0), startYear(0), titleLC(newTitle)
{ }

// Description: Destructor.
LawChange::~LawChange()
{ }

// Description: Returns true if this law change is in effect.
//
// Returns: True if this law change is in effect.
//
// Arguments:
//   eligYear: Year of eligibility.
//   benYear: Year of benefit.
bool LawChange::isEffective( int eligYear, int benYear ) const
{
   int year = (phaseType == 0) ? eligYear : benYear;
   return (ind > 0 && year >= startYear && year <= endYear);
}

// Description: Checks range of a law-change starting year.
//
// Throws: PiaException of type PIA_IDS_LAWCHG1 if starting year is out of
//   range.
//
// Arguments:
//   start: Starting year to check.
void LawChange::startCheck( int start )
{
   if (start < startYearLC || start > endYearLC)
      throw PiaException(PIA_IDS_LAWCHG1);
}

// Description: Checks range of a bendpoint value.
//
// Throws: PiaException of type PIA_IDS_LAWCHG4 if bend point is out of
//   range.
//
// Arguments:
//   bendPoint: Value to check.
void LawChange::bendPointCheck( double bendPoint )
{
   if (bendPoint < 0. || bendPoint > 99999.)
      throw PiaException(PIA_IDS_LAWCHG4);
}

// Description: Checks range of a wage increase proportion.
//
// Throws: PiaException of type PIA_IDS_LAWCHG2 if wage increase proportion
//   is out of range.
//
// Arguments:
//   proportionBendPoints: Value to check.
void LawChange::proportionBendPointsCheck( double proportionBendPoints )
{
   if (proportionBendPoints < .0 || proportionBendPoints > 1.0)
      throw PiaException(PIA_IDS_LAWCHG2);
}

// Description: Checks range of a benefit formula percentage.
//
// Throws: PiaException of type PIA_IDS_LAWCHG3 if benefit formula percentage
//   is out of range.
//
// Arguments:
//   perc: Value to check.
void LawChange::percCheck( double perc )
{
   if (perc < .0 || perc > 100.0)
      throw PiaException(PIA_IDS_LAWCHG3);
}

// Description: Checks range of a bend point increase percentage.
//
// Throws: PiaException of type PIA_IDS_LAWCHG3 if bend point increase
//   percentage is out of range.
//
// Arguments:
//   bpinc: Value to check.
void LawChange::bpincCheck( double bpinc )
{
   if (bpinc < 0.0 || bpinc > 100.0)
      throw PiaException(PIA_IDS_LAWCHG3);
}

// Description: Checks range of a constant decline percentage.
//
// Throws: PiaException of type PIA_IDS_LAWCHG5 if constant decline percentage
//   is out of range.
//
// Arguments:
//   constantPerc: Value to check.
void LawChange::constantPercCheck( double constantPerc )
{
   if (constantPerc < 0.0 || constantPerc > 9.99)
      throw PiaException(PIA_IDS_LAWCHG5);
}

// Description: Writes a line with first and last years.
//
// Arguments:
//   out: Open output stream.
void LawChange::writeFirstLine( std::ostream& out ) const
{
   out << poutNoComma(getStartYear(), 4) << " ";
   out << poutNoComma(getEndYear(), 4) << " ";
   out << poutNoComma(getPhaseType());
}

// Description: Writes a line with first and last years.
//
// Arguments:
//   out: Open output stream.
void LawChange::write( std::ostream& out ) const
{
   if (getInd() > 0) {
      writeFirstLine(out);
      out << endl;
   }
}

// Description: Reads a line and parses first and last years and type of
//   phase-in.
//
// Returns: Vector of words contained in the line.  The first 3 words are the
//   first and last years and the type of phase-in.
//
// Throws: PiaException of type PIA_IDS_LAWCHG6 if unable to parse input line.
//
// Arguments:
//   infile: Open input stream.
std::vector< std::string > LawChange::getFirstLine( std::istream& infile )
{
   string str;  // input line
   vector< string > retvec;  // return vector

   getline(infile, str);
   if (infile.eof() || infile.fail())
      throw PiaException(PIA_IDS_LAWCHG6);
   retvec = parseString(str);
   if (retvec.size() < 3)
      throw PiaException(PIA_IDS_LAWCHG6);
   setStartYear(atoi(retvec[0].c_str()));
   setEndYear(atoi(retvec[1].c_str()));
   setPhaseType(atoi(retvec[2].c_str()));
   return retvec;
}

// Description: Reads all the lines required for this law-change.
//
// Arguments:
//   infile: Open input stream.
void LawChange::read( std::istream& infile )
{
   if (getInd() > 0) {
      getFirstLine(infile);
   }
}

// Description: Prepares output strings for this change from present law.
//
// Arguments:
//   outputString: Table of output strings.
void LawChange::prepareStrings( std::vector<std::string> &outputString )
{
   if (getInd() > 0) {
      outputString.push_back(getTitle());
      if (getPhaseType() == 0)
         outputString.push_back("Effective for new eligibles");
      else
         outputString.push_back("Effective immediately for all persons");
   }
}
