// $Id: LawChangeDECLINEPERC.cpp 1.8 2004/03/31 14:01:52EST sfmckay Exp  $
//
// Functions needed for class to manage parameters for a proposal to
// decrease the benefit formula percentages by a constant rate.

#include <vector>
#include "LawChangeDECLINEPERC.h"
#include "PiaException.h"
#include "Resource.h"
#include "comma.h"
#include "oactcnst.h"
#include "StringParser.h"
#include "percpia.h"

using namespace std;

// Description: Constructor.
//
// Remarks: Allocates memory for the benefit formula percentages.
LawChangeDECLINEPERC::LawChangeDECLINEPERC() :
LawChange("declining benefit formula percents")
{
   for (int i = 0; i < 4; i++)
      percpa[i] = new DoubleAnnual(YEAR37, endYearLC);
}

// Description: Destructor.
//
// Remarks: Releases memory.
LawChangeDECLINEPERC::~LawChangeDECLINEPERC()
{
   for (int i = 0; i < 4; i++)
      delete percpa[i];
}

// Description: Reads all the lines required for this law-change.
//
// Arguments:
//   infile: Open input stream.
void LawChangeDECLINEPERC::read( std::istream& infile )
{
   string str;  // input string

   if (getInd() > 0) {
      vector< string > strvec = getFirstLine(infile);
      int yearNum = 0;
      if (strvec.size() < 6)
         throw PiaException(PIA_IDS_LAWCHG6);
      for (int i1 = 0; i1 < 3; i1++) {
         setFactor(yearNum, i1, atof(strvec[i1+3].c_str()));
      }
      setYear(yearNum, getStartYear());
      // read the remaining lines
      if (getInd() > 1) {
         for (yearNum = 1; yearNum < getInd(); yearNum++) {
            getline(infile, str);
            if (infile.eof() || infile.fail())
               throw PiaException(PIA_IDS_LAWCHG6);
            strvec = parseString(str);
            if (strvec.size() < 4)
               throw PiaException(PIA_IDS_LAWCHG6);
            setYear(yearNum, atoi(strvec[0].c_str()));
            for (int i1 = 0; i1 < 3; i1++) {
               setFactor(yearNum, i1, atof(strvec[i1 + 1].c_str()));
            }
         }
      }
      // set all the percentages
      percPiaCal();
   }
}

// Description: Writes a line with first and last years, and percentage.
//
// Arguments:
//   out: Open output stream.
void LawChangeDECLINEPERC::write( std::ostream& out ) const
{
   if (getInd() > 0) {
      writeFirstLine(out);
      int yearNum = 0;
      for (int i1 = 0; i1 < 3; i1++)
         out << " " << poutNoComma(getFactor(yearNum, i1), 4, 2);
      out << endl;
      // write the remaining lines
      if (getInd() > 1) {
         for (yearNum = 1; yearNum < getInd(); yearNum++) {
            out << poutNoComma(getYear(yearNum), 4) << " ";
            for (int i1 = 0; i1 < 3; i1++)
               out << " " << poutNoComma(getFactor(yearNum, i1), 4, 2);
            out << endl;
         }
      }
   }
}

// Description: Prepares output strings for this change from present law.
//
// Arguments:
//   outputString: Table of output strings.
void LawChangeDECLINEPERC::prepareStrings(
std::vector<std::string> &outputString )
{
   if (getInd() > 0) {
      string s;
      outputString.push_back(getTitle());
      if (getPhaseType() == 0)
         outputString.push_back("Effective for new eligibles");
      else
         outputString.push_back("Effective immediately for all persons");
      for (int i2 = 0; i2 < getInd(); i2++) {
         s = "Percentages decline by ";
         s += poutNoComma(getFactor(i2, 0), 1, 2);
         s += ", ";
         s += poutNoComma(getFactor(i2, 1), 1, 2);
         s += ", and ";
         s += poutNoComma(getFactor(i2, 2), 1, 2);
         s += " starting in ";
         s += poutNoComma(getYear(i2));
         outputString.push_back(s);
      }
      s = "Last year of decline is ";
      s += poutNoComma(getEndYear());
      outputString.push_back(s);
   }
}

// Description: Sets pia formula percentages.
//
// Remarks: Initializes the pia formula percentages in the year prior to the
// effective year with the present-law percentages, then applies the rate of
// decline for each percentage for each year in the future.
void LawChangeDECLINEPERC::percPiaCal()
{
   if (getInd() > 0) {
      for (int i = 0; i < 3; i++) {
         // initialize first percentage
         (*percpa[i])[getYear(0) - 1] = PercPia::PERC[i];
         // handle each interval of declining percentages
         for (int yearNum = 0; yearNum < getInd(); yearNum++) {
            double factor = 1. - getFactor(yearNum, i) / 100.;
            int endYr;  // ending year for this interval
            if (yearNum == getInd() - 1)
               endYr = min(percpa[i]->getLastYear(), getEndYear());
            else
               endYr = min(percpa[i]->getLastYear(),
                  getYear(yearNum + 1) - 1);
            for (int year = getYear(yearNum); year <= endYr; year++) {
               (*percpa[i])[year] = (*percpa[i])[year - 1] * factor;
            }
         }
         // extend to last possible year
         if (getEndYear() < percpa[i]->getLastYear()) {
            double ultimatePerc = (*percpa[i])[getEndYear()];
            for (int year = getEndYear() + 1;
            year <= percpa[i]->getLastYear(); year++) {
               (*percpa[i])[year] = ultimatePerc;
            }
         }
      }
   }
}
