// $Id: LawChangeNEWFORMULA.cpp 1.8 2004/03/31 14:01:53EST sfmckay Exp  $
//
// Functions needed for class to manage parameters for a proposal to
// change the pia formula bend points and/or percentages.

#include <vector>
#include "LawChangeNEWFORMULA.h"
#include "PiaException.h"
#include "Resource.h"
#include "comma.h"
#include "oactcnst.h"
#include "StringParser.h"

using namespace std;

// Description: Constructor.
//
// Remarks: Allocates memory for each element of the bend point and percentage
// arrays.
LawChangeNEWFORMULA::LawChangeNEWFORMULA() :
LawChange("new benefit formula")
{
   for (int i = 0; i < 3; i++)
      bendpalt[i] = new DoubleAnnual(YEAR37, endYearLC);
   for (int i = 0; i < 4; i++)
      percpa[i] = new DoubleAnnual(YEAR37, endYearLC);
}

// Description: Destructor.
//
// Remarks: Deletes allocated memory.
LawChangeNEWFORMULA::~LawChangeNEWFORMULA()
{
   for (int i = 0; i < 3; i++)
      delete bendpalt[i];
   for (int i = 0; i < 4; i++)
      delete percpa[i];
}

// Description: Reads all the lines required for this law-change.
//
// Arguments:
//   infile: Open input stream.
void LawChangeNEWFORMULA::read( std::istream& infile )
{
   string str;  // input string

   if (getInd() > 0) {
      vector< string > strvec = getFirstLine(infile);
      for (int i2 = getStartYear(); i2 <= getEndYear(); i2++) {
         getline(infile, str);
         if (infile.eof() || infile.fail())
            throw PiaException(PIA_IDS_LAWCHG6);
         strvec = parseString(str);
         // set first percentage
         if (strvec.size() < 5)
            throw PiaException(PIA_IDS_LAWCHG6);
         // set percentages
         for (int i1 = 0; i1 < 3; i1++) {
            double dtemp = atof(strvec[i1].c_str());
            setAltPercPia(getStartYear(), i1, dtemp);
         }
         // set bend points
         for (int i1 = 1; i1 < 3; i1++) {
            double dtemp = atof(strvec[i1 + 2].c_str());
            setAltBendPia(getStartYear(), i1, dtemp);
         }
      }
   }
}

// Description: Writes a line with first and last years, and percentage.
//
// Arguments:
//   out: Open output stream.
void LawChangeNEWFORMULA::write( std::ostream& out ) const
{
   if (getInd() > 0) {
      writeFirstLine(out);
      out << endl;
      for (int year = getStartYear(); year <= getEndYear(); year++) {
         for (int i1 = 0; i1 < 3; i1++)
            out << poutNoComma(getAltPercPia(year, i1), 4, 2) << " ";
         for (int i1 = 1; i1 < 3; i1++)
            out << poutNoComma(getAltBendPia(year, i1), 7) << " ";
         out << endl;
      }
   }
}

// Description: Prepares output strings for this change from present law.
//
// Arguments:
//   outputString: Table of output strings.
void LawChangeNEWFORMULA::prepareStrings(
std::vector<std::string> &outputString )
{
   if (getInd() > 0) {
      outputString.push_back(getTitle());
      if (getPhaseType() == 0)
         outputString.push_back("Effective for new eligibles");
      else
         outputString.push_back("Effective immediately for all persons");
      const int i2 = getStartYear();
      string s = "PIA formula is ";
      s += poutNoComma(getAltPercPia(i2, 0) * 100., 1, 2);
      s += "% of first $";
      s += poutComma(getAltBendPia(i2, 1), 1, 0);
      s += " of AIME +";
      outputString.push_back(s);
      s = poutNoComma(getAltPercPia(i2, 1) * 100., 1, 2);
      s += "% of AIME up to $";
      s += poutComma(getAltBendPia(i2, 2), 1, 0);
      s += " +";
      outputString.push_back(s);
      s = poutNoComma(getAltPercPia(i2, 2) * 100., 1, 2);
      s += "% of remainder for those eligible in ";
      s += poutNoComma(i2);
      outputString.push_back(s);
   }
}
