// $Id: LawChangeTAXRATECHG.cpp 1.7 2006/11/08 11:05:05EST 044579 Development  $
//
// Functions needed for class to manage parameters for a proposal to
// change the OASDHI tax rates.

#include <vector>
#include "LawChangeTAXRATECHG.h"
#include "PiaException.h"
#include "Resource.h"
#include "comma.h"
#include "oactcnst.h"
#include "StringParser.h"

using namespace std;

// Description: Constructor.
LawChangeTAXRATECHG::LawChangeTAXRATECHG() :
LawChange("tax rate change"), numTaxrates(0)
{ }

// Description: Reads all the lines required for this law-change.
//
// Arguments:
//   infile: Open input stream.
void LawChangeTAXRATECHG::read( std::istream& infile )
{
   if (getInd() > 0) {
      vector< string > strvec = getFirstLine(infile);
      if (strvec.size() < 4)
         throw PiaException(PIA_IDS_LAWCHG6);
      setNumTaxrates(atoi(strvec[3].c_str()));
      for (int i1 = 0; i1 < getNumTaxrates(); i1++) {
         string str;
         getline(infile, str);
         if (infile.eof() || infile.fail())
            throw PiaException(PIA_IDS_LAWCHG6);
         strvec = parseString(str);
         if (strvec.size() < 4)
            throw PiaException(PIA_IDS_LAWCHG6);
         setYearTaxrate(i1, atoi(strvec[0].c_str()));
         // parse employee-employer rates
         setTaxrate(i1, TaxRates::EMPLOYEERATE, TaxRates::OASITF,
            atof(strvec[1].c_str()));
         setTaxrate(i1, TaxRates::EMPLOYEERATE, TaxRates::DITF,
            atof(strvec[2].c_str()));
         setTaxrate(i1, TaxRates::EMPLOYEERATE, TaxRates::OASDITF,
            atof(strvec[3].c_str()));
         if (getInd() == 2) {
            // parse self-employed rates
            if (strvec.size() < 7)
               throw PiaException(PIA_IDS_LAWCHG6);
            setTaxrate(i1, TaxRates::SELFEMPRATE, TaxRates::OASITF,
               atof(strvec[4].c_str()));
            setTaxrate(i1, TaxRates::SELFEMPRATE, TaxRates::DITF,
               atof(strvec[5].c_str()));
            setTaxrate(i1, TaxRates::SELFEMPRATE, TaxRates::OASDITF,
               atof(strvec[6].c_str()));
         }
         else {
            // set self-employed rates equal to twice ee-er rates
            setSelfEmpTaxrate(i1);
         }
      }
   }
}

// Description: Writes a line with first and last years, and step size.
//
// Arguments:
//   out: Open output stream.
void LawChangeTAXRATECHG::write( std::ostream& out ) const
{
   if (getInd() > 0) {
      writeFirstLine(out);
      out << " " << getNumTaxrates() << endl;
      for (int i1 = 0; i1 < getNumTaxrates(); i1++) {
         out << getYearTaxrate(i1);
         // write employee-employer rates
         out << " " << poutNoComma(getTaxrate(i1, TaxRates::EMPLOYEERATE,
            TaxRates::OASITF), 5, 4);
         out << " " << poutNoComma(getTaxrate(i1, TaxRates::EMPLOYEERATE,
            TaxRates::DITF), 5, 4);
         out << " " << poutNoComma(getTaxrate(i1, TaxRates::EMPLOYEERATE,
            TaxRates::OASDITF), 5, 4);
         if (getInd() == 2) {
            // write self-employed rates
            out << " " << poutNoComma(getTaxrate(i1, TaxRates::SELFEMPRATE,
               TaxRates::OASITF), 5, 4);
            out << " " << poutNoComma(getTaxrate(i1, TaxRates::SELFEMPRATE,
               TaxRates::DITF), 5, 4);
            out << " " << poutNoComma(getTaxrate(i1, TaxRates::SELFEMPRATE,
               TaxRates::OASDITF), 5, 4);
         }
         out << endl;
      }
   }
}

// Description: Prepares output strings for this change from present law.
//
// Arguments:
//   outputString: Table of output strings.
void LawChangeTAXRATECHG::prepareStrings(
std::vector<std::string> &outputString )
{
   if (getInd() > 0) {
      outputString.push_back(getTitle());
      if (getPhaseType() == 0)
         outputString.push_back("Effective for new eligibles");
      else
         outputString.push_back("Effective immediately for all persons");
      outputString.push_back((getInd() == 1) ?
         "OASI, DI, and HI ee-er rates change as follows:" :
         "OASI, DI, and HI rates for ee-er and self-emp change as follows:");
      for (int i2 = 0; i2 < getNumTaxrates(); i2++) {
         string s = poutNoComma(getYearTaxrate(i2));
         // write employee-employer rates
         s += "   ";
         s += poutNoComma(getTaxrate(i2, TaxRates::EMPLOYEERATE,
            TaxRates::OASITF), 5, 4);
         s += "   ";
         s += poutNoComma(getTaxrate(i2, TaxRates::EMPLOYEERATE,
            TaxRates::DITF), 5, 4);
         s += "   ";
         s += poutNoComma(getTaxrate(i2, TaxRates::EMPLOYEERATE,
            TaxRates::OASDITF), 5, 4);
         if (getInd() == 2) {
            // write self-employed rates
            s += "   ";
            s += poutNoComma(getTaxrate(i2, TaxRates::SELFEMPRATE,
               TaxRates::OASITF), 5, 4);
            s += "   ";
            s += poutNoComma(getTaxrate(i2, TaxRates::SELFEMPRATE,
               TaxRates::DITF), 5, 4);
            s += "   ";
            s += poutNoComma(getTaxrate(i2, TaxRates::SELFEMPRATE,
               TaxRates::OASDITF), 5, 4);
         }
         outputString.push_back(s);
      }
   }
}

// Description: Returns tax rate.
//
// Returns: Tax rate for specified parameters.
//
// Arguments:
//   yearNum: Number of tax rate interval (0 to numTaxrates - 1).
//   eeer: Type of tax rate.
//   trustfund: Type of taxes.
//     0: OASI, 1: DI, 2: HI.
double LawChangeTAXRATECHG::getTaxrate( int yearNum, TaxRates::taxrateType eeer,
TaxRates::trustfundType trustfund ) const
{
   switch (trustfund) {
   case 0:
      return oasiTaxrate[eeer][yearNum];
   case 1:
      return diTaxrate[eeer][yearNum];
   default:
      return hiTaxrate[eeer][yearNum];
   }
}

// Description: Sets tax rate.
//
// Arguments:
//   yearNum: Number of tax rate interval (0 to numTaxrates - 1).
//   eeer: Type of tax rate.
//   trustfund: Type of taxes.
//     0: OASI, 1: DI, 2: HI.
//   newRate: New tax rate.
void LawChangeTAXRATECHG::setTaxrate( int yearNum, TaxRates::taxrateType eeer,
 TaxRates::trustfundType trustfund, double newRate )
{
   switch (trustfund) {
   case 0:
      oasiTaxrate[eeer][yearNum] = newRate;
   case 1:
      diTaxrate[eeer][yearNum] = newRate;
   default:
      hiTaxrate[eeer][yearNum] = newRate;
   }
}

// Description: Set self-employed rates equal to twice ee-er rates.
//
// Arguments:
//   yearNum: Number of tax rate interval (0 to numTaxrates - 1).
void LawChangeTAXRATECHG::setSelfEmpTaxrate( int yearNum )
{
   oasiTaxrate[1][yearNum] = 2. * oasiTaxrate[0][yearNum];
   diTaxrate[1][yearNum] = 2. * diTaxrate[0][yearNum];
   hiTaxrate[1][yearNum] = 2. * hiTaxrate[0][yearNum];
}
