// $Id: PiaCalPL.cpp 1.48 2005/10/19 08:56:18EDT 044579 Exp  $
//
// Functions to calculate a present-law PIA.

#include <utility>  // for rel_ops
#include "PiaCalPL.h"
#include "WageInd.h"
#include "FrozMin.h"
#include "SpecMin.h"
#include "PiaTable.h"
#include "TransGuar.h"
#include "ReindWid.h"
#include "OldStart.h"
#include "DibGuar.h"
#include "ChildCareCalc.h"
#include "PiaException.h"
#include "UserAssumptions.h"
#include "SecondaryArray.h"
#include "piaparms.h"
#include "DebugCase.h"
#if defined(DEBUGCASE)
#include "Trace.h"
#include "comma.h"
#endif

using namespace std;
#if !defined(__SGI_STL_INTERNAL_RELOPS)
using namespace std::rel_ops;
#endif

// Description: Initializes a PiaCalPL instance.
//
// Arguments:
//   newWorkerData: Worker's basic data.
//   newPiaData: Worker's calculation data.
//   newPiaParams: Pia calculation parameters.
PiaCalPL::PiaCalPL( WorkerDataGeneral& newWorkerData, PiaData& newPiaData,
const PiaParams& newPiaParams ) :
PiaCal(newWorkerData, newPiaData, newPiaParams)
{ }

// Description: Destructor.
PiaCalPL::~PiaCalPL()
{
   releaseMemory();
}

// Description: Calculates PIA and MFB, not including re-indexed widow(er).
//
// Arguments:
//   entDate: The date of entitlement.
void PiaCalPL::piaCal( const DateMoyr& entDate )
{
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": At top of PiaCalPL::piaCal";
      Trace::writeLine(s);
   }
#endif
   piaData.initialize();
   piaData.earn50Cal(workerData);
   setAmend90(entDate);

   // release any memory from a prior case
   releaseMemory();
   // get new memory
   int maxYear = piaParams.getMaxyear();
   PiaMethod* method;
   if (OldStart::isApplicable(piaData)) {
      method = new OldStart(workerData, piaData, piaParams, maxYear, entDate);
      // find old-start n
      nelapsedCal(piaData.compPeriodOld, entDate);
      nCal(piaData.compPeriodOld, entDate);
      piaMethod.push_back(method);
      oldStart = dynamic_cast<OldStart *>(method);
   }
   if (PiaTable::isApplicable(workerData, piaData, ioasdi)) {
      method = new PiaTable(workerData, piaData, piaParams, maxYear);
      piaMethod.push_back(method);
      piaTable = dynamic_cast<PiaTable *>(method);
   }
   if (WageInd::isApplicable(workerData, piaData)) {
      method = new WageInd(workerData, piaData, piaParams, maxYear);
      piaMethod.push_back(method);
      wageInd = dynamic_cast<WageInd *>(method);
   }
   if (TransGuar::isApplicable(workerData, piaData, ioasdi)) {
      method = new TransGuar(workerData, piaData, piaParams, maxYear);
      piaMethod.push_back(method);
      transGuar = dynamic_cast<TransGuar *>(method);
   }
   if (SpecMin::isApplicable(workerData)) {
      method = new SpecMin(workerData, piaData, piaParams, maxYear);
      piaMethod.push_back(method);
      specMin = dynamic_cast<SpecMin *>(method);
   }
   if (FrozMin::isApplicable(workerData, piaData)) {
      method = new FrozMin(workerData, piaData, piaParams, maxYear);
      piaMethod.push_back(method);
      frozMin = dynamic_cast<FrozMin *>(method);
   }
   if (ChildCareCalc::isApplicable(workerData, piaData, ioasdi)) {
      method = new ChildCareCalc(workerData, piaData, piaParams, maxYear);
      piaMethod.push_back(method);
      childCareCalc = dynamic_cast<ChildCareCalc *>(method);
   }
   if (DibGuar::isApplicable(workerData, piaData, ioasdi)) {
      method = new DibGuar(workerData, piaData, piaParams, maxYear);
      piaMethod.push_back(method);
      dibGuar = dynamic_cast<DibGuar *>(method);
   }
   for (vector< PiaMethod * >::iterator iter = piaMethod.begin();
   iter != piaMethod.end(); iter++) {
      (*iter)->setApplicable(PiaMethod::APPLICABLE);
      (*iter)->setIoasdi(ioasdi);
      (*iter)->setPebsAssumptions(pebsAssumptions);
      (*iter)->calculate();
   }

   resetAmend90();
   piaCal1();
   piaCal2();
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": Returning from PiaCalPL::piaCal";
      Trace::writeLine(s);
   }
#endif
}

// Description: Calculates PIA and benefit.
//
// Arguments:
//   entDate: The date of entitlement.
void PiaCalPL::calculate2( const DateMoyr& entDate )
{
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": At top of PiaCalPL::calculate2";
      Trace::writeLine(s);
   }
#endif
   PiaCal::calculate2(entDate);
   // calculate benefit
   piaCal(entDate);
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": Returning from PiaCalPL::calculate2";
      Trace::writeLine(s);
   }
#endif
}

// Description: Calculates PIA for one re-indexed widow(er).
//
// Arguments:
//   widowData: Widow basic data.
//   widowPiaData: Computed information for widow.
//   secondary: Benefits for widow.
void PiaCalPL::reindWidCal( const WorkerDataGeneral& widowData,
const PiaData& widowPiaData, Secondary& secondary )
{
   if (ReindWid::isApplicable(workerData, piaData, widowPiaData.getEligYear(),
      secondary)) {
      ReindWid *reindWid = new ReindWid(workerData, piaData, piaParams,
         piaParams.getMaxyear(), widowPiaData, widowData, secondary);
      widowMethod.push_back(reindWid);
      reindWid->setApplicable(PiaMethod::APPLICABLE);
      reindWid->setIoasdi(ioasdi);
      reindWid->setPebsAssumptions(pebsAssumptions);
      reindWid->calculate();
   }
}
