// $Id: PiaMethod.cpp 1.54 2007/03/09 17:09:24EST 044579 Development  $
//
// Functions to calculate a PIA.

#include <cmath>  // for floor and ceil
#include <numeric>
#include <algorithm>
#include <utility>  // for pair
#include "PiaMethod.h"
#include "oactcnst.h"
#include "UserAssumptions.h"
#include "PiaException.h"
#include "piaparms.h"
#include "DebugCase.h"
#if defined(DEBUGCASE)
#include "DateFormatter.h"
#include "Trace.h"
#include "comma.h"
#endif

using namespace std;

// Description: Initializes a PiaMethod instance.
//
// Arguments:
//   newWorkerData: Worker data.
//   newPiaData: Pia calculation data.
//   newPiaParams: Pia calculation parameters.
//   newMaxyear: Maximum year of projection.
//   newTitle: Title of this method.
//   newMethod: Number of this method.
PiaMethod::PiaMethod( const WorkerDataGeneral& newWorkerData,
const PiaData& newPiaData, const PiaParams& newPiaParams,
int newMaxyear, const std::string& newTitle, pia_type newMethod ) :
ame(0.0), ameTotal(0.0), cap(0.0), ftearn(0.0), iapp(NOT_APPLICABLE),
indCap(NO_CAP), ioasdi(WorkerDataGeneral::NO_BEN), tableNum(NO_TABLE),
piasub(0.0), methodType(newMethod), mfbsub(0.0), pebsAssumptions(false),
piaEligTotal(), piaTotal(), piaWindfall(), methodTitle(newTitle),
workerData(newWorkerData), piaData(newPiaData), piaParams(newPiaParams),
dirty(false), iorder(YEAR37, newMaxyear), mfbElig(YEAR37, newMaxyear),
mfbEnt(), piaElig(YEAR37, newMaxyear), piaEnt(), earnMultiplied(YEAR37,
newMaxyear), earnIndexed(YEAR37, newMaxyear)
{
   fill_n(yearCpi, (int)MAXIYCPI, 0);
}

// Description: Destructor.
PiaMethod::~PiaMethod()
{ }

// Description: Initializes computed values to zero.
void PiaMethod::initialize()
{
   if (dirty) {
      fill_n(yearCpi, (int)MAXIYCPI, 0);
      iapp = NOT_APPLICABLE;
      indCap = NO_CAP;
      earnIndexed.deleteContents();
      iorder.deleteContents();
      piaEnt.set(0.0);
      ame = cap = 0.0;
      mfbEnt.set(0.0);
      piaElig.deleteContents();
      mfbElig.deleteContents();
      dirty = false;
   }
}

// Description: Saves pia before application of windfall elimination provision.
void PiaMethod::setPiaWindfall()
{
   piaWindfall.set(piaElig[yearCpi[FIRST_YEAR]]);
}

// Description: Calculates number of years of coverage for special minimum
//   or windfall elimination provision.
//
// Remarks: Calculates number of years of coverage for special minimum if
//   method is SPEC_MIN; otherwise calculates number of years of coverage for
//   windfall elimination provision. Calls specMinYearsCal to do the
//   calculation.
//
// Returns: Number of years of coverage.
int PiaMethod::specMinYearsTotalCal()
{
   const double pre1951 = piaData.getEarnTotal50(workerData.getTotalize() ?
      PiaData::EARN_WITH_TOTALIZATION : PiaData::EARN_NO_TOTALIZATION);
   const int i3 = min(workerData.getIend(), piaData.getEarnYear());
   if (methodType == SPEC_MIN) {
      return(specMinYearsCal(pre1951, piaData.earnOasdi, i3,
         piaParams.getYocAmountSpecMinArray(), iorder));
   }
   else {
      return(specMinYearsCal(pre1951, piaData.earnOasdi, i3,
         piaParams.getYocAmountWindfallArray()));
   }
}

// Description: Calculates number of years of coverage, using passed argument
//   for amounts required for coverage.
//
// Returns: Number of years of coverage.
//
// Arguments:
//   pre1951: Pre-1951 earnings.
//   earnings: Annual earnings, starting in 1951.
//   year: Last year of earnings considered.
//   yocAmount: Amount required for year of coverage.
int PiaMethod::specMinYearsCal( double pre1951, const DoubleAnnual& earnings,
int year, const DoubleAnnual& yocAmount )
{
   int totyears = min(14, static_cast<int>(floor(pre1951 / 900.)));
   // count years after 1950
   for (int i1 = YEAR51; i1 <= year; i1++) {
      if (earnings[i1] > yocAmount[i1] - .009) {
         totyears++;
      }
   }
   return(totyears);
}

// Remarks: The version with the iordert argument saves indicators for years
//   used.
//
// Arguments:
//   iordert: 1 if year of coverage, 0 otherwise, for years after 1950;
//     total pre-1951 years of coverage for 1950.
int PiaMethod::specMinYearsCal( double pre1951, const DoubleAnnual& earnings,
int year, const DoubleAnnual& yocAmount, IntAnnual& iordert )
{
   int totyears = min(14, static_cast<int>(floor(pre1951 / 900.)));
   iordert[YEAR50] = totyears;
   // count years after 1950
   for (int i1 = YEAR51; i1 <= year; i1++) {
      if (earnings[i1] > yocAmount[i1] - .009) {
         iordert[i1] = 1;
         totyears++;
      }
   }
   return(totyears);
}

// Description: Saves pia before application of totalization provision.
void PiaMethod::setPiaTotal()
{
   piaTotal = piaEnt;
}

// Description: Saves pia at eligibility before application of totalization
//   provision.
void PiaMethod::setPiaEligTotal()
{
   piaEligTotal.set(piaElig[yearCpi[FIRST_YEAR]]);
}

// Description: Saves aime before application of totalization provision.
void PiaMethod::setAmeTotal()
{
   ameTotal = getAme();
}

// Description: Calculates portion of pia at eligibility in each section of
//   mfb formula.
//
// Arguments:
//   piasub: Pia to portion.
//   piapart: Parts of the pia.
//   bendMfbTemp: Bend points to use.
void PiaMethod::setPortionPiaElig( double piasub, double piapart[],
const BendMfb& bendMfbTemp )
{
   // calculate portion of PIA in each interval
   piapart[0] = min(piasub, bendMfbTemp[1]);
   piapart[1] = max(0., min(piasub - bendMfbTemp[1],
      bendMfbTemp[2] - bendMfbTemp[1]));
   piapart[2] = max(0., min(piasub - bendMfbTemp[2],
      bendMfbTemp[3] - bendMfbTemp[2]));
   piapart[3] = max(piasub - bendMfbTemp[3], 0.);
}

// Description: Sets years of benefit increases.
void PiaMethod::setYearCpi()
{
   int i1 = workerData.getBenefitDate().getYear();
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ", BenefitDate().getYear() ";
      s += poutNoComma(i1);
      s += ": Setting yearCpi[YEAR_BEN] in PiaMethod::setYearCpi";
      Trace::writeLine(s);
   }
#endif
   yearCpi[YEAR_BEN] =
      (static_cast<int>(workerData.getBenefitDate().getMonth()) <
      piaParams.getMonthBeninc(i1)) ? i1 - 1 : i1;
   DateMoyr dateMoyr = (ioasdi == WorkerDataGeneral::SURVIVOR) ?
      DateMoyr(workerData.getDeathDate()) : workerData.getEntDate();
   i1 = dateMoyr.getYear();
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ", ioasdi ";
      s += poutNoComma(static_cast<int>(ioasdi));
      if (ioasdi == WorkerDataGeneral::SURVIVOR) {
         s += ", DeathDate().getYear() ";
         s += poutNoComma(i1);
      }
      else {
         s += ", EntDate().getYear() ";
         s += poutNoComma(i1);
      }
      s += ": Setting yearCpi[YEAR_ENT] in PiaMethod::setYearCpi";
      Trace::writeLine(s);
   }
#endif
   yearCpi[YEAR_ENT] = (static_cast<int>(dateMoyr.getMonth()) <
      piaParams.getMonthBeninc(i1)) ? i1 - 1 : i1;
}

// Description: Applies benefit increases to 1977 Amendments PIA or MFB.
//
// Returns: PIA or MFB increased by benefit increases.
//
// Remarks: The first benefit increase applied is for year year1; it is applied
// to pia77 at index (year1 - 1). The values are calculated and saved in pia77
// sequentially.
//
// Arguments:
//   pia77: PIA or MFB to be increased.
//   year1: First year of benefit increases.
//   dateMoyr2: Date up to which to apply benefit increases.
//   catchupYear: Year of eligibility for catch-up benefit increases.
double PiaMethod::applyColasElig( DoubleAnnual& pia77, int year1,
 const DateMoyr& dateMoyr2, int catchupYear ) const
{
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      ostringstream strm;
      strm << workerData.getIdString() << ", year1 " << year1
         << ", dateMoyr2 " << DateFormatter::toString(dateMoyr2)
         << ", catchupYear " << catchupYear
         << ": At top of PiaMethod::applyColas";
      Trace::writeLine(strm.str());
   }
#endif
   // do not apply any increases prior to 1979
   if (year1 < YEAR79) {
      return(pia77[year1]);
   }
   const int year2 = dateMoyr2.getYear();
   const int year3 = (static_cast< int >(dateMoyr2.getMonth()) <
      piaParams.getMonthBeninc(year2)) ? year2 - 1 : year2;
   // check for special 1999 increase (with extra 0.1 percent)
   // must be eligible in 1999 or earlier, and benefit in August 2001 or later
   if (year1 <= 1999 && !(dateMoyr2 < PiaParams::amend01)) {
      // apply increases through 1998
      for (int i1 = year1; i1 <= 1998; i1++) {
#if defined(DEBUGCASE)
         if (isDebugPid(workerData.getIdNumber())) {
            string s("");
            s += workerData.getIdString();
            s += ", cola for year ";
            s += poutNoComma(i1);
            s += ": Calling piaParams.applyCola";
            Trace::writeLine(s);
         }
#endif
         pia77[i1] = piaParams.applyCola(pia77[i1 - 1], i1);
      }
      // apply 1999 special increase
#if defined(DEBUGCASE)
      if (isDebugPid(workerData.getIdNumber())) {
         string s("");
         s += workerData.getIdString();
         s += ": Calling piaParams.applyCola99";
         Trace::writeLine(s);
      }
#endif
      pia77[1999] = piaParams.applyCola99(pia77[1998]);
      // apply increases after 1999
      for (int i1 = 2000; i1 <= year3; i1++) {
#if defined(DEBUGCASE)
         if (isDebugPid(workerData.getIdNumber())) {
            string s("");
            s += workerData.getIdString();
            s += ", cola for year ";
            s += poutNoComma(i1);
            s += ": Calling piaParams.applyCola";
            Trace::writeLine(s);
         }
#endif
         pia77[i1] = piaParams.applyCola(pia77[i1 - 1], i1, catchupYear);
      }
   }
   // otherwise use usual benefit increases
   else {
      // if year of benefit equals year of eligibility, and benefit date is
      // prior to benefit increase month, there are no CPI increases applied
      for (int i1 = year1; i1 <= year3; i1++) {
#if defined(DEBUGCASE)
         if (isDebugPid(workerData.getIdNumber())) {
            string s("");
            s += workerData.getIdString();
            s += ", cola for year ";
            s += poutNoComma(i1);
            s += ": Calling piaParams.applyCola";
            Trace::writeLine(s);
         }
#endif
         pia77[i1] = piaParams.applyCola(pia77[i1 - 1], i1, catchupYear);
#if defined(DEBUGCASE)
         if (isDebugPid(workerData.getIdNumber())) {
            string s("");
            s += workerData.getIdString();
            s += ", amount with cola ";
            s += poutNoComma(pia77[i1], 6, 2);
            s += ": After calling piaParams.applyCola";
            Trace::writeLine(s);
         }
#endif
      }
   }
   return(pia77[year3]);
}

// Description: Orders earnings and sets indicator for the highest years.
//
// Remarks: The number of years selected could be less than the desired
//   number, if there are not that many years of earnings.
//
// Arguments:
//   first: First year of earnings.
//   last: Last year of earnings.
//   number: Number of years to be selected.
void PiaMethod::orderEarnings( int first, int last, int number )
{
   int i1,i2;  // loop counters
   vector< pair< double, int > > wearn;  // earnings array

   int numtosort = last - first + 1;
   if (numtosort > 0) {
      for (i1 = 0; i1 < numtosort; i1++) {  // initialize array
         wearn.push_back(make_pair(earnIndexed[first + i1], first + i1));
      }
      sort(wearn.begin(),wearn.end());
      i2 = max(numtosort - number, 0);
      for (i1 = i2; i1 < numtosort; i1++) {
         iorder[wearn[i1].second] = 1;
      }
   }
}

// Description: Computes total earnings and AME or AIME.
//
// Remarks:  Highest earning years must already be selected.  If there are not
//   enough selected years, years of zero earnings will be added.
//
// Throws: PiaException of type PIA_IDS_METHOD if there is an inconsistency in
//   the number of years desired and the ordered earnings (only in debug mode).
//
// Arguments:
//   first: First year of earnings.
//   last: Last year of earnings.
//   number: Number of years to be selected.
void PiaMethod::totalEarnCal( int first, int last, int number )
{
   double tearn = 0.;  // sum of high n years of earnings
   int count = 0;  // number of years used in sum

   for (int i1 = first; i1 <= last; i1++) {
      if (iorder[i1] == 1) {
         tearn += earnIndexed[i1];
         count++;
      }
   }
#if !defined(NDEBUG)
   if (count <= 0 || count > number)
      throw PiaException(PIA_IDS_METHOD);
#endif
   setftearn(tearn);
   setAme(floor(tearn / (static_cast<double>(number) * 12.)));
}

// Description: Calculates pro rata totalization benefit.
void PiaMethod::prorate()
{
   setPiaEligTotal();
   setPiaTotal();
   const double piat =
      BenefitAmount::round(piaData.getQcTotal() * piaEnt.get() /
      static_cast<double>(4 * piaData.compPeriodNew.getN()),
      yearCpi[YEAR_BEN]);
   piaEnt.set(piat);
   piaElig[yearCpi[FIRST_YEAR]] = piaParams.deconvertPia(piaData.getEligYear(),
      yearCpi[YEAR_BEN] - yearCpi[FIRST_YEAR], piaEnt.get(),
      workerData.getBenefitDate());
}

// Description: Calculates an MFB at eligibility under the 1977 law.
//
// Returns: MFB at eligibility.
//
// Arguments:
//   portionPiaElig: Portion of mfb in each formula interval.
//   percMfbTemp: 4 mfb formula percentages.
//   year: Year to use for rounding.
double PiaMethod::mfbCal( const double portionPiaElig[],
const PercMfb& percMfbTemp, int year )
{
   //  multiply percent by PIA portion and add
   double temp = inner_product(percMfbTemp.begin(), percMfbTemp.end(),
      portionPiaElig, 0.0);
   return BenefitAmount::round(temp, year);
}

// Description: Applies DI maximum provision, including COLAs to date of
//   benefit.
//
// Returns: MFB from DI maximum provision, including COLAs to date of benefit.
//
// Arguments:
//   amedimax: Ame for DI maximum calculation.
//   piamax: Pia at date of benefit.
double PiaMethod::diMax( double amedimax, double piamax )
{
   double capt;  // percentage used in calculation
   dis_mfb_cap_type icapt;  // indicator for type of cap

   const int year = piaData.getEligYear() - 1;  // year of initial benefit
   mfbElig[year] = dimaxElig(amedimax, piaElig[year], year, capt, icapt);
   setCap(capt);
   setIndCap(icapt);
   return(max(applyColas(mfbElig, piaData.getEligYear(),
      workerData.getBenefitDate()), piamax));
}

// Description: Applies DI maximum provision as of year of eligibility.
//
// Returns: MFB at eligibility.
//
// Arguments:
//   amedimax: Aime for DI maximum calculation.
//   piaelig: Pia as of year of eligibility.
//   year: Year prior to year of eligibility.
//   capt: Percentage used in calculation (1.5, .85, or 1.0).
//   icapt: Type of calculation (returned).
double PiaMethod::dimaxElig( double amedimax, double piaelig,
int year, double& capt, dis_mfb_cap_type& icapt )
{
   double mfbelig = BenefitAmount::round(0.85 * amedimax, year);
   double mfbelt = BenefitAmount::round(1.5 * piaelig, year);
   if (mfbelig > mfbelt) {
      // cap is 150% of PIA
      capt = 1.5;
      icapt = PIA_1_5;
      mfbelig = mfbelt;
   }
   else if (mfbelig < piaelig) {
      // MFB is at floor of 100% of PIA
      capt = 1.0;
      icapt = PIA_1_0;
      mfbelig = piaelig;
   }
   else {
      // cap is 85% of AIME
      capt = 0.85;
      icapt = AIME_85;
   }
   return(mfbelig);
}

// Description: Returns windfall indicator.
//
// Returns: NOWINDFALLELIM.
PiaMethod::WindfallElimType PiaMethod::getWindfall() const
{
   return(NOWINDFALLELIM);
}

// Description: Returns number of years of coverage for windfall elimination
//   provision.
//
// Returns: 0.
int PiaMethod::getYearsTotal() const
{
   return(0);
}

// Description: Determines if windfall elimination provision applies.
//
// Remarks: The windfall elimination provision applies to old-age and
//   disability benefits, not survivor benefits. It also applies only if the
//   year of eligibility is 1986 or later, and if there is a noncovered
//   pension with an entitlement date after 1985.
//
// Returns: True if windfall elimination provision applies, false otherwise.
bool PiaMethod::wep_app() const
{
   if (piaData.getEligYear() > 1985 && ioasdi != WorkerDataGeneral::SURVIVOR
        && (workerData.getBenefitDate() >= workerData.getPubpenDate()))
      return(piaData.getPubpenApp(workerData) > 0.f
        && (workerData.getPubpenDate().getYear() == 0
        || workerData.getPubpenDate().getYear() > 1985));
   else
      return(false);
}
