// $Id: PiaTable.cpp 1.37 2006/12/06 07:47:05EST 044579 Development  $
//
// Functions to calculate a Pia Table PIA.

#include <fstream>
#include <utility>  // for rel_ops
#include "PiaTable.h"
#include "oactcnst.h"
#include "PiaException.h"
#include "piaparms.h"
#include "DebugCase.h"
#if defined(DEBUGCASE)
#include "Trace.h"
#include "comma.h"
#endif

using namespace std;
#if !defined(__SGI_STL_INTERNAL_RELOPS)
using namespace std::rel_ops;
#endif

// Description: Initializes a PiaTable instance.
//
// Arguments:
//   newWorkerData: Worker basic data.
//   newPiaData: Pia calculation data.
//   newPiaParams: Pia calculation parameters.
//   newMaxyear: Maximum year of projection.
PiaTable::PiaTable( const WorkerDataGeneral& newWorkerData,
const PiaData& newPiaData, const PiaParams& newPiaParams, int newMaxyear ) :
OldPia(newWorkerData, newPiaData, newPiaParams, newMaxyear,
"New-Start Calculation (pre-1977 Act)", PIA_TABLE)
{
   initialize();
}

// Description: Destructor.
PiaTable::~PiaTable()
{ }

// Description: Determines applicability of method.
//
// Remarks: The version with no arguments calls the static version with 3
//   arguments.
//
// Returns: True if method is applicable.
bool PiaTable::isApplicable()
{
   if (isApplicable(workerData, piaData, ioasdi)) {
      iapp = APPLICABLE;
      return(true);
   }
   return(false);
}

// Arguments:
//   workerData: Worker basic data.
//   piaData: Pia calculation data.
//   ioasdi: Type of benefit.
bool PiaTable::isApplicable( const WorkerDataGeneral& workerData,
const PiaData& piaData, WorkerDataGeneral::ben_type ioasdi )
{
   if (piaData.getEligYear() < YEAR79 &&
      workerData.getBenefitDate() >= PiaParams::amend52 &&
      (ioasdi != WorkerDataGeneral::SURVIVOR ||
      DateMoyr(workerData.getDeathDate()) >= PiaParams::amend52) ) {
      return(true);
   }
   return(false);
}

// Description: Computes Pia Table PIA.
//
// Throws: PiaException of type PIA_IDS_PIATABLE1 if year of entitlement is out
//   of range (only in debug mode). PiaException of type PIA_IDS_PIATABLE2 if
//   year of benefit is out of range (only in debug mode).
void PiaTable::calculate()
{
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": Starting PiaTable::calculate";
      Trace::writeLine(s);
   }
#endif
   DateMoyr dateMoyr = (ioasdi == WorkerDataGeneral::SURVIVOR) ?
      DateMoyr(workerData.getDeathDate()) : workerData.getEntDate();
#if !defined(NDEBUG)
   if (dateMoyr.getYear() < YEAR37)
      throw PiaException(PIA_IDS_PIATABLE1);
#endif
   int i1 = dateMoyr.getYear();
   yearCpi[YEAR_ENT] = (i1 < YEAR51) ? i1 :
      (static_cast<int>(dateMoyr.getMonth()) >=
         piaParams.getMonthBeninc(i1)) ? i1 : i1 - 1;
#if !defined(NDEBUG)
   if (workerData.getBenefitDate().getYear() < YEAR37)
      throw PiaException(PIA_IDS_PIATABLE2);
#endif
   i1 = workerData.getBenefitDate().getYear();
   yearCpi[YEAR_BEN] =
      (static_cast<int>(workerData.getBenefitDate().getMonth()) >=
      piaParams.getMonthBeninc(i1)) ? i1 : i1 - 1;
   yearCpi[FIRST_YEAR] = 1975;
   const AverageWage& earnings = workerData.getTotalize() ?
      piaData.earnTotalizedLimited : piaData.earnOasdiLimited;
   i1 = piaData.getEarn50(PiaData::EARN_WITH_TOTALIZATION);
   int i4 = piaData.getEarnYear();
   for (int i2 = i1; i2 <= i4; i2++) {
      if (!piaData.freezeYears.isFreezeYear(i2))
         earnIndexed[i2] = earnings[i2];
   }
   int i3 = piaData.compPeriodNew.getN();
   orderEarnings(i1, i4, i3);
   totalEarnCal(i1, i4, i3);
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ", ame ";
      s += poutNoComma(getAme(), 5, 0);
      s += ": After totalEarnCal";
      Trace::writeLine(s);
   }
#endif
   if (workerData.getBenefitDate() < PiaParams::amend742) {
      setTableNum(oldPiaCal());
   }
   else {
      setTableNum(cpiBase(workerData.getBenefitDate(), false, getAme(), 
         !workerData.getTotalize()));
   }
   piaEnt.set(piasub);
   mfbEnt.set(mfbsub);
   if (workerData.getTotalize()) {
      piaElig[yearCpi[FIRST_YEAR]] = piaEnt.get();
      prorate();
      piaEnt.set(piaElig[yearCpi[FIRST_YEAR]]);
      static_cast<void>(mfbOldCal(true));
   }
   setDirty();
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": Returning from PiaTable::calculate";
      Trace::writeLine(s);
   }
#endif
}
