// $Id: QcArray.cpp 1.29 2006/11/08 11:05:34EST 044579 Development  $
//
// Functions to handle an array of annual number of quarters of coverage.

#include <utility>  // for rel_ops
#include "QcArray.h"
#include "comma.h"
#include "oactcnst.h"
#include "PiaException.h"
#include "Resource.h"

using namespace std;
#if !defined(__SGI_STL_INTERNAL_RELOPS)
using namespace std::rel_ops;
#endif

// Description: Constructor.
//
// Remarks: The version with two integer arguments initializes the first and
// last years, resizes the data array, and sets all data values to zero. The
// version with a QcArray argument copies all values from the argument to this
// instance.
//
// Throws: PiaException of type PIA_IDS_QCARRAY1 if first year is out of range
//   (only in debug mode).
//
// Arguments:
//   firstyr: New first year of data.
//   lastyr: New last year of data.
QcArray::QcArray( int firstyr, int lastyr ) :
firstYear((unsigned short)firstyr), lastYear((unsigned short)lastyr),
theData((lastyr - firstyr + 1) / QcPacked::NUM_QCS + 1)
{
#ifndef NDEBUG
   if (firstyr < YEAR37)
      throw PiaException(PIA_IDS_QCARRAY1);
#endif
}

// Arguments:
//   qcArray: The QcArray to copy from.
QcArray::QcArray( const QcArray& qcArray ) : firstYear(qcArray.firstYear),
lastYear(qcArray.lastYear), theData(qcArray.theData)
{ }

// Arguments:
//   firstyr: First year to zero.
//   lastyr: Last year to zero.
void QcArray::deleteContents( int firstyr, int lastyr )
{
   for (int i = max((int)firstYear, firstyr); i <= min(lastyr, (int)lastYear);
      i++)
      set(i, 0u);
}

// Description: Converts a quarters of coverage array to a string.
//
// Returns: A quarters of coverage array as a string.
//
// Arguments:
//   firstyr: First year to write.
//   lastyr: Last year to write.
std::string QcArray::toString( int firstyr, int lastyr ) const
{
   string temp("");
   for (int i = max((int)firstYear, firstyr); i <= min((int)lastYear, lastyr);
      i++) {
      temp += poutNoComma(get(i));
   }
   return temp;
}

// Description: Sets values for the specified years.
//
// Remarks: The version with value uses that amount in all specified years.
// The version with qcArray sets the values in this QcArray from another
// QcArray.
//
// Throws: PiaException of type PIA_IDS_QCARRAY1 if year is out of range
//   (only in debug mode).
//
// Arguments:
//   value: Amount to use in each year.
//   firstYeart: First year for which to set QCs.
//   lastYeart: Last year for which to set QCs.
void QcArray::assign( unsigned value, int firstYeart, int lastYeart )
{
#if !defined(NDEBUG)
   if (firstYeart < firstYear || lastYeart > lastYear) {
      throw PiaException(PIA_IDS_QCARRAY1);
   }
#endif
   for (int year = firstYeart; year <= lastYeart; year++) {
      const int yearInd = (year - firstYear) % QcPacked::NUM_QCS;
      const int dataInd = (year - firstYear) / QcPacked::NUM_QCS;
      theData[dataInd].set(yearInd, value);
   }
}

// Arguments:
//   qcArray: QcArray from which number of quarters of coverage is set.
void QcArray::assign( const QcArray &qcArray, int firstYeart, int lastYeart )
{
#if !defined(NDEBUG)
   if (firstYeart < firstYear || lastYeart > lastYear ||
      lastYeart > qcArray.lastYear) {
      throw PiaException(PIA_IDS_QCARRAY1);
   }
#endif
   for (int year = firstYeart; year <= lastYeart; year++) {
      const int yearInd = (year - firstYear) % QcPacked::NUM_QCS;
      const int dataInd = (year - firstYear) / QcPacked::NUM_QCS;
      theData[dataInd].set(yearInd, qcArray.theData[dataInd].get(yearInd));
   }
}

// Description: Sums quarters of coverage over a period of years or quarters.
//
// Returns: Sum of quarters of coverage.
//
// Throws: PiaException of type PIA_IDS_QCARRAY3 if years are out of range
//   (only in debug mode). PiaException of type PIA_IDS_QCARRAY4 if quarters
//   are out of range (only in debug mode).
//
// Arguments:
//   firstyr: First year to sum.
//   lastyr: Last year to sum.
//   startValue: Starting value.
unsigned QcArray::accumulate( int firstyr, int lastyr,
unsigned startValue ) const
{
#ifndef NDEBUG
   if (firstyr < firstYear || lastyr < firstYear || firstyr > lastYear ||
      lastyr > lastYear)
      throw PiaException(PIA_IDS_QCARRAY3);
#endif
   for (int year = firstyr; year <= lastyr; year++) {
      const int yearInd = (year - firstYear) % QcPacked::NUM_QCS;
      const int dataInd = (year - firstYear) / QcPacked::NUM_QCS;
      startValue += theData[dataInd].get(yearInd);
   }
   return(startValue);
}

// Arguments:
//   qtrYear1: First quarter and year to sum.
//   qtrYear2: Last quarter and year to sum.
//   startValue: Starting value.
unsigned QcArray::accumulate( const QtrYear& qtrYear1, const QtrYear& qtrYear2,
unsigned startValue ) const
{
   const int year1 = (int)qtrYear1.getYear();
   const int year2 = (int)qtrYear2.getYear();
#ifndef NDEBUG
   qtrYear1.check();
   qtrYear2.check();
   if (year1 < firstYear || year2 < firstYear ||
      year1 > lastYear || year2 > lastYear)
      throw PiaException(PIA_IDS_QCARRAY4);
#endif
   if (qtrYear1 > qtrYear2)
      return(startValue);
   // handle case with two different years
   if (year1 < year2) {
      // start with qc's in first year, starting with first quarter of period
      startValue += min(get(year1), 4u - qtrYear1.getQuarter());
      // continue with full years (use =, not +=, because accumulate adds
      // onto starting value of startValue)
      startValue = accumulate(year1 + 1, qtrYear2.getYear() - 1, startValue);
      // end with qc's in last year, ending with last quarter of period
      startValue += min(get(year2), qtrYear2.getQuarter() + 1);
   }
   // handle case within same year
   else {
      startValue += min(qtrYear2.getQuarter() - qtrYear1.getQuarter() + 1,
         get(year1));
   }
   return(startValue);
}

// Description: Returns the number of quarters of coverage for one year.
//
// Returns: The number of quarters of coverage for one year.
//
// Throws: PiaException of type PIA_IDS_QCARRAY1 if year is out of range.
//
// Arguments:
//   year: Year for which number of quarters of coverage is desired.
unsigned QcArray::get( int year ) const
{
#if !defined(NDEBUG)
   if (year < firstYear || year > lastYear)
      throw PiaException(PIA_IDS_QCARRAY1);
#endif
   const int index = (year - firstYear) / QcPacked::NUM_QCS;
   return theData[index].get(year - firstYear - QcPacked::NUM_QCS * index);
}

// Description: Sets the number of quarters of coverage for one year.
//
// Throws: PiaException of type PIA_IDS_QCARRAY1 if year is out of range.
//
// Arguments:
//   year: Year for which number of quarters of coverage is set.
//   value: Number of quarters of coverage for this year.
void QcArray::set( int year, unsigned value )
{
#if !defined(NDEBUG)
   if (year < firstYear || year > lastYear)
      throw PiaException(PIA_IDS_QCARRAY1);
#endif
   const int index = (year - firstYear) / QcPacked::NUM_QCS;
   theData[index].set(year - firstYear - QcPacked::NUM_QCS * index, value);
}
