// $Id: ReindWid.cpp 1.41 2007/03/08 14:11:52EST 044579 Development  $
//
// Functions to calculate a reindexed widow PIA

#include "ReindWid.h"
#include "UserAssumptions.h"
#include "Resource.h"
#include "piaparms.h"
#include "oactcnst.h"
#include "DebugCase.h"
#if defined(DEBUGCASE)
#include "Trace.h"
#include "comma.h"
#endif

using namespace std;

// Description: Initializes a ReindWid instance.
//
// Arguments:
//   newWorkerData: Worker basic data.
//   newPiaData: Pia calculation data.
//   newPiaParams: Pia calculation parameters.
//   newMaxyear: Maximum year of projection.
//   newWidowPiaData: Widow calculation data.
//   newWidowData: Widow basic data.
//   newSecondary: Secondary benefit data.
ReindWid::ReindWid( const WorkerDataGeneral& newWorkerData,
const PiaData& newPiaData, const PiaParams& newPiaParams, int newMaxyear,
const PiaData& newWidowPiaData, const WorkerDataGeneral& newWidowData,
Secondary& newSecondary ) :
WageIndGeneral(newWorkerData, newPiaData, newPiaParams, newMaxyear,
"Re-indexed Widow (1983 Act)", REIND_WID), eligYear(0),
widowPiaData(newWidowPiaData), widowData(newWidowData), secondary(newSecondary)
{ }

// Description: Destructor.
ReindWid::~ReindWid()
{ }

// Description: Initializes computed values to zero.
void ReindWid::initialize()
{
   WageIndGeneral::initialize();
   eligYear = 0;
}

// Description: Determines applicability of method.
//
// Remarks: The version with no arguments calls the static version with 4
//   arguments.
//
// Returns: True if method is applicable.
bool ReindWid::isApplicable()
{
   if (isApplicable(workerData, piaData, widowPiaData.getEligYear(),
      secondary)) {
      iapp = APPLICABLE;
      return(true);
   }
   else
      return(false);
}

// Arguments:
//   workerData: Worker basic data.
//   piaData: Pia calculation data.
//   widowEligYear: Widow(er)'s year of eligibility.
//   secondary: Widow(er)'s secondary benefit information.
bool ReindWid::isApplicable( const WorkerDataGeneral& workerData,
const PiaData& piaData, int widowEligYear, const Secondary& secondary )
{
   try {
   // date of age 62
   boost::gregorian::date age62 = Date::addYears(piaData.getBirthDateMinus1(),
      62);
   if (piaData.getEligYear() > 1978 && workerData.getIend() > YEAR50 &&
      secondary.bic.isWidow() && workerData.getDeathDate() < age62 &&
      !workerData.getTotalize()) {
      // re-indexed widow guarantee
      int year = (secondary.bic.getMajorBic() == 'W') ?
         secondary.entDate.getYear() : widowEligYear;
      if (year > 1984 || workerData.getDeathDate().year() >= 1985) {
         return(true);
      }
      else
         return(false);
   }
   else {
      return(false);
   }
   } catch (exception& e) {
      throw PiaException(string("In ReindWid::isApplicable: ") + e.what());
   } catch (...) {
      throw PiaException("In ReindWid::isApplicable: unknown exception");
   }
}

// Description: Computes reindexed widow PIA.
void ReindWid::calculate()
{
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": Starting ReindWid::calculate";
      Trace::writeLine(s);
   }
#endif
   setEligYear();
   const AverageWage& earnings = workerData.getTotalize() ?
      piaData.earnTotalizedLimited : piaData.earnOasdiLimited;
   int i1 = piaData.getEarn50(PiaData::EARN_NO_TOTALIZATION);
   int i2 = piaData.getEarnYear();
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ", first year of earnings ";
      s += poutNoComma(i1, 4);
      s += ", last year of earnings ";
      s += poutNoComma(i2, 4);
      s += ": Before indexEarnings in ReindWid::calculate";
      Trace::writeLine(s);
   }
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ", earnings in ";
      s += poutNoComma(i1, 4);
      s += " = ";
      s += poutNoComma(earnings[i1], 4);
      s += ": Before indexEarnings in ReindWid::calculate";
      Trace::writeLine(s);
   }
#endif
   indexEarnings(i1, eligYear - 2, i2, earnings,
      earnMultiplied, earnIndexed, piaParams.getFqIndex());
   int i3 = piaData.compPeriodNew.getN();
   orderEarnings(i1, i2, i3);
   totalEarnCal(i1, i2, i3);
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ", pia at entitlement ";
      s += piaEnt.toString();
      s += ": After totalEarnCal in ReindWid::calculate";
      Trace::writeLine(s);
   }
#endif
   // project applicable bend points from 1979 bend points
   bendPointCal(eligYear, bendPia);
   // find portion of AIME in each AIME interval
   setPortionAime(ame, portionAime, bendPia);
   // set pia formula percentages
   piaParams.percPiaCal(eligYear, percPia);
   yearCpi[FIRST_YEAR] = eligYear - 1;
   yearCpi[YEAR_ELIG] = eligYear;
   piaEnt.set(aimepiaCal(portionAime, percPia, eligYear - 1));
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ", pia at entitlement ";
      s += piaEnt.toString();
      s += ", yearCpi[FIRST_YEAR] ";
      s += poutNoComma(yearCpi[FIRST_YEAR]);
      s += ": After aimepiaCal in ReindWid::calculate";
      Trace::writeLine(s);
   }
#endif
   piaElig[yearCpi[FIRST_YEAR]] = piaEnt.get();
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": Ready to call setYearCpi in ReindWid::calculate";
      Trace::writeLine(s);
   }
#endif
   setYearCpi();
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": After setYearCpi in ReindWid::calculate";
      Trace::writeLine(s);
   }
#endif
   piaEnt.set(applyColas(piaElig, eligYear, workerData.getBenefitDate()));
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ", pia at entitlement ";
      s += piaEnt.toString();
      s += ": After applyColas in ReindWid::calculate";
      Trace::writeLine(s);
   }
#endif
   if (eligYear > UserAssumptions::getIstart() && isPebsAssumptions()) {
      piaRealWageGain = piaEnt;
      realWageGainAdj(eligYear);
   }
   setDirty();
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": Returning from ReindWid::calculate";
      Trace::writeLine(s);
   }
#endif
}

// Description: Sets year of eligibility in reindexed widow guarantee method.
void ReindWid::setEligYear()
{
   eligYear = min(max(static_cast<int>(piaData.getEligDate().getYear()),
      widowPiaData.getEligYear()),
      static_cast<int>(piaData.getBirthDateMinus1().year()) + 62);
}
