// $Id: RetireTest.cpp 1.13 2005/08/24 14:14:40EDT 044579 Exp  $
//
// Functions to manage retirement test calculations.

#include "RetireTest.h"

using namespace std;

// Description: Constructor.
//
// Remarks: The version with newBaseYear calls the constructors for rtea62m,
// rtea62, rtea65m, and rtea65 with 2 arguments (newBaseYear and newLastYear).
// The version without newBaseYear calls the constructors for those members
// with 1 argument (newLastYear), which implies a first year of 1937.
//
// Arguments:
//   newPiaParams: Pia calculation parameters.
//   newBaseYear: First year allowed. If omitted, first year is 1937.
//   newLastYear: Last year allowed.
RetireTest::RetireTest( const PiaParams& newPiaParams, int newBaseYear,
int newLastYear ) : rtea62m(newBaseYear, newLastYear),
rtea62(newBaseYear, newLastYear), rtea65m(newBaseYear, newLastYear),
rtea65(newBaseYear, newLastYear), rteaHigh(), piaParams(newPiaParams)
{ }

RetireTest::RetireTest( const PiaParams& newPiaParams, int newLastYear ) :
rtea62m(newLastYear), rtea62(newLastYear), rtea65m(newLastYear),
rtea65(newLastYear), rteaHigh(), piaParams(newPiaParams)
{ }

// Description: Returns how many months of person's benefit will be withheld
//   according to the retirement test.
//
// Remarks: Some benies (i.e. disabled workers and disabled widow(er)s) are
// not affected by the retirement test and should not have this function
// called for them.
//
// Returns: How many months of person's benefit will be withheld
//   according to the retirement test.
//
// Arguments:
//   age: The age at end of year.
//   earnings: Earnings in year.
//   year: Year being considered.
//   benefit: Monthly benefit before reduction.
double RetireTest::earningsTestCal( const Age& age, double earnings, int year,
double benefit ) const
{
   // no months of benefits if no benefit
   if (benefit <= 0.0)
      return 0.0;

   double reduction = 0.0;  // amount of annual reduction in benefits

   if (age < RtGeneral::getMaxAgePL(year)) {
      if (year > 1977 && age < piaParams.maxDibAgePL(year)) {
         if (earnings > rtea62[year]) {
            reduction = static_cast<int>((earnings - rtea62[year]) /
               Rtea62m::getFraction(year));
         }
      }
      else if (year > 1960 && year < 1973) {
         if (earnings > rteaHigh[year]) {
            reduction = static_cast<int>((rteaHigh[year] - rtea65[year]) /
               Rtea65m::getFraction(year));
            reduction += static_cast<int>((earnings - rteaHigh[year]) /
               RteaHigh::getFraction());
         }
         else if (earnings > rtea65[year]) {
            reduction = static_cast<int>((earnings - rtea65[year]) /
               Rtea65m::getFraction(year));
         }
      }
      else {
         if (earnings > rtea65[year]) {
            reduction = static_cast<int>((earnings - rtea65[year]) /
               Rtea65m::getFraction(year));
         }
      }
   }

   // calculate number of months of benefit lost
   return min(reduction / benefit, 12.0);
}

// Description: Projects all retirement test exempt amounts.
//
// Arguments:
//   firstYear: First year of projection.
void RetireTest::project( int firstYear )
{
   rtea62m.project(firstYear, piaParams.getFqArray(),
      piaParams.getCpiincArray());
   rtea62.convertMonthToAnnual(rtea62m);
   rtea65m.project(firstYear, piaParams.getFqArray(),
      piaParams.getCpiincArray());
   rtea65.convertMonthToAnnual(rtea65m);
}
