// $Id: ScaledEarnFactors.cpp 1.7 2006/01/05 10:46:36EST 044579 Exp 277133(2007/05/22 08:55:33EDT) $
//
// Functions to manipulate scaled earnings factors.

#include "ScaledEarnFactors.h"
#include "Trace.h"
#include "comma.h"
#include "EarnProject.h"
#include "avgwg.h"

using namespace std;

// Description: Initializes scaled earnings factors.
//
// Remarks: Resizes highFactors, averageFactors, and lowFactors to
// newLastAge - newFirstAge + 1.
//
// Arguments:
//   newFirstAge: First age of data.
//   newLastAge: Last age of data.
ScaledEarnFactors::ScaledEarnFactors( int newFirstAge, int newLastAge ) :
highFactors(newLastAge - newFirstAge + 1),
averageFactors(newLastAge - newFirstAge + 1),
lowFactors(newLastAge - newFirstAge + 1), fieldWidth(10), numDecimals(6),
firstAge(newFirstAge), lastAge(newLastAge)
{
   highFactors.assign(lastAge - firstAge + 1, 0.0);
   averageFactors.assign(lastAge - firstAge + 1, 0.0);
   lowFactors.assign(lastAge - firstAge + 1, 0.0);
}

// Description: Deletes allocated memory.
ScaledEarnFactors::~ScaledEarnFactors()
{ }

// Description: Returns high earner factor for an age.
//
// Returns: High earner factor for an age.
//
// Arguments:
//   theAge: The age for which factor is desired.
double ScaledEarnFactors::getHighEarnFactor( int theAge ) const
{
   return (theAge >= firstAge && theAge <= lastAge) ?
      highFactors[theAge - firstAge] : 0;
}

// Description: Returns average earner factor for an age.
//
// Returns: Average earner factor for an age.
//
// Arguments:
//   theAge: The age for which factor is desired.
double ScaledEarnFactors::getAverageEarnFactor( int theAge ) const
{
   return (theAge >= firstAge && theAge <= lastAge) ?
      averageFactors[theAge - firstAge] : 0;
}

// Description: Returns low earner factor for an age.
//
// Returns: Low earner factor for an age.
//
// Arguments:
//   theAge: The age for which factor is desired.
double ScaledEarnFactors::getLowEarnFactor( int theAge ) const
{
   return (theAge >= firstAge && theAge <= lastAge) ?
      lowFactors[theAge - firstAge] : 0;
}

// Description: Sets earnings in an EarnProject to high earnings between two
//   ages.
//
// Remarks: For each year, sets earnings equal to highFactors times the average
// wage.
//
// Arguments:
//   earnProject: Place to set earnings.
//   averageWage: The average wage index.
//   birthYear: Worker's year of birth.
//   ageStart: First year of high earnings.
//   ageEnd: Last year of high earnings.
void ScaledEarnFactors::setHighEarnings( EarnProject& earnProject,
const AverageWage& averageWage, int birthYear, int ageStart, int ageEnd )
{
   int startYear = max(earnProject.getFirstYear(), birthYear + ageStart);
   int endYear = min(earnProject.getLastYear(), birthYear + ageEnd);
   for (int i = startYear; i <= endYear; i++) {
      earnProject.setEarnpebs(i,
         getHighEarnFactor(i - birthYear) * averageWage[i]);
   }
}

// Description: Sets earnings in an EarnProject to average earnings between two 
//   ages.
//
// Remarks: For each year, sets earnings equal to averageFactors times the
// average wage.
//
// Arguments:
//   earnProject: Place to set earnings.
//   averageWage: The average wage index.
//   birthYear: Worker's year of birth.
//   ageStart: First year of average earnings.
//   ageEnd: Last year of average earnings.
void ScaledEarnFactors::setAverageEarnings( EarnProject& earnProject,
const AverageWage& averageWage, int birthYear, int ageStart, int ageEnd )
{
   int startYear = max(earnProject.getFirstYear(), birthYear + ageStart);
   int endYear = min(earnProject.getLastYear(), birthYear + ageEnd);
   for (int i = startYear; i <= endYear; i++) {
      earnProject.setEarnpebs(i,
         getAverageEarnFactor(i - birthYear) * averageWage[i]);
   }
}

// Description: Sets earnings in an EarnProject to low earnings between two 
//   ages.
//
// Remarks: For each year, sets earnings equal to lowFactors times the average
// wage.
//
// Arguments:
//   earnProject: Place to set earnings.
//   averageWage: The average wage index.
//   birthYear: Worker's year of birth.
//   ageStart: First year of low earnings.
//   ageEnd: Last year of low earnings.
void ScaledEarnFactors::setLowEarnings( EarnProject& earnProject,
const AverageWage& averageWage, int birthYear, int ageStart, int ageEnd )
{
   int startYear = max(earnProject.getFirstYear(), birthYear + ageStart);
   int endYear = min(earnProject.getLastYear(), birthYear + ageEnd);
   for (int i = startYear; i <= endYear; i++) {
      earnProject.setEarnpebs(i,
         getLowEarnFactor(i - birthYear) * averageWage[i]);
   }
}

// Description: Reads scaled earnings factors from file.
//
// Arguments:
//   in: Input file. (It must already be open.)
//   ageStart: The age to begin reading from file.
//   ageEnd: The age to end reading from file.
void ScaledEarnFactors::read( std::istream& in, int ageStart, int ageEnd )
{
   int temp;
   double tempFactors[3];
   for (int i = ageStart; i <= ageEnd; i++) {
      // Ignore age at beginning of line.
      in >> temp;
      in >> tempFactors[0];
      in >> tempFactors[1];
      in >> tempFactors[2];
      if (i >= firstAge && i <= lastAge) {
         lowFactors[i - firstAge] = tempFactors[0];
         averageFactors[i - firstAge] = tempFactors[1];
         highFactors[i - firstAge] = tempFactors[2];
      }
   }
}

// Description: Writes scaled earnings factors to file.
//
// Arguments:
//   out: Output file. (It must already be open.)
//   ageStart: The age to begin writing to file.
//   ageEnd: The age to end writing to file.
void ScaledEarnFactors::write( std::ostream& out, int ageStart, int ageEnd )
{
   const int i1 = max(ageStart, firstAge);
   const int i2 = min(ageEnd, lastAge);
   for (int i = i1; i <= i2; i++) {
      out << i
          << poutNoComma(lowFactors[i - firstAge], fieldWidth, numDecimals)
          << poutNoComma(averageFactors[i - firstAge], fieldWidth, numDecimals)
          << poutNoComma(highFactors[i - firstAge], fieldWidth, numDecimals)
          << endl;
   }
}
