// $Id: SgaGeneral.cpp 1.8 2006/11/08 11:05:33EST 044579 Development  $
//
// Functions to handle historical and projected SGA (Substantial Gainfull
// Activity) amounts, stored in arrays.

#include <cmath>  // for floor
#include "SgaGeneral.h"
#include "PiaException.h"

using namespace std;

// Description: Constructor.
//
// Remarks: The SGA amounts start in 1961 and go through newLastYear.
//
// Arguments:
//   newLastYear: New last year.
//   newBaseSgaYear: New year for base SGA amount for projections.
SgaGeneral::SgaGeneral( int newLastYear, int newBaseSgaYear ) :
DoubleAnnual(1961, newLastYear), baseSgaYear(newBaseSgaYear),
baseAverageWageYear(newBaseSgaYear - 2)
{ }

// Description: Destructor.
SgaGeneral::~SgaGeneral()
{ }

// Description: Converts monthly SGA amounts to annual amounts, starting with
//   the first projected year.
//
// Remarks: Historical amounts are initialized in the constructor (because, for
// some years, the annual amount is not simply 12 times the monthly amount).
//
// Arguments:
//   sgaM: Monthly SGA amounts.
void SgaGeneral::convertMonthToAnnual( const SgaGeneral& sgaM )
{
   for (int i = FIRST_PROJ_YEAR; i <= getLastYear(); i++) { 
      theData[i - baseYear] = sgaM[i] * 12.0;
   }
}

// Description: Projects the SGA monthly amounts.
//
// Arguments:
//   averageWages: Average wages to use for projection.
void SgaGeneral::project( const DoubleAnnual& averageWages )
{
   for (int i = FIRST_PROJ_YEAR; i <= getLastYear(); i++) {
      theData[i - baseYear] = project1(theData[baseSgaYear - baseYear],
         averageWages[i - 2], averageWages[baseAverageWageYear],
         theData[i - 1 - baseYear]);
   }
}
// Description: Calculates a monthly SGA amount using a base amount and
//   a ratio of average wages.
//
// Returns: Increased monthly SGA amount.
//
// Arguments:
//   baseAmount: Value of SGA amount in base year.
//   numerator: Numerator used in projection.
//   denominator: Denominator used in projection.
//   previous: Value of SGA amount in previous year.
double SgaGeneral::project1( double baseAmount, double numerator,
double denominator, double previous )
{
   // Round to the nearest 10 dollars.
   double temp = floor(((numerator / denominator) * baseAmount) / 10.0 + 0.50);
   // Make sure it's not less than last year's amount.
   return max(10.0 * temp, previous);
}
