// $Id: SpecMin.cpp 1.38 2005/10/06 11:55:55EDT 044579 Exp  $
//
// Functions to calculate a special minimum PIA.

#include "SpecMin.h"
#include "piaparms.h"
#include "oactcnst.h"
#include "DebugCase.h"
#if defined(DEBUGCASE)
#include "Trace.h"
#include "comma.h"
#endif

using namespace std;

// Description: Initializes a SpecMin instance.
//
// Remarks: Initializes the member variables to 0, then calls initialize.
//
// Arguments:
//   newWorkerData: Worker basic data.
//   newPiaData: Pia calculation data.
//   newPiaParams: Pia calculation parameters.
//   newMaxyear: Maximum year of projection.
SpecMin::SpecMin( const WorkerDataGeneral& newWorkerData,
const PiaData& newPiaData, const PiaParams& newPiaParams, int newMaxyear ) :
OldPia(newWorkerData, newPiaData, newPiaParams, newMaxyear, "Special Minimum",
SPEC_MIN), specMinYearsExcess(0), specMinAmount(0.), yearsTotal(0)
{
   initialize();
}

// Description: Destructor.
SpecMin::~SpecMin()
{ }

// Description: Initializes computed values to zero.
void SpecMin::initialize()
{
   PiaMethod::initialize();
   specMinYearsExcess = yearsTotal = 0;
   specMinAmount = 0.;
}

// Description: Determines applicability of method.
//
// Remarks: To be applicable, the benefit date must be at least
// PiaParams::amend722 (January 1973), and it cannot be a totalization benefit.
//
// The version with no arguments calls the static version with 1 argument.
//
// Returns: True if method is applicable.
bool SpecMin::isApplicable()
{
   if (isApplicable(workerData)) {
      iapp = APPLICABLE;
      return(true);
   }
   else
      return(false);
}

// Arguments:
//   workerData: Worker basic data.
bool SpecMin::isApplicable( const WorkerDataGeneral& workerData )
{
   if (!(workerData.getBenefitDate() < PiaParams::amend722) &&
      !workerData.getTotalize()) {
      return(true);
   }
   else
      return(false);
}

// Description: Computes special minimum PIA.
void SpecMin::calculate()
{
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": Starting SpecMin::calculate";
      Trace::writeLine(s);
   }
#endif
   // find years of coverage
   yearsTotal = specMinYearsTotalCal();
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ", yearsTotal = ";
      s += poutNoComma(yearsTotal);
      s += ": After specMinYearsTotalCal in SpecMin::calculate";
      Trace::writeLine(s);
   }
#endif
   const DateMoyr& benefitDate = workerData.getBenefitDate();
   setSpecMinAmount(piaParams.specMinAmountCal(benefitDate));
   // limit usable years of coverage
   setSpecMinYearsExcess(specMinYearsExcessCal());
   const int i1 = benefitDate.getYear();
   yearCpi[YEAR_BEN] = yearCpi[YEAR_ENT] =
      (static_cast<int>(benefitDate.getMonth()) >=
      piaParams.getMonthBeninc(i1)) ? i1 : i1 - 1;
   if (!(benefitDate < PiaParams::amend772)) {
      // benefit increases start in 1979
      yearCpi[FIRST_YEAR] = 1978;
      yearCpi[YEAR_ELIG] = YEAR79;
      const int excessYears = getSpecMinYearsExcess();
      piaElig[yearCpi[FIRST_YEAR]] =
         piaParams.getSpecMinPia(PiaParams::amend772, excessYears);
      // find January 1979 MFB
      mfbElig[yearCpi[FIRST_YEAR]] =
         piaParams.getSpecMinMfb(PiaParams::amend772, excessYears);
      // apply benefit increases through year before benefit year
      for (int i2 = yearCpi[1]; i2 < i1; i2++) {
         piaElig[i2] = piaParams.getSpecMinPia(DateMoyr(12, i2), excessYears);
         mfbElig[i2] = piaParams.getSpecMinMfb(DateMoyr(12, i2), excessYears);
      }
      piaElig[i1] = piaParams.getSpecMinPia(benefitDate, excessYears);
      mfbElig[i1] = piaParams.getSpecMinMfb(benefitDate, excessYears);
      // apply benefit increase in benefit year, if applicable
      mfbEnt.set(mfbElig[i1]);
      piaEnt.set(piaElig[i1]);
   }
   else {
      yearCpi[FIRST_YEAR] = 1975;
      // PIA equals dollar amount times years of coverage
      piaElig[yearCpi[FIRST_YEAR]] =
         static_cast<double>(getSpecMinYearsExcess()) * getSpecMinAmount();
      // find old-law MFB from PIA table for pre-1977 special minimum
      piaEnt.set(piaElig[yearCpi[FIRST_YEAR]]);
      setAme(mfbOldCal(false));
   }
   setDirty();
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ", pia = ";
      s += poutNoComma(piaEnt.get());
      s += ": Returning from SpecMin::calculate";
      Trace::writeLine(s);
   }
#endif
}

// Description: Limits usable years of coverage.
//
// Returns: Usable years of coverage (in excess of 10, up to 20).
int SpecMin::specMinYearsExcessCal() const
{
   // limit usable years of coverage to those in excess of 10, up to 20
   // usable years
   return(min(max(yearsTotal - 10, 0), piaParams.specMinMaxYearsPL()));
}
