// $Id: TaxBenefits.cpp 1.8 2003/01/29 15:14:44EST sfmckay Exp  $
//
// Functions to calculate tax on benefits.

#include <algorithm>
#include "TaxBenefits.h"

using namespace std;

const double TaxBenefits::SINGLEOASDITHRESHOLD = 25000.;
const double TaxBenefits::MARRIEDOASDITHRESHOLD = 32000.;
const double TaxBenefits::SINGLEOASDHITHRESHOLD = 34000.;
const double TaxBenefits::MARRIEDOASDHITHRESHOLD = 44000.;
const double TaxBenefits::SINGLEOASDHITAX = 0.5 *
   (TaxBenefits::SINGLEOASDHITHRESHOLD - TaxBenefits::SINGLEOASDITHRESHOLD);
const double TaxBenefits::MARRIEDOASDHITAX = 0.5 *
   (TaxBenefits::MARRIEDOASDHITHRESHOLD - TaxBenefits::MARRIEDOASDITHRESHOLD);

// Description: Initializes a TaxBenefits instance.
TaxBenefits::TaxBenefits()
{ }

// Description: Destructor.
TaxBenefits::~TaxBenefits()
{ }

// Description: Returns taxable benefits for OASDI trust funds.
//
// Returns: Taxable benefits for OASDI trust funds.
//
// Arguments:
//   year: Year for which benefits are being taxed.
//   magi: Modified adjusted gross income (excluding benefits).
//   benefits: Social Security benefits (including spouse's benefits, if
//     married). Should be reduced by deduction for contributions, if
//     applicable.
//   married: True if married filing jointly.
double TaxBenefits::taxOasdi( int year, double magi, double benefits,
bool married ) const
{
   if (benefits <= 0.)
      return(0.);
   double threshold = getThreshold(married, year);
   double proportion = getProportion(year);
   double magiPlusBenefits = magi + proportion * benefits;
   if (magiPlusBenefits > threshold)
      return min(proportion * (magiPlusBenefits - threshold),
         proportion * benefits);
   else
      return(0.);
}

// Description: Returns taxable benefits for OASDHI trust funds.
//
// Returns: Taxable benefits for OASDHI trust funds.
//
// Arguments:
//   year: Year for which benefits are being taxed.
//   magi: Modified adjusted gross income (excluding benefits).
//   benefits: Social Security benefits (including spouse's benefits, if
//     married, reduced by deduction for contributions).
//   married: True if married filing jointly.
double TaxBenefits::taxOasdhi( int year, double magi, double benefits,
bool married ) const
{
   if (year < 1994)
      return taxOasdi(year, magi, benefits, married);
   if (benefits <= 0.)
      return(0.);
   double threshold = married ? MARRIEDOASDHITHRESHOLD : SINGLEOASDHITHRESHOLD;
   double magiPlusHalf = magi + .5 * benefits;
   if (magiPlusHalf < threshold)
      return taxOasdi(year, magi, benefits, married);
   double part1 = married ? MARRIEDOASDHITAX : SINGLEOASDHITAX;
   double part2 = min(part1, .5 * benefits);
   double part3 = .85 * (magiPlusHalf - threshold);
   return min(.85 * benefits, part2 + part3);
}

// Description: Returns appropriate OASDI threshold.
//
// Returns: Appropriate OASDI threshold.
//
// Arguments:
//   married: True if married filing jointly.
double TaxBenefits::getThreshold( bool married, int ) const
{
   return(married ? MARRIEDOASDITHRESHOLD : SINGLEOASDITHRESHOLD);
}

// Description: Returns proportion of benefits that are taxable for OASDI.
//
// Returns: 0.5 for present law.
double TaxBenefits::getProportion( int ) const
{
   return 0.5;
}

// Description: Returns the amount of deductions allowed under the
//   Ball taxation plan.
//
// Returns: Zero for present law.
double TaxBenefits::deductionsCal( unsigned , double , double , int ) const
{
   return 0.0;
}
