// $Id: TaxBenefitsLC.cpp 1.16 2003/09/24 13:23:52EDT sfmckay Exp  $
//
// Functions to calculate tax on benefits with law changes.

#include <algorithm>
#include "TaxBenefitsLC.h"
#include "LawChangeArray.h"
#include "LawChangeTAXBENCHG.h"

using namespace std;

// Description: Initializes a TaxBenefitsLC instance.
//
// Arguments:
//   newLawChange: Law change data.
TaxBenefitsLC::TaxBenefitsLC( const LawChangeArray& newLawChange ) :
lawChange(newLawChange)
{
   thresholdsCal();
}

// Description: Destructor.
TaxBenefitsLC::~TaxBenefitsLC()
{ }

// Description: Returns taxable benefits for OASDHI trust funds (same as
//   taxable for OASDI for proposal to tax as for private pension plans).
//
// Returns: Taxable benefits for OASDHI trust funds.
//
// Arguments:
//   year: Year for which benefits are being taxed.
//   magi: Modified adjusted gross income (excluding benefits).
//   benefits: Social Security benefits (including spouse's benefits, if
//     married, reduced by deduction for contributions).
//   married: True if married filing jointly.
double TaxBenefitsLC::taxOasdhi( int year, double magi, double benefits,
bool married ) const
{
   LawChangeTAXBENCHG *lawChangeTAXBENCHG = lawChange.lawChangeTAXBENCHG;
   // assume effective for all
   if (lawChangeTAXBENCHG->isEffective(year, year)) {
      return(TaxBenefits::taxOasdi(year, magi, benefits, married));
   }
   else {
      return(TaxBenefits::taxOasdhi(year, magi, benefits, married));
   }
}

// Description: Returns the amount of deductions allowed under the
//   Ball taxation plan.
//
// Returns: Amount of deductions allowed under the Ball taxation plan.
//
// Arguments:
//   startAge: Starting age of deductions.
//   aggContribs: Aggregate contribution (no interest) over person's lifetime
//     into OASDI trust funds through taxes.
//   taxRecovered: Tax recovered through deductions.
//   year: Year for which benefits are being taxed.
double TaxBenefitsLC::deductionsCal( unsigned startAge, double aggContribs,
double taxRecovered, int year ) const
{
   LawChangeTAXBENCHG *lawChangeTAXBENCHG = lawChange.lawChangeTAXBENCHG;
   if (lawChangeTAXBENCHG->isEffective(year, year) &&
      (lawChangeTAXBENCHG->getInd() == 2 ||
      lawChangeTAXBENCHG->getInd() == 3)) {
      double deduction = 0.0;
      // Figure out deduction
      if (taxRecovered < aggContribs) {
         double payments = static_cast<double>(paymentsCal(startAge));
         deduction = (aggContribs / payments) * 12.0;
         // limit deduction to amount of outstanding contributions
         deduction = min(deduction, aggContribs - taxRecovered);
      }
      return deduction;
   }
   else {
      return TaxBenefits::deductionsCal(startAge, aggContribs, taxRecovered,
         year);
   }
}

// Description: Returns number of payments over which contributions are spread
//   for Ball taxation plan.
//
// Returns: Number of payments over which contributions are spread for Ball
//   taxation plan.
//
// Arguments:
//   age: The age of taxpayer.
int TaxBenefitsLC::paymentsCal( unsigned age )
{
   if (age <= 55) {
      return 360;
   } else if (age <= 60) {
      return 310;
   } else if (age <= 65) {
      return 260;
   } else if (age <= 70) {
      return 210;
   } else {
      return 160;
   }
}

// Description: Fills in declining threshold amounts.
void TaxBenefitsLC::thresholdsCal()
{
   thresholds[0][0] = SINGLEOASDITHRESHOLD;
   thresholds[1][0] = MARRIEDOASDITHRESHOLD;
   for (int j = 0; j < 2; j++) {
      for (int i = 1; i < 11; i++) {
         thresholds[j][i] =
            (10.0 - static_cast<double>(i)) * thresholds[j][0] / 10.0;
      }
   }
}

// Description: Returns appropriate threshold.
//
// Returns: Appropriate threshold.
//
// Arguments:
//   married: True if married filing jointly.
//   year: Year for which benefits are being taxed.
double TaxBenefitsLC::getThreshold( bool married, int year ) const
{
   LawChangeTAXBENCHG *lawChangeTAXBENCHG = lawChange.lawChangeTAXBENCHG;
   // assume effective for all
   if (lawChangeTAXBENCHG->isEffective(year, year) &&
      (lawChangeTAXBENCHG->getInd() == 1 ||
      lawChangeTAXBENCHG->getInd() == 3)) {
      int index = min(max(
         year - lawChangeTAXBENCHG->getStartYear() + 1, 0), 10);
      return thresholds[married ? 1 : 0][index];
   }
   else {
      return TaxBenefits::getThreshold(married, year);
   }
}

// Description: Returns proportion of benefits that are taxable for OASDI.
//
// Returns: Proportion of benefits that are taxable for OASDI.
//
// Arguments:
//   year: Year for which benefits are being taxed.
double TaxBenefitsLC::getProportion( int year ) const
{
   LawChangeTAXBENCHG *lawChangeTAXBENCHG = lawChange.lawChangeTAXBENCHG;
   // assume effective for all
   if (lawChangeTAXBENCHG->isEffective(year, year) &&
      (lawChangeTAXBENCHG->getInd() == 2 ||
      lawChangeTAXBENCHG->getInd() == 3)) {
      return 1.0;
   }
   else {
      return TaxBenefits::getProportion(year);
   }
}
