// $Id: TransGuar.cpp 1.33 2007/03/08 14:12:02EST 044579 Development  $
//
// Functions to calculate a transitional guarantee PIA.

#include "TransGuar.h"
#include "piaparms.h"
#include "oactcnst.h"

using namespace std;

// Description: Initializes a TransGuar instance.
//
// Arguments:
//   newWorkerData: Worker basic data.
//   newPiaData: Pia calculation data.
//   newPiaParams: Pia calculation parameters.
//   newMaxyear: Maximum year of projection.
TransGuar::TransGuar( const WorkerDataGeneral& newWorkerData,
const PiaData& newPiaData, const PiaParams& newPiaParams, int newMaxyear ) :
OldPia(newWorkerData, newPiaData, newPiaParams, newMaxyear,
"Transitional Guarantee (1977 Act)", TRANS_GUAR), bendMfb(), percMfb()
{
   for (int i2 = 0; i2 < 4; i2++)
      portionPiaElig[i2] = 0.;
}

// Description: Destructor.
TransGuar::~TransGuar()
{ }

// Description: Initializes computed values to zero.
void TransGuar::initialize()
{
   PiaMethod::initialize();
   bendMfb.deleteContents();
   for (int i2 = 0; i2 < 4; i2++)
      portionPiaElig[i2] = 0.;
}

// Description: Determines applicability of method.
//
// Remarks: The version with no arguments calls the static version with 3
//   arguments.
//
// Returns: True if method is applicable.
bool TransGuar::isApplicable()
{
   if (isApplicable(workerData, piaData, ioasdi)) {
      iapp = APPLICABLE;
      return(true);
   }
   else
      return(false);
}

// Arguments:
//   workerData: Worker basic data.
//   piaData: Pia calculation data.
//   ioasdi: Type of benefit.
bool TransGuar::isApplicable( const WorkerDataGeneral& workerData,
const PiaData& piaData, WorkerDataGeneral::ben_type ioasdi )
{
   try {
   int itrans = 5;  // transitional guarantee period
   // date of age 62
   const boost::gregorian::date age62 =
     Date::addYears(piaData.getBirthDateMinus1(), 62);
   if (piaData.getEligYear() >= YEAR79 &&
      static_cast<int>(piaData.getEligDate().getYear()) < YEAR79 + itrans &&
      ioasdi != WorkerDataGeneral::DISABILITY &&
      (ioasdi != WorkerDataGeneral::SURVIVOR ||
      !(workerData.getDeathDate() < age62)) &&
      !workerData.getTotalize() && workerData.getIbegin() < YEAR79) {
      return(true);
   }
   else {
      return(false);
   }
   } catch (exception& e) {
      throw PiaException(string("In TransGuar::isApplicable: ") + e.what());
   }
}

// Description: Computes transitional guarantee PIA.
void TransGuar::calculate()
{
   const AverageWage& earnings = workerData.getTotalize() ?
      piaData.earnTotalizedLimited : piaData.earnOasdiLimited;
   // set earnings to be used up to year prior to eligibility
   int i2 = piaData.getEarn50(PiaData::EARN_NO_TOTALIZATION);
   for (int i1 = i2; i1 < piaData.getEligYear(); i1++) {
      if (!piaData.freezeYears.isFreezeYear(i1))
         earnIndexed[i1] = earnings[i1];
   }
   // order the earnings and compute average monthly earnings
   const int i3 = piaData.getEligYear() - 1;
   const int i4 = piaData.getEligYear();
   const int i5 = piaData.compPeriodNew.getN();
   orderEarnings(i2, i3, i5);
   totalEarnCal(i2, i3, i5);
   // calculate Dec 1978 PIA
   yearCpi[FIRST_YEAR] = i3;
   yearCpi[YEAR_ELIG] = i4;
   setTableNum(cpiBase(PiaParams::amend771, true, getAme(), false));
   piaEnt.set(piasub);
   mfbEnt.set(mfbsub);
   bendMfb.project(i4, piaParams.getFqArray());
   piaElig[i3] = piaEnt.get();
   // find AIME MFB from wage-indexed formula
   setPortionPiaElig(piaElig[i3], portionPiaElig, bendMfb);
   mfbElig[i3] = mfbCal(portionPiaElig, percMfb, i3);
   // apply cpi increases to PIA and MFB
   setYearCpi();
   piaEnt.set(applyColas(piaElig, i4, workerData.getBenefitDate()));
   mfbEnt.set(applyColas(mfbElig, i4, workerData.getBenefitDate()));
   setDirty();
}
