// $Id: UserAssumptions.cpp 1.15 2005/05/06 17:51:57EDT sfmckay Exp  $
//
// Functions to initialize user-specified assumptions.

#include <locale> // for islower
#include "UserAssumptions.h"
#include "wbgenrl.h"
#include "oactcnst.h"
#include "PiaException.h"
#include "Resource.h"

using namespace std;

// Define static class members
int UserAssumptions::istart;
int UserAssumptions::maxyear;

// Description: Initializes user-specified assumptions.
//
// Arguments:
//   newLastYear: Maximum projected year.
UserAssumptions::UserAssumptions( int newLastYear ) :
awincproj(YEAR37, newLastYear), baseOasdi(YEAR37, newLastYear),
base77(YEAR37, newLastYear), biproj(YEAR51, newLastYear), catchup(), titleAw(),
titleBi(), ialtbi(AssumptionType::FLAT), anscch('N'),
ialtaw(AssumptionType::FLAT), ibasch(BaseChangeType::AUTOMATIC),
jaltbi(AssumptionType::NONE), jaltaw(AssumptionType::NONE),
jbasch(BaseChangeType::NONE)
{
   maxyear = newLastYear;
}

// Description: Initializes all variables.
void UserAssumptions::deleteContents()
{
   zeroJalt();
   ialtbi = ialtaw = AssumptionType::FLAT;
   ibasch = BaseChangeType::AUTOMATIC;
   anscch = 'N';
   titleAw = "";
   titleBi = "";
   catchup.deleteContents();
   baseOasdi.deleteContents();
   base77.deleteContents();
   biproj.deleteContents();
   awincproj.deleteContents();
}

// Description: Sets last year of data arrays.  Should be called when benefit
//   date is set in WorkerData.
//
// Arguments:
//   year: Year to use.
void UserAssumptions::setLastYear( int year )
{
   lastYear = year;
}

// Description: Checks for valid assumption for Statement.
//
// Throws: PiaException of type PIA_IDS_PEBSASM if benefit increase indicator
//   or average wage increase indicator is out of range in Statement case.
void UserAssumptions::pebsasmCheck() const
{
   try {
      if (AssumptionType::EXPANDED_PEBES) {
         AssumptionType::ialtbiCheck1(ialtbi);
         AssumptionType::ialtawCheck1(ialtaw);
      }
      else {
         AssumptionType::ialtbiCheck2(ialtbi);
         AssumptionType::ialtawCheck2(ialtaw);
      }
   } catch (PiaException&) {
      throw PiaException(PIA_IDS_PEBSASM);
   }
}

// Description: Sets average wage increase indicator.
//
// Arguments:
//   ialtawt: Value to use.
void UserAssumptions::setIaltaw( int ialtawt )
{
   AssumptionType::ialtawCheck1(ialtawt);
   ialtaw = static_cast<AssumptionType::assum_type>(ialtawt);
}

// Description: Sets benefit increase indicator.
//
// Arguments:
//   ialtbit: Value to use.
void UserAssumptions::setIaltbi( int ialtbit )
{
   AssumptionType::ialtbiCheck1(ialtbit);
   ialtbi = static_cast<AssumptionType::assum_type>(ialtbit);
}

// Description: Sets current year.
//
// Arguments:
//   istartt: Value to use.
void UserAssumptions::setIstart( int istartt )
{
   try {
      istartCheck(istartt);
      istart = istartt;
      catchup.setCstart(istartt);
   } catch (PiaException&) {
      // do not do anything yet; eventually set an indicator saying
      // assumptions are not allowed
   }
}

// Description: Sets catch-up increase indicator.
//
// Arguments:
//   ansccht: Value to use.
void UserAssumptions::setAnscch( char ansccht )
{
   anscchCheck(ansccht);
   if (islower(ansccht))
      ansccht = (char)toupper(ansccht);
   anscch = ansccht;
}

// Description: Sets wage base indicator.
//
// Arguments:
//   ibascht: Value to use.
void UserAssumptions::setIbasch( int ibascht )
{
   BaseChangeType::check(ibascht);
   ibasch = static_cast<BaseChangeType::change_type>(ibascht);
}

// Description: Checks catch-up increase indicator.
//
// Throws: PiaException of type PIA_IDS_ANSCCH if catch-up increase indicator
//   is out of range.
//
// Arguments:
//   ansccht: Value to check.
void UserAssumptions::anscchCheck( char ansccht )
{
   if (ansccht != 'Y' && ansccht != 'N' && ansccht != 'y' && ansccht != 'n')
      throw PiaException(PIA_IDS_ANSCCH);
}

// Description: Checks current year.
//
// Throws: PiaException of type PIA_IDS_START1 if starting year is before 1979.
//   PiaException of type PIA_IDS_START2 if starting year is after maximum year
//   allowed.
//
// Arguments:
//   istartt: Value to check.
void UserAssumptions::istartCheck( int istartt )
{
   if (istartt < YEAR79)
      throw PiaException(PIA_IDS_START1);
   if (istartt > maxyear)
      throw PiaException(PIA_IDS_START2);
}

// Description: Resets assumptions triggers.
//
// Remarks:  Should be called after updating assumptions for user-specified
// assumptions.
void UserAssumptions::resetTriggers()
{
   setJaltbi(getIaltbi());
   setJaltaw(getIaltaw());
   setJbasch(getIbasch());
}
