// $Id: WageInd.cpp 1.32 2005/10/06 11:55:55EDT 044579 Exp  $
//
// Functions to calculate a wage-indexed PIA.

#include "WageInd.h"
#include "UserAssumptions.h"
#include "piaparms.h"
#include "DebugCase.h"
#if defined(DEBUGCASE)
#include "Trace.h"
#include "comma.h"
#endif

using namespace std;

// Description: Initializes a WageInd instance.
//
// Arguments:
//   newWorkerData: Worker basic data.
//   newPiaData: Pia calculation data.
//   newPiaParams: Pia calculation parameters.
//   newMaxyear: Maximum year of projection.
WageInd::WageInd( const WorkerDataGeneral& newWorkerData,
const PiaData& newPiaData, const PiaParams& newPiaParams, int newMaxyear ) :
WageIndGeneral(newWorkerData, newPiaData, newPiaParams, newMaxyear,
"Wage-Indexed Formula (1977 Act)", WAGE_IND)
{ }

// Description: Destructor.
WageInd::~WageInd()
{ }

// Description: Determines applicability of method.
//
// Remarks: The version with no arguments calls the static version with 2
//   arguments.
//
// Returns: True if method is applicable.
bool WageInd::isApplicable()
{
   if (isApplicable(workerData, piaData)) {
      iapp = APPLICABLE;
      return(true);
   }
   else
      return(false);
}

// Arguments:
//   workerData: Worker basic data.
//   piaData: Pia calculation data.
bool WageInd::isApplicable( const WorkerDataGeneral& workerData,
const PiaData& piaData )
{
   if (piaData.getEligYear() > 1978 && (workerData.getIend() > 1950 ||
      workerData.getTotalize())) {
      return(true);
   }
   else
      return(false);
}

// Description: Computes wage indexed PIA.
void WageInd::calculate()
{
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": Starting WageInd::calculate";
      Trace::writeLine(s);
   }
#endif
   setTableNum(PL_1977);
   const AverageWage& earnings = workerData.getTotalize() ?
      piaData.earnTotalizedLimited : piaData.earnOasdiLimited;
   int i1 = piaData.getEarn50(PiaData::EARN_WITH_TOTALIZATION);
   int i2 = piaData.getEarnYear();
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ", first year of earnings ";
      s += poutNoComma(i1, 4);
      s += ", last year of earnings ";
      s += poutNoComma(i2, 4);
      s += ": Before indexEarnings in WageInd::calculate";
      Trace::writeLine(s);
   }
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ", earnings in ";
      s += poutNoComma(i1, 4);
      s += " = ";
      s += poutNoComma(earnings[i1], 4);
      s += ": Before indexEarnings in WageInd::calculate";
      Trace::writeLine(s);
   }
#endif
   indexEarnings(i1, piaData.getEligYear() - 2, i2, earnings, earnMultiplied,
      earnIndexed, piaParams.getFqIndex());
   int i3 = piaData.compPeriodNew.getN();
   orderEarnings(i1, i2, i3);
   totalEarnCal(i1, i2, i3);
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ", aime ";
      s += poutNoComma(ame, 6);
      s += ": After totalEarnCal in WageInd::calculate";
      Trace::writeLine(s);
   }
#endif
   const int i5 = piaData.getEligYear();
   // project applicable bend points from 1979 bend points
   bendPointCal(i5, bendPia);
   // find portion of AIME in each AIME interval
   setPortionAime(ame, portionAime, bendPia);
   // set pia formula percentages
   piaParams.percPiaCal(i5, percPia);
   const int i4 = piaData.getEligYear() - 1;
   yearCpi[FIRST_YEAR] = i4;
   yearCpi[YEAR_ELIG] = i5;
   piaElig[i4] = aimepiaCal(portionAime, percPia, i4);
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ", pia at eligibility ";
      s += poutNoComma(piaElig[i4], 6, 2);
      s += ": After aimepiaCal in WageInd::calculate";
      Trace::writeLine(s);
   }
#endif
   // apply windfall elimination provision
   if (wep_app())
      windfallCal();
   // apply benefit increases to PIA
   setYearCpi();
   piaEnt.set(applyColas(piaElig, i5, workerData.getBenefitDate()));
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ", pia at entitlement ";
      s += piaEnt.toString();
      s += ": After applyColas in WageInd::calculate";
      Trace::writeLine(s);
   }
#endif
   bendMfb.project(i5, piaParams.getFqArray());
   // apply totalization
   if (workerData.getTotalize()) {
      prorate();
      setAmeTotal();
      piasub = piaElig[i4];
      setAme(deconvertAme(bendPia, percPia));
   }
   // calculate the AIME MFB
   setPortionPiaElig(piaElig[i4], portionPiaElig, bendMfb);
   mfbElig[i4] = mfbCal(portionPiaElig, percMfb, i4);
   mfbEnt.set(applyColas(mfbElig, i5, workerData.getBenefitDate()));
   if (i5 > UserAssumptions::getIstart() && isPebsAssumptions()) {
      piaRealWageGain = piaEnt;
      mfbRealWageGain = mfbEnt;
      realWageGainAdj(i5);
   }
   setDirty();
}
