// $Id: WageIndGeneral.cpp 1.18 2005/10/06 11:55:55EDT 044579 Exp  $
//
// Functions to calculate a wage-indexed PIA.

#include <cmath>  // for floor
#include <algorithm>
#include "WageIndGeneral.h"
#include "piaparms.h"
#include "UserAssumptions.h"
#include "DebugCase.h"
#if defined(DEBUGCASE)
#include "Trace.h"
#include "comma.h"
#endif

using namespace std;

// Description: Initializes a WageIndGeneral instance.
//
// Arguments:
//   newWorkerData: Worker data.
//   newPiaData: Pia calculation data.
//   newPiaParams: Pia calculation parameters.
//   newMaxyear: Maximum year of projection.
//   newTitle: Title of this method.
//   newMethod: Number of this method.
WageIndGeneral::WageIndGeneral( const WorkerDataGeneral& newWorkerData,
const PiaData& newPiaData, const PiaParams& newPiaParams, int newMaxyear,
const std::string& newTitle, pia_type newMethod ) :
PiaMethod(newWorkerData, newPiaData, newPiaParams, newMaxyear, newTitle,
newMethod), bendMfb(), bendPia(), percMfb(), percPia(), percWind(),
mfbRealWageGain(), piaRealWageGain(), windfallInd(NOWINDFALLELIM),
yearsTotal(0), indexYearAvgWage(0.0)
{
   fill_n(portionAime, 3, 0.0);
   fill_n(portionPiaElig, 4, 0.0);
}

// Description: Destructor.
WageIndGeneral::~WageIndGeneral()
{ }

// Description: Initializes computed values to zero.
void WageIndGeneral::initialize()
{
   PiaMethod::initialize();
   windfallInd = NOWINDFALLELIM;
   yearsTotal = 0;
   bendMfb.deleteContents();
   bendPia.deleteContents();
   fill_n(portionAime, 3, 0.);
   fill_n(portionPiaElig, 4, 0.0);
   indexYearAvgWage = 0.0;
}

// Description: Applies windfall elimination provision for wage-indexed
//   calculation.
void WageIndGeneral::windfallCal()
{
   double test;  // temporary value

   // find years of coverage for wep
   yearsTotal = specMinYearsTotalCal();
   percWind.windfall(piaData.getEligYear(),
      workerData.getBenefitDate().getYear(), yearsTotal);
   if (yearsTotal >= 30)
      setWindfall(HAS30YEARS);
   else {
      setPiaWindfall();
      int eligYear = piaData.getEligYear() - 1;
      const double halfpension = BenefitAmount::round(
         0.5 * piaData.getPubpenApp(workerData), eligYear);
      const int year1 = yearCpi[FIRST_YEAR];
      const double piaelt = piaElig[year1] - halfpension;
      piaElig[year1] = piaelt;
      test = aimepiaCal(portionAime, percWind, eligYear);
      // check for one-half pension guarantee
      if (test > piaelt) {
         setWindfall(REDUCEDPERC);
         piaElig[year1] = test;
      }
      else {
         setWindfall(ONEHALFPENSION);
         piaElig[year1] = piaelt;
      }
   }
}

// Description: Returns number of years of coverage for windfall elimination
//   provision.
//
// Returns: Number of years of coverage for windfall elimination provision.
int WageIndGeneral::getYearsTotal() const
{
   return yearsTotal;
}

// Description: Indexes earnings, using specified average wage series.
//
// Arguments:
//   year1: First year of earnings.
//   year2: Base year for indexing.
//   year3: Last year of earnings.
//   earnings: Unindexed earnings.
//   uearnt: Earnings multiplied by average earnings.
//   vearn: Indexed earnings.
//   fq: Indexing series.
void WageIndGeneral::indexEarnings( int year1, int year2, int year3,
const DoubleAnnual& earnings, DoubleAnnual& uearnt, DoubleAnnual& vearn,
const DoubleAnnual& fq )
{
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ", first year ";
      s += poutNoComma(year1);
      s += ", base year ";
      s += poutNoComma(year2);
      s += ", last year ";
      s += poutNoComma(year3);
      s += ": Starting WageIndGeneral::indexEarnings";
      Trace::writeLine(s);
   }
#endif
   indexYearAvgWage = fq[year2];
   for (int i1 = year1; i1 <= year2; i1++) {
      if (!piaData.freezeYears.isFreezeYear(i1)) {
         uearnt[i1] = indexYearAvgWage * earnings[i1];
         double temp = uearnt[i1] / fq[i1];
         // round to nearest penny
         vearn[i1] = floor(temp * 100. + .5) / 100.;
#if defined(DEBUGCASE)
         if (isDebugPid(workerData.getIdNumber())) {
            string s("");
            s += workerData.getIdString();
            s += ", year ";
            s += poutNoComma(i1);
            s += ", earnings ";
            s += poutNoComma(earnings[i1]);
            s += ", indexed earnings ";
            s += poutNoComma(vearn[i1]);
            s += ": Indexing earnings";
            Trace::writeLine(s);
         }
#endif
      }
      else {
         uearnt[i1] = 0.;
         vearn[i1] = 0.;
#if defined(DEBUGCASE)
         if (isDebugPid(workerData.getIdNumber())) {
            string s("");
            s += workerData.getIdString();
            s += ", year ";
            s += poutNoComma(i1);
            s += ", earnings ";
            s += poutNoComma(earnings[i1]);
            s += ", indexed earnings ";
            s += poutNoComma(vearn[i1]);
            s += ": Not using earnings in indexing period";
            Trace::writeLine(s);
         }
#endif
      }
   }
   for (int i2 = year2; i2 <= year3; i2++) {
      if (!piaData.freezeYears.isFreezeYear(i2)) {
         uearnt[i2] = 0.;
         vearn[i2] = earnings[i2];
#if defined(DEBUGCASE)
         if (isDebugPid(workerData.getIdNumber())) {
            string s("");
            s += workerData.getIdString();
            s += ", year ";
            s += poutNoComma(i2);
            s += ", earnings ";
            s += poutNoComma(earnings[i2]);
            s += ", indexed earnings ";
            s += poutNoComma(vearn[i2]);
            s += ": Using unindexed earnings";
            Trace::writeLine(s);
         }
#endif
      }
      else {
         uearnt[i2] = 0.;
         vearn[i2] = 0.;
#if defined(DEBUGCASE)
         if (isDebugPid(workerData.getIdNumber())) {
            string s("");
            s += workerData.getIdString();
            s += ", year ";
            s += poutNoComma(i2);
            s += ", earnings ";
            s += poutNoComma(earnings[i2]);
            s += ", indexed earnings ";
            s += poutNoComma(vearn[i2]);
            s += ": Not using earnings after indexing period";
            Trace::writeLine(s);
         }
#endif
      }
   }
}

// Description: Calculates an AIME PIA under the 1977 law.
//
// Returns: The PIA.
//
// Arguments:
//   portionAime: Portion of AIME in each benefit formula interval.
//   percPiaTemp: 3 benefit formula percentages.
//   year: Year to use for rounding.
double WageIndGeneral::aimepiaCal( const double portionAime[],
const PercPia& percPiaTemp, int year )
{
   double piaelt = inner_product(percPiaTemp.begin(), percPiaTemp.end(),
      portionAime, 0.0);
   return BenefitAmount::round(piaelt, year);
}

// Description: Projects bend points.  In this version, that means retrieving
//   the bend points from bpPiaOut, where they were projected and stored
//   earlier.
//
// Arguments:
//   eligYear: Year of eligibility.
//   bendPiaTemp: Benefit formula bend points.
void WageIndGeneral::bendPointCal( int eligYear, BendPia& bendPiaTemp ) const
{
   bendPiaTemp[1] = piaParams.bpPiaOut.getBppia1(eligYear);
   bendPiaTemp[2] = piaParams.bpPiaOut.getBppia2(eligYear);
}

// Description: Applies real-wage-gain adjustment.
//
// Arguments:
//   eligYear: Year of eligibility.
void WageIndGeneral::realWageGainAdj( int eligYear )
{
   double factor = 1.0 + 0.01 * (eligYear - UserAssumptions::getIstart());
   piaEnt.set(piaEnt.get() * factor);
   piaEnt.round(UserAssumptions::getIstart() - 1);
   if (methodType == PiaMethod::WAGE_IND) {
      mfbEnt.set(mfbEnt.get() * factor);
      mfbEnt.round(UserAssumptions::getIstart() - 1);
   }
}

// Description: Deconverts AIME from raw PIA.
//
// Returns: Deconverted AIME.
//
// Arguments:
//   bendPiaTemp: Benefit formula bend points (starting with 0).
//   percPiaTemp: Benefit formula percentages.
double WageIndGeneral::deconvertAme( const BendPia& bendPiaTemp,
const PercPia& percPiaTemp ) const
{
   double rv;  // return value

   if (piasub < percPiaTemp[0] * bendPiaTemp[1]) {
      rv = piasub / percPiaTemp[0];
   }
   else {
      double temp = (percPiaTemp[0] - percPiaTemp[1]) * bendPiaTemp[1];
      if (piasub < (temp + percPiaTemp[1] * bendPiaTemp[2]))
         rv = (piasub - temp) / percPiaTemp[1];
      else {
         rv = (piasub - temp - (percPiaTemp[1] - percPiaTemp[2]) *
            bendPiaTemp[2]) / percPiaTemp[2];
      }
   }
   if (piaData.getEligYear() > 1982)
      return(ceil(rv));
   else
      return(floor(rv));
}

// Description: Calculates portion of aime in each section of pia formula.
//
// Arguments:
//   amesub: Aime to portion.
//   aimepart: Parts of the aime.
//   bendPiaTemp: Bend points to use.
void WageIndGeneral::setPortionAime( double amesub, double aimepart[],
const BendPia& bendPiaTemp )
{
   aimepart[0] = min(amesub, bendPiaTemp[1]);
   aimepart[1] = max(0.0, min(amesub - bendPiaTemp[1],
      bendPiaTemp[2] - bendPiaTemp[1]));
   aimepart[2] = max(amesub - bendPiaTemp[2], 0.);
}
