// $Id: WorkerDataGeneral.cpp 1.47 2007/03/21 10:40:24EDT 044579 Development  $
//
// Functions to initialize a WorkerDataGeneral structure and check data.

#include <utility>  // for rel_ops
#include "WorkerDataGeneral.h"
#include "oactcnst.h"
#include "wbgenrl.h"
#include "PiaException.h"
#include "Resource.h"
#include "DebugCase.h"
#if defined(DEBUGCASE)
#include "Trace.h"
#include "comma.h"
#include "DateFormatter.h"
#endif

using namespace std;
#if !defined(__SGI_STL_INTERNAL_RELOPS)
using namespace std::rel_ops;
#endif

// Maximum year of projection is 2100, unless changed.
int WorkerDataGeneral::maxyear = YEAR2100;
// Quarters of coverage are summed before 1978, unless changed.
int WorkerDataGeneral::qclumpyear = 1977;

// Description: Initializes a WorkerDataGeneral structure.
//
// Remarks: Should call setMaxyear before constructing any WorkerDataGeneral.
// The version with a WorkerDataGeneral argument copies all the data in the
// argument to this instance.
WorkerDataGeneral::WorkerDataGeneral() :
childCareYears(YEAR37, maxyear), qc(YEAR37, maxyear), qctot51td(0), qctottd(0),
joasdi(NO_BEN), valdi(0), ibegin(0), iend(0), theBits(string("00010000001")),
birthDate(), benefitDate(), entDate(), deathDate()
{ }

// Arguments:
//   workerData: The structure to copy from.
WorkerDataGeneral::WorkerDataGeneral( const WorkerDataGeneral& workerData ) :
childCareYears(workerData.childCareYears), qc(workerData.qc),
qctot51td(workerData.qctot51td), qctottd(workerData.qctottd),
joasdi(workerData.joasdi), valdi(workerData.valdi), ibegin(workerData.ibegin),
iend(workerData.iend), theBits(workerData.theBits),
birthDate(workerData.birthDate), benefitDate(workerData.benefitDate),
entDate(workerData.entDate), deathDate(workerData.deathDate)
{
   for (int i = 0; i < NUM_DISAB_PERIODS; i++) {
      disabPeriod[i] = workerData.disabPeriod[i];
   }
}

// Description: Destroys a workerdata structure.
WorkerDataGeneral::~WorkerDataGeneral( )
{ }

// Description: Initializes all variables to default values.
void WorkerDataGeneral::deleteContents()
{
   zeroOasdi();
   valdi = 0;
   ibegin = iend = 0;
   qctottd = qctot51td = 0;
   theBits.reset();
   theBits[SEX_BIT] = Sex::FEMALE;
   theBits[INDEARN_BIT] = true;
   birthDate = boost::gregorian::date(boost::date_time::not_a_date_time);
   benefitDate.deleteContents();
   entDate.deleteContents();
   deathDate = boost::gregorian::date(boost::date_time::not_a_date_time);
   for (int i = 0; i < 2; i++) {
      disabPeriod[i].deleteContents();
   }
   qc.deleteContents();
   childCareYears.deleteContents();
}

// Arguments:
//   ibegint: First year of earnings.
//   ibirtht: Year of birth.
void WorkerDataGeneral::ibeginCheck( int ibegint, int ibirtht )
{
   int i1;  // first possible year of earnings

   if (ibirtht < YEAR37)
      i1 = YEAR37;
   else
      i1 = ibirtht;
   if (ibegint < i1)
      throw PiaException(PIA_IDS_BEGIN1);
}

// Description: Sets first month of waiting period.
//
// Remarks: The version with only an index argument sets the first month of the
// waiting period to the first month after onset. It does that by calling
// DisabPeriod::waitperDateCal and passing the resulting date as
// an argument to the version of this function with two arguments.
//
// The version with two arguments sets the first month of the waiting period to
// the specified date.
//
// Arguments:
//   index: Which disability period to use (0 for most recent, 1 for next most
//     recent).
void WorkerDataGeneral::setWaitperDate( unsigned index )
{
   if (valdi > index) {
      DateMoyr waitpert = disabPeriod[index].waitperDateCal();
      if (waitpert != disabPeriod[index].getWaitperDate()) {
         setWaitperDate(index, waitpert);
      }
   }
}

// Arguments:
//   dateMoyr: The date to use.
void WorkerDataGeneral::setWaitperDate( unsigned index,
 const DateMoyr& dateMoyr )
{
   if (valdi > index) {
#ifndef NDEBUG
      DisabPeriod::waitperDateCheck(dateMoyr);
#endif
#if defined(DEBUGCASE)
      if (isDebugPid(getIdNumber())) {
         ostringstream strm;
         strm << getIdString() << ", index " << index << ", dateMoyr "
            << DateFormatter::toString(dateMoyr) << ": Setting waitperDate";
         Trace::writeLine(strm.str());
      }
#endif
      disabPeriod[index].setWaitperDate(dateMoyr);
   }
}

// Description: Sets date of disability onset.
//
// Arguments:
//   index: Which disability period to use (0 for most recent, 1 for next most
//     recent).
//   dateModyyr: The date to use.
void WorkerDataGeneral::setOnsetDate( unsigned index,
 const boost::gregorian::date& dateModyyr )
{
#ifndef NDEBUG
   DisabPeriod::onsetDateCheck(dateModyyr);
#endif
#if defined(DEBUGCASE)
   if (isDebugPid(getIdNumber())) {
      string s("");
      s += getIdString();
      s += ", index ";
      s += poutNoComma(index);
      s += ", dateModyyr ";
      s += DateFormatter::toString(dateModyyr);
      s += ": Setting onsetDate";
      Trace::writeLine(s);
   }
#endif
   disabPeriod[index].setOnsetDate(dateModyyr);
}

// Description: Sets date of disability cessation.
//
// Arguments:
//   index: Which disability period to use (0 for most recent, 1 for next most
//     recent).
//   dateMoyr: The date to use.
void WorkerDataGeneral::setCessationDate( unsigned index,
 const DateMoyr& dateMoyr )
{
   if ((index == 1 && valdi > 1) || (index == 0 && needDiCess())) {
#ifndef NDEBUG
      DisabPeriod::cessationDateCheck(dateMoyr);
#endif
#if defined(DEBUGCASE)
      if (isDebugPid(getIdNumber())) {
         ostringstream strm;
         strm << getIdString() << ", index " << index << ", dateMoyr "
           << DateFormatter::toString(dateMoyr) << ": Setting cessationDate";
         Trace::writeLine(strm.str());
      }
#endif
      disabPeriod[index].setCessationDate(dateMoyr);
   }
   else {
#if defined(DEBUGCASE)
      if (isDebugPid(getIdNumber())) {
         string s("");
         s += getIdString();
         s += ", index ";
         s += poutNoComma(index);
         s += ": Zeroing out cessationDate";
         Trace::writeLine(s);
      }
#endif
      disabPeriod[index].deleteCessationDate();
   }
}

// Description: Sets pia at disability cessation.
//
// Arguments:
//   index: Which disability period to use (0 for most recent, 1 for next most
//     recent).
//   newPia: The pia to use.
void WorkerDataGeneral::setCessationPia( unsigned index, float newPia )
{
   if ((index == 1 && valdi > 1) || (index == 0 && needDiCess())) {
#if defined(DEBUGCASE)
      if (isDebugPid(getIdNumber())) {
         string s("");
         s += getIdString();
         s += ", index ";
         s += poutNoComma(index);
         s += ", newPia ";
         s += poutNoComma(newPia, 10, 2);
         s += ": Setting cessationPia";
         Trace::writeLine(s);
      }
#endif
      disabPeriod[index].setCessationPia(newPia);
   }
}

// Description: Sets mfb at disability cessation.
//
// Arguments:
//   index: Which disability period to use (0 for most recent, 1 for next most
//     recent).
//   newMfb: The mfb to use.
void WorkerDataGeneral::setCessationMfb( unsigned index, float newMfb )
{
   if ((index == 1 && valdi > 1) || (index == 0 && needDiCess())) {
#if defined(DEBUGCASE)
      if (isDebugPid(getIdNumber())) {
         string s("");
         s += getIdString();
         s += ", index ";
         s += poutNoComma(index);
         s += ", newMfb ";
         s += poutNoComma(newMfb, 10, 2);
         s += ": Setting cessationMfb";
         Trace::writeLine(s);
      }
#endif
      disabPeriod[index].setCessationMfb(newMfb);
   }
}

// Arguments:
//   onsetDatet: The date of onset.
//   deathDatet: The date of death.
//   valdit: Number of periods of disability.
//   birthDatet: The date of birth.
//   ioasdi: Type of benefit.
//   entDatet: The date of entitlement.
//   benDatet: The date of benefit.
void WorkerDataGeneral::disCheck( const boost::gregorian::date& onsetDatet,
const boost::gregorian::date& deathDatet, int valdit,
const boost::gregorian::date& birthDatet, ben_type ioasdi,
const DateMoyr& entDatet, const DateMoyr& benDatet )
{
   // do not bother checking if there are no periods of disability
   if (valdit > 0) {
      // check for disability before birth
      disCheck(onsetDatet, birthDatet, ioasdi, entDatet, benDatet);
      // check for disability after death
      disCheck(onsetDatet, deathDatet, ioasdi);
   }
}

void WorkerDataGeneral::disCheck( const boost::gregorian::date& onsetDatet,
const boost::gregorian::date& birthDatet, ben_type ioasdi,
const DateMoyr& entDatet, const DateMoyr& benDatet )
{
   if (onsetDatet.is_not_a_date()) {
      throw PiaException(PIA_IDS_ONSET5);
   }
   if (birthDatet.is_not_a_date()) {
      throw PiaException(PIA_IDS_ONSET5);
   }
   if (onsetDatet < birthDatet) {
      // disability before birth
      throw PiaException(PIA_IDS_ONSET5);
   }
   if (ioasdi != NO_BEN) {
      if (ioasdi == SURVIVOR) {
         if (benDatet < onsetDatet)
            // disability after benefit date
            throw PiaException(PIA_IDS_ONSET6);
      }
      else {
         if (entDatet < onsetDatet)
            // disability after entitlement
            throw PiaException(PIA_IDS_ONSET6);
      }
   }
}

// Arguments:
//   onset1Datet: The date of prior disability onset.
//   onsetDatet: The date of most recent disability onset.
//   valdit: Number of periods of disability.
//   birthDatet: The date of birth.
void WorkerDataGeneral::dis1Check( const boost::gregorian::date& onset1Datet,
const boost::gregorian::date& onsetDatet, int valdit,
const boost::gregorian::date& birthDatet )
{
   if (valdit > 1) {
      if (onset1Datet < birthDatet)
         // disability before birth
         throw PiaException(PIA_IDS_ONSET5);
      if (onsetDatet < onset1Datet)
         // prior disability after current disability
         throw PiaException(PIA_IDS_ONSET8);
   }
}

// Description: Checks date of disability cessation, using passed values.
//
// Throws: PiaException of type PIA_IDS_DICESS4 if cesation is before onset.
//   PiaException of type PIA_IDS_DICESS6 if entitlement is before cessation.
//   PiaException of type PIA_IDS_DICESS7 if cessation is after death in
//   a survivor case.
//
// Arguments:
//   cessationDatet: The date of disability cessation.
//   onsetDatet: The date of disability onset.
//   deathDatet: The date of death.
//   entDatet: The date of entitlement.
//   ioasdi: Type of benefit.
void WorkerDataGeneral::cessationCheck2( const DateMoyr& cessationDatet,
const boost::gregorian::date& onsetDatet, const boost::gregorian::date& deathDatet,
const DateMoyr& entDatet, ben_type ioasdi )
{
   // check for disability cessation before onset
   if (cessationDatet < onsetDatet)
      throw PiaException(PIA_IDS_DICESS4);
   if (ioasdi == SURVIVOR) {
      // check for disability cessation after death
      if (DateMoyr(deathDatet) < cessationDatet) {
         throw PiaException(PIA_IDS_DICESS7);
      }
   }
   else {
      // check for disability cessation after current entitlement
      if (entDatet < cessationDatet) {
         throw PiaException(PIA_IDS_DICESS6);
      }
   }
}

void WorkerDataGeneral::cessation1Check() const
{
   if (joasdi == DISABILITY) {
      cessation1Check(disabPeriod[1].getCessationDate(),
         disabPeriod[1].getOnsetDate(), entDate);
   }
   else  {
      cessation1Check(disabPeriod[1].getCessationDate(),
         disabPeriod[1].getOnsetDate(), disabPeriod[0].getEntDate());
   }
}

// Arguments:
//   entDatet: The date of current entitlement.
void WorkerDataGeneral::cessation1Check( const DateMoyr& cessation1Datet,
const boost::gregorian::date& onset1Datet, const DateMoyr& entDatet )
{
   if (cessation1Datet < onset1Datet)
      // disability cessation before onset
      throw PiaException(PIA_IDS_DICESS4);
   if (entDatet < cessation1Datet)
      // prior disability cessation after second entitlement
      throw PiaException(PIA_IDS_DICESS5);
}

// Description: Sets date of entitlement to disability.
//
// Arguments:
//   index: Which disability period to use (0 for most recent, 1 for next most
//     recent).
//   dateMoyr: The date to use.
void WorkerDataGeneral::setPriorentDate( unsigned index,
 const DateMoyr& dateMoyr )
{
#if defined(DEBUGCASE)
   if (isDebugPid(getIdNumber())) {
      ostringstream strm;
      strm << getIdString() << ", index " << index << ", dateMoyr "
         << DateFormatter::toString(dateMoyr) << ": Setting priorentDate";
      Trace::writeLine(strm.str());
   }
#endif
   disabPeriod[index].setEntDate(dateMoyr);
}

// Description: Sets date of prior entitlement to disability.
//
// Arguments:
//   index: Which disability period to use (0 for most recent, 1 for next most
//     recent).
//   dateMoyr: The date to check and use.
void WorkerDataGeneral::setPriorentDateCheck( unsigned index,
 const DateMoyr& dateMoyr )
{
#ifndef NDEBUG
   DisabPeriod::entDateCheck(dateMoyr);
#endif
#if defined(DEBUGCASE)
   if (isDebugPid(getIdNumber())) {
      ostringstream strm;
      strm << getIdString() << ", index " << index << ", dateMoyr "
         << DateFormatter::toString(dateMoyr) << ": Setting priorentDate";
      Trace::writeLine(strm.str());
   }
#endif
   disabPeriod[index].setEntDate(dateMoyr);
}

// Arguments:
//   priorentDatet: The date of prior entitlement.
//   onsetDatet: The date of current disability onset.
//   entDatet: The date of current entitlement.
//   ioasdit: The type of benefit.
void WorkerDataGeneral::priorentCheck( const DateMoyr& priorentDatet,
const boost::gregorian::date& onsetDatet, const DateMoyr& entDatet, ben_type ioasdit )
{
   if (ioasdit == OLD_AGE && !(priorentDatet < entDatet)) {
      throw PiaException(PIA_IDS_PRRENT4);
   }
   if (priorentDatet < onsetDatet)
      // prior entitlement to disability before disability onset
      throw PiaException(PIA_IDS_PRRENT5);
}

// Arguments:
//   priorent1t: The date of second prior entitlement.
//   ionsett: The date of current disability onset.
//   priorentt: The date of prior entitlement.
void WorkerDataGeneral::priorent1Check( const DateMoyr& priorent1t,
const boost::gregorian::date& ionsett, const DateMoyr& priorentt )
{
   if (!(priorent1t < priorentt))
      throw PiaException(PIA_IDS_PRRENT4);
   if (priorent1t < ionsett)
      // prior entitlement to disability before disability onset
      throw PiaException(PIA_IDS_PRRENT5);
}

// Description: Sets date of benefit.
//
// Remarks: The version with an argument uses that date to set the date of
//   benefit, if the recalculation indicator has been set or this is a survivor
//   benefit.  Otherwise, the benefit date is set to the date of initial
//   entitlement. The version with no arguments sets the benefit date to the
//   date of initial entitlement.
//
// Throws: PiaException of type PIA_IDS_BENDATE2 if month is out of range. 
//   PiaException of type PIA_IDS_BENDATE3 if year is before 1940. PiaException
//   of type PIA_IDS_BENDATE4 if year is after maximum allowed. All are in
//   debug mode only.
//
// Arguments:
//   newBenefitDate: The date to use.
void WorkerDataGeneral::setBenefitDate( const DateMoyr& newBenefitDate )
{
   if (getRecalc() || joasdi == SURVIVOR) {
#ifndef NDEBUG
      benefitDate1Check(newBenefitDate);
#endif
#if defined(DEBUGCASE)
      if (isDebugPid(getIdNumber())) {
         ostringstream strm;
         strm << getIdString() << ", newBenefitDate "
            << DateFormatter::toString(newBenefitDate)
            << ": Setting benefitDate";
         Trace::writeLine(strm.str());
      }
#endif
      benefitDate = newBenefitDate;
   }
   else
      setBenefitDate();
}

void WorkerDataGeneral::setBenefitDate()
{
#if defined(DEBUGCASE)
   if (isDebugPid(getIdNumber())) {
      ostringstream strm;
      strm << getIdString() << ", entDate " << DateFormatter::toString(entDate)
         << ": Setting benefitDate to entDate";
      Trace::writeLine(strm.str());
   }
#endif
   benefitDate = entDate;
}

// Description: Zeroes out earnings and quarters of coverage before first year
//   and after last year of specified earnings.
//
// Arguments:
//   ibegint: First year of specified earnings.
//   iendt: Last year of specified earnings.
void WorkerDataGeneral::zeroEarn( int ibegint, int iendt )
{
   qc.deleteContents(YEAR37, ibegint - 1);
   qc.deleteContents(iendt + 1, maxyear);
}

// Description: Sets date of entitlement.
//
// Arguments:
//   newEntDate: The date to use.
void WorkerDataGeneral::setEntDate( const DateMoyr& newEntDate )
{
#ifndef NDEBUG
   entCheck(newEntDate);
#endif
#if defined(DEBUGCASE)
   if (isDebugPid(getIdNumber())) {
      ostringstream strm;
      strm << getIdString() << ", newEntDate "
         << DateFormatter::toString(newEntDate) << ": Setting entDate";
      Trace::writeLine(strm.str());
   }
#endif
   entDate = newEntDate;
}

// Description: Initializes some Statement data (no disability and no date
//   of death).
void WorkerDataGeneral::setPebsData()
{
   valdi = 0;
   deathDate = boost::gregorian::date(boost::date_time::not_a_date_time);
   disabPeriod[0].deleteContents();
}

// Description: Checks quarters of coverage to 1977 for consistency, assuming
//   both qcs for 1937-1977 and qcs for 1951-1977 have been set to
//   reasonable values.
//
// Throws: PiaException of type PIA_IDS_QC51TD if number of quarters of
//   coverage, 1951-77, is out of range.
void WorkerDataGeneral::qctdCheck2()
{
   // limit number of qcs 1937-1977 if first year of earnings is after 1950
   if (ibegin > 1950) {
      qc51tdCheck(qctottd);
      qctot51td = qctottd;
   }
   if (iend < 1951)
      qctot51td = 0;
   if (ibegin > 1977)
      qctottd = qctot51td = 0;
}

// Description: Sets summary quarters of coverage, 1937 to 1977.
//
// Arguments:
//   qct: New summary quarters of coverage.
void WorkerDataGeneral::setQctd( int qct )
{
#ifndef NDEBUG
   qctdCheck(qct);
#endif
   qctottd = (unsigned char)qct;
}

// Description: Sets summary quarters of coverage, 1951 to 1977.
//
// Throws: PiaException of type PIA_IDS_QC51TD if number of quarters of
//   coverage, 1951-77, is out of range (debug mode only).
//
// Arguments:
//   qct: New summary quarters of coverage.
void WorkerDataGeneral::setQc51td( int qct )
{
#ifndef NDEBUG
   qc51tdCheck(qct);
#endif
   qctot51td = (unsigned char)qct;
}

// Description: Sets type of benefit.
//
// Throws: PiaException of type PIA_IDS_JOASDI if type of benefit is out of
//   range (debug mode only).
//
// Arguments:
//   newJoasdi: New type of benefit.
void WorkerDataGeneral::setJoasdi( int newJoasdi )
{
#ifndef NDEBUG
   joasdiCheck(newJoasdi);
#endif
   joasdi = static_cast< unsigned char >(newJoasdi);
}

// Description: Sets the sex of the worker.
//
// Throws: PiaException of type PIA_IDS_SEX if sex is not MALE or FEMALE
//   (only in debug mode).
//
// Arguments:
//   newSex: New sex indicator.
void WorkerDataGeneral::setSex( Sex::sex_type newSex )
{
#ifndef NDEBUG
   Sex::check(newSex);
#endif
#if defined(DEBUGCASE)
   if (isDebugPid(getIdNumber())) {
      string s("");
      s += getIdString();
      s += ", newSex ";
      s += poutNoComma(static_cast<int>(newSex));
      s += ": Setting sex";
      Trace::writeLine(s);
   }
#endif
   theBits[SEX_BIT] = (newSex > 0);
}

// Description: Sets date of birth.
//
// Arguments:
//   newBirthDate: The date to use.
void WorkerDataGeneral::setBirthDate( const boost::gregorian::date& newBirthDate )
{
#ifndef NDEBUG
   birth2Check(newBirthDate);
#endif
#if defined(DEBUGCASE)
   if (isDebugPid(getIdNumber())) {
      string s("");
      s += getIdString();
      s += ", newBirthDate ";
      s += DateFormatter::toString(newBirthDate);
      s += ": Setting birthDate";
      Trace::writeLine(s);
   }
#endif
   birthDate = newBirthDate;
}

// Description: Sets first year of earnings, including any projected earnings.
//   Also sets first year of HI earnings (ignored if indicator for
//   HI earnings is not set before attempting to use HI earnings).
//
// Arguments:
//   newIbegin: New first year of earnings.
void WorkerDataGeneral::setIbegin( int newIbegin )
{
#ifndef NDEBUG
   ibegin2Check(newIbegin);
#endif
   ibegin = (unsigned short)newIbegin;
   //if (projback == 0)
   //   earnpebs.setFirstYear(newIbegin);
   // HI earnings start in same year, but at least 1983
   //earnhi.setFirstYear(max(1983, ibegin));
}

// Description: Checks first year of earnings in a Statement case.
//
// Throws: PiaException of type PIA_IDS_BEGIN3 if first year of earnings is
//   before 1937. PiaException of type PIA_IDS_BEGIN4 if first year of earnings
//   is in starting year of projections or later.
//
// Arguments:
//   ibegint: Year to check.
//   istartt: Starting year of projections.
void WorkerDataGeneral::ibegin1Check( int ibegint, int istartt )
{
   if (ibegint < YEAR37)
      throw PiaException(PIA_IDS_BEGIN3);
   if (ibegint > istartt - 1)
      throw PiaException(PIA_IDS_BEGIN4);
}

// Description: Checks first year of earnings in a non-Statement case.
//
// Throws: PiaException of type PIA_IDS_BEGIN3 if first year of earnings is
//   before 1937. PiaException of type PIA_IDS_BEGIN5 if first year of earnings
//   is after maximum year allowed.
//
// Arguments:
//   ibegint: Year to check.
void WorkerDataGeneral::ibegin2Check( int ibegint )
{
   if (ibegint < YEAR37)
      throw PiaException(PIA_IDS_BEGIN3);
   if (ibegint > maxyear)
      throw PiaException(PIA_IDS_BEGIN5);
}

// Description: Checks last year of earnings.
//
// Throws: PiaException of type PIA_IDS_END1 if last year of earnings is
//   before 1937. PiaException of type PIA_IDS_END2 if last year of earnings
//   is after maximum year allowed.
//
// Arguments:
//   iendt: Year to check.
void WorkerDataGeneral::iendCheck( int iendt )
{
   if (iendt < YEAR37)
      throw PiaException(PIA_IDS_END1);
   if (iendt > maxyear)
      throw PiaException(PIA_IDS_END2);
}

// Description: Sets last year of earnings, including any projected earnings.
//
// Arguments:
//   newIend: New last year of earnings.
void WorkerDataGeneral::setIend( int newIend )
{
#ifndef NDEBUG
   iendCheck(newIend);
#endif
   iend = (unsigned short)newIend;
}

// Description: Sets number of periods of disability.
//
// Arguments:
//   newValdi: New number of periods of disability.
void WorkerDataGeneral::setValdi( int newValdi )
{
#ifndef NDEBUG
   if (joasdi == DISABILITY) {
      valdi2Check(newValdi);
   }
   else {
      valdi1Check(newValdi);
   }
#endif
#if defined(DEBUGCASE)
   if (isDebugPid(getIdNumber())) {
      string s("");
      s += getIdString();
      s += ", newValdi ";
      s += poutNoComma(newValdi);
      s += ": Setting valdi";
      Trace::writeLine(s);
   }
#endif
   valdi = static_cast< unsigned char >(newValdi);
}

// Description: Sets date of death.
//
// Arguments:
//   newDeathDate: The date to use.
void WorkerDataGeneral::setDeathDate( const boost::gregorian::date& newDeathDate )
{
#ifndef NDEBUG
   deathCheck2(newDeathDate);
#endif
#if defined(DEBUGCASE)
   if (isDebugPid(getIdNumber())) {
      string s("");
      s += getIdString();
      s += ", newDeathDate ";
      s += DateFormatter::toString(newDeathDate);
      s += ": Setting deathDate";
      Trace::writeLine(s);
   }
#endif
   deathDate = newDeathDate;
}

// Description: Returns true if benefit increase assumptions are needed.
//
// Returns: True if benefit increase assumptions are needed.
//
// Arguments:
//   year: First year of projection period.
bool WorkerDataGeneral::need_bi( int year ) const
{
   // no need for assumptions if no benefit calculated
   if (getJoasdi() == NO_BEN)
      return(false);
   // check for faulty starting date
   if (year < YEAR79)
      return(false);
   return(!(benefitDate < DateMoyr(boost::date_time::Dec, year)));
}

// Description: Returns true if average wage increase assumptions are needed.
//
// Returns: True if average wage increase assumptions are needed.
//
// Arguments:
//   year: First year of projection period.
//   lastTwoYears: True if need to look at last 2 years of historical period.
bool WorkerDataGeneral::need_aw( int year, bool lastTwoYears ) const
{
#if defined(DEBUGCASE)
   if (isDebugPid(getIdNumber())) {
      string s("");
      s += getIdString();
      s += ", year ";
      s += poutNoComma(year);
      s += ", lastTwoYears ";
      s += (lastTwoYears ? "true" : "false");
      s += ": At top of WorkerDataGeneral::need_aw";
      Trace::writeLine(s);
   }
#endif
   // no need for assumptions if no benefit calculated
   if (getJoasdi() == NO_BEN)
      return(false);
   // check for faulty starting date
   if (year < YEAR79)
      return(false);
   if (lastTwoYears) {
      year -= 2;
   }
#if defined(DEBUGCASE)
   if (isDebugPid(getIdNumber())) {
      string s("");
      s += getIdString();
      s += ", (int)benefitDate.getYear() ";
      s += poutNoComma((int)benefitDate.getYear());
      s += ", year ";
      s += poutNoComma(year);
      s += ": At bottom of WorkerDataGeneral::need_aw";
      Trace::writeLine(s);
   }
#endif
   return(static_cast<int>(benefitDate.getYear()) > year);
}

// Description: Returns true if wage base assumptions are needed.
//
// Returns: True if wage base assumptions are needed.
//
// Throws: PiaException of type PIA_IDS_NEEDBASES if year is out of range
//   (only in debug mode).
//
// Arguments:
//   year: First year of projection period.
bool WorkerDataGeneral::need_bases( int year ) const
{
#ifndef NDEBUG
   if (year < YEAR37)
      throw PiaException(PIA_IDS_NEEDBASES);
#endif
   return(static_cast<int>(benefitDate.getYear()) > year);
}

// Description: Checks that date of benefit is in range.
//
// Throws: PiaException of type PIA_IDS_BENDATE2 if month is out of range. 
//   PiaException of type PIA_IDS_BENDATE3 if year is before 1940. PiaException
//   of type PIA_IDS_BENDATE4 if year is after maximum allowed.
//
// Remarks: The version with no arguments uses the stored benefit date, passing
// it to the static version with one argument.
void WorkerDataGeneral::benefitDate1Check() const
{
   benefitDate1Check(benefitDate);
}

// Arguments:
//   datetemp: The date to check.
void WorkerDataGeneral::benefitDate1Check( const DateMoyr& datetemp )
{
   try {
      datetemp.monthCheck();
   } catch (PiaException e) {
      throw PiaException(PIA_IDS_BENDATE2);
   }
   if (datetemp.getYear() < 1940)
      throw PiaException(PIA_IDS_BENDATE3);
   if (static_cast<int>(datetemp.getYear()) > maxyear)
      throw PiaException(PIA_IDS_BENDATE4);
}

// Arguments:
//   datetemp: The date to check.
void WorkerDataGeneral::birth1Check( const boost::gregorian::date& datetemp,
 int istartt )
{
   if (datetemp.is_not_a_date()) {
      throw PiaException(PIA_IDS_BIRTH3);
   }
   if (static_cast<unsigned int>(datetemp.year()) < Date::lowyear) {
      throw PiaException(PIA_IDS_BIRTH2);
   }
   if (static_cast<int>(datetemp.year()) > istartt - 1) {
      throw PiaException(PIA_IDS_BIRTH1);
   }
}

// Arguments:
//   datetemp: The date to check.
void WorkerDataGeneral::birth2Check( const boost::gregorian::date& datetemp )
{
   if (datetemp.is_not_a_date()) {
      throw PiaException(PIA_IDS_BIRTH3);
   }
   if (static_cast<unsigned int>(datetemp.year()) < Date::lowyear) {
      throw PiaException(PIA_IDS_BIRTH2);
   }
   if (static_cast<int>(datetemp.year()) > maxyear) {
      throw PiaException(PIA_IDS_BIRTH5);
   }
}

// Description: Checks date of death (checks in survivor case for death in 1937
//   or later).
//
// Throws: PiaException of type PIA_IDS_DEATH3 if death before 1937 in a
//   survivor case.
//
// Arguments:
//   datetemp: The date to check.
//   ioasdi: Type of benefit.
void WorkerDataGeneral::deathCheck1( const boost::gregorian::date& datetemp,
ben_type ioasdi )
{
   if (ioasdi == SURVIVOR)
      if (static_cast<int>(datetemp.year()) < YEAR37)
         // death before 1937
         throw PiaException(PIA_IDS_DEATH3);
   deathCheck2(datetemp);
}

// Description: Checks date of death for basic correctness.
//
// Throws: PiaException of type PIA_IDS_DEATH1 if month of death is out of
//   range. PiaException of type PIA_IDS_DEATH2 if year of death is after
//   maximum allowed.
//
// Arguments:
//   datetemp: The date to check.
void WorkerDataGeneral::deathCheck2( const boost::gregorian::date& datetemp )
{
   if (datetemp.is_not_a_date()) {
      throw PiaException(PIA_IDS_DEATH1);
   }
   if (static_cast<int>(datetemp.year()) > maxyear)
      throw PiaException(PIA_IDS_DEATH2);
}

// Arguments:
//   datetemp: The date to check.
void WorkerDataGeneral::entCheck( const DateMoyr& datetemp )
{
   try {
      datetemp.monthCheck();
   } catch (PiaException e) {
      throw PiaException(PIA_IDS_ENT1);
   }
   if (datetemp.getYear() < 1940)
      throw PiaException(PIA_IDS_ENT2);
   if (static_cast<int>(datetemp.getYear()) > maxyear)
      throw PiaException(PIA_IDS_ENT3);
}

// Description: Checks public pension amount.
//
// Throws: PiaException of type PIA_IDS_PUBPEN if public pension amount
//   is out of range.
//
// Arguments:
//   pubpent: Pension to check.
void WorkerDataGeneral::pubpenCheck( float pubpent )
{
   try {
      WageBaseGeneral::check(static_cast<double>(pubpent));
   } catch (PiaException e) {
      throw PiaException(PIA_IDS_PUBPEN);
   }
}

// Description: Checks public pension entitlement date.
//
// Throws: PiaException of type PIA_IDS_NONPENDATE1 or PIA_IDS_NONPENDATE2
//   if public pension entitlement date is out of range (before date of birth
//   or after date of death).
//
// Arguments:
//   pubpenDatet: Pension to check.
//   birthDatet: Birth date.
//   deathDatet: Death date (could be not_a_date).
void WorkerDataGeneral::pubpenDateCheck( const DateMoyr& pubpenDatet,
   const boost::gregorian::date& birthDatet,
   const boost::gregorian::date& deathDatet )
{
   if (!birthDatet.is_not_a_date()) {
      DateMoyr tempDate(birthDatet);
      if (pubpenDatet < birthDatet) {
         throw PiaException(PIA_IDS_NONPENDATE1);
      }
   }
   if (!deathDatet.is_not_a_date()) {
      DateMoyr tempDate(deathDatet);
      if (pubpenDatet > tempDate) {
         throw PiaException(PIA_IDS_NONPENDATE2);
      }
   }
}

// Description: Returns primary beneficiary code.
//
// Returns: primary beneficiary code.
//
//   0 - was never a primary beneficiary.
//   1 - is a primary beneficiary.
//   2 - is a deceased primary beneficiary.
//
// Arguments:
//   ioasdi: Type of benefit.
int WorkerDataGeneral::isPrimary( ben_type ioasdi ) const
{
   if (ioasdi == SURVIVOR) {
      // assume was old-age if over age 62 or disabled
      if ((deathDate.year() > 62 + birthDate.year()) || valdi > 0)
         return(2);
      else
         return(0);
   }
   if (ioasdi == OLD_AGE || ioasdi == DISABILITY)
      return(1);
   else
      return(0);
}

// Description: Returns true if quarters of coverage by year from regular
//   earnings are needed, false otherwise.
//
// Throws: PiaException of type PIA_IDS_QCLUMPYEAR if qclumpyear is out of
//   range (only in debug mode).
//
// Returns: True if quarters of coverage by year from regular
//   earnings are needed, false otherwise.
bool WorkerDataGeneral::needQcsByYear() const
{
#ifndef NDEBUG
   if (qclumpyear < YEAR37 || qclumpyear > 1977)
      throw PiaException(PIA_IDS_QCLUMPYEAR);
#endif
   return(getTotalize() || joasdi == PEBS_CALC || qclumpyear < 1977 ||
      // check for disability case with onset within 10 years of qclumpyear
      (joasdi == DISABILITY && disabPeriod[0].getOnsetDate() <
      boost::gregorian::date(static_cast<unsigned short>(11 + qclumpyear), 1u, 1u)) ||
      // check for survivor case with death within 13 quarters of qclumpyear
      (joasdi == SURVIVOR && deathDate < boost::gregorian::date(
      static_cast<unsigned short>(4 + qclumpyear), 4u, 1u)) ||
      getIndms() || getIndrr() || ibegin < 1950);
}

// Description: Returns true if ok to enable earnings entry dialog box.
//
// Returns: True if ok to enable earnings entry dialog box.
bool WorkerDataGeneral::getEarnEnable() const
{
   // do not allow earnings entry until years are consistent
   if (getIndearn()) {
      if (birthDate.is_not_a_date()) {
         return false;
      }
      try {
         ibeginCheck();
      } catch (PiaException e) {
         return false;
      }
      return true;
   }
   // if no regular earnings, must have railroad earnings
   return getIndrr();
}

// Description: Sets earnings indicator.
//
// Arguments:
//   newIndearn: New earnings indicator.
void WorkerDataGeneral::setIndearn( bool newIndearn )
{
#if defined(DEBUGCASE)
   if (isDebugPid(getIdNumber())) {
      string s("");
      s += getIdString();
      s += ", indearn ";
      s += poutNoComma(theBits[INDEARN_BIT] ? 1 : 0);
      s += ", newIndearn ";
      s += poutNoComma(newIndearn ? 1 : 0);
      s += ": Setting indearn";
      Trace::writeLine(s);
   }
#endif
   theBits[INDEARN_BIT] = newIndearn;
}

// Description: Returns earliest entitlement date on this record.
//
// Returns: Earliest entitlement date on this record.
//
// Remarks: If there are multiple periods of disability, take the first
// disability entitlement date. Otherwise, for old-age or survivor with a prior
// disability, use the prior disability entitlement. Finally, for disability
// entitlement with only one period of disability, or old-age or survivor with
// no disability, use current entitlement.
//
// Note that for a survivor with no prior disability, this returns a date of
// zero. This function should probably not be used in that case.
const DateMoyr& WorkerDataGeneral::getEarliestEntDate() const
{
   // if there are multiple periods of disability, take the first disability
   // entitlement date
   if (valdi > 1) {
      return disabPeriod[valdi - 1].getEntDate();
   }
   // for old-age or survivor with a prior disability, use disability
   // entitlement
   else if ((joasdi == OLD_AGE || joasdi == SURVIVOR) && valdi == 1) {
      return disabPeriod[0].getEntDate();
   }
   // for disability entitlement with only one period of disability, or old-age
   // or survivor with no disability, use current entitlement
   else {
      return getEntDate();
   }
}
