// $Id: avgwg.cpp 1.19 2006/11/24 08:54:44EST 044579 Development  $
//
// Functions to handle average wage amount calculations.

#include "avgwg.h"
#include "awinc.h"
#include "Resource.h"
#include "PiaException.h"

// Description: Initializes width and precision.
//
// Arguments:
//   newBaseYear: Year that array starts in.
//   newLastYear: Maximum year of projection.
AverageWage::AverageWage( int newBaseYear, int newLastYear ) :
DoubleAnnual(newBaseYear, newLastYear)
{
   setWidth(9);
   setPrecision(2);
}

// Description: Destroys average wage array.
AverageWage::~AverageWage()
{ }

// Description: Projects average wage amounts for a period of years.
//
// Remarks: If lastYeart is not specified, the projection goes to the last year
// of the vector of average wage amounts.
//
// Arguments:
//   averageWageInc: Average wage percentage increases (e.g. 5.0 for 5 percent
//     increase).
//   firstYeart: First year to project.
void AverageWage::project( const Awinc& averageWageInc, int firstYeart )
{
   for (int i = firstYeart; i <= getLastYear(); i++)
      projectForward(i, averageWageInc[i] / 100. + 1.);
}

// Arguments:
//   factor: Multiplicative factor used to increase each year (e.g. 1.05).
//   firstYeart: First year to project.
//   lastYeart: Last year to project to. If not specified, amounts are
//     projected to last year.
void AverageWage::project( double factor, int firstYeart, int lastYeart )
{
   for (int i = firstYeart; i <= lastYeart; i++)
      projectForward(i, factor);
}

// Throws: PiaException of type PIA_IDS_AVGWG if year is out of range (only
//   in debug mode).
//
// Arguments:
//   year: Year for which average wage is desired.
//   factor: Multiplicative factor used to increase previous year.
void AverageWage::projectForward( int year, double factor )
{
#ifndef NDEBUG
   if (year <= getBaseYear() || year > getLastYear())
      throw PiaException(PIA_IDS_AVGWG);
#endif
   theData[year - baseYear] =
      projectForward(theData[year - 1 - baseYear], factor);
}

// Throws: PiaException of type PIA_IDS_AVGWG if year is out of range (only
//   in debug mode).
//
// Arguments:
//   year: Year for which average wage is desired.
//   factor: Multiplicative factor used to decrease from following year.
void AverageWage::projectBack( int year, double factor )
{
#ifndef NDEBUG
   if (year < getBaseYear() || year >= getLastYear())
      throw PiaException(PIA_IDS_AVGWG);
#endif
   theData[year - baseYear] =
      projectBack(theData[year + 1 - baseYear], factor);
}

// Description: Returns the first year that data > 4.5, or returns 0.
//
// Returns: The first year that data > 4.5, or returns 0.
int AverageWage::startYear() const
{
   for (int yr = baseYear; yr <= getLastYear(); yr++)
      if (theData[yr - baseYear] > 4.5)
         return(yr);
   return(0);
}

// Description: Returns the last year that data > 4.5, or returns 0.
//
// Returns: The last year that data > 4.5, or returns 0.
int AverageWage::endYear() const 
{
   for (int yr = getLastYear(); yr >= baseYear; yr--)
      if (theData[yr - baseYear] > 4.5)
         return(yr);
   return(0);
}
