// $Id: awbidtfl.cpp 1.19 2006/11/08 11:04:34EST 044579 Development  $
//
// Functions to handle historical benefit increases, average wages, OASDI
// wage bases, old-law wage bases, and HI wage bases in a file.

#include <fstream>
#include <iomanip>
#include "AwbiDataFile.h"
#include "Resource.h"
#include "comma.h"
#include "PiaException.h"
#include "Path.h"

using namespace std;

// Description: Initializes historical parameters info.
//
// Arguments:
//   newLastYear: Last year of wage bases.
//   newMaxyear: Maximum projected year.
//   newDirectory: Directory with historical parameters file.
AwbiDataFile::AwbiDataFile( int newLastYear, int newMaxyear,
const std::string& newDirectory ) :
AwbiData(newLastYear, newMaxyear), pathname()
{
   pathname = Path::combine(newDirectory, "awbixxxxx.dat");
}

// Description: Reads historical parameters.
//
// Remarks:  The version with no arguments opens an input file and passes
//   that file as an argument to the version with one argument.
//
// Throws: PiaException of type PIA_IDS_AWBI1 if unable to open input file.
//   PiaException of type PIA_IDS_AWBI3 if unable to read all required data.
void AwbiDataFile::read()
{
   ifstream in; // input stream

   in.open(pathname.c_str(), ios::in);
   if (!in.is_open())
      throw PiaException(PIA_IDS_AWBI1);
   read(in);
   in.close();
}

// Arguments:
//   in: An open input stream to read from.
void AwbiDataFile::read( std::istream& in )
{
   float ftemp;  // temporary float
   double dtemp;  // temporary double
   int i1;  // loop counter

   // read benefit increases
   for (i1 = cpiinc.getBaseYear(); i1 <= lastYear - 1; i1++) {
      if (!(in >> ftemp))
         throw PiaException(PIA_IDS_AWBI3);
      if (in.eof() || in.fail())
         throw PiaException(PIA_IDS_AWBI3);
      cpiinc[i1] = ftemp;
   }
   // read average wages
   for (i1 = fq.getBaseYear(); i1 <= lastYear - 2; i1++) {
      if (!(in >> dtemp))
         throw PiaException(PIA_IDS_AWBI3);
      if (in.eof() || in.fail())
         throw PiaException(PIA_IDS_AWBI3);
      fq[i1] = dtemp;
   }
   // read wage bases
   for (i1 = baseOasdi.getBaseYear(); i1 <= lastYear; i1++) {
      if (!(in >> dtemp))
         throw PiaException(PIA_IDS_AWBI3);
      if (in.eof() || in.fail())
         throw PiaException(PIA_IDS_AWBI3);
      baseOasdi[i1] = dtemp;
   }
   // HI bases equal OASDI bases through 1990
   baseHi.assign(baseOasdi, baseHi.getBaseYear(), HI_BASE_YEAR - 1);
   // read HI bases beginning 1991
   for (i1 = HI_BASE_YEAR; i1 <= lastYear; i1++) {
      if (!(in >> dtemp))
         throw PiaException(PIA_IDS_AWBI3);
      if (in.eof() || in.fail())
         throw PiaException(PIA_IDS_AWBI3);
      baseHi[i1] = dtemp;
   }
   // read old-law bases
   for (i1 = base77.getBaseYear(); i1 <= lastYear; i1++) {
      if (!(in >> dtemp))
         throw PiaException(PIA_IDS_AWBI3);
      if (in.eof() || in.fail())
         throw PiaException(PIA_IDS_AWBI3);
      base77[i1] = dtemp;
   }
}

// Description: Saves historical parameters.
//
// Remarks:  The version with no arguments opens an output file and passes
//   that file as an argument to the version with one argument.
//
// Throws: PiaException of type PIA_IDS_AWBI2 if unable to open output file.
void AwbiDataFile::write()
{
   ofstream out;  // output stream

   out.open(pathname.c_str(), ios::out);
   if (!out.is_open())
      throw PiaException(PIA_IDS_AWBI2);
   out.setf(ios::fixed, ios::floatfield);
   write(out);
   out.close();
}

// Arguments:
//   out: Open output stream to write to.
void AwbiDataFile::write( std::ostream& out )
{
   int i1;  // loop counter

   out.precision(cpiinc.getPrecision());
   for (i1 = cpiinc.getBaseYear(); i1 <= lastYear - 1; i1++)
      out << setw(4) << cpiinc[i1] << endl;
   out.precision(fq.getPrecision());
   for (i1 = fq.getBaseYear(); i1 <= lastYear - 2; i1++)
      out << setw(10) << fq[i1] << endl;
   out.precision(baseOasdi.getPrecision());
   for (i1 = baseOasdi.getBaseYear(); i1 <= lastYear; i1++)
      out << setw(10) << baseOasdi[i1] << endl;
   out.precision(baseHi.getPrecision());
   for (i1 = HI_BASE_YEAR; i1 <= lastYear; i1++)
      out << setw(10) << baseHi[i1] << endl;
   out.precision(base77.getPrecision());
   for (i1 = base77.getBaseYear(); i1 <= lastYear; i1++)
      out << setw(10) << base77[i1] << endl;
}

// Description: Forms a file name for historical parameters file.
void AwbiDataFile::setAwbiFile()
{
   const string directoryName(Path::getDirectoryName(pathname));
   const string coreName("awbi" + poutNoComma(getLastYear()) + ".dat");
   pathname = Path::combine(directoryName, coreName);
}

// Description: Dummy function (descendant class may use it).
void AwbiDataFile::setData()
{ }
