// $Id: awincfl.cpp 1.22 2006/11/08 11:04:33EST 044579 Development  $
//
// Functions to handle projected average wage increases stored on disk.

#include <fstream>
#include "AwincFile.h"
#include "avgwg.h"
#include "Resource.h"
#include "comma.h"
#include "PiaException.h"
#include "AssumptionType.h"
#include "Path.h"

using namespace std;

// Description: Initializes projected average wage increase info.
//
// Arguments:
//   newIstart: Starting year of average wage increases.
//   newMaxyear: Maximum projected year.
//   newDirectory: Directory with average wage increase file.
AwincFile::AwincFile( int newIstart, int newMaxyear,
const std::string& newDirectory ) : AwincDoc(newIstart, newMaxyear),
pathname()
{
   pathname = Path::combine(newDirectory, "awxxx.dat");
}

// Description: Reads projected average wage increases.
//
// Throws: PiaException of type PIA_IDS_FQINC4 if unable to open input file.
//   PiaException of type PIA_IDS_FQINC3 if unable to read all required data.
//
// Arguments:
//   altNum: Desired alternative number (1-4).
void AwincFile::read( int altNum )
{
   int temp;  // temporary istart
   string str;  // temporary string

   setAwFile(altNum);
   ifstream in(pathname.c_str(), ios::in);
   if (!in.is_open())
      throw PiaException(PIA_IDS_FQINC4);
   // read title
   getline(in, str);
   awincTitle = str;
   // read first year
   if (!(in >> temp))
      throw PiaException(PIA_IDS_FQINC3);
   if (in.eof() || in.fail())
      throw PiaException(PIA_IDS_FQINC3);
   firstYear = temp;
   // read all data
   try {
      awinc.read(in, firstYear, awinc.getLastYear());
   } catch (PiaException e) {
      throw PiaException(PIA_IDS_FQINC3);
   }
   in.close();
}

// Description: Saves projected average wage increases.
//
// Throws: PiaException of type PIA_IDS_FQINC2 if unable to open output file.
//
// Arguments:
//   altNum: Desired alternative number (1-4).
void AwincFile::write( int altNum )
{
   setAwFile(altNum);
   ofstream out(pathname.c_str(), ios::out);
   if (!out.is_open())
      throw PiaException(PIA_IDS_FQINC2);
   out.setf(ios::fixed, ios::floatfield);
   // write title
   out << awincTitle << endl;
   // write first year of data
   out << firstYear << endl;
   // write all data
   awinc.write(out, firstYear, awinc.getLastYear());
   out.close();
}

// Description: Forms a file name for projected average wage increase file.
//
// Arguments:
//   altNum: Desired alternative number (1-4).
void AwincFile::setAwFile( int altNum )
{
   const string directoryName(Path::getDirectoryName(pathname));
   const string coreName("aw" + poutNoComma(firstYear) + poutNoComma(altNum) +
     ".dat");
   pathname = Path::combine(directoryName, coreName);
}

// Description: Dummy function.
void AwincFile::setData( int )
{ }

void AwincFile::setData( int, const DoubleAnnual& )
{ }

// Description: Sets title of assumptions if not read in.
//
// Arguments:
//   altNum: Which set of assumptions.
void AwincFile::setTitle( int altNum )
{
   if (AssumptionType::isNotTR(altNum)) {
      awincTitle = avgwage[altNum - AssumptionType::FLAT];
   }
}

void AwincFile::setTitle( int altNum, const std::string& )
{
   if (AssumptionType::isNotTR(altNum)) {
      awincTitle = avgwage[altNum - AssumptionType::FLAT];
   }
}
