// $Id: awincnf.cpp 1.46 2008/04/25 12:59:43EDT 277133 Development  $
//
// Functions to handle projected average wage increases stored in arrays.

#include "AwincNonFile.h"
#include "avgwg.h"
#include "Resource.h"
#include "AssumptionType.h"
#include "BaseYearNonFile.h"
#include "oactcnst.h"

using namespace std;

// Remarks: Historical average wage increases, 1978-2006.
// When updating in November, add a value to this array.
const double AwincNonFile::awincHist[] = {
                         7.941048,  8.747891,  9.007392,
  10.066281,  5.505224,  4.871540,  5.878443,  4.260533,
   2.968106,  6.377448,  4.925132,  3.959390,  4.619158,
   3.726559,  5.152396,  0.860023,  2.683910,  4.008373,
   4.890539,  5.835092,  5.233866,  5.572834,  5.529993,
   2.385645,  1.002888,  2.444538,  4.648767,  3.659027,
   4.596306
};

// Remarks: Titles for 2008 Trustees Report update.
const std::string AwincNonFile::titletrOriginal[NUMASSUMSETS] = {
   "2008 Trustees Report Alternative I",
   "2008 Trustees Report Alternative II",
   "2008 Trustees Report Alternative III",
   "No increase beyond 2006 average wage"
};

// Remarks: Average wage increases for 2008 Trustees Report, years 2007-2022.
const double AwincNonFile::awincprojtrOriginal[NUMASSUMSETS][NUMPROJYEARS] = {
   {           4.257335, 4.575535, 4.016637, 3.611586,
     3.477440, 3.429010, 3.352976, 3.287613, 3.223807,
     3.216810, 3.310969, 3.348112, 3.370924, 3.353595,
     3.339953, 3.337070 },
   {           4.283440, 4.083730, 4.114647, 4.019252,
     3.942502, 3.946248, 3.994518, 3.922357, 3.841512,
     3.825009, 3.807959, 3.847469, 3.873409, 3.856560,
     3.842548, 3.842552 },
   {           4.168619, 2.226531, 4.873576, 4.376646, 
     3.510816, 6.456969, 7.076204, 5.312961, 4.379459, 
     4.230465, 4.252871, 4.342707, 4.374487, 4.360634,
     4.346433, 4.348304 },
   {      0.0, 0.0, 0.0, 0.0,
     0.0, 0.0, 0.0, 0.0, 0.0,
     0.0, 0.0, 0.0, 0.0, 0.0,
     0.0, 0.0 }
};

// Remarks: Ultimate average wage increases for 2008 Trustees Report.
// These have been the same for several years.
const double AwincNonFile::awincult[NUMASSUMSETS] = {
   3.4, 3.9, 4.4, 0.0
};

// Description: Initializes projected average wage increase info with
//   latest Trustees Report assumptions.
//
// Arguments:
//   newIstart: Starting year of average wage increases.
//   newMaxyear: Maximum projected year.
AwincNonFile::AwincNonFile( int newIstart, int newMaxyear ) :
AwincDoc(newIstart, newMaxyear)
{
   // Set last year of projected average wage increases, inclusive of
   // TR_YEAR - 1.
   const int LASTYEAR = BaseYearNonFile::TR_YEAR + NUMPROJYEARS - 2;
   // ensure bounds are great enough to hold data
   const int YEAR78 = YEAR79 - 1;
   const int i1 = min(YEAR78, newIstart);
   const int i2 = max(LASTYEAR, newMaxyear);
   for (int i = 0; i < NUMASSUMSETS; i++) {
      awincprojtr[i].resetYears(i1, i2);
      // Set historical data
      for (int year = YEAR78; year < BaseYearNonFile::YEAR - 1; year++) {
         awincprojtr[i][year] = awincHist[year - YEAR78];
      }
      // set title and projected values
      read(i+1);
   }
}

// Description: Destructor.
AwincNonFile::~AwincNonFile()
{ }

// Description: Restore values to original values.
//
// Arguments:
//   altNum: Desired alternative number (1-4).
void AwincNonFile::read( int altNum )
{
   try {
      // set title
      titletr[altNum - 1] = titletrOriginal[altNum - 1];
      // Set last year of projected average wage increases, inclusive of
      // TR_YEAR - 1.
      const int LASTYEAR = BaseYearNonFile::TR_YEAR + NUMPROJYEARS - 2;
      // set values in short-range
      for (int year = BaseYearNonFile::YEAR - 1; year <= LASTYEAR; year++) {
         awincprojtr[altNum - 1][year] =
            awincprojtrOriginal[altNum - 1][year - BaseYearNonFile::YEAR + 1];
      }
      // set values in long-range
      for (int year = LASTYEAR + 1;
         year <= awincprojtr[altNum - 1].getLastYear(); year++) {
         awincprojtr[altNum - 1][year] = awincult[altNum - 1];
      }
   } catch (PiaException& e) {
      string s("In AwincNonFile::read: ");
      s += e.what();
      throw PiaException(s);
   }
}

// Description: Dummy function.
void AwincNonFile::write( int )
{ }

// Description: Sets data for one alternative.
//
// Remarks: The version with one argument uses the data stored for the
//   specified alternative. The version with two arguments stores the data for
//   the specified alternative, then sets the current data to that data.
//
// Arguments:
//   altNum: Desired alternative number (1-7).
void AwincNonFile::setData( int altNum )
{
   const int i1 = max(firstYear, awinc.getBaseYear());
   if (AssumptionType::isTR(altNum)) {
      awinc.assign(awincprojtr[altNum - 1], i1, awinc.getLastYear());
   }
   else {
      awinc.assign(0.0, i1, awinc.getLastYear());
   }
}

// Arguments:
//   newData: Data to use.
void AwincNonFile::setData( int altNum, const DoubleAnnual& newData )
{
   if (AssumptionType::isTR(altNum)) {
      awincprojtr[altNum - 1].assign(newData, firstYear,
         awincprojtr[altNum - 1].getLastYear());
   }
   setData(altNum);
}

// Description: Sets the current title.
//
// Remarks: The version with one argument uses the title stored for the
//   specified alternative. The version with two arguments stores the title
//   for the specified alternative, then sets the current title to that title.
//
// Arguments:
//   altNum: Desired alternative number (1-7).
void AwincNonFile::setTitle( int altNum )
{
   if (AssumptionType::isNotTR(altNum)) {
      awincTitle = avgwage[altNum - AssumptionType::FLAT];
   }
   else {
      awincTitle = titletr[altNum - 1];
   }
}

// Arguments:
//   newTitle: Title to use.
void AwincNonFile::setTitle( int altNum, const std::string& newTitle )
{
   if (AssumptionType::isNotTR(altNum)) {
      awincTitle = avgwage[altNum - AssumptionType::FLAT];
   }
   else {
      titletr[altNum - 1] = newTitle;
      awincTitle = titletr[altNum - 1];
   }
}
