// $Id: biprojfl.cpp 1.23 2006/11/08 11:04:34EST 044579 Development  $
//
// Functions to handle projected benefit increases stored on disk.

#include <fstream>
#include <iomanip>
#include "BiprojFile.h"
#include "Resource.h"
#include "comma.h"
#include "PiaException.h"
#include "AssumptionType.h"
#include "Path.h"

using namespace std;

// Description: Initializes projected benefit increase info.
//
// Arguments:
//   newIstart: Starting year of benefit increases.
//   newMaxyear: Maximum projected year.
//   newDirectory: Directory with benefit increase file.
BiprojFile::BiprojFile( int newIstart, int newMaxyear,
const std::string& newDirectory ) : Biproj(newIstart, newMaxyear),
pathname()
{
   pathname = Path::combine(newDirectory, "bixxx.dat");
}

// Description: Reads projected benefit increases.
//
// Throws: PiaException of type PIA_IDS_CPIINC4 if unable to open input file.
//   PiaException of type PIA_IDS_CPIINC3 if unable to read all required data.
//
// Arguments:
//   altNum: Desired alternative number (1-4).
void BiprojFile::read( int altNum )
{
   int temp;  // temporary istart
   string str;

   setBiFile(altNum);
   ifstream in(pathname.c_str(), ios::in);
   if (!in.is_open())
      throw PiaException(PIA_IDS_CPIINC4);
   // read title
   getline(in, str);
   biprojTitle = str;
   // read first year
   if (!(in >> temp))
      throw PiaException(PIA_IDS_CPIINC3);
   if (in.eof() || in.fail())
      throw PiaException(PIA_IDS_CPIINC3);
   firstYear = temp;
   // read all data
   try {
      theData.read(in, firstYear, theData.getLastYear());
   } catch (PiaException&) {
      throw PiaException(PIA_IDS_CPIINC3);
   }
   in.close();
}

// Description: Saves projected benefit increases.
//
// Throws: PiaException of type PIA_IDS_CPIINC2 if unable to open output file.
//
// Arguments:
//   altNum: Desired alternative number (1-4).
void BiprojFile::write( int altNum )
{
   setBiFile(altNum);
   ofstream out(pathname.c_str(), ios::out);
   if (!out.is_open())
      throw PiaException(PIA_IDS_CPIINC2);
   out.setf(ios::fixed, ios::floatfield);
   // write title
   out << biprojTitle << endl;
   // write first year of data
   out << firstYear << endl;
   // write all data
   theData.write(out, firstYear, theData.getLastYear());
   out.close();
}

// Description: Forms a file name for projected benefit increase file.
//
// Arguments:
//   altNum: Desired alternative number (1-4).
void BiprojFile::setBiFile( int altNum )
{
   const string directoryName(Path::getDirectoryName(pathname));
   const string coreName("bi" + poutNoComma(firstYear) + poutNoComma(altNum) +
     ".dat");
   pathname = Path::combine(directoryName, coreName);
}

// Description: Dummy function.
void BiprojFile::setData( int )
{ }

void BiprojFile::setData( int, const DoubleAnnual& )
{ }

// Description: Sets title of assumptions if not read in.
//
// Arguments:
//   altNum: Which set of assumptions.
void BiprojFile::setTitle( int altNum )
{
   if (AssumptionType::isNotTR(altNum)) {
      biprojTitle = beninc[altNum - AssumptionType::FLAT];
   }
}

void BiprojFile::setTitle( int altNum, const std::string& )
{
   if (AssumptionType::isNotTR(altNum)) {
      biprojTitle = beninc[altNum - AssumptionType::FLAT];
   }
}
