// $Id: biprojnf.cpp 1.49 2008/04/25 12:59:43EDT 277133 Development  $
//
// Functions to handle projected benefit increases stored in arrays.

#include "BiprojNonFile.h"
#include "Resource.h"
#include "PiaException.h"
#include "AssumptionType.h"
#include "BaseYearNonFile.h"
#include "AwbiDataNonFile.h"
#include "oactcnst.h"

using namespace std;

// Description: Titles for 2008 Trustees Report update
const string BiprojNonFile::titletrOriginal[NUMASSUMSETS] = {
   "2008 Trustees Report Alternative I",
   "2008 Trustees Report Alternative II",
   "2008 Trustees Report Alternative III",
   "No increase beginning with 2008 benefit increase"
};

// Remarks: Benefit increases for 2008 Trustees Report, 2008 to 2022
const double BiprojNonFile::biprojtrOriginal[NUMASSUMSETS][NUMPROJYEARS] = {
   {           2.4, 1.9, 1.8,
     1.8, 1.8, 1.8, 1.8, 1.8, 
     1.8, 1.8, 1.8, 1.8, 1.8,
     1.8, 1.8 },
   {           2.7, 2.5, 2.8, 
     2.8, 2.8, 2.8, 2.8, 2.8,
     2.8, 2.8, 2.8, 2.8, 2.8,
     2.8, 2.8 },
   {           3.5, 2.8, 3.1, 
     4.6, 5.9, 5.5, 4.6, 3.9, 
     3.8, 3.8, 3.8, 3.8, 3.8,
     3.8, 3.8 },
   {           0.0, 0.0, 0.0, 
     0.0, 0.0, 0.0, 0.0, 0.0, 
     0.0, 0.0, 0.0, 0.0, 0.0,
     0.0, 0.0 }
};

// Remarks: Ultimate benefit increases for 2008 Trustees Report.
// These have been the same for several years.
const double BiprojNonFile::biult[NUMASSUMSETS] = {
   1.8, 2.8, 3.8, 0.0
};

// Description: Initializes projected benefit increase info for latest Trustees
//   Report.
//
// Arguments:
//   newIstart: Starting year of benefit increases.
//   newMaxyear: Maximum projected year.
BiprojNonFile::BiprojNonFile( int newIstart, int newMaxyear ) :
Biproj(newIstart, newMaxyear)
{
   // Set last year of projected benefit increases, inclusive of TR_YEAR.
   const int LASTYEAR = BaseYearNonFile::TR_YEAR + NUMPROJYEARS - 1;
   // ensure bounds are great enough to hold data
   const int i1 = min(YEAR79, newIstart);
   const int i2 = max(LASTYEAR, newMaxyear);
   for (int i = 0; i < NUMASSUMSETS; i++) {
      biprojtr[i].resetYears(i1, i2);
      // Set historical data
      for (int year = YEAR79; year < BaseYearNonFile::YEAR; year++) {
         biprojtr[i][year] = AwbiDataNonFile::getCpiincData(year);
      }
      // set title and projected values
      read(i+1);
   }
}

// Description: Destructor.
BiprojNonFile::~BiprojNonFile()
{ }

// Description: Restore values to original values.
//
// Arguments:
//   altNum: Desired alternative number (1-4).
void BiprojNonFile::read( int altNum )
{
   try {
      // set title
      titletr[altNum - 1] = titletrOriginal[altNum - 1];
      // Set last year of projected benefit increases, inclusive of TR_YEAR.
      const int LASTYEAR = BaseYearNonFile::TR_YEAR + NUMPROJYEARS - 1;
      // set values in short-range
      for (int year = BaseYearNonFile::YEAR; year <= LASTYEAR; year++) {
         biprojtr[altNum - 1][year] =
            biprojtrOriginal[altNum - 1][year - BaseYearNonFile::YEAR];
      }
      // set values in long-range
      for (int year = LASTYEAR + 1; year <= biprojtr[altNum - 1].getLastYear();
      year++) {
         biprojtr[altNum - 1][year] = biult[altNum - 1];
      }
   } catch (PiaException& e) {
      string s("In BiprojNonFile::read: ");
      s += e.what();
      throw PiaException(s);
   }
}

// Description: Dummy function.
void BiprojNonFile::write( int )
{ }

// Description: Sets data for one alternative.
//
// Remarks: The version with one argument uses the data stored for the
//   specified alternative. The version with two arguments stores the data
//   for the specified alternative, then sets the current data to that data.
//
// Arguments:
//   altNum: Desired alternative number (1-7).
void BiprojNonFile::setData( int altNum )
{
   const int i1 = max(firstYear, theData.getBaseYear());
   if (AssumptionType::isTR(altNum)) {
      theData.assign(biprojtr[altNum - 1], i1, theData.getLastYear());
   }
   else {
      theData.assign(0.0, i1, theData.getLastYear());
   }
}

// Arguments:
//   newData: Data to use.
void BiprojNonFile::setData( int altNum, const DoubleAnnual& newData )
{
   if (AssumptionType::isTR(altNum)) {
      biprojtr[altNum - 1].assign(newData, firstYear,
         biprojtr[altNum - 1].getLastYear());
   }
   setData(altNum);
}

// Description: Sets the current title.
//
// Remarks: The version with one argument uses the title stored for the
//   specified alternative. The version with two arguments stores the title for
//   the specified alternative, then sets the current title to that title.
//
// Arguments:
//   altNum: Desired alternative number (1-7).
void BiprojNonFile::setTitle( int altNum )
{
   if (AssumptionType::isNotTR(altNum)) {
      biprojTitle = beninc[altNum - AssumptionType::FLAT];
   }
   else {
      biprojTitle = titletr[altNum - 1];
   }
}

// Arguments:
//   newTitle: Title to use.
void BiprojNonFile::setTitle( int altNum, const std::string& newTitle )
{
   if (AssumptionType::isNotTR(altNum)) {
      biprojTitle = beninc[altNum - AssumptionType::FLAT];
   }
   else {
      titletr[altNum - 1] = newTitle;
      biprojTitle = titletr[altNum - 1];
   }
}
