// $Id: cachup.cpp 1.13 2006/11/08 11:04:49EST 044579 Development  $
//
// Functions to handle catch-up benefit increases.

#include <cmath>
#include <vector>
#include "oactcnst.h"
#include "cachup.h"
#include "Resource.h"
#include "PiaException.h"
#include "comma.h"
#include "StringParser.h"

using namespace std;

// Description: Initializes catch-up benefit increase data.
CatchupIncreases::CatchupIncreases()
{
   fill(theData, theData + NUM_CATCHUP_INCREASES, 0.0);
}

// Description: Destroys catch-up benefit increase data.
CatchupIncreases::~CatchupIncreases()
{ }

// Description: Parses catch-up increases for one year of eligibility from a
//   string.
//
// Throws: PiaException of type PIA_IDS_READERR if unable to parse all required
//   data.
//
// Arguments:
//   str: String to parse.
void CatchupIncreases::parseString( std::string str )
{
   vector< string > strvec = ::parseString(str);
   if (strvec.size() < NUM_CATCHUP_INCREASES)
      throw PiaException(PIA_IDS_READERR);
   for (int i = 0; i < NUM_CATCHUP_INCREASES; i++) {
      double dtemp = atof(strvec[i].c_str());
      Catchup::check(dtemp);
      set(i, dtemp);
   }
}

// Description: Returns catch-up increases for one year of eligibility as a
//   string.
//
// Returns: Catch-up increases for one year of eligibility as a string.
std::string CatchupIncreases::toString() const
{
   string s("");
   for (int i1 = 0; i1 < NUM_CATCHUP_INCREASES; i1++)
      s += poutNoComma(theData[i1], 4, 1);
   return s;
}

// Description: Initializes catch-up benefit increase data.
//
// Arguments:
//   newCstart: Starting year of catch-up benefit increases. If omitted, the
//     starting year is set to 0.
Catchup::Catchup( int newCstart ) : cstart(newCstart)
{ }

Catchup::Catchup() : cstart(0)
{ }

// Description: Destroys catch-up benefit increases.
Catchup::~Catchup()
{ }

// Description: Checks for existence of catch-up benefit increase.
//
// Throws: PiaException of type PIA_IDS_CACHUP4 if either argument year is out
//   of range (only in debug mode).
//
// Returns: False if there is no catch-up increase, true otherwise.
//
// Arguments:
//   eligYear: Year of eligibility for catch-up benefit increases.
//   calYear: Calendar year of catch-up benefit increase.
bool Catchup::exist( int eligYear, int calYear ) const
{
#if !defined(NDEBUG)
   if (eligYear < 1900 || calYear < YEAR37)
      throw PiaException(PIA_IDS_CACHUP4);
#endif
   const int cstart2 = cstart + 2;
   if (calYear < cstart2 ||
      calYear >= cstart2 + CatchupIncreases::NUM_CATCHUP_INCREASES) {
      return(false);
   }
   if (eligYear < cstart || eligYear >= cstart + NUM_CATCHUP_ELIG_YEARS)
      return(false);
   if (catchup[eligYear - cstart].get(calYear - cstart2) < .05)
      return(false);
   return(true);
}

// Description: Sets one catch-up increase.
//
// Throws: PiaException of type PIA_IDS_CACHUP4 if either argument year is out
//   of range (only in debug mode).
//
// Arguments:
//   eligYear: Year of eligibility.
//   calYear: Calendar year of increase.
//   newCatchup: New value for catch-up increase.
void Catchup::set( int eligYear, int calYear, double newCatchup )
{
#if !defined(NDEBUG)
   if (eligYear < 1900 || calYear < YEAR37)
      throw PiaException(PIA_IDS_CACHUP4);
#endif
   check(newCatchup);
   const int cstart2 = cstart + 2;
   if (eligYear >= cstart && eligYear < cstart + NUM_CATCHUP_ELIG_YEARS &&
      calYear >= cstart2 &&
      calYear < cstart2 + CatchupIncreases::NUM_CATCHUP_INCREASES) {
      catchup[eligYear - cstart].set(calYear - cstart2, newCatchup);
   }
}

// Description: Checks a catch-up benefit increase.
//
// Throws: PiaException of type PIA_IDS_CACHUP1 if catch-up benefit increase
//   is out of range.
//
// Arguments:
//   newCatchup: Value to check.
void Catchup::check( double newCatchup )
{
   if (newCatchup > 99.99 || newCatchup < 0.0)
      throw PiaException(PIA_IDS_CACHUP1);
}

// Description: Zeroes out catch-up benefit increases.
void Catchup::deleteContents()
{
   for (int i2 = 0; i2 < NUM_CATCHUP_ELIG_YEARS; i2++) {
      for (int i1 = 0; i1 < CatchupIncreases::NUM_CATCHUP_INCREASES; i1++) {
         catchup[i2].set(i1, 0.0);
      }
   }
}

// Description: Parses all catch-up increases for one year of eligibility
//   from a string.
//
// Arguments:
//   index: Index of year of eligibility to parse (0-9).
//   inputLine: Line to parse.
void Catchup::parseString( int index, std::string inputLine )
{
   catchup[index].parseString(inputLine);
}
