// $Id: cachupfl.cpp 1.17 2005/12/12 09:44:14EST 044579 Exp  $
//
// Functions to handle catch-up benefit increases.

#include <fstream>
#include <cmath>
#include "CatchupFile.h"
#include "Resource.h"
#include "comma.h"
#include "PiaException.h"
#include "AssumptionType.h"
#include "Path.h"

using namespace std;

// Description: Initializes catch-up benefit increase file.
//
// Arguments:
//   newCstart: Starting year of catch-up benefit increases.
//   newDirectory: Directory with catch-up benefit increases file.
CatchupFile::CatchupFile( int newCstart, const std::string& newDirectory ) :
CatchupDoc(newCstart), pathname()
{
   pathname = Path::combine(newDirectory, "cuxxxxx.dat");
}

// Description: Destroys catch-up benefit increase file.
CatchupFile::~CatchupFile()
{ }

// Description: Reads all catch-up increases.
//
// Throws: PiaException of type PIA_IDS_CACHUP2 if unable to open input file.
//
// Arguments:
//   altNum: Desired alternative number (1-4).
void CatchupFile::read( int altNum )
{
   string inputLine;

   setCuFile(altNum);
   ifstream in(pathname.c_str(), ios::in);
   if (!in.is_open())
      throw PiaException(PIA_IDS_CACHUP2);
   for (int i = 0; i < 10; i++) {
      getline(in, inputLine);
      catchup.parseString(i, inputLine);
   }
   in.close();
}

// Description: Writes all catch-up increases.
//
// Throws: PiaException of type PIA_IDS_CACHUP3 if unable to open output file.
//
// Arguments:
//   altNum: Desired alternative number (1-4).
void CatchupFile::write( int altNum )
{
   setCuFile(altNum);
   ofstream out(pathname.c_str(), ios::out);
   if (!out.is_open())
      throw PiaException(PIA_IDS_CACHUP3);
   out.setf(ios::fixed, ios::floatfield);
   for (int i2 = 0; i2 < 10; i2++)
      out << catchup.getCatchupIncreases(i2).toString() << endl;
   out.close();
}

// Description: Forms a file name for projected catch-up benefit increase file.
//
// Arguments:
//   altNum: Desired alternative number (1-7).
void CatchupFile::setCuFile( int altNum )
{
   const string directoryName(Path::getDirectoryName(pathname));
   const string coreName("cu" + poutNoComma(catchup.getCstart()) +
     poutNoComma(altNum) + ".dat");
   pathname = Path::combine(directoryName, coreName);
}

// Description: Deletes current catch-up increases if other than TR assumption.
//
// Arguments:
//   altNum: Desired alternative number (1-7).
void CatchupFile::setData( int altNum )
{
   if (AssumptionType::isNotTR(altNum)) {
      catchup.deleteContents();
   }
}

void CatchupFile::setData( int altNum, const Catchup& )
{
   if (AssumptionType::isNotTR(altNum)) {
      catchup.deleteContents();
   }
}
