// $Id: msdates.cpp 1.25 2007/03/09 17:09:24EST 044579 Development  $
//
// Functions to read and write a military service dates record.

#include <sstream>
#include <iomanip>
#include <utility>  // for rel_ops
#include "msdates.h"
#include "PiaException.h"
#include "Resource.h"
#include "DateFormatter.h"

using namespace std;
#if !defined(__SGI_STL_INTERNAL_RELOPS)
using namespace std::rel_ops;
#endif

// First possible date of military service wage credits.
const DateMoyr MilServDates::firstMilServDate = DateMoyr(9, 1940);
// Last possible date of military service wage credits.
const DateMoyr MilServDates::lastMilServDate = DateMoyr(12, 2001);

// Description: Creates a MilServDates record.
MilServDates::MilServDates() : startDate(), endDate(),
count1(0), count2(0), theType(' ')
{ }

// Description: Destroys a MilServDates record.
MilServDates::~MilServDates()
{ }

// Description: Parses a line of data into a record, including count1, count2,
//   and type.
//
// Arguments:
//   line: Line to parse.
void MilServDates::parseLine( const std::string& line )
{
   setCount1(atoi(line.substr(0, 1).c_str()));
   setCount2(atoi(line.substr(1, 1).c_str()));
   parseDates(line.substr(2));
   setType(line[14]);
}

// Description: Parses a line of data into two dates.
//
// Arguments:
//   line: Line to parse.
void MilServDates::parseDates( const std::string& line )
{
   startDate = DateFormatter::month_year_from_undelimited_us_string(
      line.substr(0, 6));
   endDate = DateFormatter::month_year_from_undelimited_us_string(
      line.substr(6, 6));
}

// Description: Returns a string with the two dates, without spaces.
//
// Returns: A string with the two dates, without spaces.
std::string MilServDates::getDatesString() const
{
   string s(DateFormatter::toString(startDate, "n"));
   s += DateFormatter::toString(endDate, "n");
   return s;
}

// Description: Returns a string with the military service info, with spaces
//   appropriate for a print output.
//
// Returns: A string with the military service info.
std::string MilServDates::toFormattedString() const
{
   ostringstream strm;
   strm << getCount1() << setw(4) << getCount2()
      << setw(5) << startDate.getMonth()
      << setw(9) << startDate.getYear()
      << setw(4) << endDate.getMonth()
      << setw(8) << endDate.getYear()
      << setw(4) << getType();
   return(strm.str());
}

// Description: Returns a string including count1, count2, the two dates, and
//   the type.
//
// Returns: A string including count1, count2, the two dates, and the type.
std::string MilServDates::toString() const
{
   ostringstream strm;
   strm << setw(2) << getCount1() << setw(2) << getCount2()
      << getDatesString() << getType();
   return strm.str();
}

// Description: Checks a military service record for consistent dates.
//
// Throws: PiaException of type PIA_IDS_MILSERV1 if starting date is after
//   ending date.
void MilServDates::check() const
{
   startDateCheck();
   endDateCheck();
   // check for consistent dates
   if (startDate > endDate)
      throw PiaException(PIA_IDS_MILSERV1);
}

// Description: Returns pre-1951 quarters of coverage in a one-line record.
//
// Throws: PiaException of type PIA_IDS_MILSERV6 if years are out of range
//   (only in debug mode).
//
// Returns: Pre-1951 quarters of coverage in a one-line record.
int MilServDates::getQc3750() const
{
   int firstqc;  // index of first qc earned
   int lastqc;  // index of last qc earned

   if (startDate.getYear() < 1951) {
      firstqc = 4 * startDate.getYear() + (startDate.getMonth() - 1) / 3;
      lastqc = (endDate.getYear() < 1951) ?
         (4 * endDate.getYear() + (endDate.getMonth() - 1) / 3) :
         4 * 1950 + 3;
#if !defined(NDEBUG)
      if (lastqc - firstqc + 1 > 56)
         throw PiaException(PIA_IDS_MILSERV6);
#endif
      return(lastqc - firstqc + 1);
   }
   else
      return(0);
}

// Description: Returns quarters of coverage for one year in a one-line record.
//
// Returns: Quarters of coverage for one year in a one-line record.
//
// Arguments:
//   year: Year for which quarters are desired.
int MilServDates::getQcov( unsigned year ) const
{
   int firstqc;  // index of first qc earned
   int lastqc;  // index of last qc earned

   if (startDate.getYear() <= year && endDate.getYear() >= year) {
      firstqc = (startDate.getYear() < year) ?
         0 : (startDate.getMonth() - 1) / 3;
      lastqc = (endDate.getYear() > year) ?
         3 : (endDate.getMonth() - 1) / 3;
      return(lastqc - firstqc + 1);
   }
   else
      return(0);
}

// Description: Returns pre-1951 wage credits in a one-line record.
//
// Returns: Pre-1951 wage credits in a one-line record.
//
// Throws: PiaException of type PIA_IDS_MILSERV7 if years are out of range
//   (only in debug mode).
double MilServDates::getEarn3750() const
{
   int firstmonth;  // index of first month of credits
   int lastmonth;  // index of last month of credits

   if (startDate.getYear() < 1951) {
      firstmonth = 12 * startDate.getYear() +
         startDate.getMonth();
      lastmonth = (endDate.getYear() < 1951) ?
         (12 * endDate.getYear() + endDate.getMonth()) :
         12 * 1950 + 12;
#if !defined(NDEBUG)
      if (lastmonth - firstmonth + 1 > 168)
         throw PiaException(PIA_IDS_MILSERV7);
#endif
      return((lastmonth - firstmonth + 1) * 160.);
   }
   else
      return(0.);
}

// Description: Returns wage credits for one year in a one-line record.
//
// Returns: Wage credits for one year in a one-line record.
//
// Arguments:
//   year: Year for which wage credits are desired.
double MilServDates::getEarn( unsigned year ) const
{
   int firstmonth;  // index of first month of credits
   int lastmonth;  // index of last month of credits

   if (startDate.getYear() <= year && endDate.getYear() >= year) {
      // Based on earnings, not months, after 1977.  But we don't have
      // earnings, so we just return the maximum.
      if (year > 1977) {
         return 1200.;
      }
      firstmonth = (startDate.getYear() < year) ?
         1 : startDate.getMonth();
      lastmonth = (endDate.getYear() > year) ?
         12 : endDate.getMonth();
      // convert to quarters after 1956
      if (year > 1956) {
         firstmonth = (firstmonth + 2) / 3;
         lastmonth = (lastmonth + 2) / 3;
         return((lastmonth - firstmonth + 1) * 300.);
      }
      else
         return((lastmonth - firstmonth + 1) * 160.);
   }
   else
      return(0.);
}

// Description: Zeroes out dates of military service.
void MilServDates::deleteContents()
{
   count1 = count2 = 0;
   theType = ' ';
   startDate.deleteContents();
   endDate.deleteContents();
}

// Throws: PiaException of type PIA_IDS_MILSERV4 if month or year is out of
//   range.
//
// Arguments:
//   date: The date to check.
void MilServDates::startDateCheck( const DateMoyr& date )
{
   if (date < firstMilServDate || date > lastMilServDate)
      throw PiaException(PIA_IDS_MILSERV4);
}

// Throws: PiaException of type PIA_IDS_MILSERV5 if month or year is out of
//   range.
//
// Arguments:
//   date: The date to check.
void MilServDates::endDateCheck( const DateMoyr& date )
{
   if (date < firstMilServDate || date > lastMilServDate)
      throw PiaException(PIA_IDS_MILSERV5);
}
