// $Id: msdatesvec.cpp 1.13 2005/10/06 11:55:51EDT 044579 Exp  $
//
// Functions to read and write an array of military service dates.

#include "MilServDatesVec.h"
#include "comma.h"
#include "PiaException.h"
#include "Resource.h"

using namespace std;

// Description: Creates a MilServDatesVec record.
MilServDatesVec::MilServDatesVec() : msCount(0)
{ }

// Description: Destroys a MilServDatesVec record.
MilServDatesVec::~MilServDatesVec()
{ }

// Description: Checks a multiline record.
//
// Throws: PiaException of type PIA_IDS_MILSERV1 if starting date is after
//   ending date in any record.
void MilServDatesVec::check() const
{
   for (unsigned i = 0; i < getMSCount(); i++) {
      msdates[i].check();
   }
}

// Description: Returns pre-1951 quarters of coverage in a multiline record.
//
// Returns: Pre-1951 quarters of coverage in a multiline record.
unsigned MilServDatesVec::getQc3750() const
{
   unsigned rv = 0;  // return value

   for (unsigned i = 0; i < getMSCount(); i++)
      rv += msdates[i].getQc3750();
   return(rv);
}

// Description: Returns quarters of coverage for one year in a multiline
//   record.
//
// Returns: Quarters of coverage for one year in a multiline record.
//
// Arguments:
//   year: Year for which quarters are desired.
unsigned MilServDatesVec::getQcov( unsigned year ) const
{
   unsigned rv = 0;  // return value

   for (unsigned i = 0; i < getMSCount(); i++)
      rv += msdates[i].getQcov(year);
   return(min(rv, 4u));
}

// Description: Returns pre-1951 wage credits in a multiline record.
//
// Returns: Pre-1951 wage credits in a multiline record.
double MilServDatesVec::getEarn3750() const
{
   double rv = 0.;  // return value

   for (unsigned i = 0; i < getMSCount(); i++)
      rv += msdates[i].getEarn3750();
   return(rv);
}

// Description: Returns wage credits for one year in a multiline record.
//
// Returns: Wage credits for one year in a multiline record.
//
// Arguments:
//   year: Year for which wage credits are desired.
double MilServDatesVec::getEarn( unsigned year ) const
{
   double rv = 0.0;  // return value

   for (unsigned i = 0; i < getMSCount(); i++)
      rv += msdates[i].getEarn(year);
   // eliminate any possible double-counting
   if (year > 1956) {
      return min(rv, 1200.0);
   }
   else {
      return min(rv, 1920.0);
   }
}

// Description: Zeroes out dates of military service.
void MilServDatesVec::deleteContents()
{
   msCount = 0;
   for (unsigned i = 0; i < MAX_MILSERVDATES; i++)
      msdates[i].deleteContents();
}

// Description: Sets the number of military service records.
//
// Throws: PiaException of type PIA_IDS_MILSERV2 if number of military service
//   periods is out of range.
//
// Arguments:
//   newMSCount: New number of military service records.
void MilServDatesVec::setMSCount( unsigned newMSCount )
{
   msCountCheck(newMSCount);
   msCount = newMSCount;
}

// Description: Checks the number of military service periods.
//
// Throws: PiaException of type PIA_IDS_MILSERV2 if number of military service
//   periods is out of range.
//
// Arguments:
//   newMSCount: Number of military service periods to check.
void MilServDatesVec::msCountCheck( unsigned newMSCount )
{
   if (newMSCount > MAX_MILSERVDATES)
      throw PiaException(PIA_IDS_MILSERV2);
}
