// $Id: piacal.cpp 1.157 2008/04/25 13:00:09EDT 277133 Development  $
//
// Functions to calculate a PIA.

#include <cmath>  // for floor
#include <utility>  // for rel_ops
#include "piacal.h"
#include "oactcnst.h"
#include "OldStart.h"
#include "PiaException.h"
#include "WorkerDataArray.h"
#include "PiadataArray.h"
#include "Resource.h"
#include "piaparms.h"
#include "WageBase.h"
#include "ReindWid.h"
#include "WageInd.h"
#include "SpecMin.h"
#include "PiaTable.h"
#include "Pifc.h"
#include "DebugCase.h"
#if defined(DEBUGCASE)
#include "Trace.h"
#include "comma.h"
#include "DateFormatter.h"
#endif

using namespace std;
#if !defined(__SGI_STL_INTERNAL_RELOPS)
using namespace std::rel_ops;
#endif

// Description: Initializes a PiaCal instance.
//
// Arguments:
//   newWorkerData: Worker's basic data.
//   newPiaData: Worker's pia calculation data.
//   newPiaParams: Pia calculation parameters.
PiaCal::PiaCal( WorkerDataGeneral& newWorkerData, PiaData& newPiaData,
const PiaParams& newPiaParams ) :
workerData(newWorkerData), piaData(newPiaData), piaParams(newPiaParams),
piaMethod(), widowMethod(), wageInd(0), oldStart(0), specMin(0), piaTable(0),
frozMin(0), childCareCalc(0), transGuar(0), highPiaMethod(0),
ioasdi(WorkerData::NO_BEN), pebsAssumptions(false)
{ }

// Description: Destructor.
PiaCal::~PiaCal()
{ }

// Description: Figures high pia and mfb.
void PiaCal::piaCal1()
{
   try {
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      Trace::writeLine(workerData.getIdString() +
         ": At top of PiaCal::piaCal1");
   }
#endif
   setHighPia();
   // find support PIA
   setSupportPia();
   // Apply DI maximum.
   if (ioasdi == WorkerDataGeneral::DISABILITY &&
      piaData.getEligYear() > 1978 &&
      workerData.getEarliestEntDate() >= PiaParams::amend80) {
      const double dimaxTemp = (wageInd == static_cast<WageInd *>(0)) ?
         0.0 : wageInd->diMax(wageInd->getAme(), piaData.highPia.get());
      for (vector< PiaMethod * >::iterator iter = piaMethod.begin();
      iter != piaMethod.end(); iter++) {
         if ((*iter)->getApplicable() > 0 &&
            (*iter)->getMethod() != PiaMethod::DIB_GUAR) {
            (*iter)->mfbEnt.set(dimaxTemp);
         }
      }
   }
   setHighMfb();
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      Trace::writeLine(workerData.getIdString() +
         ": Returning from PiaCal::piaCal1");
   }
#endif
   } catch (exception& e) {
      throw PiaException(string("In PiaCal::piaCal1: ") + e.what());
   }
}

// Description: Figures benefit for primary.
void PiaCal::piaCal2()
{
   try {
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      Trace::writeLine(workerData.getIdString() +
         ": At top of PiaCal::piaCal2");
   }
#endif
   int i1 = workerData.getBenefitDate().getYear();
   if (static_cast<int>(workerData.getBenefitDate().getMonth()) <
      piaParams.getMonthBeninc(i1)) {
      i1--;
   }
   // calculate benefit payable for primary
   if (ioasdi != WorkerData::SURVIVOR) {
      ardriCal();
      // check for special minimum with delayed retirement credit
      if (piaData.getIapps() >= 0) {
         piaData.benefit.set(BenefitAmount::round(piaData.getArf() *
            piaData.supportPia.get(), i1));
         setArfApp();
      }
      else {
         if (workerData.getEntDate() < PiaParams::amend50) {
            piaData.benefit = piaData.highPia;
         }
         else {
            piaData.benefit.set(BenefitAmount::round(piaData.getArf() *
               piaData.highPia.get(), i1));
         }
      }
      // round to lower dollar if June 1982 or later
      piaData.roundedBenefit = piaData.benefit;
      piaData.roundedBenefit.round1(workerData.getBenefitDate());
   }
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      Trace::writeLine(workerData.getIdString() +
         ": Returning from PiaCal::piaCal2");
   }
#endif
   } catch (exception& e) {
      throw PiaException(string("In PiaCal::piaCal2: ") + e.what());
   }
}

// Description: Figures benefits for auxiliaries.
//
// Arguments:
//   widowArray: Widow and other family members' calculation data.
//   secondaryArray: Secondary benefit data.
void PiaCal::piaCal3( PiaDataArray& widowArray,
SecondaryArray& secondaryArray )
{
   int i1 = workerData.getBenefitDate().getYear();
   if (static_cast<int>(workerData.getBenefitDate().getMonth()) <
      piaParams.getMonthBeninc(i1)) {
      i1--;
   }
   // calculate benefit for each family member using usual pia
   for (int i2 = 0; i2 < widowArray.getFamSize(); i2++) {
      Secondary * secondaryptr = secondaryArray.secondary[i2];
      // check for special minimum with delayed retirement credit
      // this currently does not work because there is no arf in a survivor
      // case
      //if (secondaryptr->bic.getMajorBic() == 'D' && piaData.getIapps() >= 0){
      //   bentemp = piaData.getArf() * piaData.supportPia.get();
      //   bentemp.round(i1);
      //   setArfApp();
      //}
      const double bentemp = piaData.highPia.get();
      secondaryptr->pifc.set(piaData.getPifc());
      // calculate full benefit
      secondaryptr->setFullBenefit(BenefitAmount::round(bentemp *
         secondaryptr->getBenefitFactor(), i1));
   }
   // reset benefit for widow(er)s using widow(er) pia
   for (vector< ReindWid * >::iterator iter = widowMethod.begin();
   iter != widowMethod.end(); iter++) {
      Secondary& secondary = (*iter)->secondary;
      // check for re-indexed widow(er) method for widow(er)s
      if (piaData.highPia < (*iter)->piaEnt) {
         const double bentemp = (*iter)->piaEnt.get();
         secondary.pifc.set('W');
         // calculate full benefit
         secondary.setFullBenefit(BenefitAmount::round(bentemp *
            secondary.getBenefitFactor(), i1));
      }
   }
   // reduce benefit for family maximum
   if (ioasdi == WorkerData::SURVIVOR)
      applySurvivorMfb(secondaryArray, widowArray.getFamSize(), i1);
   else
      applyLifeMfb(secondaryArray, widowArray.getFamSize(), i1);
   // reduce benefits for age, then round
   for (int i2 = 0; i2 < widowArray.getFamSize(); i2++) {
      Secondary * secondaryptr = secondaryArray.secondary[i2];
      if (secondaryptr->bic.isReducible()) {
         secondaryptr->setReducedBenefit(BenefitAmount::round(
            secondaryptr->getArf() * secondaryptr->getBenefit(), i1));
      }
      else {
         secondaryptr->setReducedBenefit(secondaryptr->getBenefit());
      }
      // round to lower dollar if June 1982 or later
      double bentemp = secondaryptr->getReducedBenefit();
      bentemp = BenefitAmount::round1(bentemp, workerData.getBenefitDate());
      secondaryptr->setRoundedBenefit(bentemp);
   }
}

// Description: Sets support pia, if applicable pia is special minimum and
//   there is delayed retirement credit involved.
//
// Remarks: Delayed retirement credit cannot be applied to a special minimum
// pia, but it can be applied to the support pia (the next-highest pia) when
// calculating the old-age benefit or widow(er) benefit.
void PiaCal::setSupportPia()
{
   if (piaData.fullRetAge < piaData.ageEnt &&
      piaData.getIappn() == PiaMethod::SPEC_MIN) {
      // method producing support pia
      PiaMethod *supportMethod = static_cast<PiaMethod *>(0);
      for (vector< PiaMethod * >::iterator iter = piaMethod.begin();
      iter != piaMethod.end(); iter++) {
         if (piaData.supportPia < (*iter)->piaEnt &&
            (*iter)->getMethod() != PiaMethod::SPEC_MIN) {
            piaData.supportPia = (*iter)->piaEnt;
            piaData.setIapps((*iter)->getMethod());
            supportMethod = (*iter);
#if defined(DEBUGCASE)
            if (isDebugPid(workerData.getIdNumber())) {
               string s("");
               s += workerData.getIdString();
               s += ", ";
               s += (*iter)->getTitle();
               s += ", piaEnt ";
               s += (*iter)->piaEnt.toString();
               s += ": Setting iapps in setSupportPia";
               Trace::writeLine(s);
            }
#endif
         }
         else {
#if defined(DEBUGCASE)
            if (isDebugPid(workerData.getIdNumber())) {
               string s("");
               s += workerData.getIdString();
               s += ", ";
               s += (*iter)->getTitle();
               s += ", piaEnt ";
               s += (*iter)->piaEnt.toString();
               s += ": Not setting iapps in setSupportPia";
               Trace::writeLine(s);
            }
#endif
         }
      }
#if defined(DEBUGCASE)
      if (isDebugPid(workerData.getIdNumber())) {
         string s("");
         s += workerData.getIdString();
         s += ", getIapps() ";
         s += poutNoComma(piaData.getIapps());
         s += ": Ready to call setApplicable";
         Trace::writeLine(s);
      }
#endif
      if (supportMethod != static_cast<PiaMethod *>(0)) {
         supportMethod->setApplicable(PiaMethod::SUPPORT_PIA);
      }
   }
}

// Description: Sets applicable pia.
//
// Remarks: This should be called after all the pia's have been calculated.
// It also sets the indicator for type of pia.
void PiaCal::setHighPia()
{
   piaData.setIappn(-1);
   for (vector< PiaMethod * >::iterator iter = piaMethod.begin();
   iter != piaMethod.end(); iter++) {
      // set highest PIA
      if (piaData.highPia < (*iter)->piaEnt) {
         piaData.highPia = (*iter)->piaEnt;
         piaData.setIappn((*iter)->getMethod());
         highPiaMethod = (*iter);
      }
   }
   if (highPiaMethod != static_cast<PiaMethod *>(0)) {
      highPiaMethod->setApplicable(PiaMethod::HIGH_PIA);
   }
   setPifc();
}

// Description: Sets applicable mfb.
//
// Remarks: This should be called after the high pia is set. The mfb is based
// on the same type as the high pia, except when the high pia is based on the
// re-indexed widow(er) method. In that case, the high mfb is gotten from the
// wage-indexed method.
void PiaCal::setHighMfb()
{
   if (piaData.getIappn() > -1) {
      if (piaData.getIappn() == PiaMethod::REIND_WID) {
         piaData.highMfb = wageInd->mfbEnt;
      }
      else {
         piaData.highMfb = highPiaMethod->mfbEnt;
      }
   }
}

// Description: Sets indicator for delayed retirement credit in special minimum
// case.
void PiaCal::setArfApp()
{
   if (specMin != static_cast<SpecMin *>(0)) {
      if (specMin->piaEnt < piaData.benefit) {
         piaData.setArfApp(PiaData::SUPPORT_BEN);
      }
      else {
         piaData.benefit = specMin->piaEnt;
         piaData.setArfApp(PiaData::SPEC_MIN_BEN);
      }
   }
}

// Description: Sets applicable pifc.
void PiaCal::setPifc()
{
   const OldStart::OldStartType methodOs =
      (oldStart == static_cast<OldStart *>(0)) ?
      OldStart::OS1939 : oldStart->getMethodOs();
   const PiaMethod::WindfallElimType windfall =
      (wageInd == static_cast<WageInd *>(0)) ?
      PiaMethod::NOWINDFALLELIM : wageInd->getWindfall();
   piaData.setPifc(Pifc::pifcCal(piaData.getIappn(), windfall,
      methodOs, piaData.getAmend90(), workerData.getTotalize()));
}

// Description: Checks date of entitlement for a primary benie.
//
// Remarks: Assumes type of benefit is WorkerDataGeneral::OLD_AGE or
// WorkerDataGeneral::DISABILITY.
//
// Throws: PiaException of type PIA_IDS_ARDRI1 if age is out of range.
//   PiaException of type PIA_IDS_ARDRI2 if age is before early retirement age
//   for OAB. PiaException of type PIA_IDS_ARDRI7 if DIB entitlement before
//   1957. PiaException of type PIA_IDS_ARDRI8 if age under 50 for DIB before
//   1960. PiaException of type PIA_IDS_ARDRI9 if age over full retirement age
//   for DIB.
void PiaCal::ageEntCheck() const
{
   if (piaData.ageEnt.getYears() <= 0)  // impossible age
      throw PiaException(PIA_IDS_ARDRI1);

   // old-age

   if (ioasdi == WorkerDataGeneral::OLD_AGE) {
      // retirement before earliest possible age
      if (piaData.ageEnt < piaData.earlyRetAge)
         throw PiaException(PIA_IDS_ARDRI2);
   }

   // disability

   if (ioasdi == WorkerDataGeneral::DISABILITY) {
      if (workerData.getEntDate().getYear() <= 1959) {
         if (workerData.getEntDate().getYear() < 1957)
            throw PiaException(PIA_IDS_ARDRI7);
         if (piaData.ageEnt.getYears() < 50)
            throw PiaException(PIA_IDS_ARDRI8);
         return;
      }
      if (!(piaData.ageEnt < piaData.fullRetAge)) {
#if defined(DEBUGCASE)
         if (isDebugPid(workerData.getIdNumber())) {
            string s("");
            s += workerData.getIdString();
            s += ", ageEnt ";
            s += piaData.ageEnt.toString();
            s += ", fullRetAge ";
            s += piaData.fullRetAge.toString();
            s += ": DIB too old in PiaCal::ardriCal";
            Trace::writeLine(s);
         }
#endif
         throw PiaException(PIA_IDS_ARDRI9);
      }
   }
}

// Description: Calculates the number of months of early or delayed retirement
//   for a primary benie.
//
// Remarks: If old-age, it calls monthsArCal or monthsDriCal, if entitlement is
// before or after full retirement age, respectively. If disability, it calls
// monthsArDICal.
//
// Assumes type of benefit is WorkerDataGeneral::OLD_AGE or
// WorkerDataGeneral::DISABILITY.
void PiaCal::ardriCal() const
{
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": At top of PiaCal::ardriCal";
      Trace::writeLine(s);
   }
#endif

   // delayed retirement increment or early retirement reduction
   if (ioasdi == WorkerData::OLD_AGE) {  // only for old-age
      // early retirement reduction
      if (piaData.ageEnt < piaData.fullRetAge) {
         monthsArCal();
      }
      else {  // delayed retirement credit
         monthsDriCal();
      }
   }

   // disability

   if (ioasdi == WorkerData::DISABILITY) {
      monthsArDICal();
   }
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": Returning from PiaCal::ardriCal";
      Trace::writeLine(s);
   }
#endif
}

// Description: Calculates the benefit factor and number of months of early or
//   delayed retirement for a dependent or survivor.
//
// Throws: PiaException of type PIA_IDS_ARDRI10 if aged wife age is before
//   early retirement age. PiaException of type PIA_IDS_JSURV if invalid bic.
//   PiaException of type PIA_IDS_ARDRI4 if disabled widow(er) age is before
//   earliest possible age. PiaException of type PIA_IDS_ARDRI5 if disabled
//   widow(er) entitlement is before benefit date. PiaException of type
//   PIA_IDS_ARDRI6 if aged widow(er) age is before earliest possible age.
//
// Arguments:
//   widowData: Family member's basic information.
//   widowPiaData: Family member's primary benefit data.
//   secondaryData: Family member's secondary benefit data.
//   entDate: Entitlement date.
void PiaCal::ardriAuxCal( WorkerDataGeneral *widowData, PiaData *widowPiaData,
Secondary *secondaryData, const DateMoyr& entDate ) const
{
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": At top of PiaCal::ardriAuxCal";
      Trace::writeLine(s);
   }
#endif
   // life cases

   if (ioasdi == WorkerData::OLD_AGE || ioasdi == WorkerData::DISABILITY) {
      switch (secondaryData->bic.getMajorBic())
      {
      case 'B':  // all wives
         if (!secondaryData->bic.isYoungSpouse()) {
            // handle aged wife (assumes earliest possible retirement age is
            // same as for old-age benie; true so far)
            widowPiaData->earlyRetAge = piaParams.earlyAgeOabCal(
               widowData->getSex(), widowPiaData->getBirthDateMinus1());
            if (widowPiaData->ageEnt < widowPiaData->earlyRetAge)
               throw PiaException(PIA_IDS_ARDRI10);
            // widow(er)'s full retirement age
            widowPiaData->fullRetAge = piaParams.fullRetAgeCal(
               widowPiaData->getBirthDateMinus1().year() + 62);
            monthsArAgedSpouseCal(widowPiaData, secondaryData);
#if defined(DEBUGCASE)
            if (isDebugPid(workerData.getIdNumber())) {
               string s("");
               s += workerData.getIdString();
               s += ": Returning from PiaCal::ardriAuxCal for case aged B";
               Trace::writeLine(s);
            }
#endif
            return;
         }
         // fall through for young wives
      case 'C':  // children
         monthsYngLifeCal(secondaryData);
#if defined(DEBUGCASE)
         if (isDebugPid(workerData.getIdNumber())) {
            string s("");
            s += workerData.getIdString();
            s += ": Returning from PiaCal::ardriAuxCal for case C or young B";
            Trace::writeLine(s);
         }
#endif
         return;
      default:  // no life auxiliary
         throw PiaException(PIA_IDS_JSURV);
      }
   }

   // survivors

   if (ioasdi == WorkerData::SURVIVOR) {
      switch (secondaryData->bic.getMajorBic())
      {
      case 'C':  // young survivors
      case 'E':
         monthsYngSurvCal(secondaryData);
#if defined(DEBUGCASE)
         if (isDebugPid(workerData.getIdNumber())) {
            string s("");
            s += workerData.getIdString();
            s += ": Returning from PiaCal::ardriAuxCal for case C or E";
            Trace::writeLine(s);
         }
#endif
         return;
      case 'W':  // disabled widow
         widowPiaData->earlyRetAge = piaParams.earlyAgeDisWidCal(
            workerData.getBenefitDate());
         if (widowPiaData->ageEnt < widowPiaData->earlyRetAge)
            throw PiaException(PIA_IDS_ARDRI4);
         if (!(widowPiaData->ageEnt < piaParams.earlyAgeWidCal(
            workerData.getBenefitDate()))) {
            throw PiaException(PIA_IDS_ARDRI5);
         }
         // disabled widow's full retirement age
         widowPiaData->fullRetAge = piaParams.fullRetAgeCal(
            widowPiaData->getBirthDateMinus1().year() + 60);
         monthsArDisWidCal(widowPiaData, secondaryData, entDate);
#if defined(DEBUGCASE)
         if (isDebugPid(workerData.getIdNumber())) {
            string s("");
            s += workerData.getIdString();
            s += ": Returning from PiaCal::ardriAuxCal for case W";
            Trace::writeLine(s);
         }
#endif
         return;
      case 'D':  // aged widow
         widowPiaData->earlyRetAge =
            piaParams.earlyAgeWidCal(workerData.getBenefitDate());
         if (widowPiaData->ageEnt < widowPiaData->earlyRetAge)
            throw PiaException(PIA_IDS_ARDRI6);
         // widow's full retirement age
         widowPiaData->fullRetAge = piaParams.fullRetAgeCal(
            widowPiaData->getBirthDateMinus1().year() + 60);
         monthsArWidCal(widowPiaData, secondaryData);
#if defined(DEBUGCASE)
         if (isDebugPid(workerData.getIdNumber())) {
            string s("");
            s += workerData.getIdString();
            s += ": Returning from PiaCal::ardriAuxCal for case D";
            Trace::writeLine(s);
         }
#endif
         return;
      default:  // no survivor
         throw PiaException(PIA_IDS_JSURV);
      }
   }
}

// Description: Calculates number of months of delayed retirement credit for
//   OAB and sets increment factor.
//
// Remarks: First finds the date first fully insured by calling fullInsDateCal,
// since delayed retirement credits are only available if fully insured. Then
// it calls the static PiaParams::monthsDriCal to get the number of months.
// It uses that number to calculate the total amount of delayed retirement
// credit by calling the static PiaParams::factorDriCal.
//
// This is called from ardriCal.
void PiaCal::monthsDriCal() const
{
   int eligYear = piaData.getEligDate().getYear();
   piaData.setFullInsDate(fullInsDateCal());
   piaData.setMonthsArdri(PiaParams::monthsDriCal(piaData.getFullRetDate(),
      eligYear, piaData.getBirthDateMinus1(), workerData.getEntDate(),
      workerData.getBenefitDate(), piaData.getFullInsDate()));
   piaData.setArf(PiaParams::factorDriCal(piaData.getMonthsArdri(), eligYear));
}

// Description: Calculates number of months of actuarial reduction for OAB and
//   sets reduction factor.
void PiaCal::monthsArCal() const
{
   piaData.setMonthsArdri(piaParams.monthsArCal(piaData.ageEnt,
      piaData.fullRetAge));
   piaData.setArf(piaParams.factorArCal(piaData.getMonthsArdri()));
}

// Description: Calculates number of months of actuarial reduction for DIB and
//   sets reduction factor.
void PiaCal::monthsArDICal() const
{
   piaData.setMonthsArdri(piaParams.monthsArDICal(workerData.getOabEntDate(),
      workerData.getOabCessDate()));
   piaData.setArf(piaParams.factorArCal(piaData.getMonthsArdri()));
}

// Description: Calculates number of months of actuarial reduction for disabled
//   widow(er) and sets reduction factor.
//
// Arguments:
//   widowData: Family member's information.
//   secondaryData: Family member's secondary benefit.
//   entDate: The date of entitlement.
void PiaCal::monthsArDisWidCal( PiaData *widowData,
Secondary *secondaryData, const DateMoyr& entDate ) const
{
   secondaryData->setMonthsArdri(piaParams.monthsArDisWidCal(
      widowData->ageEnt, workerData.getBenefitDate(), widowData->fullRetAge));
   secondaryData->setBenefitFactor(piaParams.factorDisWidCal(
      widowData->getEligYear(), workerData.getBenefitDate()));
   secondaryData->setArf(piaParams.factorArDisWidCal(
      secondaryData->getMonthsArdri(), entDate));
}

// Description: Calculates number of months of actuarial reduction for young
//   survivor and sets benefit factor.
//
// Arguments:
//   secondaryData: Family member's secondary benefit.
void PiaCal::monthsYngSurvCal( Secondary *secondaryData ) const
{
   secondaryData->setMonthsArdri(0);
   secondaryData->setBenefitFactor(piaParams.factorYngSurvCal());
}

// Description: Calculates number of months of actuarial reduction for young
//   auxiliary and sets benefit factor.
//
// Arguments:
//   secondaryData: Family member's secondary benefit.
void PiaCal::monthsYngLifeCal( Secondary *secondaryData ) const
{
   secondaryData->setMonthsArdri(0);
   secondaryData->setBenefitFactor(piaParams.factorYngLifeCal());
}

// Description: Calculates full benefit factor and number of months of
//   actuarial reduction for aged widow(er) and sets reduction factor.
//
// Arguments:
//   widowData: Family member's information.
//   secondaryData: Family member's secondary benefit.
void PiaCal::monthsArWidCal( PiaData *widowData,
Secondary *secondaryData ) const
{
   secondaryData->setMonthsArdri(piaParams.monthsArWidCal(widowData->ageEnt,
      workerData.getBenefitDate(), widowData->fullRetAge));
   secondaryData->setBenefitFactor(piaParams.factorAgedWidCal(
      secondaryData->getMonthsArdri(), widowData->ageEnt,
      widowData->getEligYear(), workerData.getBenefitDate()));
   secondaryData->setArf(piaParams.factorArWidCal(
      secondaryData->getMonthsArdri(), widowData->ageEnt,
      workerData.getBenefitDate(), widowData->fullRetAge));
}

// Description: Calculates full benefit factor and number of months of
//   actuarial reduction for aged wife or husband and sets reduction factor.
//
// Arguments:
//   widowData: Family member's information.
//   secondaryData: Family member's secondary benefit.
void PiaCal::monthsArAgedSpouseCal( PiaData *widowData,
Secondary *secondaryData ) const
{
   secondaryData->setMonthsArdri(piaParams.monthsArAgedSpouseCal(
      widowData->ageEnt, workerData.getBenefitDate(), widowData->fullRetAge));
   secondaryData->setBenefitFactor(piaParams.factorAgedSpouseCalPL());
   secondaryData->setArf(piaParams.factorArAgedSpouseCal(
      secondaryData->getMonthsArdri()));
}

// Description: Calculates PIA and benefit.
//
// Throws: PiaException of type PIA_IDS_QCTOT0 if no quarters of coverage in
//   a totalization case.
//
// Arguments:
//   entDate: The date of entitlement.
void PiaCal::calculate2( const DateMoyr& entDate )
{
   try {
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      Trace::writeLine(workerData.getIdString() +
         ": At top of PiaCal::calculate2");
   }
#endif
   piaData.earnTotal50Cal0();
   piaData.freezeYearsCal(workerData, ioasdi, piaParams.fullRetAgeCalDI(
      piaData.getBirthDateMinus1().year() + 62, entDate.getYear()));
   piaData.earnYearCal(workerData, ioasdi);
   if (workerData.getTotalize()) {
      if (piaData.getQcTotal() == 0)
         throw PiaException(PIA_IDS_QCTOT0);
      piaData.setRelEarnPositionAverage(relEarnPositionCal());
      piaData.earnTotal50Cal1();
      // limit earnings attributed for totalization to wage base
      piaParams.getBaseOasdiArray().earnLimit(piaData.earnTotalizedLimited,
         piaData.earnTotalized);
   }
   nCal(piaData.compPeriodNew, entDate);
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      ostringstream strm;
      strm << workerData.getIdString() << ", n "
         << piaData.compPeriodNew.getN()
         << ": After nCal in PiaCal::calculate2";
      Trace::writeLine(strm.str());
   }
#endif
   } catch (exception& e) {
      throw PiaException(string("In PiaCal::calculate2: ") + e.what());
   }
}

// Description: Calculates annual quarters of coverage and insured status.
void PiaCal::qcCal()
{
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": At top of PiaCal::qcCal";
      Trace::writeLine(s);
   }
#endif
   piaData.earnTotal50Cal0();
   piaData.setQc3750simp();
   piaData.qcCal(workerData, piaParams.qcamt);
   piaData.finsCode.set(insCal(ioasdi == WorkerData::SURVIVOR ?
      workerData.getDeathDate() : boost::gregorian::date(
      static_cast<unsigned short>(workerData.getEntDate().getYear()),
      static_cast<unsigned short>(workerData.getEntDate().getMonth()), 1u),
      workerData.isPrimary(ioasdi)));
   piaData.setFinsCode2(fins2Cal(workerData, piaData, ioasdi));
   // calculate DI quarters of coverage
   if (ioasdi == WorkerData::DISABILITY) {
      piaData.disInsCode.set(disInsCal(workerData, piaData,
         workerData.getEntDate(), 1));
   }
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": Returning from PiaCal::qcCal";
      Trace::writeLine(s);
   }
#endif
}

// Description: Calculates relative earnings position.
//
// Throws: PiaException of type PIA_IDS_RELERNPOS if first year of earnings is
//   out of range (only in debug mode).
//
// Returns: Relative earnings position.
float PiaCal::relEarnPositionCal()
{
   double reptot = 0.0;  // total of annual rep's
   int i1;  // loop counter
   double earnstt;  // temporary earnings

   // set years of earnings
   piaData.setIbeginTotal(workerData);
#if !defined(NDEBUG)
   if (piaData.getIbeginTotal() < YEAR37)
      throw PiaException(PIA_IDS_RELERNPOS);
#endif
   piaData.setIendTotal(workerData);
   // calculate relative earnings position
   for (i1 = piaData.getIbeginTotal(); i1 <= piaData.getIendTotal(); i1++) {
      if (piaData.qcov.get(i1) > 0) {
         // set proportioned wage base
         double test = static_cast<double>(piaData.qcov.get(i1)) *
            piaParams.getBaseOasdi(i1) / 4.0;
         earnstt = (i1 < workerData.getIbegin() ||
            i1 > workerData.getIend()) ? 0.0 : piaData.earnOasdi[i1];
         double temprep = min(earnstt, test) / piaParams.getFq(i1);
         piaData.relEarnPosition[i1] =
            static_cast<float>(floor(temprep * 100000.0 + 0.5) / 100000.0);
         reptot += piaData.relEarnPosition[i1];
      }
      else
         piaData.relEarnPosition[i1] = 0.0f;
   }
   double rv = floor(400000.0 * reptot /
      static_cast<double>(piaData.getQcTotal()) + 0.5) / 100000.0;
   int temp = (ioasdi == WorkerData::SURVIVOR &&
      static_cast<int>(workerData.getDeathDate().year()) ==
      piaData.getEligYear()) ?
      piaData.getEligYear() + 1 : piaData.getEligYear();
   for (i1 = piaData.getIbeginTotal(); i1 <= piaData.getIendTotal(); i1++) {
      // fill in earnings from 22 to eligibility, plus other years with
      // at least 1 qc
      earnstt = 0.0;
      const int birthYear = piaData.getBirthDateMinus1().year();
      if (((i1 < birthYear + 22 || i1 >= temp) && piaData.qcov.get(i1) > 0) ||
         (i1 > birthYear + 21 && i1 < temp)) {
         earnstt = rv * piaParams.getFq(i1);
      }
      // prorate by quarter in year of death
      if (!workerData.getDeathDate().is_not_a_date()) {
         if (i1 == static_cast<int>(workerData.getDeathDate().year())) {
            const int qtr = (workerData.getDeathDate().month() + 2) / 3;
            earnstt *= qtr / 4.0;
         }
      }
      piaData.earnTotalized[i1] = floor(100.0 * earnstt + 0.5) / 100.0;
   }
   return static_cast<float>(rv);
}

// Description: Returns ending elapsed year for a beneficiary case.
//
// Returns: Ending elapsed year for a beneficiary case.
//
// Remarks: Assumes eligibility year has already been calculated.
//
// Arguments: 
//   entDate: Entitlement date.
int PiaCal::nelapsed2Cal( const DateMoyr& entDate )
{
   // account for 1960 ending year for 1960 or later method,
   // nondisability
   int elap2 = (entDate.getYear() > 1960 &&
      piaData.getEligYear() < 1961 &&
      workerData.getValdi() == 0) ? 1960 : piaData.getEligYear() - 1;
   if (ioasdi == WorkerData::SURVIVOR &&
      elap2 > static_cast<int>(workerData.getDeathDate().year()) - 1)
      elap2 = static_cast<int>(workerData.getDeathDate().year()) - 1;
   return(elap2);
}

// Description: Determine number of elapsed years.
//
// Remarks: The version with an entitlement date calculates the ending elapsed
// year, then calls the version with an ending elapsed year argument.
//
// Arguments:
//   compPeriod: Computation period to use (new or old).
//   entDate: Entitlement date.
void PiaCal::nelapsedCal( CompPeriod& compPeriod, const DateMoyr& entDate )
{
   // calculate last elapsed year
   int elapsed2 = nelapsed2Cal(entDate);
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": In PiaCal::nelapsedCal, elapsed2 = ";
      s += poutNoComma(elapsed2);
      Trace::writeLine(s);
   }
#endif
   nelapsedCal(compPeriod, elapsed2);
}

// Arguments:
//   compPeriod: Computation period to use (new or old).
//   elapsed2: Ending elapsed year.
void PiaCal::nelapsedCal( CompPeriod& compPeriod, int elapsed2 )
{
   // calculate first elapsed year
   int elapsed1 =
      max(static_cast<int>(piaData.getBirthDateMinus1().year()) + 21,
      compPeriod.getBaseYear());
   compPeriod.setNElapsed(elapsed2 - elapsed1);
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": In PiaCal::nelapsedCal, calling didropCal";
      Trace::writeLine(s);
   }
#endif
   compPeriod.setDiYears(didropCal(workerData, elapsed1, elapsed2,
      piaData.partialFreezeYears));
   if (compPeriod.getDiYears() > 0) {
      compPeriod.setNElapsed(compPeriod.getNElapsed() -
         compPeriod.getDiYears());
   }
   if (compPeriod.getNElapsed() < 2)
      compPeriod.setNElapsed(2);
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": At bottom of PiaCal::nelapsedCal, nelapsed = ";
      s += poutNoComma(compPeriod.getNElapsed());
      Trace::writeLine(s);
   }
#endif
}

// Description: Determine number of computation years and dropout years, before
//   considering 1-for-5 dropout.
//
// Remarks: Assumes elapsed years have already been computed.
//
// Arguments:
//   compPeriod: Computation period to use (new or old).
//   entDate: Entitlement date.
void PiaCal::nCal1( CompPeriod& compPeriod, const DateMoyr& entDate )
{
   // if entitlement in June 1992 or later, use 5 dropout
   if (entDate >= PiaParams::amend90)
      compPeriod.setNDrop(5);
   else {
      // if death is in Sept 1954 or later, use 5 dropout
      if (ioasdi == WorkerData::SURVIVOR) {
         // if disability freeze, use 5 dropout for disability in Sept
         // 1954 or later
         if (workerData.getValdi() > 0) {
            compPeriod.setNDrop(
               (DateMoyr(workerData.disabPeriod[0].getOnsetDate()) >=
               PiaParams::amend54) ? 5 : 0);
         }
         else {
            compPeriod.setNDrop((DateMoyr(workerData.getDeathDate()) >=
               PiaParams::amend54) ? 5 : 0);
         }
      }
      else {
         // if disability freeze, use 5 dropout for disability in Sept
         // 1954 or later
         if (workerData.getValdi() > 0) {
            compPeriod.setNDrop(
               (DateMoyr(workerData.disabPeriod[0].getOnsetDate()) >=
               PiaParams::amend54) ? 5 : 0);
         }
         else {
            // if entitlement is in Sept 1954 or later, use 5 dropout
            compPeriod.setNDrop(
               (workerData.getEntDate() >= PiaParams::amend54) ? 5 : 0);
         }
      }
   }
}

// Description: Determine number of computation years and dropout years.
//
// Remarks: Assumes elapsed years have already been computed.
//
// Arguments:
//   compPeriod: Computation period to use (new or old).
//   entDate: Entitlement date.
void PiaCal::nCal( CompPeriod& compPeriod, const DateMoyr& entDate )
{
   nCal1(compPeriod, entDate);
   // Apply DI 1-for-5 rule if:
   // old-age case with prior disability entitlement in July 1980 or later
   // that lasted to age 61, or
   // disability case with entitlement in July 1980 or later
   if ((ioasdi == WorkerDataGeneral::OLD_AGE && workerData.getValdi() > 0 &&
      workerData.disabPeriod[0].getEntDate() >= PiaParams::amend80 &&
      workerData.disabPeriod[0].getCessationDate() >=
      DateMoyr(workerData.getBirthDate().month(),
      workerData.getBirthDate().year() + 61))
      || (workerData.getEntDate() >= PiaParams::amend80 &&
      ioasdi == WorkerData::DISABILITY))
      // 1-for-5 dropout rule for DI, if not survivor
      compPeriod.setNDrop(min(compPeriod.getNElapsed() / 5, 5));

   compPeriod.setN(compPeriod.getNElapsed() - compPeriod.getNDrop());
   if (compPeriod.getN() < 2) {  // nx is minimum of 2
      compPeriod.setN(2);
      compPeriod.setNDrop(compPeriod.getNElapsed() - compPeriod.getN());
   }
   if (ioasdi == WorkerData::OLD_AGE &&
      workerData.getEntDate().getYear() < 1958 &&
      compPeriod.getBaseYear() == YEAR50) {
      // old-age modification prior to 1958
      compPeriod.setN(piaData.getEligYear() - 1953);
      if (compPeriod.getN() < 2) {
         compPeriod.setN(2);
         compPeriod.setNDrop(compPeriod.getNElapsed() - compPeriod.getN());
      }
   }
   return;
}

// Description: Projects earnings and fills in earnings to be used in
//   calculations.
void PiaCal::earnProjection() const
{
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": At top of PiaCal::earnProjection";
      Trace::writeLine(s);
   }
#endif
   piaData.earnProjection(workerData);
   piaData.earnHiCal(workerData);
   // limit earnings to wage base
   piaParams.getBaseOasdiArray().earnLimit(piaData.earnOasdiLimited,
      piaData.earnOasdi);
   piaParams.getBaseHiArray().earnLimit(piaData.earnHiLimited,
      piaData.earnHi);
}

// Description: Checks primary data for validity and does some preliminary
//   calculations.
//
// Arguments:
//   entDate: Entitlement date.
//
// Throws: PiaException of many different types if any data is out of range.
void PiaCal::dataCheck( const DateMoyr& entDate )
{
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": At top of PiaCal::dataCheck";
      Trace::writeLine(s);
   }
#endif
   workerData.joasdiCheck();
   // check for no earnings of any type
   workerData.earningsCheck();
   // check birthdate before anything whose check depends on it being valid.
   workerData.birth2Check();
   // check first year of earnings, if there are any earnings
   if (workerData.getIndearn())
      workerData.ibeginCheck();
   setIoasdi(workerData.getJoasdi());
   if (ioasdi != WorkerData::SURVIVOR) {
      workerData.entCheck();
      if (workerData.getRecalc()) {
         workerData.benefitDate2Check();
      }
      else {
         workerData.setBenefitDate();
      }
   }
   //if (piaData.getIndearn()) {
   //   earnProject.iendCheck(piaData.getIbegin(), piaData.getIend());
   //}
   piaData.setBirthDateMinus1(workerData.getBirthDate());
   piaData.fullRetAge =
      piaParams.fullRetAgeCal(piaData.getBirthDateMinus1().year() + 62);
   piaData.setFullRetDate();
   // calculate early retirement age
   if (ioasdi == WorkerData::OLD_AGE)  // only for old-age
      piaData.earlyRetAge = piaParams.earlyAgeOabCal(workerData.getSex(),
         piaData.getBirthDateMinus1());
   if (ioasdi != WorkerData::SURVIVOR) {
      // calculate age at entitlement
      piaData.ageEnt = workerData.getEntDate() -
         DateMoyr(piaData.getBirthDateMinus1());
      // calculate age at benefit date
      piaData.ageBen = workerData.getBenefitDate() -
         DateMoyr(piaData.getBirthDateMinus1());
   }
   workerData.qctdCheck2();
   workerData.qcCheck();
   // check noncovered pension
   if (getPubpenEnable(workerData)) {
      WorkerData::pubpenCheck(workerData.getPubpen());
      WorkerData::pubpenCheck(workerData.getPubpenReservist());
   }
   else {
      workerData.setPubpen(0.f);
      workerData.setPubpenReservist(0.f);
   }
   workerData.valdiCheck();
   if (workerData.getValdi()) {
      workerData.disCheck();
      if (DateMoyr(workerData.disabPeriod[0].getOnsetDate()) >=
         piaData.getFullRetDate()) {
         // disability after full retirement age
         throw PiaException(PIA_IDS_ONSET11);
      }
      if (ioasdi == WorkerDataGeneral::OLD_AGE && workerData.getValdi() > 0) {
         if (workerData.disabPeriod[0].getEntDate() >=
            piaData.getFullRetDate()) {
            // prior entitlement after full retirement age
            throw PiaException(PIA_IDS_PRRENT7);
         }
         workerData.priorentCheck();
#if defined(DEBUGCASE)
         if (isDebugPid(workerData.getIdNumber())) {
            string s("");
            s += workerData.getIdString();
            s += ": At end of workerData.priorentCheck of PiaCal::dataCheck";
            Trace::writeLine(s);
         }
#endif
      }
      if (workerData.needDiCess()) {
#if defined(DEBUGCASE)
         if (isDebugPid(workerData.getIdNumber())) {
            string s("");
            s += workerData.getIdString();
            s += ", cessationDate ";
            s += DateFormatter::toString(workerData.disabPeriod[0].getCessationDate());
            s += ", onsetDate ";
            s += DateFormatter::toString(workerData.disabPeriod[0].getOnsetDate());
            s += ", deathDate ";
            s += DateFormatter::toString(workerData.getDeathDate());
            s += ", entDate ";
            s += DateFormatter::toString(workerData.getEntDate());
            s += ", joasdi ";
            s += poutNoComma(static_cast<int>(workerData.getJoasdi()));
            s += ": Ready to call workerData.cessationCheck in ";
            s += "PiaCal::dataCheck";
            Trace::writeLine(s);
         }
#endif
         workerData.cessationCheck();
#if defined(DEBUGCASE)
         if (isDebugPid(workerData.getIdNumber())) {
            string s("");
            s += workerData.getIdString();
            s += ": Returned from workerData.cessationCheck in ";
            s += "PiaCal::dataCheck";
            Trace::writeLine(s);
         }
#endif
      }
      if (workerData.getValdi() > 1) {
#if defined(DEBUGCASE)
         if (isDebugPid(workerData.getIdNumber())) {
            string s("");
            s += workerData.getIdString();
            s += ", cessationDate 1 ";
            s += DateFormatter::toString(workerData.disabPeriod[1].getCessationDate());
            s += ", onsetDate 1 ";
            s += DateFormatter::toString(workerData.disabPeriod[1].getOnsetDate());
            s += ", entDate 1 ";
            s += DateFormatter::toString(workerData.disabPeriod[1].getEntDate());
            s += ", onsetDate 0 ";
            s += DateFormatter::toString(workerData.disabPeriod[0].getOnsetDate());
            s += ", entDate 0 ";
            s += DateFormatter::toString(workerData.disabPeriod[0].getEntDate());
            s += ", deathDate ";
            s += DateFormatter::toString(workerData.getDeathDate());
            s += ", entDate ";
            s += DateFormatter::toString(workerData.getEntDate());
            s += ", joasdi ";
            s += poutNoComma(static_cast<int>(workerData.getJoasdi()));
            s += ": Ready to call workerData.dis1Check in ";
            s += "PiaCal::dataCheck";
            Trace::writeLine(s);
         }
#endif
         workerData.dis1Check();
         workerData.cessation1Check();
      }
   }
   if (ioasdi == WorkerData::DISABILITY) {
#if defined(DEBUGCASE)
      if (isDebugPid(workerData.getIdNumber())) {
         string s("");
         s += workerData.getIdString();
         s += ": Ready to call workerData.waitpdCheck in PiaCal::dataCheck";
         Trace::writeLine(s);
      }
#endif
      workerData.waitpdCheck();
   }
   eligYearCal(workerData, piaData, ioasdi);
   if (ioasdi != WorkerData::SURVIVOR) {
      ageEntCheck();
   }
   nelapsedCal(piaData.compPeriodNew, entDate);
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": Returning from PiaCal::dataCheck";
      Trace::writeLine(s);
   }
#endif
}

// Description: Checks auxiliary data for validity and does some preliminary
//   calculations.
//
// Throws: PiaException of type PIA_IDS_BENDATE6 if secondary entitlement
//   after benefit date.
//
// Arguments:
//   widowDataArray: Basic information for all family members.
//   widowArray: Calculated data for all family members.
//   secondaryArray: Benefits for all family members.
void PiaCal::dataCheckAux( WorkerDataArray& widowDataArray,
PiaDataArray& widowArray, SecondaryArray& secondaryArray )
{
   Secondary * secondaryptr;
   WorkerData * widowDataptr;
   PiaData * widowPiaDataptr;
   int i;  // loop index

#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": At top of PiaCal::dataCheckAux, widowArray.getFamSize = ";
      s += poutNoComma(widowArray.getFamSize());
      Trace::writeLine(s);
   }
#endif
   // check auxiliary dates of entitlement against benefit date
   for (i = 0; i < widowArray.getFamSize(); i++) {
      secondaryptr = secondaryArray.secondary[i];
      if (secondaryptr->bic.getMajorBic() == ' ')
         break;
      try {
         WorkerData::benefitDate2Check(secondaryptr->entDate,
            workerData.getBenefitDate());
      } catch (PiaException e) {
         throw PiaException(PIA_IDS_BENDATE6);
      }
   }
   // check survivor information
   if (workerData.getJoasdi() == WorkerData::SURVIVOR) {
      // check date of birth against date of death
      workerData.deathCheck();
      // check for good survivor bics and dates of entitlement against death
      // date
      for (i = 0; i < widowArray.getFamSize(); i++) {
         secondaryptr = secondaryArray.secondary[i];
         if (secondaryptr->bic.getMajorBic() == ' ')
            break;
         Bic::survivorCheck(secondaryptr->bic.getMajorBic());
         WorkerData::deathCheck3(workerData.getDeathDate(),
            secondaryptr->entDate);
      }
   }
   for (i = 0; i < widowArray.getFamSize(); i++) {
      secondaryptr = secondaryArray.secondary[i];
      widowDataptr = widowDataArray.workerData[i];
      widowPiaDataptr = widowArray.piaData[i];
      secondaryptr->check();
      if (secondaryptr->bic.isReducible()) {
         widowPiaDataptr->setBirthDateMinus1(widowDataptr->getBirthDate());
         // calculate spouse's or widow(er)'s age at entitlement
         widowPiaDataptr->ageEnt = secondaryptr->entDate -
            DateMoyr(widowPiaDataptr->getBirthDateMinus1());
         if (secondaryptr->bic.getMajorBic() == 'W') {
            widowCheck(*widowDataptr, *secondaryptr);
         }
      }
   }
   if (workerData.getJoasdi() != WorkerData::PEBS_CALC) {
      // determine widow's year of eligibility
      setEligYearWidow(widowDataArray, widowArray, secondaryArray);
      // calculate benefit factor for all family members
      for (i = 0; i < widowArray.getFamSize(); i++) {
         widowDataptr = widowDataArray.workerData[i];
         widowPiaDataptr = widowArray.piaData[i];
         secondaryptr = secondaryArray.secondary[i];
         if (secondaryptr->bic.getMajorBic() == ' ') {
#if defined(DEBUGCASE)
            if (isDebugPid(workerData.getIdNumber())) {
               string s("");
               s += workerData.getIdString();
               s += ": Not calling ardriAuxCal because bic is blank";
               Trace::writeLine(s);
            }
#endif
            break;
         }
         ardriAuxCal(widowDataptr, widowPiaDataptr, secondaryptr,
            secondaryptr->entDate);
      }
   }
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": Returning from PiaCal::dataCheckAux";
      Trace::writeLine(s);
   }
#endif
}

// Description: Calculates the insured status code for one date.
//
// Returns: The insured status code for one date.
//
// Remarks: Requires adjusted date of birth, qcov filled in from 1937 to date,
//   qctot50, dates of disability onset and cessation. The version with 2
//   arguments calls the static version with 5 arguments.
//
// Arguments:
//   dateModyyr: The date for which insured status code is required.
//   isWasPrimary: Worker's primary beneficiary status.
//     0:  never a primary beneficiary.
//     1:  is a primary beneficiary.
//     2:  is a deceased primary beneficiary.
InsCode::InsCodeType PiaCal::insCal( const boost::gregorian::date& dateModyyr,
int isWasPrimary )
{
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": At top of PiaCal::insCal(dateModyyr, isWasPrimary)";
      Trace::writeLine(s);
   }
#endif
   // set quarter and year of desired insured status
   QtrYear qtrYear(dateModyyr);
   piaData.qcTotalCal(workerData, qtrYear);
   InsCode::InsCodeType rv = insCal(workerData, piaData, ioasdi, dateModyyr,
      isWasPrimary);
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": Returning from PiaCal::insCal(dateModyyr, isWasPrimary)";
      Trace::writeLine(s);
   }
#endif
   return rv;
}

// Description: Calculates the date fully insured status was first attained
// (at full retirement age or later).
//
// Remarks: First checks for fully insured status as of full retirement age.
// If uninsured at that time, checks every future quarter through the quarter
// of the benefit date. For this purpose, being insured for totalization
// benefits is considered to be fully insured.
//
// Requires adjusted date of birth, qcov filled in from 1937 to date,
// qctot50, dates of disability onset and cessation.
//
// This is called from monthsDriCal.
//
// Returns: The date fully insured status was first attained
// (at full retirement age or later).
DateMoyr PiaCal::fullInsDateCal() const
{
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": At top of PiaCal::fullInsDateCal";
      Trace::writeLine(s);
   }
#endif
   // quarter and year of insured status (start with date full retirement
   // age is attained)
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ", piaData.getFullRetDate() ";
      s += DateFormatter::toString(piaData.getFullRetDate());
      s += ": Creating qtrYear";
      Trace::writeLine(s);
   }
#endif
   QtrYear qtrYear(piaData.getFullRetDate());
   do {
#if defined(DEBUGCASE)
      if (isDebugPid(workerData.getIdNumber())) {
         string s("");
         s += workerData.getIdString();
         s += ", qtrYear ";
         s += qtrYear.toString();
         s += ": Calling piaData.qcTotalCal";
         Trace::writeLine(s);
      }
#endif
      piaData.qcTotalCal(workerData, qtrYear);
#if defined(DEBUGCASE)
      if (isDebugPid(workerData.getIdNumber())) {
         string s("");
         s += workerData.getIdString();
         s += ", qcTotal ";
         s += poutNoComma(piaData.getQcTotal());
         s += ", qtrYear ";
         s += qtrYear.toString();
         s += ": Calling insCal";
         Trace::writeLine(s);
      }
#endif
      InsCode::InsCodeType rv =
         insCal(workerData, piaData, ioasdi, qtrYear, 1);
      if (InsCode(rv).isFullyOrTotalInsured()) {
#if defined(DEBUGCASE)
         if (isDebugPid(workerData.getIdNumber())) {
            string s("");
            s += workerData.getIdString();
            s += ", fullInsDate ";
            s += DateFormatter::toString(qtrYear.toDateMoyr());
            s += ": Successfully returning from PiaCal::fullInsDateCal";
            Trace::writeLine(s);
         }
#endif
         return qtrYear.toDateMoyr();
      }
      else {
         // advance to next quarter and try again
         qtrYear.add(1);
      }
   } while (qtrYear.toDateMoyr() < workerData.getBenefitDate());
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ", fullInsDate ";
      s += DateFormatter::toString(qtrYear.toDateMoyr());
      s += ": Unsuccessfully returning from PiaCal::fullInsDateCal";
      Trace::writeLine(s);
   }
#endif
   // stop checking at benefit date
   return qtrYear.toDateMoyr();
}

// Description: Sets eligibility year for widow.
//
// Arguments:
//   widowDataArray: Basic information for all family members.
//   widowArray: Calculated data for all family members.
//   secondaryArray: Benefits for all family members.
void PiaCal::setEligYearWidow( WorkerDataArray& widowDataArray,
PiaDataArray& widowArray, SecondaryArray& secondaryArray ) const
{
   for (int i = 0; i < widowArray.getFamSize(); i++) {
      Secondary * secondaryptr = secondaryArray.secondary[i];
      if (secondaryptr->bic.isWidow()) {
         WorkerData * widowDataptr = widowDataArray.workerData[i];
         PiaData * widowPiaDataptr = widowArray.piaData[i];
         widowPiaDataptr->setEligYear(eligYearWidowCal(
            secondaryptr->bic.getMajorBic(),
            widowPiaDataptr->getBirthDateMinus1().year(),
            widowDataptr->disabPeriod[0].getOnsetDate()));
      }
   }
}

// Description: Returns eligibility year for widow.
//
// Returns: Eligibility year for widow.
//
// Arguments:
//   majorBic: Type of survivor.
//     blank: none.
//     C: child.
//     E: mother/father.
//     W: disabled widow(er).
//     D: aged widow(er).
//   byear: Adjusted year of birth of widow.
//   onsetDate: Date of onset of disability (unused if not disabled widow(er)).
int PiaCal::eligYearWidowCal( char majorBic, int byear,
boost::gregorian::date onsetDate )
{
   try {
      if (majorBic == 'W') {
         return(max(static_cast<int>(onsetDate.year()), byear + 50));
      }
      else {
         return(byear + 60);
      }
   } catch (exception& e) {
      throw PiaException(string("In PiaCal::eligYearWidowCal: ") + e.what());
   }
}

// Description: Applies maximum family benefit to each family benefit,
//   after ratio has been determined.
//
// Arguments:
//   secondaryArray: Benefits for all family members.
//   famSize: Number of family members.
//   ratio: Ratio to be applied to each family benefit.
//   year: Year of rounding.
void PiaCal::applyMfb( SecondaryArray& secondaryArray, int famSize,
double ratio, int year )
{
   // do not increase benefits
   if (ratio > 1.)
      ratio = 1.;
   // do not set benefits less than zero
   if (ratio < 0.)
      ratio = 0.;
   // apply ratio to full benefit
   for (int i1 = 0; i1 < famSize; i1++) {
      Secondary *secondary = secondaryArray.secondary[i1];
      if (secondary->eligibleForMax()) {
         secondary->setBenefit(BenefitAmount::round(
            ratio * secondary->getFullBenefit(), year));
      }
      else {
         secondary->setBenefit(secondary->getFullBenefit());
      }
   }
}

// Description: Calculates PIA for all re-indexed widow(er)s.
//
// Remarks:  This should be called immediately after piaCal in survivor cases.
//   (It doesn't hurt to call it in other than survivor cases, but it will
//   simply return without doing anything.)
//
// Arguments:
//   widowDataArray: Basic information for all family members.
//   widowArray: Calculated information for all family members.
//   secondaryArray: Benefits for all family members.
void PiaCal::reindWidCalAll( const WorkerDataArray& widowDataArray,
PiaDataArray& widowArray, SecondaryArray& secondaryArray )
{
   for (int i = 0; i < PiaDataArray::MAXFAMSIZE; i++) {
      if (secondaryArray.secondary[i]->isWidow()) {
         reindWidCal(*widowDataArray.workerData[i], *widowArray.piaData[i],
            *secondaryArray.secondary[i]);
      }
   }
}

// Description: Sets indicator for entitlement after 1990 amendments.
//
// Arguments:
//   entDate: Entitlement date.
void PiaCal::setAmend90( const DateMoyr& entDate )
{
   // use 1990 methods if entitlement in June 1992 or later, unless dib
   // conversion with dib entitlement prior to June 1992
   bool b1 = (entDate >= PiaParams::amend90);
   bool b2 = (workerData.getValdi() == 0 ||
      workerData.disabPeriod[0].getCessationDate() < entDate);
   piaData.setAmend90(b1 && b2);
}

// Description: Resets indicator for earnings used after 1991.
void PiaCal::resetAmend90()
{
   try {
   // if already true, return
   if (piaData.getAmend90()) {
      return;
   }
   // if pia table method unused, return
   if (piaTable == static_cast<PiaTable *>(0) ||
      piaTable->getApplicable() == 0) {
      return;
   }
   int lastYear = piaTable->iorder.getLastYear();
   bool b1 = false;
   // check for any earnings in 1992 or later
   for (int i = 1992; i <= lastYear; i++) {
      if (piaTable->iorder[i] > 0 && piaTable->earnIndexed[i] > 0.5) {
         b1 = true;
      }
   }
   piaData.setAmend90(b1);
   } catch (exception& e) {
      throw PiaException(string("In PiaCal::resetAmend90: ") + e.what());
   }
}

// Description: Returns true if ok to enable noncovered pension entry box.
//   Note that this is a static function.
//
// Returns: True if ok to enable noncovered pension entry box.
//
// Arguments:
//   workerData: Worker's basic data.
bool PiaCal::getPubpenEnable( const WorkerDataGeneral& workerData )
{
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": Starting PiaCal::getPubpenEnable";
      Trace::writeLine(s);
   }
#endif

   if (workerData.getBirthDate().is_not_a_date()) {
      throw PiaException("Birth date out of range");
   }
   try {
      boost::gregorian::date birthDate1 = workerData.getBirthDate() -
         boost::gregorian::date_duration(1);
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": in PiaCal::getPubpenEnable.  Joasdi = ";
      s += poutNoComma(workerData.getJoasdi());
      Trace::writeLine(s);
   }
#endif

      return (workerData.getJoasdi() != WorkerData::SURVIVOR &&
         birthDate1.year() + 62 > 1985);
   } catch (PiaException e) {
      // if date of birth has not yet been set, do not allow noncovered
      // pension entry
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": exception in PiaCal::getPubpenEnable.  Returning false by default.";
      Trace::writeLine(s);
   }
#endif

      return false;
   }
}

// Arguments:
//   workerData: Worker's basic data.
//   piaData: Benefit calculation data.
//   ioasdi: Type of benefit.
//   qtrYear: The date for which insured status code is required.
//   isWasPrimary: Worker's primary beneficiary status.
//     0:  never a primary beneficiary.
//     1:  is a primary beneficiary.
//     2:  is a deceased primary beneficiary.
InsCode::InsCodeType PiaCal::insCal( const WorkerDataGeneral& workerData,
PiaData& piaData, WorkerDataGeneral::ben_type ioasdi, const QtrYear& qtrYear,
int isWasPrimary )
{
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": At top of PiaCal::insCal(workerData, piaData, ioasdi, qtrYear, ";
      s += "isWasPrimary)";
      Trace::writeLine(s);
   }
#endif
   piaData.setQcReq(qcReqCal(workerData, piaData, ioasdi, qtrYear.getYear()));
   piaData.setQcReqPerm(qcReqPermCal(workerData, piaData, ioasdi));
   // calculate quarters of coverage for currently insured status
   piaData.setQcCurrent(qcCurrentCal(piaData, qtrYear));
   // deaths
   //if (isdead(date))
   //   return(deadIns(isWasPrimary));
   InsCode::InsCodeType rv = fins1Cal(workerData, piaData, qtrYear,
      isWasPrimary);
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": Returning from PiaCal::insCal(workerData, piaData, ioasdi, ";
      s += "qtrYear, isWasPrimary)";
      Trace::writeLine(s);
   }
#endif
   return rv;
}

// Description: Calculates required quarters of coverage for fully insured
//   status at end of year.
//
// Remarks: Note that this is a static function.
//
// Returns: Required quarters of coverage for fully insured
//   status at end of year.
//
// Arguments:
//   workerData: Worker's basic data.
//   piaData: Benefit calculation data.
//   ioasdi: Type of benefit.
//   year: Year for which number is required.
int PiaCal::qcReqCal( const WorkerDataGeneral& workerData, PiaData& piaData,
WorkerData::ben_type ioasdi, int year )
{
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": At top of PiaCal::qcReqCal";
      Trace::writeLine(s);
   }
#endif
   // set last elapsed year by looking at year of eligibility to old-age
   // benefits
   piaData.setEligDate(eligYearCal1(workerData, piaData, ioasdi, 0));
   int elapsed2 = min(year, 
      static_cast<int>(piaData.getEligDate().getYear()) - 1);
   // set first elapsed year
   int elapsed1 = max(
      static_cast<int>(piaData.getBirthDateMinus1().year() + 21), 1950);
   // check for years wholly or partially within period of disability
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": In PiaCal::qcReqCal, calling didropCal";
      Trace::writeLine(s);
   }
#endif
   int diYears = didropCal(workerData, elapsed1, elapsed2,
      piaData.partialFreezeYears);
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": Returning from PiaCal::qcReqCal";
      Trace::writeLine(s);
   }
#endif
   return min(40, max(6, elapsed2 - elapsed1 - diYears));
}

// Description: Calculates required quarters of coverage for permanently
//   insured status.
//
// Remarks: Note that this is a static function.
//
// Returns: Required quarters of coverage for permanently insured status.
//
// Arguments:
//   workerData: Worker's basic data.
//   piaData: Benefit calculation data.
//   ioasdi: Type of benefit.
int PiaCal::qcReqPermCal( const WorkerDataGeneral& workerData,
PiaData& piaData, WorkerDataGeneral::ben_type ioasdi )
{
   int didropout = 0;
   int elap1 = max(
      static_cast<int>(piaData.getBirthDateMinus1().year() + 21), 1950);
   piaData.setEligDate(eligYearCal1(workerData, piaData, ioasdi, 0));
   int elap2 = piaData.getEligDate().getYear() - 1;
   if (workerData.getValdi()) {
      FreezeYears frzyrst;  // temporary freeze years
#if defined(DEBUGCASE)
      if (isDebugPid(workerData.getIdNumber())) {
         string s("");
         s += workerData.getIdString();
         s += ": In PiaCal::qcReqPermCal, calling didropCal";
         Trace::writeLine(s);
      }
#endif
      didropout = didropCal(workerData, elap1, elap2, frzyrst);
   }
   return min(40, max(6, elap2 - elap1 - didropout));
}

// Description: Calculates quarters of coverage earned for currently insured
//   status.
//
// Remarks: Note that this is a static function.
//
// Returns: Quarters of coverage earned for currently insured status.
//
// Arguments:
//   piaData: Benefit calculation data.
//   qtrYear: Calendar quarter and year through which qc's are required.
int PiaCal::qcCurrentCal( const PiaData& piaData, const QtrYear& qtrYear )
{
   QtrYear startQtrYear;  // starting quarter and year of measuring period

#if !defined(NDEBUG)
   qtrYear.check();
#endif
   // use 13-quarter measuring period
   startQtrYear = qtrYear;
   startQtrYear.subtract(12);
   if (startQtrYear < PiaParams::year37qtr1)
      startQtrYear = PiaParams::year37qtr1;
   int tempqccur = piaData.qcov.accumulate(startQtrYear, qtrYear, 0u);
   return tempqccur;
}

// Description: Returns the insured status code for a deceased worker.
//
// Remarks: Note that this is a static function.
//
// Returns: The insured status code for a deceased worker.
//
// Arguments:
//   iswasPrimary: Worker's primary beneficiary status.
//     0:  never a primary beneficiary.
//     1:  is a primary beneficiary.
//     2:  is a deceased primary beneficiary.
InsCode::InsCodeType PiaCal::deadIns( int iswasPrimary )
{
   return((iswasPrimary == 2) ?
      InsCode::PRIMARYDEATH : InsCode::NONPRIMARYDEATH);
}

// Description: Computes year of eligibility.
//
// Arguments:
//   workerData: Worker's basic data.
//   piaData: Benefit calculation data.
//   ioasdi: Type of benefit.
void PiaCal::eligYearCal( const WorkerDataGeneral& workerData,
PiaData& piaData, WorkerDataGeneral::ben_type ioasdi )
{
   // determine worker's month and year of eligibility
   piaData.setEligDate(eligYearCal1(workerData, piaData, ioasdi, 0));
   // take disability into account
   piaData.setEligYear(eligYearCal2(workerData, piaData, ioasdi,
      piaData.getEligDate().getYear()));
}

// Description: Computes date of eligibility before considering disability.
//
// Remarks: Note that this is a static function.
//
// Returns: The date of eligibility before considering disability.
//
// Arguments:
//   workerData: Worker's basic data.
//   piaData: Benefit calculation data.
//   ioasdi: Type of benefit.
//   jind7: 0 for present law, 1-3 for change to age-65 comp point.
DateMoyr PiaCal::eligYearCal1( const WorkerDataGeneral& workerData,
PiaData& piaData, WorkerDataGeneral::ben_type ioasdi, int jind7 )
{
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": At top of PiaCal::eligYearCal1";
      Trace::writeLine(s);
   }
#endif
#if !defined(NDEBUG)
   if (piaData.getBirthDateMinus1().is_not_a_date()) {
      throw PiaException("birthDateMinus1 is not a date");
   }
#endif
   const int birthYear = piaData.getBirthDateMinus1().year();
   // start with month and year of age 62, plus any additional amount
   DateMoyr eligDateTemp(piaData.getBirthDateMinus1().month(),
      birthYear + 62 + jind7);
      // account for age-65 computation point for older male workers
   if (workerData.isMale()) {
      if (birthYear < 1914 && birthYear > 1909)
         eligDateTemp = DateMoyr(12, 1975);
      if (birthYear < 1910)
         eligDateTemp = DateMoyr(eligDateTemp.getMonth(), birthYear + 65);
   }
   if (ioasdi == WorkerData::SURVIVOR) {
      DateMoyr deathDateTemp(workerData.getDeathDate());
      if (deathDateTemp < eligDateTemp) {
         // for survivors, use month and year of death, if earlier
         eligDateTemp = deathDateTemp;
      }
   }
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": Returning from PiaCal::eligYearCal1";
      Trace::writeLine(s);
   }
#endif
   return eligDateTemp;
}

// Description: Computes year of eligibility after considering disability.
//
// Remarks: Note that this is a static function.
//
// Returns: Year of eligibility after considering disability.
//
// Arguments:
//   workerData: Worker's basic data.
//   piaData: Benefit calculation data.
//   ioasdi: Type of benefit (old-age, survivor, or disability).
//   eligYear: Year of attainment of age 62, for old-age and disability,
//     or year of death, for survivor.
int PiaCal::eligYearCal2( const WorkerDataGeneral& workerData,
PiaData& piaData, WorkerDataGeneral::ben_type ioasdi, int eligYear )
{
   int rv = eligYear;
   if (workerData.getValdi() > 0) {
      // determine which year of onset to use
      int onsetyear;
      const DisabPeriod& disabPeriod0 = workerData.disabPeriod[0];
      const DisabPeriod& disabPeriod1 = workerData.disabPeriod[1];
      if (workerData.getValdi() < 2 ||
         disabPeriod1.getCessationDate().getMonths(
         disabPeriod0.getOnsetDate()) > 12) {
         onsetyear = disabPeriod0.getOnsetDate().year();
      }
      else {
         onsetyear = disabPeriod1.getOnsetDate().year();
      }
      if (ioasdi != WorkerData::DISABILITY) {
         // consider disability onset prior to eligibility
         if (disabPeriod0.getCessationDate().getMonths(
            piaData.getEligDate()) < 13 && onsetyear < rv) {
            rv = onsetyear;
         }
      }
      if (ioasdi == WorkerData::DISABILITY) {
         if (rv > onsetyear)
            rv = onsetyear;
      }
   }
   return(rv);
}

// Description: Calculates and returns the insured status code for one date.
//
// Remarks: Requires that total and required quarters of coverage have been
// calculated, and that worker is alive.  Note that this is a static function.
//
// Returns: The insured status code for one date.
//
// Arguments:
//   workerData: Worker's basic data.
//   piaData: Benefit calculation data.
//   qtrYear: Quarter and year of desired insured status.
//   iswasPrimary: Worker's primary beneficiary status.
//     0:  never a primary beneficiary.
//     1:  is a primary beneficiary.
InsCode::InsCodeType PiaCal::fins1Cal( const WorkerDataGeneral& workerData,
PiaData& piaData, const QtrYear& qtrYear, int iswasPrimary )
{
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ", qcTotal ";
      s += poutNoComma(piaData.getQcTotal());
      s += ", qcReq ";
      s += poutNoComma(piaData.getQcReq());
      s += ": At top of PiaCal::fins1Cal";
      Trace::writeLine(s);
   }
#endif
   if (piaData.getQcTotal() < piaData.getQcReq()) {
      // insured for totalization benefits
      if (iswasPrimary > 0 && workerData.getTotalize())
         return(InsCode::TOTALIZED);
      // check for deemed insured status
      if (workerData.getDeemedind() && qtrYear.getYear() > 1983)
         if (deemedInsCal(piaData, qtrYear))
            return(InsCode::DEEMED);
      // not fully insured
      if (piaData.getQcTotal() == 0)
         return(InsCode::NOQCS);
      if (piaData.getQcCurrent() > 5)
         // currently insured
         return(InsCode::CURRENTNOTFULL);
      else {
         if (piaData.getQcTotal() > 2 && piaData.getQcTotal() +
            (workerData.isMale() ? 1887 : 1890) >
            static_cast<int>(piaData.getBirthDateMinus1().year()))
            // transitionally insured
            return(InsCode::TRANSITIONAL);
         else
            //uninsured in all ways
            return(InsCode::UNINSURED);
      }
   }
   else {
      // fully insured
      if (piaData.getQcTotal() < piaData.getQcReqPerm()) {
         // not permanently insured
         if (piaData.getQcCurrent() > 5)
            // currently insured
            return(InsCode::FULLANDCURRENT);
         else
            return(InsCode::FULLNOTCURRENT);
      }
      else {
         if (piaData.getQcCurrent() > 5)
            // currently insured
            return(InsCode::PERMANDCURRENT);
         else
            return(InsCode::PERMNOTCURRENT);
      }
   }
}

// Description: Calculates and returns the OACT insured status code for one
//   date.
//
// Remarks: Requires that CWHS insured status code has been calculated.  Note
// that this is a static function.
//
// Returns: The OACT insured status code.
//
//   1 - Fully insured in nontotalization case.
//   2 - Not fully insured, but currently insured, survivor.
//   3 - Totalization case with more than 5 qc's, but not fully insured.
//   4 - Not fully insured nontotalization case other than survivor.
//   5 - Not fully or currently insured survivor.
//   6 - Totalization case with less than 6 qc's.
//   7 - Fully insured totalization case.
//
// Arguments:
//   workerData: Worker's basic data.
//   piaData: Benefit calculation data.
//   ioasdi: Type of benefit.
char PiaCal::fins2Cal( const WorkerDataGeneral& workerData,
const PiaData& piaData, WorkerDataGeneral::ben_type ioasdi )
{
   if (ioasdi == WorkerData::SURVIVOR) {
      if (piaData.finsCode.isFullyInsured())
         return('1');
      else
         return(piaData.finsCode.isCurrentlyInsured() ? '2' : '5');
   }
   else {
      if (workerData.getTotalize()) {
         if (piaData.getQcTotal() < 6)
            return('6');
         return(piaData.finsCode.isFullyInsured() ? '7' : '3');
      }
      else
         return(piaData.finsCode.isFullyInsured() ? '1' : '4');
   }
}

// Description: Checks for deemed insured status, assuming eligible for deemed
//   insured status provision to apply.
//
// Returns: True if deemed insured, false otherwise.
//
// Arguments:
//   piaData: Worker's calculated data.
//   qtrYear: Quarter and year for which insured status code is required.
bool PiaCal::deemedInsCal( PiaData& piaData, const QtrYear& qtrYear )
{
   // determine number of QC's required
   piaData.setDeemedQcReq(PiaParams::deemedQcReqCal(
      piaData.getBirthDateMinus1().year()));
   if (piaData.getDeemedQcReq() < 0)
      return(false);
#if !defined(NDEBUG)
   qtrYear.check();
#endif
   int tempQctot = piaData.qcov.accumulate(PiaParams::qtr184, qtrYear, 0u);
   return (tempQctot >= piaData.getDeemedQcReq());
}

// Description: Calculates the disability insured status code for one year.
//
// Remarks: Note that this is a static function.
//
// Returns: The disability insured status code.
//
// Arguments:
//   workerData: Worker's basic data.
//   piaData: Benefit calculation data.
//   dateMoyr: The date for which insured status code is required.
//   iswasPrimary: Worker's primary beneficiary status.
//     0:  never a primary beneficiary.
//     1:  is a primary disability beneficiary.
//     2:  was a primary beneficiary.
DisInsCode::DisInsCodeType PiaCal::disInsCal(
const WorkerDataGeneral& workerData, PiaData& piaData,
const DateMoyr& dateMoyr, int iswasPrimary )
{
   // handle the cases that do not require any computation first
   if (workerData.isdead(dateMoyr)) {
      // deaths
      return (iswasPrimary == 2) ?
         DisInsCode::PRIMARYDEATH : DisInsCode::NONPRIMARYDEATH;
   }
   // check for no quarters of coverage
   if (piaData.getQcTotal() == 0) {
      return DisInsCode::NOQCS;
   }
   int i;  // loop counter
   // set quarter and year after attainment of age 21
   QtrYear tempQtrYear(piaData.getBirthDateMinus1());
   tempQtrYear.add(QtrYear(1,21));
   // look at period after cessation
   int trial = 0;
   const DisabPeriod& disabPeriod = workerData.disabPeriod[0];
   piaData.qcDisReq2Cal(workerData, dateMoyr);
   if (piaData.getQcDisQtr2() < 40 && workerData.getValdi() > 0 &&
      disabPeriod.getWaitperDate().getYear() > 0) {
      trial = 4 * (disabPeriod.getWaitperDate().getYear() -
         disabPeriod.getOnsetDate().year()) +
         (disabPeriod.getWaitperDate().getMonth() + 2) / 3 -
         (disabPeriod.getOnsetDate().month() + 2) / 3;
   }
   // try ending quarters from onset to waiting period, in reverse order so
   // end up at earliest age (to see if special test is required)
   for (i = trial; i >= 0; i--) {
      piaData.qcDisReqCal(workerData, dateMoyr, i, false, false);
      piaData.qcDisCal(workerData);
      if (piaData.getQcTotalDis() >= piaData.getQcDisReq())
         break;
   }
   // check special insured status, if necessary and beginning quarter is
   // before age 21
   if (piaData.getQcTotalDis() < piaData.getQcDisReq() &&
      ((piaData.getQcDisDate5().getYear() > 0 &&
      piaData.getQcDisDate5() < tempQtrYear) ||
      (piaData.getQcDisDate5().getYear() == 0 &&
      piaData.getQcDisDate1() < tempQtrYear))) {
      for (i = 0; i <= trial; i++) {
         piaData.qcDisReqCal(workerData, dateMoyr, i, true, false);
         piaData.qcDisCal(workerData);
         if (piaData.getQcTotalDis() >= piaData.getQcDisReq())
            break;
      }
   }
   // if still not insured, try using freeze period in previous disability
   if (piaData.getQcTotalDis() < piaData.getQcDisReq() &&
      piaData.getQcDisDate5().getYear() > 0) {
      for (i = trial; i >= 0; i--) {
         piaData.qcDisReqCal(workerData, dateMoyr, i, false, true);
         piaData.qcDisCal(workerData);
         if (piaData.getQcTotalDis() >= piaData.getQcDisReq())
            break;
      }
   }
   // check special insured status, if necessary and beginning quarter is
   // before age 21, using freeze period in previous disability
   if (piaData.getQcTotalDis() < piaData.getQcDisReq() &&
      piaData.getQcDisDate5().getYear() > 0 &&
      piaData.getQcDisDate5() < tempQtrYear) {
      for (i = 0; i <= trial; i++) {
         piaData.qcDisReqCal(workerData, dateMoyr, i, true, true);
         piaData.qcDisCal(workerData);
         if (piaData.getQcTotalDis() >= piaData.getQcDisReq())
            break;
      }
   }
   // not fully insured
   if (piaData.getQcTotal() < piaData.getQcReq()) {
      if (iswasPrimary == 1 && workerData.getTotalize())
         return(DisInsCode::TOTALIZED);  // totalization benefits
      if (piaData.getQcTotalDis() < piaData.getQcDisReq())
         return(DisInsCode::NOTFULLNOT2040);  // uninsured in all ways
      else
         return(DisInsCode::NOTFULLHAS2040);  // not fully insured, meets 20/40
   }
   else {
      // fully insured
      if (piaData.getQcTotalDis() < piaData.getQcDisReq()) {
         if (iswasPrimary == 1 && workerData.getTotalize())
            return(DisInsCode::TOTALIZED);  // totalization benefits
         if (workerData.getBlindind())
            return(DisInsCode::FULLANDBLIND);  // fully insured (blindness)
         else {
            // fully insured, not 20/40 insured
            return(DisInsCode::FULLNOT2040);
         }
      }
      else {
         // primary beneficiary
         if (iswasPrimary == 1)
            return(DisInsCode::FULLANDDIB);  // currently a DIB
         else {
            if (piaData.getQcDisQtr() == 40) {
               // 20/40 disability insured status
               return(DisInsCode::FULLAND2040);
            }
            else {
               // special disability insured status
               return(DisInsCode::FULLANDSPECIAL);
            }
         }
      }
   }
}

// Description: Calculates number of years to drop out of elapsed period due to
//   disability.  Note that this is a static function.
//
// Returns: Number of years to drop out of elapsed period due to disability.
//
// Arguments:
//   workerData: Worker's basic data.
//   elapsed1: First year of elapsed period.
//   elapsed2: Last year of elapsed period.
//   frzyrst: Freeze years information.
int PiaCal::didropCal( const WorkerDataGeneral& workerData, int elapsed1,
int elapsed2, FreezeYears& frzyrst )
{
   int rv = 0;  // return value

   frzyrst.deleteContents();
   if (workerData.getValdi() == 0)
      return(rv);
   // check latest period of disability
   const int onsetYear = workerData.disabPeriod[0].getOnsetDate().year();
   const int cessYear = workerData.disabPeriod[0].getCessationDate().getYear();
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": In PiaCal::didropCal, elapsed2 = ";
      s += poutNoComma(elapsed2);
      s += ", onsetYear = ";
      s += poutNoComma(onsetYear);
      Trace::writeLine(s);
   }
#endif
   if (onsetYear <= elapsed2) {
#if defined(DEBUGCASE)
      if (isDebugPid(workerData.getIdNumber())) {
         string s("");
         s += workerData.getIdString();
         s += ": In PiaCal::didropCal, setting year1 to max of ";
         s += poutNoComma(onsetYear);
         s += " and ";
         s += poutNoComma(elapsed1 + 1);
         Trace::writeLine(s);
      }
#endif
      frzyrst.setYear1(max(onsetYear, elapsed1 + 1));
      // set last year of di dropout period
      frzyrst.setYear2((cessYear >= onsetYear && cessYear <= elapsed2) ?
         cessYear : elapsed2);
      rv = frzyrst.getYear2() - frzyrst.getYear1() + 1;
#if defined(DEBUGCASE)
      if (isDebugPid(workerData.getIdNumber())) {
         string s("");
         s += workerData.getIdString();
         s += ": In PiaCal::didropCal, after most recent disability, rv = ";
         s += poutNoComma(rv);
         Trace::writeLine(s);
      }
#endif
   }
   // check prior period of disability
   if (workerData.getValdi() == 2) {
      const int onset1Year = workerData.disabPeriod[1].getOnsetDate().year();
      const int cess1Year =
         workerData.disabPeriod[1].getCessationDate().getYear();
      if (onset1Year <= elapsed2 && (frzyrst.getYear1() == 0 ||
         onset1Year < frzyrst.getYear1())) {
         // set first year of prior di dropout period
#if defined(DEBUGCASE)
         if (isDebugPid(workerData.getIdNumber())) {
            string s("");
            s += workerData.getIdString();
            s += ": In PiaCal::didropCal, setting year3 to max of ";
            s += poutNoComma(onset1Year);
            s += " and ";
            s += poutNoComma(elapsed1 + 1);
            Trace::writeLine(s);
         }
#endif
         frzyrst.setYear3(max(onset1Year, elapsed1 + 1));
         // set last year of prior di dropout period
#if defined(DEBUGCASE)
         if (isDebugPid(workerData.getIdNumber())) {
            string s("");
            s += workerData.getIdString();
            s += ": In PiaCal::didropCal, setting year4 to ";
            s += poutNoComma(cess1Year);
            Trace::writeLine(s);
         }
#endif
         frzyrst.setYear4((cess1Year >= onset1Year && cess1Year <= elapsed2) ?
            cess1Year : elapsed2);
         if (frzyrst.getYear1() > 0 &&
            frzyrst.getYear4() >= frzyrst.getYear1()) {
            frzyrst.setYear4(frzyrst.getYear1() - 1);
         }
         rv += frzyrst.getYear4() - frzyrst.getYear3() + 1;
      }
   }
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": In PiaCal::didropCal, year1 = ";
      s += poutNoComma(frzyrst.getYear1());
      s += ", year2 = ";
      s += poutNoComma(frzyrst.getYear2());
      s += ", returning ";
      s += poutNoComma(rv);
      Trace::writeLine(s);
   }
#endif
   return(rv);
}

// Description: Releases memory.
void PiaCal::releaseMemory()
{
   // clear general methods
   for (vector< PiaMethod * >::iterator iter1 = piaMethod.begin();
   iter1 != piaMethod.end(); iter1++) {
      delete *iter1;
   }
   piaMethod.clear();
   wageInd = static_cast<WageInd *>(0);
   oldStart = static_cast<OldStart *>(0);
   specMin = static_cast<SpecMin *>(0);
   piaTable = static_cast<PiaTable *>(0);
   highPiaMethod = static_cast<PiaMethod *>(0);
   frozMin = static_cast<FrozMin *>(0);
   childCareCalc = static_cast<ChildCareCalc *>(0);
   transGuar = static_cast<TransGuar *>(0);
   dibGuar = static_cast<DibGuar *>(0);
   // clear widow methods
   for (vector< ReindWid * >::iterator iter2 = widowMethod.begin();
   iter2 != widowMethod.end(); iter2++) {
      delete *iter2;
   }
   widowMethod.clear();
}
