// $Id: piacallc.cpp 1.98 2007/03/08 14:11:32EST 044579 Development  $
//
// Functions to calculate a PIA involving a change in law.

#include <algorithm>  // for sort
#include <functional> // for greater()
#include <utility>  // for rel_ops and pair
#include "PiaCalLC.h"
#include "LawChangeAGE65COMP.h"
#include "LawChangeCHILDCARECREDIT.h"
#include "OldStartLC.h"
#include "PiaTableLC.h"
#include "WageIndLC.h"
#include "ReindWidLC.h"
#include "TransGuarLC.h"
#include "SpecMinLC.h"
#include "FrozMin.h"
#include "DibGuarLC.h"
#include "ChildCareCalcLC.h"
#include "SecondaryArray.h"
#include "WageBase.h"
#include "LawChangeArray.h"
#include "oactcnst.h"
#include "DebugCase.h"
#include "Trace.h"
#include "comma.h"

using namespace std;
#if !defined(__SGI_STL_INTERNAL_RELOPS)
using namespace std::rel_ops;
#endif

// Description: Initializes a PiaCalLC instance.
//
// Arguments:
//   newWorkerData: Worker's basic data.
//   newPiaData: Worker's calculation data.
//   newPiaParams: Pia calculation parameters.
//   newLawChange: Law change parameters.
PiaCalLC::PiaCalLC( WorkerDataGeneral& newWorkerData, PiaData& newPiaData,
const PiaParams& newPiaParams, const LawChangeArray& newLawChange ) :
PiaCal(newWorkerData, newPiaData, newPiaParams), lawChange(newLawChange)
{ }

// Description: Destructor.
PiaCalLC::~PiaCalLC()
{
   releaseMemory();
}

// Description: Returns ending elapsed year for a beneficiary case.
//
// Remarks: Assumes eligibility year has already been calculated.
//
// Returns: Ending elapsed year for a beneficiary case.
//
// Arguments:
//   entDate: The date of entitlement.
int PiaCalLC::nelapsed2Cal( const DateMoyr& entDate )
{
   int eligYearTemp;

   LawChangeAGE65COMP *lawChangeAGE65COMP = lawChange.lawChangeAGE65COMP;
   // account for age-65 computation point if proposed law
   if (lawChangeAGE65COMP->isEffective(piaData.getEligDate().getYear(),
      workerData.getBenefitDate().getYear())) {
      // phase in the new computation point
      int i2 = (static_cast<int>(piaData.getEligDate().getYear()) -
         lawChangeAGE65COMP->getStartYear()) /
         lawChangeAGE65COMP->getStep() + 1;
      int i1 = min(i2, lawChangeAGE65COMP->getInd());
      DateMoyr eligDateTemp = eligYearCal1(workerData, piaData, ioasdi, i1);
      eligYearTemp = eligYearCal2(workerData, piaData, ioasdi,
         eligDateTemp.getYear());
   }
   else
      eligYearTemp = piaData.getEligYear();
   // account for 1960 ending year for 1960 or later method,
   // nondisability
   int elapsed2 = (entDate.getYear() > 1960 &&
      piaData.getEligYear() < 1961 &&
      workerData.getValdi() == 0) ? 1960 : eligYearTemp - 1;
   if (ioasdi == WorkerDataGeneral::SURVIVOR &&
      elapsed2 > static_cast<int>(workerData.getDeathDate().year()) - 1) {
      elapsed2 = static_cast<int>(workerData.getDeathDate().year()) - 1;
   }
   return(elapsed2);
}

// Description: Determine number of computation years and dropout years.
//
// Remarks: Assumes elapsed years have already been computed.
//
// Arguments:
//   compPeriod: Computation period to use (new or old).
//   entDate: Entitlement date.
void PiaCalLC::nCal( CompPeriod& compPeriod, const DateMoyr& entDate )
{
   nCal1(compPeriod, entDate);
   // Apply DI 1-for-5 rule if:
   // old-age case with prior disability entitlement in June 1980 or later
   // that lasted to age 61, or
   // disability case with entitlement in June 1980 or later
   const DisabPeriod& disabPeriod = workerData.disabPeriod[0];
   if ((ioasdi == WorkerDataGeneral::OLD_AGE && workerData.getValdi() > 0 &&
      disabPeriod.getEntDate() >= PiaParams::amend80 &&
      disabPeriod.getCessationDate() >=
      DateMoyr(workerData.getBirthDate().month(),
      workerData.getBirthDate().year() + 61))
      || (workerData.getEntDate() >= PiaParams::amend80 &&
      ioasdi == WorkerDataGeneral::DISABILITY)) {
      // 1-for-5 dropout rule for DI, if not survivor
      compPeriod.setNDrop(min(compPeriod.getNElapsed() / 5, 5));
   }
   LawChange *lawChangeDIDROP5 = lawChange.lawChange[LawChange::DIDROP5];
   if (lawChangeDIDROP5->getInd() > 0 &&
      static_cast< int >(entDate.getYear()) >=
      lawChangeDIDROP5->getStartYear() &&
      piaData.getEligYear() >= lawChangeDIDROP5->getStartYear() - 2) {
      compPeriod.setNDrop(5);
   }

   compPeriod.setN(compPeriod.getNElapsed() - compPeriod.getNDrop());
   if (compPeriod.getN() < 2) {  // nx is minimum of 2
      compPeriod.setN(2);
      compPeriod.setNDrop(compPeriod.getNElapsed() - compPeriod.getN());
   }
   if (ioasdi == WorkerDataGeneral::OLD_AGE &&
      workerData.getEntDate().getYear() < 1958 &&
      compPeriod.getBaseYear() == YEAR50) {
      // old-age modification prior to 1958
      compPeriod.setN(piaData.getEligYear() - 1953);
      if (compPeriod.getN() < 2) {
         compPeriod.setN(2);
         compPeriod.setNDrop(compPeriod.getNElapsed() - compPeriod.getN());
      }
   }
   return;
}

// Description: Projects earnings and fills in earnings to be used in
//   calculations.
void PiaCalLC::earnProjection() const
{
   earnProjectionStatic(workerData, piaData, piaParams, lawChange);
}

// Description: Adds childcare credits into regular OASDI earnings, using
//   passed arguments rather than member variables.
//
// Remarks:  Note that this function is static.
//
// Arguments:
//   workerData: Basic worker data.
//   piaData: Benefit calculation data.
//   lawChange: Law-change parameters.
void PiaCalLC::childCareEarningsCal( const WorkerDataGeneral& workerData,
PiaData& piaData, const LawChangeArray& lawChange )
{
   LawChangeCHILDCARECREDIT *lawChangeCHILDCARECREDIT =
      lawChange.lawChangeCHILDCARECREDIT;
   if (lawChangeCHILDCARECREDIT->isEffective(piaData.getEligYear(),
      workerData.getBenefitDate().getYear())) {
      int firstYear = workerData.getIbegin();
      int lastYear = workerData.getIend();
      for (int yr = firstYear; yr <= lastYear; yr++) {
         if (piaData.childCareOrder.getBit(yr)) {
#if defined(DEBUGCASE)
            if (isDebugPid(workerData.getIdNumber())) {
               string s("Id ");
               s += workerData.getIdString();
               s += ": In PiaCalLC::childCareEarningsCal: earnings in year ";
               s += poutNoComma(yr);
               s += " were ";
               s += poutNoComma(piaData.earnOasdi[yr], 1, 2);
               s += " and are being bumped up to ";
               s += poutNoComma(piaData.getChildCareMaxCredit(yr), 1, 2);
               Trace::writeLine(s);
            }
#endif
            piaData.earnOasdi[yr] = piaData.getChildCareMaxCredit(yr);
         }
      }
   }
}

// Description: Adjusts QCs for child care credits, using passed arguments
//   rather than member variables.
//
// Remarks:  Note that this function is static.
//
// Arguments:
//   workerData: Basic worker data.
//   piaData: Benefit calculation data.
//   piaParams: Benefit calculation parameters.
//   lawChange: Law-change parameters.
void PiaCalLC::childCareQcCal( const WorkerDataGeneral& workerData,
PiaData& piaData, const PiaParams& piaParams, const LawChangeArray& lawChange )
{
   LawChangeCHILDCARECREDIT *lawChangeCHILDCARECREDIT =
      lawChange.lawChangeCHILDCARECREDIT;
   if (lawChangeCHILDCARECREDIT->isEffective(piaData.getEligYear(),
      workerData.getBenefitDate().getYear())) {
      int firstYear = workerData.getIbegin();
      // For simplicity, we won't allow QC's to be credited for the
      // current year.
      int lastYear = workerData.getIend() - 1;
      for (int yr = firstYear; yr <= lastYear; yr++) {
         // The childCareOrder bits should have already been set
         // by a previous call to childCareCreditCal().
         if (piaData.childCareOrder.getBit(yr)) {
            piaData.qcov.set(yr, Qc::qcCal(
               piaData.getChildCareMaxCredit(yr), piaParams.qcamt[yr]));
         }
      }
   }
}

// Description: Sets best child care years to use, using passed arguments
//   rather than member variables.
//
// Remarks:  Note that this function is static.
//
// Arguments:
//   workerData: Basic worker data.
//   piaData: Benefit calculation data.
//   piaParams: Benefit calculation parameters.
//   lawChange: Law-change parameters.
void PiaCalLC::childCareCreditCal( const WorkerDataGeneral& workerData,
PiaData& piaData, const PiaParams& piaParams, const LawChangeArray& lawChange )
{
   LawChangeCHILDCARECREDIT *lawChangeCHILDCARECREDIT =
      lawChange.lawChangeCHILDCARECREDIT;
   if (lawChangeCHILDCARECREDIT->isEffective(piaData.getEligYear(),
      workerData.getBenefitDate().getYear())) {
      int i;  // loop counter
      // The maximum credit this person can get for a child care year.
      DoubleAnnual maxCredit(workerData.getIbegin(), workerData.getIend());
      // array of ratios (earnings/maxCredit)
      vector< pair< double, int > > earnRatio;
      int totalYears = 0;  // total number of child care years
      const int startYear =
         lawChangeCHILDCARECREDIT->getChildCareYearsBeforeEffYear() ?
         workerData.getIbegin() :
         max(workerData.getIbegin(), lawChangeCHILDCARECREDIT->getStartYear());
      for (i = startYear; i <= workerData.getIend(); i++) {
         if (workerData.childCareYears.getBit(i)) {
            maxCredit[i] = childCareMaxCreditCal(workerData, piaData.earnOasdi,
               piaParams.getFqArray(), *lawChangeCHILDCARECREDIT, i);
            if (piaData.earnOasdi[i] < maxCredit[i]) {
               earnRatio.push_back(
                  make_pair(piaData.earnOasdi[i] / maxCredit[i], i));
               totalYears++;
            }
         }
      }
      // If person could possibly get more child care years than
      // the maximum allowed, then sort for the best ones.
      if (totalYears > lawChangeCHILDCARECREDIT->getMaxYears()) {
         sort(earnRatio.begin(), earnRatio.end());
         // Cap total number of child care years.
         totalYears = lawChangeCHILDCARECREDIT->getMaxYears();
      }
      // Set total number of child care years awarded.
      piaData.setChildCareYearsTotal(totalYears);
      // Take lowest ratios, and indicate their years with
      // booleans in childCareOrder.
      for (i = 0; i < totalYears; i++) {
         piaData.childCareOrder.setBit(earnRatio[i].second, true);
         piaData.childCareMaxCredit.push_back(
           make_pair(maxCredit[earnRatio[i].second], earnRatio[i].second));
      }
   }
}

// Description: Calculates the maximum child care credit.
//
// Returns: The maximum child care credit.
//
// Remarks:  Note that this function is static.
//
// Arguments:
//   workerData: Basic worker data.
//   earnOasdi: OASDI covered earnings.
//   awi: Average wage indexing array.
//   lawChange: Law-change parameters for CHILDCARECREDIT.
//   year: The year being considered.
double PiaCalLC::childCareMaxCreditCal( const WorkerDataGeneral& workerData,
const DoubleAnnual& earnOasdi, const AverageWage& awi,
const LawChangeCHILDCARECREDIT& lawChange, int year )
{
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": At top of PiaCalLC::childCareMaxCreditCal";
   }
#endif
#if !defined(NDEBUG)
   if (workerData.getIbegin() < earnOasdi.getBaseYear()) {
      string s("In PiaCalLC::childCareMaxCreditCal, ");
      s += "ibegin can not be less than baseYear: ibegin ";
      s += poutNoComma(workerData.getIbegin());
      s += ", baseYear ";
      s += poutNoComma(earnOasdi.getBaseYear());
      throw PiaException(s);
   }
   if (year > earnOasdi.getLastYear()) {
      string s("In PiaCalLC::childCareMaxCreditCal, ");
      s += "year can not be more than lastYear: year ";
      s += poutNoComma(year);
      s += ", lastYear ";
      s += poutNoComma(earnOasdi.getLastYear());
      throw PiaException(s);
   }
#endif
   double maxCredit;
   if (lawChange.getInd() <= 2) {
      // For ind == 1 or 2, the maxCredit is a specified percentage of the AWI.
      maxCredit = awi[year] * lawChange.getFqRatio();
   }
   else {
      // For ind == 3 or 4, the maxCredit is a specified percentage of the
      // average of some of the highest years of (indexed) earnings for the
      // worker.
      // Array of indexed earnings.
      vector<double> earnInd;
      for (int i = workerData.getIbegin(); i < year; i++) {
         earnInd.push_back(earnOasdi[i] * awi[year] / awi[i]);
      }
      // Sort the earnInd from highest to lowest.
      sort(earnInd.begin(), earnInd.end(), greater<double>());
      // Number of years to drop off the beginning of the numHigh years.
      const int numToDrop = lawChange.getNumDropExtremeEarnYears();
      // Number of highest earning years to use.
      const int numHigh = lawChange.getNumTopEarnYears() - numToDrop;
      // Total earnings for the high years we're using.
      double totalEarn = 0.0;
      for (int i = 0 + numToDrop; i < min(numHigh, (int)earnInd.size()); i++) {
         totalEarn += earnInd[i];
      }
      // Average the earnings.
      const double avgEarn = totalEarn / (numHigh - numToDrop);
      maxCredit = avgEarn * lawChange.getAvgEarnPct();
   }
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": At bottom of PiaCalLC::childCareMaxCreditCal";
      s += ", earnings:";
      for (int i = workerData.getIbegin(); i < year; i++) {
        s += " " + poutNoComma(earnOasdi[i],1,2);
      }
      s += ", maxCredit ";
      s += poutNoComma(maxCredit,1,2);
      s += " for ";
      s += poutNoComma(year);
      Trace::writeLine(s);
   }
#endif
   return maxCredit;
}

// Description: Projects earnings and fills in earnings to be used in
//   calculations, using passed arguments rather than member variables.
//
// Remarks: Note that this function is static.
//
// Arguments:
//   workerData: Basic worker data.
//   piaData: Benefit calculation data.
//   piaParams: Benefit calculation parameters.
//   lawChange: Law-change parameters.
void PiaCalLC::earnProjectionStatic( const WorkerDataGeneral& workerData,
PiaData& piaData, const PiaParams& piaParams, const LawChangeArray& lawChange )
{
   piaData.earnProjection(workerData);
   // add in child care credits, if any
   childCareEarningsCal(workerData, piaData, lawChange);
   piaData.earnHiCal(workerData);
   // limit earnings to wage base
   piaParams.getBaseOasdiArray().earnLimit(piaData.earnOasdiLimited,
      piaData.earnOasdi);
   piaParams.getBaseHiArray().earnLimit(piaData.earnHiLimited,
      piaData.earnHi);
}

// Description: Calculates PIA and MFB, not including re-indexed widow(er).
//
// Arguments:
//   entDate: The date of entitlement.
void PiaCalLC::piaCal( const DateMoyr& entDate )
{
   try {
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      Trace::writeLine(workerData.getIdString() +
         ": At top of PiaCalLC::piaCal");
   }
#endif
   piaData.initialize();
   piaData.earn50Cal(workerData);
   setAmend90(entDate);
   // release any memory from a prior case
   releaseMemory();
   // get new memory
   int maxYear = piaParams.getMaxyear();
   PiaMethod* method;
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      ostringstream strm;
      strm << workerData.getIdString() << ", piaData.getQcTotal50() = "
        << piaData.getQcTotal50()
        << ", piaData.getBirthDateMinus1().getYear() = "
        << piaData.getBirthDateMinus1().year()
        << ": Calling OldStartLC::isApplicable";
      Trace::writeLine(strm.str());
   }
#endif
   if (OldStartLC::isApplicable(entDate, workerData.getBenefitDate(), piaData,
      lawChange)) {
      method = new OldStartLC(workerData, piaData, piaParams,
         maxYear, entDate, lawChange);
      // find old-start n
      nelapsedCal(piaData.compPeriodOld, entDate);
      nCal(piaData.compPeriodOld, entDate);
      piaMethod.push_back(method);
      oldStart = dynamic_cast<OldStart *>(method);
   }
   if (PiaTableLC::isApplicable(workerData, piaData, ioasdi, lawChange,
      entDate)) {
      method = new PiaTableLC(workerData, piaData, piaParams, maxYear,
         lawChange, entDate);
      piaMethod.push_back(method);
      piaTable = dynamic_cast<PiaTable *>(method);
   }
   if (WageIndLC::isApplicable(workerData, piaData, lawChange, entDate)) {
      method = new WageIndLC(workerData, piaData, piaParams, maxYear,
         lawChange, entDate);
      piaMethod.push_back(method);
      wageInd = dynamic_cast<WageInd *>(method);
   }
   if (TransGuarLC::isApplicable(workerData, piaData, ioasdi, lawChange)) {
      method = new TransGuarLC(workerData, piaData, piaParams, maxYear,
         lawChange);
      piaMethod.push_back(method);
      transGuar = dynamic_cast<TransGuar *>(method);
   }
   if (SpecMinLC::isApplicable(workerData, lawChange)) {
      method = new SpecMinLC(workerData, piaData, piaParams, maxYear,
         lawChange);
      piaMethod.push_back(method);
      specMin = dynamic_cast<SpecMin *>(method);
   }
   if (FrozMin::isApplicable(workerData, piaData)) {
      method = new FrozMin(workerData, piaData, piaParams, maxYear);
      piaMethod.push_back(method);
      frozMin = dynamic_cast<FrozMin *>(method);
   }
   if (ChildCareCalcLC::isApplicable(workerData, piaData, ioasdi,
      lawChange)) {
      method = new ChildCareCalcLC(workerData, piaData,
         piaParams, maxYear, lawChange);
      piaMethod.push_back(method);
      childCareCalc = dynamic_cast<ChildCareCalc *>(method);
   }
   if (DibGuarLC::isApplicable(entDate, workerData.getBenefitDate(), 
      workerData, piaData, ioasdi, lawChange)) {
      method = new DibGuar(workerData, piaData, piaParams, maxYear);
      piaMethod.push_back(method);
      dibGuar = dynamic_cast<DibGuar *>(method);
   }
   for (vector< PiaMethod * >::iterator iter = piaMethod.begin();
   iter != piaMethod.end(); iter++) {
      (*iter)->setApplicable(PiaMethod::APPLICABLE);
      (*iter)->setIoasdi(ioasdi);
      (*iter)->setPebsAssumptions(pebsAssumptions);
      (*iter)->calculate();
   }

   resetAmend90();
   piaCal1();
   piaCal2();
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      Trace::writeLine(workerData.getIdString() +
         ": Returning from PiaCalLC::piaCal");
   }
#endif
   } catch (exception& e) {
      throw PiaException(string("In PiaCalLC::piaCal: ") + e.what());
   }
}

// Description: Calculates PIA and benefit.
//
// Arguments:
//   entDate: The date of entitlement.
void PiaCalLC::calculate2( const DateMoyr& entDate )
{
   try {
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      Trace::writeLine(workerData.getIdString() +
         ": At top of PiaCalLC::calculate2");
   }
#endif
   PiaCal::calculate2(entDate);
   // calculate benefit
   piaCal(entDate);
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      Trace::writeLine(workerData.getIdString() +
         ": Returning from PiaCalLC::calculate2");
   }
#endif
   } catch (exception& e) {
      throw PiaException(string("In PiaCalLC::calculate2: ") + e.what());
   }
}

// Description: Calculates PIA for one re-indexed widow(er).
//
// Arguments:
//   widowData: Widow basic data.
//   widowPiaData: Calculation information for widow.
//   secondary: Benefits for widow.
void PiaCalLC::reindWidCal( const WorkerDataGeneral& widowData,
const PiaData& widowPiaData, Secondary& secondary )
{
   if (ReindWidLC::isApplicable(workerData, piaData,
      widowPiaData.getEligYear(), secondary, lawChange)) {
      ReindWid *reindWid = new ReindWidLC(workerData, piaData, piaParams,
         piaParams.getMaxyear(), widowPiaData, widowData, secondary,
         lawChange);
      widowMethod.push_back(reindWid);
      reindWid->setApplicable(PiaMethod::APPLICABLE);
      reindWid->setIoasdi(ioasdi);
      reindWid->setPebsAssumptions(pebsAssumptions);
      reindWid->calculate();
   }
}
