// $Id: piadata.cpp 1.133 2007/11/07 07:27:49EST 277133 Development  $
//
// Various functions to set up data to calculate and print out a PIA.

#include <utility>  // for rel_ops
#include "piadata.h"
#include "oactcnst.h"
#include "qc.h"
#include "PiaException.h"
#include "Resource.h"
#include "DebugCase.h"
#if defined(DEBUGCASE)
#include "DateFormatter.h"
#include "Trace.h"
#include "comma.h"
#endif

using namespace std;
#if !defined(__SGI_STL_INTERNAL_RELOPS)
using namespace std::rel_ops;
#endif

// Description: Starting point of annual qc's, if using sum of qc's
// through 1950.
const QtrYear PiaData::qtr151(0, 1951);
// Description: Starting point of annual qc's, if using sum of qc's
// through 1977.
const QtrYear PiaData::qtr178(0, 1978);
// Description: The earliest possible retirement date.
const DateMoyr PiaData::jan1937(1, 1937);

// Description: Initializes arrays.
PiaData::PiaData() : ageBen(), ageEnt(), benefit(),
childCareOrder(YEAR37, WorkerDataGeneral::getMaxyear()),
childCareMaxCredit(),
compPeriodNew(YEAR50), compPeriodOld(YEAR36), disInsCode(), earlyRetAge(),
earnMilServ(YEAR37, WorkerDataGeneral::getMaxyear()),
earnHi(YEAR37, WorkerDataGeneral::getMaxyear()),
earnHiLimited(YEAR37, WorkerDataGeneral::getMaxyear()),
earnOasdi(YEAR37, WorkerDataGeneral::getMaxyear()),
earnOasdiLimited(YEAR37, WorkerDataGeneral::getMaxyear()),
earnTotalized(YEAR37, WorkerDataGeneral::getMaxyear()),
earnTotalizedLimited(YEAR37, WorkerDataGeneral::getMaxyear()),
finsCode(), fullRetAge(),
freezeYears(), highMfb(), highPia(), partialFreezeYears(),
qcov(YEAR37, WorkerDataGeneral::getMaxyear()), qcovMilServ(YEAR37, 1956),
relEarnPosition(YEAR37, WorkerDataGeneral::getMaxyear()), roundedBenefit(),
supportPia(), amend90(false), arf(0.0), arfApp(NO_SPEC_MIN_DRC),
childCareYearsTotal(0), deemedQcReq(0), deemedQctot(0),
doEarlyQcs(true), earn3750ms(0.), earnYear(0), eligDate(), finsCode2(' '),
fullInsDate(), fullRetDate(), iappn(0), iapps(0), ibeginAll(0), ibeginTotal(0),
ielgyr(0), iendAll(0), iendTotal(0), kbirth(), monthsArdri(0), 
overMax(false), pifc(' '), qc3750ms(0), qc3750simp(0), qcCurrent(0),
qcDisDate1(), qcDisDate2(), qcDisDate3(), qcDisDate4(), qcDisDate5(),
qcDisDate6(), qcDisQtr(0), qcDisQtr2(0),qcDisReq(0), qcDisYears(0), qcReq(0),
qcReqPerm(0), qcTotal(0), qcTotal50(0), qcTotal51(0), qcTotalDis(0),
relEarnPositionAverage(0.0)
{ }

// Description: Destroys arrays.
PiaData::~PiaData( )
{ }

// Description: Initializes variables to default values.
void PiaData::deleteContents()
{
   pifc = ' ';
   iappn = 0;
   ageEnt.deleteContents();
   ageBen.deleteContents();
   fullRetAge.deleteContents();
   earlyRetAge.deleteContents();
   eligDate.deleteContents();
   fullInsDate.deleteContents();
   fullRetDate.deleteContents();
   compPeriodNew.deleteContents();
   qcov.deleteContents();
   qcovMilServ.deleteContents();
   childCareOrder.deleteContents();
   childCareMaxCredit.clear();
   childCareYearsTotal = 0;
   qc3750simp = qc3750ms = qcTotal = qcTotal50 = 0;
   earn3750ms = earnTotal50[0] = earnTotal50[1] = 0.;
}

// Description: Calculates sum of regular pre-1951 earnings.
//
// Throws: PiaException if ibeginAll (first year of all earnings) is out of
//   range (only in debug mode).
void PiaData::earnTotal50Cal0()
{
#if !defined(NDEBUG)
   WorkerDataGeneral::ibegin2Check(ibeginAll);
#endif
   if (ibeginAll < 1937 || ibeginAll > 1950) {
      earnTotal50[0] = 0.0;
   }
   else {
      earnTotal50[0] = min(earnOasdi.accumulate(ibeginAll, 1950u, 0.0),
         42000.0);
   }
}

// Description: Calculates sum of pre-1951 totalization earnings.
//
// Throws: PiaException if ibeginTotal (first year of totalization earnings) is
//   out of range (only in debug mode).
void PiaData::earnTotal50Cal1()
{
#if !defined(NDEBUG)
   WorkerDataGeneral::ibegin2Check(ibeginTotal);
#endif
   if (ibeginTotal < 1937 || ibeginTotal > 1950) {
      earnTotal50[1] = 0.0;
   }
   else {
      earnTotal50[1] = min(earnTotalized.accumulate(ibeginTotal, 1950u, 0.0),
         42000.0);
   }
}

// Description: Determines number of required quarters of coverage for
//   DI insured status.
//
// Throws: PiaException of type PIA_IDS_DISREQCAL1 if year in most recent
//   freeze period is out of range (only in debug mode). PiaException of type
//   PIA_IDS_DISREQCAL2 if year in next most recent freeze period is out of
//   range (only in debug mode).
//
// Arguments:
//   workerData: Basic worker data.
//   date: The date for which required quarters of coverage are desired.
//   trial: Number of quarters to go beyond onset quarter.
//   dospecial: Indicator for special insured status calculation.
//     false:  do not do special insured status calculation.
//     true:  do special insured status calculation.
//   dofreeze: Indicator for assuming period between onset and waiting period
//     in a prior period of disability is a freeze period.
//
//     false - do not assume it is a freeze period (excluded period starts with
//             waiting period).
//     true  - assume it is a freeze period (excluded period starts with
//             onset).
void PiaData::qcDisReqCal( const WorkerDataGeneral& workerData,
const DateMoyr& date, int trial, bool dospecial, bool dofreeze )
{
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": Entering PiaData::qcDisReqCal for date";
      s += DateFormatter::toString(date);
      Trace::writeLine(s);
      s = workerData.getIdString();
      s += ": qcDisDate1 = ";
      s += qcDisDate1.toString();
      s += ", qcDisDate2 = ";
      s += qcDisDate2.toString();
      s += ", qcDisDate3 = ";
      s += qcDisDate3.toString();
      s += ", qcDisDate4 = ";
      s += qcDisDate4.toString();
      s += ", qcDisDate5 = ";
      s += qcDisDate5.toString();
      s += ", qcDisDate6 = ";
      s += qcDisDate6.toString();
      Trace::writeLine(s);
   }
#endif
   QtrYear tempQtrYear;  // temporary (quarter,year) date

   if (workerData.getValdi() > 0) {
      qcDisDate2 = workerData.disabPeriod[0].getOnsetDate();
      qcDisDate2.add(trial);
   }
   else
      qcDisDate2 = date;
   // set quarter and year after attainment of age 21
   tempQtrYear = getBirthDateMinus1();
   tempQtrYear.add(QtrYear(1,21));
   // calculate number of elapsed quarters
   if (qcDisQtr2 == 40) {
      // only use period after disability
      qcDisQtr = 40;
      qcDisDate1.deleteContents();
      qcDisDate2.deleteContents();
   }
   else {
      // set beginning quarter and year, based on (40 - qcdiqtr2) quarters
      qcDisDate1 = qcDisDate2;
      qcDisDate1.subtract(39 - qcDisQtr2);
      // check for special insured status if period starts before age 21
      if (dospecial && qcDisDate1 < tempQtrYear) {
         qcDiSpec(qcDisQtr2, 0);
      }
      if (qcDisDate1.getYear() == 0)
         qcDisQtr = qcDisQtr2;
      else {
         // check for period beginning before 1937
         if (static_cast<int>(qcDisDate1.getYear()) < YEAR37) {
            qcDisDate1.setQuarter(0);
            qcDisDate1.setYear(YEAR37);
         }
         qcDisQtr = QtrYear::diff(qcDisDate1, qcDisDate2) + 1 + qcDisQtr2;
      }
   }
   // consider period before prior disability
   if (workerData.getValdi() < 2 || qcDisDate1.getYear() == 0 ||
      !(qcDisDate1 < QtrYear(workerData.disabPeriod[1].getCessationDate()))) {
      // do not need period before prior disability
      qcDisDate5.deleteContents();
      qcDisDate6.deleteContents();
   }
   else {
      // reset beginning quarter after prior disability, starting with quarter
      // of cessation
      qcDisDate1 = workerData.disabPeriod[1].getCessationDate();
      // if cessation quarter is not a qc, go to next quarter
      int tempqcs = (qcDisDate1.getYear() < qcDisDate2.getYear()) ?
         4 - qcDisDate1.getQuarter() :
         qcDisDate2.getQuarter() - qcDisDate1.getQuarter();
#if !defined(NDEBUG)
      if (qcDisDate1.getYear() < YEAR37)
         throw PiaException(PIA_IDS_DISREQCAL1);
#endif
      if (static_cast<int>(qcov.get(qcDisDate1.getYear())) < tempqcs)
         qcDisDate1.add(1);
      // check that period is still consistent
      if (qcDisDate1 > qcDisDate2) {
         qcDisDate1.deleteContents();
         qcDisDate2.deleteContents();
      }
      // recalculate qcs elapsed after prior disability
      if (qcDisDate1.getYear() == 0)
         qcDisQtr = qcDisQtr2;
      else
         qcDisQtr = QtrYear::diff(qcDisDate1, qcDisDate2) + 1 + qcDisQtr2;
      // set ending quarter and year
      // if first quarter is a qc, use it
      qcDisDate6 = (dofreeze ?
         DateMoyr(workerData.disabPeriod[1].getOnsetDate()) :
         workerData.disabPeriod[1].getWaitperDate());
      // if first quarter is not a qc, stop in preceding quarter
#if !defined(NDEBUG)
      if (qcDisDate6.getYear() <= 0)
         throw PiaException(PIA_IDS_DISREQCAL2);
#endif
      if (qcov.get(qcDisDate6.getYear()) <= qcDisDate6.getQuarter())
         qcDisDate6.subtract(1);
      // set beginning quarter and year, based on (40 - qcdiqtr) quarters
      qcDisDate5 = qcDisDate6;
      qcDisDate5.subtract(39 - qcDisQtr);
      // check for special insured status if period starts before age 21
      if (dospecial && qcDisDate5 < tempQtrYear)
         qcDiSpec(qcDisQtr, 1);
      if (qcDisDate5.getYear() > 0) {
         // check for period beginning before 1937
         if (static_cast<int>(qcDisDate5.getYear()) < YEAR37) {
            qcDisDate5.setQuarter(0);
            qcDisDate5.setYear(YEAR37);
         }
         qcDisQtr += QtrYear::diff(qcDisDate5, qcDisDate6) + 1;
      }
   }
   // calculate number of elapsed years
   qcDisYears = (qcDisQtr + 2) / 4;
   // calculate number of qc's required
   qcDisReq = qcDisQtr / 2;
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": Exiting PiaData::qcDisReqCal for date";
      s += DateFormatter::toString(date);
      Trace::writeLine(s);
      s = workerData.getIdString();
      s += ": qcDisDate1 = ";
      s += qcDisDate1.toString();
      s += ", qcDisDate2 = ";
      s += qcDisDate2.toString();
      s += ", qcDisDate3 = ";
      s += qcDisDate3.toString();
      s += ", qcDisDate4 = ";
      s += qcDisDate4.toString();
      s += ", qcDisDate5 = ";
      s += qcDisDate5.toString();
      s += ", qcDisDate6 = ";
      s += qcDisDate6.toString();
      Trace::writeLine(s);
   }
#endif
}

// Description: Determines number of required quarters of coverage for DI
//   insured status, based on elapsed quarters after disability cessation.
//
// Arguments:
//   workerData: Basic worker data.
//   date: The date for which required quarters of coverage are desired.
void PiaData::qcDisReq2Cal( const WorkerDataGeneral& workerData,
const DateMoyr& date )
{
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": Entering PiaData::qcDisReq2Cal for date";
      s += DateFormatter::toString(date);
      Trace::writeLine(s);
      s = workerData.getIdString();
      s += ": qcDisDate1 = ";
      s += qcDisDate1.toString();
      s += ", qcDisDate2 = ";
      s += qcDisDate2.toString();
      s += ", qcDisDate3 = ";
      s += qcDisDate3.toString();
      s += ", qcDisDate4 = ";
      s += qcDisDate4.toString();
      s += ", qcDisDate5 = ";
      s += qcDisDate5.toString();
      s += ", qcDisDate6 = ";
      s += qcDisDate6.toString();
      Trace::writeLine(s);
   }
#endif
   qcDisDate3 = workerData.disabPeriod[0].getCessationDate();
   qcDisDate4 = date;
   if (workerData.getValdi() == 0 ||
      workerData.disabPeriod[0].getCessationDate().getYear() == 0 ||
      !(qcDisDate3 < qcDisDate4)) {
      qcDisQtr2 = 0;
      qcDisDate3.deleteContents();
      qcDisDate4.deleteContents();
   }
   else {
      qcDisDate3.add(1);
      // check for no more than 40 quarters
      qcDisQtr2 = QtrYear::diff(qcDisDate3, qcDisDate4) + 1;
      if (qcDisQtr2 > 40) {
         // set beginning quarter and year, based on 40 quarters
         qcDisDate3 = qcDisDate4;
         qcDisDate3.subtract(39);
         qcDisQtr2 = 40;
      }
   }
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": Exiting PiaData::qcDisReq2Cal for date";
      s += DateFormatter::toString(date);
      Trace::writeLine(s);
      s = workerData.getIdString();
      s += ": qcDisDate1 = ";
      s += qcDisDate1.toString();
      s += ", qcDisDate2 = ";
      s += qcDisDate2.toString();
      s += ", qcDisDate3 = ";
      s += qcDisDate3.toString();
      s += ", qcDisDate4 = ";
      s += qcDisDate4.toString();
      s += ", qcDisDate5 = ";
      s += qcDisDate5.toString();
      s += ", qcDisDate6 = ";
      s += qcDisDate6.toString();
      Trace::writeLine(s);
   }
#endif
}

// Description: Sets first quarter and year of disability insured period for
//   special insured status (called only if calculated first quarter and
//   year are in quarter of age 21 or earlier).
//
// Arguments:
//   qcdiqtrt: Number of elapsed quarters accounted for before considering
//     special insured status.
//   period: Period to consider (0 for prior to most recent disability, 1 for
//     prior to next most recent disability).
void PiaData::qcDiSpec( int qcdiqtrt, int period )
{
   QtrYear* tempQcDisDate1;
   QtrYear* tempQcDisDate2;
   if (period == 0) {
      tempQcDisDate1 = &qcDisDate1;
      tempQcDisDate2 = &qcDisDate2;
   }
   else {
      tempQcDisDate1 = &qcDisDate5;
      tempQcDisDate2 = &qcDisDate6;
   }
   // start with quarter after attainment of age 21
   *tempQcDisDate1 = getBirthDateMinus1();
   tempQcDisDate1->add(QtrYear(1,21));
   // set minimum number of qcs that must be in special period
   int tempqcs = 12 - qcdiqtrt;
   if (tempqcs <= 0) {
      tempQcDisDate1->deleteContents();
      tempQcDisDate2->deleteContents();
      return;
   }
   // check for fewer than tempqcs qc's
   if (QtrYear::diff(*tempQcDisDate1, *tempQcDisDate2) < tempqcs - 1) {
      // use tempqcs quarters
      *tempQcDisDate1 = *tempQcDisDate2;
      tempQcDisDate1->subtract(tempqcs - 1);
   }
}

// Description: Calculates earned quarters of coverage in disability insured
//   period.
//
// Throws: PiaException of type PIA_IDS_QCDISCAL if any starting or ending
//   dates are out of range (only in debug mode).
//
// Remarks: Assumes starting and ending dates have already been set, by
//   calling qcDisReqCal.
//
// Arguments:
//   workerData: Basic worker data.
void PiaData::qcDisCal( const WorkerDataGeneral& workerData )
{
   qcTotalDis = 0;
#ifndef NDEBUG
   if (qcDisDate1.getYear() < 0)
      throw PiaException(PIA_IDS_QCDISCAL);
#endif
   if (qcDisDate1.getYear() > 0) {
#ifndef NDEBUG
      if (qcDisDate2.getYear() <= 0)
         throw PiaException(PIA_IDS_QCDISCAL);
#endif
      qcTotalDis = qcov.accumulate(qcDisDate1, qcDisDate2, qcTotalDis);
#if defined(DEBUGCASE)
      if (isDebugPid(workerData.getIdNumber())) {
         string s("");
         s += workerData.getIdString();
         for (unsigned year = qcDisDate1.getYear();
         year <= qcDisDate2.getYear(); year++) {
            s += ", qcov[";
            s += poutNoComma(year);
            s += "] = ";
            s += poutNoComma(qcov.get(year));
         }
         s += ": Using qcov in PiaData::qcDisCal";
         Trace::writeLine(s);
         s = "Id ";
         s += workerData.getIdString();
         s += ", qcDisDate1 = ";
         s += qcDisDate1.toString();
         s += ", qcDisDate2 = ";
         s += qcDisDate2.toString();
         s += ": Set qcTotalDis to ";
         s += poutNoComma(qcTotalDis);
         Trace::writeLine(s);
      }
#endif
   }
#ifndef NDEBUG
   if (qcDisDate3.getYear() < 0)
      throw PiaException(PIA_IDS_QCDISCAL);
#endif
   if (qcDisDate3.getYear() > 0) {
#ifndef NDEBUG
      if (qcDisDate4.getYear() <= 0)
         throw PiaException(PIA_IDS_QCDISCAL);
#endif
      qcTotalDis = qcov.accumulate(qcDisDate3, qcDisDate4, qcTotalDis);
#if defined(DEBUGCASE)
      if (isDebugPid(workerData.getIdNumber())) {
         string s("");
         s += workerData.getIdString();
         for (unsigned year = qcDisDate3.getYear();
         year <= qcDisDate4.getYear(); year++) {
            s += ", qcov[";
            s += poutNoComma(year);
            s += "] = ";
            s += poutNoComma(qcov.get(year));
         }
         s += ": Using qcov in PiaData::qcDisCal";
         Trace::writeLine(s);
         s = "Id ";
         s += workerData.getIdString();
         s += ", qcDisDate3 = ";
         s += qcDisDate3.toString();
         s += ", qcDisDate4 = ";
         s += qcDisDate4.toString();
         s += ": Set qcTotalDis to ";
         s += poutNoComma(qcTotalDis);
         Trace::writeLine(s);
      }
#endif
   }
#ifndef NDEBUG
   if (qcDisDate5.getYear() < 0)
      throw PiaException(PIA_IDS_QCDISCAL);
#endif
   if (qcDisDate5.getYear() > 0) {
#ifndef NDEBUG
      if (qcDisDate6.getYear() <= 0)
         throw PiaException(PIA_IDS_QCDISCAL);
#endif
      qcTotalDis = qcov.accumulate(qcDisDate5, qcDisDate6, qcTotalDis);
#if defined(DEBUGCASE)
      if (isDebugPid(workerData.getIdNumber())) {
         string s("");
         s += workerData.getIdString();
         for (unsigned year = qcDisDate5.getYear();
         year <= qcDisDate6.getYear(); year++) {
            s += ", qcov[";
            s += poutNoComma(year);
            s += "] = ";
            s += poutNoComma(qcov.get(year));
         }
         s += ": Using qcov in PiaData::qcDisCal";
         Trace::writeLine(s);
         s = "Id ";
         s += workerData.getIdString();
         s += ", qcDisDate5 = ";
         s += qcDisDate5.toString();
         s += ", qcDisDate6 = ";
         s += qcDisDate6.toString();
         s += ": Set qcTotalDis to ";
         s += poutNoComma(qcTotalDis);
         Trace::writeLine(s);
      }
#endif
   }
}

// Description: Returns first year with a quarter of coverage; 1937 if any QC's
//   before 1951, 1936 if no QC's.
//
// Returns: First year with a quarter of coverage; 1937 if any QC's
//   before 1951, 1936 if no QC's.
//
// Arguments:
//   workerData: Basic worker data.
int PiaData::getFirstQcYear( const WorkerDataGeneral& workerData ) const
{
   if (getQcTotal50() > 0 || workerData.getQcovRR3750() > 0)
      return(YEAR37);
   for (int i = 1951; i <= WorkerDataGeneral::getMaxyear(); i++)
      if (workerData.qc.get(i) > 0)
         return(i);
   return(1936);
}

// Description: Returns last year with a quarter of coverage, but at least
// 1950.
//
// Returns: Last year with a quarter of coverage, but at least 1950.
//
// Arguments:
//   workerData: Basic worker data.
int PiaData::getLastQcYear( const WorkerDataGeneral& workerData ) const
{
   for (int i = WorkerDataGeneral::getMaxyear(); i > 1950; i--)
      if (workerData.qc.get(i) > 0)
         return(i);
   return(1950);
}

// Description: Calculates total quarters of coverage.  Assumes annual quarters
//   of coverage are already known, either from 1951 or from 1978.
//
// Arguments:
//   workerData: Basic worker data.
//   qtrYear: Calendar quarter and year through which qc's are calculated.
void PiaData::qcTotalCal( const WorkerDataGeneral& workerData,
const QtrYear& qtrYear )
{
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": At top of PiaData::qcTotalCal";
      Trace::writeLine(s);
   }
#endif
#if !defined(NDEBUG)
   qtrYear.check();
#endif
   setQc3750simp();
   setQcTotal50(workerData.getQctottd() - workerData.getQctot51td() +
      workerData.getQcovRR3750() + qc3750ms);
   int i1 = max(qcTotal50, qc3750simp);
   // use annual quarters of coverage, if necessary
   if (workerData.hasQcsByYear()) {
#if defined(DEBUGCASE)
      if (isDebugPid(workerData.getIdNumber())) {
         string s("");
         s += workerData.getIdString();
         s += ", starting qtrYear ";
         s += qtr151.toString();
         s += ", ending qtrYear ";
         s += qtrYear.toString();
         s += ": Calling qcov.accumulate";
         Trace::writeLine(s);
      }
#endif
      qcTotal51 = qcov.accumulate(qtr151, qtrYear, 0u);
   }
   // use lump sum to 1977, annual quarters thereafter
   else {
#if defined(DEBUGCASE)
      if (isDebugPid(workerData.getIdNumber())) {
         string s("");
         s += workerData.getIdString();
         s += ", starting year 1951";
         s += ", ending year 1956";
         s += ": Calling qcovMilServ.accumulate";
         Trace::writeLine(s);
      }
#endif
      qcTotal51 = qcovMilServ.accumulate(1951, 1956,
         workerData.getQctot51td());
      qcTotal51 += workerData.getQcovRR5177();
#if defined(DEBUGCASE)
      if (isDebugPid(workerData.getIdNumber())) {
         string s("");
         s += workerData.getIdString();
         s += ", starting qtrYear ";
         s += qtr178.toString();
         s += ", ending qtrYear ";
         s += qtrYear.toString();
         s += ": Calling qcov.accumulate";
         Trace::writeLine(s);
      }
#endif
      qcTotal51 = qcov.accumulate(qtr178, qtrYear, qcTotal51);
   }
   setQcTotal(i1 + qcTotal51);
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": Returning from PiaData::qcTotalCal";
      Trace::writeLine(s);
   }
#endif
}

// Description: Calculates first year of earnings after 1950.
//
// Arguments:
//   workerData: Basic worker data.
void PiaData::earn50Cal( const WorkerDataGeneral& workerData )
{
   int i1 = getIbeginAll();
   earn50[0] = max(i1, 1951);
   earn50[1] = max(workerData.getTotalize() ? ibeginTotal : i1, 1951);
}

// Description: Returns applicable noncovered pension.
//
// Returns: Applicable noncovered pension. As of January 1995, the rules for
//   determining the noncovered pension amount changed, so someone with a
//   noncovered pension before 1995 could have a different noncovered pension
//   as of January 1995. This function uses the benefit date to determine the
//   correct noncovered pension.
//
// Arguments:
//   workerData: Basic worker data.
float PiaData::getPubpenApp( const WorkerDataGeneral& workerData ) const
{
   if (workerData.getReservist() &&
      workerData.getBenefitDate().getYear() > 1994) {
      return(workerData.getPubpenReservist());
   }
   else {
      return(workerData.getPubpen());
   }
}

// Description: Initializes computed values to zero.
void PiaData::initialize()
{
   compPeriodOld.deleteContents();
   arfApp = NO_SPEC_MIN_DRC;
   childCareYearsTotal = 0;
   supportPia = 0.0;
   highMfb.set(0.0);
   highPia.set(0.0);
   iapps = iappn = -1;
}

// Description: Determines freeze period.
//
// Arguments:
//   workerData: Basic worker data.
//   ioasdi: Type of benefit.
//   fullRetAgeDI: Full retirement age for benefit calculation purposes.
void PiaData::freezeYearsCal( const WorkerDataGeneral& workerData,
WorkerDataGeneral::ben_type ioasdi, const Age& fullRetAgeDI )
{
   int year1;  // year before attainment of full retirement age
   int year2;  // year before assumed attainment of full retirement age

   freezeYears.deleteContents();
   // determine years of freeze for latest period of disability
   if (workerData.getValdi() > 0) {
      const DisabPeriod& disabPeriod = workerData.disabPeriod[0];
      freezeYears.setYear1((disabPeriod.getOnsetDate().month() ==
         static_cast< int >(boost::date_time::Jan) &&
         disabPeriod.getOnsetDate().day() == 1) ?
         (int)disabPeriod.getOnsetDate().year() :
         (int)disabPeriod.getOnsetDate().year() + 1);
      if (ioasdi == WorkerDataGeneral::DISABILITY) {
         year1 = kbirth.year() + fullRetAgeDI.getYears() - 1;
         if (fullRetAgeDI.getMonths() + kbirth.month() > 12) {
            year1++;
         }
         year2 = disabPeriod.getWaitperDate().getYear() +
            fullRetAgeDI.getYears() - 63;
         if (fullRetAgeDI.getMonths() +
            disabPeriod.getWaitperDate().getMonth() > 12) {
            year2++;
         }
         freezeYears.setYear2(
            min(static_cast<int>(workerData.getBenefitDate().getYear()) - 1,
            min(year2, year1)));
      }
      else {
         freezeYears.setYear2((disabPeriod.getCessationDate().getMonth() ==
         boost::date_time::Dec) ? disabPeriod.getCessationDate().getYear() :
         disabPeriod.getCessationDate().getYear() - 1);
      }
   }
   // determine years of freeze for prior period of disability
   if (workerData.getValdi() > 1) {
      const DisabPeriod& disabPeriod = workerData.disabPeriod[1];
      freezeYears.setYear3((disabPeriod.getOnsetDate().month() ==
         static_cast<int>(boost::date_time::Jan) &&
         disabPeriod.getOnsetDate().day() == 1) ?
         (int)disabPeriod.getOnsetDate().year() :
         (int)disabPeriod.getOnsetDate().year() + 1);
      freezeYears.setYear4((disabPeriod.getCessationDate().getMonth() ==
         static_cast<int>(boost::date_time::Dec)) ?
         (int)disabPeriod.getCessationDate().getYear() :
         (int)disabPeriod.getCessationDate().getYear() - 1);
   }
}

// Description: Determines last year of allowable earnings.
//
// Arguments:
//   workerData: Basic worker data.
//   ioasdi: Type of benefit.
void PiaData::earnYearCal( const WorkerDataGeneral& workerData,
WorkerDataGeneral::ben_type ioasdi )
{
   // in survivor case, use earnings through year of death; otherwise stop
   // in year before year of benefit
   earnYear = (ioasdi == WorkerDataGeneral::SURVIVOR) ?
      (int)workerData.getDeathDate().year() :
      (int)workerData.getBenefitDate().getYear() - 1;
}

// Description: Sets last year of attributed earnings in totalization case.
//
// Arguments:
//   workerData: Basic worker data.
void PiaData::setIendTotal( const WorkerDataGeneral& workerData )
{
   iendTotal = max(ielgyr, workerData.getIend());
   if (workerData.getIndrr())
      iendTotal = max(iendTotal, workerData.getLastYearRR());
}

// Description: Sets first year of attributed earnings in totalization case.
//
// Arguments:
//   workerData: Basic worker data.
void PiaData::setIbeginTotal( const WorkerDataGeneral& workerData )
{
   ibeginTotal = max(static_cast<int>(kbirth.year() + 22), YEAR37);
   if (workerData.getIndearn() && ibeginTotal > workerData.getIbegin())
      ibeginTotal = workerData.getIbegin();
}

// Description: Sets date of attainment of normal retirement age, but no
//   earlier than January 1937.
//
// Remarks:  Assumes that day before birth and full retirement age have
//   already been calculated.
void PiaData::setFullRetDate()
{
   fullRetDate = DateMoyr(getBirthDateMinus1()) + fullRetAge;
   if (fullRetDate < jan1937)
      fullRetDate = jan1937;
}

// Description: Sets some quarter of coverage amounts.
//
// Arguments:
//   workerData: Basic worker data.
void PiaData::setQc3( WorkerDataGeneral& workerData )
{
   if (WorkerDataGeneral::getQcLumpYear() > 1950) {
      qcTotal51 = workerData.getQctot51td();
   }
   else {
      setQcTotal50((getEarnTotal50(EARN_NO_TOTALIZATION) < 49.9) ?
         0 : static_cast<int>(getEarnTotal50(EARN_NO_TOTALIZATION)) / 500 + 1);
      workerData.setQctd(getQcTotal50());
      workerData.setQc51td(0);
   }
}

void PiaData::setQc3750simp()
{
   qc3750simp =
      min(static_cast<int>(getEarnTotal50(EARN_NO_TOTALIZATION) / 400.0), 56);
}

// Description: Updates military service earnings and quarters of coverage.
//
// Arguments:
//   workerData: Basic worker data.
void PiaData::updateMilServ( const WorkerDataGeneral& workerData )
{
   int i;  // loop counter

   qc3750ms = workerData.getQcovMS3750();
   for (i = 1951; i <= 1956; i++) 
      qcovMilServ.set(i, workerData.getQcovMS(i));
   earn3750ms = workerData.getEarnMS3750();
   earnMilServ.deleteContents();
   for (i = 1951; i <= workerData.getLastYearMS(); i++) 
      earnMilServ[i] = workerData.getEarnMS(i);
}

// Description: Calculates annual quarters of coverage.
//
// Throws: PiaException of type PIA_IDS_QCCAL if amount required for quarter
//   of coverage is not initialized (only in debug mode). PiaException if
//   years of earnings extend beyond years of amount required for quarter of
//   coverage (only in debug mode).
//
// Arguments:
//   workerData: Basic worker data.
//   qcamt: Amount required for quarter of coverage.
void PiaData::qcCal( WorkerDataGeneral& workerData, const Qcamt& qcamt )
{
   unsigned qctemp;  // temporary quarters of coverage
   int i1;  // loop counter

#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": At top of PiaData::qcCal";
      Trace::writeLine(s);
   }
#endif
   try {
   workerData.qc.deleteContents(WorkerDataGeneral::getQcLumpYear() + 1,
      workerData.qc.getLastYear());
   // at this point, qctot51 has qcs up to 1977, excluding military service
   // and railroad
   int i2 = max(WorkerDataGeneral::getQcLumpYear(), 1950);
   if (doEarlyQcs) {
      for (i1 = YEAR37; i1 <= i2; i1++) {
         qctemp = workerData.qc.get(i1);
         if (workerData.getIndrr()) {
            qctemp += workerData.getQcovRR(i1);
#if defined(DEBUGCASE)
            if (isDebugPid(workerData.getIdNumber())) {
               string s("");
               s += workerData.getIdString();
               s += ": Adding ";
               s += poutNoComma(workerData.getQcovRR(i1));
               s += " railroad qcs for year ";
               s += poutNoComma(i1);
               Trace::writeLine(s);
            }
#endif
         }
         if (workerData.getIndms() && i1 < 1957) {
            qctemp += qcovMilServ.get(i1);
#if defined(DEBUGCASE)
            if (isDebugPid(workerData.getIdNumber())) {
               string s("");
               s += workerData.getIdString();
               s += ": Adding ";
               s += poutNoComma(qcovMilServ.get(i1));
               s += " military service qcs for year ";
               s += poutNoComma(i1);
               Trace::writeLine(s);
            }
#endif
         }
         qcov.set(i1, min(4u, qctemp));
#if defined(DEBUGCASE)
         if (isDebugPid(workerData.getIdNumber())) {
            string s("");
            s += workerData.getIdString();
            s += ": Setting qcs for year ";
            s += poutNoComma(i1);
            s += " to ";
            s += poutNoComma(qctemp);
            Trace::writeLine(s);
         }
#endif
      }
   }
   for (i1 = i2 + 1; i1 <= iendAll; i1++) {
#if !defined(NDEBUG)
      if (i1 > qcamt.getLastYear())
         throw PiaException("Year is out of range");
      if (qcamt[i1] < 1.)
         throw PiaException(PIA_IDS_QCCAL);
#endif
      qctemp = Qc::qcCal(earnOasdi[i1], qcamt[i1]);
#if defined(DEBUGCASE)
      if (isDebugPid(workerData.getIdNumber())) {
         string s("");
         s += workerData.getIdString();
         s += ": Setting qcs for year ";
         s += poutNoComma(i1);
         s += " to ";
         s += poutNoComma(qctemp);
         Trace::writeLine(s);
      }
#endif
      workerData.qc.set(i1, qctemp);
      qcov.set(i1, qctemp);
   }
   } catch (PiaException& e) {
      string s(e.what());
      s += " in PiaData::qcCal";
      throw PiaException(s);
   }
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": Returning from PiaData::qcCal";
      Trace::writeLine(s);
   }
#endif
}

// Description: Projects earnings and fills in earnings to be used in
//   calculations.
//
// Arguments:
//   workerData: Basic worker data.
void PiaData::earnProjection( const WorkerDataGeneral& workerData )
{
   int i1,i2,i3;  // loop counters
   double dtemp;  // temporary amount

#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": At top of PiaData::earnProjection";
      Trace::writeLine(s);
   }
#endif
   earnOasdi.deleteContents();
   earnHi.deleteContents();
   earnTotalized.deleteContents();
   earnOasdiLimited.deleteContents();
   earnHiLimited.deleteContents();
   earnTotalizedLimited.deleteContents();
   // start with actual earnings
   ibeginAll = workerData.getIbegin();
   iendAll = workerData.getIend();
   if (workerData.getIndearn()) {
#if defined(DEBUGCASE)
      if (isDebugPid(workerData.getIdNumber())) {
         string s("");
         s += workerData.getIdString();
         s += ": Ready to set regular earnings";
         Trace::writeLine(s);
      }
#endif
      for (i1 = ibeginAll; i1 <= iendAll; i1++) {
#if defined(DEBUGCASE)
         if (isDebugPid(workerData.getIdNumber())) {
            string s("");
            s += workerData.getIdString();
            s += ", year ";
            s += poutNoComma(i1);
            s += ", earnings ";
            s += poutNoComma(workerData.getEarnOasdi(i1));
            s += ": Setting regular earnings";
            Trace::writeLine(s);
         }
#endif
         earnOasdi[i1] = workerData.getEarnOasdi(i1);
      }
   }
   // add in railroad earnings
   if (workerData.getIndrr()) {
#if defined(DEBUGCASE)
      if (isDebugPid(workerData.getIdNumber())) {
         string s("");
         s += workerData.getIdString();
         s += ": Handling railroad earnings";
         Trace::writeLine(s);
      }
#endif
      if (workerData.getLastYearRR() > 1950) {
         i2 = max(workerData.getFirstYearRR(), 1951);
         i3 = workerData.getLastYearRR();
         if (!workerData.getIndearn()) {
            ibeginAll = i2;
            iendAll = i3;
         }
         else {
            if (ibeginAll > i2)
               ibeginAll = i2;
            if (iendAll < i3)
               iendAll = i3;
         }       
         for (i1 = i2; i1 <= i3; i1++) {
            dtemp = earnOasdi[i1] + workerData.getEarnRR(i1);
            earnOasdi[i1] = dtemp;
         }
      }
      // add in pre-1951 earnings
      if (workerData.getEarnRR3750() > 0.) {
         if (ibeginAll < 1937 || ibeginAll > 1950)
            ibeginAll = 1950;
         if (iendAll < 1950)
            iendAll = 1950;
         dtemp = earnOasdi[1950] + workerData.getEarnRR3750();
         earnOasdi[1950] = dtemp;
      }
   }
   // add in military service credits
#if defined(DEBUGCASE)
   if (isDebugPid(workerData.getIdNumber())) {
      string s("");
      s += workerData.getIdString();
      s += ": Ready to call updateMilServ";
      Trace::writeLine(s);
   }
#endif
   updateMilServ(workerData);
   if (workerData.getIndms()) {
#if defined(DEBUGCASE)
      if (isDebugPid(workerData.getIdNumber())) {
         string s("");
         s += workerData.getIdString();
         s += ": Handling military service";
         Trace::writeLine(s);
      }
#endif
      if (getEarn3750ms() > 0.) {
         if (ibeginAll > 1950)
            ibeginAll = 1950;
         if (iendAll < 1950)
            iendAll = 1950;
         dtemp = earnOasdi[1950] + getEarn3750ms();
         earnOasdi[1950] = dtemp;
      }
      int lastYear = workerData.getLastYearMS();
      if (lastYear >= 1951) {
         int firstYear = workerData.getFirstYearMS();
         if (firstYear < ibeginAll || ibeginAll == 0)
            ibeginAll = firstYear;
         if (lastYear > iendAll)
            iendAll = lastYear;
         for (i1 = firstYear; i1 <= lastYear; i1++) {
            dtemp = earnOasdi[i1] + earnMilServ[i1];
            earnOasdi[i1] = dtemp;
         }
      }
   }
}

// Description: Calculates HI earnings by adding OASDI earnings and excess HI
//   earnings.
//
// Arguments:
//   workerData: Basic worker data.
void PiaData::earnHiCal( const WorkerDataGeneral& workerData )
{
   // initialize HI earnings
   earnHi.deleteContents();
   // add OASDI earnings plus excess HI earnings
   if (workerData.getIndearn())
      for (int i1 = workerData.getIbegin(); i1 <= workerData.getIend(); i1++) {
         earnHi[i1] = earnOasdi[i1] + workerData.getEarnHi(i1);
#if defined(DEBUGCASE)
         if (isDebugPid(workerData.getIdNumber())) {
            string s("");
            s += workerData.getIdString();
            s += ", year ";
            s += poutNoComma(i1);
            s += ", HI earnings ";
            s += poutNoComma(workerData.getEarnHi(i1));
            s += ": PiaData::earnHiCal";
            Trace::writeLine(s);
         }
#endif
      }
}

// Description: Returns maximum child care credit amount for a specified year.
//
// Returns: Maximum child care credit amount for a specified year.
//
// Arguments:
//   yearOfCredit: Year of the credit
double PiaData::getChildCareMaxCredit( int yearOfCredit ) const
{
   for (unsigned i = 0; i < childCareMaxCredit.size(); i++)
      if (yearOfCredit == childCareMaxCredit[i].second) {
         return childCareMaxCredit[i].first;
      }
   return 0.0;
}

// Description: Sets day before worker's birth.
//
// Arguments:
//   birthDate: Worker's date of birth.
void PiaData::setBirthDateMinus1( const boost::gregorian::date& birthDate )
{
   try {
      kbirth = birthDate - boost::gregorian::date_duration(1);
   } catch (exception& e) {
      throw PiaException(string("In PiaData::setBirthDateMinus1") + e.what());
   }
}
