// $Id: piaparms.cpp 1.102 2007/02/28 15:06:12EST 044579 Development  $
//
// Functions to handle PIA calculation parameters.

#include <utility>  // for rel_ops
#include "piaparms.h"
#include "wbgenrl.h"
#include "baseyear.h"
#include "bpmfbout.h"
#include "oactcnst.h"
#include "Resource.h"
#include "PiaException.h"
#include "WageBase.h"
#include "BenefitAmount.h"
#include "awbidat.h"
#include "BaseYearNonFile.h"

using namespace std;
#if !defined(__SGI_STL_INTERNAL_RELOPS)
using namespace std::rel_ops;
#endif

// date of 1950 amendments
const DateMoyr PiaParams::amend50(9, 1950);
// date of 1952 amendments
const DateMoyr PiaParams::amend52(9, 1952);
// date of 1954 amendments
const DateMoyr PiaParams::amend54(9, 1954);
// date of 1956 amendments
const DateMoyr PiaParams::amend56(11, 1956);
// first birth date for which age 62 is effective for females
const boost::gregorian::date PiaParams::amend561(1894, 10, 31);
// second birth date for which age 62 is effective for females
const boost::gregorian::date PiaParams::amend562(1891, 11, 30);
// date of 1958 amendments
const DateMoyr PiaParams::amend58(1, 1959);
// date of 1961 amendments
const DateMoyr PiaParams::amend61(8, 1961);
// first birth date for which age 62 is effective for males
const boost::gregorian::date PiaParams::amend611(1899, 7, 31);
// second birth date for which age 62 is effective for males
const boost::gregorian::date PiaParams::amend612(1896, 8, 31);
// date of 1965 amendments for benefit increase
const DateMoyr PiaParams::amend651(1, 1965);
// date of 1965 amendments for change to age 60
const DateMoyr PiaParams::amend652(9, 1965);
// date of 1965 amendments for old start
const DateMoyr PiaParams::amend653(1, 1966);
// date of 1967 amendments for old start
const DateMoyr PiaParams::amend671(1, 1968);
// date of 1967 amendments for benefit increase
const DateMoyr PiaParams::amend672(2, 1968);
// date of 1969 amendments
const DateMoyr PiaParams::amend69(1, 1970);
// date of 1970 amendments
const DateMoyr PiaParams::amend70(1, 1971);
// date of 1972 amendments for benefit increase
const DateMoyr PiaParams::amend721(9, 1972);
// date of 1972 amendments for special minimum
const DateMoyr PiaParams::amend722(1, 1973);
// date of 1974 amendments for partial increase
const DateMoyr PiaParams::amend741(3, 1974);
// date of 1974 amendments for full increase
const DateMoyr PiaParams::amend742(6, 1974);
// date of 1977 amendments for frozen pia table
const DateMoyr PiaParams::amend771(12, 1978);
// date of 1977 amendments for wage indexing
const DateMoyr PiaParams::amend772(1, 1979);
// date of 1980 amendments
const DateMoyr PiaParams::amend80(7, 1980);
// date of birth for 1981 amendments, for age 62 and 1 month
const boost::gregorian::date PiaParams::amend811(1919, 8, 31);
// date of 1981 amendments (month, day, and year)
const boost::gregorian::date PiaParams::amend812(1981, 9, 1);
// date of 1981 amendments (month and year)
const DateMoyr PiaParams::amend813(9, 1981);
// date of 1981 amendments, for dollar-down rounding
const DateMoyr PiaParams::amend82(6, 1982);
// date of 1983 amendments
const DateMoyr PiaParams::amend83(1, 1984);
// date of 1988 amendments
const DateMoyr PiaParams::amend88(1, 1989);
// date of 1990 amendments
const DateMoyr PiaParams::amend90(6, 1992);
// date of special 0.1 percent benefit increase
const DateMoyr PiaParams::amend01(8, 2001);
// amount per old start increment year
const double PiaParams::amtperincyr(1650.);
// age 16
const Age PiaParams::age16(16, 0);
// age 18
const Age PiaParams::age18(18, 0);
// age 19
const Age PiaParams::age19(19, 0);
// age 22
const Age PiaParams::age22(22, 0);
// age 50
const Age PiaParams::age50(50, 0);
// age 60
const Age PiaParams::age60(60, 0);
// age 62
const Age PiaParams::age62(62, 0);
// age 62 and 1 month
const Age PiaParams::age621(62, 1);
// age 65
const Age PiaParams::age65(65, 0);
// age 65 and 2 months
const Age PiaParams::age65_2(65, 2);
// age 65 and 4 months
const Age PiaParams::age65_4(65, 4);
// age 65 and 6 months
const Age PiaParams::age65_6(65, 6);
// age 65 and 8 months
const Age PiaParams::age65_8(65, 8);
// age 65 and 10 months
const Age PiaParams::age65_10(65, 10);
// age 66
const Age PiaParams::age66(66, 0);
// age 66 and 2 months
const Age PiaParams::age66_2(66, 2);
// age 66 and 4 months
const Age PiaParams::age66_4(66, 4);
// age 66 and 6 months
const Age PiaParams::age66_6(66, 6);
// age 66 and 8 months
const Age PiaParams::age66_8(66, 8);
// age 66 and 10 months
const Age PiaParams::age66_10(66, 10);
// age 67
const Age PiaParams::age67(67, 0);
// age 70
const Age PiaParams::age70(70, 0);
// age 72
const Age PiaParams::age72(72, 0);
// age 75
const Age PiaParams::age75(75, 0);
// age 199 (impossible age)
const Age PiaParams::age199(199, 0);
// temporary benefit increase, March-May 1974
const double PiaParams::beninc74(7.0);
// Description: Factor for calculating widow(er) benefits.
const double PiaParams::factor825 = 0.825;
// Description: Factor for calculating mfbs.
const double PiaParams::factor150 = 1.50;
// Description: Factor for calculating mfbs.
const double PiaParams::factor175 = 1.75;
// Bend points for old-start PIB formula.
const int PiaParams::bendOS[] = { 0, 50, 250 };
// Old-start PIB formula percentages.
const double PiaParams::percOS[] = { .40, .10 };
// Description: Percentage of old-law wage base required for year of
//   coverage for special minimum.
//
// Remarks:
//
// i=0 - before 1991.
// i=1 - after 1990.
const double PiaParams::specMinPerc[] = { 0.25, 0.15 };
// Description: First quarter for which a quarter of coverage could be earned.
const QtrYear PiaParams::year37qtr1(0, YEAR37);
// Description: Starting point of deemed insured.
const QtrYear PiaParams::qtr184(0, 1984);

// Description: Initializes arrays.
//
// Arguments:
//   newIstart: Year after last known benefit increase. If not specified,
//     YEAR in BaseYearNonFile is used.
//   newMaxyear: Maximum projected year.
PiaParams::PiaParams( int newIstart, int newMaxyear ) :
qcamt(newMaxyear), bpMfbOut(newMaxyear), bpPiaOut(newMaxyear),
percPiaOut(newMaxyear), baseHi(newMaxyear), monthBeninc(YEAR51, newMaxyear),
noChange(YEAR37, newMaxyear), percSpecMin(YEAR37, newMaxyear),
yocAmountSpecMin(YEAR37, newMaxyear), yocAmountWindfall(YEAR37, newMaxyear),
cpiinc(YEAR51, newMaxyear), catchup(newIstart),
fq(YEAR37, newMaxyear), fqinc(newMaxyear), istart(newIstart),
maxyear(newMaxyear), specMinPia(), specMinPia2001(), specMinMfb(),
specMinMfb2001(), recalcInd(YEAR37, newMaxyear), titleAw(), titleBi()
{ }

// Arguments:
//   newMaxyear: Maximum projected year.
PiaParams::PiaParams( int newMaxyear ) :
qcamt(newMaxyear), bpMfbOut(newMaxyear), bpPiaOut(newMaxyear),
percPiaOut(newMaxyear), baseHi(newMaxyear), monthBeninc(YEAR51, newMaxyear),
noChange(YEAR37, newMaxyear), percSpecMin(YEAR37, newMaxyear),
yocAmountSpecMin(YEAR37, newMaxyear), yocAmountWindfall(YEAR37, newMaxyear),
cpiinc(YEAR51, newMaxyear), catchup(BaseYearNonFile::YEAR),
fq(YEAR37, newMaxyear), fqinc(newMaxyear), istart(BaseYearNonFile::YEAR),
maxyear(newMaxyear), specMinPia(), specMinPia2001(), specMinMfb(),
specMinMfb2001(), recalcInd(YEAR37, newMaxyear), titleAw(), titleBi()
{ }

// Description: Destroys arrays.
PiaParams::~PiaParams( )
{ }

// Description: Initializes some data.
void PiaParams::initdata()
{
   int i1;  // loop counter

   for (i1 = YEAR37; i1 < 1991; i1++)
      percSpecMin[i1] = static_cast<float>(specMinPerc[0]);
   for (i1 = 1991; i1 <= maxyear; i1++)
      percSpecMin[i1] = static_cast<float>(specMinPerc[1]);
   monthBeninc[1952] = 9;
   monthBeninc[1954] = 9;
   monthBeninc[1956] = 11;
   monthBeninc[1959] = 1;
   monthBeninc[1961] = 8;
   monthBeninc[1965] = 1;
   monthBeninc[1968] = 2;
   monthBeninc[1970] = 1;
   monthBeninc[1971] = 1;
   monthBeninc[1972] = 9;
   for (i1 = 1974; i1 <= 1982; i1++)
      monthBeninc[i1] = 6;
   for (i1 = 1983; i1 <= maxyear; i1++)
      monthBeninc[i1] = 12;

   // These years need recalculations because of an increase in the
   // minimum PIA that could make a person's new PIA be greater than
   // it would have been had the person just gotten the regular COLA
   recalcInd.setBit(1961, true);  // minimum PIA: $40
   recalcInd.setBit(1965, true);  // minimum PIA: $44
   recalcInd.setBit(1968, true);  // minimum PIA: $55
   recalcInd.setBit(1970, true);  // minimum PIA: $64
   // These 3 years need recalculations because of changes to the way
   // special mins were calculated.
   recalcInd.setBit(1973, true);
   recalcInd.setBit(1974, true);
   recalcInd.setBit(YEAR79, true);
   // This year needs a recalc because of a special provision giving an
   // increase in benefits of 1/10 of 1 percent starting in August 2001.
   recalcInd.setBit(2001, true);
   // If law-change file is read in after initialization, then this function
   // must be called again.
   projectPerc();
}

// Description: Sets the first projected year, including for members.
//
// Arguments:
//   newIstart: New first projected year.
void PiaParams::setIstart( int newIstart )
{
   BaseYear::yearCheck(newIstart);
   istart = newIstart;
}

// Description: Sets data from historical average wages and benefit increase
//   file.
//
// Arguments:
//   awbiData: Structure with historical average wages and benefit increases.
void PiaParams::setData( AwbiData& awbiData )
{
   const int year = awbiData.getLastYear();
   cpiinc.assign(awbiData.cpiinc, YEAR37, year - 1);
   baseHi.assign(awbiData.baseHi, YEAR37, year);
   fq.assign(awbiData.fq, YEAR37, year - 2);
}

// Description: Applies catch-up benefit increases.
//
// Returns: Increased benefit amount.
//
// Throws: PiaException of type PIA_IDS_BICCHP if either year is out of range
//   (debug mode only).
//
// Arguments:
//   bcatch: Amount to be increased.
//   year: Year of catch-up benefit increase.
//   eligYear: Year of eligibility for catch-up benefit increases.
double PiaParams::benincCatchUp( double bcatch, int year, int eligYear ) const
{
#ifndef NDEBUG
   if (year < YEAR37 || eligYear < 1900) {
      throw PiaException(PIA_IDS_BICCHP);
   }
#endif
   if (catchup.exist(eligYear, year)) {
      double rv = bcatch * (catchup.get(eligYear, year) / 100. + 1.);
      return BenefitAmount::round(rv, year);
   }
   else {
      return bcatch;
   }
}

// Description: Sets indicators for no change allowed.
//
// Throws: PiaException of type PIA_IDS_SETNOCH if year is out of range (debug
//   mode only).
//
// Arguments:
//   firstYear: First year change is allowed.
void PiaParams::setNoChange( int firstYear )
{
#ifndef NDEBUG
   if (firstYear < YEAR37) {
      throw PiaException(PIA_IDS_SETNOCH);
   }
#endif
   noChange.assign(1, YEAR37, firstYear);
   noChange.assign(0, firstYear, maxyear);
}

// Description: Sets historical average wage increases.
void PiaParams::setHistFqinc()
{
   fqinc.project(fq, YEAR37 + 1, istart - 2);
}

// Description: Finds last AMW in old-law table.
//
// Returns: Last AMW in old-law table.
//
// Arguments:
//   year: Year for which last AMW is desired.
long PiaParams::lastAmw( int year ) const
{
   long baseTemp = static_cast<long>(getBaseOasdi(year));
   // check to see if wage base is exact multiple of $60
   // ($22,900 in 1979 and $25,900 in 1980 are not)
   return((baseTemp % 60 > 0) ? (baseTemp / 60 + 1) * 5 : baseTemp / 12);
}

// Description: Returns full retirement age based on present law.
//
// Returns: Full retirement age based on present law.
//
// Arguments:
//   eligYear: Year of eligibility (age 62 for oab or wife/husband, 60 for
//     widow(er)).
Age PiaParams::fullRetAgeCalPL( int eligYear )
{
   if (eligYear < 2000)  // age 65
      return age65;
   if (eligYear > 1999 && eligYear < 2005) {
      switch (eligYear - 1999)
      {
      case 1:
         // age 65 and 2 months
         return age65_2;
      case 2:
         // age 65 and 4 months
         return age65_4;
      case 3:
         // age 65 and 6 months
         return age65_6;
      case 4:
         // age 65 and 8 months
         return age65_8;
      case 5:
         // age 65 and 10 months
         return age65_10;
      default:
         // impossible
         throw PiaException("Impossible age in PiaParams::fullRetAgeCalPL");
      }
   }
   if (eligYear > 2004 && eligYear < 2017)  // age 66
      return age66;
   if (eligYear > 2016 && eligYear < 2022) {
      switch (eligYear - 2016)
      {
      case 1:
         // age 66 and 2 months
         return age66_2;
      case 2:
         // age 66 and 4 months
         return age66_4;
      case 3:
         // age 66 and 6 months
         return age66_6;
      case 4:
         // age 66 and 8 months
         return age66_8;
      case 5:
         // age 66 and 10 months
         return age66_10;
      default:
         // impossible
         throw PiaException("Impossible age in PiaParams::fullRetAgeCalPL");
      }
   }
   // age 67
   return age67;
}

// Description: Returns full retirement age for benefit calculations involving
//   disability freeze.
//
// Returns: Full retirement age for benefit calculations involving
//   disability freeze.  In this class, it is based on present law,
//   but a subclass could change it.
//
// Arguments:
//   eligYear: Year of eligibility (age 62 for oab or wife/husband, 60 for
//     widow(er)).
//   currentYear: Current year.
Age PiaParams::fullRetAgeCalDIPL( int eligYear, int currentYear )
{
   if (currentYear < 2000) {
      return age65;
   }
   else {
      return fullRetAgeCalPL(eligYear);
   }
}

// Description: Returns amount of monthly delayed retirement credit.
//
// Returns: Amount of monthly delayed retirement credit.
//
// Arguments:
//   eligYear: Year of eligibility.
double PiaParams::retCredit( int eligYear )
{
   if (eligYear < YEAR79)
      return(1.0 / 1200.0);
   if (eligYear >= YEAR79 && eligYear < 1987)
      return(1.0 / 400.0);
   if (eligYear > 1986 && eligYear < 2005) {
      return(static_cast<double>((eligYear - 1985) / 2) / 2400.0 +
         1.0 / 400.0);
   }
   return(2.0 / 300.0);
}

// Description: Returns earliest possible retirement age for old-age benie or
//   aged spouse, based on present law.
//
// Returns: Earliest possible retirement age for old-age benie or
//   aged spouse, based on present law.
//
// Arguments:
//   sex: Sex of number holder (MALE for male, FEMALE for female).
//   kbirth: The date of birth, adjusted to previous day.
Age PiaParams::earlyAgeOabCalPL( Sex::sex_type sex,
 const boost::gregorian::date& kbirth )
{
   if (sex == Sex::MALE) {
      // change to age 62
      if (amend611 < kbirth) {
         if (amend811 < kbirth && kbirth.day() != 1)
            return age621;
         else
            return age62;
      }
      else if (kbirth <= amend611 && amend612 < kbirth) {
         return DateMoyr::diff(kbirth, amend61);
      }
      else {
         return age65;
      }
   }
   else {
      // change to age 62
      if (amend561 < kbirth) {
         if (amend811 < kbirth && kbirth.day() != 1)
            return age621;
         else
            return age62;
      }
      else if (kbirth <= amend561 && amend562 < kbirth) {
         return DateMoyr::diff(kbirth, amend56);
      }
      else {
         return age65;
      }
   }
}

// Description: Returns earliest possible aged widow(er)'s retirement age.
//
// Returns: Earliest possible aged widow(er)'s retirement age.
//
// Arguments:
//   benefitDate: Benefit date.
Age PiaParams::earlyAgeWidCal( const DateMoyr& benefitDate )
{
   if (benefitDate < amend56)
      return age65;
   if (benefitDate < amend652)
      return age62;
   return age60;
}

// Description: Returns earliest possible disabled widow(er)'s retirement age.
//
// Returns: Earliest possible disabled widow(er)'s retirement age.
//
// Arguments:
//   benefitDate: Benefit date.
Age PiaParams::earlyAgeDisWidCal( const DateMoyr& benefitDate )
{
   // Not allowed until 1967
   if (benefitDate < amend672)
      return age199;
   return age50;
}

// Description: Returns maximum DI beneficiary age at end of year, based on
//   present law.
//
// Returns: Maximum DI beneficiary age at end of year, based on present law.
//
// Arguments:
//   year: Year of benefit.
Age PiaParams::maxDibAgePL( int year )
{
   if (year < 2003)  // age 65
      return age65;
   if (year > 2002 && year < 2008) {
      switch (year - 2002)
      {
      case 1:
         // age 65 and 2 months
         return age65_2;
      case 2:
         // age 65 and 4 months
         return age65_4;
      case 3:
         // age 65 and 6 months
         return age65_6;
      case 4:
         // age 65 and 8 months
         return age65_8;
      case 5:
         // age 65 and 10 months
         return age65_10;
      default:
         // impossible
         throw PiaException("Impossible age in PiaParams::maxDibAgePL");
      }
   }
   if (year > 2007 && year < 2021)  // age 66
      return age66;
   if (year > 2020 && year < 2026) {
      switch (year - 2020)
      {
      case 1:
         // age 66 and 2 months
         return age66_2;
      case 2:
         // age 66 and 4 months
         return age66_4;
      case 3:
         // age 66 and 6 months
         return age66_6;
      case 4:
         // age 66 and 8 months
         return age66_8;
      case 5:
         // age 66 and 10 months
         return age66_10;
      default:
         // impossible
         throw PiaException("Impossible age in PiaParams::maxDibAgePL");
      }
   }
   // age 67
   return age67;
}

// Description: Updates amounts required for special minimum year of coverage.
void PiaParams::updateYocAmountSpecMin()
{
   double factor;  // percentage of old-law base required for year of
      // coverage

   for (int i1 = YEAR51; i1 <= yocAmountSpecMin.getLastYear(); i1++) {
      factor = getPercSpecMin(i1);
      yocAmountSpecMin[i1] = factor * getBase77(i1);
   }
   factor = getSpecMinPerc(PRE1991);
   for (int i1 = YEAR51; i1 <= yocAmountWindfall.getLastYear(); i1++) {
      yocAmountWindfall[i1] = factor * getBase77(i1);
   }
}

// Description: Updates average wages, using specified amounts.
//
// Remarks: After filling in the average wages, it projects the percentage
// increases in average wages in fqinc. Then it calls projectFq to project
// the amounts dependent on average wages.
//
// Throws: PiaException of type PIA_IDS_UPDATEFQ if average wage is zero (only
//   in debug mode).
//
// Arguments:
//   userfq: Specified average wages.
//   firstYeart: First year of specified average wages.
//   lastYeart: Last year of specified average wages.
void PiaParams::updateFq( const DoubleAnnual& userfq, int firstYeart,
int lastYeart )
{
   fq.assign(userfq, firstYeart, lastYeart);
   for (int i = firstYeart; i <= lastYeart; i++) {
#if !defined(NDEBUG)
      if (fq[i - 1] < 0.001)
         throw PiaException(PIA_IDS_UPDATEFQ);
#endif
      fqinc[i] = (fq[i] / fq[i - 1] - 1.) * 100.;
   }
   projectFq();
}

// Description: Updates increases in average wages, using specified
// percentages.
//
// Remarks: After filling in the percentage increases in average wages, it
// projects the amount of average wages. Then it calls projectFq to project
// the amounts dependent on average wages.
//
// Arguments:
//   userFqinc: Specified average wage percentage increases.
//   userTitle: Specified title of assumptions.
//   firstYeart: First year of specified increases.
//   lastYeart: Last year of specified increases.
void PiaParams::updateFqinc( const DoubleAnnual& userFqinc,
const std::string& userTitle, int firstYeart, int lastYeart )
{
   // last year of specified assumptions
   const int lastyear = min(lastYeart, getMaxyear());
   if (lastyear < getMaxyear()) {
      fqinc.assign(0.0, lastyear + 1, getMaxyear());
   }
   fqinc.assign(userFqinc, firstYeart, lastyear);
   titleAw = userTitle;
   fq.project(fqinc, getIstart() - 1);
   projectFq();
}

// Description: Projects amounts dependent on the average annual wage.
void PiaParams::projectFq()
{
   qcamt.project(fq, 1978);
   bpMfbOut.setData(getFqArray());
   bpPiaOut.setData(getFqBppia());
}

// Description: Projects benefit formula percentages.
void PiaParams::projectPerc()
{
   for (int year = YEAR79; year <= maxyear; year++) {
      percPiaOut.setData(year, PercPia::PERC[0], PercPia::PERC[1],
         PercPia::PERC[2]);
   }
}

// Description: Resizes special minimum arrays.
void PiaParams::resizeSpecMin()
{
   int n = specMinMaxYears();
   specMinPia.resize(n, DoubleAnnual(YEAR37, maxyear));
   specMinPia2001.resize(n);
   specMinMfb.resize(n, DoubleAnnual(YEAR37, maxyear));
   specMinMfb2001.resize(n);
}

// Description: Projects special minimum amounts.
//
// Arguments:
//   baseYear: First year of special minimum amounts (1979 for present law).
//   lastYear: Last year of projection.
void PiaParams::projectSpecMin( int baseYear, int lastYear )
{
   int i1;  // loop counter

   double amountPerYear = specMinAmountCal(DateMoyr(1, baseYear));
   for (int i2 = 0; i2 < specMinMaxYears(); i2++) {
      double pia = (i2 + 1) * amountPerYear;
      double mfb = BenefitAmount::round(factor150 * pia, baseYear);
      if (baseYear == YEAR79) {
         specMinPia[i2][1978] = pia;
         specMinMfb[i2][1978] = mfb;
         // handle years up through 2000
         for (i1 = baseYear; i1 <= min(2000, lastYear); i1++) {
            pia = applyCola(pia, i1);
            specMinPia[i2][i1] = pia;
            mfb = applyColaMfb(mfb, i1, pia);
            specMinMfb[i2][i1] = mfb;
         }
         // handle 2001 (special August and regular December numbers)
         if (lastYear > 2000) {
            // recalculate values for December 1999, adding 0.1 percent
            double pia1999 = applyCola99(specMinPia[i2][1998]);
            double mfb1999 = applyColaMfb99(specMinMfb[i2][1998], pia1999);
            // recalculate values for December 2000, adding 0.1 percent
            // these become effective August 2001
            pia1999 = applyCola(pia1999, 2000);
            specMinPia2001[i2] = pia1999;
            mfb1999 = applyColaMfb(mfb1999, 2000, pia);
            specMinMfb2001[i2] = mfb1999;
            // calculate values for December 2001
            pia = applyCola(pia1999, 2001);
            specMinPia[i2][2001] = pia;
            mfb = applyColaMfb(mfb1999, 2001, pia);
            specMinMfb[i2][2001] = mfb;
         }
      }
      // handle years after 2001
      for (i1 = max(2002, baseYear); i1 <= lastYear; i1++) {
         pia = applyCola(pia, i1, YEAR79);
         specMinPia[i2][i1] = pia;
         mfb = applyColaMfb(mfb, i1, YEAR79, pia);
         specMinMfb[i2][i1] = mfb;
      }
   }
}

// Description: Updates benefit increases.
//
// Remarks: The version with 5 arguments updates with catch-up increases.
//   The version with 4 arguments updates benefit increases without catch-up
//   increases.
//
// Arguments:
//   userBiproj: User-entered benefit increases.
//   userCatchup: User-entered catch-up benefit increases.
//   userTitle: User-entered title of benefit increases.
//   firstYeart: First year of user-specified benefit increases.
//   lastYeart: Last year of user-specified benefit increases.
void PiaParams::updateCpiinc( const DoubleAnnual& userBiproj,
const Catchup& userCatchup, const std::string& userTitle, int firstYeart,
int lastYeart )
{
   int lastyear;  // last year of specified assumptions

   if (lastYeart < getMaxyear()) {
      lastyear = lastYeart;
      cpiinc.assign(0.0f, lastyear + 1, getMaxyear());
   }
   else
      lastyear = getMaxyear();
   cpiinc.assign(userBiproj, firstYeart, lastyear);
   catchup = userCatchup;
   titleBi = userTitle;
   projectCpiinc();
}

void PiaParams::updateCpiinc( const DoubleAnnual& userBiproj,
const std::string& userTitle, int firstYeart, int lastYeart )
{
   int lastyear;  // last year of specified assumptions

   if (lastYeart < getMaxyear()) {
      lastyear = lastYeart;
      cpiinc.assign(0.0, lastyear + 1, getMaxyear());
   }
   else
      lastyear = getMaxyear();
   cpiinc.assign(userBiproj, firstYeart, lastyear);
   catchup.deleteContents();
   titleBi = userTitle;
   projectCpiinc();
}

// Description: Returns benefit amount decreased by cola.
//
// Remarks: The version without an eligYear argument does not 
//   consider any catch-up benefit decreases.  The version with an
//   eligYear argument does consider catch-up benefit decreases.
//
// Returns: Benefit amount decreased by cola, without considering
//   any catch-up benefit decreases.  
//
// Arguments: 
//   pia: Amount to be decreased.
//   year: Year of decrease.
double PiaParams::unApplyCola( double pia, int year ) const
{
  return BenefitAmount::unApplyCola(pia, getCpiinc(year), year);
}

// Arguments:
//   pia: Amount to be decreased.
//   year: Year of decrease.
//   eligYear: Year of eligibility for catch-up benefit decreases.
double PiaParams::unApplyCola( double pia, int year, int eligYear ) const
{
  double rv = BenefitAmount::unApplyCola(pia, getCpiinc(year), year);
  return( unbiCatchUp(year, rv, eligYear));
}


// Description: Returns benefit amount decreased by special 1999 cola
//   (includes extra 0.1 percent decrease.)
//
// Returns: Benefit amount decreased by special 1999 cola (includes
//   extra 0.1 percent decrease.)
//
// Arguments:
//   pia: Amount to be decreased.
double PiaParams::unApplyCola99( double pia ) const
{
   return BenefitAmount::unApplyCola(pia, getCpiinc(1999) + 0.1, 1999);
}

// Description: Deconverts PIA back to year of eligibility.
//
// Returns: Deconverted PIA.
//
// Arguments:
//   eligYear: Year of eligibility.
//   number: Number of years for which benefit increases are applied.
//   pia77: PIA or MFB to be increased.
//   benDate: Date of benefit.
double PiaParams::deconvertPia( int eligYear, int number, double pia77,
           const DateMoyr& benDate) const
{
   double rv = pia77;
   // return if there are no CPI increases applied
   if (number <= 0)
      return rv;
   // do not handle year of eligibility prior to 1979
   if (eligYear < YEAR79)
      return rv;
   // check for special 1999 increase (with extra 0.1 percent)
   // must be eligible in 1999 or earlier, and benefit in August 2001 or later
   if (eligYear <= 1999 && !(benDate < PiaParams::amend01)) {
     // remove increases for 2000 and later
     for (int i1 = eligYear + number - 1; i1 >= 2000; i1--) {
        double temp = BenefitAmount::unApplyCola(rv, getCpiinc(i1), i1);
        rv = unbiCatchUp(i1, temp, eligYear);
     }
     // remove increase for 1999
     double temp2 = unApplyCola99( rv );
     rv = temp2;
     // remove any remaining increases
     for (int i2 = 1998; i2 >= eligYear; i2--) {
        double temp = BenefitAmount::unApplyCola(rv, getCpiinc(i2), i2);
        rv = unbiCatchUp(i2, temp, eligYear);
     }
     // divide out increases, starting with last one
//     for (int i1 = eligYear + number - 1; i1 >= eligYear; i1--) {
//        double temp = BenefitAmount::unApplyCola(rv, getCpiinc(i1), i1);
//        rv = unbiCatchUp(i1, temp, eligYear);
//     }
   }
   else {
     // divide out increases, starting with last one
     for (int i1 = eligYear + number - 1; i1 >= eligYear; i1--) {
        double temp = BenefitAmount::unApplyCola(rv, getCpiinc(i1), i1);
        rv = unbiCatchUp(i1, temp, eligYear);
     }
   }
   return(rv);
}

// Description: Divides out catch-up benefit increases.
//
// Throws: PiaException of type PIA_IDS_UNBICCHP if either year is out of
//   range (debug mode only).
//
// Returns: Amount with catch-up benefit increase removed.
//
// Arguments:
//   year: Year of catch-up benefit increase.
//   bcatch: Amount to be unincreased.
//   eligYear: Year of eligibility.
double PiaParams::unbiCatchUp( int year, double bcatch, int eligYear ) const
{
#ifndef NDEBUG
   if (year < YEAR37 || eligYear < YEAR37)
      throw PiaException(PIA_IDS_UNBICCHP);
#endif
   if (catchup.exist(eligYear, year)) {
      double rv = bcatch / (catchup.get(eligYear, year) / 100. + 1.);
      return BenefitAmount::unround(rv, year);
   }
   else
      return bcatch;
}

// Description: Calculates number of months of actuarial reduction for OAB.
//
// Returns: Number of months of actuarial reduction for OAB.
//
// Arguments:
//   age: The age for which reduction is being calculated.
//   fullRetAge: Full retirement age.
int PiaParams::monthsArCal( const Age& age, const Age& fullRetAge )
{
   return(fullRetAge.getYears() * 12 + fullRetAge.getMonths() -
      (age.getYears() * 12 + age.getMonths()));
}

// Description: Calculates reduction factor for OAB or DIB.
//
// Returns: Reduction factor for OAB or DIB.
//
// Remarks: The reduction is 5/9 of 1 percent for each of the first 36 months
// of reduction, and 5/12 of 1 percent for each additional month. Assumes a
// maximum of 60 months of reduction.
//
// Arguments:
//   monthsArdri: Number of months of reduction.
double PiaParams::factorArCalPL( int monthsArdri )
{
   return (monthsArdri <= 36) ?
      1.0 - (static_cast< double >(monthsArdri) * 5.0 / 900.0) :
      1.0 - (36.0 * 5.0 / 900.0) -
      ((static_cast< double >(monthsArdri) - 36.0) * 5.0 / 1200.0);
}

// Description: Calculates reduction factor for wife/husband.  Assumes a maximum
//   of 60 months of reduction.
//
// Returns: Reduction factor for wife/husband.
//
// Arguments:
//   monthsArdri: Number of months of reduction.
double PiaParams::factorArAgedSpouseCalPL( int monthsArdri )
{
   // do not allow any delayed retirement credit
   if (monthsArdri < 0)
      return 0.;
   else
      return (monthsArdri <= 36) ?
         1. - static_cast<double>(monthsArdri) * 25. / 3600. :
         1. - 0.25 - ((static_cast<double>(monthsArdri) - 36.) * 5. / 1200.);
}

// Description: Calculates number of months of delayed retirement credit for
//   OAB.
//
// Remarks: Delayed retirement credit can be earned for months beginning in
//   January 1971, but only if the worker has attained full retirement age and
//   is fully insured.  Credits stop at age 72 or 70.
//
// Returns: Number of months of delayed retirement credit for OAB.
//
// Arguments:
//   fullRetDate: The date of attainment of full retirement age.
//   eligYear: Year of eligibility.
//   dobadj: Adjusted date of birth.
//   entDate: The date of entitlement.
//   benefitDate: The date of benefit.
//   fullInsDate: The date fully insured status is attained.
int PiaParams::monthsDriCal( const DateMoyr& fullRetDate, int eligYear,
const boost::gregorian::date& dobadj, const DateMoyr& entDate,
const DateMoyr& benefitDate, const DateMoyr& fullInsDate )
{
   int i2;  // index of month after last month to which delayed
      // retirement credit can apply
   int i4;  // index of month of age 72 or 70
   DateMoyr tempDate;  // temporary date

   // set index of month of attainment of full retirement age, or month of
   // attainment of fully insured status, if later.
   int i3 = max(fullRetDate.index(), fullInsDate.index());
   // set index of first month to which delayed retirement credit can
   // apply (Jan 1971 = 0)
   int i1 = max(i3, 0);
   // decrease maximum age for DRI from 72 to 70 in Jan 1984
   if (eligYear <= 1975) {
      tempDate = DateMoyr(dobadj) + age72;
      i4 = tempDate.index();
      int i8 = amend83.index();
      if (eligYear >= 1974 && i4 > i8)
         i4 = i8;
   }
   else {
      tempDate = DateMoyr(dobadj) + age70;
      i4 = tempDate.index();
   }
   // set index of month of entitlement
   int i5 = entDate.index();
   // set index of January of year of entitlement, or month of
   // attainment of full retirement age, if later
   tempDate = DateMoyr(boost::date_time::Jan, entDate.getYear());
   int i6 = max(i3, tempDate.index());
   // set index of month of benefit
   int i7 = benefitDate.index();
   // if already age 70 (or 72), use DRCs through age 70 (or 72)
   if (i4 <= i5)
      i2 = i4;
   else {
      // if calculating benefit after age 70 (or 72), or in year after
      // entitlement, use DRCs through entitlement
      if (i4 <= i7 || benefitDate.getYear() > entDate.getYear())
         i2 = i5;
      // otherwise use DRCs through end of year before entitlement
      else
         i2 = i6;
   }
   // no DRC until January 1973
   if (entDate < amend722)
      i2 = 0;
   return(max(i2 - i1, 0));
}

// Description: Calculates delayed retirement increment factor for OAB.
//
// Returns: Delayed retirement increment factor for OAB.
//
// Arguments:
//   monthsArdri: Number of months of increment.
//   eligYear: Year of eligibility.
double PiaParams::factorDriCal( int monthsArdri, int eligYear )
{
   return(1. + static_cast<double>(monthsArdri) * retCredit(eligYear));
}

// Description: Calculates number of months of actuarial reduction for disabled
//   widow(er).
//
// Returns: Number of months of actuarial reduction for disabled widow(er).
//
// Remarks: Before 1984, includes all months from entitlement date to age 65.
// Starting in 1984, includes months only from age 60 to full retirement age.
//
// Arguments:
//   age: The age for which reduction is being calculated.
//   benefitDate: The date of benefit.
//   fullRetAge: Full retirement age of widow(er).
int PiaParams::monthsArDisWidCal( const Age& age, const DateMoyr& benefitDate,
const Age& fullRetAge )
{
   if (benefitDate < amend83)
      return((65 - age.getYears()) * 12 - age.getMonths());
   return (fullRetAge.getYears() - 60) * 12 + fullRetAge.getMonths();
}

// Description: Calculates reduction factor for disabled widow(er).
//
// Returns: Reduction factor for disabled widow(er).
//
// Arguments:
//   monthsArdri: Number of months of reduction. This includes months to full
//     retirement age. These are ignored starting in 1984, when all disabled
//     widow(er)s get 71.5 percent of PIA. But before 1984, we must use the
//     number of months prior to age 60 to calculate the correct reduction.
//     So we subtract 60 (the number of months from 60 to 65) from monthsArdri
//     before calculating the reduction factor.
//   entDate: The date of entitlement.
double PiaParams::factorArDisWidCal( int monthsArdri, const DateMoyr& entDate )
{
   if (entDate < amend722)
      return(.69167 - (static_cast<double>(monthsArdri - 60) * 43. / 19800.));
   if (entDate < amend83)
      return(.715 - (static_cast<double>(monthsArdri - 60) * 43. / 24000.));
   return(.715);
}

// Description: Calculates number of months of actuarial reduction for aged
//   widow(er).
//
// Returns: Mumber of months of actuarial reduction for aged widow(er).
//   Returns 0 if entitlement is at or after full retirement age.
//
// Arguments:
//   age: The age for which reduction is being calculated.
//   benefitDate: The date of benefit.
//   fullRetAge: Full retirement age of widow(er).
int PiaParams::monthsArWidCal( const Age& age, const DateMoyr& benefitDate,
const Age& fullRetAge )
{
   if (benefitDate < amend61) {
      return(0);
   }
   if (benefitDate < amend722) {
      if (age.getYears() >= 62)
         return(0);
      else {
         return((62 - age.getYears()) * 12 - age.getMonths());
      }
   }
   // handle 1973 and later
   if (!(age < fullRetAge)) {
      return(0);
   }
   else {
      return(
         fullRetAge.getYears() * 12 + fullRetAge.getMonths() -
         (age.getYears() * 12 + age.getMonths()));
   }
}

// Description: Calculates number of months of actuarial reduction for aged
//   wife/husband.
//
// Returns: Number of months of actuarial reduction for aged wife/husband.
//   Returns 0 if entitlement is at or after full retirement age.
//
// Arguments:
//   age: The age for which reduction is being calculated.
//   benefitDate: The date of benefit.
//   fullRetAge: Full retirement age of wife/husband.
int PiaParams::monthsArAgedSpouseCal( const Age& age,
const DateMoyr& benefitDate, const Age& fullRetAge )
{
   if (benefitDate < amend56)
      return(0);
   if (!(age < fullRetAge))
      return(0);
   else
      return((fullRetAge.getYears() * 12 + fullRetAge.getMonths()) -
         (age.getYears() * 12 + age.getMonths()));
}

// Description: Calculates reduction factor for aged widow(er).
//
// Returns: Reduction factor for aged widow(er).
//
// Arguments:
//   monthsArdri: Number of months of reduction.
//   age: The age for which reduction is being calculated.
//   benefitDate: The date of benefit.
//   fullRetAge: Full retirement age of widow(er).
double PiaParams::factorArWidCal( int monthsArdri, const Age& age,
const DateMoyr& benefitDate, const Age& fullRetAge ) const
{
   int i7;  // maximum possible number of months of reduction

   if (benefitDate < amend722) {
      return(1.);
   }
   // handle 1973 and later
   if (!(age < fullRetAge))
      return(1.);
   else {
      i7 = fullRetAge.getYears() * 12 + fullRetAge.getMonths() - 60 * 12;
      return (1. - (static_cast<double>(monthsArdri) * .285 /
         static_cast<double>(i7)));
   }
}

// Description: Returns factor for widow/widower benefit before age reduction.
//
// Returns: Factor for widow/widower benefit before age reduction.
//
// Arguments:
//   monthsArdri: Months of reduction.
//   age: The age of widow/widower.
//   benefitDate: The date of benefit.
double PiaParams::factorAgedWidCalPL( int monthsArdri, const Age& age,
const DateMoyr& benefitDate )
{
   if (benefitDate < amend61) {
      return(.75);
   }
   if (benefitDate < amend722) {
      if (age.getYears() >= 62)
         return(factor825);
      else {
         return((factor825 - (static_cast<double>(monthsArdri) * 5. / 900.)));
      }
   }
   // handle 1973 and later
   return(1.0);
}

// Description: Returns factor for disabled widow/widower benefit before age
// reduction.
//
// Returns: Factor for disabled widow/widower benefit before age reduction.
//
// Remarks: For benefits prior to 1973, returns 0.825. For 1973 and later,
// returns 1.0, even though all disabled widow/widower benefits are 71.5
// percent of PIA. We must do it this way because the unreduced benefit is
// used when applying the maximum family benefit. After the MFB, we reduce the
// benefit for age (0.715 factor).
//
// Arguments:
//   benefitDate: The date of benefit.
double PiaParams::factorDisWidCalPL( const DateMoyr& benefitDate )
{
   if (benefitDate < amend722) {
      return(factor825);
   }
   // handle 1973 and later
   return(1.0);
}

// Description: Calculates reduction factor for young survivor.
//
// Returns: .75 for present law.
double PiaParams::factorYngSurvCal() const
{
   return(.75);
}

// Description: Calculates reduction factor for young life benie.
//
// Returns: .50 for present law.
double PiaParams::factorYngLifeCal() const
{
   return(.50);
}

// Description: Calculates number of months of reduction for disabled worker.
//
// Returns: The number of months from oabEntDate to oabCessDate.
//
// Remarks: Disabled workers do not have any reduction unless they received an
// old-age benefit prior to their disability benefit.
//
// Arguments:
//   oabEntDate: The date of entitlement to an old-age benefit prior to the
//     disabled worker benefit. Should be the default DateMoyr (0 month and 0
//     year) if none.
//   oabCessDate: The date of cessation of an old-age benefit prior to the
//     disabled worker benefit. Should be the default DateMoyr (0 month and 0
//     year) if none.
int PiaParams::monthsArDICal( const DateMoyr& oabEntDate,
const DateMoyr& oabCessDate )
{
   const Age tempAge = DateMoyr::diff(oabEntDate, oabCessDate);
   return(tempAge.getMonths() + 12 * tempAge.getYears());
}

// Description: Returns oldest age of child for which parent can
//   get young wife/husband or mother/father benefits.
//
// Returns: Infinite age (199) if isDisabled is true. Otherwise, returns 18 if
// benefitDate is before the 1981 amendments, 16 if at that date or later.
//
// Arguments:
//   benefitDate: Benefit date.
//   isDisabled: True if child is disabled, false otherwise.
Age PiaParams::childAgeForMFCal( const DateMoyr& benefitDate, bool isDisabled )
{
   if (isDisabled)
      return age199;
   return (benefitDate < amend813) ? age18 : age16;
}

// Description: Returns amount of lump sum benefit.
//
// Returns: Simply $255.
double PiaParams::lumpSum()
{
   return(255.0);
}

// Description: Returns maximum child beneficiary age.
//
// Returns: Maximum child beneficiary age.
//
// Arguments:
//   benefitDate: The date of benefit.
//   isDisabled: True if child is disabled, false otherwise.
//   isStudent: True if student, false otherwise.
Age PiaParams::maxChildAge( const DateMoyr& benefitDate, bool isDisabled,
   bool isStudent )
{
   if (benefitDate < amend652)  // age 18
      return age18;
   if (!isDisabled) {
      if (isStudent) {
         if (benefitDate < amend813)
            return age22;
         else
            return age19;
      }
      else {
         return age18;
      }
   }
   else
      return age199;
}

// Description: Returns benefit amount increased by cola.
//
// Remarks: The version without an eligYear argument does not consider any
//   catch-up benefit increases. The version with an eligYear argument does
//   consider any catch-up benefit increases.
//
// Throws: PiaException of type PIA_IDS_APPCOLA1 if year is out of range (debug
//   mode only).
//
// Returns: Benefit amount increased by cola, without considering
//   any catch-up benefit increases.
//
// Arguments:
//   pia: Amount to be increased.
//   year: Year of increase.
double PiaParams::applyCola( double pia, int year ) const
{
#ifndef NDEBUG
   if (year < YEAR37) {
      throw PiaException(PIA_IDS_APPCOLA1);
   }
#endif
   return BenefitAmount::applyCola(pia, getCpiinc(year), year);
}

// Arguments:
//   pia: Amount to be increased.
//   year: Year of increase.
//   eligYear: Year of eligibility for catch-up benefit increases.
double PiaParams::applyCola( double pia, int year, int eligYear ) const
{
#ifndef NDEBUG
   if (year < YEAR37 || eligYear < 1900) {
      throw PiaException(PIA_IDS_APPCOLA2);
   }
#endif
   double rv = BenefitAmount::applyCola(pia, getCpiinc(year), year);
   return(benincCatchUp(rv, year, eligYear));
}

// Description: Returns benefit amount increased by special 1999 cola (includes
// extra 0.1 percent increase).
//
// Returns: Benefit amount increased by special 1999 cola (includes
//   extra 0.1 percent increase).
//
// Arguments:
//   pia: Amount to be increased.
double PiaParams::applyCola99( double pia ) const
{
   return BenefitAmount::applyCola(pia, getCpiinc(1999) + 0.1, 1999);
}

// Description: Returns maximum family benefit amount increased by cola, with
//   check that result is at least 150 percent of pia.
//
// Remarks: The version without an eligYear argument does not consider any
//   catch-up benefit increases. The version with an eligYear argument does
//   consider any catch-up benefit increases.
//
// Throws: PiaException of type PIA_IDS_APPCOLA3 if year is out of range (debug
//   mode only).
//
// Returns: Maximum family benefit amount increased by cola.
//
// Arguments:
//   mfb: Amount to be increased.
//   year: Year of increase.
//   pia: Pia to check against.
double PiaParams::applyColaMfb( double mfb, int year, double pia ) const
{
#ifndef NDEBUG
   if (year < YEAR37) {
      throw PiaException(PIA_IDS_APPCOLA3);
   }
#endif
   // set temporary return value
   double rv = BenefitAmount::applyCola(mfb, getCpiinc(year), year);
   // check to see that mfb is at least 150 percent of pia
   double mfbt = BenefitAmount::round(factor150 * pia, year);
   return max(rv, mfbt);
}

// Arguments:
//   mfb: Amount to be increased.
//   year: Year of increase.
//   eligYear: Year of eligibility for catch-up benefit increases.
//   pia: Pia to check against.
double PiaParams::applyColaMfb( double mfb, int year, int eligYear,
double pia ) const
{
#ifndef NDEBUG
   if (year < YEAR37 || eligYear < 1900) {
      throw PiaException(PIA_IDS_APPCOLA4);
   }
#endif
   double rv = applyCola(mfb, year, eligYear);
   // check to see that mfb is at least 150 percent of pia
   double mfbt = BenefitAmount::round(factor150 * pia, year);
   return max(mfbt, rv);
}

// Description: Returns maximum family benefit amount increased by special 1999
//   cola (includes extra 0.1 percent increase), with check that result is at
//   least 150 percent of pia.
//
// Returns: Maximum family benefit amount increased by cola.
//
// Arguments:
//   mfb: Amount to be increased.
//   pia: Pia to check against.
double PiaParams::applyColaMfb99( double mfb, double pia ) const
{
   // set temporary return value
   double rv = BenefitAmount::applyCola(mfb, getCpiinc(1999) + 0.1, 1999);
   // check to see that mfb is at least 150 percent of pia
   double mfbt = BenefitAmount::round(factor150 * pia, 1999);
   return max(rv, mfbt);
}

// Description: Returns amount per year of coverage in special minimum.
//
// Returns: Amount per year of coverage in special minimum.
//
// Arguments:
//   date1: The date of desired amount.
double PiaParams::specMinAmountCalPL( const DateMoyr& date1 )
{
   // set amount per year depending on date of entitlement
   if (date1 < amend722)
      return 0.00;
   // from Jan 1973 to Feb 1974, was $8.50 per year of coverage
   if (date1 < amend741)
      return(8.50);
   // from March 1974 to Dec 1978, was $9.00 per year of coverage
   if (date1 < amend772)
      return(9.00);
   // for January 1979 and later, use $11.50 per year of coverage
   return(11.50);
}

// Description: Returns special minimum pia as of a specified month.
//
// Returns: Special minimum pia.
//
// Arguments:
//   dateMoyr: Month and year for which pia is desired (January 1979 or later).
//   yoc: Years of coverage in excess of 10 (1-20).
double PiaParams::getSpecMinPia( const DateMoyr& dateMoyr, int yoc ) const
{
   unsigned year = dateMoyr.getYear();
   if (static_cast<int>(dateMoyr.getMonth()) < getMonthBeninc(year)) {
      // in 2001, apply extra 0.1 percent in August
      if (year == 2001 && dateMoyr.getMonth() >= 8) {
         return((yoc > 0) ? specMinPia2001[yoc-1] : 0.0);
      }
      else {
         year--;
      }
   }
   // if did not return above, use regular special minimum table
   return((yoc > 0) ? specMinPia[yoc-1][year] : 0.0);
}

// Description: Returns special minimum mfb as of a specified month.
//
// Returns: Special minimum mfb.
//
// Arguments:
//   dateMoyr: Month and year for which mfb is desired (January 1979 or later).
//   yoc: Years of coverage in excess of 10 (1-20).
double PiaParams::getSpecMinMfb( const DateMoyr& dateMoyr, int yoc ) const
{
   unsigned year = dateMoyr.getYear();
   if (static_cast<int>(dateMoyr.getMonth()) < getMonthBeninc(year)) {
      // in 2001, apply extra 0.1 percent in August
      if (year == 2001 && dateMoyr.getMonth() >= 8) {
         return((yoc > 0) ? specMinMfb2001[yoc-1] : 0.0);
      }
      else {
         year--;
      }
   }
   // if did not return above, use regular special minimum table
   return((yoc > 0) ? specMinMfb[yoc-1][year] : 0.0);
}

// Description: Returns number of quarters of coverage required for deemed
//   insured status.
//
// Returns: Number of quarters of coverage required for deemed
//   insured status.  Returns -1 if ineligible because of age (born
//   January 2, 1929 or later).
//
// Remarks: Note that this is a static function.
//
// Arguments:
//   year: Adjusted year of birth.
int PiaParams::deemedQcReqCal( int year )
{
   if (year < 1924)
      return(6);
   if (year < 1925)
      return(8);
   if (year < 1926)
      return(12);
   if (year < 1927)
      return(16);
   if (year < 1929)
      return(20);
   // if ineligible, because of age, use -1
   return(-1);
}

// Description: Returns widow(er) benefit reduced for RIB-LIM.
//
// Returns: Reduced widow(er) benefit (with rounding applied).
//
// Remarks: This should only be called in cases where the deceased worker was
// an OAB before death. Note that this is a static function.
//
// Before 1973, the widow benefit was 82.5 percent of PIA. Starting in 1973,
// there was an age-related factor applied to the PIA, resulting in benefits
// that could be more or less than 82.5 percent of PIA. As part of that change,
// widow benefits were limited to what the deceased worker was receiving as an
// OAB. If he was not receiving an OAB, there was no limit. He could have been
// receiving a benefit as low as 80 percent of PIA. But if he was receiving
// less than 82.5 percent of PIA, the widow can still get 82.5 percent of PIA,
// so as not to get less than she could have gotten before the law changed.
//
// Arguments:
//   widowBen: Widow(er) benefit reduced for widow(er)'s age, but not for
//     RIB-LIM.
//   oabPia: PIA upon which widow(er) benefit is based.  Note that this is not
//     a re-indexed widow(er) PIA, since the OAB must have survived to age 62.
//   oabBen: Benefit that deceased OAB was receiving, increased by colas up to
//     benefitDate (with rounding applied).
//   benefitDate: The date of benefit.
double PiaParams::ribLimCalPL( double widowBen, double oabPia, double oabBen,
const DateMoyr& benefitDate )
{
   // before 1973, no RIB-LIM (widow benefit was only 82.5 percent)
   if (benefitDate < amend722) {
      return widowBen;
   }
   // find 82.5 percent of pia
   const double pia825 =
     BenefitAmount::round(factor825 * oabPia, benefitDate.getYear());
   // limit widow benefit to greater of 82.5 percent of pia, or benefit OAB was
   // receiving
   const double maxWidowBen = max(pia825, oabBen);
   return min(widowBen, maxWidowBen);
}
