// $Id: piaparmsLC.cpp 1.68 2007/02/28 15:06:13EST 044579 Development  $
//
// Functions to handle changed law PIA parameters.

#include <cfloat>  // for FLT_MAX
#include "piaparmsLC.h"
#include "bppiaout.h"
#include "oactcnst.h"
#include "BenefitAmount.h"
#include "AwbiDataNonFile.h"
#include "LawChangeArray.h"
#include "LawChangeBPFRACWAGE.h"
#include "LawChangeNEWFORMULA.h"
#include "LawChangeDECLINEPERC.h"
#include "LawChangeBPSPECRATE.h"
#include "LawChangeBPMINCONST.h"
#include "LawChangeMARRLENGTH.h"
#include "LawChangeNEWSPECMIN.h"
#include "LawChangeNRACHANGE.h"
#include "LawChangeWIFEFACTOR.h"
#include "LawChangeWIDFACTOR.h"
#include "LawChangeCHILDCAREDROPOUT.h"
#include "LawChangeCHILDCARECREDIT.h"
#include "LawChangeWAGEBASECHG.h"
#include "LawChangeCOLACHANGE.h"

using namespace std;

// Description: Initializes arrays.
//
// Remarks: The version without newIstart uses the year in BaseYearNonFile, and
// constructs an AwbiDataNonFile using that year.
//
// Arguments:
//   newIstart: Year after last known benefit increase.
//   newMaxyear: Maximum projected year.
//   awbiData: Structure with historical amounts.
//   newLawChange: Law change parameters.
PiaParamsLC::PiaParamsLC( int newIstart, int newMaxyear, AwbiData& awbiData,
const LawChangeArray& newLawChange ) :
PiaParams(newIstart, newMaxyear),
baseOasdi(newLawChange, maxyear), base77(newLawChange, maxyear),
altCpiinc(YEAR51, newMaxyear), colaCaps(YEAR51, newMaxyear),
fqbppia(YEAR37, newMaxyear), fqindex(YEAR37, newMaxyear),
lawChange(newLawChange), mfbColaCaps(YEAR51, newMaxyear),
specMinPiaExtra(), specMinMfbExtra()
{
   setData(awbiData);
   initdata();
   setAltCpiinc();
   setFqIndex();
   setFqBppia();
}

PiaParamsLC::PiaParamsLC( int newMaxyear,
const LawChangeArray& newLawChange ) :
PiaParams(newMaxyear), baseOasdi(newLawChange, maxyear),
base77(newLawChange, maxyear), altCpiinc(YEAR51, newMaxyear),
colaCaps(YEAR51, newMaxyear), fqbppia(YEAR37, newMaxyear),
fqindex(YEAR37, newMaxyear), lawChange(newLawChange), mfbColaCaps(YEAR51,
newMaxyear), specMinPiaExtra(), specMinMfbExtra()
{
   AwbiDataNonFile awbiData(newMaxyear);
   setData(awbiData);
   initdata();
   setAltCpiinc();
   setFqIndex();
   setFqBppia();
}

// Description: Destroys arrays.
PiaParamsLC::~PiaParamsLC( )
{ }

// Description: Initializes some data.
void PiaParamsLC::initdata()
{
   PiaParams::initdata();
   // For various types of lawchanges that are being phased in immediately
   // (i.e. phaseType == 1), then need to do a recalculation for benefits
   // the first year it is effective.
   LawChangeCHILDCARECREDIT *lawChangeCHILDCARECREDIT =
      lawChange.lawChangeCHILDCARECREDIT;
   if (lawChangeCHILDCARECREDIT->getInd() > 0 &&
      lawChangeCHILDCARECREDIT->getPhaseType() == 1) {
      recalcInd.setBit(lawChangeCHILDCARECREDIT->getStartYear(), true);
   }
   LawChangeNEWSPECMIN *lawChangeNEWSPECMIN = lawChange.lawChangeNEWSPECMIN;
   if (lawChangeNEWSPECMIN->getInd() > 0 &&
      lawChangeNEWSPECMIN->getPhaseType() == 1) {
      recalcInd.setBit(lawChangeNEWSPECMIN->getStartYear(), true);
   }
}

// Description: Sets benefit increases when changed from present law.
void PiaParamsLC::setAltCpiinc()
{
   LawChangeCOLACHANGE *lawChangeCOLACHANGE = lawChange.lawChangeCOLACHANGE;
   if (lawChangeCOLACHANGE->getInd() == 1) {
      const int lastYear = min(altCpiinc.getLastYear(),
         lawChangeCOLACHANGE->getEndYear());
      const int firstYear = lawChangeCOLACHANGE->getStartYear();
      // use present-law colas before first specified year
      altCpiinc.assign(cpiinc, 1951, firstYear - 1);
      // use changed colas during specified period
      for (int i = firstYear; i <= lastYear; i++) {
         altCpiinc[i] = cpiinc[i] + lawChangeCOLACHANGE->getColaAdj();
      }
      // use present-law colas after last specified year
      altCpiinc.assign(cpiinc, lastYear + 1, altCpiinc.getLastYear());
   }
}

// Description: Sets series of average wages or other series used for indexing
//   pia formula bend points.
void PiaParamsLC::setFqBppia()
{
   // increase at fraction of wage rate
   LawChangeBPFRACWAGE* lawChangeBPFRACWAGE = lawChange.lawChangeBPFRACWAGE;
   if (lawChangeBPFRACWAGE->getInd() > 0) {
      const int lastYear = min(fqbppia.getLastYear(),
         lawChangeBPFRACWAGE->getEndYear() - 2);
      const int firstYear = lawChangeBPFRACWAGE->getStartYear() - 2;
      fqbppia.assign(getFqArray(), 1977, firstYear - 1);
      double temp = getFq(firstYear - 1);
      // project by percent of average wage increase
      for (int i1 = firstYear; i1 <= lastYear; i1++) {
         double factor = 1. + lawChangeBPFRACWAGE->getProportionBendPoints() *
            getFqinc(i1) / 100.;
         temp *= factor;
         fqbppia[i1] = temp;
      }
      // continue projecting at wage rate beyond ending year
      for (int i2 = lastYear + 1; i2 <= fqbppia.getLastYear(); i2++) {
         double factor = getFq(i2) / getFq(i2-1);
         temp *= factor;
         fqbppia[i2] = temp;
      }
      return;
   }
   // increase at cpi rate
   LawChange *lawChangeBPCPI = lawChange.lawChange[LawChange::BPCPI];
   if (lawChangeBPCPI->getInd() > 0) {
      const int lastYear = min(fqbppia.getLastYear(),
         lawChangeBPCPI->getEndYear() - 2);
      const int firstYear = lawChangeBPCPI->getStartYear() - 2;
      fqbppia.assign(getFqArray(), 1977, firstYear - 1);
      double temp = getFq(firstYear - 1);
      // project by cpi increase
      for (int i1 = firstYear - 2; i1 <= lastYear - 2; i1++) {
         double factor = 1. + getCpiinc(i1) / 100.;
         temp *= factor;
         fqbppia[i1] = temp;
      }
      // continue projecting at wage rate beyond ending year
      for (int i2 = lastYear + 1; i2 <= fqbppia.getLastYear(); i2++) {
         double factor = getFq(i2) / getFq(i2-1);
         temp *= factor;
         fqbppia[i2] = temp;
      }
      return;
   }
   // project off of alternate bend points
   LawChangeNEWFORMULA* lawChangeNEWFORMULA = lawChange.lawChangeNEWFORMULA;
   if (lawChangeNEWFORMULA->getInd() > 0) {
      const int firstYear = lawChangeNEWFORMULA->getStartYear() - 2;
      fqbppia.assign(getFqArray(), 1977, firstYear - 1);
      // remainder of bend points are set in projectfq
      return;
   }
   LawChangeBPSPECRATE* lawChangeBPSPECRATE = lawChange.lawChangeBPSPECRATE;
   if (lawChangeBPSPECRATE->getInd() > 0) {
      const int lastYear = min(fqbppia.getLastYear(),
         lawChangeBPSPECRATE->getEndYear() - 2);
      const int firstYear = lawChangeBPSPECRATE->getStartYear() - 2;
      fqbppia.assign(getFqArray(), 1977, firstYear - 1);
      double temp = getFq(firstYear - 1);
      // project by ad hoc bend point increases
      for (int i1 = firstYear; i1 <= lastYear; i1++) {
         double factor = 1. + lawChangeBPSPECRATE->getBendPiaInc(i1) / 100.;
         temp *= factor;
         fqbppia[i1] = temp;
      }
      // continue projecting at wage rate beyond ending year
      for (int i2 = lastYear + 1; i2 <= fqbppia.getLastYear(); i2++) {
         double factor = getFq(i2) / getFq(i2-1);
         temp *= factor;
         fqbppia[i2] = temp;
      }
      return;
   }
   LawChangeBPMINCONST *lawChangeBPMINCONST = lawChange.lawChangeBPMINCONST;
   if (lawChangeBPMINCONST->getInd() > 0) {
      const int lastYear = min(fqbppia.getLastYear(),
         lawChangeBPMINCONST->getEndYear() - 2);
      const int firstYear = lawChangeBPMINCONST->getStartYear() - 2;
      fqbppia.assign(getFqArray(), 1977, firstYear - 1);
      double temp = getFq(firstYear - 1);
      // project by average wage increase minus a percentage
      for (int i1 = firstYear; i1 <= lastYear; i1++) {
         double factor = 1. + (getFqinc(i1) -
            lawChangeBPMINCONST->getConstantPerc()) / 100.;
         temp *= factor;
         fqbppia[i1] = temp;
      }
      // continue projecting at wage rate beyond ending year
      for (int i2 = lastYear + 1; i2 <= fqbppia.getLastYear(); i2++) {
         double factor = getFq(i2) / getFq(i2-1);
         temp *= factor;
         fqbppia[i2] = temp;
      }
      return;
   }
}

// Description: Returns benefit amount increased by cola.
//
// Remarks: The version with 2 arguments does not consider any catch-up
//   benefit increases; the version with 3 arguments does.
//
// Returns: Increased benefit amount.
//
// Arguments:
//   pia: Amount to be increased.
//   year: Year of increase.
double PiaParamsLC::applyCola( double pia, int year ) const
{
   LawChangeCOLACHANGE *lawChangeCOLACHANGE = lawChange.lawChangeCOLACHANGE;
   if (lawChangeCOLACHANGE->getInd() == 2 &&
      lawChangeCOLACHANGE->isEffective(year, year)) {
      return min(PiaParams::applyCola(pia, year), pia + colaCaps[year]);
   }
   return PiaParams::applyCola(pia, year);
}

// Arguments:
//   pia: Amount to be increased.
//   year: Year of increase.
//   eligYear: Year of eligibility for catch-up benefit increases.
double PiaParamsLC::applyCola( double pia, int year, int eligYear ) const
{
   LawChangeCOLACHANGE *lawChangeCOLACHANGE = lawChange.lawChangeCOLACHANGE;
   if (lawChangeCOLACHANGE->getInd() == 2 &&
      lawChangeCOLACHANGE->isEffective(year, year)) {
      return min(PiaParams::applyCola(pia, year, eligYear),
         pia + colaCaps[year]);
   }
   return PiaParams::applyCola(pia, year, eligYear);
}

// Description: Returns maximum family benefit amount increased by cola, with
//   check that result is at least 150 percent of pia.
//
// Remarks: The version with 3 arguments does not consider any catch-up
//   benefit increases; the version with 4 arguments does.
//
// Returns: Increased maximum family benefit amount.
//
// Arguments:
//   mfb: Amount to be increased.
//   year: Year of increase.
//   pia: Pia to check against.
double PiaParamsLC::applyColaMfb( double mfb, int year, double pia ) const
{
   LawChangeCOLACHANGE *lawChangeCOLACHANGE = lawChange.lawChangeCOLACHANGE;
   if (lawChangeCOLACHANGE->getInd() == 2 &&
      lawChangeCOLACHANGE->isEffective(year, year)) {
      return min(PiaParams::applyColaMfb(mfb, year, pia),
         mfb + mfbColaCaps[year]);
   }
   return PiaParams::applyColaMfb(mfb, year, pia);
}

// Arguments:
//   mfb: Amount to be increased.
//   year: Year of increase.
//   eligYear: Year of eligibility for catch-up benefit increases.
//   pia: Pia to check against.
double PiaParamsLC::applyColaMfb( double mfb, int year, int eligYear,
double pia ) const
{
   LawChangeCOLACHANGE *lawChangeCOLACHANGE = lawChange.lawChangeCOLACHANGE;
   if (lawChangeCOLACHANGE->getInd() == 2 &&
      lawChangeCOLACHANGE->isEffective(year, year)) {
      return min(PiaParams::applyColaMfb(mfb, year, eligYear, pia),
         mfb + mfbColaCaps[year]);
   }
   return PiaParams::applyColaMfb(mfb, year, eligYear, pia);
}

// Description: Returns the amount of the cap on the COLA for the given year.
//
// Returns: The amount of the cap on the COLA for the given year (returns
//   maximum float if no cap).
//
// Arguments:
//   year: Year being capped.
double PiaParamsLC::getColaCap( int year ) const
{
   LawChangeCOLACHANGE *lawChangeCOLACHANGE = lawChange.lawChangeCOLACHANGE;
   if (lawChangeCOLACHANGE->getInd() == 2)
      return(colaCaps[year]);
   // No cap on the COLA, it can be as big as a regular COLA increase.
   return FLT_MAX;
}

// Description: Sets the amount of the cap on the COLA for the given year.
//
// Arguments:
//   cap: Amount of cap.
//   year: Year being capped.
void PiaParamsLC::setColaCap( double cap, int year )
{
   LawChangeCOLACHANGE *lawChangeCOLACHANGE = lawChange.lawChangeCOLACHANGE;
   if (lawChangeCOLACHANGE->getInd() == 2)
      colaCaps[year] = cap;
}

// Description: Returns the amount of the cap on the COLA for the MFB for
//   the given year.
//
// Returns: The amount of the cap on the COLA for the MFB for the given year
//   (returns maximum float if no cap).
//
// Arguments:
//   year: Year being capped.
double PiaParamsLC::getMfbColaCap( int year ) const
{
   LawChangeCOLACHANGE *lawChangeCOLACHANGE = lawChange.lawChangeCOLACHANGE;
   if (lawChangeCOLACHANGE->getInd() == 2)
      return(mfbColaCaps[year]);
   // No cap on the COLA, it can be as big as a regular COLA increase.
   return FLT_MAX;
}

// Description: Sets the amount of the cap on the COLA for the MFB
//   for the given year.
//
// Arguments:
//   cap: Amount of cap.
//   year: Year being capped.
void PiaParamsLC::setMfbColaCap( double cap, int year )
{
   LawChangeCOLACHANGE *lawChangeCOLACHANGE = lawChange.lawChangeCOLACHANGE;
   if (lawChangeCOLACHANGE->getInd() == 2)
      mfbColaCaps[year] = cap;
}

// Description: Returns one benefit increase.
//
// Returns: One benefit increase.
//
// Arguments:
//   year: The year for which a benefit increase is desired.
double PiaParamsLC::getCpiinc( int year ) const
{
   LawChangeCOLACHANGE *lawChangeCOLACHANGE = lawChange.lawChangeCOLACHANGE;
   if (lawChangeCOLACHANGE->getInd() == 1)
      return(altCpiinc[year]);
   // return regular benefit increases if no changes in law.
   return(cpiinc[year]);
}

// Description: Returns the benefit increases.
//
// Returns: The benefit increases.
const DoubleAnnual& PiaParamsLC::getCpiincArray() const
{
   LawChangeCOLACHANGE *lawChangeCOLACHANGE = lawChange.lawChangeCOLACHANGE;
   if (lawChangeCOLACHANGE->getInd() == 1)
      return(altCpiinc);
   // return regular benefit increases if no changes in law.
   return(cpiinc);
}

// Description: Returns series of average wages or other series used for
//   indexing pia formula bend points.
//
// Returns: Series of average wages or other series used for
//   indexing pia formula bend points.
const AverageWage& PiaParamsLC::getFqBppia() const
{
   if (lawChange.lawChange[LawChange::BPFRACWAGE]->getInd() > 0 ||
      lawChange.lawChange[LawChange::BPCPI]->getInd() > 0 ||
      lawChange.lawChange[LawChange::NEWFORMULA]->getInd() > 0 ||
      lawChange.lawChange[LawChange::BPSPECRATE]->getInd() > 0 ||
      lawChange.lawChange[LawChange::BPMINCONST]->getInd() > 0)
      return(fqbppia);
   // return average wage series if no changes
   return(fq);
}

// Description: Calculate series of average wages or other series used for
//   indexing an earnings record.
void PiaParamsLC::setFqIndex()
{
   int i, i2, i3;  // loop counters
   double factor, temp;  // temporary values

   LawChange *lawChangeEARNINDCPI = lawChange.lawChange[LawChange::EARNINDCPI];
   if (lawChangeEARNINDCPI->getInd() > 0) {
      i2 = min(fqindex.getLastYear(),
         lawChangeEARNINDCPI->getEndYear() - 2);
      i3 = lawChangeEARNINDCPI->getStartYear() - 2;
      fqindex.assign(getFqArray(), YEAR37, i3 - 1);
      temp = getFq(i3 - 1);
      // project by cpi increase
      for (i = i3; i <= i2; i++) {
         factor = 1. + getCpiinc(i) / 100.;
         temp *= factor;
         fqindex[i] = temp;
      }
      // continue projecting at wage rate beyond ending year
      for (i = i2 + 1; i <= fqindex.getLastYear(); i++) {
         factor = getFq(i) / getFq(i-1);
         temp *= factor;
         fqindex[i] = temp;
      }
   }
}

// Description: Returns series of average wages or other series used for
//   indexing an earnings record.
//
// Returns: Series of average wages or other series used for
//   indexing an earnings record.
const AverageWage& PiaParamsLC::getFqIndex() const
{
   LawChange *lawChangeEARNINDCPI = lawChange.lawChange[LawChange::EARNINDCPI];
   if (lawChangeEARNINDCPI->getInd() > 0)
      return(fqindex);
   // return average wage series if no changes
   return(fq);
}

// Description: Projects average annual wage and amounts dependent on it.
void PiaParamsLC::projectFq()
{
   PiaParams::projectFq();
   LawChangeNEWFORMULA *lawChangeNEWFORMULA = lawChange.lawChangeNEWFORMULA;
   if (lawChangeNEWFORMULA->getInd() > 0) {
      int i2 = min(fqbppia.getLastYear(), lawChangeNEWFORMULA->getEndYear());
      int i3 = lawChangeNEWFORMULA->getStartYear();
      for (int i = i3; i <= i2; i++) {
         bpPiaOut.setData(i, lawChangeNEWFORMULA->getAltBendPia(i, 1),
            lawChangeNEWFORMULA->getAltBendPia(i, 2));
      }
      // project bend points beyond specified ones
      bpPiaOut.setData(getFqArray(), i2 + 1);
   }
}

// Description: Projects benefit formula percentages.
void PiaParamsLC::projectPerc()
{
   int i;  // loop counter
   int lastYear;  // last year of changing percentages
   int firstYear;  // first year of changing percentages

   PiaParams::projectPerc();
   LawChangeNEWFORMULA *lawChangeNEWFORMULA = lawChange.lawChangeNEWFORMULA;
   LawChangeDECLINEPERC *lawChangeDECLINEPERC = lawChange.lawChangeDECLINEPERC;
   if (lawChangeNEWFORMULA->getInd() > 0 ||
      lawChangeDECLINEPERC->getInd() > 0) {
      double percTemp[3];
      if (lawChangeNEWFORMULA->getInd() > 0) {
         lastYear = min(maxyear, lawChangeNEWFORMULA->getEndYear());
         firstYear = lawChangeNEWFORMULA->getStartYear();
         for (i = firstYear; i <= lastYear; i++) {
            percPiaOut.setData(i, lawChangeNEWFORMULA->getAltPercPia(i, 0),
               lawChangeNEWFORMULA->getAltPercPia(i, 1),
               lawChangeNEWFORMULA->getAltPercPia(i, 2));
         }
         // save last percentages
         percTemp[0] = lawChangeNEWFORMULA->getAltPercPia(lastYear, 0);
         percTemp[1] = lawChangeNEWFORMULA->getAltPercPia(lastYear, 1);
         percTemp[2] = lawChangeNEWFORMULA->getAltPercPia(lastYear, 2);
      }
      else {
         lastYear = min(maxyear, lawChangeDECLINEPERC->getEndYear());
         firstYear = lawChangeDECLINEPERC->getStartYear();
         for (i = firstYear; i <= lastYear; i++) {
            percPiaOut.setData(i, lawChangeDECLINEPERC->getAltPercPia(i, 0),
               lawChangeDECLINEPERC->getAltPercPia(i, 1),
               lawChangeDECLINEPERC->getAltPercPia(i, 2));
         }
         // save last percentages
         percTemp[0] = lawChangeDECLINEPERC->getAltPercPia(lastYear, 0);
         percTemp[1] = lawChangeDECLINEPERC->getAltPercPia(lastYear, 1);
         percTemp[2] = lawChangeDECLINEPERC->getAltPercPia(lastYear, 2);
      }
      // project percentages beyond specified ones
      for (i = lastYear + 1; i <= maxyear; i++) {
         percPiaOut.setData(i, percTemp[0],
            percTemp[1], percTemp[2]);
      }
   }
}

// Description: Returns amount per year of coverage in special minimum.
//
// Returns: Amount per year of coverage in special minimum.
//
// Arguments:
//   date1: The date of desired amount.
double PiaParamsLC::specMinAmountCal( const DateMoyr& date1 ) const
{
   LawChangeNEWSPECMIN *lawChangeNEWSPECMIN = lawChange.lawChangeNEWSPECMIN;
   // change dollar amount, if applicable
   if ((lawChangeNEWSPECMIN->getInd() == 1 ||
      lawChangeNEWSPECMIN->getInd() == 3) &&
      static_cast<int>(date1.getYear()) >=
      lawChangeNEWSPECMIN->getStartYear()) {
      return(lawChangeNEWSPECMIN->getAmount());
   }
   else {
      return(specMinAmountCalPL(date1));
   }
}

// Description: Projects amounts dependent on benefit increases.
void PiaParamsLC::projectCpiinc()
{
   setAltCpiinc();
   resizeSpecMin();
   LawChangeNEWSPECMIN* lawChangeNEWSPECMIN = lawChange.lawChangeNEWSPECMIN;
   if (lawChangeNEWSPECMIN->getInd() == 1 ||
      lawChangeNEWSPECMIN->getInd() == 3) {
      projectSpecMin(YEAR79, lawChangeNEWSPECMIN->getStartYear() - 1);
      projectSpecMin(lawChangeNEWSPECMIN->getStartYear(), maxyear);
   }
   else {
      projectSpecMin(YEAR79, maxyear);
   }
}

// Description: Returns maximum number of usable years for special
//   minimum.
//
// Remarks: The version with no arguments returns the maximum number
//   that could ever be for any year of eligibility and year of benefit.
//
// Returns: Maximum number of usable years for special minimum.
int PiaParamsLC::specMinMaxYears() const
{
   LawChangeNEWSPECMIN *lawChangeNEWSPECMIN = lawChange.lawChangeNEWSPECMIN;
   // change maximum years, if applicable
   if ((lawChangeNEWSPECMIN->getInd() % 4) >= 2) {
      return max(specMinMaxYearsPL(), lawChangeNEWSPECMIN->getYears() - 10);
   }
   else {
      return specMinMaxYearsPL();
   }
}

// Arguments:
//   eligYear: Year of eligibility for which maximum number of years is
//     desired.
//   benYear: Year of benefit for which maximum number of years is desired.
int PiaParamsLC::specMinMaxYears( int eligYear, int benYear ) const
{
   LawChangeNEWSPECMIN *lawChangeNEWSPECMIN = lawChange.lawChangeNEWSPECMIN;
   // change maximum years, if applicable
   if (lawChangeNEWSPECMIN->isEffective(eligYear, benYear) &&
      (lawChangeNEWSPECMIN->getInd() % 4) >= 2) {
      return(lawChangeNEWSPECMIN->getYears() - 10);
   }
   else {
      return specMinMaxYearsPL();
   }
}

// Description: Returns full retirement age.
//
// Returns: Full retirement age.
//
// Arguments:
//   eligYear: Year of eligibility (age 62 for oab or wife/husband, 60 for
//     widow(er)).
Age PiaParamsLC::fullRetAgeCal( int eligYear ) const
{
   LawChangeNRACHANGE *lawChangeNRACHANGE = lawChange.lawChangeNRACHANGE;
   if (lawChangeNRACHANGE->getInd() > 0) {
      // eliminate increase beyond age 65 for alternative 1
      if (lawChangeNRACHANGE->getInd() == 1)
         return age65;
      // eliminate plateau between 66 and 67 for alternative 2 or 3
      if (lawChangeNRACHANGE->getInd() > 1) {
         if (eligYear < 2000)  // age 65
            return age65;
         if (eligYear > 1999 && eligYear < 2005)
            // age 65 and 2 months to age 65 and 10 months
            return Age(65, 2 * (eligYear - 1999));
         if (eligYear > 2004 && eligYear < 2006)  // age 66
            return age66;
         if (eligYear > 2005 && eligYear < 2011)
            // age 66 and 2 months to age 66 and 10 months
            return Age(66, 2 * (eligYear - 2005));
         // if alternative 2, stop at age 67
         if (lawChangeNRACHANGE->getInd() == 2) {
            // age 67
            return age67;
         }
         // if alternative 3, continue on forever
         else {
            // calculate increase in nra over age 67, in months
            int months = (eligYear - 2011) / 2;
            // convert to years and months
            int years = 67 + months / 12;
            months %= 12;
            return Age(years, months);
         }
      }
      // impossible parameter
      else
         return age199;
   }
   else
      return(PiaParams::fullRetAgeCalPL(eligYear));
}

// Description: Returns full retirement age for benefit calculations involving
//   disability freeze.
//
// Returns: Full retirement age for benefit calculations involving
//   disability freeze.
//
// Arguments:
//   eligYear: Year of eligibility (age 62 for oab or wife/husband, 60 for
//     widow(er)).
//   currentYear: Current year.
Age PiaParamsLC::fullRetAgeCalDI( int eligYear, int currentYear ) const
{
   LawChangeNRACHANGE *lawChangeNRACHANGE = lawChange.lawChangeNRACHANGE;
   // assume that any change does not take effect for benefit calculations
   // until 2006
   if (lawChangeNRACHANGE->getInd() > 0 && currentYear > 2005) {
      return fullRetAgeCal(eligYear);
   }
   else {
      return PiaParams::fullRetAgeCalDIPL(eligYear, currentYear);
   }
}

// Description: Returns maximum DI beneficiary age at end of year.
//
// Returns: Maximum DI beneficiary age at end of year.
//
// Arguments:
//   year: Year of benefit.
Age PiaParamsLC::maxDibAge( int year ) const
{
   LawChangeNRACHANGE *lawChangeNRACHANGE = lawChange.lawChangeNRACHANGE;
   if (lawChangeNRACHANGE->getInd() > 0) {
      // eliminate increase beyond age 65
      if (lawChangeNRACHANGE->getInd() == 1)
         return age65;
      // eliminate plateau between 66 and 67
      if (lawChangeNRACHANGE->getInd() > 1) {
         if (year < 2003)  // age 65
            return age65;
         if (year > 2002 && year < 2008)
            // age 65 and 2 months to age 65 and 10 months
            return Age(65, 2 * (year - 2002));
         if (year > 2007 && year < 2010)  // age 66
            return age66;
         if (year > 2009 && year < 2015)
            // age 66 and 2 months to age 66 and 10 months
            return Age(66, 2 * (year - 2009));
         // if alternative 2, stop at age 67
         if (lawChangeNRACHANGE->getInd() == 2) {
            // age 67
            return age67;
         }
         // if alternative 3, continue on forever
         else {
            // calculate years, taking 24 years to increment by one full year,
            // plus an extra year (explained in maxDibAge.xls)
            int years = 67 + (year - 2015) / 25;
            // calculate year from which to determine months
            int year25 = 2016 + (years - 67) * 25;
            // calculate months
            int months = max(0, (year - year25) / 2);
            return Age(years, months);
         }
      }
      // impossible parameter
      else
         return age199;
   }
   else
      return(PiaParams::maxDibAgePL(year));
}

// Description: Returns factor for wife/husband benefit before age reduction.
//
// Returns: Factor for wife/husband benefit before age reduction.
//
// Arguments:
//   eligYear: Spouse's year of eligibility.
//   benYear: Year of benefit.
double PiaParamsLC::factorAgedSpouseCal( int eligYear, int benYear ) const
{
   LawChangeWIFEFACTOR *lawChangeWIFEFACTOR = lawChange.lawChangeWIFEFACTOR;
   if (lawChangeWIFEFACTOR->isEffective(eligYear, benYear)) {
      if (lawChangeWIFEFACTOR->getInd() == 1) {
         // effective immediately
         return(.33);
      }
      else {
         if (eligYear < lawChangeWIFEFACTOR->getStartYear()) {
            return PiaParams::factorAgedSpouseCalPL();
         }
         else {
            // phase in over 17 years
            return(max(.33, .50 - (eligYear -
               lawChangeWIFEFACTOR->getStartYear() + 1) * .01));
         }
      }
   }
   else {
      return PiaParams::factorAgedSpouseCalPL();
   }
}

// Description: Returns factor for widow/widower benefit before age reduction.
//
// Returns: Factor for widow/widower benefit before age reduction. This could
//   be an increased factor (up to 112.5 percent), if LawChange::WIDFACTOR is
//   effective.
//
// Arguments:
//   monthsArdri: Months of reduction.
//   age: The age of widow/widower.
//   eligYear: Year of eligibility.
//   benefitDate: The date of benefit.
double PiaParamsLC::factorAgedWidCal( int monthsArdri, const Age& age,
   int eligYear, const DateMoyr& benefitDate ) const
{
   LawChangeWIDFACTOR *lawChangeWIDFACTOR = lawChange.lawChangeWIDFACTOR;
   // if proposal to increase widow factor is effective, calculate new factor
   if (lawChangeWIDFACTOR->getInd() < 3 &&
      lawChangeWIDFACTOR->isEffective(eligYear, benefitDate.getYear())) {
      if (lawChangeWIDFACTOR->getInd() == 1) {
         // effective immediately
         return(1.125);
      }
      else {
         if (eligYear < lawChangeWIDFACTOR->getStartYear()) {
            return PiaParams::factorAgedWidCalPL(monthsArdri, age,
               benefitDate);
         }
         else {
            // phase in over 40 years
            return(min(1.125, 1.0 + (eligYear -
               lawChangeWIDFACTOR->getStartYear() + 1) * 0.125 / 40.));
         }
      }
   }
   // otherwise return present-law factor
   else {
      return PiaParams::factorAgedWidCalPL(monthsArdri, age, benefitDate);
   }
}

// Description: Returns factor for disabled widow/widower benefit before age
// reduction.
//
// Returns: Factor for disabled widow/widower benefit before age reduction.
//
// Remarks: Returns factorDisWidCalPL. There are no law changes built in for
// disabled widow/widower benefits yet.
//
// Arguments:
//   eligYear: Year of eligibility.
//   benefitDate: The date of benefit.
double PiaParamsLC::factorDisWidCal( int eligYear,
const DateMoyr& benefitDate ) const
{
   return factorDisWidCalPL(benefitDate);
}

// Description: Updates wage bases.
//
// Remarks: The version with no arguments updates bases for average
//   wage changes. The version with arguments updates wage bases for
//   user entries and/or for average wage changes.
void PiaParamsLC::updateBases()
{
   int i;  // loop index
   int startyear;  // starting year of automatic projection

   LawChangeWAGEBASECHG *lawChangeWAGEBASECHG = lawChange.lawChangeWAGEBASECHG;
   // Handle regular wage bases
   if (lawChangeWAGEBASECHG->getInd() == 1 ||
      lawChangeWAGEBASECHG->getInd() == 3) {
      for (i = lawChangeWAGEBASECHG->getStartYear();
      i <= lawChangeWAGEBASECHG->getEndYear(); i++) {
         baseOasdi[i] = lawChangeWAGEBASECHG->getBaseOasdi(i);
      }
      startyear = lawChangeWAGEBASECHG->getEndYear() + 1;
   }
   else {
      startyear = getIstart() + 1;
   }
   baseOasdi.project(fq, getCpiincArray(), startyear, baseOasdi.getLastYear());

   // Handle 1977-law wage bases
   if (lawChangeWAGEBASECHG->getInd() == 2 ||
      lawChangeWAGEBASECHG->getInd() == 3) {
      for (i = lawChangeWAGEBASECHG->getStartYear();
      i <= lawChangeWAGEBASECHG->getEndYear(); i++) {
         base77[i] = lawChangeWAGEBASECHG->getBase77(i);
      }
      startyear = lawChangeWAGEBASECHG->getEndYear() + 1;
   }
   else {
      startyear = getIstart() + 1;
   }
   base77.project(fq, getCpiincArray(), startyear, base77.getLastYear());

   // HI bases are always set automatically
   baseHi.project(fq, getCpiincArray(), getIstart() + 1, baseHi.getLastYear());
}

// Arguments:
//   userbase: User-entered bases.
//   userbase77: User-entered old-law bases.
//   ibasch: Indicator for base changes (1=none, 2=change).
//   firstYeart: First year to update.
//   lastYeart: Last year to update.
void PiaParamsLC::updateBases( const DoubleAnnual& userbase,
const DoubleAnnual& userbase77, int ibasch, int firstYeart, int lastYeart )
{
   if (ibasch == 2) {
      // set ad hoc OASDI bases
      baseOasdi.assign(userbase, firstYeart, lastYeart);
      // project OASDI bases
      baseOasdi.project(fq, getCpiincArray(), lastYeart + 1,
         baseOasdi.getLastYear());
      // set ad hoc old-law bases
      base77.assign(userbase77, firstYeart, lastYeart);
      // project old-law OASDI bases
      base77.project(fq, getCpiincArray(), lastYeart + 1,
         base77.getLastYear());
      // HI bases are always set automatically
      baseHi.project(fq, getCpiincArray(), getIstart() + 1,
         baseHi.getLastYear());
   }
   else {
      updateBases();
   }
}

// Description: Calculates reduction factor for OAB.
//
// Returns: Reduction factor for OAB.
//
// Arguments:
//   monthsArdri: Number of months of reduction.
double PiaParamsLC::factorArCal( int monthsArdri ) const
{
   LawChangeNRACHANGE *lawChangeNRACHANGE = lawChange.lawChangeNRACHANGE;
   if (lawChangeNRACHANGE->getInd() > 0 && monthsArdri > 60) {
      return (monthsArdri <= 84) ?
         // use 3/8 percent per month for years 6 and 7
         1. - (36. * 5. / 900.) - (24. * 5. / 1200.) -
         ((static_cast<double>(monthsArdri) - 60.) * 3. / 800.) :
         // use 1/3 percent per month for years 8 and later
         1. - (36. * 5. / 900.) - (24. * 5. / 1200.) - (24. * 3. / 800.) -
         ((static_cast<double>(monthsArdri) - 84.) / 300.);
   }
   else {
      return PiaParams::factorArCalPL(monthsArdri);
   }
}

// Description: Calculates reduction factor for wife/husband.
//
// Arguments:
//   monthsArdri: Number of months of reduction.
double PiaParamsLC::factorArAgedSpouseCal( int monthsArdri ) const
{
   LawChangeNRACHANGE *lawChangeNRACHANGE = lawChange.lawChangeNRACHANGE;
   if (lawChangeNRACHANGE->getInd() > 0 && monthsArdri > 60) {
      return (monthsArdri <= 84) ?
         // use 3/8 percent per month for years 6 and 7
         1. - 0.25 - (24. * 5. / 1200.) -
         ((static_cast<double>(monthsArdri) - 60.) * 3. / 800.) :
         // use 1/3 percent per month for years 8 and later
         1. - 0.25 - (24. * 5. / 1200.) - (24. * 3. / 800.) -
         ((static_cast<double>(monthsArdri) - 84.) / 300.);
   }
   else {
      return PiaParams::factorArAgedSpouseCalPL(monthsArdri);
   }
}

// Description: Returns maximum amount of earnings in a childcare dropout year.
//
// Returns: Maximum amount of earnings in a childcare dropout year.
//
// Arguments:
//   eligYear: Year of eligibility.
//   benYear: Year for which amount is desired.
double PiaParamsLC::getChildcareDropoutAmount( int eligYear,
int benYear ) const
{
   LawChangeCHILDCAREDROPOUT *lawChangeCHILDCAREDROPOUT =
      lawChange.lawChangeCHILDCAREDROPOUT;
   if (lawChangeCHILDCAREDROPOUT->isEffective(eligYear, benYear)) {
      return lawChangeCHILDCAREDROPOUT->getFqRatio() * fq[benYear];
   }
   return getChildcareDropoutAmountPL();
}

// Description: Returns maximum number of childcare dropout years.
//
// Arguments:
//   eligYear: Year of eligibility.
//   benYear: Year for which number is desired.
int PiaParamsLC::getMaxChildcareDropoutYears( int eligYear, int benYear ) const
{
   LawChangeCHILDCAREDROPOUT *lawChangeCHILDCAREDROPOUT =
      lawChange.lawChangeCHILDCAREDROPOUT;
   if (lawChangeCHILDCAREDROPOUT->isEffective(eligYear, benYear)) {
      return lawChangeCHILDCAREDROPOUT->getMaxYears();
   }
   return getMaxChildcareDropoutYearsPL();
}

// Description: Sets earliest possible retirement age for old-age benie or aged
//   spouse.
//
// Arguments:
//   sex: Sex of number holder (MALE for male, FEMALE for female).
//   kbirth: The date of birth, adjusted to previous day.
Age PiaParamsLC::earlyAgeOabCal( Sex::sex_type sex,
const boost::gregorian::date& kbirth ) const
{
   return(earlyAgeOabCalPL(sex, kbirth));
}

// Description: Returns maximum age of child for childcare dropout year or
//   childcare credit.
//
// Returns: Maximum age of child for childcare dropout year.
int PiaParamsLC::getMaxChildcareAge() const
{
   LawChangeCHILDCAREDROPOUT *lawChangeCHILDCAREDROPOUT =
      lawChange.lawChangeCHILDCAREDROPOUT;
   if (lawChangeCHILDCAREDROPOUT->getInd() > 0) {
      return lawChangeCHILDCAREDROPOUT->getMaxAge();
   }
   LawChangeCHILDCARECREDIT *lawChangeCHILDCARECREDIT =
      lawChange.lawChangeCHILDCARECREDIT;
   if (lawChangeCHILDCARECREDIT->getInd() > 0) {
      return lawChangeCHILDCARECREDIT->getMaxAge();
   }
   return PiaParams::getMaxChildcareAgePL();
}

// Description: Resizes special minimum arrays.
void PiaParamsLC::resizeSpecMin()
{
   PiaParams::resizeSpecMin();
   int n = specMinMaxYears();
   // for changed law allocate space for initial amounts
   specMinPiaExtra.resize(n);
   specMinMfbExtra.resize(n);
}

// Description: Projects special minimum amounts.
//
// Arguments:
//   baseYear: First year of special minimum amounts (1979 for present law).
//   lastYear: Last year of projection.
void PiaParamsLC::projectSpecMin( int baseYear, int lastYear )
{
   int i1;  // loop counter

   double amountPerYear = specMinAmountCal(DateMoyr(1, baseYear));
   for (int i2 = 0; i2 < specMinMaxYears(); i2++) {
      double pia = (i2 + 1) * amountPerYear;
      double mfb = BenefitAmount::round(1.5 * pia, baseYear);
      // for present law, save initial amounts in 1978 slot
      if (baseYear == YEAR79) {
         specMinPia[i2][1978] = pia;
         specMinMfb[i2][1978] = mfb;
         // handle years up through 2000
         for (i1 = baseYear; i1 <= min(2000, lastYear); i1++) {
            pia = applyCola(pia, i1);
            specMinPia[i2][i1] = pia;
            mfb = applyColaMfb(mfb, i1, pia);
            specMinMfb[i2][i1] = mfb;
         }
         // recalculate values for December 1999, adding 0.1 percent
         double pia1999 = applyCola99(specMinPia[i2][1998]);
         double mfb1999 = applyColaMfb99(specMinMfb[i2][1998], pia1999);
         // recalculate values for December 2000, adding 0.1 percent
         // these become effective August 2001
         pia1999 = applyCola(pia1999, 2000);
         specMinPia2001[i2] = pia1999;
         mfb1999 = applyColaMfb(mfb1999, 2000, pia);
         specMinMfb2001[i2] = mfb1999;
         // calculate values for December 2001
         pia = applyCola(pia1999, 2001);
         specMinPia[i2][2001] = pia;
         mfb = applyColaMfb(mfb1999, 2001, pia);
         specMinMfb[i2][2001] = mfb;
      }
      // for changed law (after 2001) save initial amounts in extra vectors
      else {
         specMinPiaExtra[i2] = pia;
         specMinMfbExtra[i2] = mfb;
      }
      // handle years after 2001
      for (i1 = max(2002, baseYear); i1 <= lastYear; i1++) {
         pia = applyCola(pia, i1, YEAR79);
         specMinPia[i2][i1] = pia;
         mfb = applyColaMfb(mfb, i1, YEAR79, pia);
         specMinMfb[i2][i1] = mfb;
      }
   }
}

// Description: Returns special minimum pia as of a specified month.
//
// Arguments:
//   dateMoyr: Month and year for which pia is desired (January 1979 or later).
//   yoc: Years of coverage in excess of 10 (1-20).
double PiaParamsLC::getSpecMinPia( const DateMoyr& dateMoyr, int yoc ) const
{
   LawChangeNEWSPECMIN *lawChangeNEWSPECMIN = lawChange.lawChangeNEWSPECMIN;
   int year = static_cast<int>(dateMoyr.getYear());
   if (static_cast<int>(dateMoyr.getMonth()) < getMonthBeninc(year)) {
      // in 2001, apply extra 0.1 percent in August
      if (year == 2001 && dateMoyr.getMonth() >= 8) {
         return((yoc > 0) ? specMinPia2001[yoc-1] : 0.0);
      }
      // if this is the year where the amounts changed, use the special amounts
      else if ((lawChangeNEWSPECMIN->getInd() == 1 ||
         lawChangeNEWSPECMIN->getInd() == 3) &&
         year == lawChangeNEWSPECMIN->getStartYear()) {
         return((yoc > 0) ? specMinPiaExtra[yoc-1] : 0.0);
      }
      else {
         year--;
      }
   }
   // if did not return above, use regular special minimum table
   return((yoc > 0) ? specMinPia[yoc-1][year] : 0.);
}

// Description: Returns special minimum mfb as of a specified month.
//
// Arguments:
//   dateMoyr: Month and year for which mfb is desired (January 1979 or later).
//   yoc: Years of coverage in excess of 10 (1-20).
double PiaParamsLC::getSpecMinMfb( const DateMoyr& dateMoyr, int yoc ) const
{
   LawChangeNEWSPECMIN *lawChangeNEWSPECMIN = lawChange.lawChangeNEWSPECMIN;
   int year = dateMoyr.getYear();
   if (static_cast<int>(dateMoyr.getMonth()) < getMonthBeninc(year)) {
      // in 2001, apply extra 0.1 percent in August
      if (year == 2001 && dateMoyr.getMonth() >= 8) {
         return((yoc > 0) ? specMinMfb2001[yoc-1] : 0.0);
      }
      // if this is the year where the amounts changed, use the special amounts
      else if ((lawChangeNEWSPECMIN->getInd() == 1 ||
         lawChangeNEWSPECMIN->getInd() == 3) &&
         year == lawChangeNEWSPECMIN->getStartYear()) {
         return((yoc > 0) ? specMinMfbExtra[yoc-1] : 0.0);
      }
      else {
         year--;
      }
   }
   // if did not return above, use regular special minimum table
   return((yoc > 0) ? specMinMfb[yoc-1][year] : 0.0);
}

// Description: Sets data from historical average wages and benefit increase
//   file.
//
// Arguments:
//   awbiData: Structure with historical average wages and benefit increases.
void PiaParamsLC::setData( AwbiData& awbiData )
{
   PiaParams::setData(awbiData);
   const int year = awbiData.getLastYear();
   baseOasdi.assign(awbiData.baseOasdi, YEAR37, year);
   base77.assign(awbiData.base77, YEAR37, year);
}

// Description: Returns the number of years of marriage needed to be eligible
//   for a divorced spouse benefit.
//
// Arguments:
//   dateMoyr: The date (year and month) when checking if eligible for
//     divorced spouse benefit.
//
// Returns: Number of years of marriage needed to be eligible
//   for a divorced spouse benefit.
int PiaParamsLC::getMarrLengthForDivBen( const DateMoyr& dateMoyr ) const
{
   const int year = dateMoyr.getYear();
   LawChangeMARRLENGTH *lawChangeMARRLENGTH = lawChange.lawChangeMARRLENGTH;
   if (lawChangeMARRLENGTH->isEffective(year, year))
      return lawChangeMARRLENGTH->getMarrLengthForDivBen();
   else
      return getMarrLengthForDivBenPL(dateMoyr);
}

// Description: Returns widow(er) benefit reduced for RIB-LIM.
//
// Returns: Reduced widow(er) benefit.
//
// Remarks: This should only be called in cases where the deceased worker was
//   an OAB before death.
//
// Arguments:
//   widowBen: Widow(er) benefit reduced for widow(er)'s age, but not for
//     RIB-LIM.
//   oabPia: PIA upon which widow(er) benefit is based.  Note that this is not
//     a re-indexed widow(er) PIA, since the OAB must have survived to age 62.
//   oabBen: Benefit that deceased OAB was receiving.
//   benefitDate: The date of benefit.
double PiaParamsLC::ribLimCal( double widowBen, double oabPia, double oabBen,
const DateMoyr& benefitDate ) const
{
   LawChangeWIDFACTOR *lawChangeWIDFACTOR = lawChange.lawChangeWIDFACTOR;
   // if law change is effective, do not apply RIB-LIM
   if (lawChangeWIDFACTOR->getInd() == 3 &&
      lawChangeWIDFACTOR->isEffective(benefitDate.getYear(),
      benefitDate.getYear())) {
      return widowBen;
   }
   // otherwise apply present-law RIB-LIM
   else {
      return ribLimCalPL(widowBen, oabPia, oabBen, benefitDate);
   }
}
