// $Id: piaparmsPL.cpp 1.36 2007/02/28 15:06:14EST 044579 Development  $
//
// Functions to handle present law PIA parameters.

#include <cfloat>  // for FLT_MAX
#include "piaparmsPL.h"
#include "WageBase.h"
#include "oactcnst.h"
#include "AwbiDataNonFile.h"

using namespace std;

// Description: Initializes arrays.
//
// Remarks: The version without newIstart uses the year in BaseYearNonFile, and
// constructs an AwbiDataNonFile using that year.
//
// Arguments:
//   newIstart: Year after last known benefit increase. If not specified,
//     YEAR in BaseYearNonFile is used.
//   newMaxyear: Maximum projected year.
//   awbiData: Structure with historical amounts.
PiaParamsPL::PiaParamsPL( int newIstart, int newMaxyear, AwbiData& awbiData ) :
PiaParams(newIstart, newMaxyear), baseOasdi(newMaxyear), base77(newMaxyear)
{
   setData(awbiData);
   initdata();
}

PiaParamsPL::PiaParamsPL( int newMaxyear ) :
PiaParams(newMaxyear), baseOasdi(newMaxyear), base77(newMaxyear)
{
   AwbiDataNonFile awbiData(newMaxyear);
   setData(awbiData);
   initdata();
}

// Description: Destroys arrays.
PiaParamsPL::~PiaParamsPL( )
{ }

// Description: Returns the amount of the cap on the COLA for the given year.
//
// Returns: Maximum float value.
double PiaParamsPL::getColaCap( int ) const
{
   // No cap on the COLA, it can be as big as possible.
   return FLT_MAX;
}

// Description: Sets the amount of the cap on the COLA for the given year.
void PiaParamsPL::setColaCap( double, int )
{ }

// Description: Returns the amount of the cap on the COLA for the MFB
//   for the given year.
//
// Returns: Maximum float value.
double PiaParamsPL::getMfbColaCap( int ) const
{
   // No cap on the COLA, it can be as big as possible.
   return FLT_MAX;
}

// Description: Sets the amount of the cap on the COLA for the MFB
//   for the given year.
void PiaParamsPL::setMfbColaCap( double, int )
{ }

// Description: Returns series of average wages used for indexing pia formula
//   bend points.
//
// Returns: Series of average wages used for indexing pia formula bend points.
const AverageWage& PiaParamsPL::getFqBppia() const
{
   return(fq);
}

// Description: Returns series of average wages used for indexing an earnings
//   record.
//
// Returns: Series of average wages used for indexing an earnings record.
const AverageWage& PiaParamsPL::getFqIndex() const
{
   return(fq);
}

// Description: Returns maximum amount of earnings in a childcare dropout year.
//
// Returns: 0 for present law.
double PiaParamsPL::getChildcareDropoutAmount( int, int ) const
{
   return getChildcareDropoutAmountPL();
}

// Description: Returns maximum number of childcare dropout years.
//
// Returns: 3 for present law.
int PiaParamsPL::getMaxChildcareDropoutYears( int, int ) const
{
   return getMaxChildcareDropoutYearsPL();
}

// Description: Returns maximum number of usable years for special minimum.
//
// Remarks: The version with no arguments returns the maximum number that
//   could ever be for any year of eligibility and year of benefit.
//
// Returns: Maximum number of usable years for special minimum.
int PiaParamsPL::specMinMaxYears() const
{
   return specMinMaxYearsPL();
}

int PiaParamsPL::specMinMaxYears( int, int ) const
{
   return specMinMaxYearsPL();
}

// Description: Returns amount per year of coverage in special minimum.
//
// Returns: Amount per year of coverage in special minimum.
//
// Arguments:
//   date1: The date of desired amount.
double PiaParamsPL::specMinAmountCal( const DateMoyr& date1 ) const
{
   return specMinAmountCalPL(date1);
}

// Description: Returns maximum DI beneficiary age at end of year.
//
// Returns:  Maximum DI beneficiary age at end of year, based on present law.
//
// Arguments:
//   year: Year of benefit.
Age PiaParamsPL::maxDibAge( int year ) const
{
   return maxDibAgePL(year);
}

// Description: Sets earliest possible retirement age for old-age benie or aged
//   spouse.
//
// Returns: Earliest possible retirement age, based on present law.
//
// Arguments:
//   sex: Sex of number holder (MALE for male, FEMALE for female).
//   kbirth: The date of birth, adjusted to previous day.
Age PiaParamsPL::earlyAgeOabCal( Sex::sex_type sex,
const boost::gregorian::date& kbirth ) const
{
   return(earlyAgeOabCalPL(sex, kbirth));
}

// Description: Returns factor for wife/husband benefit before age reduction.
//
// Returns: Factor for wife/husband benefit before age reduction.
double PiaParamsPL::factorAgedSpouseCal( int, int ) const
{
   return factorAgedSpouseCalPL();
}

// Description: Returns full retirement age.
//
// Returns: Full retirement age based on present law.
//
// Arguments:
//   eligYear: Year of eligibility (age 62 for oab or wife/husband, 60 for
//     widow(er)).
Age PiaParamsPL::fullRetAgeCal( int eligYear ) const
{
   return fullRetAgeCalPL(eligYear);
}

// Description: Returns factor for widow/widower benefit before age reduction.
//
// Returns: Factor for widow/widower benefit before age reduction.
//
// Remarks: Returns factorAgedWidCalPL.
//
// Arguments:
//   monthsArdri: Months of reduction.
//   age: The age of widow/widower.
//   benefitDate: The date of benefit.
double PiaParamsPL::factorAgedWidCal( int monthsArdri, const Age& age,
   int, const DateMoyr& benefitDate ) const
{
   return factorAgedWidCalPL(monthsArdri, age, benefitDate);
}

// Description: Returns factor for disabled widow/widower benefit before age
// reduction.
//
// Returns: Factor for disabled widow/widower benefit before age reduction.
//
// Remarks: Returns factorDisWidCalPL.
//
// Arguments:
//   benefitDate: The date of benefit.
double PiaParamsPL::factorDisWidCal( int, const DateMoyr& benefitDate ) const
{
   return factorDisWidCalPL(benefitDate);
}

// Description: Returns reduction factor for wife/husband.
//
// Returns: Reduction factor for wife/husband.
//
// Arguments:
//   monthsArdri: Number of months of reduction.
double PiaParamsPL::factorArAgedSpouseCal( int monthsArdri ) const
{
   return factorArAgedSpouseCalPL(monthsArdri);
}

// Description: Calculates reduction factor for OAB.
//
// Returns: Reduction factor for OAB.
//
// Arguments:
//   monthsArdri: Number of months of reduction.
double PiaParamsPL::factorArCal( int monthsArdri ) const
{
   return factorArCalPL(monthsArdri);
}

// Description: Returns full retirement age for benefit calculations involving
//   disability freeze.
//
// Returns: Full retirement age for benefit calculations involving
//   disability freeze.
//
// Arguments:
//   eligYear: Year of eligibility (age 62 for oab or wife/husband, 60 for
//     widow(er)).
//   currentYear: Current year.
Age PiaParamsPL::fullRetAgeCalDI( int eligYear, int currentYear ) const
{
   return fullRetAgeCalDIPL(eligYear, currentYear);
}

// Description: Projects amounts dependent on benefit increases.
void PiaParamsPL::projectCpiinc()
{
   resizeSpecMin();
   projectSpecMin(YEAR79, maxyear);
}

// Description: Updates wage bases for average wage changes.
void PiaParamsPL::updateBases()
{
   const int firstYeart = getIstart() + 1;
   // project present-law OASDI bases
   baseOasdi.project(fq, getCpiincArray(), firstYeart,
      baseOasdi.getLastYear());
   // project HI bases
   baseHi.project(fq, getCpiincArray(), firstYeart, baseHi.getLastYear());
   // project old-law OASDI bases
   base77.project(fq, getCpiincArray(), firstYeart, base77.getLastYear());
}

// Arguments:
//   userbase: User-entered bases.
//   userbase77: User-entered old-law bases.
//   ibasch: Indicator for base changes (1=none, 2=change).
//   firstYeart: First year to update.
//   lastYeart: Last year to update.
void PiaParamsPL::updateBases( const DoubleAnnual& userbase,
const DoubleAnnual& userbase77, int ibasch, int firstYeart, int lastYeart )
{
   if (ibasch == 2) {
      // set ad hoc OASDI bases
      baseOasdi.assign(userbase, firstYeart, lastYeart);
      // project OASDI bases
      baseOasdi.project(fq, getCpiincArray(), lastYeart + 1,
         baseOasdi.getLastYear());
      // set ad hoc old-law bases
      base77.assign(userbase77, firstYeart, lastYeart);
      // project old-law OASDI bases
      base77.project(fq, getCpiincArray(), lastYeart + 1,
         base77.getLastYear());
      // HI bases are always set automatically
      baseHi.project(fq, getCpiincArray(), getIstart() + 1,
         baseHi.getLastYear());
   }
   else {
      updateBases();
   }
}

// Description: Returns maximum age of child for childcare dropout year.
//
// Returns: 3.
int PiaParamsPL::getMaxChildcareAge() const
{
   return getMaxChildcareAgePL();
}

// Description: Sets data from historical average wages and benefit increase
//   file.
//
// Arguments:
//   awbiData: Structure with historical average wages and benefit increases.
void PiaParamsPL::setData( AwbiData& awbiData )
{
   PiaParams::setData(awbiData);
   const int year = awbiData.getLastYear();
   baseOasdi.assign(awbiData.baseOasdi, YEAR37, year);
   base77.assign(awbiData.base77, YEAR37, year);
}

// Description: Returns the number of years of marriage needed to be eligible
//   for a divorced spouse benefit.
//
// Arguments:
//   dateMoyr: The date (year and month) when checking if eligible for
//     divorced spouse benefit.
//
// Returns: Number of years of marriage needed to be eligible
//   for a divorced spouse benefit.
int PiaParamsPL::getMarrLengthForDivBen( const DateMoyr& dateMoyr ) const
{
   return getMarrLengthForDivBenPL(dateMoyr);
}

// Description: Returns widow(er) benefit reduced for RIB-LIM.
//
// Returns: Reduced widow(er) benefit.
//
// Remarks: This should only be called in cases where the deceased worker was
//   an OAB before death.
//
// Arguments:
//   widowBen: Widow(er) benefit reduced for widow(er)'s age, but not for
//     RIB-LIM.
//   oabPia: PIA upon which widow(er) benefit is based.  Note that this is not
//     a re-indexed widow(er) PIA, since the OAB must have survived to age 62.
//   oabBen: Benefit that deceased OAB was receiving.
//   benefitDate: The date of benefit.
double PiaParamsPL::ribLimCal( double widowBen, double oabPia, double oabBen,
const DateMoyr& benefitDate ) const
{
   return ribLimCalPL(widowBen, oabPia, oabBen, benefitDate);
}
