// $Id: piaread.cpp 1.76 2008/04/25 13:01:05EDT 277133 Development  $
//
// Functions to read a case from disk.

#include <stdexcept>
#include <fstream>
#include <iomanip>
#include "piaread.h"
#include "WorkerDataArray.h"
#include "PiadataArray.h"
#include "UserAssumptions.h"
#include "SecondaryArray.h"
#include "awinc.h"
#include "wbgenrl.h"
#include "oactcnst.h"
#include "PiaException.h"
#include "Resource.h"
#include "AssumptionType.h"
#include "DateFormatter.h"

using namespace std;

// Description: Number of columns at beginning of each line reserved for
//   line number.
int PiaRead::width = 2;
// Description: Number of columns for earnings and bases.
int PiaRead::earnWidth = 11;

// Description: Initializes references to data.
//
// Arguments:
//   newWorkerData: Worker's calculation data.
//   newWidowDataArray: Widow and other family members' basic data.
//   newWidowArray: Widow and other family members' calculation data.
//   newUserAssumptions: User-specified assumptions.
//   newSecondaryArray: Secondary benefit information.
PiaRead::PiaRead( WorkerData& newWorkerData,
WorkerDataArray& newWidowDataArray, PiaDataArray& newWidowArray,
UserAssumptions& newUserAssumptions, SecondaryArray& newSecondaryArray ) :
workerData(newWorkerData), widowDataArray(newWidowDataArray),
widowArray(newWidowArray), userAssumptions(newUserAssumptions),
secondaryArray(newSecondaryArray), inputLine(), unusedLine(false), lineType(0)
{ }

// Description: Reads case from file.
//
// Remarks: Call deleteContents before this function
//   if reading more than one case.
//
// Returns:
//
//   0                    - successful read, with another record following.
//   PIA_IDS_READEOF      - successful read, with end of file following.
//   PIA_IDS_READMORE     - successful read, with additional data following
//     (for subclass).
//   Other positive value - unsuccessful read.
//
// Throws: PiaException of type PIA_IDS_READERR if error reading file.
//
// Arguments:
//   in: Input stream.
unsigned int PiaRead::read( std::istream& in )
{
   if (!unusedLine) {
      getline(in, inputLine);
      if (in.eof() || in.fail())
         return(PIA_IDS_READERR);
   }
   if (atoi(inputLine.substr(0, width).c_str()) != 1)
      return(PIA_IDS_READERR);
   parseSsn(inputLine.substr(width));
   // assume first there are no regular earnings, until some are read
   workerData.setIndearn(false);
   workerData.setMqge(false);

   // get next line(s), until a line type of 1
   do {
      getline(in, inputLine);
      if (in.eof() || inputLine.size() == 0) {
         unusedLine = false;
         if (workerData.getJoasdi() == WorkerData::PEBS_CALC) {
            userAssumptions.pebsasmCheck();
            setPebsData();
         }
         // extend user assumptions to end of period, in case benefit date
         // changes
         if (userAssumptions.getIaltbi() == AssumptionType::OTHER_ASSUM) {
            const int year = workerData.getBenefitDate().getYear();
            const double temp = userAssumptions.biproj[year];
            userAssumptions.biproj.assign(temp, year + 1,
               WorkerData::getMaxyear());
         }
         if (userAssumptions.getIaltaw() == AssumptionType::OTHER_ASSUM) {
            const int year = workerData.getBenefitDate().getYear();
            const double temp = userAssumptions.awincproj[year];
            userAssumptions.awincproj.assign(temp, year + 1,
               WorkerData::getMaxyear());
         }
         return(PIA_IDS_READEOF);
      }
      if (in.fail()) {
         unusedLine = false;
         return(PIA_IDS_READERR);
      }
      lineType = atoi(inputLine.substr(0, width).c_str());
      switch (lineType)
      {
      case 1:  // starting new record
         unusedLine = true;
         if (workerData.getJoasdi() == WorkerData::PEBS_CALC) {
            userAssumptions.pebsasmCheck();
            setPebsData();
         }
         return(0);
      case 2:  // date of death
         parseDeath(inputLine.substr(width));
         break;
      case 3:  // type of benefit
         parseTob(inputLine.substr(width));
         break;
      case 4:  // date of benefit
         parseBendate(inputLine.substr(width));
         break;
      case 5:  // pebes information
         parsePebes(inputLine.substr(width));
         break;
      case 6:  // years of earnings
         workerData.setIndearn(true);
         parseYears(inputLine.substr(width));
         break;
      case 7:  // backward projection amounts
         parseBack(inputLine.substr(width));
         break;
      case 8:  // forward projection amounts
         parseFwrd(inputLine.substr(width));
         break;
      case 9:  // disability information for most recent period
         parseDisab1(inputLine.substr(width));
         break;
      case 10:  // disability information for second most recent period
         parseDisab2(inputLine.substr(width));
         break;
      case 11:  // military service dates
         parseMsdates(inputLine.substr(width));
         break;
      case 12:  // noncovered pension
         parsePubpen(inputLine.substr(width));
         break;
      case 13:  // totalization indicator
         parseTotalize(inputLine.substr(width));
         break;
      case 14:  // blind indicator
         parseBlind(inputLine.substr(width));
         break;
      case 15:  // deemed insured indicator
         parseDeemed(inputLine.substr(width));
         break;
      case 16:  // worker's name
         parseNhname(inputLine.substr(width));
         break;
      case 17:  // worker's address
      case 18:
      case 19:
         parseNhaddr(lineType - 17, inputLine.substr(width));
         break;
      case 20:  // annual type of earnings
         parseEarnType(inputLine.substr(width));
         break;
      case 21:  // annual type of taxes
         parseTaxType(inputLine.substr(width));
         break;
      case 22:  // earnings
      case 23:
      case 24:
      case 25:
      case 26:
      case 27:
      case 28:
      case 29:
         parseEarnOasdi(lineType - 22, inputLine.substr(width));
         break;
      case 30:  // Medicare earnings
      case 31:
      case 32:
      case 33:
      case 34:
      case 35:
      case 36:
      case 37:
         workerData.setMqge(true);
         parseEarnHi(lineType - 30, inputLine.substr(width));
         break;
      case 38:  // noncovered pension after military reservist pension removal
         parsePubpenReservist(inputLine.substr(width));
         break;
      case 39:  // oab entitlement before most recent dib
         parseOabent(inputLine.substr(width));
         break;
      case 40:  // assumption indicators
         parseAssump(inputLine.substr(width));
         break;
      case 41:  // projected benefit increases
      case 42:
      case 43:
      case 44:
         parseBi(lineType - 41, inputLine.substr(width));
         break;
      case 45:  // catch-up benefit increases
      case 46:
      case 47:
      case 48:
      case 49:
      case 50:
      case 51:
      case 52:
      case 53:
      case 54:
         parseCatchup(lineType - 45, inputLine.substr(width));
         break;
      case 55:  // title of projected benefit increases
         parseTitlebi(inputLine.substr(width));
         break;
      case 56:
      case 57:
      case 58:
      case 59:
         parseAw(lineType - 56, inputLine.substr(width));
         break;
      case 60:  // title of projected average wage increases
         parseTitleaw(inputLine.substr(width));
         break;
      case 61:  // parse OASDI bases
      case 62:
      case 63:
      case 64:
         parseBases(lineType - 61, inputLine.substr(width));
         break;
      case 65:  // parse old-law bases
      case 66:
      case 67:
      case 68:
         parseBases77(lineType - 65, inputLine.substr(width));
         break;
      case 69:  // family member information
      case 70:
      case 71:
      case 72:
      case 73:
      case 74:
      case 75:
      case 76:
      case 77:
      case 78:
      case 79:
      case 80:
      case 81:
      case 82:
      case 83:
         parseFamilyMember(lineType - 69, inputLine.substr(width));
         break;
      case 84:  // railroad qcs and earnings 1937 to 1950
         workerData.setIndrr(true);
         workerData.railRoadData.parse3750(inputLine.substr(width), earnWidth);
         break;
      case 85:  // first and last years of railroad earnings
         workerData.setIndrr(true);
         workerData.railRoadData.parseYears(inputLine.substr(width));
         break;
      case 86:  // annual quarters of coverage, 1951 to 1977
         workerData.railRoadData.parseQcs(inputLine.substr(width));
         break;
      case 87:  // first decade of earnings
      case 88:  // second decade of earnings
      case 89:  // third decade of earnings
      case 90:  // fourth decade of earnings
      case 91:  // fifth decade of earnings
      case 92:  // sixth decade of earnings
      case 93:  // seventh decade of earnings
      case 94:  // eighth decade of earnings
         workerData.railRoadData.parseEarnings(lineType - 87, 
            inputLine.substr(width), earnWidth);
         break;
      case 95:  // summary quarters of coverage
         parseQc1(inputLine.substr(width));
         break;
      case 96:  // annual quarters of coverage
         parseQc2(inputLine.substr(width));
         break;
      case 97:  // annual child care years
         parseChildCareYears(inputLine.substr(width));
         break;
      default:
         return((lineType > 99) ? PIA_IDS_READMORE : PIA_IDS_READERR);
      }
   } while (lineType != 1);
   // should never reach here, due to return in switch statements
   return(PIA_IDS_READEOF);
}

// Description: Sets data for Social Security Statement.
void PiaRead::setPebsData()
{
   workerData.setPebsData();
}

// Description: Parses Social Security number, sex, and date of birth.
//
// Arguments:
//   line: Line to parse.
void PiaRead::parseSsn( const std::string& line )
{
   try {
      workerData.ssn.setSsnFull(line.substr(0,9));
      workerData.setSex(
         static_cast<Sex::sex_type>(atoi(line.substr(9, 1).c_str())));
      const boost::gregorian::date dateModyyr =
         DateFormatter::date_from_undelimited_us_string(line.substr(10, 8));
      WorkerData::birth2Check(dateModyyr);
      workerData.setBirthDate(dateModyyr);
   } catch (out_of_range&) {
      throw PiaException(PIA_IDS_READERR);
   } catch (PiaException& e) {
      if (e.getNumber() == PIA_IDS_DATEYEAR) {
         throw PiaException(PIA_IDS_BIRTH2);
      }
      if (e.getNumber() == PIA_IDS_DATEMONTH) {
         throw PiaException(PIA_IDS_BIRTH3);
      }
      if (e.getNumber() == PIA_IDS_DATEDAY) {
         throw PiaException(PIA_IDS_BIRTH4);
      }
      throw PiaException(e);
   }
}

// Description: Parses date of death.
//
// Arguments:
//   line: Line to parse.
void PiaRead::parseDeath( const std::string& line )
{
   try {
      boost::gregorian::date dateModyyr;
      // allow for file without day of death, from early versions
      if (line.size() > 6) {
         dateModyyr =
            DateFormatter::date_from_undelimited_us_string(line.substr(0, 8));
      }
      else {
         DateMoyr dateMoyr =
            DateFormatter::month_year_from_undelimited_us_string(
            line.substr(0, 6));
         dateModyyr = boost::gregorian::date(
            static_cast<unsigned short>(dateMoyr.getYear()),
            static_cast<unsigned short>(dateMoyr.getMonth()), 1);
      }
      workerData.setDeathDate(dateModyyr);
   } catch (out_of_range&) {
      throw PiaException(PIA_IDS_READERR);
   }
}

// Description: Parses type of benefit and date of entitlement.
//
// Remarks: The date of entitlement is ignored in a survivor case.  The date of
//   benefit is set equal to date of entitlement. This will be overwritten if
//   there is a line 4.
//
// Arguments:
//   line: Line to parse.
void PiaRead::parseTob( const std::string& line )
{
   try {
      workerData.setJoasdi(atoi(line.substr(0, 1).c_str()));
      // do not use date of entitlement in a survivor case
      if (workerData.getJoasdi() != WorkerData::SURVIVOR) {
         DateMoyr dateMoyr =
            DateFormatter::month_year_from_undelimited_us_string(
            line.substr(1, 6));
         workerData.setEntDate(dateMoyr);
         workerData.setBenefitDate();
      }
   } catch (out_of_range&) {
      throw PiaException(PIA_IDS_READERR);
   }
}

// Description: Parses date of benefit.
//
// Arguments:
//   line: Line to parse.
void PiaRead::parseBendate( const std::string& line )
{
   try {
      workerData.setRecalc(true);
      DateMoyr dateMoyr =
         DateFormatter::month_year_from_undelimited_us_string(
         line.substr(0, 6));
      workerData.setBenefitDate(dateMoyr);
   } catch (out_of_range&) {
      throw PiaException(PIA_IDS_READERR);
   }
}

// Description: Parses Social Security Statement information.
//
// Remarks: This version does nothing.
//
// Arguments:
//   line: Line to parse.
void PiaRead::parsePebes( const std::string& )
{ }

// Description: Parses years of earnings.
//
// Arguments:
//   line: Line to parse.
void PiaRead::parseYears( const std::string& line )
{
   try {
      const int year1 = atoi(line.substr(0, 4).c_str());
      workerData.setIbegin(year1);
      workerData.ibeginCheck();
      const int year2 = atoi(line.substr(4, 4).c_str());
      workerData.setIend(year2);
      setEarnProjectYears(year1, year2);
   } catch (out_of_range&) {
      throw PiaException(PIA_IDS_READERR);
   }
}

// Description: Sets the years in the EarnProject class.
//
// Remarks: This version does nothing.
void PiaRead::setEarnProjectYears( int, int )
{ }

// Description: Parses backward projection amounts.
//
// Remarks: This version does nothing.
void PiaRead::parseBack( const std::string& )
{ }

// Description: Parses forward projection amounts.
//
// Remarks: This version does nothing.
void PiaRead::parseFwrd( const std::string& )
{ }

// Description: Parses disability information for most recent period.
//
// Arguments:
//   line: Line to parse.
void PiaRead::parseDisab1( const std::string& line )
{
   try {
      // fill in a temporary DisabPeriod
      DisabPeriod disabPeriod;
      disabPeriod.parseString(line);
      // set date of onset
      workerData.setOnsetDate(0, disabPeriod.getOnsetDate());
      workerData.setValdi(1);
      workerData.disCheck();
      // set date of prior entitlement
      if (workerData.getJoasdi() == WorkerData::NO_BEN) {
         workerData.setPriorentDate(0, disabPeriod.getEntDate());
      }
      else {
         workerData.setPriorentDateCheck(0, disabPeriod.getEntDate());
         workerData.priorentCheck();
      }
      // set first month of waiting period
      //if (workerData.getJoasdi() == WorkerData::DISABILITY ||
      //    workerData.getJoasdi() == WorkerData::NO_BEN) {
         DisabPeriod::waitperDateCheck(disabPeriod.getWaitperDate());
         workerData.setWaitperDate(0, disabPeriod.getWaitperDate());
         if (workerData.getJoasdi() == WorkerData::DISABILITY)
            workerData.waitpdCheck();
      //}
      if (workerData.getJoasdi() != WorkerData::DISABILITY) {
         // set month and year of disability cessation
         if (disabPeriod.getCessationDate().getYear() != 0 ||
            workerData.getJoasdi() != WorkerData::NO_BEN) {
            DisabPeriod::cessationDateCheck(disabPeriod.getCessationDate());
            workerData.setCessationDate(0, disabPeriod.getCessationDate());
            workerData.setCessationPia(0, disabPeriod.getCessationPia());
            workerData.setCessationMfb(0, disabPeriod.getCessationMfb());
         }
      }
   } catch (out_of_range&) {
      throw PiaException(PIA_IDS_READERR);
   }
}

// Description: Parses disability information for second most recent period.
//
// Arguments:
//   line: Line to parse.
void PiaRead::parseDisab2( const std::string& line )
{
   try {
      // fill in a temporary DisabPeriod
      DisabPeriod disabPeriod;
      disabPeriod.parseString(line);
      // set date of onset
      workerData.setOnsetDate(1, disabPeriod.getOnsetDate());
      workerData.setValdi(2);
      workerData.dis1Check();
      // set date of prior entitlement
      if (workerData.getJoasdi() == WorkerData::NO_BEN) {
         workerData.setPriorentDate(1, disabPeriod.getEntDate());
      }
      else {
         workerData.setPriorentDateCheck(1, disabPeriod.getEntDate());
         workerData.priorent1Check();
      }
      // set first month of prior waiting period
      workerData.setWaitperDate(1, disabPeriod.getWaitperDate());
      // set month and year of prior disability cessation
      if (disabPeriod.getCessationDate().getYear() != 0 ||
         workerData.getJoasdi() != WorkerData::NO_BEN) {
         DisabPeriod::cessationDateCheck(disabPeriod.getCessationDate());
         workerData.setCessationDate(1, disabPeriod.getCessationDate());
         workerData.setCessationPia(1, disabPeriod.getCessationPia());
         workerData.setCessationMfb(1, disabPeriod.getCessationMfb());
      }
      if (workerData.getJoasdi() == WorkerData::DISABILITY) {
         workerData.waitpd1Check();
      }
   } catch (out_of_range&) {
      throw PiaException(PIA_IDS_READERR);
   }
}

// Description: Parses family member's information.
//
// Arguments:
//   lineNumber: Family member number (0 to 14).
//   line: Line to parse.
void PiaRead::parseFamilyMember( int lineNumber, const std::string& line )
{
   try {
      widowArray.setFamSize(lineNumber + 1);
      Secondary * secondaryptr = secondaryArray.secondary[lineNumber];
      WorkerData * widowDataptr = widowDataArray.workerData[lineNumber];
      // parse bic code
      secondaryptr->bic.set(line.substr(0,2));
      // parse date of birth
      boost::gregorian::date dateModyyr =
         DateFormatter::date_from_undelimited_us_string(line.substr(2, 8));
      WorkerData::birth2Check(dateModyyr);
      widowDataptr->setBirthDate(dateModyyr);
      // parse date of entitlement
      DateMoyr dateMoyr =
         DateFormatter::month_year_from_undelimited_us_string(line.substr(10, 6));
      secondaryptr->entDate = dateMoyr;
      // parse widow(er)'s date of onset
      if (secondaryptr->bic.getMajorBic() == 'W') {
         dateModyyr =
            DateFormatter::date_from_undelimited_us_string(line.substr(16, 8));
         widowDataptr->setOnsetDate(0, dateModyyr);
      }
   } catch (out_of_range&) {
      throw PiaException(PIA_IDS_READERR);
   }
}

// Description: Parses noncovered pension.
//
// Arguments:
//   line: Line to parse.
void PiaRead::parsePubpen( const std::string& line )
{
   try {
      workerData.setPubpen(static_cast<float>(atof(line.substr(0, 10).c_str())));
      if (line.length() == 16) {
         DateMoyr dateMoyr =
            DateFormatter::month_year_from_undelimited_us_string(
            line.substr(10, 6));
         workerData.setPubpenDate(dateMoyr);
      }
   } catch (out_of_range&) {
      throw PiaException(PIA_IDS_READERR);
   }
}

// Description: Parses noncovered pension after military reservist pension
//   removal.
//
// Arguments:
//   line: Line to parse.
void PiaRead::parsePubpenReservist( const std::string& line )
{
   workerData.setReservist(true);
   workerData.setPubpenReservist(
      static_cast<float>(atof(line.substr(0, 10).c_str())));
}

// Description: Parses totalization indicator.
//
// Arguments:
//   line: Line to parse.
void PiaRead::parseTotalize( const std::string& line )
{
   workerData.setTotalize(
      (atoi(line.substr(0, 1).c_str()) > 0) ? true : false);
}

// Description: Parses blind indicator.
//
// Arguments:
//   line: Line to parse.
void PiaRead::parseBlind( const std::string& line )
{
   workerData.setBlindind(
      (atoi(line.substr(0, 1).c_str()) > 0) ? true : false);
}

// Description: Parses deemed insured indicator.
//
// Arguments:
//   line: Line to parse.
void PiaRead::parseDeemed( const std::string& line )
{
   workerData.setDeemedind(
      (atoi(line.substr(0, 1).c_str()) > 0) ? true : false);
}

// Description: Parses worker's name.
//
// Arguments:
//   line: Line to parse.
void PiaRead::parseNhname( const std::string& line )
{
   workerData.setNhname(line);
}

// Description: Parses worker's address.
//
// Arguments:
//   linenumber: Address line number (0-2).
//   line: Line to parse.
void PiaRead::parseNhaddr( int linenumber, const std::string& line )
{
   workerData.setNhaddr(linenumber, line);
}

// Description: Parses up to 20 projected benefit increases.
//
// Arguments:
//   lineNumber: Number of increase line (0-3)
//   line: Line to parse.
void PiaRead::parseBi( int lineNumber, const std::string& line )
{
   try {
      userAssumptions.istartCheck(istart2);
      const int firstyear = istart2 + 20 * lineNumber;
      const int lastyear = min(firstyear + 19,
         static_cast<int>(workerData.getBenefitDate().getYear()));
      for (int i1 = firstyear; i1 <= lastyear; i1++) {
         float tempf = static_cast<float>(atof(
            line.substr(4 * (i1 - firstyear), 4).c_str()));
         Biproj::cpiincCheck(tempf);
         userAssumptions.biproj[i1] = tempf;
      }
   } catch (out_of_range&) {
      throw PiaException(PIA_IDS_READERR);
   }
}

// Description: Parses 8 catch-up benefit increases.
//
// Arguments:
//   lineNumber: Line number of increases (0-9).
//   line: Line to parse.
void PiaRead::parseCatchup( int lineNumber, const std::string& line )
{
   try {
      CatchupIncreases catchupIncreases;
      catchupIncreases.parseString(line.substr(4));
      const int cstart = userAssumptions.catchup.getCstart();
      for (int i1 = 0; i1 < 8; i1++) {
         userAssumptions.catchup.set(cstart + lineNumber,
            cstart + 2 + i1, catchupIncreases.get(i1));
      }
      userAssumptions.setAnscch('Y');
   } catch (out_of_range&) {
      throw PiaException(PIA_IDS_READERR);
   }
}

// Description: Parses title of projected benefit increases.
//
// Arguments:
//   line: Line to parse.
void PiaRead::parseTitlebi( const std::string& line )
{
   userAssumptions.setTitleBi(line);
}

// Description: Parses title of projected average wage increases.
//
// Arguments:
//   line: Line to parse.
void PiaRead::parseTitleaw( const std::string& line )
{
   userAssumptions.setTitleAw(line);
}

// Description: Parses projected average wage increases.
//
// Arguments:
//   lineNumber: Number of increase line (0-3).
//   line: Line to parse.
void PiaRead::parseAw( int lineNumber, const std::string& line )
{
   try {
      userAssumptions.istartCheck(istart2 - 1);
      const int firstyear = istart2 - 1 + 20 * lineNumber;
      const int lastyear = min(firstyear + 19,
         static_cast<int>(workerData.getBenefitDate().getYear()));
      for (int i1 = firstyear; i1 <= lastyear; i1++) {
         double tempd = atof(line.substr(6 * (i1 - firstyear), 6).c_str());
         Awinc::fqincCheck(tempd);
         userAssumptions.awincproj[i1] = tempd;
      }
   } catch (out_of_range&) {
      throw PiaException(PIA_IDS_READERR);
   }
}

// Description: Parses type of earnings vector.
//
// Remarks: This version does nothing.
void PiaRead::parseEarnType( const std::string& )
{ }

// Description: Parses OASDI-covered earnings for up to 10 years, without
// projection.
//
// Arguments:
//   lineNumber: Number of earnings line (0-7).
//   line: Line to parse.
void PiaRead::parseEarnOasdi( int lineNumber, const std::string& line )
{
   const int firstYear = workerData.getIbegin() + 10 * lineNumber;
   const int lastYear = min(firstYear + 9, workerData.getIend());
   try {
      for (int i1 = firstYear; i1 <= lastYear; i1++) {
         workerData.setEarnOasdi(i1, atof(
            line.substr(earnWidth * (i1 - firstYear), earnWidth).c_str()));
      }
   } catch (out_of_range&) {
      throw PiaException(PIA_IDS_READERR);
   }
   if ((int)line.length() < earnWidth * (lastYear - firstYear + 1)) {
      throw PiaException(PIA_IDS_READERR);
   }
}

// Description: Parses assumption indicators.
//
// Arguments:
//   line: Line to parse.
void PiaRead::parseAssump( const std::string& line )
{
   try {
      istart2 = atoi(line.substr(0, 4).c_str());
      userAssumptions.istartCheck(istart2);
      userAssumptions.setIaltbi(atoi(line.substr(4, 1).c_str()));
      userAssumptions.setIaltaw(atoi(line.substr(5, 1).c_str()));
      // parse base change indicator
      userAssumptions.setIbasch(atoi(line.substr(6, 1).c_str()));
   } catch (out_of_range&) {
      throw PiaException(PIA_IDS_READERR);
   }
}

// Description: Parses up to 20 OASDI wage bases.
//
// Arguments:
//   lineNumber: Number of earnings line (0-3).
//   line: Line to parse.
void PiaRead::parseBases( int lineNumber, const std::string& line )
{
   try {
      userAssumptions.istartCheck(istart2 + 1);
      userAssumptions.setJbasch(BaseChangeType::NONE);
      const int firstyear = istart2 + 1 + 20 * lineNumber;
      for (int i1 = firstyear; i1 <= min(firstyear + 19,
         static_cast<int>(workerData.getBenefitDate().getYear())); i1++) {
         const double temp =
            atof(line.substr(earnWidth * (i1 - firstyear), earnWidth).c_str());
         WageBaseGeneral::check(temp);
         userAssumptions.baseOasdi[i1] = temp;
      }
   } catch (out_of_range&) {
      throw PiaException(PIA_IDS_READERR);
   }
}

// Description: Parses up to 20 old-law wage bases.
//
// Arguments:
//   lineNumber: Number of earnings line (0-3).
//   line: Line to parse.
void PiaRead::parseBases77( int lineNumber, const std::string& line )
{
   try {
      userAssumptions.istartCheck(istart2 + 1);
      userAssumptions.setJbasch(BaseChangeType::NONE);
      const int firstyear = istart2 + 1 + 20 * lineNumber;
      for (int i1 = firstyear; i1 <= min(firstyear + 19,
         static_cast<int>(workerData.getBenefitDate().getYear())); i1++) {
         const double temp =
            atof(line.substr(earnWidth * (i1 - firstyear), earnWidth).c_str());
         WageBaseGeneral::check(temp);
         userAssumptions.base77[i1] = temp;
      }
   } catch (out_of_range&) {
      throw PiaException(PIA_IDS_READERR);
   }
}

// Description: Parses up to 10 years of Medicare earnings.
//
// Arguments:
//   lineNumber: Number of earnings line (0-7).
//   line: Line to parse.
void PiaRead::parseEarnHi( int lineNumber, const std::string& line )
{
   try {
      const int firstyear = workerData.firstEarnHiYear() + 10 * lineNumber;
      const int lastyear = min(firstyear + 9, workerData.getIend());
      for (int i1 = firstyear; i1 <= lastyear; i1++) {
         workerData.setEarnHi(i1, atof(line.substr(earnWidth *
            (i1 - firstyear), earnWidth).c_str()));
      }
      workerData.setMqge(true);
   } catch (out_of_range&) {
      throw PiaException(PIA_IDS_READERR);
   }
}

// Description: Parses type of taxes.
//
// Arguments:
//   line: Line to parse.
void PiaRead::parseTaxType( const std::string& line )
{
   try {
      const int firstyear = workerData.getIbegin();
      const int lastyear = workerData.getIend();
      for (int i1 = firstyear; i1 <= lastyear; i1++) {
         workerData.setTaxType(i1,
            atoi(line.substr(i1 - firstyear, 1).c_str()));
      }
   } catch (out_of_range&) {
      throw PiaException(PIA_IDS_READERR);
   }
}

// Description: Parses summary quarters of coverage.
//
// Arguments:
//   line: Line to parse.
void PiaRead::parseQc1( const std::string& line )
{
   try {
      workerData.setQctd(atoi(line.substr(0, 3).c_str()));
      workerData.setQc51td(atoi(line.substr(3, 3).c_str()));
      workerData.qctdCheck2();
   } catch (out_of_range&) {
      throw PiaException(PIA_IDS_READERR);
   }
}

// Description: Parses annual quarters of coverage.
//
// Arguments:
//   line: Line to parse.
void PiaRead::parseQc2( const std::string& line )
{
   try {
      workerData.setQcsByYear(true);
      const int firstyear = workerData.getIbegin();
      const int lastyear = workerData.lastQcyr();
      for (int i1 = firstyear; i1 <= lastyear; i1++) {
         workerData.qc.set(i1, static_cast<unsigned>(atoi(
            line.substr(i1 - firstyear, 1).c_str())));
      }
      workerData.setQctd(workerData.qc.accumulate(YEAR37, lastyear, 0u));
      workerData.setQc51td(workerData.qc.accumulate(YEAR51, lastyear, 0u));
   } catch (out_of_range&) {
      throw PiaException(PIA_IDS_READERR);
   }
}

// Description: Parses military service dates.
//
// Arguments:
//   line: Line to parse.
void PiaRead::parseMsdates( const std::string& line )
{
   try {
      // there are 12 characters per military service date pair
      workerData.milServDatesVec.setMSCount(
         static_cast<unsigned int>(line.size() / 12));
      for (unsigned i = 0; i < workerData.milServDatesVec.getMSCount(); i++) {
         // first check dates in a temporary military service dates
         MilServDates msDates;
         msDates.parseDates(line.substr(12 * i));
         MilServDates::startDateCheck(msDates.startDate);
         MilServDates::endDateCheck(msDates.endDate);
         msDates.check();
         // if ok (no exception thrown), put into permanent dates
         workerData.milServDatesVec.msdates[i].parseDates(line.substr(12 * i));
      }
   } catch (out_of_range&) {
      throw PiaException(PIA_IDS_READERR);
   }
}

// Description: Parses oab entitlement prior to most recent disability.
//
// Arguments:
//   line: Line to parse.
void PiaRead::parseOabent( const std::string& line )
{
   try {
      // parse date of oab entitlement
      DateMoyr dateMoyr =
         DateFormatter::month_year_from_undelimited_us_string(
         line.substr(0, 6));
      workerData.setOabEntDate(dateMoyr);
      // parse month and year of oab cessation
      dateMoyr = DateFormatter::month_year_from_undelimited_us_string(
         line.substr(6, 6));
      workerData.setOabCessDate(dateMoyr);
   } catch (out_of_range&) {
      throw PiaException(PIA_IDS_READERR);
   }
}

// Description: Parses annual child care years.
//
// Arguments:
//   line: Line to parse.
void PiaRead::parseChildCareYears( const std::string& line )
{
   try {
      const int firstyear = workerData.getIbegin();
      const int lastyear = workerData.getIend();
      for (int i1 = firstyear; i1 <= lastyear; i1++) {
         workerData.childCareYears.setBit(i1, 
            atoi(line.substr(i1 - firstyear, 1).c_str()));
      }
   } catch (out_of_range&) {
      throw PiaException(PIA_IDS_READERR);
   }
}
