// $Id: piareadAny.cpp 1.2 2006/03/15 10:27:50EST 277133 Development  $
//
// Functions to read a case from disk.

#include <stdexcept>
#include <fstream>
#include <iomanip>
#include "piareadAny.h"
#include "WorkerDataArray.h"
#include "PiadataArray.h"
#include "UserAssumptions.h"
#include "SecondaryArray.h"
#include "EarnProject.h"
#include "pebs.h"
#include "awinc.h"
#include "wbgenrl.h"
#include "oactcnst.h"
#include "PiaException.h"
#include "Resource.h"
#include "AssumptionType.h"

using namespace std;

// Description: Initializes references to data.
//
// Arguments:
//   newWorkerData: Worker's calculation data.
//   newWidowDataArray: Widow and other family members' basic data.
//   newWidowArray: Widow and other family members' calculation data.
//   newUserAssumptions: User-specified assumptions.
//   newSecondaryArray: Secondary benefit information.
//   newEarnProject: Projected earnings.
//   newPebs: Statement information.
PiaReadAny::PiaReadAny( WorkerData& newWorkerData,
WorkerDataArray& newWidowDataArray, PiaDataArray& newWidowArray,
UserAssumptions& newUserAssumptions, SecondaryArray& newSecondaryArray,
EarnProject& newEarnProject, Pebs& newPebs ) :
PiaRead(newWorkerData, newWidowDataArray, newWidowArray, newUserAssumptions,
newSecondaryArray), earnProject(newEarnProject), pebs(newPebs)
{ }

// Description: Sets data for Social Security Statement.
void PiaReadAny::setPebsData()
{
   earnProject.setPebsData(UserAssumptions::getIstart(),
      workerData.getIbegin());
   workerData.setPebsData();
}

// Description: Parses Social Security Statement information.
//
// Arguments:
//   line: Line to parse.
void PiaReadAny::parsePebes( const std::string& line )
{
   try {
      pebs.setMonthnow(atoi(line.substr(0, 2).c_str()));
      pebs.setAgePlan(atoi(line.substr(2, 2).c_str()));
   } catch (out_of_range&) {
      throw PiaException(PIA_IDS_READERR);
   }
}

// Description: Sets the years in the EarnProject class.
//
// Arguments:
//   year1: The first year of earnings.
//   year2: The last year of earnings.
void PiaReadAny::setEarnProjectYears( int year1, int year2 )
{
   earnProject.setFirstYear(year1);
   earnProject.setLastYear(year2);
   earnProject.iendCheck(workerData.getIbegin(), workerData.getIend());
}

// Description: Parses backward projection amounts.
//
// Arguments:
//   line: Line to parse.
void PiaReadAny::parseBack( const std::string& line )
{
   try {
      earnProject.setProjback(atoi(line.substr(0, 1).c_str()));
      earnProject.setPercback(
         static_cast<float>(atof(line.substr(1, 6).c_str())));
      const int ibegint = atoi(line.substr(7, 4).c_str());
      EarnProject::ibegin2Check(ibegint);
      earnProject.setFirstYear(ibegint);
   } catch (out_of_range&) {
      throw PiaException(PIA_IDS_READERR);
   }
}

// Description: Parses forward projection amounts.
//
// Arguments:
//   line: Line to parse.
void PiaReadAny::parseFwrd( const std::string& line )
{
   try {
      earnProject.setProjfwrd(atoi(line.substr(0, 1).c_str()));
      earnProject.setPercfwrd(
         static_cast<float>(atof(line.substr(1, 6).c_str())));
      const int iendt = atoi(line.substr(7, 4).c_str());
      EarnProject::iendCheck(iendt);
      earnProject.setLastYear(iendt);
   } catch (out_of_range&) {
      throw PiaException(PIA_IDS_READERR);
   }
}

// Description: Parses type of earnings vector.
//
// Arguments:
//   line: Line to parse.
void PiaReadAny::parseEarnType( const std::string& line )
{
   try {
      const int firstYear = earnProject.getFirstYear();
      const int lastYear = earnProject.getLastYear();
      for (int i1 = firstYear; i1 <= lastYear; i1++) {
         earnProject.setEarntype(i1,
            atoi(line.substr(i1 - firstYear, 1).c_str()));
      }
   } catch (out_of_range&) {
      throw PiaException(PIA_IDS_READERR);
   }
}

// Description: Parses earnings for up to 10 years, with projection.
//
// Arguments:
//   lineNumber: Number of earnings line (0-7).
//   line: Line to parse.
void PiaReadAny::parseEarnOasdi( int lineNumber, const std::string& line )
{
   const int firstYear = earnProject.getFirstYear() + 10 * lineNumber;
   const int lastYear = min(firstYear + 9, earnProject.getLastYear());
   try {
      for (int i1 = firstYear; i1 <= lastYear; i1++) {
         earnProject.setEarnpebs(i1, atof(line.substr(earnWidth *
            (i1 - firstYear), earnWidth).c_str()));
      }
   } catch (out_of_range&) {
      throw PiaException(PIA_IDS_READERR);
   }
   if ((int)line.length() < earnWidth * (lastYear - firstYear + 1)) {
      throw PiaException(PIA_IDS_READERR);
   }
}
