// $Id: piawrite.cpp 1.70 2007/03/09 17:09:34EST 044579 Development  $
//
// Functions to save a case to disk.

#include <fstream>
#include <iomanip>
#include "piawrite.h"
#include "WorkerDataArray.h"
#include "PiadataArray.h"
#include "UserAssumptions.h"
#include "SecondaryArray.h"
#include "awinc.h"
#include "comma.h"
#include "PiaException.h"
#include "AssumptionType.h"
#include "oactcnst.h"
#include "DateFormatter.h"

using namespace std;

// Description: Number of columns at beginning of each line reserved for
//   line number.
int PiaWrite::width = 2;
// Description: Number of columns for earnings and bases.
int PiaWrite::earnWidth = 11;
// Description: Number of decimal places for earnings and bases.
int PiaWrite::numDecimals = 2;

// Description: Initializes references to data.
//
// Arguments:
//   newWorkerData: Worker's calculation data.
//   newWidowDataArray: Widow and other family members' basic data.
//   newWidowArray: Widow and other family members' calculation data.
//   newUserAssumptions: User-specified assumptions.
//   newSecondaryArray: Secondary benefit information.
PiaWrite::PiaWrite( const WorkerData& newWorkerData,
const WorkerDataArray& newWidowDataArray, const PiaDataArray& newWidowArray,
const UserAssumptions& newUserAssumptions,
const SecondaryArray& newSecondaryArray ) :
workerData(newWorkerData), widowDataArray(newWidowDataArray),
widowArray(newWidowArray), userAssumptions(newUserAssumptions),
secondaryArray(newSecondaryArray)
{ }

// Description: Saves a case to disk.
//
// Arguments:
//   out: Output stream.
void PiaWrite::write( std::ostream& out )
{
   writeSsn(out);
   writeDeath(out);
   writeTob(out);
   writeBenDate(out);
   writePebes(out);
   writeYears(out);
   writeBack(out);
   writeFwrd(out);
   writeDisab1(out);
   writeDisab2(out);
   writeMsdates(out);
   writePubpen(out);
   writeTotalize(out);
   writeBlind(out);
   writeDeemed(out);
   writeNhname(out);
   writeNhaddr(out);
   writeEarnType(out);
   writeTaxType(out);
   writeEarnOasdi(out);
   writeEarnHi(out);
   writePubpenReservist(out);
   writeOabEnt(out);
   writeAssump(out);
   writeBi(out);
   writeAw(out);
   writeBases(out);
   writeFamilyMembers(out);
   writeRailRoad(out);
   writeQc1(out);
   writeQc2(out);
   writeChildCareYears(out);
}

// Description: Writes Social Security number, sex, and date of birth.
//
// Arguments:
//   out: Output stream.
void PiaWrite::writeSsn( std::ostream& out ) const
{
   out << poutNoComma(1, width, 0, true)
       << workerData.ssn.toString("n")
       << static_cast<int>(workerData.getSex());
   out << DateFormatter::toString(workerData.getBirthDate(), "n") << endl;
}

// Description: Writes date of death.
//
// Arguments:
//   out: Output stream.
void PiaWrite::writeDeath( std::ostream& out ) const
{
   if (!workerData.getDeathDate().is_not_a_date()) {
      out << poutNoComma(2, width, 0, true)
          << DateFormatter::toString(workerData.getDeathDate(), "n") << endl;
   }
}

// Description: Writes type of benefit and date of entitlement.
//
// Remarks: Only written if there is a benefit.
//
// Arguments:
//   out: Output stream.
void PiaWrite::writeTob( std::ostream& out ) const
{
   if (static_cast<int>(workerData.getJoasdi()) > 0) {
      out << setfill('0') << setw(width) << 3 << setfill(' ')
          << static_cast<int>(workerData.getJoasdi())
          << DateFormatter::toString(workerData.getEntDate(), "n") << endl;
   }
}

// Description: Writes date of benefit.
//
// Remarks: Only written if this is a recalculation, or a survivor benefit.
//
// Arguments:
//   out: Output stream.
void PiaWrite::writeBenDate( std::ostream& out ) const
{
   if (workerData.getRecalc()
      || (workerData.getJoasdi() == WorkerData::SURVIVOR)) {
      out << setfill('0') << setw(width) << 4 << setfill(' ')
          << DateFormatter::toString(workerData.getBenefitDate(), "n") << endl;
   }
}

// Description: Writes pebes information.
//
// Remarks: This version does nothing.
void PiaWrite::writePebes( std::ostream& ) const
{ }

// Description: Writes years of earnings.
//
// Arguments:
//   out: Output stream.
void PiaWrite::writeYears( std::ostream& out ) const
{
   // do not try to save if no earnings have been set
   if (workerData.getIndearn()) {
      try {
         workerData.ibeginCheck();
      } catch (PiaException&) {
         return;
      }
   }
   else
      return;
   out << poutNoComma(6, width, 0, true);
   out << setw(4) << workerData.getIbegin()
       << setw(4) << workerData.getIend() << endl;
}

// Description: Writes backward projection amounts.
//
// Remarks: This version does nothing.
void PiaWrite::writeBack( std::ostream& ) const
{ }

// Description: Writes forward projection amounts.
//
// Remarks: This version does nothing.
void PiaWrite::writeFwrd( std::ostream& ) const
{ }

// Description: Writes disability information for most recent period.
//
// Remarks: Does not write anything unless there is a period of disability.
// Does not write the date of cessation for a current period of disability
// (the type of benefit is WorkerDataGeneral::DISABILITY).
//
// Arguments:
//   out: Output stream.
void PiaWrite::writeDisab1( std::ostream& out ) const
{
   if (workerData.getValdi() > 0) {
      string format((workerData.getJoasdi() == WorkerData::DISABILITY) ?
         "c" : "");
      out << poutNoComma(9, width, 0, true)
          << workerData.disabPeriod[0].toString(format)
          << endl;
   }
}

// Description: Writes disability information for second most recent period.
//
// Arguments:
//   out: Output stream.
void PiaWrite::writeDisab2( std::ostream& out ) const
{
   if (workerData.getValdi() > 1) {
      out << poutNoComma(10, width, 0, true)
          << workerData.disabPeriod[1].toString()
          << endl;
   }
}

// Description: Writes information for all family members.
//
// Arguments:
//   out: Output stream.
void PiaWrite::writeFamilyMembers( std::ostream& out ) const
{
   for (int i = 0; i < widowArray.getFamSize(); i++) {
      writeFamilyMember(out, i);
   }
}

// Description: Writes information for one family member.
//
// Arguments:
//   out: Output stream.
//   i: Number of family member (0 to 14).
void PiaWrite::writeFamilyMember( std::ostream& out, int i ) const
{
   Secondary * secondaryptr = secondaryArray.secondary[i];
   WorkerData * widowDataptr = widowDataArray.workerData[i];
   out << setfill('0') << setw(width) << 69 + i << setfill(' ')
       << secondaryptr->bic.toString();
   // write date of birth
   out << DateFormatter::toString(widowDataptr->getBirthDate(), "n");
   // write date of entitlement
   out << DateFormatter::toString(secondaryptr->entDate, "n");
   if (secondaryptr->bic.getMajorBic() == 'W') {
      out << DateFormatter::toString(
         widowDataptr->disabPeriod[0].getOnsetDate(), "n");
   }
   out << endl;
}

// Description: Writes noncovered pension.
//
// Remarks: Writes the non-covered pension amount if it is greater than zero.
// Also writes the pension start date if it exists.
//
// Arguments:
//   out: Output stream.
void PiaWrite::writePubpen( std::ostream& out ) const
{
   if (workerData.getPubpen() > 0.0f) {
      out << setfill('0') << setw(width) << 12 << setfill(' ')
          << poutNoComma(workerData.getPubpen(), 10, 2);
      if (workerData.getPubpenDate().getYear() > 0) {
         out << DateFormatter::toString(workerData.getPubpenDate(), "n");
      }
      out << endl;
   }
}

// Description: Writes noncovered pension after military reservist pension
//   removal.
//
// Arguments:
//   out: Output stream.
void PiaWrite::writePubpenReservist( std::ostream& out ) const
{
   if (workerData.getReservist()) {
      out << poutNoComma(38, width, 0, true);
      out << poutNoComma(workerData.getPubpenReservist(), 10, 2) << endl;
   }
}

// Description: Writes totalization indicator.
//
// Arguments:
//   out: Output stream.
void PiaWrite::writeTotalize( std::ostream& out ) const
{
   if (workerData.getTotalize()) {
      out << poutNoComma(13, width, 0, true);
      out << workerData.getTotalize() << endl;
   }
}

// Description: Writes blind indicator.
//
// Arguments:
//   out: Output stream.
void PiaWrite::writeBlind( std::ostream& out ) const
{
   if (workerData.getBlindind()) {
      out << poutNoComma(14, width, 0, true);
      out << workerData.getBlindind() << endl;
   }
}

// Description: Writes deemed insured indicator.
//
// Arguments:
//   out: Output stream.
void PiaWrite::writeDeemed( std::ostream& out ) const
{
   if (workerData.getDeemedind()) {
      out << poutNoComma(15, width, 0, true);
      out << workerData.getDeemedind() << endl;
   }
}

// Description: Writes name of worker.
//
// Arguments:
//   out: Output stream.
void PiaWrite::writeNhname( std::ostream& out ) const
{
   if (workerData.getNhname().length() > 0) {
      out << poutNoComma(16, width, 0, true);
      out << workerData.getNhname() << endl;
   }
}

// Description: Writes address of worker.
//
// Arguments:
//   out: Output stream.
void PiaWrite::writeNhaddr( std::ostream& out ) const
{
   for (int i1 = 0; i1 < 3; i1++) {
      if (workerData.getNhaddr(i1).length() > 0) {
         out << poutNoComma(17 + i1, width, 0, true);
         out << workerData.getNhaddr(i1) << endl;
      }
   }
}

// Description: Writes type of earnings for all years.
//
// Remarks: This version does nothing.
void PiaWrite::writeEarnType( std::ostream& ) const
{ }

// Description: Writes type of taxes for all years.
//
// Arguments:
//   out: Output stream.
void PiaWrite::writeTaxType( std::ostream& out ) const
{
   // do not try to save if no earnings have been set
   if (workerData.getIndearn()) {
      try {
         workerData.ibeginCheck();
      } catch (PiaException&) {
         return;
      }
   }
   else {
      return;
   }
   if (workerData.anySelfEmp()) {
      out << poutNoComma(21, width, 0, true);
      for (int i1 = workerData.getIbegin(); i1 <= workerData.getIend(); i1++)
         out << static_cast<int>(workerData.getTaxType(i1));
      out << endl;
   }
}

// Description: Writes OASDI-covered earnings for all years, using unprojected
// earnings array.
//
// Throws: PiaException of type PIA_IDS_WRITEEARN if first year of earnings is
//   out of range (only in debug mode).
//
// Arguments:
//   out: Output stream.
void PiaWrite::writeEarnOasdi( std::ostream& out ) const
{
   // do not try to save if no earnings have been set
   if (workerData.getIndearn()) {
      try {
         workerData.ibeginCheck();
      } catch (PiaException&) {
         return;
      }
   }
   else {
      return;
   }
#if !defined(NDEBUG)
   if (workerData.getIbegin() < YEAR37)
      throw PiaException(PIA_IDS_WRITEEARN);
#endif
   out.precision(numDecimals);
   const int numrows =
      ((workerData.getIend() - workerData.getIbegin()) / 10) + 1;
   for (int i2 = 0; i2 < numrows; i2++) {
      out << poutNoComma(22 + i2, width, 0, true);
      const int firstyear = workerData.getIbegin() + 10 * i2;
      const int lastyear = min(workerData.getIend(), firstyear + 9);
      for (int i1 = firstyear; i1 <= lastyear; i1++) {
         out << setw(earnWidth) << workerData.getEarnOasdi(i1);
      }
      out << endl;
   }
}

// Description: Writes summary quarters of coverage.
//
// Arguments:
//   out: Output stream.
void PiaWrite::writeQc1( std::ostream& out ) const
{
   if (workerData.getQctottd() > 0) {
      out << poutNoComma(95, width, 0, true);
      out << setw(3) << workerData.getQctottd()
          << setw(3) << workerData.getQctot51td() << endl;
   }
}

// Description: Writes annual quarters of coverage.
//
// Arguments:
//   out: Output stream.
void PiaWrite::writeQc2( std::ostream& out ) const
{
   // do not try to save if no earnings have been set
   if (workerData.getIndearn()) {
      try {
         workerData.ibeginCheck();
      } catch (PiaException&) {
         return;
      }
   }
   else
      return;
   const int firstyear = workerData.getIbegin();
   const int lastyear = workerData.lastQcyr();
   if (workerData.hasQcsByYear() && lastyear >= firstyear) {
      out << poutNoComma(96, width, 0, true)
         << workerData.qc.toString(firstyear, lastyear) << endl;
   }
}

// Description: Writes Medicare earnings.
//
// Arguments:
//   out: Output stream.
void PiaWrite::writeEarnHi( std::ostream& out ) const
{
   // Don't write anything if there are no earnings.
   if (workerData.getIndearn()) {
      try {
         workerData.ibeginCheck();
      } catch (PiaException&) {
         return;
      }
   } else {
      return;
   }
   if (workerData.getMqge()) {
      const int firstYear1 = workerData.firstEarnHiYear();
      out.precision(numDecimals);
      const int numrows = ((workerData.getIend() - firstYear1) / 10) + 1;
      for (int i2 = 0; i2 < numrows; i2++) {
         out << poutNoComma(30 + i2, width, 0, true);
         const int firstyear2 = firstYear1 + 10 * i2;
         const int lastyear = min(workerData.getIend(), firstyear2 + 9);
         for (int i1 = firstyear2; i1 <= lastyear; i1++) {
            out << setw(earnWidth) << workerData.getEarnHi(i1);
         }
         out << endl;
      }
   }
}

// Description: Writes assumption indicators.
//
// Remarks: Assumption indicators are written only if this is a case that needs
// assumptions (a benefit in the future).
//
// Arguments:
//   out: Output stream.
void PiaWrite::writeAssump( std::ostream& out ) const
{
   if (workerData.need_aw(UserAssumptions::getIstart(), false) ||
      workerData.need_bi(UserAssumptions::getIstart()) ||
      userAssumptions.getIbasch() == BaseChangeType::ADHOC) {
      out << poutNoComma(40, width, 0, true);
      out << setw(4) << userAssumptions.getIstart()
         << static_cast<int>(userAssumptions.getIaltbi())
         << static_cast<int>(userAssumptions.getIaltaw())
         << userAssumptions.getIbasch() << endl;
   }
}

// Description: Writes benefit increase assumptions.
//
// Remarks: Benefit increase assumptions are written if they are not one
// of the stored sets of assumptions.
//
// Arguments:
//   out: Output stream.
void PiaWrite::writeBi( std::ostream& out ) const
{
   if (userAssumptions.getIaltbi() == AssumptionType::OTHER_ASSUM) {
      out.precision(1);
      const int i3 = static_cast<int>(workerData.getBenefitDate().getYear());
      const int numrows = ((i3 - UserAssumptions::getIstart()) / 20) + 1;
      for (int i2 = 0; i2 < numrows; i2++) {
         out << poutNoComma(41 + i2, width, 0, true);
         const int firstyear = UserAssumptions::getIstart() + 20 * i2;
         for (int i1 = firstyear; i1 <= min(i3, firstyear + 19); i1++) {
            out << setw(4) << userAssumptions.biproj[i1];
         }
         out << endl;
      }
      if (userAssumptions.getAnscch() == 'Y') {
         for (int i1 = 0; i1 < Catchup::NUM_CATCHUP_ELIG_YEARS; i1++) {
            out << poutNoComma(45 + i1, width, 0, true)
               << userAssumptions.catchup.getCatchupIncreases(i1).toString();
         }
      }
      out << poutNoComma(55, width, 0, true);
      out << userAssumptions.titleBi << endl;
   }
}

// Description: Writes average wage increase assumptions.
//
// Remarks: Average wage increase assumptions are written if they are not one
// of the stored sets of assumptions.
//
// Arguments:
//   out: Output stream.
void PiaWrite::writeAw( std::ostream& out ) const
{
   if (userAssumptions.getIaltaw() == AssumptionType::OTHER_ASSUM) {
      const int i3 = static_cast<int>(workerData.getBenefitDate().getYear());
      out.precision(2);
      const int numrows = ((i3 - UserAssumptions::getIstart() + 1) / 20) + 1;
      for (int i2 = 0; i2 < numrows; i2++) {
         out << poutNoComma(56 + i2, width, 0, true);
         const int firstyear = UserAssumptions::getIstart() - 1 + 20 * i2;
         for (int i1 = firstyear; i1 <= min(i3, firstyear + 19); i1++) {
            out << setw(6) << userAssumptions.awincproj[i1];
         }
         out << endl;
      }
      out << poutNoComma(60, width, 0, true);
      out << userAssumptions.titleAw << endl;
   }
}

// Description: Writes wage base assumptions.
//
// Remarks: Projected wage bases are written if they are ad hoc.
//
// Arguments:
//   out: Output stream.
void PiaWrite::writeBases( std::ostream& out ) const
{
   if (userAssumptions.getIbasch() == BaseChangeType::ADHOC) {
      out.precision(numDecimals);
      const int i3 = userAssumptions.baseOasdi.getLastYear();
      const int numrows = ((i3 - UserAssumptions::getIstart() - 1) / 20) + 1;
      for (int i2 = 0; i2 < numrows; i2++) {
         out << poutNoComma(61 + i2, width, 0, true);
         const int firstyear = UserAssumptions::getIstart() + 1 + 20 * i2;
         for (int i1 = firstyear; i1 <= min(i3, firstyear + 19); i1++) {
            out << setw(earnWidth) << userAssumptions.baseOasdi[i1];
         }
         out << endl;
      }
      for (int i2 = 0; i2 < numrows; i2++) {
         out << poutNoComma(65 + i2, width, 0, true);
         const int firstyear = UserAssumptions::getIstart() + 1 + 20 * i2;
         for (int i1 = firstyear; i1 <= min(i3, firstyear + 19); i1++) {
            out << setw(earnWidth) << userAssumptions.base77[i1];
         }
         out << endl;
      }
   }
}

// Description: Writes military service dates.
//
// Arguments:
//   out: Output stream.
void PiaWrite::writeMsdates( std::ostream& out ) const
{
   if (workerData.milServDatesVec.getMSCount() > 0) {
      out << poutNoComma(11, width, 0, true);
      for (unsigned i = 0; i < workerData.milServDatesVec.getMSCount(); i++) {
         out << workerData.milServDatesVec.msdates[i].getDatesString();
      }
      out << endl;
   }
}

// Description: Writes oab entitlement prior to most recent dib.
//
// Arguments:
//   out: Output stream.
void PiaWrite::writeOabEnt( std::ostream& out ) const
{
   if (workerData.getOabEntDate().getYear() > 0) {
      out << setfill('0') << setw(width) << 39 << setfill(' ')
          << DateFormatter::toString(workerData.getOabEntDate(), "n")
          << DateFormatter::toString(workerData.getOabCessDate(), "n")
          << endl;
   }
}

// Description: Writes annual child care years.
//
// Arguments:
//   out: Output stream.
void PiaWrite::writeChildCareYears( std::ostream& out ) const
{
   bool needwrite = false;
   if (workerData.getIndearn()) {
      try {
         workerData.ibeginCheck();
      } catch (PiaException&) {
         return;
      }
   } else {
      return;
   }
   for (int i1 = workerData.getIbegin(); i1 <= workerData.getIend(); i1++) {
      if (workerData.childCareYears.getBit(i1)) {
         needwrite = true;
      }
   }
   if (needwrite) {
      out << poutNoComma(97, width, 0, true);
      for (int i1 = workerData.getIbegin(); i1 <= workerData.getIend(); i1++)
         out << static_cast<int>(workerData.childCareYears.getBit(i1));
      out << endl;
   }
}

// Description: Writes railroad earnings and qcs.
//
// Arguments:
//   out: Output stream.
void PiaWrite::writeRailRoad( std::ostream& out ) const
{
   // write pre-1951 qcs and earnings, if any
   if (workerData.railRoadData.getEarn3750() > 0.) {
      out << poutNoComma(84, width, 0, true)
         << workerData.railRoadData.get3750String(earnWidth, numDecimals) 
         << endl;
   }
   // write post-1950 qcs and earnings, if any
   if (workerData.railRoadData.getLastYear() > 1950) {
      out << poutNoComma(85, width, 0, true)
          << workerData.railRoadData.getYearsString() << endl;
      if (workerData.railRoadData.getFirstYear() < 1978) {
         out << poutNoComma(86, width, 0, true)
            << workerData.railRoadData.getQcString() << endl;
      }
      // write earnings by decade
      for (int i = 0; i < workerData.railRoadData.getDecades(); i++) {
         out << poutNoComma(87 + i, width, 0, true)
            << workerData.railRoadData.getDecadeEarningsString(i, earnWidth, 
            numDecimals) << endl;
      }
   }
}
