// $Id: piawriteAny.cpp 1.1 2005/07/21 16:11:57EDT 044579 Exp 277133(2006/03/14 09:46:55EST) $
//
// Functions to save a case to disk.

#include <fstream>
#include <iomanip>
#include "piawriteAny.h"
#include "WorkerDataArray.h"
#include "PiadataArray.h"
#include "UserAssumptions.h"
#include "SecondaryArray.h"
#include "EarnProject.h"
#include "pebs.h"
#include "awinc.h"
#include "comma.h"
#include "PiaException.h"
#include "AssumptionType.h"

using namespace std;

// Description: Initializes references to data.
//
// Arguments:
//   newWorkerData: Worker's calculation data.
//   newWidowDataArray: Widow and other family members' basic data.
//   newWidowArray: Widow and other family members' calculation data.
//   newUserAssumptions: User-specified assumptions.
//   newSecondaryArray: Secondary benefit information.
//   newEarnProject: Projected earnings.
//   newPebs: Statement information.
PiaWriteAny::PiaWriteAny( const WorkerData& newWorkerData,
const WorkerDataArray& newWidowDataArray, const PiaDataArray& newWidowArray,
const UserAssumptions& newUserAssumptions,
const SecondaryArray& newSecondaryArray, const EarnProject& newEarnProject,
const Pebs& newPebs ) :
PiaWrite(newWorkerData, newWidowDataArray, newWidowArray, newUserAssumptions,
newSecondaryArray),
earnProject(newEarnProject), pebs(newPebs)
{ }

// Description: Writes pebes information.
//
// Arguments:
//   out: Output stream.
void PiaWriteAny::writePebes( std::ostream& out ) const
{
   if (workerData.getJoasdi() == WorkerData::PEBS_CALC) {
      out << poutNoComma(5, width, 0, true);
      out << setw(2) << pebs.getMonthnow()
          << setw(2) << pebs.getAgePlan() << endl;
   }
}

// Description: Writes backward projection amounts.
//
// Arguments:
//   out: Output stream.
void PiaWriteAny::writeBack( std::ostream& out ) const
{
   if (earnProject.getProjback() > 0) {
      out.precision(2);
      out << poutNoComma(7, width, 0, true);
      out << earnProject.getProjback()
          << setw(6) << earnProject.getPercback();
      out << setw(4) << earnProject.getFirstYear() << endl;
   }
}

// Description: Writes forward projection amounts.
//
// Arguments:
//   out: Output stream.
void PiaWriteAny::writeFwrd( std::ostream& out ) const
{
   if (earnProject.getProjfwrd() > 0) {
      out.precision(2);
      out << poutNoComma(8, width, 0, true);
      out << earnProject.getProjfwrd()
          << setw(6) << earnProject.getPercfwrd();
      out << setw(4) << earnProject.getLastYear() << endl;
   }
}

// Description: Writes type of earnings for all years.
//
// Throws: PiaException of type PIA_IDS_EARNTYPE1 if year of projection is out
//   of range (only in debug mode).
//
// Arguments:
//   out: Output stream.
void PiaWriteAny::writeEarnType( std::ostream& out ) const
{
   bool needwrite = false;

   // do not try to save if no earnings have been set
   if (workerData.getIndearn()) {
      try {
         workerData.ibeginCheck();
      } catch (PiaException&) {
         return;
      }
   }
   else
      return;
#if !defined(NDEBUG)
   if (earnProject.getFirstYear() < YEAR37)
      throw PiaException(PIA_IDS_EARNTYPE1);
#endif
   const int firstyear = earnProject.getFirstYear();
   const int lastyear = earnProject.getLastYear();
   for (int i1 = firstyear; i1 <= lastyear; i1++) {
      if (earnProject.getEarntype()[i1] != EarnProject::ENTERED)
         needwrite = true;
   }
   if (needwrite) {
      out << poutNoComma(20, width, 0, true);
      for (int i1 = firstyear; i1 <= lastyear; i1++) {
         out << earnProject.getEarntype()[i1];
      }
      out << endl;
   }
}

// Description: Writes earnings for all years, using projected earnings array.
//
// Throws: PiaException of type PIA_IDS_WRITEEARN if first year of earnings is
//   out of range (only in debug mode).
//
// Arguments:
//   out: Output stream.
void PiaWriteAny::writeEarnOasdi( std::ostream& out ) const
{
   // do not try to save if no earnings have been set
   if (workerData.getIndearn()) {
      try {
         workerData.ibeginCheck();
      } catch (PiaException&) {
         return;
      }
   }
   else {
      return;
   }
#if !defined(NDEBUG)
   if (earnProject.getFirstYear() < YEAR37)
      throw PiaException(PIA_IDS_WRITEEARN);
#endif
   if (earnProject.anyEntered()) {
      out.precision(numDecimals);
      const int numrows =
         ((earnProject.getLastYear() - earnProject.getFirstYear()) / 10) + 1;
      for (int i2 = 0; i2 < numrows; i2++) {
         out << poutNoComma(22 + i2, width, 0, true);
         const int firstyear = earnProject.getFirstYear() + 10 * i2;
         const int lastyear = min(earnProject.getLastYear(), firstyear + 9);
         for (int i1 = firstyear; i1 <= lastyear; i1++) {
            out << setw(earnWidth) << earnProject.earnpebs[i1];
         }
         out << endl;
      }
   }
}

// Description: Writes assumption indicators.
//
// Arguments:
//   out: Output stream.
void PiaWriteAny::writeAssump( std::ostream& out ) const
{
   const bool needAw = earnProject.need_aw(UserAssumptions::getIstart(),
      workerData.getIend());
   if (workerData.need_aw(UserAssumptions::getIstart(), needAw) ||
      workerData.need_bi(UserAssumptions::getIstart()) ||
      userAssumptions.getIbasch() == BaseChangeType::ADHOC) {
      out << poutNoComma(40, width, 0, true);
      out << setw(4) << userAssumptions.getIstart()
         << static_cast<int>(userAssumptions.getIaltbi())
         << static_cast<int>(userAssumptions.getIaltaw())
         << userAssumptions.getIbasch() << endl;
   }
}
