// $Id: railroad.cpp 1.29 2006/03/29 11:20:44EST 044579 Development  $
//
// Functions to read and write a railroad record.

#include <algorithm>
#include "railroad.h"
#include "comma.h"
#include "oactcnst.h"

using namespace std;

// Description: Initializes last year of railroad data.
//
// Remarks:  First year is always 1937, for simplicity.
//
// Arguments:
//   year: New last year of data.
RailRoadData::RailRoadData( int year ) : qc3750(0), earn3750(0),
firstYear(YEAR37), lastYear(YEAR37), qc(YEAR37, 1977), earnings(YEAR37, year)
{ }

// Description: Destroys railroad data.
RailRoadData::~RailRoadData()
{ }

// Description: Zeroes out all railroad data.
void RailRoadData::deleteContents()
{
   setQc3750(0);
   qc.deleteContents();
   setEarn3750(0.);
   earnings.deleteContents();
   firstYear = YEAR37;
   lastYear = YEAR37;
}

// Description: Parses one line of earnings.
//
// Arguments:
//   decade: Decade to use.
//     0:  first 10 years.
//     1:  second 10 years.
//     etc. up to eighth 10 years.
//   line: Line to parse.
//   width: Width of each earnings field.
void RailRoadData::parseEarnings( int decade, const std::string& line,
int width )
{
   const int startyear = firstYear + 10 * decade;
   const int endyear = min(startyear + 9, lastYear);
   for (int i = startyear; i <= endyear; i++) {
      earnings[i] = atof(line.substr((i - startyear) * width, width).c_str());
   }
}

// Description: Parses line of qcs.
//
// Arguments:
//   line: Line to parse.
void RailRoadData::parseQcs( const std::string& line )
{
   for (int i = 1951; i <= 1977; i++)
      qc.set(i, static_cast<unsigned>(atoi(line.substr(i - 1951, 1).c_str())));
}

// Description: Returns string of qcs.
//
// Returns: String of qcs.
std::string RailRoadData::getQcString() const
{
   string s("");
   for (int i = 1951; i <= 1977; i++)
      s += poutComma(qc.get(i));
   return(s);
}

// Description: Returns true if there are any quarters of coverage.
//
// Returns: True if there are any quarters of coverage.
bool RailRoadData::isNonzero() const
{
   if (earn3750 > 0.0)
      return(true);
   for (int yr = 1951; yr <= earnings.getLastYear(); yr++) {
      if (earnings[yr] > 0.0)
         return(true);
   }
   return(false);
}

// Description: Returns number of decades of post-1950 earnings.
//
// Returns: Number of decades of post-1950 earnings.
int RailRoadData::getDecades() const
{
   if (lastYear > 0)
      return(1 + (lastYear - firstYear) / 10);
   else
      return(0);
}

// Description: Parses line of first and last years.
//
// Arguments:
//   line: Line to parse.
void RailRoadData::parseYears( const std::string& line )
{
   firstYear = atoi(line.substr(0, 4).c_str());
   lastYear = atoi(line.substr(4, 4).c_str());
}

// Description: Parses line of pre-1951 QCs and earnings.
//
// Arguments:
//   line: Line to parse.
//   width: Width of each earnings field.
void RailRoadData::parse3750( const std::string& line, int width )
{
   setQc3750(atoi(line.substr(0, 2).c_str()));
   setEarn3750(atof(line.substr(2, width).c_str()));
}

// Description: Returns string of decade of earnings.
//
// Returns: String of decade of earnings.
//
// Arguments:
//   decade: Decade to use.
//     0:  first 10 years.
//     1:  second 10 years.
//     etc. up to eighth 10 years.
//   width: Width of each earnings field.
//   decimals: Number of decimals in each earnings field.
std::string RailRoadData::getDecadeEarningsString( int decade, int width,
int decimals ) const
{
   string s("");
   const int startyear = firstYear + 10 * decade;
   const int endyear = min(startyear + 9, lastYear);
   for (int i = startyear; i <= endyear; i++) {
      s += poutNoComma(earnings[i], width, decimals);
   }
   return(s);
}

// Description: Returns string of pre-1951 QCs and earnings.
//
// Returns: String of pre-1951 QCs and earnings.
//
// Arguments:
//   width: Width of each earnings field.
//   decimals: Number of decimals in each earnings field.
std::string RailRoadData::get3750String( int width, int decimals ) const
{
   string s("");
   if (getEarn3750() > 0.) {
      s += poutNoComma(getQc3750(), 2);
      s += poutNoComma(getEarn3750(), width, decimals);
   }
   return(s);
}

// Description: Returns string of first and last years of earnings.
//
// Returns: String of first and last years of earnings.
std::string RailRoadData::getYearsString() const
{
   string s("");
   if (getLastYear() > 1950) {
      s += poutNoComma(getFirstYear(), 4);
      s += poutNoComma(getLastYear(), 4);
   }
   return(s);
}
