// $Id: rtgenrl.cpp 1.30 2007/11/06 09:08:44EST 277133 Development  $
//
// Functions to handle retirement test calculations.

#include "rtgenrl.h"
#include "Resource.h"
#include "PiaException.h"
#include "BaseYearNonFile.h"
#include "piaparms.h"

using namespace std;

// The age 62 monthly exempt amounts, 1951-2008.
const double Rtea62m::theAmts[] = {
   50.0,    50.0,   75.0,   75.0,   80.0,  // 1951-1955
   80.0,    80.0,   80.0,  100.0,  100.0,  // 1956-1960
   100.0,  100.0,  100.0,  100.0,  100.0,  // 1961-1965
   125.0,  125.0,  140.0,  140.0,  140.0,  // 1966-1970
   140.0,  140.0,  175.0,  200.0,  210.0,  // 1971-1975
   230.0,  250.0,  270.0,  290.0,  310.0,  // 1976-1980
   340.0,  370.0,  410.0,  430.0,  450.0,  // 1981-1985
   480.0,  500.0,  510.0,  540.0,  570.0,  // 1986-1990
   590.0,  620.0,  640.0,  670.0,  680.0,  // 1991-1995
   690.0,  720.0,  760.0,  800.0,  840.0,  // 1996-2000
   890.0,  940.0,  960.0,  970.0, 1000.0,  // 2001-2005
  1040.0, 1080.0, 1130.0 // 2006-2008
};
// The age 65 monthly exempt amounts, 1951-2008.
const double Rtea65m::theAmts[] = {
    50.0,   50.0,   75.0,   75.0,   80.0,  // 1951-1955
    80.0,   80.0,   80.0,  100.0,  100.0,  // 1956-1960
   100.0,  100.0,  100.0,  100.0,  100.0,  // 1961-1965
   125.0,  125.0,  140.0,  140.0,  140.0,  // 1966-1970
   140.0,  140.0,  175.0,  200.0,  210.0,  // 1971-1975
   230.0,  250.0, 333.33333, 375.0, 416.66666,  // 1976-1980
   458.33333, 500.0, 550.0, 580.0,  610.0,  // 1981-1985
   650.0,  680.0,  700.0,  740.0,  780.0,  // 1986-1990
   810.0,  850.0,  880.0,  930.0,  940.0,  // 1991-1995
  1041.66666, 1125.0, 1208.33333, 1291.66666, 1416.66666,  // 1996-2000
  2083.33333, 2500.0, 2560.0, 2590.0, 2650.0,  // 2001-2005
  2770.0, 2870.0, 3010.0 // 2006-2008
};

// Description: Exempt amounts, 1961-1972.
const double RteaHigh::rteaHigh[YEAR72 - YEAR61 + 1] = {
   1500.0, 1700.0, 1700.0, 1700.0, 1700.0, 2700.0,
   2700.0, 2880.0, 2880.0, 2880.0, 2880.0, 2880.0
};

// Description: Initializes width, precision, and historical values for monthly
//   test under full retirement age.
//
// Arguments:
//   newBaseYear: First year of data. If omitted, first year is 1937.
//   newLastYear: Last year of projection.
Rtea62m::Rtea62m( int newLastYear ) : RtGeneral(newLastYear)
{
   initialize();
}

Rtea62m::Rtea62m( int newBaseYear, int newLastYear ) :
RtGeneral(newBaseYear, newLastYear)
{
   initialize();
}

// Description: Initializes historical data.
void Rtea62m::initialize()
{
   const int i1 = max(getBaseYear(), YEAR51);
   const int i2 = min(getLastYear(), BaseYearNonFile::YEAR);
   for (int i = i1; i <= i2; i++) {
      theData[i - baseYear] = theAmts[i - YEAR51];
   }
}

// Description: Projects retirement test exempt amounts for ages under full
//   retirement age.
//
// Arguments:
//   firstYear: First year of projection.
//   averageWages: Average wage series.
//   cpi: Benefit increase series.
void Rtea62m::project( int firstYear, const DoubleAnnual& averageWages,
const DoubleAnnual& cpi )
{
   int count = 0;  // count of missed increases

   for (int i = firstYear; i <= getLastYear(); i++) {
      if (cpi[i - 1] < .05) {
         theData[i - baseYear] = theData[i - 1 - baseYear];
         count++;
      }
      else {
         if (i >= 1995) {
            theData[i - baseYear] = project1(theData[1994 - baseYear],
               averageWages[i - 2], averageWages[1992],
               theData[i - 1 - baseYear]);
         }
         else {
            theData[i - baseYear] = project1(theData[i - 1 - count - baseYear],
               averageWages[i - 2], averageWages[i - 3 - count],
               theData[i - 1 - baseYear]);
         }
         count = 0;
      }
   }
}

// Description: Returns x in 1-for-x rule for earnings offset.
//
// Returns: x in 1-for-x rule for earnings offset.
//
// Arguments:
//   year: Year for which fraction is desired.
int Rtea62m::getFraction( int year )
{
   if (year < 1961)
      return(1);
   return(2);
}

// Description: Initializes width, precision, and historical values for monthly
//   test over full retirement age.
//
// Arguments:
//   newBaseYear: First year of data. If omitted, first year is 1937.
//   newLastYear: Last year of projection.
Rtea65m::Rtea65m( int newLastYear ) : RtGeneral(newLastYear)
{
   initialize();
}

Rtea65m::Rtea65m( int newBaseYear, int newLastYear ) :
RtGeneral(newBaseYear, newLastYear)
{
   initialize();
}

// Description: Initializes historical data.
void Rtea65m::initialize()
{
   const int i1 = max(getBaseYear(), YEAR51);
   const int i2 = min(getLastYear(), BaseYearNonFile::YEAR);
   for (int i = i1; i <= i2; i++) {
      theData[i - baseYear] = theAmts[i - YEAR51];
   }
}

// Description: Projects retirement test exempt amounts for ages 65-69.
//
// Arguments:
//   firstYear: First year of projection.
//   averageWages: Average wage series.
//   cpi: Benefit increase series.
void Rtea65m::project( int firstYear, const DoubleAnnual& averageWages,
const DoubleAnnual& cpi )
{
   int count = 0;  // count of missed increases

   for (int i = firstYear; i <= getLastYear(); i++) {
      if (cpi[i - 1] < .05) {
         theData[i - baseYear] = theData[i - 1 - baseYear];
         count++;
      }
      else {
         if (i < 1995) {
            theData[i - baseYear] = project1(theData[i - 1 - count - baseYear],
               averageWages[i - 2], averageWages[i - 3 - count],
               theData[i - 1 - baseYear]);
         }
         else if (i == 1995) {
            theData[i - baseYear] = project1(theData[1994 - baseYear],
               averageWages[i - 2], averageWages[1992],
               theData[i - 1 - baseYear]);
         }
         else if (i >= 1996 && i <= 2002) {
            theData[i - baseYear] = theAmts[i - YEAR51];
         }
         else if (i > 2002) {
            theData[i - baseYear] = project1(theData[2002 - baseYear],
               averageWages[i - 2], averageWages[2000],
               theData[i - 1 - baseYear]);
         }
         count = 0;
      }
   }
}

// Description: Returns x in 1-for-x rule for earnings offset.
//
// Returns: x in 1-for-x rule for earnings offset.
//
// Arguments:
//   year: Year for which fraction is desired.
int Rtea65m::getFraction( int year )
{
   if (year < 1961)
      return(1);
   if (year < 1990)
      return(2);
   return(3);
}

// Description: Initializes width, precision, and historical values for annual
//   test at second tier.
//
// Remarks: First and last years are 1961 and 1972.
RteaHigh::RteaHigh() : RtGeneral(YEAR61, YEAR72)
{
   initialize();
}

// Description: Initializes historical data.
void RteaHigh::initialize()
{
   for (int i = baseYear; i <= getLastYear(); i++) {
      theData[i - baseYear] = rteaHigh[i - YEAR61];
   }
}

// Description: Returns x in 1-for-x rule for earnings offset.
//
// Returns: Always returns 1.
int RteaHigh::getFraction()
{
   return(1);
}

// Description: Initializes width and precision.
//
// Arguments:
//   newBaseYear: First year of data. If not specified, arrays start in 1937.
//   newLastYear: Last year of projection.
RtGeneral::RtGeneral( int newLastYear ) : DoubleAnnual(YEAR37, newLastYear)
{
   setWidth(6);
   setPrecision(0);
}

RtGeneral::RtGeneral( int newBaseYear, int newLastYear ) :
DoubleAnnual(newBaseYear, newLastYear)
{
   setWidth(6);
   setPrecision(0);
}

// Description: Destructor.
RtGeneral::~RtGeneral()
{ }

// Description: Calculates a monthly retirement test exempt amount using a base
//   amount and a ratio of average wages.
//
// Returns: The new monthly retirement test exempt amount.
//
// Throws: PiaException of type PIA_IDS_RTGENRL if denominator is out of range
//   (only in debug mode).
//
// Arguments:
//   baseamt: Value of exempt amount in base year.
//   numerator: Numerator used in projection.
//   denominator: Denominator used in projection.
//   previous: Value of exempt amount in previous year.
double RtGeneral::project1( double baseamt, double numerator,
double denominator, double previous )
{
#ifndef NDEBUG
   if (denominator < 1.0)
      throw PiaException(PIA_IDS_RTGENRL);
#endif
   // k is exempt amount in multiples of $10
   const int exempt10 =
      static_cast<int>((((numerator / denominator) * baseamt) + 4.99) / 10.0);
   // do not allow decrease in exempt amount
   return(max(static_cast<double>(exempt10) * 10.0, previous));
}

// Description: Converts monthly retirement test exempt amounts to annual
//   amounts.
//
// Arguments:
//   rteam: Monthly retirement test exempt amounts.
void RtGeneral::convertMonthToAnnual( const RtGeneral& rteam )
{
   for (int i = getBaseYear(); i <= getLastYear(); i++) { 
      theData[i - baseYear] = rteam[i] * 12.0;
   }
}

// Description: Returns maximum age for which earnings test applies.
//
// Returns: Maximum age for which earnings test applies.
//
// Arguments:
//   year: Year for which age is desired.
Age RtGeneral::getMaxAgePL( int year )
{
   if (year < 1955)
      return PiaParams::age75;
   if (year < 1983)
      return PiaParams::age72;
   if (year < 2000)
      return PiaParams::age70;
   return PiaParams::maxDibAgePL(year);
}

// Description: Initializes width and precision.
//
// Arguments:
//   newBaseYear: First year of data. If omitted, first year is 1937.
//   newLastYear: Last year of projection.
Rtea62::Rtea62( int newLastYear ) : RtGeneral(newLastYear)
{  }

Rtea62::Rtea62( int newBaseYear, int newLastYear ) :
RtGeneral(newBaseYear, newLastYear)
{  }

// Description: Initializes width and precision.
//
// Arguments:
//   newBaseYear: First year of data. If omitted, first year is 1937.
//   newLastYear: Last year of projection.
Rtea65::Rtea65( int newLastYear ) : RtGeneral(newLastYear)
{  }

Rtea65::Rtea65( int newBaseYear, int newLastYear ) :
RtGeneral(newBaseYear, newLastYear)
{  }
