// $Id: wbgenrl.cpp 1.19 2006/11/08 11:05:54EST 044579 Development  $
//
// Functions to handle wage base calculations.

#include <cmath>  // for floor
#include "wbgenrl.h"
#include "Resource.h"
#include "oactcnst.h"
#include "PiaException.h"

using namespace std;

// Description: Increase in average wage due to deferred comp in 1990.
const double WageBaseGeneral::defcomp = 0.0149249;

// Description: Initializes width and precision.
//
// Remarks: The version without a newBaseYear argument starts the data array
// in 1937.
//
// Arguments:
//   newWageBaseInd: Type of wage base.
//
//     @table
//     value  meaning
//     0      present law OASDI.
//     1      prior law (ignore 1990-92 ad hoc increases).
//     2      old law (ignore 1979-81 ad hoc increases).
//     3      HI.
//   newLastYear: Last year of projection.
WageBaseGeneral::WageBaseGeneral( int newWageBaseInd, int newLastYear ) :
DoubleAnnual(YEAR37, newLastYear), wageBaseInd(newWageBaseInd)
{
   setWidth(7);
   setPrecision(0);
}

// Arguments:
//   newBaseYear: First year of data. If not specified, array starts in 1937.
WageBaseGeneral::WageBaseGeneral( int newWageBaseInd, int newBaseYear,
int newLastYear ) : DoubleAnnual(newBaseYear, newLastYear),
wageBaseInd(newWageBaseInd)
{
   setWidth(7);
   setPrecision(0);
}

// Description: Destructor.
WageBaseGeneral::~WageBaseGeneral()
{ }

// Description: Projects wage bases.
//
// Throws: PiaException of type PIA_IDS_BASE1 if first year is out of range
//   (only in debug mode).
//
// Arguments:
//   averageWages: Average wages for wage base projection.
//   cpi: Benefit increases.
//   firstYeart: First year of projection.
//   lastYeart: Last year of projection.
void WageBaseGeneral::project( const DoubleAnnual& averageWages,
const DoubleAnnual& cpi, int firstYeart, int lastYeart )
{
   int iflag; // number of bases considered since last increase
   double factor;  // factor to apply to previous base

#ifndef NDEBUG
   if (firstYeart < 1975)
      throw PiaException(PIA_IDS_BASE1);
#endif
   for (int i1 = firstYeart; i1 <= lastYeart; i1++) {
      iflag = 0;
      while (static_cast<double>(cpi[i1 + iflag - 1]) < 0.1) {
         // if no benefit increase, wage base equals last previously
         // set base
         theData[i1 + iflag - baseYear] = theData[i1 - 1 - baseYear];
         iflag++;
         if (i1 + iflag > getLastYear())
            return;
      }
      int i3 = i1 + iflag;
      // set intermediate, unrounded wage base
      double baseun = theData[i1 - 1 - baseYear];
      // apply factor prior to 1995, project from average wage thereafter
      if (i3 < 1995) {
         for (int i2 = 0; i2 <= iflag; i2++) {
            if (wageBaseInd != 1 && i1 + i2 > 1989 && i1 + i2 < 1993) {
               if (i1 + i2 == 1990) {
                  factor = averageWages[i1+i2-2] /
                     averageWages[i1+i2-3] + 0.02;
               }
               else if (i1 + i2 == 1991) {
                  factor = (averageWages[i1+i2-2] +
                     0.02 * averageWages[i1+i2-3]) /
                     (averageWages[i1+i2-3] +
                     0.02 * averageWages[i1+i2-4]);
               }
               else {
                  factor = averageWages[i1+i2-2] * (1.0 + defcomp) /
                     (averageWages[i1+i2-3] +
                     0.02 * averageWages[i1+i2-4]);
               }
            }
            else {
               factor = averageWages[i1+i2-2] / averageWages[i1+i2-3];
            }
            baseun = (baseun + 0.001) * factor;
         }
      }
      else {
         factor = averageWages[i3 - 2] / averageWages[1992];
         if (wageBaseInd == 2) {
            baseun = 45000.0 * factor;
         }
         else {
            baseun = (wageBaseInd < 2) ? 60600.0 * factor : 999999999.0;
         }
      }
      // handle ad hoc increases 1979-81
      if (wageBaseInd == 2 && i3 >= YEAR79 && i3 < 1982) {
         if (i3 == YEAR79)
            theData[i3 - baseYear] = 22900.0;
         else if (i3 == 1980)
            theData[i3 - baseYear] = 25900.0;
         else
            theData[i3 - baseYear] = 29700.0;
      }
      else {
         theData[i3 - baseYear] = 300.0 * floor(baseun / 300.0 + 0.5);
      }
      // do not allow a decrease in the base
      if (theData[i3 - baseYear] < theData[i3 - 1 - baseYear]) {
         theData[i3 - baseYear] = theData[i3 - 1 - baseYear];
      }
      i1 = i3;
   } // end for i1
}

// Description: Checks a wage base.
//
// Throws: PiaException of type PIA_IDS_BASE2 if wage base is out of range.
//
// Arguments:
//   baset: Value to check.
void WageBaseGeneral::check( double baset )
{
   if (baset > MAXEARN || baset < 0.0)
      throw PiaException(PIA_IDS_BASE2);
}

// Description: Limits earnings to wage base.
//
// Arguments:
//   earnings1: Array of limited earnings.
//   earnings2: Array of unlimited earnings.
void WageBaseGeneral::earnLimit( DoubleAnnual& earnings1,
const DoubleAnnual& earnings2 ) const
{
   const int i1 = min(earnings1.getLastYear(), earnings2.getLastYear());
   const int i2 = max(earnings1.getBaseYear(), earnings2.getBaseYear());
   for (int i3 = i2; i3 <= i1; i3++) {
      earnings1[i3] = min(earnings2[i3], theData[i3 - baseYear]);
   }
}

// Description: Projects wage bases after a period of ad hoc wage bases.
//
// Arguments:
//   avgwages: Average wages for wage base projection.
//   cpi: Benefit increases.
//   lastYearLC:  Last year of ad hoc wage bases.
void WageBaseGeneral::projectLC( const DoubleAnnual& avgwages,
const DoubleAnnual& cpi, int lastYearLC )
{
   for (int i1 = lastYearLC + 1; i1 <= getLastYear(); i1++) {
      if (cpi[i1 - 1] < 0.1) {
         // if no benefit increase, wage base equals last previously
         // set base
         theData[i1 - baseYear] = theData[i1 - 1 - baseYear];
         continue;
      }
      // project by average wage after ad hoc increases
      const double factor = avgwages[i1 - 2] / avgwages[lastYearLC - 2];
      double baseun = theData[lastYearLC - baseYear] * factor;
      // do not allow a decrease in the base
      baseun = max(300. * floor(baseun / 300.0 + 0.5),
         theData[i1 - 1 - baseYear]);
      theData[i1 - baseYear] = baseun;
   }
}
