// $Id: wrkrdata.cpp 1.80 2007/03/09 17:09:44EST 044579 Development  $
//
// Functions to initialize a WorkerData structure and check data.

#include "wrkrdata.h"
#include "PiaException.h"
#include "Resource.h"
#include "DebugCase.h"
#include "oactcnst.h"
#include "DateFormatter.h"
#if defined(DEBUGCASE)
#include "Trace.h"
#include "comma.h"
#endif

using namespace std;

// Description: Initializes a WorkerData structure.  Should call setMaxyear
//   before constructing any WorkerData.
WorkerData::WorkerData() : WorkerDataGeneral(),
ssn(), milServDatesVec(), railRoadData(maxyear), earnOasdi(YEAR37, maxyear),
earnHi(YEAR37, maxyear), taxType(YEAR37, maxyear), nhname(""), pubpen(0.0f),
pubpenDate(), pubpenReservist(0.0f), oabEntDate(), oabCessDate()
{
   for (int i = 0; i < NUM_ADDRESS_LINES; i++)
      nhaddr[i].resize(0);
}

// Description: Destroys a WorkerData structure.
WorkerData::~WorkerData( )
{ }

// Description: Initializes all variables to default values.
void WorkerData::deleteContents()
{
   WorkerDataGeneral::deleteContents();
   ssn.deleteContents();
   nhname.resize(0);
   for (int i = 0; i < NUM_ADDRESS_LINES; i++)
      nhaddr[i].resize(0);
   milServDatesVec.deleteContents();
   railRoadData.deleteContents();
   earnOasdi.deleteContents();
   earnHi.deleteContents();
   taxType.deleteContents();
   pubpen = pubpenReservist = 0.0f;
   pubpenDate.deleteContents();
   oabCessDate.deleteContents();
   oabEntDate.deleteContents();
}

// Description: Zeroes out earnings and quarters of coverage before first year
//   and after last year of specified earnings.
//
// Arguments:
//   ibegint: First year of specified earnings.
//   iendt: Last year of specified earnings.
void WorkerData::zeroEarn( int ibegint, int iendt )
{
   WorkerDataGeneral::zeroEarn(ibegint, iendt);
   zeroEarnOasdi(ibegint, iendt);
   zeroEarnHi(ibegint, iendt);
}

// Description: Zeroes out OASDI earnings before first year
//   and after last year of specified earnings.
//
// Arguments:
//   ibegint: First year of specified earnings.
//   iendt: Last year of specified earnings.
void WorkerData::zeroEarnOasdi( int ibegint, int iendt )
{
   earnOasdi.assign(0.0, YEAR37, ibegint - 1);
   earnOasdi.assign(0.0, iendt + 1, maxyear);
}

// Description: Zeroes out HI earnings before first year
//   and after last year of specified earnings.
//
// Arguments:
//   ibegint: First year of specified earnings.
//   iendt: Last year of specified earnings.
void WorkerData::zeroEarnHi( int ibegint, int iendt )
{
   earnHi.assign(0.0, YEAR37, ibegint - 1);
   earnHi.assign(0.0, iendt + 1, maxyear);
}

// Description: Sets date of cessation of oab prior to most recent dib.
//
// Arguments:
//   datetemp: The date to use.
void WorkerData::setOabCessDate( const DateMoyr& datetemp )
{
   if (valdi > 0) {
#ifndef NDEBUG
      DisabPeriod::cessationDateCheck(datetemp);
#endif
      oabCessDate = datetemp;
   }
   else
      oabCessDate.deleteContents();
}

// Description: Sets date of oab entitlement prior to most recent dib.
//
// Arguments:
//   datetemp: The date to use.
void WorkerData::setOabEntDate( const DateMoyr& datetemp )
{
   if (valdi > 0) {
#ifndef NDEBUG
      DisabPeriod::cessationDateCheck(datetemp);
#endif
      oabEntDate = datetemp;
   }
   else
      oabEntDate.deleteContents();
}

// Description: Sets OASDI-covered earnings.
//
// Throws: PiaException of type PIA_IDS_EARNPEBS if earnings are out of range
//   (only in debug mode).
//
// Arguments:
//   year: Year of earnings.
//   newEarn: Amount of earnings.
void WorkerData::setEarnOasdi( int year, double newEarn )
{
#ifndef NDEBUG
   try {
      WageBaseGeneral::check(newEarn);
   } catch (PiaException e) {
      throw PiaException(PIA_IDS_EARNPEBS);
   }
#endif
#if defined(DEBUGCASE)
   if (isDebugPid(getIdNumber())) {
      string s("");
      s += getIdString();
      s += ", year ";
      s += poutNoComma(year);
      s += ", earnings ";
      s += poutNoComma(newEarn, 5, 2);
      s += ": Setting OASDI earnings";
      Trace::writeLine(s);
   }
#endif
   earnOasdi[year] = newEarn;
}

// Description: Sets Medicare earnings for the specified year.
//
// Throws: PiaException of type PIA_IDS_EARNHI if earnings are out of range
//   (only in debug mode).
//
// Arguments:
//   year: Year of earnings.
//   newEarnHi: Amount of earnings.
void WorkerData::setEarnHi( int year, double newEarnHi )
{
#ifndef NDEBUG
   try {
      WageBaseGeneral::check(newEarnHi);
   } catch (PiaException e) {
      throw PiaException(PIA_IDS_EARNHI);
   }
#endif
#if defined(DEBUGCASE)
   if (isDebugPid(getIdNumber())) {
      string s("");
      s += getIdString();
      s += ", year ";
      s += poutNoComma(year);
      s += ", earnings ";
      s += poutNoComma(newEarnHi, 5, 2);
      s += ": Setting HI earnings";
      Trace::writeLine(s);
   }
#endif
   earnHi[year] = newEarnHi;
}

// Description: Sets type of taxes for the specified year.
//
// Throws: PiaException of type TAXTYPEYR if year is out of range (only in
//   debug mode).
//
// Arguments:
//   year: Year of taxes.
//   newTaxType: Type of taxes.
void WorkerData::setTaxType( int year, int newTaxType )
{
#ifndef NDEBUG
   if (year < YEAR37)
      throw PiaException(PIA_IDS_TAXTYPEYR);
   taxTypeCheck(newTaxType);
#endif
#if defined(DEBUGCASE)
   if (isDebugPid(getIdNumber())) {
      string s("");
      s += getIdString();
      s += ", year ";
      s += poutNoComma(year);
      s += ", tax type ";
      s += poutNoComma(newTaxType);
      s += ": Setting tax type";
      Trace::writeLine(s);
   }
#endif
   taxType.setBit(year, newTaxType);
}

// Description: Sets amount of monthly noncovered pension.
//
// Throws: PiaException of type PIA_IDS_PUBPEN if pension is out of range
//   (only in debug mode).
//
// Arguments:
//   newPubpen: New amount of monthly noncovered pension.
void WorkerData::setPubpen( float newPubpen )
{
#ifndef NDEBUG
   pubpenCheck(newPubpen);
#endif
#if defined(DEBUGCASE)
   if (isDebugPid(getIdNumber())) {
      string s("");
      s += getIdString();
      s += ", pubpen ";
      s += poutNoComma(newPubpen, 5, 2);
      s += ": Setting pubpen";
      Trace::writeLine(s);
   }
#endif
   pubpen = newPubpen;
}

// Description: Checks the date of noncovered pension entitlement for
//   consistency with date of birth and date of death.
void WorkerData::pubpenDateCheck2() const
{
   WorkerData::pubpenDateCheck(pubpenDate, birthDate, deathDate);
}

// Description: Sets date of noncovered pension entitlement.
//
// Arguments:
//   newPubpen: New date of noncovered pension entitlement.
void WorkerData::setPubpenDate( const DateMoyr& newPubpenDate )
{
#if defined(DEBUGCASE)
   if (isDebugPid(getIdNumber())) {
      ostringstream strm;
      strm << getIdString() << ", pubpenDate "
         << DateFormatter::toString(newPubpenDate) << ": Setting pubpenDate";
      Trace::writeLine(strm.str());
   }
#endif
   pubpenDateCheck(newPubpenDate, birthDate, deathDate);
   pubpenDate = newPubpenDate;
}

// Description: Clears date of noncovered pension entitlement.
void WorkerData::clearPubpenDate()
{
   pubpenDate = DateMoyr(0,0);
}

// Description: Sets amount of monthly noncovered pension after December 1994,
//   after removing military reservist pension.
//
// Throws: PiaException of type PIA_IDS_PUBPEN if pension is out of range
//   (only in debug mode).
//
// Arguments:
//   newPubpen: New amount of noncovered pension after December 1994.
void WorkerData::setPubpenReservist( float newPubpen )
{
#ifndef NDEBUG
   pubpenCheck(newPubpen);
#endif
#if defined(DEBUGCASE)
   if (isDebugPid(getIdNumber())) {
      string s("");
      s += getIdString();
      s += ", pubpen ";
      s += poutNoComma(newPubpen, 5, 2);
      s += ": Setting pubpenReservist";
      Trace::writeLine(s);
   }
#endif
   pubpenReservist = newPubpen;
}
