// $Id: PiaPageOut.cpp 1.27 2007/03/09 17:10:15EST 044579 Development  $
//
// Functions to write out results of benefit calculation.

#include "PiaPageOut.h"
#include "comma.h"
#include "FormatString.h"
#include "oactcnst.h"
#include "PiaException.h"
#include "Resource.h"
#include "piaparms.h"
#include "WageInd.h"
#include "boost/date_time/gregorian/gregorian.hpp"
#include "boost/date_time/local_time/local_time_io.hpp"
#include "DateFormatter.h"

using namespace std;

// Description: Years of Social Security Acts.
const char *PiaPageOut::tableLabel[10] = {
   "1952", "1954", "1958", "1965", "1967", "1969", "1971", "1972", "1973",
   "1977"
};

// Description: Warning for totalization benefit (less than 6 qcs, or insured).
const std::string PiaPageOut::warnTotalization(
   "Hypothetical totalization benefit!!!");

// Description: Initializes PiaPageOut.
PiaPageOut::PiaPageOut() : PageOut()
{ }

// Description: Destructor.
PiaPageOut::~PiaPageOut()
{ }

// Description: Prepares number of computation years in a string table.
//
// Arguments:
//   workerData: Worker basic data.
//   piaData: Pia calculation data.
//   compPeriod: Computation period (old or new).
void PiaPageOut::compYear( const WorkerDataGeneral& workerData,
const PiaData& piaData, const CompPeriod& compPeriod )
{
   string s;  // temporary output string

   if (workerData.getValdi() > 0) {
      if (piaData.freezeYears.hasYears1() || piaData.freezeYears.hasYears2()) {
         outputString.push_back("Years wholly within disability freeze:");
         s = "   ";
         if (piaData.freezeYears.hasYears2()) {
            s += poutNoComma(piaData.freezeYears.getYear3());
            s += " to ";
            s += poutNoComma(piaData.freezeYears.getYear4());
         }
         if (piaData.freezeYears.hasYears1() && piaData.freezeYears.hasYears2())
            s += " and ";
         if (piaData.freezeYears.hasYears1()) {
            s += poutNoComma(piaData.freezeYears.getYear1());
            s += " to ";
            s += poutNoComma(piaData.freezeYears.getYear2());
         }
         outputString.push_back(s);
      }
      else
         outputString.push_back("No years wholly within disability freeze");
      outputString.push_back("");
      if (piaData.partialFreezeYears.hasYears1() ||
         piaData.partialFreezeYears.hasYears2()) {
         outputString.push_back("Years partially within disability freeze:");
         s = "   ";
         if (piaData.partialFreezeYears.hasYears2()) {
            s += poutNoComma(piaData.partialFreezeYears.getYear3());
            s += " to ";
            s += poutNoComma(piaData.partialFreezeYears.getYear4());
         }
         if (piaData.partialFreezeYears.hasYears1() &&
            piaData.partialFreezeYears.hasYears2())
            s += " and ";
         if (piaData.partialFreezeYears.hasYears1()) {
            s += poutNoComma(piaData.partialFreezeYears.getYear1());
            s += " to ";
            s += poutNoComma(piaData.partialFreezeYears.getYear2());
         }
         outputString.push_back(s);
      }
      else
         outputString.push_back(
            "No years partially within disability freeze");
      outputString.push_back("");
   }
   s = "Number of di dropout years = ";
   s += poutNoComma(compPeriod.getDiYears());
   outputString.push_back(s);
   outputString.push_back("");
   s = "Number of elapsed years = ";
   s += poutNoComma(compPeriod.getNElapsed());
   outputString.push_back(s);
   outputString.push_back("");
   s = "Number of dropout years = ";
   s += poutNoComma(compPeriod.getNDrop());
   outputString.push_back(s);
   outputString.push_back("");
   s = "Number of computation years = ";
   s += poutNoComma(compPeriod.getNElapsed());
   s += " - ";
   s += poutNoComma(compPeriod.getNDrop());
   s += " = ";
   s += poutNoComma(compPeriod.getN());
   outputString.push_back(s);
   outputString.push_back("");
}

// Description: Prepares benefit increases used in PIA calculation in a string
//   table.
//
// Remarks: The version with five arguments is passed an int which determines 
//   the starting and ending years.  The version with six arguments is passed
//   the starting and ending years instead.  
//
// Arguments:
//   piaMethod: Pia calculation method.
//   piaParams: Pia calculation parameters.
//   eligYear: Year of eligibility for catch-up benefit increases.
//   type: Type of writeout.
//     0 to go from first year to entitlement.
//     1 to go from eligibility to entitlement.
//     2 to go from entitlement to benefit date.
//     3 to go from first year to benefit date.
//   firstYear: First year to print increases.
//   lastYear: Last year to print increases.
//   piamfb: Indicator for PIA and MFB printout.
//     0 to not print out PIA or MFB.
//     1 for PIA.
//     2 for MFB.
//     3 for PIA and MFB.
void PiaPageOut::biOut( const PiaMethod& piaMethod,
const PiaParams& piaParams, int eligYear, int type, int piamfb )
{
   string s;  // temporary string

#ifndef NDEBUG
   if (eligYear < YEAR37)
      throw PiaException(PIA_IDS_WISTRING);
#endif
   const PiaMethod::cpi_year_type cpiYear1 = firstYearCal(type);
   const PiaMethod::cpi_year_type cpiYear2 = endYearCal(type);
   outputString.push_back("CPI increases applied:");
   const int i3 = (type == 2) ? piaMethod.getCpiYear(cpiYear1) + 1 :
      piaMethod.getCpiYear(cpiYear1);
#ifndef NDEBUG
   if (i3 < YEAR37)
      throw PiaException(PIA_IDS_WISTRING);
#endif
   for (int i1 = i3; i1 <= piaMethod.getCpiYear(cpiYear2); i1++) {
      if (piaParams.getMonthBeninc(i1) > 0) {
         DateMoyr datetemp(piaParams.getMonthBeninc(i1), i1);
         if (piaParams.getCatchup().exist(eligYear, i1)) {
            s = "   ";
            s += poutNoComma(piaParams.getCatchup().get(eligYear, i1), 4, 1);
            s += " % for ";
            s += DateFormatter::toString(datetemp);
            s += ", catch-up";
            outputString.push_back(s);
         }
         s = "   ";
         s += poutNoComma(piaParams.getCpiinc(i1), 4, 1);
         s += " % for ";
         s += DateFormatter::toString(datetemp);
         if (piamfb != 0) {
            s += ":  ";
            if (piamfb % 2 == 1)
               s += BenefitAmount(piaMethod.piaElig[i1]).toString();
            if (piamfb > 1) {
               s += "  ";
               s += BenefitAmount(piaMethod.mfbElig[i1]).toString();
            }
         }
         outputString.push_back(s);
      }
   }
   outputString.push_back("");
}

void PiaPageOut::biOut( const PiaMethod& piaMethod,
const PiaParams& piaParams, int eligYear, int firstYear, int lastYear, 
int piamfb )
{
   string s;  // temporary string

#ifndef NDEBUG
   if (eligYear < YEAR37)
      throw PiaException(PIA_IDS_WISTRING);
#endif
   outputString.push_back("CPI increases applied:");
   for (int i1 = firstYear; i1 <= lastYear; i1++) {
      if (piaParams.getMonthBeninc(i1) > 0) {
         DateMoyr datetemp(piaParams.getMonthBeninc(i1), i1);
         if (piaParams.getCatchup().exist(eligYear, i1)) {
            s = "   ";
            s += poutNoComma(piaParams.getCatchup().get(eligYear, i1), 4, 1);
            s += " % for ";
            s += DateFormatter::toString(datetemp);
            s += ", catch-up";
            outputString.push_back(s);
         }
         s = "   ";
         s += poutNoComma(piaParams.getCpiinc(i1), 4, 1);
         s += " % for ";
         s += DateFormatter::toString(datetemp);
         if (piamfb != 0) {
            s += ":  ";
            if (piamfb % 2 == 1)
               s += BenefitAmount(piaMethod.piaElig[i1]).toString();
            if (piamfb > 1) {
               s += "  ";
               s += BenefitAmount(piaMethod.mfbElig[i1]).toString();
            }
         }
         outputString.push_back(s);
      }
   }
   outputString.push_back("");
}

// Description: Calculates the first year type for printing a range of benefit
// increases.
//
// Returns: The ending year type.
//
// Arguments:
//   type: Type of writeout.
//     0 to go from first year to entitlement.
//     1 to go from eligibility to entitlement.
//     2 to go from entitlement to benefit date.
//     3 to go from first year to benefit date.
PiaMethod::cpi_year_type PiaPageOut::firstYearCal( int type )
{
   switch (type) {
   case 0:
      return PiaMethod::FIRST_YEAR;
   case 1:
      return PiaMethod::YEAR_ELIG;
   case 2:
      return PiaMethod::YEAR_ENT;
   case 3:
      return PiaMethod::FIRST_YEAR;
   default:
      throw PiaException("PiaPageOut::endYearCal: bad value for type");
   }
}

// Description: Calculates the ending year type for printing a range of benefit
// increases.
//
// Returns: The ending year type.
//
// Arguments:
//   type: Type of writeout.
//     0 to go from first year to entitlement.
//     1 to go from eligibility to entitlement.
//     2 to go from entitlement to benefit date.
PiaMethod::cpi_year_type PiaPageOut::endYearCal( int type )
{
   switch (type) {
   case 0:
   case 1:
      return PiaMethod::YEAR_ENT;
   case 2:
   case 3:
      return PiaMethod::YEAR_BEN;
   default:
      throw PiaException("PiaPageOut::endYearCal: bad value for type");
   }
}

// Description: Prepares table change automatic increases used in PIA
//   calculation in a string table.
//
// Arguments:
//   piaMethod: Pia calculation method.
//   piaParams: Pia calculation parameters.
//   eligYear: Year of eligibility for catch-up benefit increases.
//   firstYear: First year to write.
//   lastYear: Last year to write.
//   piamfb: Indicator for PIA and MFB printout.
//     0 to not print out PIA or MFB.
//     1 for PIA.
//     2 for MFB.
//     3 for PIA and MFB.
void PiaPageOut::newTableOut( const PiaMethod& piaMethod,
const PiaParams& piaParams, int eligYear, int firstYear, int lastYear,
int piamfb )
{
   string s;  // temporary string

#ifndef NDEBUG
   if (eligYear < YEAR37)
      throw PiaException(PIA_IDS_WISTRING);
#endif
   outputString.push_back("Table maximum changes applied:");
   for (int i1 = firstYear; i1 <= lastYear; i1++) {
      if (piaParams.getMonthBeninc(i1) > 0) {
         DateMoyr datetemp(piaParams.getMonthBeninc(i1), i1);
         s = "     Max AME of ";
         s += poutNoComma(piaParams.lastAmw(i1), 4);
         s += " for ";
         s += DateFormatter::toString(datetemp);
         if (piamfb != 0) {
            s += ":  ";
            if (piamfb % 2 == 1)
               s += BenefitAmount(piaMethod.piaElig[i1]).toString();
            if (piamfb > 1) {
               s += "  ";
               s += BenefitAmount(piaMethod.mfbElig[i1]).toString();
            }
         }
         outputString.push_back(s);
      }
   }
   outputString.push_back("");
}

// Description: Prepares PIA and MFB at entitlement in a string table.
//
// Arguments:
//   piaMethod: Pia calculation method.
void PiaPageOut::piamfb1( const PiaMethod& piaMethod )
{
   const int yearEnt = piaMethod.getCpiYear(PiaMethod::YEAR_ENT);
   piaEntOut(piaMethod.piaElig[yearEnt]);
   outputString.push_back("");
   if (piaMethod.getIndCap() == PiaMethod::PIA_1_5 ||
      piaMethod.getIndCap() == PiaMethod::PIA_1_0) {
      // cap is 1.0 or 1.5 times PIA
      disMfbOut(piaMethod.getCap(), piaMethod.piaElig[yearEnt],
         piaMethod.mfbElig[yearEnt]);
   }
   else {
      string s = "MFB at entitlement = ";
      s += BenefitAmount(piaMethod.mfbElig[yearEnt]).toString();
      outputString.push_back(s);
      outputString.push_back("");
   }
}

// Description: Prepares PIA and MFB at benefit date in a string table.
//
// Arguments:
//   piaMethod: Pia calculation method.
void PiaPageOut::piamfb2( const PiaMethod& piaMethod )
{
   string s = "PIA at benefit date = ";
   s += piaMethod.piaEnt.toString();
   outputString.push_back(s);
   outputString.push_back("");
   if (piaMethod.getIndCap() == PiaMethod::PIA_1_5 ||
      piaMethod.getIndCap() == PiaMethod::PIA_1_0) {
      // cap is 1.0 or 1.5 times PIA
      disMfbOut(piaMethod.getCap(),
         piaMethod.piaEnt.get(), piaMethod.mfbEnt);
   }
   else {
      s = "MFB at benefit date = ";
      s += piaMethod.mfbEnt.toString();
      outputString.push_back(s);
      outputString.push_back("");
   }
}

// Description: Prepares PIA and MFB from June 1974 table in a string table.
//
// Arguments:
//   piaMethod: Pia calculation method.
void PiaPageOut::piamfb3( const PiaMethod& piaMethod )
{
   const int firstYear = piaMethod.getCpiYear(PiaMethod::FIRST_YEAR) - 1;
   string s = "PIA from June 1974 table = ";
   s += poutNoComma(piaMethod.piaElig[firstYear], 4, 2);
   outputString.push_back(s);
   outputString.push_back("");
   if (piaMethod.getIndCap() == PiaMethod::PIA_1_5 ||
      piaMethod.getIndCap() == PiaMethod::PIA_1_0) {
      // cap is 1.0 or 1.5 times PIA
      outputString.push_back("MFB at this date:");
      string s = "   ";
      s += poutNoComma(piaMethod.getCap(), 4, 2);
      s += " * ";
      s += poutComma(piaMethod.piaElig[firstYear], 4, 2);
      outputString.push_back(s);
      s = "   (MFB cap on DI cases) = ";
      s += poutNoComma(piaMethod.mfbElig[firstYear], 4, 2);;
      outputString.push_back(s);
      outputString.push_back("");
   }
   else {
      s = "MFB from June 1974 table = ";
      s += poutNoComma(piaMethod.mfbElig[firstYear], 4, 2);
      outputString.push_back(s);
      outputString.push_back("");
   }
}

// Description: Prepares MFB calculation in a string table.
//
// Remarks: The version with 4 arguments handles the DI family maximum, while
// the version with 3 arguments does not.
//
// Arguments:
//   piaMethod: Pia calculation method number.
//   portionPiaElig: Portion of pia in each mfb formula interval.
//   percMfb: Mfb formula percentages.
//   aime: Aime used in pia formula.
void PiaPageOut::mfbOut( const PiaMethod& piaMethod,
const double portionPiaElig[], const PercMfb& percMfb, double aime )
{
   if (piaMethod.getIndCap() == PiaMethod::PIA_1_5 ||
      piaMethod.getIndCap() == PiaMethod::PIA_1_0) {
      // cap is 1.0 or 1.5 times PIA
      return;
   }
   outputString.push_back("MFB at eligibility:");
   if (piaMethod.getIndCap() == PiaMethod::AIME_85) {
      // cap is 85% of AIME
      disMfbOut(piaMethod.getCap(), aime,
         piaMethod.mfbElig[piaMethod.getCpiYear(PiaMethod::FIRST_YEAR)]);
   }
   else {
      mfbOut(piaMethod, portionPiaElig, percMfb);
   }
}

void PiaPageOut::mfbOut( const PiaMethod& piaMethod,
const double portionPiaElig[], const PercMfb& percMfb )
{
   string s;  // temporary string

   // usual formula
   for (int i2 = 0; i2 < 3; i2++) {
      s = "   ";
      s += poutNoComma(percMfb[i2], 4, 2);
      s += " * ";
      s += poutComma(portionPiaElig[i2], 4, 2);
      s += " +";
      outputString.push_back(s);
   }
   s = "   ";
   s += poutNoComma(percMfb[3], 4, 2);
   s += " * ";
   s += poutComma(portionPiaElig[3], 4, 2);
   s += " = ";
   const int year = piaMethod.getCpiYear(PiaMethod::FIRST_YEAR);
   s += BenefitAmount(piaMethod.mfbElig[year]).toString();
   outputString.push_back(s);
   outputString.push_back("");
}

// Description: Prepares real-wage-gain adjustment in a string table.
//
// Arguments:
//   eligYear: Year of eligibility.
//   istart: Current year.
//   piaMethod: Pia calculation method.
//   rwgpia: Pia before real-wage-gain adjustment.
//   rwgmfb: Mfb before real-wage-gain adjustment.
void PiaPageOut::printRealWageGain( int eligYear, int istart,
const PiaMethod& piaMethod, BenefitAmount rwgpia, BenefitAmount rwgmfb )
{
   outputString.push_back(
      "Real-wage-gain adjustment for PEBES calculation:");
   string s = "   Factor = ";
   s += poutNoComma(1.+.01 * static_cast<double>(eligYear-istart), 4, 2);
   outputString.push_back(s);
   s = "   PIA before adjustment = ";
   s += rwgpia.toString();
   outputString.push_back(s);
   if (piaMethod.getMethod() == PiaMethod::WAGE_IND) {
      s = "   MFB before adjustment = ";
      s += rwgmfb.toString();
      outputString.push_back(s);
   }
   outputString.push_back("");
}

// Description: Prepares table heading for wage-indexed earnings in a string
//   table.
//
// Arguments:
//   avgbsyr: Average wage in base year.
void PiaPageOut::tableHead1( double avgbsyr )
{
   outputString.push_back(
      "                           earnings        indexed       high n");
   string s = "year      earnings      * ";
   s += poutComma(avgbsyr, 11, 2);
   s += "      earnings       years";
   outputString.push_back(s);
   outputString.push_back("");
}

// Description: Prepares table heading for non-indexed earnings in a string
//   table.
void PiaPageOut::tableHead2()
{
   outputString.push_back("                        High n");
   outputString.push_back("Year      Earnings       Years");
   outputString.push_back("");
}

// Description: Prepares base year in a string table.
//
// Arguments:
//   year: Base year to print.
void PiaPageOut::baseYearOut( int year )
{
   string s = "Base year for indexing = ";
   s += poutNoComma(year);
   outputString.push_back(s);
   outputString.push_back("");
}

// Description: Prepares PIA at entitlement in a string table.
//
// Arguments:
//   piaentt: Pia at entitlement to print.
void PiaPageOut::piaEntOut( BenefitAmount piaentt )
{
   string s = "PIA at entitlement = ";
   s += piaentt.toString();
   outputString.push_back(s);
}

// Description: Prepares raw pro rata PIA in a string table.
//
// Arguments:
//   rawpia: Raw pro rata PIA.
void PiaPageOut::rawPiaOut( BenefitAmount rawpia )
{
   string s = "Raw Pro Rata PIA = ";
   s += rawpia.toString();
   outputString.push_back(s);
   outputString.push_back("");
}

// Description: Prepares totalization PIAs in a string table.
//
// Arguments:
//   piaMethod: Pia calculation method.
//   qcTotal: Total number of quarters of coverage.
//   n: Number of computation years.
void PiaPageOut::totPiaOut( const PiaMethod& piaMethod, int qcTotal, int n )
{
   string s = "Theoretical PIA = ";
   s += piaMethod.getPiaTotal().toString();
   outputString.push_back(s);
   outputString.push_back("");
   s = "Pro Rata PIA = (";
   s += poutNoComma(3 * qcTotal);
   s += "/";
   s += poutNoComma(12 * n);
   s += ") * ";
   s += piaMethod.getPiaTotal().toString();
   s += " = ";
   s += piaMethod.piaEnt.toString();
   outputString.push_back(s);
   outputString.push_back("");
}

// Description: Prepares MFB cap on DI cases in a string table.
//
// Arguments:
//   capt: Percentage cap for DI MFB.
//   amepia: Ame or pia.
//   mfbt: DI MFB.
void PiaPageOut::disMfbOut( double capt, double amepia, BenefitAmount mfbt )
{
   outputString.push_back("MFB at benefit date:");
   string s = "   ";
   s += poutNoComma(capt, 4, 2);
   s += " * ";
   s += poutComma(amepia, 4, 2);
   outputString.push_back(s);
   s = "   (MFB cap on DI cases) = ";
   s += mfbt.toString();
   outputString.push_back(s);
   outputString.push_back("");
}

// Description: Prepares applicable old start pia table in a string table.
//
// Arguments:
//   piaMethod: Pia calculation method.
void PiaPageOut::osTableOut( const PiaMethod& piaMethod )
{
   string s("Applicable table: ");
   s += tableLabel[piaMethod.getTableNum()];
   s += " Act";
   outputString.push_back(s);
   outputString.push_back("");
}

// Description: Prepares MFB formula bend points in a string table.
//
// Arguments:
//   bendMfb: Mfb formula bend points.
void PiaPageOut::mfbBpOut( const BendMfb& bendMfb )
{
   string s = "MFB formula bend points = ";
   s += poutComma(bendMfb[1]);
   s += ", ";
   s += poutComma(bendMfb[2]);
   s += ", and ";
   s += poutComma(bendMfb[3]);
   outputString.push_back(s);
   outputString.push_back("");
}

// Description: Prepares PIA formula bend points in a string table.
//
// Arguments:
//   bendPia: Pia formula bend points.
void PiaPageOut::piaBpOut( const BendPia& bendPia )
{
   string s = "PIA formula bend points = ";
   s += poutComma(bendPia[1]);
   s += " and ";
   s += poutComma(bendPia[2]);
   outputString.push_back(s);
   outputString.push_back("");
}

// Description: Prepares wage-indexed pia at eligibility calculation in a
//   string table.
//
// Arguments:
//   percPia: Pia formula percentages.
//   portionAime: Parts of aime in each pia formula interval.
//   piatemp: Pia at eligibility.
void PiaPageOut::piaEligOut( const PercPia& percPia,
const double portionAime[], BenefitAmount piatemp )
{
   string s;  // temporary string

   outputString.push_back("PIA at eligibility = ");
   for (int i = 0; i < 2; i++) {
      s = "   ";
      s += poutNoComma(percPia[i], 4, 2);
      s += " * ";
      s += poutComma(portionAime[i]);
      s += " +";
      outputString.push_back(s);
   }
   s = "   ";
   s += poutNoComma(percPia[2], 4, 2);
   s += " * ";
   s += poutComma(portionAime[2]);
   s += " = ";
   s += piatemp.toString();
   outputString.push_back(s);
   outputString.push_back("");
}

// Description: Prints out name and SSN.
//
// Arguments:
//   workerData: Basic worker data.
void PiaPageOut::printNameSsn( const WorkerDataGeneral& workerData )
{
   string s;  // temporary string

   AfxFormatString1(s, PIA_IDS_NAMEOUT, workerData.getNhname());
   outputString.push_back(s);
   AfxFormatString1(s, PIA_IDS_SSNOUT, workerData.getIdString());
   outputString.push_back(s);
}

// Description: Prints out disability insured message.
//
// Arguments:
//   piaData: Pia calculation data.
void PiaPageOut::disInsOut( const PiaData& piaData )
{
   if (!piaData.disInsCode.isDisabilityInsured()) {
      string s;  // temporary string
      AfxFormatString1(s, PIA_IDS_NDISINOUT, "");
      outputString.push_back(s);
   }
}

// Description: Prints out noncovered pension amount and starting date.
//
// Arguments:
//   pubpen: Amount of public pension.
//   pubpenDate: Starting date of public pension.
void PiaPageOut::nonpenOut( double pubpen, const DateMoyr& pubpenDate )
{
   string s;  // temporary string
   AfxFormatString1(s, PIA_IDS_NONPENOUT, poutComma(pubpen, 3, 2));
   outputString.push_back(s);
   AfxFormatString1(s, PIA_IDS_NONPENDATEOUT,
      DateFormatter::toString(pubpenDate));
   outputString.push_back(s);
}

// Description: Prints out uninsured message.
//
// Arguments:
//   finsCode: Insured status code.
void PiaPageOut::nonIns( char finsCode )
{
   string s;  // temporary output string

   switch (finsCode)
   {
   case '2':
      // survivor case, currently insured only
      AfxFormatString1(s, PIA_IDS_CURINSOUT, "");
      outputString.push_back(s);
      return;
   case '4':
      // life case, not fully insured
      AfxFormatString1(s, PIA_IDS_NINSUROUT, "");
      outputString.push_back(s);
      return;
   case '5':
      // not currently or fully insured (survivor case)
      AfxFormatString1(s, PIA_IDS_NINSTOUT, "");
      outputString.push_back(s);
      return;
   case '6':
      // totalization, with less than 6 qcs
      AfxFormatString1(s, PIA_IDS_NTOTALOUT, "");
      outputString.push_back(s);
      outputString.push_back(warnTotalization);
      return;
   case '7':
      // totalization, but fully insured
      AfxFormatString1(s, PIA_IDS_INSUREOUT, "");
      outputString.push_back(s);
      outputString.push_back(warnTotalization);
      return;
   default:
      return;
   }
}

// Description: Prepares header strings for output.
void PiaPageOut::prepareHeader()
{
   boost::posix_time::time_facet* output_facet =
      new boost::posix_time::time_facet();
   output_facet->format("%a %b %d %H:%M:%S %Y");
   stringstream ss;
   ss.imbue(locale(locale::classic(), output_facet));
   boost::posix_time::ptime pt = boost::posix_time::second_clock::local_time();
   ss << pt;
   string s(ss.str());
   s.append(43, ' ');
   s += "Page ";
   s += poutNoComma(pageNum);
   outputString.push_back(s);
   for (int i = 0; i < 2; i++) {
      if (pageTitle[i].length() > 0) {
         s = " ";
         unsigned spacecount = (pageTitle[i].length() < 74u) ?
            (74u - pageTitle[i].length()) / 2u : 0u;
         s.append(spacecount, ' ');
         s += pageTitle[i];
         outputString.push_back(s);
      }
   }
   outputString.push_back("");
   outputString.push_back("");
}

// Description: Print out indexed earnings table.
//
// Arguments:
//   wageInd: Wage-indexed calculation method.
//   piaData: Pia calculation summary data.
//   eligYear: Year of eligibility.
//   earnings: Unindexed earnings.
//   firstYear: First year of earnings.
void PiaPageOut::printIndexedEarningsTable( const WageIndGeneral& wageInd,
const PiaData& piaData, int eligYear, const AverageWage& earnings,
int firstYear )
{
   tableHead1(wageInd.getIndexYearAvgWage());
   for (int i1 = firstYear; i1 <= piaData.getEarnYear(); i1++) {
      string s = poutNoComma(i1);
      s += poutComma(earnings[i1], 14, 2);
      if (i1 < eligYear - 1)
         s += poutComma(wageInd.earnMultiplied[i1], 19, 2);
      else
         s += "                   ";
      s += poutComma(wageInd.earnIndexed[i1], 14, 2);
      if (wageInd.iorder[i1] > 0)
         s += poutComma(wageInd.earnIndexed[i1], 14, 2);
      outputString.push_back(s);
   }
   outputString.push_back("");
}

// Description: Prepares windfall elimination provision calculation.
//
// Arguments:
//   wageInd: Wage-indexed calculation method.
//   pubpen: Amount of public pension.
//   pubpenDate: Starting date of public pension.
void PiaPageOut::pubPenOut( const WageIndGeneral& wageInd, double pubpen,
  const DateMoyr& pubpenDate )
{
   nonpenOut(pubpen, pubpenDate);
   outputString.push_back("");
   string s = poutNoComma(wageInd.getYearsTotal());
   s += " years of coverage for windfall provision";
   outputString.push_back(s);
   if (wageInd.getWindfall() == PiaMethod::HAS30YEARS) {
      outputString.push_back("");
      outputString.push_back("Special minimum savings clause applies");
      outputString.push_back("");
      return;
   }
   outputString.push_back("");
   outputString.push_back("PIA after windfall:");
   const int year = wageInd.getCpiYear(PiaMethod::FIRST_YEAR);
   if (wageInd.getWindfall() == PiaMethod::REDUCEDPERC) {
      for (int i = 0; i < 2; i++) {
         s = "   ";
         s += poutNoComma(wageInd.percWind[i], 4, 2);
         s += " * ";
         s += poutComma(wageInd.portionAime[i]);
         s += " +";
         outputString.push_back(s);
      }
      s = "   ";
      s += poutNoComma(wageInd.percWind[2], 4, 2);
      s += " * ";
      s += poutComma(wageInd.portionAime[2]);
      s += " = ";
      s += BenefitAmount(wageInd.piaElig[year]).toString();
      outputString.push_back(s);
      outputString.push_back("");
   }
   else {
      s = "   ";
      s += wageInd.getPiaWindfall().toString();
      s += " - .5*";
      s += poutComma(pubpen, 4, 2);
      s += " = ";
      s += BenefitAmount(wageInd.piaElig[year]).toString();
      outputString.push_back(s);
      outputString.push_back("");
   }
}
